/*
$Header: /home/elrond/baikal/src/CVS/daq98/HVdaemon.h,v 1.3 1998/11/23 14:40:08 haase Exp $
*/

#ifndef __HVDAEMON_H_
#define __HVDAEMON_H_

#include <strstream.h>
#include <String.h>
#include <vector.h>
#include <algo.h>
#include <config.h>
#include "HVcrates.h"
#include "HVprops.h"
#include "cnfreader.h"
#include "daemon.h"
#include "daemonbase.h"
#include "datatypes.h"
#include "dispatcherlib.h"
#include "errorlog.h"
#include "hwconfigitems2.h"
#include "interpreter.h"
#include "ionotifier.h"
#include "ipcclasses.h"
#include "pointerutil.h"
#include "process.h"
#include "simplevector.h"
#include "socketclass.h"

/** Class for controlling the high voltage */
class HV:public Daemon
{
	public:
		/** ctor */
		HV();

		/** dtor */
		virtual ~HV();

		/** initialization */
		int Init();

		/** destruction */
		int Destroy();

//	private:
	public:
		/** Contains information about a user */
		struct HVUser
		{
			/** holds the username */;
			String Name;
			/** first allowed channel */
			int ChFirst;
			/** last allowed channel */
			/** e.g. wischnew 1 16: Ralf is allowed to manipulate */
			/** channels 1 to 16 */
			int ChLast;
		};

		struct HVMail
		{
			/** Where the mail is to be sent to */
			String MailAddress;	
		};

		

		// use HVUser_p like this:
		// HVUser_p HVUserList_i;
		// HVUserList_i = new HVUser;
		// HVUserList_i->ChFirst = 0;
		// HVUserList_i->CHLast = 0;
	public:
/* configuring the daemon			*/
		MAKE_REFCOUNT_POINTER (HVUser, HVUser_p);
		MAKE_REFCOUNT_POINTER (HVMail, HVMail_p);


		vector<HVUser_p> HVUserList;
		vector<HVMail_p> HVMailList;
		int HVTimeInterval;

		int HVVoltageDiff;
		int HVLogLevel;
		int HVActionLevel;

/** For getting hvcrate configuration (hv.conf) */
		HVCrates HVC;

// /** Storing the demand and actual values as well as alarm conditions */
// 		vector<HVChannelConfig_p> HVChannelConfigList; // obsolete
								// *PN* 221097
/** better use an object version */
		HVProps HVP;
		
		// use HVUserList:
		// HVDevice_p HVDeviceList_i;
		// HVDeviceList_i = new HVDevice;
		// HVDeviceList_i->Offset = 0;
		// ...
		// HVDeviceList.push_back (HVDeviceList_i);

//		/** SetUp the HVpropertyList */	// obsolete *PN* 221097
//		int SetUpHVPL (const char *filename);

	public:
		/** Make a scatter plot of v2 vs v1.
			Equivalent to PAW ve/plot v2%v1.
			@param s: socket for talking to moniclient
			@return 0 if o.k., -1 if vectors have different sizes*/
		int VectorPlot2d (IOSocket &s,\
					float_inc_v &v1,\
					float_inc_v &v2);

		/** Plot a histogram.
			Books an nch-channel histogram, and puts weight 
			at position x.
			@return 0 if o.k., -1 if vectors have different sizes
			@param s: socket for talking to moniclient
			@param nch: The number of channels for the histogram
			@param x: the vector containg the x positions
			@param weights: the weight for the channels defined
				by x */
		int HistoPlot (IOSocket &s,\
				int nch,\
				float_inc_v &x,\
				float_inc_v &weights);

		/** Checks if user User has right to change HV */
		bool MayWrite (IOSocket &s, String User);

		/** Returns which user has the write permission at the moment*/
		String CurrentAuthor ();

		/** Returns a list of valid modules built from commandline
			arguments.
			*/
		void ValuesLfromCmdLine (IOSocket &s,
					int numparams, String *params,
					String_inc_v &ValuesL, bool &ok);

	protected:
		/** the event dispatcher */
		virtual int DispatchEvent();

		/** the IPC event handler */
		virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);

		/** the TIMER event handler */
		virtual int DoTIMEREvent ();

		/** returns a String with the configuration file name */
		virtual const String GetConfigFileName()const;

		/** returns a pointer to the configuration class, created with
		new on the heap, must be overwritten in derived class */
		virtual ConfigSection *GetConfigSection();

	public:
		/** returns the type of the process, must be overwritten in
		derived class */
		/* has to be public, because it is called by interpreter */
		virtual const String GetProcessType()const;

	private:
		/** manages the interpretation of the HV_CMDSet/Get */
		class SetClass: public ItemWithProperties
		{
			public:
				SetClass (HVProps &hvp): HVP (hvp)
				{
					static String Props[] = \
					{"OFF", "ON", "VOLT", "TRIP", "RAMP"};

					SetProperties (sizeof (Props)/\
							sizeof (Props[0]),\
							Props);
				}

				~SetClass () {}

				virtual int SetPropertyByNo (\
							int propnum,\
							const String &val);

				virtual const String GetPropertyByNo (\
							int propnum) const;
				virtual const String GetClassID ()
				{
					return "SetClass";
				}

			private:
				int Channel;

				int Voltage;

				int TripCurrent;

				int RampingSpeed;

				HVProps &HVP;
		};

		class GetClass: public ItemWithProperties
		{
			public:
				GetClass (HVProps &hvp): HVP (hvp)
				{
					static String Props[] = \
					{"VOLT", "TRIP", "RAMP"};

					SetProperties (sizeof (Props)/\
							sizeof (Props[0]),\
							Props);
				}

				~GetClass () {}

				virtual int SetPropertyByNo (\
							int propnum,\
							const String &val);

				virtual const String GetPropertyByNo (\
							int propnum) const;
				virtual const String GetClassID ()
				{
					return "GetClass";
				}

			private:
				int Channel;

				int Voltage;

				int TripCurrent;

				int RampingSpeed;

				HVProps &HVP;
		};

	private:
		Interpreter<HV> HV_CMD_Interpreter;

		/** to quit the HVDaemon */
		int HV_CMD_Quit ();
		/** to read the HVproperty configuration */
		int HV_CMD_ReadPropConfig (IOSocket &s, String*, int);
		/** to write the HVproperty configuration to file */
		int HV_CMD_WritePropConfig (IOSocket &s, String *, int);
		/** to list the configuration */
		int HV_CMD_List (IOSocket &s, String *, int);
		/** send configuration to the moniclient */
		int HV_CMD_View (IOSocket &s, String *, int);
		/** return the status of the HV */
		int HV_CMD_Status (IOSocket &s, String *, int);
		/** Tells you who you are */
		int HV_CMD_WhoAmI (IOSocket &s, String *, int);
		/** Tells you if you're allowed to write */
		int HV_CMD_Privs (IOSocket &s, String *, int);
		/** Make a user able to change HV */
		int HV_CMD_Author (IOSocket &s, String *, int);
		/** set a property of the HV */
		int HV_CMD_Set (IOSocket &s, String *, int);
		/** get a property of the HV */
		int HV_CMD_Get (IOSocket &s, String *, int);
		/** help for all commands */
		int HV_CMD_Help (IOSocket &s, String *, int);

		int HV_CMD_UnknownCommand (IOSocket &s, String *, int);
		int HV_CMD_WrongTypeCommand (IOSocket &s, String *, int);

		void SendAlertMail ();
};


/** to read in the HVdaemon's Configuration */
class HVConfig:public ConfigSection
{
	public:
		/** ctor */
		HVConfig(vector<HV::HVUser_p> &hvuserlist,
			vector<HV::HVMail_p> &hvmaillist,
			int &hvtimeinterval,
			int &hvvoltagediff,
			int &hvloglevel, int &hvactionlevel):\
			HVUserList (hvuserlist), HVMailList (hvmaillist),\
			HVTimeInterval (hvtimeinterval),
			HVVoltageDiff (hvvoltagediff),
			HVLogLevel (hvloglevel),
			HVActionLevel (hvactionlevel)
		{
			static String Props[] =\
	{"USER", "MAIL", "TIME", "DIFF", "LOGLEVEL", "ACTIONLEVEL"};
			SetProperties (sizeof (Props)/sizeof (String), Props);
		};

		/** copy ctor */
//		HVConfig(const HVConfig &hdc);

		/** returns a pointer to a new copy of the object on the heap */
//		virtual ItemBase *clone()const;

		/** return the keyword after ITEM as ClassID */
		virtual const String GetClassID ()
		{
			return PROCESS_TYPE_HVDAEMON;
		}

		/**return the name of the section in the config file, where the
		config items are placed */
		virtual const String GetSectionName()const;

		virtual const String GetCommonConfigSection() const;

		/** sets property propnum */
		virtual int SetPropertyByNo(int propnum,const String &val);

		/** returns property propnum */
		virtual const String GetPropertyByNo(int propnum);

		// Add references to the data that has to be read here and
		// initialize them with the constructor

		/** List containing ,,pointers'' to the users */
		vector<HV::HVUser_p> &HVUserList;
		/** List containing ,,pointers'' to the mail destinations */
		vector<HV::HVMail_p> &HVMailList;
		/** List containing the timeinterval */
		int &HVTimeInterval;
		/** Maximum allowed voltage diff between Soll and Ist */
		int &HVVoltageDiff;
		/** Information about logging.
		See globalmacros.h for details */
		int &HVLogLevel;
		int &HVActionLevel;
};

/** to read the demand voltage of the channels */
class HVPropConfig: public ConfigSection
{
	public:
		HVPropConfig (vector<HVChannelConfig_p> &hvproplist):
			HVPropList (hvproplist)
		{
			static String Props[] = {"DETECTOR", "CHANNEL", "POS",
				"VOLT", "TRIP", "RAMP"};
			SetProperties (sizeof (Props)/sizeof (String), Props);
		}

		/** copyctor (none) */

		virtual const String GetClassID ()
		{
			return PROCESS_TYPE_HVDAEMON;
		}

		virtual const String GetSectionName () const;

//		virtual const String GetCommonConfigSection () const;

// 		virtual const String SetPropertyByNo (int propnum,\
// 					const String &val);

		virtual const String GetPropertyByNo (int propnum);

		vector<HVChannelConfig_p> &HVPropList;
};

#endif // #ifndef __HVDAEMON_H_
