#ifndef _HVPROPS_H_
#define _HVPROPS_H_

/* $Header: /home/elrond/baikal/src/CVS/daq98/HVprops.h,v 1.3 1998/11/23 14:40:11 haase Exp $ */

#include <String.h>
#include <config.h>
#include "hwconfigitems2.h"
#include "socketclass.h"

#define MAXHVPROPSLIST 1

enum ValidProps_t {actual, backup1};	// tells which list you are 
						// looking at
						// for use in GoFirst ()

/** A class for managing the HV property configuration.
	It stores up to MAXHVPROPSLIST 
	configurations and supplies pointers to the up-to-date and a future
	configurations. Currently MAXHVPROPSLIST is set to 1 and
	multiconfiguration is disabled. (281097) */
class HVProps
{
	public:
		/** Constructor. */
		HVProps (/*IOSocket &s*/);
		/** Destructor. */
		~HVProps ();

		/** Reads configuration from file filename.
			Makes it the
			actual configuration if update is true. If update
			is false, it trashes the backup config with the 
			read values. 
			@return -1 on failure, 0 on success */
		int ReadConfiguration (const char* filename,\
						bool update,\
						const char* hwfilename);

		/** Check if a configuration has been set.
			This is achieved by checking if the configuration
			list is empty.
			@return true if list is empty, false if not */
		bool Empty ();

		/** Go to the first element in a list.
			This means setting an
			internal iterator tho the begin of the list.
			@param vp determines the list to look at: actual
				sets the internal iterator to the ActualList,
				backup1 sets it to the BackupList
			@return Always returns 0 */
		int GoFirst (ValidProps_t vp);

		/** Advance the internal internal iterator.
			Moves the internal iterator from its current
			position to the next, if its not pointing to the last.
			@return 0 if ok, -1 if one attempts to move behind
			the end of the list.
		*/
		int Advance ();

		/** Gets the set of props.
			The returned String contains the property names
			and their values. Example:\\
			"ID 23 MAP 23 Voltage 1300..."\\
			Use Advance to  advance the internal iterator.
			@return If the internal iterator points to eol,
			an empty string is returned. */
		String Get ();

		/** Same as get, but restricted to the ID.
			@return ID */
		ID_t GetID ();

		/** Get ID for module specified by its position.
			Get the ID for a module described by its
			position on a string.
			@param short unsigned Number: The string
			@param short unsigned Position: The position on the
								string.
			@return ID_t: The ID of the module, -1 if not
								found. */
		ID_t GetID (unsigned short Number, unsigned short Position);

		/** Same as get, but restricted to the HVChannel.
			@return HVChannel */
		unsigned short GetHVChannel ();
	
		/** Same as get, but restricted to the StringNumber.
			@return StringNumber */
		unsigned short GetStringNumber ();

		/** Same as get, but restricted to the StringPosition.
			@return StringPosition */
		unsigned short GetStringPosition ();

		/** Same as get, but restricted to the Voltage.
			@return Voltage */
		int GetVoltage ();

		/** Same as get, but restricted to the LastVoltage.
			@return LastVoltage */
		int GetLastVoltage ();

		/** Same as get, but restricted to the trip current.
			@return TripCurrent */
		int GetTripCurrent ();

		/** Same as get, but restricted to the ramping speed.
			@return RampingSpeed */
		int GetRampSpeed ();

		/** Same as get, but restricted to the UserChange.
			@return UserChange */
		bool GetUserChange ();

		/** Same as get, but restricted to the Alarm.
			@return Alarm */
		bool GetAlarm ();	

		/** Set the Voltage.
			Writes voltage to the location where the
			internal iterator points to.	*/
		void SetVoltage (int voltage);

		/** Set the voltage for a certain ID. */
		void SetVoltage (ID_t id, int voltage);

		/** Sets the LastVoltage.	*/
		void SetLastVoltage (int lastvoltage);

		/** Sets the UserChange. \Label{SUC}	*/
		void SetUserChange (bool userchange);

		/** Sets the UserChange for id.
			Same as \Ref{SUC} but specifies a (logical) channel.
			@param ID_t id: log. channel */
		void SetUserChange (ID_t id, bool userchange);
		

		/** Sets the Alarm.	*/
		void SetAlarm (bool alarm);	

		/** Check the internal iterator.
			Checks if the internal iterator points to the end
			of the list.
			@return true if it points to last element in the
			list. */
		bool IsLast ();

		/** Check if an ID is valid.
			Look if the the given ID is mapped to a HVChannel.
			@param ID_t ID: the logical channel
			@return true if valid, false is not. */
		bool IsValid (ID_t ID);

		/** Check if a stringnumber is defined.
			Checks if the string number is defined.
			@param unsigned short number: the number to be checked
			@return true if string is defined */
		bool StringValid (unsigned short Number);

		/** Get the first module of a string.
			Gets the lowest number in a string designated by
			Number.
			@param unsigned short Number: Number of the string 
			@return unsigned short: the number of the
				desired module. */
		unsigned short FirstModule (unsigned short Number);

		/** Get the last module of a string.
			Gets the highest number in a string designated by
			Number.
			@param unsigned short Number: Number of the string 
			@return unsigned short: the number of the
				desired module. */
		unsigned short LastModule (unsigned short Number);

		/** Map an ID to a HVChannel.
			Looks for the HVChannel which is connected
			to the (logical) channel speciefied by id.
			@param int id: the logical channel
			@return the HVChannel if it is found, -1 if not. */
		unsigned short Map (ID_t id);

		/** Updates configuration.
			Updates configuration by bending ActualList to
			BackupList,
			then discards the list pointed to by BackupList */
//		int Update ();

		/** Activate the actual configuration.
			This means switching on all HV according to the
			values specified in ActualList */
		int Activate ();

		/** Activates a channel of the configuration.
			This means switching on HV on one channel.
			@param channel: the desired channel */
//		int Activate (int channel);

		/** Activates a range of channels
			Switches on a range of channels.
			@param from: first channel in range.
			@param to: last chanel in range. */
//		int Activate (int from, int to);

	private:
		IOSocket S;
		vector<HVChannelConfig_p> List[MAXHVPROPSLIST];

		/** points to the list which is in use. */
		vector<HVChannelConfig_p> *ActualList;
		/** points to the list which is (still) inactive. */
//		vector<HVChannelConfig_p>
//			*BackupList[MAXHVPROPSLIST-1];

		/** returns a free list. If all are in use, it clears the
			BackupList and returns its index */
//		int free ();

		/** clears list n */
//		int clear (int n);

		/** clear list clist */
		int clear (vector<HVChannelConfig_p> &clist);

		/** the internal iterator */
		vector<HVChannelConfig_p>::iterator internal_iterator;

		/** Memorize this from GoFirst for use in IsLast.
			Otherwise nothing is
			known which list is read out */
		ValidProps_t ValidProps;
};

/** returns true if p1->ID < p2->ID for use with vector::sort */
bool HVPropsCompare (HVChannelConfig_p p1, HVChannelConfig_p p2);

#endif __HVPROPS_H__
