# -------------------------------------------------------------------------
# This macro checks whether the user has invoked incompatible 
# --enable options
# -------------------------------------------------------------------------

AC_DEFUN(ST_CHECK_EXCLUSIVE_ENABLE_OPTIONS,[
  if test "[${enable_]patsubst([$1], -, _)+set}" = set; then
    if test "[${enable_]patsubst([$2], -, _)+set}" = set; then
      if test "[${enable_]patsubst([$2], -, _)}" = no; then
        true
      else
        $3
      fi
    fi
  fi
])

AC_DEFUN(ST_CHECK_EXCLUSIVE_ENABLE_N_OPTIONS,[
  if test "[${enable_]patsubst([$1], -, _)+set}" = set; then
    if test "[${enable_]patsubst([$2], -, _)+set}" = set; then
      if test "[${enable_]patsubst([$2], -, _)}" = no; then
        $3
      else
        true
      fi
    fi
  fi
])

dnl Copyright (C) 1988 Eleftherios Gkioulekas <lf@amath.washington.edu>
dnl  
dnl This file is free software; as a special exception the author gives
dnl unlimited permission to copy and/or distribute it, with or without 
dnl modifications, as long as this notice is preserved.
dnl 
dnl This program is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
dnl implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 

# -----------------------------------------------------------------------
# This macro tests the C++ compiler for various portability problem.
# 1. Defines CXX_HAS_NO_BOOL if the compiler does not support the bool
#    data type
# 2. Defines CXX_HAS_BUGGY_FOR_LOOPS if the compiler has buggy
#    scoping for the for-loop
# Seperately we provide some config.h.bot code to be added to acconfig.h
# that implements work-arounds for these problems.
# -----------------------------------------------------------------------

dnl ACCONFIG TEMPLATE
dnl #undef CXX_HAS_BUGGY_FOR_LOOPS
dnl #undef CXX_HAS_NO_BOOL
dnl #undef USE_ASSERT
dnl END ACCONFIG

AC_DEFUN(ST_CPP_PORTABILITY,[
  dnl
  dnl Check for common C++ portability problems
  dnl

  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS

  dnl Check whether we have bool
  AC_MSG_CHECKING(whether C++ has bool)
  AC_TRY_RUN([main() { bool b1=true; bool b2=false; }],
             [ AC_MSG_RESULT(yes) ],
             [ AC_MSG_RESULT(no)
               AC_DEFINE(CXX_HAS_NO_BOOL) ],
             [ AC_MSG_WARN(Don't cross-compile)]
            )

  dnl Test whether C++ has buggy for-loops
  AC_MSG_CHECKING(whether C++ has buggy scoping in for-loops)
  AC_TRY_COMPILE([#include <iostream.h>], [
   for (int i=0;i<10;i++) { }
   for (int i=0;i<10;i++) { }
], [ AC_MSG_RESULT(no) ],
   [ AC_MSG_RESULT(yes)
     AC_DEFINE(CXX_HAS_BUGGY_FOR_LOOPS) ])

  dnl Done with the portability checks
  AC_LANG_RESTORE
])

# -----------------------------------------------------------------------
# This macro determines hardware-vendor-os information and 
# sets the variable ``canonical_host_type'' to that information
# ------------------------------------------------------------------------

dnl ACCONFIG TEMPLATE
dnl #undef YOUR_OS
dnl END ACCONFIG

AC_DEFUN(LF_HOST_TYPE, [
  AC_CANONICAL_HOST
  if test -z "$host"
  then
    host=unknown
  fi
  canonical_host_type=$host
  if test "$host" = unknown
  then
    AC_MSG_WARN(configuring for unknown system type)
  fi
  AC_SUBST(canonical_host_type)
  AC_DEFINE_UNQUOTED(YOUR_OS,"$canonical_host_type")
])


# -------------------------------------------------------------------------
# Use this macro to configure your C compiler
# When called the macro does the following things:
# 1. It finds an appropriate C compiler.
#    If you passed the flag --with-cc=foo then it uses that
#    particular compiler
# 2. Check whether the compiler works.
# 3. Checks whether the compiler accepts the -g 
# -------------------------------------------------------------------------

AC_DEFUN(LF_CONFIGURE_CC,[
  dnl Find out what C compiler to use
  AC_ARG_WITH(cc,
              [  --with-cc=CC            Use another C compiler],
              [ CC=$withval ],
              [ AC_CHECK_PROGS(CC, gcc egcc cc /usr/ucb/cc) ])
  
  dnl Check if the compiler works
  AC_MSG_CHECKING([whether $CC works])
  AC_TRY_COMPILER([main(){return(0);}],lf_cc_works,lf_cc_cross)
  AC_MSG_RESULT($lf_cc_works)
  if test $lf_cc_works = no
  then
    AC_MSG_ERROR([Fatal Error: $CC cannot create executables])
  fi

  if test $lf_with_debug = yes
  then
    dnl Check whether the compiler accepts the -g flag
    LF_CHECK_CC_FLAG(-g)
  fi

  dnl Add these useful things
#  CFLAGS="${CFLAGS} -I."
#  LDFLAGS="-L."
 
  dnl Export CC and CFLAGS to the Makefile
  AC_SUBST(CC)
  AC_SUBST(CFLAGS)
])

# -----------------------------------------------------------------
# This macro should be called to configure your C++ compiler.
# When called, the macro does the following things:
# 1. It finds an appropriate C++ compiler
#    If you passed the flag --with-cxx=foo, then it uses that
#    particular compiler
# 2. Checks whether the compiler accepts the -g 
# ------------------------------------------------------------------

AC_DEFUN(LF_CONFIGURE_CXX,[

  dnl Find out what C++ compiler to use
  AC_ARG_WITH(cxx,
              [  --with-cxx=CXX          Use another C++ compiler],
              [ CXX=$withval ],
              [ AC_CHECK_PROGS(CXX, g++ eg++ c++ gcc CC cxx cc++, gcc) ])
  
  dnl Check if the compiler works
  AC_MSG_CHECKING([whether $CXX works])
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS
  AC_TRY_COMPILER([main(){return(0);}],lf_cxx_works,lf_cxx_cross)
  AC_LANG_RESTORE
  AC_MSG_RESULT($lf_cxx_works)
  if test $lf_cxx_works = no
  then
    AC_MSG_ERROR([Fatal Error: $CXX cannot create executables])
  fi

  if test $lf_with_debug = yes
  then
    dnl Check whether the compiler accepts the -g flag
    LF_CHECK_CXX_FLAG(-g)
  fi
  
  dnl Check for the C++ preprocessor
  AC_PROG_CXXCPP

  dnl Substitute CXX and CXXFLAGS
  AC_SUBST(CXX)
  AC_SUBST(CXXFLAGS)
])

# --------------------------------------------------------------------------
# Check whether the C++ compiler accepts a certain flag
# If it does it adds the flag to CXXFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#   LF_CHECK_CXX_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_CXX_FLAG,[
  echo 'void f(){}' > conftest.cc
  for i in $1
  do
    AC_MSG_CHECKING([whether $CXX accepts $i])
    if test -z "`${CXX} $i -c conftest.cc 2>&1`"
    then
      CXXFLAGS="${CXXFLAGS} $i"
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.cc conftest.o
])

# --------------------------------------------------------------------------
# Check whether the C compiler accepts a certain flag
# If it does it adds the flag to CFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#  LF_CHECK_CC_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_CC_FLAG,[
  echo 'void f(){}' > conftest.c
  for i in $1
  do
    AC_MSG_CHECKING([whether $CC accepts $i])
    if test -z "`${CC} $i -c conftest.c 2>&1`"
    then
      CFLAGS="${CFLAGS} $i"
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.c conftest.o
])

# --------------------------------------------------------------------------
# Check whether the Fortran compiler accepts a certain flag
# If it does it adds the flag to FFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#  LF_CHECK_F77_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_F77_FLAG,[
  cat << EOF > conftest.f
c....:++++++++++++++++++++++++
      PROGRAM MAIN
      PRINT*,'Hello World!'
      END
EOF
  for i in $1
  do
    AC_MSG_CHECKING([whether $F77 accepts $i])
    if test -z "`${F77} $i -c conftest.f 2>&1`"
    then
      FFLAGS="${FFLAGS} $i"
      AC_MSG_RESULT(yes)  
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.f conftest.o
])

# --------------------------------------------------------------------------
# Provide the configure script with an --with-opt option that
# sets the optimization level. Call this command AFTER you have
# configured ALL your compilers.
# Usage:
#  LF_SET_OPTIMIZATION
# --------------------------------------------------------------------------

AC_DEFUN(LF_SET_OPTIMIZATION,[
  dnl Check for --with-opt
  AC_MSG_CHECKING([whether user wants optimization])
  AC_ARG_WITH(opt,
              [  --with-opt=N            Use optimization level N],
              [ lf_opt_level=$withval
                AC_MSG_RESULT(yes) ],
              [ lf_opt_level=0
                AC_MSG_RESULT(no) ])
 
  dnl Now check whether the compilers in use accept the corresponding
  dnl -O flag
  if test $lf_opt_level -gt 0
  then
    if test -n "${CC}"
    then
      LF_CHECK_CC_FLAG(-O${lf_opt_level})
    fi
    if test -n "${CXX}"
    then
      LF_CHECK_CXX_FLAG(-O${lf_opt_level})
    fi
    if test -n "${F77}"
    then
      LF_CHECK_F77_FLAG(-O${lf_opt_level})
    fi
  fi
])

# ----------------------------------------------------------------------
# Provide the configure script with an --with-warnings option that
# turns on warnings. Call this command AFTER you have configured ALL your
# compilers. 
# ----------------------------------------------------------------------

AC_DEFUN(LF_SET_WARNINGS,[
  dnl Check for --with-warnings
  AC_MSG_CHECKING([whether user wants warnings])
  AC_ARG_WITH(warnings,
              [  --with-warnings         Turn on warnings],
              [ lf_warnings=yes ], [ lf_warnings=no ])
  AC_MSG_RESULT($lf_warnings)
  
  dnl Warnings for the two main compilers
  cc_warning_flags="-Wall"
  cxx_warning_flags="-Wall -Woverloaded-virtual -Wtemplate-debugging"
  if test $lf_warnings = yes
  then
    if test -n "${CC}"
    then
      LF_CHECK_CC_FLAG($cc_warning_flags)
    fi
    if test -n "${CXX}" 
    then
      LF_CHECK_CXX_FLAG($cxx_warning_flags)
    fi
  fi
])


# Copyright (C) 1988 Eleftherios Gkioulekas <lf@amath.washington.edu>
# Copyright (C) 1996 John W. Eaton
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
# The following set of macros will allow you to mix Fortran and C/C++
# in a portable manner. This work is based on the autoconf macros written
# by John W. Eaton for GNU Octave, which is also distributed under the
# terms of the GNU public license.

# These macros define the following symbols
dnl ACCONFIG TEMPLATE
dnl #undef F77_APPEND_UNDERSCORE
dnl #undef F77_UPPERCASE_NAMES
dnl END ACCONFIG

# Also, it is important that programs have access to the f77func macro
# which is defined as follows:
dnl ACCONFIG BOTTOM
dnl #ifndef f77func
dnl #  if defined (F77_APPEND_UNDERSCORE)
dnl #    if defined (F77_UPPERCASE_NAMES)
dnl #      define f77func(f, F) F##_
dnl #    else
dnl #      define f77func(f, F) f##_
dnl #    endif
dnl #  else
dnl #    if defined (F77_UPPERCASE_NAMES)
dnl #      define f77func(f, F) F
dnl #    else
dnl #      define f77func(f, F) f
dnl #    endif
dnl #  endif
dnl #endif
dnl END ACCONFIG

# --------------------------------------------------------------------------
# This macro checks if $F77 is the GNU Fortran compiler. It sets the
# variable ``lf_cv_f77_is_g77'' with ``yes'' or ``no'' depending on
# the answer
# --------------------------------------------------------------------------

AC_DEFUN(LF_PROG_G77,[
  AC_MSG_CHECKING(whether ${F77-f77} is the GNU Fortran compiler)
  if $use_g77
  then
    lf_cv_f77_is_g77=yes
  else
    f77_output=`$F77 -v 2>&1 | grep "GNU F77"`
    if test -n "$f77_output"
    then
      lf_cv_f77_is_g77=yes
    else
      lf_cv_f77_is_g77=no
    fi
  fi
  AC_MSG_RESULT($lf_cv_f77_is_g77)
])

# --------------------------------------------------------------------------
# See what libraries are used by the Fortran compiler
# Write a minimal program and compiler it with -v. I don't know what to
# do if your compiler doesn't have -v
# You must call LF_HOST_TYPE before calling this macro.
# The result is returned in the variable FLIBS which is made
# available in Makefile.am
# ALSO: requires ac_cv_prog_gcc
# --------------------------------------------------------------------------

AC_DEFUN(LF_FLIBS,[
  AC_MSG_CHECKING(for Fortran libraries)
  dnl
  dnl Write a minimal program and compile it with -v. I don't know
  dnl what to do if your compiler doesn't have -v
  dnl
  changequote(, )dnl
  echo "      END" > conftest.f
  foutput=`${F77-f77} -v -o conftest conftest.f 2>&1`
  dnl
  dnl The easiest thing to do for xlf output is to replace all the commas
  dnl with spaces.  Try to only do that if the output is really from xlf,
  dnl since doing that causes problems on other systems.
  dnl
  xlf_p=`echo $foutput | grep xlfentry`
  if test -n "$xlf_p"; then
    foutput=`echo $foutput | sed 's/,/ /g'`
  fi
  dnl
  ld_run_path=`echo $foutput | \
    sed -n -e 's/^.*LD_RUN_PATH *= *\([^ ]*\).*/\1/p'`
  dnl
  dnl We are only supposed to find this on Solaris systems...
  dnl Uh, the run path should be absolute, shouldn't it?
  dnl
  case "$ld_run_path" in
    /*)
      if test "$ac_cv_prog_gcc" = yes; then
        ld_run_path="-Xlinker -R -Xlinker $ld_run_path"
      else
        ld_run_path="-R $ld_run_path"
      fi
    ;;
    *)
      ld_run_path=
    ;;
  esac
  dnl
  flibs=
  lflags=
  dnl
  dnl If want_arg is set, we know we want the arg to be added to the list,
  dnl so we don't have to examine it.
  dnl
  want_arg=
  dnl
  for arg in $foutput; do
    old_want_arg=$want_arg
    want_arg=
  dnl
  dnl None of the options that take arguments expect the argument to
  dnl start with a -, so pretend we didn't see anything special.
  dnl
    if test -n "$old_want_arg"; then
      case "$arg" in
        -*)
        old_want_arg=
        ;;
      esac
    fi
    case "$old_want_arg" in
      '')
        case $arg in
        /*.a)
          exists=false
          for f in $lflags; do
            if test x$arg = x$f; then
              exists=true
            fi
          done
          if $exists; then
            arg=
          else
            lflags="$lflags $arg"
          fi
        ;;
        -bI:*)
          exists=false
          for f in $lflags; do
            if test x$arg = x$f; then
              exists=true
            fi
          done
          if $exists; then
            arg=
          else
            if test "$ac_cv_prog_gcc" = yes; then
              lflags="$lflags -Xlinker $arg"
            else
              lflags="$lflags $arg"
            fi
          fi
        ;;
        -lang* | -lcrt0.o | -lc | -lgcc)
          arg=
        ;;
        -[lLR])
          want_arg=$arg
          arg=
        ;;
        -[lLR]*)
          exists=false
          for f in $lflags; do
            if test x$arg = x$f; then
              exists=true
            fi
          done
          if $exists; then
            arg=
          else
            case "$arg" in
              -lkernel32)
                case "$canonical_host_type" in
                  *-*-cygwin32)
                  ;;
                  *)
                    lflags="$lflags $arg"
                  ;;
                esac
              ;;
              -lm)
              ;;
              *)
                lflags="$lflags $arg"
              ;;
            esac
          fi
        ;;
        -u)
          want_arg=$arg
          arg=
        ;;
        -Y)
          want_arg=$arg
          arg=
        ;;
        *)
          arg=
        ;;
        esac
      ;;
      -[lLR])
        arg="$old_want_arg $arg"
      ;;
      -u)
        arg="-u $arg"
      ;;
      -Y)
  dnl
  dnl Should probably try to ensure unique directory options here too.
  dnl This probably only applies to Solaris systems, and then will only
  dnl work with gcc...
  dnl
        arg=`echo $arg | sed -e 's%^P,%%'`
        SAVE_IFS=$IFS
        IFS=:
        list=
        for elt in $arg; do
        list="$list -L$elt"
        done
        IFS=$SAVE_IFS
        arg="$list"
      ;;
    esac
  dnl
    if test -n "$arg"; then
      flibs="$flibs $arg"
    fi
  done
  if test -n "$ld_run_path"; then
    flibs_result="$ld_run_path $flibs"
  else
    flibs_result="$flibs"
  fi
  changequote([, ])dnl
  rm -f conftest.f conftest.o conftest
  dnl
  dnl Phew! Done! Now, output the result
  dnl
  FLIBS="$flibs_result"
  AC_MSG_RESULT([$FLIBS])
  AC_SUBST(FLIBS)
])

# -------------------------------------------------------------------------
# Some Linux systems require the -u MAIN__ flag to be passed to the 
# compiler. If so, the FORTRAN_MAIN_FLAG variable is assigned that
# flag and is exported to Makefile.am
# -------------------------------------------------------------------------

AC_DEFUN(LF_F77_MAIN_FLAG,[
  FORTRAN_MAIN_FLAG=
  case "$canonical_host_type" in
    *-linux-*)
      FORTRAN_MAIN_FLAG="-u MAIN__"
    ;;
  esac
  if test -n "$FORTRAN_MAIN_FLAG"; then
    AC_MSG_RESULT([defining FORTRAN_MAIN_FLAG to be $FORTRAN_MAIN_FLAG])
  fi
  AC_SUBST(FORTRAN_MAIN_FLAG)
])

# -------------------------------------------------------------------------
# Check whether the Fortran compiler uses uppercase external names.
# If it does, then we define the macro F77_UPPERCASE_NAMES
# Requires maybe the NM variable to be set to the nm program?
# -------------------------------------------------------------------------

AC_DEFUN(LF_F77_UPPERCASE_NAMES,[
  AC_MSG_CHECKING([whether $F77 uses uppercase external names])
  lf_cv_f77_uppercase_names=no
  cat > conftest.f <<EOF
      subroutine xxyyzz ()
      return
      end
EOF
  if ${F77-f77} -c conftest.f 1>&AC_FD_CC 2>&AC_FD_CC; then
    if test "`nm conftest.o | grep XXYYZZ`" != ""; then
      lf_cv_f77_uppercase_names=yes
    fi
  fi
  AC_MSG_RESULT([$lf_cv_f77_uppercase_names])
  if test "$lf_cv_f77_uppercase_names" = yes; then
    AC_DEFINE(F77_UPPERCASE_NAMES,1)
  fi
  rm -f conftest.f
])

# --------------------------------------------------------------------------
# Check whether the Fortran compiler appends underscores to external names
# If it does, then define the macro F77_APPEND_UNDERSCORE
# --------------------------------------------------------------------------

AC_DEFUN(LF_F77_APPEND_UNDERSCORE,[
  AC_MSG_CHECKING([whether $F77 appends underscores to external names])
  lf_cv_f77_append_underscore=no
  cat > conftest.f <<EOF
      subroutine xxyyzz ()
      return
      end
EOF
  if ${F77-f77} -c conftest.f 1>&AC_FD_CC 2>&AC_FD_CC; then
    if test "$lf_cv_f77_uppercase_names" = yes; then
      if test "`${NM-nm} conftest.o | grep XXYYZZ_`" != ""; then
        lf_cv_f77_append_underscore=yes
      fi
    else
      if test "`${NM-nm} conftest.o | grep xxyyzz_`" != ""; then
        lf_cv_f77_append_underscore=yes
      fi
    fi
  fi
  AC_MSG_RESULT([$lf_cv_f77_append_underscore])
  if test "$lf_cv_f77_append_underscore" = yes; then
    AC_DEFINE(F77_APPEND_UNDERSCORE, 1)
  fi
  rm -f conftest.f
])

# -----------------------------------------------------------------------
# See if the Fortran compiler is compatible with f2c/
# Write a minimal program, compile it, and see if it works as expected
# Requires calls to:
#  LF_PROG_G77
#  LF_FLIBS
#  LF_F77_APPEND_UNDERSCORE
# -----------------------------------------------------------------------

AC_DEFUN(LF_F2C_F77_COMPAT,[
  AC_MSG_CHECKING([$F77/f2c compatibility])
  trap 'rm -f ftest* ctest* core; exit 1' 1 3 15
  lf_cv_f2c_f77_compat=no
  cat > ftest.f <<EOF
      INTEGER FUNCTION FORSUB (C, D)
      CHARACTER *(*) C
      INTEGER L
      DOUBLE PRECISION D
      L = LEN (C)
      WRITE (*, '(A,1X,I2)') C(1:L), INT (D)
      FORSUB = 1
      RETURN
      END
EOF

  ${F77-f77} -c ftest.f 1>&AC_FD_CC 2>&AC_FD_CC
  changequote(, )dnl
  cat > ctest.c <<EOF
#include "confdefs.h"
static char s[14];
int main ()
{
  double d = 10.0;
  int len;
  strcpy (s, "FOO-I-HITHERE");
  len = strlen (s);
#ifdef F77_APPEND_UNDERSCORE
  return (! forsub_ (s, &d, len));
#else
  return (! forsub (s, &d, len));
#endif
}
#if defined (sun)
int MAIN_ () { return 0; }
#elif defined (linux) && defined (__ELF__)
int MAIN__ () { return 0; }
#endif
EOF
  changequote([, ])
  if ${CC-cc} -c ctest.c 1>&AC_FD_CC 2>&AC_FD_CC; then
    if ${CC-cc} -o ctest ctest.o ftest.o $FLIBS -lm 1>&AC_FD_CC 2>&AC_FD_CC; then
      ctest_output=`./ctest 2>&1`
      status=$?
      if test $status -eq 0 && test "$ctest_output" = "FOO-I-HITHERE 10"; then
        lf_cv_f2c_f77_compat=yes
      fi
    fi
  fi
  rm -f ftest* ctest* core
  AC_MSG_RESULT($lf_cv_f2c_f77_compat)
])

# -------------------------------------------------------------------------
# This macro checks whether the user has invoked incompatible 
# --with options
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_EXCLUSIVE_WITH_OPTIONS,[
  if test "${with_$1+set}" = set; then
    if test "${with_$2+set}" = set; then
      if test "$with_$2" = no; then
        true
      else
        $3
      fi
    fi
  fi
])

# -------------------------------------------------------------------------
# This is the macro that users want to call.
# This macro calls all the previous stuff and configures the 
# Fortran compiler.
# The configure options --with-f77, --with-g77, or --with-f2c
# force f77, g77 or f2c to be used. It is also possible to use
# these options to specify the name of the compiler. For example, 
# `--with-g77=/opt/gnu/bin/f77' says that we are using g77 but it is
# installed as /opt/gnu/bin/f77.
#
# Requires a call to
#   LF_HOST_TYPE
# -------------------------------------------------------------------------

AC_DEFUN(LF_CONFIGURE_FORTRAN,[
  dnl
  dnl Allow the user to force us to use a compiler of his choice
  dnl

  AC_ARG_WITH(f2c,
    [  --with-f2c              use f2c even if Fortran compiler is available],
    [if test "$withval" = no; then use_f2c=false; else use_f2c=true; fi],
    use_f2c=false)
  AC_ARG_WITH(g77,
    [  --with-g77              use g77 to compile Fortran subroutines],
    [if test "$withval" = no; then use_g77=false; else use_g77=true; fi],
    use_g77=false)
  AC_ARG_WITH(f77,
    [  --with-f77              use f77 to compile Fortran subroutines],
    [if test "$withval" = no; then use_f77=false; else use_f77=true; fi],
    use_f77=false)

  dnl
  dnl Make sure only one of the above options for Fortran compilers was
  dnl specified (multiple "no" or --without-FOO options are ok).
  dnl

  LF_CHECK_EXCLUSIVE_WITH_OPTIONS(f77, g77,
    [warn_f77_and_g77="--with-f77 and --with-g77 both specified!  Using g77..."
    AC_MSG_WARN($warn_f77_and_g77)
    use_g77=false])

  LF_CHECK_EXCLUSIVE_WITH_OPTIONS(f77, f2c,
    [warn_f2c_and_f77="--with-f2c and --with-f77 both specified!  Using f77..."
    AC_MSG_WARN($warn_f2c_and_f77)
    use_f2c=false])

  LF_CHECK_EXCLUSIVE_WITH_OPTIONS(g77, f2c,
    [warn_f2c_and_g77="--with-f2c and --with-g77 both specified!  Using g77..."
    AC_MSG_WARN($warn_f2c_and_g77)
    use_f2c=false])

  dnl If we haven't been forced to use a particular Fortran compiler,
  dnl try to find one using any one of several common Un*x Fortran
  dnl compiler names.  Put fc last to avoid confusion with some vendor's
  dnl /bin/sh fc builtin.
  dnl 
  dnl     f77 : most Unix systems
  dnl    xlf : IBM / AIX
  dnl   cf77 : Cray / Unicos
  dnl    g77 : GNU Fortran
  dnl     fc : Convex
  dnl
  dnl I don't think the Cray compiler will ever work like f2c....

  if $use_f77; then
    if test "$with_f77" = yes; then
      F77=f77
    else
      F77="$with_f77"
    fi
    AC_MSG_RESULT([defining F77 to be $F77])
  elif $use_g77; then
    if test "$with_g77" = yes; then
      F77=g77
    else
      F77="$with_g77"
    fi
    AC_MSG_RESULT([defining F77 to be $F77])
  elif $use_f2c; then
    F77=
    if test "$with_f2c" = yes; then
      F2C=f2c
    else
      F2C="$with_f2c"
    fi
    AC_MSG_RESULT([defining F2C to be $F2C])
  else
    AC_CHECK_PROGS(F77, g77 f77 f90 xlf cf77 fc)
  fi
 
  have_fortran_compiler=false
  have_f2c=false

  dnl
  dnl If we are forcing use of f2c, so be it. Otherwise 
  dnl investigate if we can actually use a native Fortran compiler
  dnl
  
  if $use_f2c; then
    F77=
  else
    if test -n "$F77"; then
      LF_PROG_G77
      LF_FLIBS
      LF_F77_APPEND_UNDERSCORE
      LF_F77_UPPERCASE_NAMES
      LF_F2C_F77_COMPAT
      if test "$lf_cv_f2c_f77_compat" = no; then
        F77=
      fi
    fi
  fi

  dnl
  dnl Conclusion: do we have a native fortran compiler or don't we?
  dnl If not, then use f2c. Set FLIBS manually, and since f2c uses
  dnl underscores, set the F77_APPEND_UNDERSCORE macro.
  dnl

  AC_MSG_CHECKING(whether to use f2c)
  if test -n "$F77"; then
    have_fortran_compiler=true
    AC_MSG_RESULT(no)
  else
    AC_MSG_RESULT(yes)
    AC_DEFINE(F77_APPEND_UNDERSCORE,1)
#    F2C="`pwd`/fortran/f2c/f2c"
    FLIBS="`pwd`/fortran/libf2c/libf2c.a -lm"
    F2CFLAGS="-f -g -A"
  fi
  dnl
  dnl Signal Makefile.am whether we're using f2c
  dnl This is mainly for the benefit of the Makefile.am that
  dnl will compile f2c if necessary.
  dnl

  AM_CONDITIONAL(USE_F2C,test x$have_fortran_compiler = xfalse)

  dnl
  dnl Define @fortran_target@ to the appropriate Fortran targets
  dnl Call to LF_MAKE_FORTRAN_TARGETS will generate files containing
  dnl the two possibilities. 
  dnl

  AC_MSG_CHECKING([which fortran makefile rules to use])
  if test x$have_fortran_compiler = xfalse
  then
    fortran_target="f2c_target.am"
  else
    fortran_target="f77_target.am"
  fi
  LF_MAKE_FORTRAN_TARGETS
  AC_SUBST_FILE(fortran_target)
  AC_MSG_RESULT([$fortran_target])

  dnl
  dnl Do the other substitutions
  dnl

dnl  AC_SUBST(F2C)
  AC_SUBST(F2CFLAGS)
  AC_SUBST(FLIBS)
  AC_SUBST(F77)
  AC_SUBST(FFLAGS)
])


# Define a conditional.

AC_DEFUN(AM_CONDITIONAL,
[AC_SUBST($1_TRUE)
AC_SUBST($1_FALSE)
if $2; then
  $1_TRUE=
  $1_FALSE='#'
else
  $1_TRUE='#'
  $1_FALSE=
fi])

dnl This macro has been generated by mkm4, written by Eleftherios Gkioulekas
dnl Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
dnl This file is free software; Eleftherios Gkioulekas gives unlimited
dnl permission to copy and/or distribute it, with or without modifications,
dnl as long as this notice is preserved. 
dnl
dnl This program is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY, to the extent permitted by law; without
dnl even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
dnl PARTICULAR PURPOSE.

AC_DEFUN(LF_MAKE_FORTRAN_TARGETS,[
changequote()dnl
# This is a shell archive (produced by GNU sharutils 4.1).
# To extract the files from this archive, save it to some FILE, remove
# everything before the `!/bin/sh' line above, then type `sh FILE'.
#
# Made on 1998-05-30 09:08 PDT by <skeletor@gondwana>.
# Source directory was `/home/skeletor/src/auto/autotools-0.5.0/m4'.
#
# existing files WILL be overwritten
# This format requires very little intelligence at unshar time.
# "echo", and "sed" may be needed.
#
# This shar contains:
# length mode       name
# ------ ---------- ------------------------------------------
#    213 -rw-r--r-- f2c_target.am
#    131 -rw-r--r-- f77_target.am
#
# ============= f2c_target.am ==============
  sed 's/^X//' << 'SHAR_EOF' > 'f2c_target.am' &&
X# This target is included in your Makefile.am if you want to build
X# Fortran code with f2c
X
X.f.o:
X	$(top_builddir)/fortran/f2c/f2c $(F2CFLAGS) $<
X	$(CC)  $(CFLAGS) -c `basename $< .f`.c
X	rm -f `basename $< .f`.c
X
SHAR_EOF
  : || echo 'restore of f2c_target.am failed'
# ============= f77_target.am ==============
  sed 's/^X//' << 'SHAR_EOF' > 'f77_target.am' &&
X# This target is included in your Makefile.am if you want to build
X# Fortran code with g77 of f77.
X
X.f.o:
X	$(F77) $(FFLAGS) -c $<
X
SHAR_EOF
  : || echo 'restore of f77_target.am failed'
changequote([,])dnl
])

