#if ! defined __C3377MODULEEVAL_H__
#define __C3377MODULEEVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c3377moduleeval.h,v 1.6 1999/02/05 18:35:00 steffen Exp $
*/

#include <config.h>
#include "tdcmoduleeval.h"

class C3377_TDCModuleEvalClass_t:public TDCModuleEvalClass_t
{
     enum 
     { 
	  NUMBER_OF_CHANNELS =32,
	  BYTES_PER_INFO_UNIT=2
     };
     ClassTypeInfo(TDCModuleEvalClass_t);
public:
     inline C3377_TDCModuleEvalClass_t(DaqProcessBase *Parent)
	  :TDCModuleEvalClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,Parent){}

     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
	  const void *ModuleData;
	  u_i32 MDat1, MDat2;
	  static u_i32 local_trignb=0;                // ***** later: global counter
	  
	  int NumReads;
	  u_i16 NumBytes;
	  int CurTDCChannel;
	  s_i32 CurOMChannel;
	  int NumHits;
	  int skip_this_module=0;
	  SubEventData_StaticSize_t *CurSubEvent;
	  
	  
	  RawDataBuffer.ReadFromBuffer(NumBytes);
	  NumReads=NumBytes/BytesPerInfoUnit;
	  
	  local_trignb++; 
	  if(Parent->IsVerboseLevel(10)){
	       // ********** temp
	       if (local_trignb%10000==1){
		    char tmp[256];
		    sprintf(tmp,"C3377 Local_trig = %d\n",local_trignb);
		    LogMsg(tmp,10,10);
	       }
	  }
	  // checking
	  if (0!=NumBytes%BytesPerInfoUnit) {
	       if(Parent->IsVerboseLevel(10)){
		    char tmp[256];
		    sprintf(tmp,"**** C3377 wrong Number of Bytes = %d\n",NumBytes);
		    LogMsg(tmp,10,10);
	       }
	       // read and skip
	       skip_this_module=1;
	       for (int i=0; i<NumBytes; i++){
		    ReadSkipByteFromBuffer(RawDataBuffer);
	       }
	  }
	  if (1!=NumReads%2) {
	       if(Parent->IsVerboseLevel(10)){
		    char tmp[256];
		    sprintf(tmp,"**** C3377 wrong Number of Bytes = %d\n",NumBytes);
		    LogMsg(tmp,10,10);
	       }
	       // read and skip
	       skip_this_module=1;
	       for (int i=0; i<NumBytes; i++){
		    ReadSkipByteFromBuffer(RawDataBuffer);
	       }
	  }

	  if (NumReads==0) {     
	       if(Parent->IsVerboseLevel(10)){
		    char tmp[256];
		    sprintf(tmp,"**** %d NumReads= %d Local_Trig = %d Stop Moduleeval ****\n",
			    GetTag(),NumReads,local_trignb);
		    LogMsg(tmp,10,10);
	       }
	       skip_this_module=1;
	  }

	  // ok, this module is obsolete - skip 
	  if (skip_this_module)	       return -1;

	  ModuleData=ReadFromBuffer(RawDataBuffer);   // Header Word, ignored temporarily
	  UnpackHeader(*(u_i32*)ModuleData);

	  NumHits=(NumReads-1)/2;
	  int NumReads_Test = 1;

	  for(int i=0;i<NumHits;i++) {
	       ModuleData=ReadFromBuffer(RawDataBuffer);
	       MDat1=*(u_i32*)ModuleData;              // crazy bugfix, was static pointer !
	       NumReads_Test++;

	       if (NumReads_Test==NumReads) {     
		    cout << "**** " <<GetTag()<<" NumReads= "<<NumReads_Test<<" Local_Trig = "
			 << local_trignb<<" Stop Moduleeval ****"<<endl<<flush;
		    return -1;
	       }

	       ModuleData=ReadFromBuffer(RawDataBuffer);
	       MDat2=*(u_i32*)ModuleData;
	       UnpackData(MDat1,MDat2);
	       CurTDCChannel=tdc_channel;
	       if((CurOMChannel=GetChannelMapping(CurTDCChannel))<0){
		    // error handling, couldn't map channel
		    char txt[1000];
		    sprintf(txt,"Can't map TDC3377 with tag %i channel %i to OM channel",GetTag(),CurTDCChannel);
		    LogMsg(txt,2,2,WARNING);
		    continue;
	       }
	       CurSubEvent=TempEventBuffer->GetChannelForWriting(CurOMChannel);  // is from  1 (!!), ... , NOM !!

	       // check if too many hits are already generated byt the TDC 
	       // this happens if TDCs behave wrongly 
	       if (CurSubEvent->NoOfEdges >= MAX_TIME_HITS) {      // serious hardware overflow error !!!
		    cout << " **** TDC3377, Tag= "<<GetTag()<<":  too many edges *************"
			 <<"  NumHits,hit,edge,time,OM = " << NumHits 
			 << " " << i << " " << tdc_edge << " " << tdc_timeword << " " 
			 << CurOMChannel << endl;
	       }
	       else
	       {
		    // hit accepted
		    CurSubEvent->Times[CurSubEvent->NoOfEdges].edge=tdc_edge;
		    CurSubEvent->Times[CurSubEvent->NoOfEdges++].timeval=tdc_timeword;
		    CurSubEvent->ID=CurOMChannel;
	       }
	       

//               cout << " TDC3377, Tag= "<<GetTag()<<": NumHits,hit,edge,time,OM = " << NumHits 
//	              << " "   << i << " " << tdc_edge << " " << tdc_timeword << " " 
//                    << CurOMChannel << endl;
	  }
	  return 0;
     }

protected:
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){
	  static u_i16 Data;
	  DataBuffer.ReadFromBuffer(Data);
	  return (void*)&Data;
     }
     // to skip with Byte-reads for corrupted data
     virtual const void* ReadSkipByteFromBuffer(BinaryDataBuffer &DataBuffer){
	  static u_i8 Data;
	  DataBuffer.ReadFromBuffer(Data);
	  return (void*)&Data;
     }

//   define UnpackHeader, UnpackData instead.
// skip the virtual functions (dummy here),
// use standard prototyp since virtual
     virtual inline int Channel(const void * /*tdc_data*/){
	  return tdc_channel;
     }
     virtual inline int Edge(const void * /*tdc_data*/){
	  return tdc_edge;
     }
     virtual inline int TimeVal(const void */*tdc_data*/){
	  return tdc_timeword;
     }

     /**  Analyze the Header for TDC Module 
     */ 
     int UnpackHeader(u_i32 tdc_data){
	  u_i32 hd_module_id,hd_evtnb,hd_double,hd_tag;
//	  u_i32 hd_edge_mode;
	  int errors=0;	  

	  hd_tag = 0x01 & (tdc_data>>15);
	  
	  if (1==hd_tag) {
	       hd_double = 0x01 & (tdc_data>>14);
	       if (1!=hd_double) {
		    errors++;
		    printf (" **** TDC3377 Tag= %d, error tag header: hd_double = 0x%x ***\n",GetTag(),hd_double);
	       }
	       hd_evtnb = 0x03 & (tdc_data>>11);
	       hd_module_id = 0xff & tdc_data;
	       
//	       DEBUG_LOCAL (<< " header: evtnb, module_id = " 
//	       << hd_evtnb << " " << hd_module_id << endl);
	       
	  }
	  else {
	       errors++;
	       printf (" **** TDC3377 Tag= %d, error: No header found  ***\n",GetTag());
	       }
	  return errors;
     }
     
     /** build the time/edge/channel inormation from raw tdc words 
	 using private variables & different functions than suggested 
	 Calibrate the time form 0.5nsec to nsec !!!
     */
     int UnpackData(u_i32 tdc_data1, u_i32 tdc_data2){
	  u_i32  da1_time,da2_time,da1_edge,da2_edge;
	  u_i32  da1_chan,da2_chan;
//	  u_i32  tdc_timeword,tdc_channel,tdc_edge,tdc_errors;
	  int errors=0;               // dummy so far
	  
	  // Assume: data byte ordering is correct (tags in correct order)
	  //   no error checks so far !!!
	  

	  // first data word = odd
	  
//	  da_tag  = 0x01 & (tdc_data1>>8); 
	  da1_edge = 0x01 & (tdc_data1>>9);
	  da1_chan = 0x1f & (tdc_data1>>10);
	  da1_time = 0x0ff & tdc_data1;
	  // printf ("da1: err,chan,edge,time= %d %d %d %d\n",
	  //         da_err,da1_chan,da1_edge,da1_time);
	  
   	  // second word = even
	  
//	  da_tag  = 0x01 & (tdc_data2>>8); 
	  da2_edge = 0x01 & (tdc_data2>>9);
	  da2_chan = 0x1f & (tdc_data2>>10);
	  da2_time = 0x0ff & tdc_data2;
	  // printf ("da2: err,chan,edge,time= %d %d %d %d\n",
	  //           da_err,da2_chan,da2_edge,da2_time);
	  
	       
	  // calibrate to nsec
	  tdc_timeword = 0xffff & ((da1_time << 8) | da2_time);  
	  tdc_timeword = tdc_timeword/2;
	  
	  tdc_edge = da1_edge;
	  tdc_channel = da1_chan;
	  tdc_errors=errors;

//	  DEBUG_LOCAL (<< "hit done. chan,edge,time,err=" 
//	   << tdc_channel<<" "<<tdc_edge<<" "<<tdc_timeword<<" "<<tdc_errors<<endl);
	  return errors;
     }

private:
     u_i32  tdc_timeword,
	  tdc_channel,
	  tdc_edge,
	  tdc_errors;
};

#endif // __C3377MODULEEVAL_H__
