/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c3377tdcmodule.C,v 1.4 1999/02/05 18:35:00 steffen Exp $
*/


#include <iostream.h> 
#include "c3377tdcmodule.h"
#include "crateclasses.h"
#include "channelmodule.h"
#include "datacollector.h"


#define DEBUG_LOCAL(A)  cout   A

// Special 3377 Checks .............
#define SIMUL_3377      // Simulate a fixed event from real data          !!! ONLY FOR DEBUGGING  !!!
#undef  SIMUL_3377     

#define CURE_ZERO_3377  // Cure Zero-events with a 'frozen' real event    !!! ONLY FOR DEBUGGING  !!!
#undef  CURE_ZERO_3377 

#define DEB_EVTHEAD_MODULES   //   test the debugging with event header in the modules
#undef  DEB_EVTHEAD_MODULES



C3377_TDCModule_t::C3377_TDCModule_t(DaqProcessBase *parent): /***/
     TDCModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,parent)
{
     DEBUG_MESSAGE( << "Module created" << endl);
}

C3377_TDCModule_t::~C3377_TDCModule_t(){
     DEBUG_MESSAGE( << "Module destroyed" << endl);
}      

int C3377_TDCModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
//     DEBUG_LOCAL(   << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
	  
     // for the number of 16bit words read from the TDC
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
//     u_i32 tdcdata;
     u_i16  FIFO[MAX_FIFO_3377];          // 32Chan*16Hits*2words + 1 Header
     u_i16 NbyteFIFO=0;
#if VME_ACCESS
     u_i32 data, status, Q, X;
#endif
     static u_i32 local_trignb=0;    // ****** replace if global trigger number available
     

     /* buffer readout */
     
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumWords);

     // read out the Module FIFO   (without polling - converison time assured by trigger)

     local_trignb++;                       // ***** temp
//     if (local_trignb%10000==1)  cout << "C3377 Local_trig= "<< local_trignb
//				      << "................" << endl;     

#ifdef  DEB_EVTHEAD_MODULES              // we want to get eventnumber here         ******* temp
//     cout <<  ".. now printdummy"<<endl;
//     ((DataCollector*) Parent)->PrintDummy();
#endif     

// version 1  -  Event Ready
//     if (HasModuleData())     {
//	  Read_C3377_FIFO(FIFO,NbyteFIFO);
//     }
//     else{                                 // this should never happen !!!!
//          cout << "**** " << GetModuleConfig()->ModuleID 
//	       << " HasModuleData = NO, Local_trig="<<local_trignb
//	       << " *********** " << endl;
//     }

// version 2   - LAM & BIP ready
#if VME_ACCESS
     int bip_loop=0;
     while (1) { 
	  *cnaf_f27a0 = 0;                             // Test BIP, Q=0 if data ready
	  status = *cnaf_cbv_status;
	  Get_status_QX(status,Q,X);
	  if (!Q) break;
	  bip_loop++;
	  if (!bip_loop%1000) cout<<".......BIP_loop = "<<bip_loop<<" for "<<GetModuleConfig()->ModuleID<<endl;
     }
#endif
     Read_C3377_FIFO(FIFO,NbyteFIFO);



     // checking
     if (NbyteFIFO>=MAX_FIFO_3377){                               
	  cout << "**** " << GetModuleConfig()->ModuleID 
	       << " too many 3377 words read = " << NbyteFIFO << endl;
	  NbyteFIFO=MAX_FIFO_3377;
     }
     if (!NbyteFIFO) cout << "**** " << GetModuleConfig()->ModuleID 
			  << " ZERO FIFO CONTENT, Local_trig="<<local_trignb
			  << " *************"<< endl;
     
#ifdef CURE_ZERO_3377

     // avoid "ZERO Events"
     static  u_i32 simul=0, Nsimul=0;
     static SIMUL_FIFO[5];
     // freeze 'experimental' input from a first 'good' one
     if (simul==0 && NbyteFIFO==5) {
	  simul=1;
	  for (int i=0; i<NbyteFIFO; i++){
	       SIMUL_FIFO[i] = FIFO[i];
	  }
	  cout << " simul event defined ...........************ .................\n";
     }
     // use 'experimental input' for ZERO-Events
     if (!NbyteFIFO && simul==1){
	  Nsimul++;
	  if (Nsimul)  cout << "Cure_zero Event = " << Nsimul 
			    << " ................ " << endl;
	  NbyteFIFO=5;
	  for (int i=0; i<NbyteFIFO; i++){
	       FIFO[i]=SIMUL_FIFO[i];
	  }
     }
#endif
     
     C3377_Decode(FIFO,NbyteFIFO);             // decoding and some printout
     
     for (int i=0;i<NbyteFIFO;i++){            // 16bit-wise 
	  DataBuffer.WriteToBuffer(FIFO[i]);
	  NumWords++;
     }

     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);

     // Clear LAM & data 
#ifdef VME_ACCESS
     data = *cnaf_f9a0;               // no q response for 3377         // *** temp   goes to event/module reset
#endif     
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read" << endl);
//     DEBUG_LOCAL  ( << "Module " << GetModuleConfig()->ModuleID << " read." << endl);

     return 0;
}


#ifdef VME_ACCESS       // ..................................................................................


bool C3377_TDCModule_t::HasModuleData(){
     // This is the standard version of C3377 readout:
     //    No  polling - converison time is assured by trigger.
     // More advanced (but slower) would be:
     //    wait if "Buff in progress" or "Busy"  (with timeout)
     u_i32 status,Q,X;
     bool has=false;
//
//  version 1 - get event ready status
//
//     if (GetModuleConfig()->Enabled){
//	  *cnaf_f27a2 = 0;                             // Test Event Ready, Q=1 if data ready
//	  status = *cnaf_cbv_status;
//	  Get_status_QX(status,Q,X);
//	  if (Q) has=true;	  
//     }
//  version 2 - LAM check  & BIP-end in readout
//
     if (GetModuleConfig()->Enabled){
	  u_i32 data = *cnaf_f8a0;                             // Test Event Ready, Q=1 if data ready
	  Get_data_QX (data,Q,X);
	  if (Q) has=true;	  
     }
     return has;
}


int C3377_TDCModule_t::InitModule(StringVector_t &/*errors*/){

     u_i32       data,status,Q,X;
     long int    *cnaf_vsb;
     int         errors=0;
     time_t      useconds;
     
     int Station_Number = GetModuleConfig()->Address_Slot;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();       //better: cast only if correct type 
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     int NChan  = GetNoOfChannels();


// first all the pointers

     cnaf_cbv_status = Crate->CAMAC_Crate_Pointer (28, 0,27);
     
     cnaf_f0a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 0); // ReadFIFO
     cnaf_f8a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 8); // TestLAM
     cnaf_f9a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 9); // ClearDataLAM
     cnaf_f26a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,26); // EnableLAM
     cnaf_f26a1  = Crate->CAMAC_Crate_Pointer (Station_Number, 1,26); // EnableAquMode
     cnaf_f27a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,27); // TestBuffProgr
     cnaf_f27a1  = Crate->CAMAC_Crate_Pointer (Station_Number, 1,27); // TestBUSY
     cnaf_f27a2  = Crate->CAMAC_Crate_Pointer (Station_Number, 2,27); // TestEventReady

     cnaf_f17a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,17); // WriteContReg_0
     cnaf_f17a1  = Crate->CAMAC_Crate_Pointer (Station_Number, 1,17); // WriteContReg_1
     cnaf_f17a2  = Crate->CAMAC_Crate_Pointer (Station_Number, 2,17); // WriteContReg_2
     cnaf_f17a3  = Crate->CAMAC_Crate_Pointer (Station_Number, 3,17); // WriteContReg_3

     cnaf_f1a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 1); // ReadContReg_0
     cnaf_f1a1   = Crate->CAMAC_Crate_Pointer (Station_Number, 1, 1); // ReadContReg_1
     cnaf_f1a2   = Crate->CAMAC_Crate_Pointer (Station_Number, 2, 1); // ReadContReg_2
     cnaf_f1a3   = Crate->CAMAC_Crate_Pointer (Station_Number, 3, 1); // ReadContReg_3
	  

// do the XLINX programming

//     DEBUG_LOCAL ( << "going init TDC  " << GetModuleConfig()->ModuleID<<endl);

     data = *cnaf_f9a0;                                          // Clear
     Get_data_QX (data,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F9A0  Q,X= " << Q << " " << X);
     
     data = *Crate->CAMAC_Crate_Pointer (Station_Number,0,30);   // Programming Mode
     Get_data_QX (data,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F30A0  Q,X= " << Q << " " << X);

     *Crate->CAMAC_Crate_Pointer (Station_Number,0,22) = data;   // Select EPROM Mode 2
     status = *cnaf_cbv_status;
     Get_status_QX (status,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F22A0  Q,X= " << Q << " " << X);

     *Crate->CAMAC_Crate_Pointer (Station_Number,0,25) = data;   // Do Programming & Wait

     useconds = 500000;     // 200 msec recommended
     usleep (useconds);

     cnaf_vsb = Crate->CAMAC_Crate_Pointer (Station_Number,0,13); // check module ready
     
     Q=0;
     for (int i=0;0==Q;i++) {
	  data = *cnaf_vsb;
	  Get_data_QX (0xC0000000 & data, Q,X);
//	  if (!i%1000) DEBUG_LOCAL (<< " loop init_tdc  i= " << i << endl);
     }

//     DEBUG_LOCAL( << "Module loaded ok" << endl);
     
     // set Control Registers 0 - 3
     errors+=SetContRegister_Stand(Station_Number);

     if (errors) {
	  cout << "*****************************************" << endl;
	  cout << "** "<< errors << "Errors initializing TDC " << \
	       GetModuleConfig()->ModuleID << endl;
	  cout << "*****************************************" << endl;
     }
     else      {

	  cout  << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
//		<< Station_Number << " NChan/On= " << NChan << "  initialized." << endl;
		<< Station_Number << " NChan= " << NChan << "  initialized." << endl;
     }


     // has to go to event-loop init   ********************
     
     *cnaf_f26a0=0;   // enable LAM
     *cnaf_f26a1=0;   // enable Aquisition mode

     return 0;
}

/** Sets the TDC Control Register to Standard: CommStop / L&TEdge / SingleEventMode /FullDataFormat   */

int C3377_TDCModule_t::SetContRegister_Stand(int Station_Number){
     
     int     errors = 0;     
     int     module_id,edge_mode,read_mode,buff_mode,head_mode;
     int     max_hit,max_time; 
     u_i32   contreg00,contreg01,contreg02,contreg03,X,Q;
     u_i32   data,status;

     
     // setting the control register
     
     // control register 0 
     
     module_id = 0xFF & Station_Number;   // module ID = Camac N
     edge_mode = 1;                       // 1 = both edges
     read_mode = 0;                       // 0 = Camac
     //     buff_mode = 1;                       // 1 = multi_evt buffer mode
     buff_mode = 0;                       // 0 = single event mode
     head_mode = 0;                       // 0= always header
     
     contreg00 = (0x00ff & module_id) | ((0x01 & edge_mode)<<10) |
	  ((0x01 & read_mode)<<11) | ((0x01 & buff_mode)<<12) |
	  ((0x01 & head_mode)<<13);
     
     *cnaf_f17a0 = contreg00;     // write to Control Register 0
     status = *cnaf_cbv_status;
     Get_status_QX(status,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F17A0  Q,X= " << Q << " " << X);

     data = 0;
     data = *cnaf_f1a0;           // read back 
     if (contreg00 != (0x1FFF & data)){
	  errors++;
	  cout << " *** error setting tdc_control register 0" << endl; 
     }
//     DEBUG_LOCAL ( << " cont_reg00, written/read = " << (void*)contreg00 
//     << " " << (void*)(0x0FFFF & data) << endl);
     
     
     // control register 1
     
     contreg01 = 0;
     
     *cnaf_f17a1 = contreg01;     // write to Control Register 1
     status = *cnaf_cbv_status;
     Get_status_QX(status,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F17A1  Q,X= " << Q << " " << X);

     data = 0;
     data = *cnaf_f1a1;           // read back 
     if (contreg01 != (0x1FFF & data)){        
	  errors++;
	  cout << " *** error setting tdc_control register 1" << endl; 
     }
//     DEBUG_LOCAL ( << " cont_reg01, written/read = " << (void*)contreg01 
//     << " " << (void*)(0x0FFFF & data) <<endl);
	    
		  
     // control register 2
		  
     max_hit  = 0;      // 0=maxhit = 16 
     max_time = 0xfff;  // in 8nsec
     
     contreg02 = (0x000f & max_hit) | ((0xfff & max_time)<<4);
     
     *cnaf_f17a2 = contreg02;     // write to Control Register 2
     status = *cnaf_cbv_status;
     Get_status_QX(status,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F17A2  Q,X= " << Q << " " << X);

     data = 0;
     data = *cnaf_f1a2;           // read back 
     if (contreg02 != (0xFFFF & data)){
	  errors++;
	  cout << " *** error setting tdc_control register 2" << endl; 
     }
//     DEBUG_LOCAL ( << " cont_reg02, written/read = " << (void*)contreg02 
//     << " " << (void*)(0x0FFFF & data) << endl);
		  
		  
     // control register 3
		  
     contreg03 = 0;
     
     *cnaf_f17a3 = contreg03;     // write to Control Register 3
     status = *cnaf_cbv_status;
     Get_status_QX(status,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F17Ax  Q,X= " << Q << " " << X);

     data = 0;
     data = *cnaf_f1a3;           // read back 
     if (contreg03 != (0x1FFF & data)){
	  errors++;
	  cout << " *** error setting tdc_control register 3" << endl; 
     }
//     DEBUG_LOCAL ( << " cont_reg03, written/read = " << (void*)contreg03 
//     << " " << (void*)(0x0FFFF & data) << endl);
     
     if (!errors) {
//	  DEBUG_LOCAL(<< "Control Registers set ok." << endl);
     }
     else{
	  DEBUG_LOCAL(<< "**** Control Registers settting: " \
	  << errors <<" errors found."<< endl);
     }
     return errors;
}	

/** reads the 3377 FIFO to local buffer */

int C3377_TDCModule_t::Read_C3377_FIFO(u_i16 *FIFO_3377, u_i16 &NbyteRead){

     int     errors=0;
     u_i32   data,Q,X; 

#ifdef SIMUL_3377               // only for debugging !
     static  u_i32 simul=0, Nsimul=0;
     static SIMUL_FIFO[5];

     // use 'experimental input'
     if (simul==1){
	  Nsimul++;
	  if (Nsimul%10000==1)  cout << "Simul Event = " << Nsimul << endl;
	  NbyteRead=5;
	  for (int i=0; i<NbyteRead; i++){
	       FIFO_3377[i]=SIMUL_FIFO[i];
	  }
	  return errors;
     }
#endif

     for (NbyteRead=0; ; NbyteRead++) {
	  data = *cnaf_f0a0;
	  Get_data_QX(data,Q,X);
	  if (!Q)  break;                         // Q=0: first dummy word        
	  if (NbyteRead >= MAX_FIFO_3377) {
	       cout << " **** "<<GetModuleConfig()->ModuleID<< ": too many FIFO reads = "<<NbyteRead<<endl;
	       errors++;
	       break;
	  }
	  errors += 0x01&(!Q) + 0x01&(!X);
	  FIFO_3377[NbyteRead] = 0xffff & data;
//	  printf("FIFO,NbyteRead,data Q,X= %d 0x%08x 0x%x 0x%x\n",NbyteRead,data,Q,X);
//	  printf("%s FIFO, NbyteRead,data Q,X, err=  %d 0x%08x 0x%x 0x%x %d\n",
//		 GetModuleConfig()->ModuleID,NbyteRead,data,Q,X);
     }


#ifdef SIMUL_3377               // only for debugging !
     // just simulate the input for all following events from a first 'good' one
     cout << " NbyteRead = " << NbyteRead <<endl;
     if (simul==0 && NbyteRead==5) {
	  simul=1;
	  for (int i=0; i<NbyteRead; i++){
	       SIMUL_FIFO[i] = FIFO_3377[i];
	  }
	  cout << " simul event defined .................********...........\n";
     }
#endif     

     return errors;
}


#else    // VME_ACCESS      ..................................................................................


bool C3377_TDCModule_t::HasModuleData(){
     // dummy
     return true;
}

int C3377_TDCModule_t::InitModule(StringVector_t &/*errors*/){

     int         errors=0;
     
     int Station_Number = GetModuleConfig()->Address_Slot;
     int NChan = GetNoOfChannels();

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();          
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     cout << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
//	  << Station_Number << " NChan/On= " << NChan << "  initialized. (SIMUL)" << endl;
	  << Station_Number << " NChan= " << NChan << "  initialized. (SIMUL)" << endl;
    
     return errors;
}


/** Sets the TDC Control Register to Standard: CommStop / L&TEdge / FullDataFormat   */

int C3377_TDCModule_t::SetContRegister_Stand(int /* Station_Number */){
     
     int     errors = 0;     
     return errors;
}	

/** Emaulates reading the 3377 FIFO to local buffer */

int C3377_TDCModule_t::Read_C3377_FIFO(u_i16 *FIFO_3377, u_i16 &NbyteRead){

     int     errors=0;
     u_i16   header, word1, word2;
     u_i16   chan, time, edge;     


//  emulate 1 hit with header     

     NbyteRead = 0;
     chan = 15;

     // header
     header = 0;
     header |= (0x0FF | 99);                      // ModuleID        dummy
     header |= (0x1 << 10);                       // both edges
     header |= (0x1 << 14);                       // double word readout
     header |= (0x1 << 15);                       // header tag
     FIFO_3377[NbyteRead]=header;
     NbyteRead++;
              
     // hit 1
     time = 199;     
     edge = 0;                                     // leading edge
     
     word1 = 0;
     word1 |= (0xFF|time)>>8;                        // time value     
     word1 |= (0x1 << 8);                            // word 1
     word1 |= (0x0 << 9);                            // leading edge
     word1 |= ((0x1f & chan) << 10);                 // channel number

     word2 = 0;
     word2 |= (0xFF | time) ;                        // time value     
     word2 |= (0x0 << 8);                            // word 2
     word2 |= (0x0 << 9);                            // leading edge
     word2 |= ((0x1f & chan) << 10);                 // channel number
     
     FIFO_3377[NbyteRead]=word1;
     NbyteRead++;
     FIFO_3377[NbyteRead]=word2;
     NbyteRead++;
     
     // hit 2
     time = 99;     
     edge = 1;                                       // trailing edge

     word1 = 0;
     word1 |= (0xFF|time)>>8;                        // time value     
     word1 |= (0x1 << 8);                            // word 1
     word1 |= (0x0 << 9);                            // leading edge
     word1 |= ((0x1f & chan) << 10);                 // channel number

     word2 = 0;
     word2 |= (0xFF | time) ;                        // time value     
     word2 |= (0x0 << 8);                            // word 2
     word2 |= (0x0 << 9);                            // leading edge
     word2 |= ((0x1f & chan) << 10);                 // channel number
     
     FIFO_3377[NbyteRead]=word1;
     NbyteRead++;
     FIFO_3377[NbyteRead]=word2;
     NbyteRead++;
     
     for (int i=0; i<NbyteRead; i++) {
	  printf("%s FIFO SIMUL, NbyteRead, i, data=  %d  %d  0x%x \n",
		 GetModuleConfig()->ModuleID,NbyteRead,i,FIFO_3377[i]);
     }
     return errors;
}

#endif   // VME_ACCESS    ..................................................................................



/**  decode 3377 words and some printout  */

int C3377_TDCModule_t::C3377_Decode(u_i16 *FIFO, u_i16 NbyteFIFO){
     
//     u_i32  hd_module_id,hd_edge_mode,hd_evtnb,hd_double,hd_tag;
     u_i32  hd_module_id,hd_evtnb,hd_double,hd_tag;
     u_i32  da1_time,da2_time,da_edge,da1_edge,da2_edge;
     u_i32  da_chan,da1_chan,da2_chan,da_tag,da_err;
     u_i32  timeword,chan,edge;
     u_i32  data;

//   DEBUG_LOCAL ( << "DECODE TDC Module " << GetModuleConfig()->ModuleID <<  endl);
   
     
     // decoding the data

//     if (!NbyteFIFO) cout << "**** " << GetModuleConfig()->ModuleID 
//			  << " ZERO FIFO CONTENT ******" << endl;
     
     // loop the buffer: all formal checks & get chan/edge/time
     da_err = 0;
     for (int j=0;j<NbyteFIFO;j++) {
	  data = FIFO[j];
	  // printf (" buffer j, data = %d 0x%08x \n",j,data);
	  
	  hd_tag = 0x01 & (data>>15);
	  
	  // header 
	  if (1==hd_tag) {
	       hd_double = 0x01 & (data>>14);
	       if (1!=hd_double || 0!=j) {
		    cout << " **** "<<GetModuleConfig()->ModuleID<<": error tag header: hd_double,j"<<endl;
	       }
	       hd_evtnb = 0x03 & (data>>11);
	       hd_module_id = 0xff & data;
	       
//	       DEBUG_LOCAL (<< " header: evtnb, module_id = " 
//	       << hd_evtnb << " " << hd_module_id << endl);

	       
	  }
	  // data word
	  if (0==hd_tag) {
	       if (0==j) {
		    cout << " **** "<<GetModuleConfig()->ModuleID<<": error tag data: j=0 "<<endl;
		    da_err++;
	       }
	       
	       da_tag  = 0x01 & (data>>8); 
	       da_edge = 0x01 & (data>>9);
	       da_chan = 0x1f & (data>>10);
	       
	       // first data word = odd
	       if (1==j%2) {
		    if (1!=da_tag) {
			 cout << " **** "<<GetModuleConfig()->ModuleID<<": error odd data word order "<<endl;
			 da_err++;
		    }
		    da1_time = 0x0ff & data;
		    da1_chan = da_chan;
		    da1_edge = da_edge;
		    // printf ("da1: err,chan,edge,time= %d %d %d %d\n",\
			 //         da_err,da1_chan,da1_edge,da1_time);
	       }
			   // second word = even
	       else {        
		    if (0!=da_tag) {
			 cout << " **** "<<GetModuleConfig()->ModuleID<<": error even data word order "<<endl;
			 da_err++;
		    }
		    da2_time = 0x0ff & data;
		    da2_chan = da_chan;
			   da2_edge = da_edge;
			   // printf ("da2: err,chan,edge,time= %d %d %d %d\n",\
			 //           da_err,da2_chan,da2_edge,da2_time);
	       }
	       
	       // final check, build & reset  every even data word
	       
	       if (0==j%2 && 0!=j) {
		    if (!da_err) {
			 if (da1_chan!=da2_chan || da1_edge!=da2_edge) da_err++;
		    }
		    
		    if (!da_err){
			 timeword = 0xffff & ((da1_time << 8) | da2_time);  
			 edge = da1_edge;
			 chan = da1_chan;
//			 DEBUG_LOCAL (<< "hit done. mod_id,chan,edge,time,err=" 
//			 << hd_module_id << " " 
//			 << chan<<" "<<edge<<" "<<timeword<<" "<<da_err<<endl);
		    }
		    
		    da_err = 0;
		    da1_chan = da2_chan = 0;
		    da1_time = da2_time = 0;
		    da1_edge = da2_edge = 0;
	       }
	  }
     }	// for 
     

//     DEBUG_LOCAL (<< "event done. " << endl);
     
     return 0;

}  // decoding the data

int C3377_TDCModule_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}
