/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c3377tdcmodule.h,v 1.4 1999/02/03 03:42:11 steffen Exp $
*/

#if ! defined __C3377TDCMODULE_H__
#define  __C3377TDCMODULE_H__

#include <config.h>
#include "tdcmodule.h"

#define  MAX_FIFO_3377 32*16*2+1          // 32Chan*16Hits*2words + 1 Header

class C3377_TDCModule_t:public TDCModuleClass_t
{
     ClassTypeInfo(TDCModuleClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 32,
	  BYTES_PER_INFO_UNIT=2        //  header word, 2 data words per hit (& log-info)
     };
public:
     
     C3377_TDCModule_t(DaqProcessBase *Parent);/***/
     ~C3377_TDCModule_t();
     
     /** returns true, if the module has data or false if the module
         does not have any data */
     virtual bool HasModuleData();

     /** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);
     	
     /** this function initializes the TDC module */
     virtual int InitModule(StringVector_t &errors);

     /** Resets the module 
       @param errors Vector of errors that may occur during initialization.
       @return 0 on success. Negative number of errors occured during 
       initialization on failure.*/
     virtual int ResetModule(StringVector_t &errors);

private:
     /** get Q and X from data read in (inline) */
     inline void Get_data_QX (u_i32 data, u_i32 &Q, u_i32 &X) {
	  Q = data >> 31;
	  X = 0x01 & (data >> 30);
	  return;
     }
     /** get Q and X from the CC status word read in (inline) */
     inline void Get_status_QX (u_i32 status, u_i32 &Q, u_i32 &X) {
	  Q = 0x01 & (status >> 1);
	  X = 0x01 & status;
	  return;
     }

     /** Sets the TDC Control Register to Standard AMANDA Mode: 
	 CommStop / L&TEdge / FullDataFormat   */
     int SetContRegister_Stand(int Station_Number);

     /** reads the 3377 FIFO to local buffer */
     inline int Read_C3377_FIFO(u_i16 *FIFO_3377, u_i16 &NbyteRead);

     /**  decode 3377 words and some printout  */
     int C3377_Decode(u_i16 *FIFO, u_i16 NbyteFIFO);


private:

     long int *cnaf_cbv_status;
     long int *cnaf_f0a0,
	      *cnaf_f1a0,*cnaf_f1a1,*cnaf_f1a2,*cnaf_f1a3,
	      *cnaf_f8a0,
	      *cnaf_f9a0,
              *cnaf_f26a0,*cnaf_f26a1,
	      *cnaf_f27a0,*cnaf_f27a1,*cnaf_f27a2,
	      *cnaf_f17a0,*cnaf_f17a1,*cnaf_f17a2,*cnaf_f17a3; 
};

MAKE_REFCOUNT_POINTER(C3377_TDCModule_t, C3377_TDCModule_p);


#endif // __C3377TDCMODULE_H__
