/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c63iregmodule.C,v 1.4 1999/02/05 18:35:00 steffen Exp $
*/


#include <iostream.h> 
#include "c63iregmodule.h"
#include "crateclasses.h"
#include "channelmodule.h"
#include "datacollector.h"

#define DEBUG_LOCAL(A)  cout   A


//C63_IREGModule_t::C63_IREGModule_t(CommBase_t *parent): 
C63_IREGModule_t::C63_IREGModule_t(DaqProcessBase *parent): 
     IREGModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,parent)
{
     DEBUG_MESSAGE( << "Module created" << endl);
     events_this_c63=0;
}

C63_IREGModule_t::~C63_IREGModule_t(){
     DEBUG_MESSAGE( << "Module destroyed" << endl);
}      

int C63_IREGModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
//     DEBUG_LOCAL(   << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
	  
     // for the number of 16bit words read from the IReg
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
#ifdef VME_ACCESS
     u_i32 status,Q,X;
#endif
     u_i32 data;
     int errors=0;
//     static u_i32 local_trignb=0;    // ****** replace if global trigger number available

     
     /* buffer readout */
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumWords);

     events_this_c63++;

     // read out the Module 

#ifdef VME_ACCESS
     data = *cnaf_f0a0;
     Get_data_QX (data,Q,X);
//     DEBUG_LOCAL (<< "C63IREG  event, data,  Q,X= " << events_this_c63<<" "<<(void*)data<<" "<< Q <<" "<< X<<endl);
     errors+=0x01&(!Q) + 0x01&(!X);
#else   
     
     data = 0xcf;         // simulate
     
#endif  // VME_ACCESS

     if (events_this_c63%10000==1)  cout <<"C63IReg Local_trig#, Reg= "<<events_this_c63<<" "<< (void*) (0xffff&data)
				      << "  ................" << endl;     
//     non-Amanda-B
//     if ( (0xfd&data) )
//	  DEBUG_LOCAL (<< " IReg LocalTrig#, Reg = " << events_this_c63 <<" "<< (void*) (0xffff&data)<<endl);
     
     
     DataBuffer.WriteToBuffer(data);
     NumWords++;

     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();

//     DEBUG_LOCAL (<<"IREG: NumWords,NumBytes = "<<NumWords<<" "<<NumBytes<<endl);

     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);


     // has to go to event-loop init   ********************


#ifdef VME_ACCESS     
     *cnaf_f9a0=0;   // clear 
     status = *cnaf_cbv_status;
     Get_status_QX (status,Q,X);
//     errors+=0x01&(!Q) + 0x01&(!X);                                  // No Q from F9 on C63
//     DEBUG_LOCAL (<< "C63IREG F9A0,  Q,X= " << Q << " " << X <<endl);
#endif

     if (errors) cout << "**** " << GetModuleConfig()->ModuleID 
		      << " Error during read, Local_trig, Reg, err="<<events_this_c63<<" "<<(void*)(0xFFFF&data)
		      <<" "<<errors << " *************"<< endl;     
     
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read" << endl);
//     DEBUG_LOCAL  ( << "Module " << GetModuleConfig()->ModuleID << " read." << endl);

     return 0;
}


#ifdef VME_ACCESS


bool C63_IREGModule_t::HasModuleData(){

     // no flag available for this module 
     return true;
}

int C63_IREGModule_t::InitModule(StringVector_t &/*errors*/){

//     u_i32       status,Q,X;
     int         errors=0;
     
     
     int Station_Number = GetModuleConfig()->Address_Slot;
     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();       //better: cast only if correct type 
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);


//  all pointers

     cnaf_cbv_status = Crate->CAMAC_Crate_Pointer (Station_Number, 0,27);
     cnaf_f0a0       = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 0); // Read Module
     cnaf_f9a0       = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 9); // ClearDataLAM
     cnaf_f16a0      = Crate->CAMAC_Crate_Pointer (Station_Number, 0,16); // EnableRegisters

     *cnaf_f16a0 = 0xfffff;
//     status = *cnaf_cbv_status;
//     Get_status_QX (status,Q,X);
//     errors+=0x01&(!Q) + 0x01&(!X);                      // we dont get Q on F16A0 

     if (errors) {
	  cout << "*****************************************" << endl;
	  cout << "** "<< errors << "Errors initializing TDC " << \
	       GetModuleConfig()->ModuleID << endl;
	  cout << "*****************************************" << endl;
     }
     else      {
	  cout  << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
		<< Station_Number << "  initialized." << endl;
     }
     

     // has to go to event-loop init   ********************
     // *cnaf_f9a0=0;   // clear 

     return 0;
}

int C63_IREGModule_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

#else    // VME_ACCESS


bool C63_IREGModule_t::HasModuleData(){
     // dummy
     return true;
}

int C63_IREGModule_t::InitModule(StringVector_t &/*errors*/){

     int         errors=0;
     
     int Station_Number = GetModuleConfig()->Address_Slot;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();          
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     cout << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
	  << Station_Number << "  initialized. (SIMUL)" << endl;
    
     return errors;
}

int C63_IREGModule_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

#endif   // VME_ACCESS


