/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c63iregmodule.h,v 1.4 1999/02/03 03:42:12 steffen Exp $
*/

#if ! defined __C63IREGMODULE_H__
#define  __C63IREGMODULE_H__

#include <config.h>
#include "iregmodule.h"

class C63_IREGModule_t: public IREGModuleClass_t
{
     /** Macro defined in pointerutil.h */
     ClassTypeInfo(IREGModuleClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 1,
	  BYTES_PER_INFO_UNIT=4        //      keep full data word  (QX)
     };
public:
     /** Constructor.
	 @param numChannels Number of channels of this module.
	 @param BytesPerInfoUnit Number of bytes per info unit. */
     C63_IREGModule_t(DaqProcessBase *Parent);
     ~C63_IREGModule_t();
     
     /** returns true, if the module has data or false if the module
         does not have any data */
     virtual bool HasModuleData();

     /** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);
     	
     /** this function initializes the TDC module */
     virtual int InitModule(StringVector_t &errors);

     /** Resets the module 
       @param errors Vector of errors that may occur during initialization.
       @return 0 on success. Negative number of errors occured during 
       initialization on failure.*/
     virtual int ResetModule(StringVector_t &errors);


private:
     /** get Q and X from data read in (inline) */
     inline void Get_data_QX (u_i32 data, u_i32 &Q, u_i32 &X) {
	  Q = data >> 31;
	  X = 0x01 & (data >> 30);
	  return;
     }
     /** get Q and X from the CC status word read in (inline) */
     inline void Get_status_QX (u_i32 status, u_i32 &Q, u_i32 &X) {
	  Q = 0x01 & (status >> 1);
	  X = 0x01 & status;
	  return;
     }

     long int *cnaf_cbv_status;
     long int *cnaf_f0a0,
	  *cnaf_f9a0,
	  *cnaf_f16a0;     

     u_i32 events_this_c63;            // local trigger counter
};

MAKE_REFCOUNT_POINTER(C63_IREGModule_t, C63_IREGModule_p);

#endif // __C63IREGMODULE_H__








