/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c7164adcmodule.C,v 1.5 1999/02/05 18:35:00 steffen Exp $
*/

#include <iostream.h> 
#include "c7164adcmodule.h"
#include "crateclasses.h"
//#include "moduleclasses.h"
#include "adcmodule.h"

#define DEBUG_LOCAL(A)  cout   A


C7164_ADCModule_t::C7164_ADCModule_t(DaqProcessBase *parent): 
     ADCModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,parent)
{
     DEBUG_MESSAGE( << "Module created" << endl);
}

C7164_ADCModule_t::~C7164_ADCModule_t(){
     DEBUG_MESSAGE( << "Module destroyed" << endl);
}      


#ifdef VME_ACCESS

bool C7164_ADCModule_t::HasModuleData(){
     // This is the standard version of C7164 readout:
     // No  polling for LAM - converison time is assured by trigger.
     u_i32  status,Q,X;
     bool   yes=false;

     if (GetModuleConfig()->Enabled){
	  *cnaf_f8 = 0;                                          // Test LAM
	  status = *cnaf_cbv_status;                             // get the status
	  Get_status_QX(status,Q,X);
	  if (Q) yes=true;	  
     }
     DEBUG_MESSAGE(   << "HasModuleData " << GetModuleConfig()->ModuleID << " = " << yes << endl);

     return yes;
}

int C7164_ADCModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
//     DEBUG_LOCAL(   << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
	  
     // for the number of 16bit words read from the ADC
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
//     u_i32 adcdata;
     u_i16  FIFO[MAX_FIFO_7164];          // 32Chan*16Hits*2words + 1 Header
     u_i16 NbyteFIFO=0;

     /* buffer readout */
     
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumWords);

     // read out the Module FIFO   (without polling - converison time assured by trigger)

//     if (HasModuleData())     Read_C7164_FIFO(FIFO,NbyteFIFO);         // redundant
     Read_C7164_FIFO(FIFO,NbyteFIFO);
     if (NbyteFIFO>MAX_FIFO_7164){
	  cout << "**** " << GetModuleConfig()->ModuleID << " too many FIFO read in = " << NbyteFIFO << endl;
	  NbyteFIFO = MAX_FIFO_7164;
     }
     if (!NbyteFIFO) cout << "**** " << GetModuleConfig()->ModuleID 
			  << " ZERO FIFO CONTENT ****" << endl;
     
     C7164_Decode(FIFO,NbyteFIFO);                   // decoding and some printout
     
     for (int i=0;i<NbyteFIFO;i++){                  // 16bit-wise writiting
	  DataBuffer.WriteToBuffer(FIFO[i]);
	  NumWords++;
     }
     
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);

     // Clear LAM           *********  temp, should go to module reset

     //     u_i32 dumm = *cnaf_f10;    // does not suppress empty modules                                  
     u_i32 dumm = *cnaf_f11a1;         // just trying ...

     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read" << endl);
//     DEBUG_LOCAL( << "Module " << GetModuleConfig()->ModuleID << " read." << endl);

     return 0;
}


int C7164_ADCModule_t::InitModule(StringVector_t &/*errors*/){

     u_i32       data,Q,X;
     int         errors=0;

     int        nch=16;
     long int   test_data;
	  
     pedestal = ((ADCModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->Ped;
     threshold_low = ((ADCModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->ThreshLow;
     threshold_up = ((ADCModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->ThreshUp;
     // threshold_low=10;
     // threshold_up=4095;
 
     int Station_Number = GetModuleConfig()->Address_Slot;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();     
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);
     int NChan  = GetNoOfChannels();
     
// first all the pointers

//              ************ bIG BUG *************************   ?????????????????????
//     cnaf_cbv_status = Crate->CAMAC_Crate_Pointer (Station_Number, 0,27); // getting controller QX for write cmd's
     cnaf_cbv_status = Crate->CAMAC_Crate_Pointer (28, 0,27);         // getting controller QX for write cmd's

     cnaf_f0a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 0); // Read Chan 1    (for test)
     cnaf_f0a15  = Crate->CAMAC_Crate_Pointer (Station_Number,15, 0); // Read Chan 16   (for test)
     cnaf_f4a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 4); // Read spares data
     cnaf_f8     = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 8); // Test LAM
     cnaf_f9     = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 9); // Clear Module
     cnaf_f10    = Crate->CAMAC_Crate_Pointer (Station_Number, 0,10); // Clear LAM
     cnaf_f11a1  = Crate->CAMAC_Crate_Pointer (Station_Number, 1,11); // Reset Hit Register and LAM
     cnaf_f26a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,26); // Enable LAM
     cnaf_f17a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,17); // Select Memory 
     cnaf_f17a1  = Crate->CAMAC_Crate_Pointer (Station_Number, 1,17); //  -"-
     cnaf_f17a2  = Crate->CAMAC_Crate_Pointer (Station_Number, 2,17); //  -"-
     cnaf_f6a0   = Crate->CAMAC_Crate_Pointer (Station_Number, 0, 6); // Read ContReg
     cnaf_f19a0  = Crate->CAMAC_Crate_Pointer (Station_Number, 0,19); // Set  ContReg


     // Clear

     data = *cnaf_f9;                                          // Clear
     Get_data_QX (data,Q,X);
     errors+=0x01&(!Q) + 0x01&(!X);
//     DEBUG_LOCAL (<< " F9A0  Q,X= " << Q << " " << X << endl);

     // Set Upper, Lower Thresholds & Pedestal 

     for (int ich=0;ich<nch;ich++){

	  *cnaf_f17a1=0;            // select Lower Thr
	  *Crate->CAMAC_Crate_Pointer (Station_Number,ich,20) = threshold_low;   
	  test_data = 0x3fffffff & *Crate->CAMAC_Crate_Pointer (Station_Number,ich,1);
//	  DEBUG_LOCAL (<<" Ich, Low_thres set/read = "<<ich<<" "<<threshold_low<<" "<<test_data<<endl);
	  
	  *cnaf_f17a2=0;            // select Upper Thr
	  *Crate->CAMAC_Crate_Pointer (Station_Number,ich,20) = threshold_up;   
	  test_data = 0x3fffffff & *Crate->CAMAC_Crate_Pointer (Station_Number,ich,1);
//	  DEBUG_LOCAL (<<" Ich, Up_thres set/read = "<<ich<<" "<<threshold_up<<" "<<test_data<<endl);
	  
	  *cnaf_f17a0=0;            // select Pedestal
	  *Crate->CAMAC_Crate_Pointer (Station_Number,ich,20) = pedestal;   
	  test_data = 0x3fffffff & *Crate->CAMAC_Crate_Pointer (Station_Number,ich,1);
//	  DEBUG_LOCAL (<<" Ich, Pedestal = "<<ich<<" "<<pedestal<<" "<<(short int) (0xffff&test_data)<<endl);
     }   // ich

     // set control register

     *cnaf_f19a0=0x7;  //   0=PED, 1=LT, 2=UT    
     test_data=*cnaf_f6a0;  // read control register
     
//     printf ("Control Register read (On=1,OFF=0):");
//     printf ("PED=%1ld LowT=%1ld UpT=%1ld  ",0x1&(test_data>>0),0x1&(test_data>>1),0x1&(test_data>>2));
//     printf ("ConvDelay=%ld \n",0xFF&(test_data>>8));
     if ( (test_data&0x07) != 0x07){
	  errors++;
	  cout << "**** GetModuleConfig()->ModuleID , error with settings: "
	       << " PED= "<<(0x1&test_data)<<" LowT= "<<(0x1&(test_data>>1))<<" UpT= "<< (0x1&(test_data>>2))
	       <<endl;
     }
     
     if (errors) {
	  cout << "*****************************************" << endl;
	  cout << "** "<< errors << "Errors initializing ADC " << \
	       GetModuleConfig()->ModuleID << endl;
	  cout << "**  Press any key to continue " << endl;
	  cout << "*****************************************" << endl;
     }
     else {
	  cout << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
//	       << Station_Number << " NChan/On= " << NChan 
	       << Station_Number << " NChan= " << NChan 
	       << "  TLow/Up/Ped= " << threshold_low <<"/  "<<threshold_up<<"/ "
	       << pedestal <<"  initialized." << endl;
     }

     // has to go to event-loop init  (NextEvent) ********************

     *cnaf_f26a0=0;   // enable LAM
//     DEBUG_LOCAL (<<"LAM enabled\n");

     return 0;
}

/** reads the 7164 FIFO to local buffer */

int C7164_ADCModule_t::Read_C7164_FIFO(u_i16 *FIFO_7164, u_i16 &NbyteRead){

     int     errors=0;
     u_i32   data,Q,X; 

     for (NbyteRead=0; ; NbyteRead++) {
	  data = *cnaf_f4a0;
	  Get_data_QX(data,Q,X);
	  if (!Q)  break;
	  if (NbyteRead > MAX_FIFO_7164) {
	       cout << " **** "<<GetModuleConfig()->ModuleID<< ": too many FIFO reads = "<<NbyteRead<<endl;
	       break;
	  }
	  errors += 0x01&(!Q) + 0x01&(!X);
	  FIFO_7164[NbyteRead] = 0xFFFF & data;
//	  printf("%s FIFO, NbyteRead,data Q,X, err=  %d 0x%08x 0x%x 0x%x \n",
//		 GetModuleConfig()->ModuleID,NbyteRead,data,Q,X);
     }
     return errors;

} // Read_C7164_FIFO


#else   // VME_ACCESS ..........................................................................................

bool C7164_ADCModule_t::HasModuleData(){
     // Dummy
     bool   yes=true;

//     DEBUG_LOCAL(   << "HasModuleData " << GetModuleConfig()->ModuleID << " = " << yes << endl);
     return yes;
}

int C7164_ADCModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
//     DEBUG_LOCAL(   << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
	  
     // for the number of 16bit words read from the ADC
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
//     u_i32 adcdata;
     u_i16  FIFO[MAX_FIFO_7164];          // 32Chan*16Hits*2words + 1 Header
     u_i16 NbyteFIFO=0;

     /* buffer readout */
     
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumWords);

     // read out the Module FIFO   (without polling - converison time assured by trigger)

     if (HasModuleData())     Read_C7164_FIFO(FIFO,NbyteFIFO);
     if (NbyteFIFO>MAX_FIFO_7164){
	  cout << "**** " << GetModuleConfig()->ModuleID << " too many FIFO read in = " << NbyteFIFO << endl;
     }
     if (!NbyteFIFO) cout << "**** " << GetModuleConfig()->ModuleID 
			  << " ZERO FIFO CONTENT ****" << endl;

     
     C7164_Decode(FIFO,NbyteFIFO);                   // decoding and some printout
     
     for (int i=0;i<NbyteFIFO;i++){                  // 16bit-wise writiting
	  DataBuffer.WriteToBuffer(FIFO[i]);
	  NumWords++;
     }
     
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);

     

     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read (Simul)" << endl);
     DEBUG_LOCAL( << "Module " << GetModuleConfig()->ModuleID << " read (Simul)" << endl);

     return 0;
}


int C7164_ADCModule_t::InitModule(StringVector_t &/*errors*/){

     int  errors=0;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();     
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     int Station_Number = GetModuleConfig()->Address_Slot;
     int NChan  = GetNoOfChannels();
     threshold_low=  9;
     threshold_up=  99;
//     pedestal =    999;
//*****
     pedestal = ((ADCModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->Ped;
//     cout << " .... pedestal read = "<< ttt_ped<<endl;


//     ADCModuleConfigRecord_t CurM;
//     CurM=&ModuleConfigRecord;
//     int y = CurM.Ped;
//     cout << "......pedestal read = "<<y<<endl;
     
     

     
     cout << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
//	  << Station_Number << " NChan/On= " << NChan 
	  << Station_Number << " NChan= " << NChan 
	  << "  TLow/Up/Ped= " << threshold_low <<"/  "<<threshold_up<<"/ "
	  << pedestal <<"  initialized (SIMUL)." << endl;
     return errors;
}

/** Simulates reading the 7164 FIFO to local buffer */

int C7164_ADCModule_t::Read_C7164_FIFO(u_i16 *FIFO_7164, u_i16 &NbyteRead){

     int     errors=0;
     u_i32   data; 

     int NcSim=16;     
     for (int i=0;i< NcSim ; i++) {
	  data = (i<<12) | i;
	  FIFO_7164[NbyteRead] = 0xFFFF & data;
//	  DEBUG_LOCAL ( <<GetModuleConfig()->ModuleID<< ": FIFO Simul, i,data = "<<i<<" "<<(void*) data<<endl);
     }
     NbyteRead = NcSim;
     return errors;

} // Read_C7164_FIFO


#endif  // VME_ACCESS


/**  decode 7164 words and some printout  */

int C7164_ADCModule_t::C7164_Decode(u_i16 *FIFO, u_i16 NbyteFIFO){
     
     u_i32  amp, chan;
     u_i16  adc_data;

     // decoding the data
//     if (!NbyteFIFO) cout << "**** " << GetModuleConfig()->ModuleID 
//			  << " ZERO FIFO CONTENT ****" << endl;

     for (int i=0; i<NbyteFIFO; i++){
	  adc_data = FIFO[i];
	  chan = Channel(adc_data);
	  amp  = AmpVal(adc_data);
//	  DEBUG_LOCAL (<< GetModuleConfig()->ModuleID 
//                       <<" i, chan, amp =" <<i<<" "<<chan<<" "<<amp<<endl);
     }
     return 0;

}  // C7164_Decode

int C7164_ADCModule_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

