/*
  $Header: /home/elrond/baikal/src/CVS/daq98/c7164adcmodule.h,v 1.4 1999/02/03 03:42:12 steffen Exp $
*/

#if ! defined __C7164ADCMODULE_H__
#define  __C7164ADCMODULE_H__

#include <config.h>
#include "adcmodule.h"

#define  MAX_FIFO_7164 16                   // 16Chan*1word

class C7164_ADCModule_t:public ADCModuleClass_t
{
     ClassTypeInfo(ADCModuleClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 16,
	  BYTES_PER_INFO_UNIT=2        //  2 data words per hit 
     };
public:
     
     C7164_ADCModule_t(DaqProcessBase *Parent); /***/
     ~C7164_ADCModule_t();
     
     /** returns true, if the module has data or false if the module
         does not have any data */
     virtual bool HasModuleData();

     /** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);
     	
     /** this function initializes the ADC module */
     virtual int InitModule(StringVector_t &errors);

     /** Resets the module 
       @param errors Vector of errors that may occur during initialization.
       @return 0 on success. Negative number of errors occured during 
       initialization on failure.*/
     virtual int ResetModule(StringVector_t &errors);

private:
     /** get Q and X from data read in (inline) */
     inline void Get_data_QX (u_i32 data, u_i32 &Q, u_i32 &X) {
	  Q = data >> 31;
	  X = 0x01 & (data >> 30);
	  return;
     }
     /** get Q and X from the CC status word read in (inline) */
     inline void Get_status_QX (u_i32 status, u_i32 &Q, u_i32 &X) {
	  Q = 0x01 & (status >> 1);
	  X = 0x01 & status;
	  return;
     }

     /** reads the 7164 data to local buffer */
     inline int Read_C7164_FIFO(u_i16 *FIFO_7164, u_i16 &NbyteRead);

     /**  decode 7164 words and some printout  */
     int C7164_Decode(u_i16 *FIFO, u_i16 NbyteFIFO);

     /**  extract channel information */
     static inline int Channel(const u_i16 adc_data){
	  return (adc_data >> 12) & 0xF;
     }
     
     /**  extract Amplitude information */
     static inline int AmpVal(const u_i16 adc_data){
	  return adc_data & 0x0FFF;
     }

private:

     long int   *cnaf_cbv_status;                        // getting controller QX for write cmd's
     long int   *cnaf_f0a0;                              // read Chan 1     (for test)
     long int   *cnaf_f0a15;                             // read Chan 16    (for test)
     long int   *cnaf_f4a0;                              // read sparse data
     long int   *cnaf_f9;                                // clear Module
     long int   *cnaf_f10;                               // clear LAM
     long int   *cnaf_f11a1;                             // clear Hit Register and LAM
     long int   *cnaf_f8,*cnaf_f26a0;                    // Test/Enable LAM   
     long int   *cnaf_f17a0,*cnaf_f17a1,*cnaf_f17a2;     // select memory
     long int   *cnaf_f6a0,*cnaf_f19a0;                  // read,set cont_reg
     long int   *cnaf_f1ax,*cnaf_f20ax;                  // read/write memory
     
     int        nchan_on;                                // the number of switched on channels on the module
     int        threshold_low, threshold_up, pedestal;   // thresholds and pedestal for sparse data 
//     long int   low_thr=800, up_thr=4095, ped=100;

};

MAKE_REFCOUNT_POINTER(C7164_ADCModule_t, C7164_ADCModule_p);



#endif // __CAMACADCMODULE_H__
