/*
  $Header: /home/elrond/baikal/src/CVS/daq98/camacgpsmodule.C,v 1.4 1999/02/05 18:35:00 steffen Exp $
*/

#include "camacgpsmodule.h"
#include "crateclasses.h"

/*---------------------------------------------------------*/
/*                                                         */
/*  CAMAC GPS module class                                 */
/*  Member function definitions                            */
/*                                                         */
/*---------------------------------------------------------*/

CAMAC_GPSModuleClass_t::CAMAC_GPSModuleClass_t(DaqProcessBase *parent)
     :GPSModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT, parent){
}

CAMAC_GPSModuleClass_t::~CAMAC_GPSModuleClass_t(){
}

int CAMAC_GPSModuleClass_t::ReadToBuffer(BinaryDataBuffer &DataBuffer){
     // for the number of  9Byte Words   read from the Module
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
	  
     // write the binary tag
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumBytes); 

     // read out the module
     // Minor Time	  
     // The minor time is a binary number that represents the number of clock 
     // cycles since the last 1PPS pulse. For the standard 10MHz input a clock
     // cycle would be 100 nanoseconds. This number should range from 0 to
     // 10,000,000 base 10, or 000000 to 989680 in base 16. 
     u_i32	data0 = *cnaf_f0a0;     

     // Low Major Time
     // The low major time register holds the hours, minutes, and seconds,
     // MSB to LSB respectively, for the current time. As sample time would 
     // be as follows:
     // MSBit                         LSBit
     // 0010 0011  0100 0111  0101 1001 (234759 HEX) represents 23h 47min 59sec
     //  Hours      Minutes    Seconds
     u_i32	data1 = *cnaf_f0a1;
     
     // High Major Time & Status
     // This word holds the days of the current time, the status of the
     // GPS receiver output signal, and the GPS Time Quality Character.
     // A sample output would be as follows:
     // MSBit                         LSBit
     // 0000 1111  0000 0010  0110 1000 (0F0268 HEX) represents Good Status and 268 days
     //  SB2  SB1   TQC D100  D10   D1  Where:
     // 
     // SB2 is Status Byte 2 and is currently reserved and should always read 0000
     // SB1 is Status Byte 1
     //     Bit 1 (LSBit) 1PPS Bit: is high if 1PPS signal is present
     //     Bit 2         10MHz Bit: is hight if 10MHz bit is present
     //     Bit 3         COM Bit: is hight if RS232 information received beenvalid 
     //                            and incrementing
     //     Bit 4 (MSBit) TIME Bit: is high if RS232 information received has matched 
     //                             internal time        
     // TQC is TimeQuality Character (0 is best, F is worst)

     u_i32	data2 = *cnaf_f2a2;    // to clear
	  

     // write the data words
     u_i8 writeByte;
     writeByte=data0&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data0>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data0>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
	  
     writeByte=data1&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data1>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data1>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);

     writeByte=data2&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data2>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data2>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);

     NumWords+=1;
	  
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);
     return 0;
}       
     
int CAMAC_GPSModuleClass_t::InitModule(StringVector_t &/*errors*/)  {
	  
     int slot = GetModuleConfig()->Address_Slot;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();      
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     cnaf_f0a0 = Crate->CAMAC_Crate_Pointer (slot,0,0);
     cnaf_f0a1 = Crate->CAMAC_Crate_Pointer (slot,1,0);
     cnaf_f0a2 = Crate->CAMAC_Crate_Pointer (slot,2,0);
     cnaf_f2a2 = Crate->CAMAC_Crate_Pointer (slot,2,2);
	  
     char i[1024];
     sprintf(i,"Module %s CrateID %i:  Adr= %i initialized",
	     GetModuleConfig()->ModuleID,CrateNumber,slot);
     LogMsg(i,5,5,INFO);
     return 0;
}
