/*
  $Header: /home/elrond/baikal/src/CVS/daq98/camacgpsmodule97.C,v 1.3 1999/02/03 03:42:12 steffen Exp $
*/

#include "camacgpsmodule97.h"

CAMAC_GPSModule97Class_t::CAMAC_GPSModule97Class_t(DaqProcessBase *parent)/***/
     :GPSModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT, parent){
}

CAMAC_GPSModule97Class_t::~CAMAC_GPSModule97Class_t(){}
     
/** returns true, if the module has data or false if the module
    does not have any data */
     
     // should not be used for GPS Modules 
bool CAMAC_GPSModule97Class_t::HasModuleData(){
     return true;
}

/** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
int CAMAC_GPSModule97Class_t::ReadToBuffer(BinaryDataBuffer &DataBuffer){
     // for the number of  9Byte Words   read from the Module
     u_i16 NumWords=0;
     u_i16 NumBytes=0;
	  
     // write the binary tag
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumBytes); 

     // read out the module
     // Minor Time	  
     // The minor time is a binary number that represents the number of clock 
      // cycles since the last 1PPS pulse. For the standard 10MHz input a clock
     // cycle would be 100 nanoseconds. This number should range from 0 to
     // 10,000,000 base 10, or 000000 to 989680 in base 16. 
     u_i32	data0 = *cnaf_f0a0;     

     // Low Major Time
     // The low major time register holds the hours, minutes, and seconds,
     // MSB to LSB respectively, for the current time. As sample time would 
     // be as follows:
     // MSBit                         LSBit
     // 0010 0011  0100 0111  0101 1001 (234759 HEX) represents 23h 47min 59sec
     //  Hours      Minutes    Seconds
     u_i32	data1 = *cnaf_f0a1;
     
     // High Major Time & Status
     // This word holds the days of the current time, the status of the
     // GPS receiver output signal, and the GPS Time Quality Character.
     // A sample output would be as follows:
     // MSBit                         LSBit
     // 0000 1111  0000 0010  0110 1000 (0F0268 HEX) represents Good Status and 268 days
     // TQ/S  Y10   Y1  D100  D10   D1  Where:
     // 
     // TQ/S is combination of Time Quality Character and Status Byte
     //  when MSB is high status is good
     //  TQC      MSB  Bit 3  Bit 2  Bit 1 (LSB)
     //  1 (best)  1     0      0      1
     //  2         1     0      1      0
     //  3         1     0      1      1 
     //  4         1     1      0      0
     //  5         1     1      0      1
     // 
     //  when input fails tyhe MSB will go low
     //     Bit 1 (LSBit) 1PPS Bit: is high if 1PPS signal is present
     //     Bit 2         10MHz Bit: is high if 10MHz bit is present
     //     Bit 3         COM Bit: is hight if RS232 information received 
     //                   been valid and incrementing

     u_i32	data2 = *cnaf_f2a2;    // to clear
	  

     // write the data words
     u_i8 writeByte;
     writeByte=data0&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data0>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data0>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
	  
     writeByte=data1&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data1>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data1>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);

     writeByte=data2&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data2>>8)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);
     writeByte=(data2>>16)&0xFF;
     DataBuffer.WriteToBuffer(writeByte);

     NumWords+=1;
	  
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);
     return 0;
}       
     
/** this function initializes the module 
	 CCrate Init is done here for Crate Nb. =1 (perlim.) 
     */
int CAMAC_GPSModule97Class_t::InitModule(StringVector_t &/*errors*/)  {
	  
     int Station_Number = GetModuleConfig()->Address_Slot;

     CAMAC_Crate_Class_t *Crate;
     Crate=(CAMAC_Crate_Class_t*) GetCrate();      
     StringSplitter sp(Crate->GetCrateConfig()->ConnectedTo,' ');
     int CrateNumber=atoi(sp()[0]);

     cnaf_f0a0 = Crate->CAMAC_Crate_Pointer (Station_Number,0,0);
     cnaf_f0a1 = Crate->CAMAC_Crate_Pointer (Station_Number,1,0);
     cnaf_f0a2 = Crate->CAMAC_Crate_Pointer (Station_Number,2,0);
     cnaf_f2a2 = Crate->CAMAC_Crate_Pointer (Station_Number,2,2);
	  
     cout  << "Module " << GetModuleConfig()->ModuleID << " CrateID "<< CrateNumber << ":  Adr= " 
	   << Station_Number << "  initialized." << endl;

     return 0;
}

int CAMAC_GPSModule97Class_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

