/***************************************************************
 *
 * Structure definitions etc for the
 * PVSB 8464 PCI-VSB Master Bridge
 */
#define Tempo(t) ppc603_set_dec( (t) << 3); while( ppc603_get_dec() > 0)

#define _INIT_REG( R, a, v, d, b ) \
	R.Address = (a); \
	R.Value   = (v); \
	R.Default = (d); \
	R.Size    = (b);


/*
 * _CPU returns the PCI address of a PPC SysRam address,
 */
#define _CPU( x ) (long)(x) | SYSRAM_PCI_BASE

/*#define _CPU_BASE	  0x01f80000 */
#define _CPU_BASE	  0x00300000 
#define _CACHE_SIZE	  0x2000		/*  8 KBytes */
#define _CACHE_WORDS	 _CACHE_SIZE/sizeof(long)
#define _CPU_DEST	(_CPU_BASE + _CACHE_SIZE)
#define _SCRIPT		(_CPU_BASE + 0x6000)
#define _PCI_BASE	 _CPU ( _CPU_BASE )
#define _PCI_DEST	 _CPU ( _CPU_DEST )
#define _LOOP		  110
#define WIN_BASE         0xD4000000
#define DISPLAY_BASE        0xD4007700

#ifdef  KSEG1
#undef  KSEG1
#endif
#define KSEG1 0





/*
 * A structure to hold an offset address, a value, a default
 * value, and the size in bytes of a hardware register.
 */
typedef struct {
  long Address;
  long Value;
  long Default;
  long Size;
} _Register;

/*
 * Interesting registers for the PICOLLO Bridge
 */
typedef struct {
  _Register PCR;  /* Primary Command		   */
  _Register PSR;  /* Primary Status		   */
  _Register CLS;  /* Cache Line Size		   */
  _Register LTR;  /* Latency Timer		   */
  _Register BAR1; /* Base Address Register 1 	   */
  _Register BAR2; /* Base Address Register 2	   */
  _Register PBN;  /* Primary Bus Number		   */
  _Register SBN;  /* Secondary Bus Number	   */
  _Register UBN;  /* Subordinate Bus Number	   */
  _Register SLTR; /* Secondary Latency Timer	   */
  _Register IBR;  /* I/O Base 			   */
  _Register ILR;  /* I/O Limit 			   */
  _Register SPSR; /* Secondary Status Register 	   */
  _Register MBR;  /* Memory Base 		   */
  _Register MLR;  /* Memory Limit 	 	   */
  _Register PMBR; /* Pre-fetchable Memory Base     */
  _Register PMLR; /* Pre-fetchable Memory Limit    */
  _Register IUBR; /* IO Memory Base     	   */
  _Register IULR; /* IO Memory Limit    	   */
  _Register BCR;  /* Bridge Control 		   */
  _Register AMCR; /* Address-Map Control 	   */
  _Register PDBC; /* Primary Data-Buffer Control   */
  _Register SDBC; /* Secondary Data-Buffer Control */
  _Register WCCR; /* Write-Combining Control	   */
  _Register CMAD; /* Config Map Access Data	   */
  _Register MCAA; /* Memory map Access Address	   */
  _Register ICAA; /* I/O map Access Address	   */
  _Register PRTR; /* Primary Retry-Timer	   */
  _Register SRTR; /* Secondary Retry-Timer	   */
  _Register RCR;  /* Retry Control		   */
  _Register AMR;  /* Arbiter Mode		   */
  _Register RAC0; /* PCI REQ#0 Arbitration Control */
  _Register RAC1; /* PCI REQ#1 Arbitration Control */
  _Register RAC2; /* PCI REQ#2 Arbitration Control */
  _Register RAC3; /* PCI REQ#3 Arbitration Control */
  _Register RAC4; /* PCI REQ#4 Arbitration Control */
  _Register RAC5; /* PCI REQ#5 Arbitration Control */
  _Register RAC6; /* PCI REQ#6 Arbitration Control */
} _Bridge;

/*
 * Interesting registers for the VSB Master
 */
typedef struct {
  _Register PCR;    /* Primary Command			*/
  _Register PSR;    /* Primary Status			*/
  _Register VCR;    /* VSB Command Register		*/
  _Register VSR;    /* VSB Status Register		*/
  _Register VHED;   /* VSB Host-Entry Descriptor	*/
  _Register VDMAED; /* VSB DMA-Entry Descriptor		*/
  _Register VIHED;  /* VSB Interrupt-Handler Descriptor	*/
  _Register VIMR;   /* VSB Interrupt Mask		*/
  _Register VISR;   /* VSB Interrupt Status		*/
  _Register VBAR;   /* VSB Base Address			*/
} _VsbMaster;

/*
 * A structure to read the fixed (R/O) parameters of a device
 */
typedef struct {
  _Register VID;
  _Register DID;
  _Register RID;
  _Register CCR;
  _Register HDT;
  _Register BIST;
} _PciDevice;

/* Function prototypes, I love 'em! */
void SetBrParam(    _Bridge *bridge ),
     SetBrDefaults( _Bridge *bridge ),
     ShowBrParam(   _Bridge *bridge ),
     InitBrParam(   _Bridge *bridge ),
     WriteBrParam(  _Bridge *bridge, long base ),
     ReadBrParam(   _Bridge *bridge, long base );

void SetVsbParam(    _VsbMaster *vsb ),
     SetVsbDefaults( _VsbMaster *vsb ),
     ShowVsbParam(   _VsbMaster *vsb ),
     InitVsbParam(   _VsbMaster *vsb ),
     WriteVsbParam(  _VsbMaster *vsb, long base ),
     ReadVsbParam(   _VsbMaster *vsb, long base );

void SetPciParam(    _PciDevice *PciDevice ),
     SetPciDefaults( _PciDevice *PciDevice ),
     InitPciParam( _PciDevice *PciDevice );

void ShowRegister(  _Register *r, char *name, char *desc, long RO ),
     WriteRegister( _Register *r, long base ),
     ReadRegister(  _Register *r, long base ),
     SetBaseAddresses( void ),
     ShowPciDevice( long base, long config );
long GetRegister( _Register *r, char *name ),
     GetHexValue( char *s, long *address );

/*
 * A structure to pass parameter to vsb_test_routines
 */
typedef struct {
		long src_add;
		long des_add;
		long size;
		long opsize;
		long gen_mode;
		long cpt_mode;
		long chk_mode;
		long initial;
		long mask;
		long printok;
		long param1;
		long param2;
		long param3;
		long param4;
		long basaddr;
		long tstaddr;
		long script;
		long burst_siz;
		long siom;
		long diom;
		long time;	
}TstParam;


