/***************************************************************/
/*                                                             */
/*                 RTPC8067 HEADER FILES                       */
/*                                                             */
/***************************************************************/
/*                                                             */
/*               module  : rtpc8067.h                          */
/*               version : 1.00                                */
/*               author  : J-F GILOT                           */
/*               creation:                                     */
/*                                                             */
/*                                                             */
/***************************************************************/
/*                                                             */
/*      Copyright 1994, CES Creative Electronic System SA      */
/*        70, route du Pont-Butin, CH-1213 Petit-Lancy         */
/*                   All Rights Reserved                       */
/*                                                             */
/***************************************************************/
/*                                                             */
/*               Modification History                          */
/*                                                             */
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/
/*                                                             */
/*                                                             */
/*                                                             */
/***************************************************************/


/* PCI bridge identifier */

#define IBM_BRIDGE       0x82000   /* bridge type                   */

/* PPC603 BOOT PROM */

#define EPROM_BASE       0xffc00000   /* eprom base                   */
#define LOC0_REG         0x81000004   /* local PPC register           */

/* PPC603 SYSTEM MEMORY */

#define HOST_MEM_BASE    0x00000000   /* host system ram              */

/* PCI RESOURCES (PCI MEMORY SPACE MAPPING)*/

#ifdef OLD_SETUP
#define PCI_MEM        0xc2000000   /* pci memory for pmc extension */
#endif
#define SRAM_MEM       0xfc000000   /* 128 kb static mem            */
#define NVRAM_MEM      0xfc200000   /* nvram base                   */
#define RTC_MEM        0xfc207fe0   /* rtc base                     */
#define FPROM_MEM      0xfe000000   /* 28F400BX-B program window    */
#define FPROM1_MEM     0xfef00000   /* 28F400BX-B program window    */
#define FPROM2_MEM     0xfef80000   /* 28F400BX-B program window    */
 
/* PCI RESOURCES (PCI IO SPACE MAPPING)*/
/* 
* 970122 MAWE 0x804xxxxx -> 0x814xxxxx (8062 compatibility
*/
#ifdef OLD_MAP

#define BRIDGE_IO        0x80000000   /* PCI bridge registers         */
#define FPROM_IO         0x81000000   /* 28F400BX-B program window    */
#define FLASH_EPROM1_IO  0x80000000   /* 28F400BX-B program window    */
#define FLASH_EPROM2_IO  0x80200000   /* 28F400BX-B program window    */
#define IACK_IO          0x8040b000   /* sic 6351 IACK cycle          */
#define SIC6351_IO       0x8040b400   /* sic 6351                     */
#define Z8536_IO         0x8040b800   /* zcio 8536                    */
#define UPI82C42_IO      0x8040bc00   /* UPI 82C42 Controller         */
#define PC87312_IO       0x8040c000   /* PC 87312 super I/O           */
#define FIFO0_IO         0x8040e000   /* FIFO #0 port                 */
#define FIFO1_IO         0x8040e400   /* FIFO #1 port                 */
#define FIFO2_IO         0x8040e800   /* FIFO #2 port                 */
#define FIFO3_IO     	 0x8040ec00   /* FIFO #3 port                 */
#define FIFO4_IO      	 0x8040f000   /* FIFO #4 port                 */
#define FIFO5_IO      	 0x8040f400   /* FIFO #5 port                 */
#define FIFO6_IO      	 0x8040f800   /* FIFO #6 port                 */
#define FIFO7_IO      	 0x8040fc00   /* FIFO #7 port                 */
#define VME_CSR_IO       0x80750000   /* VME slave CSR                */
#define AM79C970_IO      0x807e0000   /* am79c970 pci io space        */
#define NCR53C820_IO     0x807e0100   /* ncr53c810 pci io space       */

#else OLD_MAP

#define BRIDGE_IO        0x80000000   /* PCI bridge registers         */
#define FPROM_IO         0x81000000   /* 28F400BX-B program window    */
#define FLASH_EPROM1_IO  0x80000000   /* 28F400BX-B program window    */
#define FLASH_EPROM2_IO  0x80200000   /* 28F400BX-B program window    */
#define IACK_IO          0x8140b000   /* sic 6351 IACK cycle          */
#define SIC6351_IO       0x8140b400   /* sic 6351                     */
#define Z8536_IO         0x8140b800   /* zcio 8536                    */
#define UPI82C42_IO      0x8140bc00   /* UPI 82C42 Controller         */
#define PC87312_IO       0x8140c000   /* PC 87312 super I/O           */
#define FIFO0_IO         0x8140e000   /* FIFO #0 port                 */
#define FIFO1_IO         0x8140e400   /* FIFO #1 port                 */
#define FIFO2_IO         0x8140e800   /* FIFO #2 port                 */
#define FIFO3_IO     	 0x8140ec00   /* FIFO #3 port                 */
#define FIFO4_IO      	 0x8140f000   /* FIFO #4 port                 */
#define FIFO5_IO      	 0x8140f400   /* FIFO #5 port                 */
#define FIFO6_IO      	 0x8140f800   /* FIFO #6 port                 */
#define FIFO7_IO      	 0x8140fc00   /* FIFO #7 port                 */
#define VME_CSR_IO       0x80750000   /* VME slave CSR                */
#define AM79C970_IO      0x807e0000   /* am79c970 pci io space        */
#define NCR53C820_IO     0x807e0100   /* ncr53c810 pci io space       */

#endif OLD_MAP

/* PCI RESOURCES (PCI CONFIG SPACE MAPPING)*/

#define AM79C970_CFG     0x80800800   /* am79c970 pci config space    */
#define NCR53C820_CFG    0x80801000   /* ncr53c810 pci config space   */
#define PMC_CFG          0x80840000   /* Mezzanine pci config space   */

/* LOCAL REGISTERS (PHYSICAL MAPPING)*/

#ifdef OLD_MAP
#define LOCREG_PHYS	 0x8040b000   /* LOCREG    */
#else OLD_MAP
#define LOCREG_PHYS	 0x8140b000   /* LOCREG    */
#endif OLD_MAP

/* VME RESOURCES (PHYSICAL MAPPING)*/

#define VMECSR_PHYS	 0xA0F50000   /* VME CSR    */
#define VSBWIN_PHYS	 0xD0000000   /* 64  Mbytes VSB (PVSB 8464)  */
#define VMEWIN0_PHYS	 0xD4000000   /* 64  Mbytes VME window #1    */
#define VMEWIN1_PHYS	 0xE0000000   /* 256 Mbytes VME window #1    */

/* BUS ERROR ADDRESS FOR DAKOTA  (PHYSICAL MAPPING) */
#define TEA_ADDR_PHYS	0xbf880000

/* PCI RESOURCES (PPC603 VIRTUAL MAPPING)*/

#define PC87312_BASE     IOBASE              /* PC 87312 super I/O   */
#define IACK_BASE        (IOBASE | 0x1000)   /* sic 6351 IACK cycle  */
#define SIC6351_BASE     (IOBASE | 0x1400)   /* sic 6351             */
#define Z8536_BASE       (IOBASE | 0x1800)   /* zcio 8536            */
#define UPI82C42_BASE    (IOBASE | 0x1c00)   /* UPI 82C42 Controller */
#define FIFO0_BASE       (IOBASE | 0x2000)   /* FIFO0                */
#define FIFO1_BASE       (IOBASE | 0x2400)   /* FIFO1                */
#define FIFO2_BASE       (IOBASE | 0x2800)   /* FIFO2                */
#define FIFO3_BASE       (IOBASE | 0x2c00)   /* FIFO3                */
#define FIFO4_BASE       (IOBASE | 0x3000)   /* FIFO4                */
#define FIFO5_BASE       (IOBASE | 0x3400)   /* FIFO5                */
#define FIFO6_BASE       (IOBASE | 0x3800)   /* FIFO6                */
#define FIFO7_BASE       (IOBASE | 0x3c00)   /* FIFO7                */
#define NVRAM_BASE       (IOBASE | 0x8000)   /* NVRAM                */
#define RTC_BASE         (IOBASE | 0xffe0)   /* RTC                  */
#define BRIDGE_BASE      (IOBASE | 0x10000)  /* PCI bridge registers */

/* rw 11/97  * and  / added */
/*#define AM79C970_BASE    (IOBASE | 0x10000) */ /* LANCE controller     */

/* BOARD REFGISTERS (VIRTUAL MAPPING)*/
#define VMECSR_BASE	 (IOBASE | 0x11000)   /* VME CSR    */
#define LOCREG_BASE      (IOBASE | 0x12000)   /* Loacal registers */

/* BUS ERROR ADDRESS FOR DAKOTA  (PHYSICAL MAPPING) */
#define TEA_ADDR_BASE    (IOBASE | 0x13000)	

/* 128 kbyte for SRAM on PCI */
#define SRAM_BASE	(IOBASE | 0x20000)    /* SRAM */

/* 4 Mbyte to program FPROM in PCI IO space*/
#define FPROM_WR_BASE	(IOBASE | 0x400000)    /* FPROM */
/* 8 Mbyte for FPROM read in PCI MEM space*/
#define FPROM_RD_BASE	(IO_MEM_SPACE)        /* FPROM */

#define VSBWIN_BASE	 0xC0000000   /* 64  Mbytes VSB (PVSB 8464)  */
#define VMEWIN0_BASE	 0xC4000000   /* 64  Mbytes VME window #0    */
#define VMEWIN1_BASE	 0xD0000000   /* 256 Mbytes VME window #1    */

/* RIO SLAVE MONITOR */
#define RIO_SLAVE_OFF	0x1a100

/* CES new vectors start here... */

#define VECT_SPURIOUS           0
#define VECT_FIF0_EF            0x1
#define VECT_FIF1_EF            0x2
#define VECT_FIF2_EF            0x3
#define VECT_FIF3_EF            0x4
#define VECT_FIF4_EF            0x5
#define VECT_FIF5_EF            0x6
#define VECT_FIF6_EF            0x7
#define VECT_FIF7_EF            0x8
#define VECT_FIF0_FF            0x9
#define VECT_FIF1_FF            0xa
#define VECT_FIF2_FF            0xb
#define VECT_FIF3_FF            0xc
#define VECT_FIF4_FF            0xd
#define VECT_FIF5_FF            0xe
#define VECT_FIF6_FF            0xf
#define VECT_FIF7_FF            0x10
#define VECT_LAN                0x11
#define VECT_SCSI               0x12

#define VECT_PC_S1              0x14
#define VECT_PC_S0              0x15
#define VECT_ZCIO               0x16
#define VECT_MAKO               0x17
#define VECT_BMA                0x18
#define VECT_PC_P0              0x19
#define VECT_VSB                0x1a
#define VECT_VMEIRQ1            0x1b
#define VECT_VMEIRQ2            0x1c
#define VECT_VMEIRQ3            0x1d
#define VECT_VMEIRQ4            0x1e
#define VECT_VMEIRQ5            0x1f
#define VECT_VMEIRQ6            0x20
#define VECT_VMEIRQ7            0x21
#define VECT_SERR               0x22

#define VECT_VMEERR             0x24
#define VECT_VMESYSF            0x25
#define VECT_PMC                0x26
#define VECT_ABO_RIO            0x27
#define VECT_ABO_HOST           0x28
#define VECT_ACF_RIO            0x29
#define VECT_ACF_HOST           0x2a

/* POWER_PC segmemts */

#define SEG0_BASE		0x00000000
#define SEG1_BASE		0x10000000
#define SEG2_BASE		0x20000000
#define SEG3_BASE		0x30000000
#define SEG4_BASE		0x40000000
#define SEG5_BASE		0x50000000
#define SEG6_BASE		0x60000000
#define SEG7_BASE		0x70000000
#define SEG8_BASE		0x80000000
#define SEG9_BASE		0x90000000
#define SEGa_BASE		0xa0000000
#define SEGb_BASE		0xb0000000
#define SEGc_BASE		0xc0000000
#define SEGd_BASE		0xd0000000
#define SEGe_BASE		0xe0000000
#define SEGf_BASE		0xf0000000

