/*
  $Header: /home/elrond/baikal/src/CVS/daq98/clientlib.C,v 1.7 1999/02/03 03:42:12 steffen Exp $
*/

#include <strstream.h>
#include "HVviewlist.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "interpreter.h"
#include "errorlog.h"
#include "transferclass.h"
#include "clientlib.h"
#include "daq98f2000.h"
#include "histogramlist.h"

Client::Client(){
     DEB_F("Client::Client()");
     CMD_Interpreter.AddFunction("Open",  &CMD_Open,
				"open HOST [PORT] -- open a connection to the server on HOST on port PORT");
     CMD_Interpreter.AddFunction("Close", &CMD_Close,
				"close            -- close the connection");
     CMD_Interpreter.AddFunction("Bye",   &CMD_Bye,
				"bye              -- close the connection and quit the Program");
     CMD_Interpreter.AddFunction("Help",  &CMD_Help,
				"help [Command]   -- print help text of command or of all commands");
     CMD_Interpreter.DefaultFunction(&CMD_UnknownCommand);
     CMD_Interpreter.WrongTypeFunction(&CMD_WrongTypeCommand);     

//   REPLY_Interpreter.AddFunction("cmd_config",   &REPLY_Config);
     REPLY_Interpreter.AddFunction("cmd_hvview",            &REPLY_HVView); // *PN*
     REPLY_Interpreter.AddFunction("cmd_pawpipe",           &REPLY_PAWPipe); // *PN*
     REPLY_Interpreter.AddFunction ("cmd_tv",               &REPLY_TableView);
     REPLY_Interpreter.AddFunction("cmd_histogram",         &REPLY_Histogram);
     REPLY_Interpreter.AddFunction("cmd_histolist",         &REPLY_HistogramList);
     REPLY_Interpreter.AddFunction("cmd_muonevent",         &REPLY_MuonEvent);
     REPLY_Interpreter.AddFunction("cmd_message",           &REPLY_Message);
     REPLY_Interpreter.AddFunction("cmd_error",             &REPLY_Error);
     REPLY_Interpreter.AddFunction("cmd_warning",           &REPLY_Warning);
     REPLY_Interpreter.AddFunction ("cmd_rusure",	    &REPLY_RUSURE); // *PN*
     REPLY_Interpreter.AddFunction("cmd_unknown",           &REPLY_Unknown);
     REPLY_Interpreter.AddFunction("cmd_info",              &REPLY_Info);
     REPLY_Interpreter.AddFunction("cmd_ready",             &REPLY_Ready);
     REPLY_Interpreter.DefaultFunction(&REPLY_Default);
     REPLY_Interpreter.WrongTypeFunction(&REPLY_WrongType);
	  
     Quit_FLAG=false;
     DefaultCmdPort=COM_SERVER_PORT;
     IsConnected=false;
     // 20 s timeout
     TimeOut=1000 * 20;
}

Client::~Client(){
     DEB_F("Client::~Client()");
}

int Client::EventLoop(){
     DEB_F("int Client::DoClient()");
     int ret;
     FileNotifier StdInNotifier(STDIN_FILENO,IONotifier::READ);
     Selecter.AddNotifier(StdInNotifier);
     while(!Quit_FLAG){
	  TEST_TEMP_FAILURE(ret=Selecter.DoSelect());
	  if(Selecter.CheckNotifier(CmdSocketNotifier)){
	       if(ServReply()<0){
		    // failure
		    // just don't care about now
	       }
	  }
	  if(Selecter.CheckNotifier(StdInNotifier)){
	       char txt[MAX_TEXT_SIZE];
	       cin.getline(txt,MAX_TEXT_SIZE);
	       
	       if(ServCommand(txt)<0){
		    // failure
		    // just don't care about now
	       }
	  }
     }
     return 0;
}

int Client::DoLogin(const char *name, const char *passwd)
{
     DEB_F("Client::DoLogin(const char *name, const char *passwd");
     char cmd[1024];
     sprintf(cmd,"CS login %s %s\r\n",name,passwd);
     ServCommand(cmd);
}

int Client::ServCommand(const char *txt){
     DEB_F("int Client::ServCommand()");
     String CmdText(txt);
     return CMD_Interpreter.Do_wo_type(this,CmdText);
}

int Client::ServReply(){
     DEB_F("int Client::ServReply()");
     const char *txt;
     int r;
     if((r=CmdSocket.IsReadReady(TimeOut))<=0){
	  if(r==0){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataProcessDaemon::ServReply(IOSocket &S)";
	       char str[200];
	       sprintf(str,"Connection timed out (%i sec).",TimeOut/1000);
	       Base::Error->SetParam(str);
	       DoReady();
	  }
     	  return -1;
     }
     TEST_FAILURE(0==(txt=CmdSocket.ReadText())?-1:0);
     
     if(REPLY_Interpreter.Do_wo_type(this,txt,CmdSocket)<0
	|| HasReplyError())
	  return -1;
     return 0;
}

int Client::SetTimeOut(int timeout)
{
     int tmp=TimeOut;
     TimeOut=timeout;
     return tmp;
}

int Client::GetTimeOut()
{
     return TimeOut;
}

int Client::ServTimer(){
     DEB_F("int Client::ServTimer()");
     return 0;
}

int Client::PrintInfoText(const char *txt){
     cout << txt << flush ;
     return 0;
}

int Client::PrintErrorText(const char *txt){
     cout << txt << flush;
     return 0;
}

int Client::PrintWarningText(const char *txt){
     cout << txt << flush;
     return 0;
}

int Client::PrintRUSUREText (const char *txt)
{
	cout << txt << flush;
	return 0;
}

int Client::PrintMessageText(const char *txt){
     cerr << txt << flush ;
     return 0;
}

int Client::DoPAWPipe (const char *txt){
	cout << txt << flush;
	return 0;
}

int Client::DoTableView (const char *txt){
	return 0;
}


int Client::DoHVView (HVList_t &/*HVList*/)	// *PN* 061097
{
//  DEB_F ("Client::DoHVView (..)");
//  ostrstream out;
//  out << *HVList.ListAll() << endl;
  
//  out.freeze (1);
//  PrintInfoText (out.str ());
//  out.freeze (0);
  return 0;
}

Client::ReplyStatus_t Client::GetReplyStatus()
{
     DEB_F("GetReplyStatus()");
     return ReplyStatus;
}

int Client::DoHistogram(MemHistogram_t &Histogram){     
     ostrstream out;
     out << Histogram().Title << endl;
     out << "NumBins:    " << Histogram().NumBin << endl;
     out << "Xmin:       " << Histogram().Xmin << endl;
     out << "Xmax:       " << Histogram().Xmax << endl;
     out << "NumEntries: " << Histogram().NumEntries << endl;
     out << "Overflows:  " << Histogram().NumOverflows << endl;
     out << "Underflows: " << Histogram().NumUnderflows << endl;
     for(unsigned int i=0;i<Histogram().NumBin;i++)
	  if(Histogram().Ent[i]>0) 
	       out << "Entry: " << i << "  " << Histogram().Ent[i] << endl;
     out << ends ;
     out.freeze(1);
     PrintInfoText(out.str());
     out.freeze(0);
     return 0;
}

int Client::DoHistogramList(HistogramList_t &HistogramList){     
     ostrstream out;
     out << "List of available histograms:\n";
     out << ends ;
     out.freeze(1);
     PrintInfoText(out.str());
     out.freeze(0);
     return 0;
}

int Client::DoReady(){
     return 0;
}

int Client::CMD_Open(String *params,int numparams){
     DEB_F("int Client::CMD_Open(..)");
     if(numparams<2) return SendUsageInfo(params[0]);
     if(IsConnected){
	  String m=String("Already connected to: ")+String(RemoteHostname)+"\n";
	  SetReplyStatus(ERROR);
	  PrintErrorText(String("Already connected to: ")+
			 String(RemoteHostname)+
			 String("\n"));
	  return 0;
     }
     port_t CmdPort;
     RemoteHostname=params[1];
     if(numparams==2) CmdPort=DefaultCmdPort;
     else CmdPort=atol(params[2]);

     if(CmdSocket.Open((const char*)RemoteHostname,CmdPort)<0){
	  	  SetReplyStatus(ERROR);
		  PrintErrorText(String("Could not open Command Connection\n")+
			 String((*Error).GetErrorMessage()));
	  CmdSocket.Close();
	  DoReady();
	  return 0;
     }
     char txt[20];
     sprintf(txt,"%s init\r\n",PROCESS_TYPE_COMMUNICATIONSERVER);
     if(CmdSocket.WriteText(txt)<0){
	  	  SetReplyStatus(ERROR);
		  PrintErrorText(String("could not write to Command Connection\n")+
			 String((*Error).GetErrorMessage()));
	  CmdSocket.Close();
	  DoReady();
	  return 0;
     }
     int r;
     if((r=CmdSocket.IsReadReady(15000))<0){
	  SetReplyStatus(ERROR);
	  PrintErrorText(String("could not read from Command Connection\n")+
			 String((*Error).GetErrorMessage()));
	  CmdSocket.Close();
	  DoReady();
	  return 0;
     }
     if(r==0){
	  SetReplyStatus(ERROR);
	  PrintErrorText(String("can not read, server did not answer within timeout periode\n")+
			 String((*Error).GetErrorMessage()));
	  CmdSocket.Close();
	  DoReady();
	  return 0;
     }
     CmdSocketNotifier.Set(CmdSocket,IONotifier::READ);
     Selecter.AddNotifier(CmdSocketNotifier);
     IsConnected=true;
     return 0;
}

int Client::CMD_Close(){
     DEB_F("int Client::CMD_Close(..)");
     if(!IsConnected){
	  SetReplyStatus(ERROR);
	  PrintErrorText("Client is not connected to a server.\n");
	  DoReady();
	  return 0;
     }
     char txt[100];
     sprintf(txt,"%s bye\r\n",PROCESS_TYPE_COMMUNICATIONSERVER);
     if(CmdSocket.WriteText(txt)<0){
	  SetReplyStatus(ERROR);
	  PrintErrorText(String("could not write to Command Connection")+
			 String((*Error).GetErrorMessage()));
	  DoReady();
     }
     Selecter.RemoveNotifier(CmdSocketNotifier);
     CmdSocket.Close();
     IsConnected=false;
     return 0;
}

int Client::CMD_Bye(){
     DEB_F("int Client::CMD_Bye(..)");
     CMD_Close();
     Quit_FLAG=true;
     QuitRequest();
     return 0;
}

int Client::CMD_Help(String *params,int numparams){
     DEB_F("int Client::CMD_Help(..)");
     String Help;
     if(numparams>1)
	  CMD_Interpreter.GetHelp(params[1],Help);
     else
	  CMD_Interpreter.GetHelp(Help);
     PrintInfoText(Help);
     return 0;
}

int Client::CMD_UnknownCommand(String *params,int){
     DEB_F("int Client::CMD_UnknownCommand(..)");
     if(!IsConnected){
	  SetReplyStatus(ERROR);
	  PrintErrorText("Client is not connected to a server.\n");
	  DoReady();
	  return 0;
     }
     params[0]+="\r\n";
     if(CmdSocket.WriteText(params[0])<0){
	  SetReplyStatus(ERROR);
	  PrintErrorText(String("could not write to Command Connection\n")+
			 String((*Error).GetErrorMessage()));
	  DoReady();
     }
     return 0;
}

int Client::CMD_WrongTypeCommand(){
     DEB_F("int Client::CMD_WrongTypeCommand(..)");
     SetReplyStatus(ERROR);
     return 0;
}

int Client::SendUsageInfo(String param){
     DEB_F("int RunServer::SenUsageInfo(..)");
     String Help;
     CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     PrintInfoText(Help);
     return 0;
}

int Client::REPLY_HVView (IOSocket &s, String * /*params*/, int /*numparms*/)
{								// *PN* 061097
  DEB_F ("Client::REPLY_HVView (..)");
   char txt[MAX_TEXT_SIZE];
  	TEST_FAILURE (s.ReadText (txt, MAX_TEXT_SIZE));
  	PrintInfoText (txt);
  	TEST_FAILURE (s.ReadText (txt, MAX_TEXT_SIZE));
  	PrintInfoText (txt);
  	TEST_FAILURE (s.ReadText (txt, MAX_TEXT_SIZE));
  	PrintInfoText (txt);
  
  return 0;
}

int Client::REPLY_PAWPipe (IOSocket &s, String * /*params*/, int /*numparams*/)
{								// *PN* 211097
	DEB_F ("Client::REPLY_PAWPipe (..)");
	char txt[MAX_TEXT_SIZE];
	TEST_FAILURE (s.ReadText (txt, MAX_TEXT_SIZE));
	SetReplyStatus (SUCCESS);
	DoPAWPipe (txt);
	return 0;
}

int Client::REPLY_TableView (IOSocket &s, String */*params*/, int /*numparams*/){
	DEB_F ("Client::REPLY_TableView (..)");
	char buffer[MAX_TEXT_SIZE];
	
	s.ReadText (buffer, MAX_TEXT_SIZE);
	DoTableView(buffer);
	return 0;
}

int Client::REPLY_Histogram(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int Client::REPLY_Histogram(..)");
     MemHistogram_t Histogram(false); // not in SHM
     TransferClass<MemHistogram_t> tc(s);
     SetReplyStatus(SUCCESS);
     TEST_FAILURE(tc.Recv(Histogram));
     PrintInfoText("Histogram recieved\n");
     return DoHistogram(Histogram);
}

int Client::REPLY_HistogramList(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int Client::REPLY_Histogram(..)");
     HistogramList_t HistogramList; // not in SHM
     TransferClass<HistogramList_t> tc(s);
     SetReplyStatus(SUCCESS);
     TEST_FAILURE(tc.Recv(HistogramList));
     return DoHistogramList(HistogramList);
}

int Client::REPLY_MuonEvent(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int Client::REPLY_Histogram(..)");
     f2000transport_t f2000trans(&cout);
     TransferClass<f2000transport_t> tc(s);
     SetReplyStatus(SUCCESS);
     TEST_FAILURE(tc.Recv(f2000trans));
     PrintInfoText("Muon Event recieved\n");
     return 0;
}

int Client::REPLY_Message(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::ReplyCmdMessage(..)");
     char txt[MAX_TEXT_SIZE];
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     SetReplyStatus(SUCCESS);
     PrintMessageText(txt);

     return 0;
}

// ignore the cmd_ready
int Client::REPLY_Ready(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::ReplyCmdMessage(..)");
     DoReady();
     return 0;
}

int Client::REPLY_Error(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::ReplyCmdMessage(..)");
     char txt[MAX_TEXT_SIZE];
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     SetReplyStatus(ERROR);
     PrintErrorText(txt);

     return 0;
}

int Client::REPLY_Warning(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::REPLY_Warning(..)");
     char txt[MAX_TEXT_SIZE];
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     SetReplyStatus(SUCCESS);
     PrintWarningText(txt);

     return 0;
}

int Client::REPLY_RUSURE (IOSocket &s, String * /*params*/,int /*numparams*/)
{
	DEB_F ("int Client::REPLY_RUSURE (..)");
	char txt[MAX_TEXT_SIZE];
	TEST_FAILURE (s.ReadText (txt, MAX_TEXT_SIZE));
	int ret = PrintRUSUREText (txt);
	
	switch (ret)
	{
		case 0: strcpy (txt, "OK\r\n");
			break;
		case 1:	strcpy (txt, "Cancel\r\n");
			break;

		default: strcpy (txt, "Cancel\r\n");
			break;
	}
//	TEST_FAILURE (s.WriteText (txt));
	
	SetReplyStatus (SUCCESS);
	return 0;
}

int Client::REPLY_Unknown(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::ReplyCmdMessage(..)");
     char txt[MAX_TEXT_SIZE];
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     SetReplyStatus(UNKNOWN);
     PrintErrorText(txt);

     return 0;
}

int Client::REPLY_Info(IOSocket &s,String * /*params*/,int /*numparams*/){
     DEB_F("int Client::ReplyCmdMessage(..)");
     char txt[MAX_TEXT_SIZE];
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     SetReplyStatus(SUCCESS);
     PrintInfoText(txt);

     return 0;
}

int Client::REPLY_Ignore(IOSocket &s,String * /*params*/,int /*numparams*/){
     return 0;
}

int Client::REPLY_Default(IOSocket &/*s*/,String *params,int numparams){
     DEB_F("int Client::ReplyCmdDefault(..)");
     SetReplyStatus(SUCCESS);
     PrintInfoText(join(params,numparams," ")+String("\n"));

     return 0;
}


int Client::REPLY_WrongType(IOSocket &/*s*/,String *params,int /*numparams*/){
      DEB_F("int Client::ReplyWrongType(..)");
      PrintInfoText(*params+String("\n"));

      return 0;
}

