/*
  $Header: /home/elrond/baikal/src/CVS/daq98/clientlib.h,v 1.7 1999/02/03 03:42:12 steffen Exp $
*/

#if ! defined __CLIENT_H__
#define __CLIENT_H__
#include <config.h>
#include <strstream.h>
#include "HVviewlist.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "interpreter.h"
#include "errorlog.h"
#include "ionotifier.h"
#include "memhistogram.h"
#include "histogramlist.h"

#define COM_SERVER_PORT 10002

/** */
class Client:public DaqProcessBase
{
public:
     /** Constructor. 
       Used to initialize some variables and to add functions to the 
       command interpreters. */
     Client();
     /** Destructor. */
     virtual ~Client();
     /** */
     int EventLoop();
     /** */
     int ServCommand(const char *txt);
     /** */
     int ServTimer();
     /** */
     int ServReply();
     /** */
     int ServReplyWCheck();
     
     int DoLogin(const char *name, const char *passwd);
     
     /** */
     inline filedescriptor_t GetSocketDescriptor(){
	  return CmdSocket.Descriptor();
     }

     /** */
     virtual int PrintMessageText(const char *txt);
     /** */
     virtual int PrintInfoText(const char *txt);
     /** */
     virtual int PrintErrorText(const char *txt);
     virtual int PrintWarningText(const char *txt);
     /** */
     virtual int PrintRUSUREText (const char *txt);
     /** */
     virtual int DoPAWPipe (const char *txt); /* *PN* 211097 */
     /** */
     virtual int DoTableView (const char *txt);	/* *PN* 041297 */ /* HH 2.1.98 */
     /** */
     virtual int DoHVView (HVList_t &HVList);	// *PN* 061097
     /** */
     virtual int DoHistogram(MemHistogram_t &Histogram);
     /** */
     virtual int DoHistogramList(HistogramList_t &Histogram);
     /** */
     virtual int DoReady();
     /** */
     String GetProcessType(){return "";}
     /** */
     virtual void QuitRequest(){}
     /** */
     int SetTimeOut(int timeout);
     /** Gets timeout value for command replies.
	 @return timeout value */
     int GetTimeOut();
     
protected:
     enum ReplyStatus_t 
     {
	  ERROR,
	  SUCCESS,
	  UNKNOWN
     };
     ReplyStatus_t ReplyStatus;

     /** Flag that indicates if client is connected to server */
     bool IsConnected;

     /** Sets reply status.
	 @param r Reply status */
     inline void SetReplyStatus(ReplyStatus_t r){
	  ReplyStatus=r;
     }
     /** Gets current reply status.
	 @return Current reply status. */
     ReplyStatus_t GetReplyStatus();

     /** Checks if there was an reply error.
	 @return {\tt false if {\tt ReplyStatus} is set to {\tt SUCCESS} or 
	 or {\tt false} otherwise. */
     inline bool HasReplyError(){
	  if(ReplyStatus!=SUCCESS) return true;
	  return false;
     }     
protected:
     SelectClass Selecter;
     SocketNotifier CmdSocketNotifier;
     ClientSocket CmdSocket;
private:
     port_t DefaultCmdPort;


     String  RemoteHostname;

     Interpreter<Client> CMD_Interpreter;
     int CMD_Open(String *params,int numparams); 
     int CMD_Close(); 
     int CMD_Bye(); 
     int CMD_Help(String *params,int numparams); 
     int CMD_UnknownCommand(String *params,int numparams); 
     int CMD_WrongTypeCommand(); 
     int SendUsageInfo(String param);

     Interpreter<Client> REPLY_Interpreter;
     int REPLY_HVView (IOSocket &s, String *params, int numparams); /* *PN* */
     int REPLY_PAWPipe (IOSocket &s, String *params, int numparams); /* *PN* */
     /** Command handler for "cmd_tv" command.
       Open a TableViewWidget in the moniclient after passing it 
       paramters about its appearance. */
     int REPLY_TableView (IOSocket &s, String *params, int numparams);
							/* *PN* 041297 */
     int REPLY_Histogram(IOSocket &s,String *params,int numparams); 
     int REPLY_HistogramList(IOSocket &s,String *params,int numparams); 
     int REPLY_MuonEvent(IOSocket &s,String *params,int numparams); 
     int REPLY_Message(IOSocket &s,String *params,int numparams);
     int REPLY_Ready(IOSocket &s,String *params,int numparams);
     int REPLY_Error(IOSocket &s,String *params,int numparams);
     int REPLY_Ignore(IOSocket &s,String *params,int numparams);
     int REPLY_Warning(IOSocket &s,String *params,int numparams);
     int REPLY_RUSURE (IOSocket &s, String *params, int numparams); /* *PN* */
     int REPLY_Unknown(IOSocket &s,String *params,int numparams);
     int REPLY_Info(IOSocket &s,String *params,int numparams);
     
     int REPLY_Default(IOSocket &s,String *params,int numparams); 
     int REPLY_WrongType(IOSocket &s,String *params,int numparams);

protected:
     int TimeOut;
};

#endif // __CLIENT_H__







