/*
$Header: /home/elrond/baikal/src/CVS/daq98/comserver.C,v 1.8 1999/02/03 03:42:13 steffen Exp $
*/

#include <pwd.h>
#define __USE_XOPEN
#include <unistd.h>
#undef __USE_XOPEN

// STL
#include <strstream.h>
//
#include "globalmacros.h"
#include "globaltypes.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "transferclass.h"
#include "base.h"
#include "comserver.h"
#include "daq98f2000.h"
#include "histogramlist.h"

CommunicationServerConfig::CommunicationServerConfig():ConfigSection(){
     static String Props[]={""};
     SetProperties(0,Props);
}
     
/** return the name of the section in the config file, where the
   config items are placed */
const String CommunicationServerConfig::GetSectionName()const{
     return  COMMUNICATIONSERVER_SECTION_NAME; // from globalmacros.h
}
     
/** sets property propnum */
int CommunicationServerConfig::SetPropertyByNo(int ,const String &){
     return 0;
}

/** ctor */
CommunicationServer::CommunicationServer():Daemon(Daemon::D_USE_SHM|Daemon::D_NOTIFY_SHM){
     DEB_F("CommunicationServer::CommunicationServer()");
     CMD_Interpreter.AddFunction("Help",&CMD_Help,
				"help [COMMAND]     -- prints help for COMMAND or for all commands");
     CMD_Interpreter.AddFunction("GetHistogram",&CMD_GetHistogram,
				"GetHistogram ID    -- send histogram ID");
     CMD_Interpreter.AddFunction("GetHistogramList",&CMD_GetHistogramList,
				"GetHistogramList   -- send list of available histograms");
     CMD_Interpreter.AddFunction("GetHisto",&CMD_GetHistogram,
				"GetHisto ID    -- sends the histogram No. ID");
     CMD_Interpreter.AddFunction("bye",&CMD_Quit,
				"bye                -- quit the server");
     CMD_Interpreter.AddFunction("Init",&CMD_Init,
				"Init          -- Initialize communication server");
     CMD_Interpreter.AddFunction("login",&CMD_Login,
				"login USER PASSWD  -- login as user USER");
     CMD_Interpreter.AddFunction("logout",&CMD_Logout,
				"logout             -- logout of the system");

     CMD_Interpreter.AddFunction("GetStatus",  &CMD_GetStatus,
				 "GetStatus\n\t-- returns the status of the DAQ - Processes");

     CMD_Interpreter.AddFunction("GetProcStat",  &CMD_GetProcStat,
				 "GetProcStat proc\n\t-- returns the status of the process withe the name proc");

//      CMD_Interpreter.AddFunction("showconfig",&CMD_ShowConfig,
// 				 "showconfig        -- shows the configuration");

     CMD_Interpreter.DefaultFunction(&CMD_UnknownCommand);
     CMD_Interpreter.WrongTypeFunction(&CMD_WrongTypeCommand);     
     
     IPC_CMD_Interpreter.AddFunction("AttachSharedMemory", &IPC_CMD_AttachSharedMemory);
     IPC_CMD_Interpreter.AddFunction("DetachSharedMemory", &IPC_CMD_DetachSharedMemory);


     IPC_CMD_Interpreter.AddFunction ("cmd_hvview", &IPC_CMD_PipeToClient);
								// *PN* 061097
     IPC_CMD_Interpreter.AddFunction ("cmd_pawpipe", &IPC_CMD_PipeToClient);
								// *PN* 211097
	IPC_CMD_Interpreter.AddFunction ("cmd_tv", &IPC_CMD_PipeToClient);
								// *PN* 041297
     IPC_CMD_Interpreter.AddFunction("cmd_message", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_error", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_warning", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_info", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction ("cmd_rusure", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction ("cmd_muonevent", &IPC_CMD_PipeEventToClient);
								// *PN* 101197
//----------------------- Internal Commands --------------------------------
     IPC_CMD_Interpreter.AddFunction("cmd_step_mode_started", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_step_mode_stopped", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_wait_next_step", &IPC_CMD_PipeToClient);
     
     IPC_CMD_Interpreter.AddFunction("cmd_dbg_error", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_dbg_info", &IPC_CMD_PipeToClient);
     IPC_CMD_Interpreter.AddFunction("cmd_dbg_message", &IPC_CMD_PipeToClient);
//--------------------------------------------------------------------------     
     IPC_CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     IPC_CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);     

     IsLoggedIn=false;
     IsStepMode=false;
}

/** dtor */
CommunicationServer::~CommunicationServer(){
     DEB_F("CommunicationServer::~CommunicationServer()");
}

/** initialization */
int CommunicationServer::Init(filedescriptor_t cmdSockDesc){
     DEB_F("CommunicationServer::Init()");

     TEST_FAILURE(Daemon::Init());
     // open the cmd socket
     TEST_FAILURE(CmdSocket.Open(cmdSockDesc));
     // set the notifier
     CmdSocketNotifier.Set(CmdSocket,IONotifier::READ);
     // and add the notifier to the selecter
     Selecter.AddNotifier(CmdSocketNotifier);
     return 0;
}

/** destruction */
int CommunicationServer::Destroy(){
     DEB_F("CommunicationServer::Destroy()");
     Daemon::Destroy();
     return CmdSocket.Close();
}

/** the event dispatcher */
int CommunicationServer::DispatchEvent(){
     // call the dispatcher of the base class
     TEST_FAILURE(Daemon::DispatchEvent());
     if (Selecter.CheckNotifier(CmdSocketNotifier)){
 	  char *txt;
 	  TEST_FAILURE(0==(txt=CmdSocket.ReadText())?-1:0);
 	  if(DoCMDEvent(txt,CmdSocket)<0){
	       TEST_FAILURE(CmdSocket.CheckOutStream());
	       CmdSocket.OutputStream() << "cmd_error\r\n"
					<< "could not perform command because\n";
	       (*Base::Error).Print(CmdSocket.OutputStream());
	       CmdSocket.OutputStream() << "\r\n" << flush;
	  }    
          // ready for next Command
          CmdSocket.OutputStream() << "cmd_ready\r\n" << flush;
     }
     return 0;
}

/** the IPC event handler */
int CommunicationServer::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     return IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

/** the Cmd event handler */
int CommunicationServer::DoCMDEvent(const String &Cmd,IOSocket &s){
     return CMD_Interpreter.Do(this,Cmd,s);
}

/** returns a String with the configuration file name */
const String CommunicationServer::GetConfigFileName()const{
     return  COMMUNICATIONSERVER_CNF_FILENAME; // from globalmacros.h
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ConfigSection *CommunicationServer::GetConfigSection(){
     return new CommunicationServerConfig(); 
}

/** returns the type of the process, must be overwritten in
    derived class */
const String CommunicationServer::GetProcessType()const{
     return PROCESS_TYPE_COMMUNICATIONSERVER; // whatever
}

int CommunicationServer::CMD_Quit(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int CommunicationServer::CMD_Quit(..)");
     Quit_FLAG=true;
     TEST_FAILURE(s.WriteText("cmd_info\r\nBye Bye\r\n"));
     return 0;
}

int CommunicationServer::CMD_Help(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::CMD_Help(..)");
     if(!IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     String Help;
     if(numparams>1)
	  CMD_Interpreter.GetHelp(params[1],Help);
     else
	  CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));
     return 0;
}

int CommunicationServer::CMD_GetHistogram(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::CMD_GetHistogram(..)");
     if(!IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     if(numparams<2) return SendUsageInfo(s,params[0]);
     if(!GotSharedMem){
	  TEST_FAILURE(s.WriteText("cmd_error\r\nNo histograms available: No shared memory allocated\r\n"));
	  return 0; 
     }
     int numHisto=numparams-1;
     for (int i=0;i<numHisto;i++){
	  // get the ID
	  ID_t ID=atol(params[i+1]);
	  // Allocate Histogram in shared mem
	  ShmMan->LockMem();
	  MemHistogram_t hist(ID);
	  if (!hist.IsValid()){
	       ShmMan->UnlockMem();
	       s.OutputStream() << "cmd_error\r\n"
				<< "histogram " << params[i+1] 
				<< " not available\r\n";
	  }else{    
	       // create a local copy of the hsitogram
	       MemHistogram_t Histogram(hist);
	       ShmMan->UnlockMem();
	       TEST_FAILURE(s.WriteText("cmd_histogram\n"));
	       TransferClass<MemHistogram_t> tc(s);
	       TEST_FAILURE(tc.Send(Histogram));
	  }
     }
     return 0;
}

int CommunicationServer::CMD_GetHistogramList(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::CMD_GetHistogramList(..)");
     if(!IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     if(!GotSharedMem){
	  TEST_FAILURE(s.WriteText("cmd_error\r\nNo histograms available: No shared memory allocated\r\n"));
	  return 0; 
     }

     HistogramList_t HistogramList;
     TEST_FAILURE(s.WriteText("cmd_histolist\r\n"));
     TransferClass<HistogramList_t> tc(s);
     TEST_FAILURE(tc.Send(HistogramList));
     return 0;
}

int  CommunicationServer::CMD_Logout(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int  CommunicationServer::CMD_Login(..)");
     if(!IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     TEST_FAILURE(Dispatcher.SetUser(IPCAddress,"")); 
     IsLoggedIn=true;
     return 0;
}

int  CommunicationServer::CMD_Login(IOSocket &s,String *params,int numparams){
     DEB_F("int  CommunicationServer::CMD_Login(..)");
     if(IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nAllready logged in\r\n"));
	  return 0;
     }
     if(numparams!=3) return SendUsageInfo(s,params[0]);
     String &Name=params[1];
#if ! defined NO_LOGIN_PROMPT
     String &Passwd=params[2];
#endif // NO_LOGIN_PROMPT
     passwd *pass_s=getpwnam((const char*)Name);
     if(pass_s){
#if ! defined NO_LOGIN_PROMPT
	  const char *salt=pass_s->pw_passwd;
	  if(compare(String(pass_s->pw_passwd),crypt((const char*)Passwd,salt))==0){
#endif // NO_LOGIN_PROMPT
	       UserName=Name;
	       
	       IsLoggedIn=true;	
	       String str=(String)"cmd_info\r\n"+Name+(String)" logged in\r\n";
	       TEST_FAILURE(s.WriteText(str));
	       TEST_FAILURE(Dispatcher.SetUser(IPCAddress,Name)); 
	       return 0;
#if ! defined NO_LOGIN_PROMPT
	  }
#endif // NO_LOGIN_PROMPT
     }
     TEST_FAILURE(s.WriteText("cmd_error\r\nLogin failed\r\n"));
     return 0;
}

int CommunicationServer::CMD_GetStatus(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int CommunicationServer::::IPC_CMD_GetStatus(..)");
     String Status;
     Dispatcher.GetStatus(Status);
     TEST_FAILURE(S.WriteText("cmd_info\r\n"));
     TEST_FAILURE(S.WriteText(Status));
     TEST_FAILURE(S.WriteText("\r\n"));
     return 0;
}

int CommunicationServer::CMD_GetProcStat(IOSocket &S,String *param,int numparams){
     DEB_F("int CommunicationServer::::IPC_CMD_GetStatus(..)");
     if(numparams!=2) return SendUsageInfo(S,param[0]);
     PortLocker PL(Dispatcher,param[1],UserName);     
     IPAddress_t IPAddr=PL.GetAddress();
     String Ret;
     if(IPAddr.Port<0){
	  Ret=String("cmd_info\r\n")+param[1]+" not running\r\n";
     } else {
	  Ret=String("cmd_info\r\n") +param[1]+" running\r\n";
     }  
     TEST_FAILURE(S.WriteText(Ret));
     return 0;
}

int  CommunicationServer::CMD_Init(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int  CommunicationServer::Init(..)");
     TEST_FAILURE(s.WriteText("cmd_info\r\nConnection established\nAMANDA DAQ Control -- Communication Server\nHello there!\r\n"));
     return 0;
}

int CommunicationServer::CMD_UnknownCommand(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int CommunicationServer::CMD_UnknownCommand(..)");
     if(!IsLoggedIn) {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     TEST_FAILURE(s.WriteText("cmd_info\r\nComserver: Unkown Command -- try help\r\n"));
     return 0;
}

int CommunicationServer::SendUsageInfo(IOSocket &s,String param){
     DEB_F("int CommunicationServer::SendUsageInfo(..)");
     String Help;
     CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     return s.WriteText(Help);
}

int CommunicationServer::CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::CMD_IgnoreCommand(..)");
     if(!IsLoggedIn){
	  TEST_FAILURE(s.WriteText("cmd_error\r\nUser not logged in\r\n"));
	  return 0;
     }
     // get port number of addressed process (address in param[1])
     PortLocker PL(Dispatcher,params[1],UserName);
     
     IPAddress_t IPAddr=PL.GetAddress();
     if((numparams<1)||(IPAddr.Port<0)){
	  TEST_FAILURE(s.WriteText("cmd_info\r\nComserver: Unkown Command -- try help\r\n"));
     }else{
	  
	  ClientSocket CS;
	  TEST_FAILURE(CS.Open(IPAddr));
	  TEST_FAILURE(CS.WriteText((const char*)(params[0]+"\r\n")));
 	  char txt[MAX_TEXT_SIZE];
	  // read from the socket and interpret the commands, there
	  // might be more than one \r\n terminated string so we must
	  // read until eof
	  while(!CS.InputStream().eof()){
	       // only return on real errors not on eof
	       if((CS.ReadText(txt,MAX_TEXT_SIZE)<0)&&(!CS.InputStream().eof())) return -1;
 	       if(strlen(txt)) {
		    
		    if(!strcasecmp(txt,"cmd_step_mode_started\r\n")) SetStepMode(true);
		    else if(!strcasecmp(txt,"cmd_step_mode_stopped\r\n")) SetStepMode(false);
		    IPC_CMD_Interpreter.Do_wo_type(this,txt,CS);
	       }
	  }
	  if(IsStepMode==false) CS.Close();
     }
     return 0;
}

int CommunicationServer::IPC_CMD_AttachSharedMemory(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int CommunicationServer::IPC_CMD_AttachSharedMemory(..)");
     return AttachSHM();
}

int CommunicationServer::IPC_CMD_DetachSharedMemory(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int CommunicationServer::IPCCmdDetachSharedMemory(..)");
     return DetachSHM();
}

int CommunicationServer::IPC_CMD_UnknownCommand(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int CommunicationServer::IPCUnknownCommand(..)");
     return 0;
}

int CommunicationServer::IPC_CMD_WrongTypeCommand(IOSocket &/*s*/,String *params,int){
     DEB_F("int CommunicationServer::IPCIgnoreCommand(..)");
     TEST_FAILURE(CmdSocket.WriteText("cmd_message\r\n"));
     TEST_FAILURE(CmdSocket.WriteText((const char*)params[0]));
     return 0;
}

int CommunicationServer::IPC_CMD_PipeToClient(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::IPCPipeToClient(..)");
     char txt[MAX_TEXT_SIZE];
     strcpy(txt,(const char*)params[0]);
     for(int i=1; i < numparams;i++) {
	  strcat(txt," ");
	  strcat(txt,(const char*)params[i]);
     }
     strcat(txt,"\r\n");
     TEST_FAILURE(CmdSocket.WriteText(txt));
     TEST_FAILURE(s.ReadText(txt,MAX_TEXT_SIZE));
     TEST_FAILURE(CmdSocket.WriteText(txt));
     return 0;
}

int CommunicationServer::IPC_CMD_PipeEventToClient(IOSocket &s,String *params,int numparams){
     DEB_F("int CommunicationServer::IPCPipeToClient(..)");
     TEST_FAILURE(CmdSocket.WriteText("cmd_muonevent\r\n"));
     TransferPipe<f2000transport_t> tp(s,CmdSocket);
     TEST_FAILURE(tp.DoPipe());
     return 0;
}

int main(int argc,char *argv[]){
     DEB_F("int main(int argc,char *argv[])");
     
     DaqProcessBase::InitializeGlobals("comserver");
     LogFile LF(DaqProcessBase::ThisProcess->GetName(),LogFile::APPEND);
     Logger::SetLogger(&LF);				
     if(argc<2){
	  cout << "Usage: " << argv[0] << " " << "SocketDescriptor" << endl;
	  return 4;
     }
     int SocketDesc=atoi(argv[1]);
     CommunicationServer RServer;
     if(RServer.Init(SocketDesc)){
	  (*Base::Error).Print();
     }else{
	  /* signal parent process, that the process was started
	     successfully */
	  kill(getppid(),SIGUSR1);
	  if(RServer.EventLoop()<0){
	       (*Base::Error).Print();
	  }
     }
     RServer.Destroy(); 
     return 0;
}










