/*
$Header: /home/elrond/baikal/src/CVS/daq98/comserver.h,v 1.6 1999/02/03 03:42:13 steffen Exp $
*/

#if ! defined __RUNSERVER_H__
#define __RUNSERVER_H__
#include <config.h>
// STL
#include <strstream.h>
#include "globalmacros.h"
#include "globaltypes.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "memhistogram.h"
#include "daemon.h"

class CommunicationServerConfig:public ConfigSection
{
public:
     /** ctor */
     CommunicationServerConfig();
     /**return the name of the section in the config file, where the
	config items are placed */
     virtual const String GetSectionName()const;
     /** sets property propnum */
     virtual int SetPropertyByNo(int propnum,const String &val);
     /** returns property propnum */
};

class CommunicationServer:public Daemon
{
public:
     /** ctor */
     CommunicationServer();
     /** dtor */
     ~CommunicationServer();
     /** initialization */
     Init(filedescriptor_t cmdSockDesc);
     /** destruction */
     Destroy();
     /** returns the type of the process, must be overwritten in
         derived class */
     virtual const String GetProcessType()const;
protected:
     /** the event dispatcher */
     virtual int DispatchEvent();
     /** the IPC event handler */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** the CMD-socket event handler */
     virtual int DoCMDEvent(const String &Cmd,IOSocket &s);
     /** returns a String with the configuration file name */
     virtual const String GetConfigFileName()const;
     /** returns a pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ConfigSection *GetConfigSection();
     /** Sets step mode flag.
	 @param m Step mode flag;
	 @return Old step mode flag. */
     inline bool SetStepMode(bool m){
	  bool tmp=IsStepMode;
	  IsStepMode=m;
	  return tmp;
     }
protected:
     /** Step mode flag */
     bool IsStepMode;
     
private:
//      typedef vector<ItemWithProperties_p> cnfvector_type;
//      /** send the config vector cnfvector with the command cmd to the
//          socket sock */
//      int SendConfig(const String &Cmd,cnfvector_type &cnfvector,IOSocket &sock);
private:
     bool IsLoggedIn;
     String UserName;
     SlaveServerSocket CmdSocket;
     SocketNotifier CmdSocketNotifier;
     
     Interpreter<CommunicationServer> CMD_Interpreter;
     Interpreter<CommunicationServer> IPC_CMD_Interpreter;
     
     int CMD_Login(IOSocket &s,String *params,int numparams);
     int CMD_Logout(IOSocket &s,String *params,int numparams);
     int CMD_Quit(IOSocket &s,String *params,int numparams);
     int CMD_Init(IOSocket &s,String *params,int numparams);
     int CMD_GetStatus(IOSocket &s,String *params,int numparams);
     int CMD_GetProcStat(IOSocket &S,String *param,int numparams);

     int CMD_Help(IOSocket &s,String *params,int numparams);
     int CMD_GetHistogram(IOSocket &s,String *params,int numparams);
     int CMD_GetHistogramList(IOSocket &s,String *params,int numparams);
     int CMD_UnknownCommand(IOSocket &s,String *params,int numparams);
     int CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams);
//     int CMD_ShowConfig(IOSocket &s,String *params,int);

     int SendUsageInfo(IOSocket &s,String param);

     int IPC_CMD_AttachSharedMemory(IOSocket &s,String *params,int numparams);
     int IPC_CMD_DetachSharedMemory(IOSocket &s,String *params,int numparams);
     int IPC_CMD_UnknownCommand(IOSocket &s,String *params,int numparams);
     int IPC_CMD_PipeToClient(IOSocket &s,String *params,int numparams);
     int IPC_CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams);
     int IPC_CMD_PipeEventToClient(IOSocket &s,String *params,int numparams);
};



#endif // __RUNSERVER_H__



