#if ! defined __CONFIGSECTION2_H__
#define __CONFIGSECTION2_H__

#include <config.h>
#include <String.h>
#include "daqprocessbase.h"
#include "stringutil.h"
#include "configsection.h"

class Base; /***/

class ConfigSection_t:public ClassWithTypeInfo_t
{
     ClassTypeInfo(ClassWithTypeInfo_t);
public:
     enum FUNCTION_t{
	  SET_BY_NAME,
	  GET_BY_NAME,
	  SET_BY_NUMBER,
	  GET_BY_NUMBER,
	  GET_PROP_NAME,
	  COUNT
     };

public:
     /** Sets a certain property.
	 @param PropID The identifier of the property for which 
	 we want to set the value.
	 @param Val New value of the property with identifier <tt>\Ref{PropID}</tt>.
	 @return 0 on succes, -1 on failure. */
     int SetProperty(const String &PropID,const String &Val){
	  int PropNumber=0;
	  return GetSetProperty((String&)PropID,(String&)Val,SET_BY_NAME,PropNumber);
     }

     /** Gets a certain property.
	 @param PropID The identifier of the property for which 
	 we want to get the value.
	 @return The value of the property with identifier <tt>\Ref{PropID}</tt>. */
     String GetProperty(const String &PropID){
	  int PropNumber=0;
	  String Val;
	  GetSetProperty((String&)PropID,Val,GET_BY_NAME,PropNumber);
	  return Val;
     }

     /** Sets the property given by a number.
	 The number can be claculated from the indentifier using the
	 <tt>\Ref{GetPropertyNumber}</tt> member function.
	 Note, that the function is pure virtual and must be overloaded
	 in every derived class.
	 @param propnum Number of porperty that we want to set.
	 @param val The value we want property <tt>\Ref{propnum}</tt> to beset to. 
	 @return 0 on success, -1 otherwise */
     int SetPropertyByNo(int propnum,const String &Val){
	  int PropNumber=0;
	  String val=Val;
	  String Prop;
	  ConvertToString(Prop,propnum);
	  return GetSetProperty(Prop,(String&)Val,SET_BY_NUMBER,PropNumber);
     }

     /** Gets the name of a property.
	 @param num Property number we want to look up.
	 @return The name of the property with number <tt>\Ref{propnum}</tt> */
     const String GetPropertyByNo(int propnum){
	  int PropNumber=0;
	  String Val;
	  String Prop;
	  ConvertToString(Prop,propnum);
	  GetSetProperty(Prop,Val,GET_BY_NUMBER,PropNumber);
	  return Val;
     }     
     /** Gets the property identifier of Property index.
	 @param index the number of the property
	 @return A String with the Name of the Property */
     const String GetPropertyName(int num){
	  int PropNumber=0;
	  String Val;
	  ConvertToString(Val,num);
	  String Prop;
	  GetSetProperty(Prop,Val,GET_PROP_NAME,PropNumber);
	  return Prop;
     }	  
     /** Gets the number of properties of the class.
       @return Number of properties that this class handles. */
     int GetNoProperties(){
	  int PropNumber=0;
	  String Prop;
	  String Val;
	  GetSetProperty(Prop,Val,COUNT,PropNumber);
	  return PropNumber;
     }     
     /** Gets name of the config section.
	 The function returns the name of the section in the configuration file, 
	 where the config items are placed. Note, that this function
	 is a pure virtual function and must be overloaded in every derived class.
	 @return Name of the configuration section */
     virtual const String GetSectionName()const=0;
protected:
     virtual int GetSetProperty(String &,String &,FUNCTION_t,int &){
	  return -1;
     }

protected:
     DaqProcessBase *Parent;
};

MAKE_REFCOUNT_POINTER(ConfigSection_t,ConfigSection_p);

#define BeginItems(BaseClass)									\
virtual int GetSetProperty(String &Prop,String &Value,FUNCTION_t Function,int &PropNumber_){	\
     int retval=-1, lastretval=-1;
 
#define EndItems(BaseClass)						\
     lastretval=BaseClass::GetSetProperty(Prop,Value,Function,PropNumber_);	\
     if(retval<lastretval)return lastretval;				\
     return retval;							\
}

inline int ItemLink_tf0(const char * prop_,
			String & Prop,
			String &Value,
			ConfigSection_t::FUNCTION_t Function,
			int &PropNumber){
     PropNumber++;
     switch(Function){
     case ConfigSection_t::SET_BY_NAME: 
	  return 0;
     case ConfigSection_t::GET_BY_NAME: 
	  return 0;
     case ConfigSection_t::SET_BY_NUMBER: 
	  return 0;
     case ConfigSection_t::GET_BY_NUMBER: 
	  return 0;
     case ConfigSection_t::GET_PROP_NAME: 
	  if(PropNumber-1==hstrtol(Value)){
	       Prop=prop_;
	       return 0;
	  } break;
     case ConfigSection_t::COUNT: return 0; break;
     }
     return 0;
}


template <class var_t>
inline int ItemLink_tf1(const char * prop_,
			String & Prop,
			String &Value,
			ConfigSection_t::FUNCTION_t Function,
			int &PropNumber,
			var_t &var_){
     PropNumber++;
     switch(Function){
     case ConfigSection_t::SET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       return ConvertFromString(Value,var_);
	  } break;
     case ConfigSection_t::GET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       return ConvertToString(Value,var_);
	  } break;
     case ConfigSection_t::SET_BY_NUMBER: 
	  if(PropNumber-1==hstrtol(Prop)){
	       return ConvertFromString(Value,var_);
	  } break;
     case ConfigSection_t::GET_BY_NUMBER: 
	  if(PropNumber-1==hstrtol(Prop)){
	       return ConvertToString(Value,var_);
	  } break;
     case ConfigSection_t::GET_PROP_NAME: 
	  if(PropNumber-1==hstrtol(Value)){
	       Prop=prop_;
	       return 0;
	  } break;
     case ConfigSection_t::COUNT: return 0; break;
     }
     return 0;
}


template <class var1_t,class var2_t>
inline int ItemLink_tf2(const char *prop_,
			String &Prop,
			String &Value,
			ConfigSection_t::FUNCTION_t Function,
			int &PropNumber,
			var1_t &var1_,
			var2_t &var2_){
     PropNumber++;
     switch(Function){
     case ConfigSection_t::SET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       StringSplitter sp(Value,' ');
	       if(sp.NumSplits()!=2) return -1;
	       if(ConvertFromString(sp()[0],var1_)<0) return -1;
	       return ConvertFromString(sp()[1],var2_);
	  } break;
     case ConfigSection_t::GET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       Value="";
	       String val;
	       ConvertToString(val,var1_);
	       Value+=val+" ";
	       ConvertToString(val,var2_);
	       Value+=val+" ";
	       return 0;
	  } break;
     case ConfigSection_t::SET_BY_NUMBER:
	  if(PropNumber-1==hstrtol(Prop)){
	       StringSplitter sp(Value,' ');
	       if(sp.NumSplits()!=2) return -1;
	       if(ConvertFromString(sp()[0],var1_)<0) return -1;
	       return ConvertFromString(sp()[1],var2_);
	  } break;
     case ConfigSection_t::GET_BY_NUMBER: 
	  if(PropNumber-1==hstrtol(Prop)){
	       Value="";
	       String val;
	       ConvertToString(val,var1_);
	       Value+=val+" ";
	       ConvertToString(val,var2_);
	       Value+=val+" ";
	       return 0;
	  } break;
     case ConfigSection_t::GET_PROP_NAME: 
	  if(PropNumber-1==hstrtol(Value)){
	       Prop=prop_;
	       return 0; 
	  } break;
     case ConfigSection_t::COUNT: return 0; break;
     }
     return 0;
}

template <class var1_t,class var2_t,class var3_t>
inline int ItemLink_tf3(const char *prop_,
			String &Prop,
			String &Value,
			ConfigSection_t::FUNCTION_t Function,
			int &PropNumber,
			var1_t &var1_,
			var2_t &var2_,
			var3_t &var3_){
     PropNumber++;
     switch(Function){
     case ConfigSection_t::SET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       StringSplitter sp(Value,' ');
	       if(sp.NumSplits()!=3) return -1;
	       if(ConvertFromString(sp()[0],var1_)<0) return -1;
	       if(ConvertFromString(sp()[1],var2_)<0) return -1;
	       return ConvertFromString(sp()[2],var3_);
	  } break;
     case ConfigSection_t::GET_BY_NAME: 
	  if(fcompare(prop_,Prop)==0){
	       Value="";
	       String val;
	       ConvertToString(val,var1_);
	       Value+=val+" ";
	       ConvertToString(val,var2_);
	       Value+=val+" ";
	       ConvertToString(val,var3_);
	       Value+=val+" ";
	       return 0;
	  } break;
     case ConfigSection_t::SET_BY_NUMBER:
	  if(PropNumber-1==hstrtol(Prop)){
	       StringSplitter sp(Value,' ');
	       if(sp.NumSplits()!=3) return -1;
	       if(ConvertFromString(sp()[0],var1_)<0) return -1;
	       if(ConvertFromString(sp()[1],var2_)<0) return -1;				
	       return ConvertFromString(sp()[2],var3_);
	  } break;
     case ConfigSection_t::GET_BY_NUMBER: 
	  if(PropNumber-1==hstrtol(Prop)){
	       Value="";
	       String val;
	       ConvertToString(val,var1_);
	       Value+=val+" ";
	       ConvertToString(val,var2_);
	       Value+=val+" ";
	       ConvertToString(val,var3_);
	       Value+=val+" ";
	       return 0;
	  } break;
     case ConfigSection_t::GET_PROP_NAME: 
	  if(PropNumber-1==hstrtol(Value)){
	       Prop=prop_; 
	       return 0; 
	  } break;	
     case ConfigSection_t::COUNT: return 0; break;
     }
     return 0;
}

#define BeginActions { 

#define ItemAction_SET(prop_,action_)							\
switch(Function){									\
 case SET_BY_NAME: if(fcompare(prop_,Prop)==0){action_;} break;				\
 case SET_BY_NUMBER: if(PropNumber_==hstrtol(Prop)){action_;} break;		\
 case GET_BY_NAME:break;								\
 case GET_BY_NUMBER: break;								\
 case GET_PROP_NAME: break;								\
 case COUNT: break;									\
 }										

#define ItemAction_GET(prop_,action_)							\
switch(Function){									\
 case SET_BY_NAME:   break;								\
 case SET_BY_NUMBER: break;								\
 case GET_BY_NAME:   if(fcompare(prop_,Prop)==0){action_;} break;			\
 case GET_BY_NUMBER:  if(PropNumber==hstrtol(Prop)){action_;} break;		\
 case GET_PROP_NAME: break;								\
 case COUNT: break;									\
 }										

#define EndActions }

#define BeginLinks   { int &PropNumber(PropNumber_);

#define ItemLink0(prop_)					\
 retval=ItemLink_tf0((const char *)prop_,			\
		     (String &)Prop,				\
		     (String &)Value,				\
		     (ConfigSection_t::FUNCTION_t)Function,	\
		     (int&)PropNumber);  

#define ItemLink1(prop_,var1)					\
 retval=ItemLink_tf1((const char *)prop_,			\
		     (String &)Prop,				\
		     (String &)Value,				\
		     (ConfigSection_t::FUNCTION_t)Function,	\
		     (int&)PropNumber,				\
		     var1);  

#define ItemLink2(prop_,var1,var2)				\
 retval=ItemLink_tf2((const char *)prop_,			\
		     (String &)Prop,				\
		     (String&)Value,				\
		     (ConfigSection_t::FUNCTION_t)Function,	\
		     (int&)PropNumber,				\
		     var1,					\
		     var2);  

#define ItemLink3(prop_,var1,var2,var3)				\
 retval=ItemLink_tf3((const char *)prop_,			\
		     (String &)Prop,				\
		     (String&)Value,				\
		     (ConfigSection_t::FUNCTION_t)Function,	\
		     (int&)PropNumber,				\
		     var1,					\
		     var2,					\
		     var3);

#define EndLinks }

#define SectionName(name_) virtual const String GetSectionName()const{return name_;}


// class Modules:public ConfigSection_t
// {
//      BeginItems(ConfigSection_t){
// 	  BeginActions{
// 	       ItemAction("Type",mc=new MC,mc_vect->push_back(mc));
// 	  }EndActions;
// 	  BeginLinks{
// 	       ItemLink1("Type",mc->Type);
// 	       ItemLink1("win",mc->Win);
// 	       ItemLink1("ID",mc->id);
// 	  }EndLinks;
//      }EndItems;
//      SectionName();
// }
#endif // __CONFIGSECTION2_H__

