/*
  $Header: /home/elrond/baikal/src/CVS/daq98/cursesstatusdispl.C,v 1.8 1999/02/08 08:08:53 steffen Exp $
*/

/*
  $Log: cursesstatusdispl.C,v $
  Revision 1.8  1999/02/08 08:08:53  steffen
  - * code for scalerbase and s7201 added and/or debugged

  Revision 1.7  1998/11/23 14:41:11  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.6  1998/11/03 06:43:53  daq
  merged pamanda2 and pamanda3 versions

  Revision 1.5  1998/08/10 23:23:19  steffen
  resolved some OSF1 problems

  Revision 1.4  1998/07/31 07:41:36  daq
  * cursesstatusdispl.C (__Progress): Fixed progress bar display for OSF output


  Revision 1.2  1998/07/27 23:59:12  steffen
  OSF specific changes

  Revision 1.1  1998/06/29 05:16:30  daq
  Added Files:
  	cursesstatusdispl.C cursesstatusdispl.h statusdispl.C
  	statusdispl.h

*/

#include <config.h>
#if defined HAVE_CURSES_H
#include "cursesstatusdispl.h"
#include <time.h>
#include <math.h>
#include <string.h>
#include <fstream.h>

CursesStatusDisplay_t::CursesStatusDisplay_t(int Update):
     StatusDisplay_t(Update)
{
     initscr();
     ProgressWin=newwin(1,COLS,LINES-1,0);
     StatusWin=newwin(LINES-1,COLS,0,0);
     scrollok(StatusWin,TRUE);
     StatusRow=0;
     ProgressRow=LINES-1;
}

CursesStatusDisplay_t::~CursesStatusDisplay_t()
{
     delete ProgressWin;
     delete StatusWin;
     endwin();
}

int CursesStatusDisplay_t::StatusLine(char * Text)
{
     wprintw(StatusWin,Text);
     wrefresh(StatusWin);
     return 0;
}

int CursesStatusDisplay_t::__Progress(float Percent, 
				      size_t NumBytes)
{
  static int Eta=0;
  time_t TimeDiff=CurTime-StartTime;
  if(TimeDiff==0){
    Eta=100000;
  }else{
    Eta=(int)rint((TimeDiff * (100.0-Percent))/Percent);
  }
  char *EtaStr=Time2ETAStr(Eta);
  char *ByteStr=Bytes2Str(NumBytes);
  int Max=COLS-strlen(EtaStr)-strlen(ByteStr)-8;
  int N=(int)rint(Max * Percent / 100.0);
  char s[128];
  sprintf(s," %s %s ",ByteStr,EtaStr);
  int i;
  char bar[COLS+1];
  strcpy(bar,"");
  for(i=0;i<N-1;i++){
    strcat(bar,"=");
  }
  if(N) strcat(bar,">");
  for(i=0;i<Max-N;i++){
    strcat(bar," ");
  }
  char line[COLS+1];
  sprintf(line,"%3i%s %s%s",(int)rint(Percent),"%%",bar,s);
  wattron(ProgressWin,A_BOLD);
  mvwprintw(ProgressWin,0,0,line);
  wattroff(ProgressWin,A_BOLD);
  wrefresh(ProgressWin);
  return 0;
}

#endif //HAVE_CURSES_H
