#if ! defined __CURSESSTATUSDISPL_H__
#define __CURSESSTATUSDISPL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/cursesstatusdispl.h,v 1.6 1999/02/08 08:08:53 steffen Exp $
*/

/*
  $Log: cursesstatusdispl.h,v $
  Revision 1.6  1999/02/08 08:08:53  steffen
  - * code for scalerbase and s7201 added and/or debugged

  Revision 1.5  1998/11/23 14:41:11  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.4  1998/08/10 23:23:19  steffen
  resolved some OSF1 problems

  Revision 1.3  1998/07/31 07:41:36  daq
  * cursesstatusdispl.C (__Progress): Fixed progress bar display for OSF output


  Revision 1.1  1998/06/29 05:16:30  daq
  Added Files:
  	cursesstatusdispl.C cursesstatusdispl.h statusdispl.C
  	statusdispl.h

*/

#include <config.h>
#include "statusdispl.h"
#if defined HAVE_CURSES_H
#include <curses.h>
#if defined erase
#undef erase
#endif
#if defined wclear
#undef wclear
#endif

/** A class that can display several status messages
    using the ncurses library */
class CursesStatusDisplay_t:public StatusDisplay_t
{
public:
     /** Constructor.
       @param Update time between updates of the progress bar 
       in seconds. If Update is {\tt 0}, the progress bar is
       always update if function {\tt \Ref{Progress}} is
     called (inherited from {\tt \Ref{StatusDisplay_t}}*/
     CursesStatusDisplay_t(int Update=0);
     
     /** Destructor */
     virtual ~CursesStatusDisplay_t();

     /** Display a status line.
	 The status line is displayed right above the progress
	 bar */
     virtual int StatusLine(char * Text);
     
protected:
     /** Display progress bar.
	 Display a progress bar with the percentage and byte
	 already read.
	 @param Percent Percentage
	 @param NumBytes Number of Bytes. If this is zero, no byte 
	 information will be displayed
	 @return 0 on success, -1 on failure */
     virtual int __Progress(float Percent, size_t NumBytes);
     	       
private:
     /** Row where status line is displayed */
     int StatusRow;
     
     /** Row where progress bar is displayed */
     int ProgressRow;

     /** The progress bar window.
	 The display is divided into two windows. This
	 is one of them.*/
     WINDOW* ProgressWin;
     
     /** The status window.
	 The display is divided into two windows. This
	 is one of them.*/
     WINDOW* StatusWin;
};


#endif // HAVE_CURSES_H
#endif // __CURSESSTATUSDISPL_H__
