/*
$Header: /home/elrond/baikal/src/CVS/daq98/daemon.C,v 1.8 1999/02/10 17:31:45 daq Exp $
*/

#include <strstream.h>
#include <String.h>
#include <string.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "ionotifier.h"
#include "cnfreader.h"
#include "daemon.h"
#include "mailer.h"
#include "Regex.h" 

Daemon::Daemon(DaemonFlags_t df){
     DEB_F("Daemon::Daemon()");
     DaemonFlags=df;

     Registered=false;
     GotSharedMem=false;
     // do we use SHM ?
     if(0!=(DaemonFlags&D_USE_SHM)){
	  // yes, create a manager for SHM
	  ShmMan=new SharedMemoryManager;
	  // set static Shared Mamory manager variable of SharedMemAllocator 
	  SharedMemAllocator_t::SetSharedMemoryManager(ShmMan);
     }else ShmMan=0;
     StepMode=OFF;
     I_CMD_Interpreter.DefaultFunction(&I_CMD_UnknownCommand);
     I_CMD_Interpreter.WrongTypeFunction(&I_CMD_WrongTypeCommand);     
     
}

Daemon::~Daemon(){
     DEB_F("Daemon::~Daemon()");
     // Did we create a SHM manager, then delete it
     if(0!=ShmMan) delete ShmMan;
}
   
int Daemon::Init(){
     DEB_F("int Daemon::Init()");

     // first Read the config file
     TEST_FAILURE(ReadConfigFile());
     
     // Register at Dispatcher and get the IPC port number
     pid.pid=getpid();
     pid.Hostname=LocalHostname;
     pid.ProcessID=GetProcessType();
     IPCAddress=Dispatcher.Register(pid,DaemonFlags&D_NOTIFY_SHM);
     TEST_FAILURE(IPCAddress.Port);
     Registered=true;
     char txt[1000];
     sprintf(txt,"Process %s [%i,%i] registered",ThisProcess->GetName(),
	     ThisProcess->GetPid(),ThisProcess->GetPPid());
     LogMsg(txt,2,2,INFO);
     
     // Create Shared Memory
     if(0!=(DaemonFlags&D_CREATE_SHM)){
	  TEST_FAILURE(Dispatcher.CreateSharedMemory(GetSHMSize(),
						     GetSHMSegments()));
     }
     // open the ipcport , read the config file
     TEST_FAILURE(DaemonBase::Init());
     
     // if we use shared Memory try to attach to it, will fail if the
     // shared memory does not exist
     if(0!=(DaemonFlags&D_USE_SHM)){
	  if(AttachSHM()<0){
	       // do we get notified if SHM is created ? 
	       if(0==(DaemonFlags&D_NOTIFY_SHM)){
		    // no, return an error
		    return -1;
	       }
	       // otherwise do not care about now, we will get
	       // notified if it is created later
	  }
     }
     return 0;
}

int Daemon::Destroy(){
     DEB_F("int Daemon::Destroy()");
     DaemonBase::Destroy();

     // do we have a shared memory manager, ( we might be attached to
     // shared memory) -> detach from SHM
     if(0!=ShmMan){
	  DetachSHM();
     }
     // Are we registered at the dispatcher ?
     if(Registered){
	  // destroy the SHM
	  if(0!=(DaemonFlags&D_CREATE_SHM)){
	       Dispatcher.DestroySharedMemory();
	  }
	  // yes
	  Dispatcher.Unregister(pid);
     }
     Registered=false;
     return 0;
}

int Daemon::AttachSHM(){
     // are we already attached, or is there no SHM manager ?, then return 
     if(GotSharedMem||(0==ShmMan)) return 0;
     key_t shmkey,semkey;
     int ret;
     // get the keys for the SHM from the dispatcher
     ret=Dispatcher.GetShmManKeys(shmkey,semkey);
     // error ?
     if((shmkey>0)&&(ret==0)){
	  // no, now we can attach to the SHM by initializing the
	  // Shared memory manager
	  ret=ShmMan->Init(shmkey,semkey);
	  // set the flag
	  GotSharedMem=true;
     };
     return ret;
}

int Daemon::DetachSHM(){
     // are we already detached, or is there no SHM manager ?, then return 
     if(!GotSharedMem||(0==ShmMan)) return 0;
     GotSharedMem=false;
     // detach by destroying the shared memory manager
     return ShmMan->Destroy();
}

int Daemon::DoIPCEvent(const String &Cmd,SlaveServerSocket &s)
{
     return I_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

bool Daemon::WasInternalCommand()
{
     return InternalCommand;
}

int Daemon::I_CMD_UnknownCommand(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int VMEDataProcess::IPCUnknownCommand(..)");
     InternalCommand=false;
     return 0;
}

int Daemon::I_CMD_WrongTypeCommand(IOSocket &/*s*/,String */*params*/,int){
     DEB_F("int VMEDataProcess::IPCIgnoreCommand(..)");
     InternalCommand=false;
     return 0;
}

int Daemon::Fork2Background(){
     int r;
     TEST_FAILURE(r=ThisProcess->Fork());
     if(r==0){  // child process
// 	  close(0);
// 	  close(1);
// 	  close(2);
	  //ok
     } else { // parent
	  // exit
	  exit(0);
     }
     return 0;
}   

void Daemon::LogMsg(const char* Msg, int LogLevelFile, int LogLevelConsole, LogType_t Type)
{
     static ostrstream *WarningStream=new ostrstream;
     static ostrstream *ErrorStream=new ostrstream;
     static time_t LastWarningTime=0;
     static time_t LastErrorTime=0;
          
     String MessageHeader="";
     char *NewMsg=new char[strlen(Msg)+MAX_TEXT_SIZE+1];
     char LogType[MAX_TEXT_SIZE];
     strcpy(LogType,LOG_TYPE_RUNLOG);
     switch(Type){
     case ERROR:
	  MessageHeader=String("***Error*** ")+ThisProcess->GetName()+String(": ");
	  strcpy(LogType,LOG_TYPE_ERRORLOG);
	  
	  break;
     case WARNING:
	  MessageHeader=String("Warning ")+ThisProcess->GetName()+String(": ");
	  strcpy(LogType,LOG_TYPE_ERRORLOG);
	  
	  break;
     case INFO:
	  break;
	  
     }
     sprintf(NewMsg,"%s%s",(const char*)MessageHeader,Msg);
     if(LogLevelFile<=VerboseLevelFile)
	  Dispatcher.LogEntry(LogType,NewMsg);
     if(LogLevelConsole<=VerboseLevelConsole){
	  cerr << NewMsg << endl;
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,String("cmd_info\r\n")+NewMsg+String("\r\n"));
     }
     time_t CurTime=time(NULL);
     String Time=asctime(gmtime(&CurTime));
     Time.gsub("\n","");
     if(Type==WARNING){
	  if(WarningStream->pcount()<31000)
	       (*WarningStream) << Time << endl << NewMsg << endl << endl;
	  if(LastWarningTime==0 || (CurTime-LastWarningTime)>=WarningIntervall){
	       LastWarningTime=CurTime;
	       (*WarningStream) << ends;
	       WarningStream->freeze(1);
	       if(Mail)Dispatcher.MailMsg(WarningStream->str());
	       WarningStream->freeze(0);
	       delete WarningStream;
	       WarningStream=new ostrstream;
	  }
     }
     else if(Type==ERROR){
	  if(ErrorStream->pcount()<31000)
	       (*ErrorStream) << Time << endl << NewMsg << endl << endl;
	  if(LastErrorTime==0 || (CurTime-LastErrorTime)>=ErrorIntervall){
	       LastErrorTime=CurTime;
	       (*ErrorStream) << ends;
	       ErrorStream->freeze(1);
	       if(Mail)Dispatcher.MailMsg(ErrorStream->str());
	       ErrorStream->freeze(0);
	       delete ErrorStream;
	       ErrorStream=new ostrstream;
	  }
     }
     delete [] NewMsg;
}


