/*
$Header: /home/elrond/baikal/src/CVS/daq98/daemon.h,v 1.4 1998/11/23 14:41:13 haase Exp $
*/

#if ! defined __DAEMON_H__
#define __DAEMON_H__

#include <config.h>
#include <strstream.h>
#include <String.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "ionotifier.h"
#include "cnfreader.h"
#include "daemonbase.h"

class Daemon:public DaemonBase /***/
{
public:
     enum { D_USE_SHM=1,     // deamon uses shared memory 
	    D_NOTIFY_SHM=2,  // daemon gets notified via IPC socket 
	                     // that SHM was created or destroyed
	    D_CREATE_SHM=4   // create the shared memory
     };
     typedef int DaemonFlags_t;
     
     /** ctor */
     Daemon(DaemonFlags_t df);
     /** dtor */
     virtual ~Daemon();
     /** initializtion of resources*/
     int Init();
     /** destruction of resources*/
     int Destroy();

     /** Sets set step mode.
	 Derived classes can poll the step mode flag and ask 
	 to press a key or so before they continue if \Ref{StepMode} is set. 
	 @param mode either <tt>Daemon::ON</tt> or <tt>Daemon::OFF</tt>
	 @return old step mode setting */
     int SetStepMode(int mode);
     
     /** Get current step mode.
	 @return -1 if single step mode is not supprted,
	 returns the current step mode otherwise */
     inline int GetStepMode();
     
     /** Returns step mode flag.
	 @return true if \Ref{StepMode} is set, false otherwise */
     inline bool IsStepMode();
     
     /** Prints a message to a common log file.
	 @param Type Log type, can be either {\tt ERROR},
	 {\tt WARNING}, or {\tt INFO}
	 @param Msg Message to print
	 @param Level Log Level ranging from 0 to 110.
	 TODO: Not defined yet */
     virtual void LogMsg(const char* Msg, int LogLevelFile=10, int LogLevelConsole=10, LogType_t Type=INFO);

protected:

     /** returns the size fo the shared memory, if the flag
         D_CREATE_SHM was set */
     virtual unsigned int GetSHMSize(){return 0;}
     /** returns the No. of segments of SHM to be created */
     virtual unsigned int GetSHMSegments(){return 0;}
     /** Attach to the shared Memory */
     int AttachSHM();
     /** detach from the shared memory */
     int DetachSHM();

     /** the IPC event handler */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     virtual bool WasInternalCommand();
     int I_CMD_UnknownCommand(IOSocket &s,String *params,int numparams);
     int I_CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams);
     int Fork2Background();
protected:
     /** Internal comman interpreter. 
       Handles internal commands (e.g. switching between modes). */
     Interpreter<Daemon> I_CMD_Interpreter;
     
     /** Flag to store operatiorn modes of the daemon */
     DaemonFlags_t DaemonFlags;

     /** Indicates step mode */
     bool StepMode;

     /** Daemon supports step mode if true. */
     bool SuppStepMode;

     /** If true the next statement can be executed while in 
	 single step mode. */
     bool Continue;
     bool InternalCommand;
     
     /** Flag to indicate wether the process is attached to sharde
	 memory or not */
     bool GotSharedMem;
     /** a pointer to the Shared Meory manager */
     SharedMemoryManager *ShmMan;
     /** the dispatcher interface */
     DispatcherLib_t Dispatcher;
private:
     /** flag to indicate wether the process is registered at the
	 dispatcher or not */
     bool Registered;
     PID_t pid;
};


#ifdef NOTHING
// Templates to use for writing a real deamon server

/* put the class defionitions in a header file and replace NAME with
   the actual name of your daemon server */

/**************************************************************************
	    Begin of template class
**************************************************************************/

class NAMEConfig:public ConfigSection
{
public:
     /** ctor */
     NAMEConfig(port_t &comport);
     /** copy ctor */
     NAMEConfig(const RealDaemonConfig &rdc);
     /** returns a pointer to a new copy of the object on the heap */
     virtual ItemBase *clone()const;
     /**return the name of the section in the config file, where the
	config items are placed */
     virtual const String GetSectionName()const;
     /** sets property propnum */
     virtual int SetPropertyByNo(int propnum,const String &val);
     /** returns property propnum */
     virtual String GetPropertyByNo(int propnum)const;
     // Add references to the data that has to be read here and
     // initialize them with the constructor
};

class NAME:public
{
public:
     /** ctor */
     NAME();
     /** dtor */
     ~NAME();
     /** initialization */
     Init();
     /** destruction */
     Destroy();
protected:
     /** the event dispatcher */
     virtual int DispatchEvent();
     /** the IPC event handler */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** returns a String with the configuration file name */
     virtual const String &GetConfigFileName()const;
     /** returns a pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ConfigSection *GetConfigClass();
     /** returns the type of the process, must be overwritten in
         derived class */
     virtual const String &GetProcessType()const;
};

NAMEConfig::NAMEConfig(port_t &comport):ConfigSection(){
     static String Props[]={add the names of the config items here, "," separated list of strings};
     SetProperties(sizeof(Props)e,Props);
}

/** copy ctor */
NAMEConfig::NAMEConfig(const NAMEConfig &rdc):ConfigSection(rdc),ComPort(rdc.ComPort){
}

/** returns a pointer to a new copy of the object on the heap */
ItemBase *NAMEConfig::clone()const{
     return new NAMEConfig(*this);
}
     
/** return the name of the section in the config file, where the
   config items are placed */
const String NAMEConfig::GetSectionName()const{
     return "NAME";
}
     
/** sets property propnum */
int SetPropertyByNo(int propnum,const String &val){
     switch(propnum){
     case 0: do something with val; break;
     case n: do something with val; break;
	  .
	  .
	  .
     default: return -1; break;
     }    
     return 0;
}


/** returns property propnum */
String NAMEConfig::GetPropertyByNo(int propnum)const;{
     switch(propnum){
     case 0: do something with val; break;
     case n: do something with val; break;
	  .
	  .
	  .
     default: return ""; break;
     }    
     return "";
}



/** ctor */
NAME::NAME():Daemon(insert flags here){
}

/** dtor */
NAME::~NAME(){
}

/** initialization */
NAME::Init(){
     TEST_FAILURE(Daemon.Init());
     // do some more initialization stuff like:
//      OtherSocket_Socket.Open(bla);
//      OtherSocket_SocketNotifier.Set(OtherSocket_Socket);
//      Selecter.AddNotifier(OtherSocket_SocketNotifier);
}

/** destruction */
NAME::Destroy(){
     Daemon::Destroy();
     do some more destruction
}

/** the event dispatcher */
int NAME::DispatchEvent(){
     // call the dispatcher of the base class
     TEST_FAILURE(Daemon::DispatchEvent());
     // check out other sockets like :
//      if (Selecter.CheckNotifier(OtherSocket_SocketNotifier)){
// 	  char *txt;
// 	  // yes then process the text message
// 	  TEST_FAILURE(0==(txt=OtherSocket_Socket.ReadText())?-1:0);
// 	  TEST_FAILURE(DoSomethingWithText(txt,OtherSocket_Socket));
//      }
     return 0;
}

/** the IPC event handler */
int NAME::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     do some stuff with the cmd
}

/** returns a String with the configuration file name */
const String &NAME::GetConfigFileName()const{
     return DEFAULT_DAEMON_CNF_FILENAME;
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ConfigSection *NAME::GetConfigSection(){
     return new NAMEConfig(bla, bla); /* constructor should take
					 references to all items that
					 are read from the config file */
}

/** returns the type of the process, must be overwritten in
    derived class */
const String &NAME::GetProcessType()const{
     return PROCESS_TYPE; // whatever
}


/**************************************************************************
	    End of template class
**************************************************************************/


class RealDaemonConfig:public ConfigSection
{
public:
     // ctor 
     RealDaemonConfig(port_t &comport):ComPort(comport){
 	  static String Props[]={"comport"};
 	  SetProperties(1,Props);
     }
     // copy ctor 
     RealDaemonConfig(const RealDaemonConfig &rdc)
	  :ConfigSection(rdc),ComPort(rdc.ComPort){}
     // returns a pointer to a new copy of the object on the heap
     ItemBase *clone()const{
	  return new RealDaemonConfig(*this);
     }       
     // return the name of the section in the config file, where the
     // config items are placed
     virtual const String GetSectionName()const{
	  return String("RealDaemon");
     }
     // sets property propnum 
     virtual int SetPropertyByNo(int propnum,const String &val){
 	  switch(propnum){
 	  case 0: ComPort=atoi(val); break;
 	  default: return -1; break;
 	  }    
 	  return 0;
     }
     /** returns property propnum */
     virtual String GetPropertyByNo(int propnum)const{
 	  switch(propnum){
 	  case 0: return ltoasc(ComPort); break;
 	  default: return ""; break;
 	  }    
 	  return "";
     }
     int &ComPort;
};


// a real daemon server might look like this: it uses one socket
// connection to talk with the client and one socket wich is inherited
// from the deamon class
class RealDaemon:public Daemon
{
public:
     /** ctor, use shared memory and get notified on creation or
         destruction of shared memory */
     RealDaemon():Daemon(D_USE_SHM|D_NOTIFY_SHM){
     }
     ~RealDaemon(){
     }
     Init(){
	  // initialize IPC socket, read config file, attach to SHM if
	  // possible
	  TEST_FAILURE(Daemon::Init());
	  // open the communication socket
	  TEST_FAILURE(ComSocket.Open(ComPort));
	  // set the notifier for the socket
	  ComSocketNotifier.Set(ComSocket);
	  // add the notifier to the selecter
	  Selecter.Add(ComSocketNotifier);
     }
     Destroy(){
	  // close the IPC socket, detach from SHM  
	  Daemon::Destroy();
	  // close the communication socket
	  ComSocket.Close();
     }
protected:
     virtual int DispatchEvent(){
	  // call the dispatcher of the base class
	  TEST_FAILURE(Daemon::DispatchEvent());
	  // check out added socket for an event
	  if (Selecter.CheckNotifier(ComSocketNotifier)){
	       char *txt;
	       // yes then process the text message
	       TEST_FAILURE(0==(txt=ComSocket.ReadText())?-1:0);
	       TEST_FAILURE(DoCOMEvent(txt,ComSocket));
	  }
	  return 0;
     }
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
	  IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
     }
     virtual int DoCOMEvent(const String &Cmd,IOSocket &s){
	  IPC_COM_Interpreter.Do_wo_type(this,Cmd,s);
     }
     virtual const String &GetConfigFileName()const{
	  return "config_file_name";
     }
     /** returns a pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ConfigSection *GetConfigSection(){
	  return new RealDaemonConfig(SocketPort);
     }
     /** returns the type of the process, must be overwritten in
         derived class */
     virtual const String &GetProcessType()const{
	  return PROCESS_TYPE_REAL_DAEMON; // a macro which might be "RD"
     }
private:
     port_t SocketPort ;
     Socket ComSocket;
     SocketNotifier ComSocketNotifier;

     Interpreter<RealDaemon> COM_CMD_Interpreter;
     int COM_CMD_Print_Command(IOSocket &S,String *param,int numparams){
	  // do something
     }
     int COM_CMD_Quit(IOSocket &S,String *param,int numparams){
	  // do something
     }
     int COM_CMD_UnknownCommand(IOSocket &S,String *param,int numparams){
	  // do something
     }
     
     Interpreter<RealDaemon> IPC_CMD_Interpreter;
     int IPC_CMD_PrintCommand(IOSocket &S,String *param,int numparams){
	  // do something
     }
     int IPC_CMD_UnknownCommand(IOSocket &S,String *param,int numparams){
	  // do something
     }
     int IPC_CMD_AttachSharedMemory(IOSocket &s,String *params,int numparams){
	  // do something
     }
     int IPC_CMD_DetachSharedMemory(IOSocket &s,String *params,int numparams){
	  // do something
     }


};




/* the main function should look like this */

int main(int /*argc*/,char **/*argv*/){
     DEB_F("int main(int argc,char *argv[])");
     // initializing some global pointer variables
     // the default signal handler class
     SignalHandlerClass defSigHandler;
     DefaultSignalHandler=&defSigHandler;
     // the process class
     Process aProcess;
     ThisProcess=&aProcess;
     // set the name of the application
     ThisProcess->SetName("dataprocess");
     // for program error logging
     LogFile LF(ThisProcess->GetName());					
     Logger::SetLogger(&LF);				
     
     // Lock the process, 
     if(ThisProcess->LockProcess()<0){
	  Logger Entry;
	  Error.Print(Entry());
	  return 1;
     }
     // create the daemon object
     DataProcessDaemon DPServer;
     // initialize the daemon, allocate recources
     if(DPServer.Init()<0){
	  Logger Entry;
	  Error.Print(Entry());
     }else{
	  // signal parent process, that the process was started
	  // successfully 
	  kill(getppid(),SIGUSR1);
	  // enter the event loop
	  if(DPServer.EventLoop()<0){
	       Logger Entry;
	       Error.Print(Entry());
	  }
     }
     // destroy the demon, free all recources
     DPServer.Destroy(); 
     ThisProcess->UnlockProcess();
     return 0; 
}

#endif // NOTHING

#endif //  __DAEMON_H__
