/*
$Header: /home/elrond/baikal/src/CVS/daq98/daemonbase.C,v 1.5 1999/01/29 08:50:03 steffen Exp $
*/

#include <strstream.h>
#include <String.h>
#include <string.h>
#include <localdir.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "ionotifier.h"
#include "cnfreader.h"
#include "daemonbase.h"
 

DaemonBase::DaemonBase(){
     DEB_F("DaemonBase::DaemonBase()");

     // set up some additional signal handler
     (*ThisProcess).GetSignalHandler().SignalAction(SIGHUP,(sighandler_t)SIG_IGN);
     (*ThisProcess).GetSignalHandler().SignalAction(SIGCONT,(sighandler_t)SIG_IGN);
     (*ThisProcess).GetSignalHandler().SignalAction(SIGTSTP,(sighandler_t)SIG_IGN);
     (*ThisProcess).GetSignalHandler().SignalAction(SIGTTIN,(sighandler_t)SIG_IGN);     
     (*ThisProcess).GetSignalHandler().SignalAction(SIGTTOU,(sighandler_t)SIG_IGN);     
     (*ThisProcess).GetSignalHandler().SignalAction(SIGPIPE,(sighandler_t)SIG_IGN);     

     Quit_FLAG=false;
}

DaemonBase::~DaemonBase(){
     DEB_F("DaemonBase::~DaemonBase()");
}
   
int DaemonBase::Init(){
     DEB_F("int DaemonBase::Init()");
     
     // open the IPC socket 
     TEST_FAILURE(IPCSocket.Open(IPCAddress.Port,5));
     // and add it a Notifier to the selecter for notification on
     // connection request
     IPCSocketNotifier.Set(IPCSocket,IONotifier::READ);
     Selecter.AddNotifier(IPCSocketNotifier);
     
     return 0;
}

int DaemonBase::Destroy(){
     DEB_F("int DaemonBase::Destroy()");
     return 0;
}

int DaemonBase::EventLoop(){
     DEB_F("DaemonBase::DoLoop()");
     int ret;
     // do the loop until someone set th Quit_flag, might be a signal
     // handler, a subroutine or some one alse
     while(!Quit_FLAG){
	  // check all sockets added to the Selecter class for a
	  // connection request
	  TEST_TEMP_FAILURE(ret=Selecter.DoSelect());
	  if(ret<0){
	       // select interrupted by a signal
	       continue;
	  }
	  // if there was no error dispatch the event
	  if(ret>0) TEST_FAILURE(DispatchEvent());
	  if(0==ret) TEST_FAILURE(DoTIMEREvent());
     }
     return 0;
}

int DaemonBase::DispatchEvent(){
     // check the IPC socket for a connection request
     if(Selecter.CheckNotifier(IPCSocketNotifier)) 
	  // yes, then class the appropriate member function
	  TEST_FAILURE(ServIPCSocket());
     return 0;
}	       

void DaemonBase::SetTimer(int time_val){
     Selecter.SetTimer(time_val);
}    

int DaemonBase::DoTIMEREvent(){
     return 0;
}

int DaemonBase::ServIPCSocket(){
     DEB_F("int ServIPCSocket()");
     const char *txt;
     filedescriptor_t sd;
     // IPC socket is a Server socket so we must accept the connection
     // and create and open a slave server socket in order to get the
     // message/command or whatever was sent
     SlaveServerSocket SSS;
     TEST_FAILURE(sd=IPCSocket.Accept());
     TEST_FAILURE(SSS.Open(sd));
     // read the text
     TEST_FAILURE(0==(txt=SSS.ReadText())?-1:0);
     // class the Event handler for the IPC message
     TEST_FAILURE(DoIPCEvent(txt,SSS));
     return SSS.Close();
}

int DaemonBase::ReadConfigFile(){
     // create an instance of the config file reader class
     ConfigReader cfr;
     // add our config section, wich must be and was created on the
     // heap using new
     // create the file stream from the file name
     String cnf_file=GetConfigFileName();
     String cnf_files[4];
     cnf_files[0]="./" + cnf_file;
     char *path=getenv("HOME");
     if(path){
	  cnf_files[1]=path + String("/.") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[1]="";
     } 
     path=getenv("DAQ_ROOT_DIR");
     if(path){
	  cnf_files[2]=path + String("/share/") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[2]="";
     } 
     cnf_files[3]= String(DAQ_ROOT_DIR) + String("/share/") + 
	  String(PACKAGE) + "/" + cnf_file;
     ifstream file;
     for(int i=0;i<4;i++){
	  file.open(cnf_files[i]);
	  if(file) {
	       Logger Entry;
	       Entry() << "Using configuration file '" << 
		    cnf_files[i] << "'." << endl;
	       i=3;
	  }
     }
     ClassWithTypeInfo_t *cs=GetConfigSection();
     if(cs!=0){
	  cfr.AddSection(cs);
     }
     ClassWithTypeInfo_t *Common=GetCommonConfigSection();
     cfr.AddSection(Common);
     // and do the read
     return cfr.Read(file);
}
