/*
$Header: /home/elrond/baikal/src/CVS/daq98/daemonbase.h,v 1.4 1999/02/03 03:42:13 steffen Exp $
*/

#if !defined __DAEMONBASE_H__
#define __DAEMONBASE_H__

#include <config.h>
#include <strstream.h>
#include <String.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "ionotifier.h"
#include "cnfreader.h"

/** Base class for all daemon processes */
class DaemonBase:public DaqProcessBase
{
public:
     /** Constructor */
     DaemonBase();

     /** Destructor */
     virtual ~DaemonBase();

     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 @return 0 on success, -1 on failure.  */
     int Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 @return 0 on success, -1 on failure.  */
     int Destroy();

     /** The event loop.
	 Dispatches incoming events and calls timer events.
	 @return 0 on success, -1 on failure.  */
     int EventLoop();

     /** Sets the time interval of the {\tt \Ref{DoTIMEREvent}} function.
         @param time_val Time between two timer events in in 0.1 seconds. A
	 value of 0 stops the timer. */
     void SetTimer(int time_val);

     /** Gets the proces type.
	 This function  must be overwritten in every derived class
	 @return Type of the process */
     virtual const String GetProcessType()const=0;

protected:
     /** Dispatches incomming events.
	 Can be overwritten in derived class to take particular actions.
	 @return 0 on success, -1 on failure. */
     virtual int DispatchEvent();

     /** The IPC event handler
	 Calls the appropriate command handling function.
	 Must be overwritter in every derived class.
	 @return 0 on success, -1 on failure. */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s)=0;
     
     /** Timer event function. 
	 The timer value can be set with {\tt \Ref{SetTimer}}
	 @return 0 on success, -1 on failure. */
     virtual int DoTIMEREvent();

     /** Gets the configuration file name.
	 Must be overwritten for every derived class.
	 @return Name of configuration file for this class */
     virtual const String GetConfigFileName()const=0;

     /** Creates a class for handling the configuration of this class.
         The class instances is created with new on the heap.
	 This function must be overwritten in derived class 
	 @return Pointer to the configuration class */
     virtual ClassWithTypeInfo_t *GetConfigSection()=0;

     /** Reads the config file 
	 @return 0 on success, -1 on failure.  */
     int ReadConfigFile();

     /** States for step mode.
	 Use \Ref{SetStepMode} to set step mode either on or off.
     */
     enum {
	  OFF,
	  ON
     };
     /** Sets set step mode.
	 Derived classes can poll the step mode flag and ask 
	 to press a key or so before they continue if \Ref{StepMode} is set. 
	 @param mode either {\tt Daemon::ON} or {\tt Daemon::OFF}
	 @return old step mode setting */
     int SetStepMode(int mode);
     
     /** Get current step mode.
	 @return -1 if single step mode is not supprted,
	 returns the current step mode otherwise */
     inline int GetStepMode();
     
     /** Checks step mode flag.
	 @return true if {\tt \Ref{StepMode}} is set, false otherwise */
     inline bool IsStepMode();
     
     
private:
     /** This function is called whenever something happend on the IPC socket
	 @return 0 on success, -1 on failure.  */
     virtual int ServIPCSocket();

protected:
     /** The SelectClass for the "communication event" building */
     SelectClass Selecter;
     /** Port of the IPC socket */
     IPAddress_t IPCAddress;
     /** The IPC socket */
     ServerSocket IPCSocket;
     /** Step mode flag */
     bool StepMode;
     /** Daemon supports step mode if true. */
     bool SuppStepMode;
          
private:
     /** Notifier for the IPC socket */
     SocketNotifier IPCSocketNotifier;
};

#endif //  __DAEMONBASE_H__
