#if ! defined __DAQ98F2000_H__
#define __DAQ98F2000_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98f2000.h,v 1.9 1999/02/05 18:35:02 steffen Exp $
*/

#include <config.h>
#include "globaltypes.h"
#include "event.h"
#include "daq98muoneventana.h"
#include "dmadmodule.h"

/** A class that produces f2000 output from muon events */
class Daq98F2000_t:public Daq98MuonEventAnalysis_t 
{
public:
     enum output_mode_t { 
	  OUTMODE_SINGLE_EVENT,        // one event per file, the user must 
	                               // take care for providing a new filestream
	                               // with SetOutputStream 
	  OUTMODE_MULTIPLE_EVENTS      // all events in one file
     };
     
     /** Constructor */
     Daq98F2000_t(DaqProcessBase *base,output_mode_t o_mode=OUTMODE_MULTIPLE_EVENTS);

     /** Destructor */
     ~Daq98F2000_t();
     
     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 @return 0 on success, -1 on failure.  */
     virtual int Init();

     /** Sets output filename.
	 @param File file name */
     void SetOutputStream(ostream *Stream);

     /** Sets year of F2K output
	 @param i year */
     void SetYear(int i);

     /** Write an CounterEvent*/
     int AnalyseCounterEvent(CounterEventHeader_t EventHeader,
			     S7201CounterEvent_t &CntEvnt);

     /** Write an DAMD Event (Status) */
     int AnalyseDMADEvent(DMAD_EventHeader_t &DMADHeader,
			  DMAD_ModuleConfigRecord_p DMAD_Event);
     
     /** Called after reading the last event.
	 This function saves all histograms that we have in
	 memory in an hbook file.
	 @return 0 on success and -1 on failure */
     virtual int AnalyseDataFinal();

protected:
     /** Analyses data.
       @param EventHeader Muon event header
       @param Event Muon event data
       @return 0 on success and -1 on failure*/ 
     virtual int __AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event);

protected:
     /** The f2000 output file*/
//     char *F2000File;

     /** The f2000 output stream*/
     ostream *F2000Stream;

     /** The f2000 output mode */
     output_mode_t output_mode;

     /** year of current event */
     int year;
     
     /** day of current event */
     int day;
     
     /** UTC of current event */
     double UTC;

     /** Year which will be used if not zero */
     int external_year;
};     

/** class to transport an event in F2000 format over a stream, see TransferClass */
class f2000transport_t{
public:
     f2000transport_t(Daq98F2000_t *f2000,MuonEventHeader_t& meh,MuonEventData_p med):
	  F2000(f2000),MEH(meh),MED(med),outputfile(0){};
     f2000transport_t(ostream *of)
	  :F2000(0),MEH(*((MuonEventHeader_t*)0)),MED(0),outputfile(of){};
     int Serialize(ostream &sstream){
	  assert(F2000);
	  F2000->SetOutputStream(&sstream);
	  F2000->AnalyseData(MEH,MED);
	  sstream << "\nfinish\r\n";
	  return 0;
     }
     int Deserialize(istream &rstream){
	  assert(outputfile);
	  char line[1000];
	  while(!rstream.eof()){
	       rstream.getline(line,999);
	       if(strcmp(line,"finish\r")==0) break;
	       *outputfile << line << "\n";
	  }
     }
     static int Pipe(istream &from,ostream &to){
	  char line[1000];
	  while(!from.eof()){
	       from.getline(line,999);
	       if(strcmp(line,"finish\r")==0) break;
	       to << line << "\n";
	  }
	  to << "finish\r\n";
	  return 0;
     }
private:
     Daq98F2000_t *F2000;
     MuonEventHeader_t& MEH;
     MuonEventData_p MED;
     ostream *outputfile;
};

#endif // __DAQ98F2000_H__
