#if ! defined __DAQ98HWCONFIG_H__
#define __DAQ98HWCONFIG_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98hwconfig.h,v 1.6 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include "daq98rawdataanalysis.h"

/** Class that sets up all hardware classes.
    For almost any kind of data analysis you need to set
    up the classes that deal with the detector hardware.
    This class does all the work for you */
class Daq98HWConfigReader_t:public Daq98RawDataAnalysis_t 
{
public:
     /** Constructor */
     Daq98HWConfigReader_t(DaqProcessBase *parent);

     /** Destructor */
     virtual ~Daq98HWConfigReader_t();
     
     /** Read detector config from file.
	 @param File input file name */
     int ReadCnfFromFile(const char *File);
     
protected:
     /** Analyses data.
	 Note that this function is pure virtual and must be overloaded
	 in every derived class.
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 1 if the data was read from the buffer and the 
	 data processing was successful. Returning 1 tells the 
	 base class to rewind the buffer that it can be used by
	 other data handling classes.\\ \\
	 0 if the data was not touched.\\ \\
	 {\tt CRITICAL\_ERROR} on, you guessed it, critical errors. This leads to the abortion
	 of the program. \\ \\
	 {\tt NONCRITICAL\_ERROR} on internal failure. 
	 It is assumed that the data was read (how else
	 could you tell that there was an error) so the buffer will be rewind after
	 that by the base class. 
	 If you did not touch the data and somehow managed to produce an error
	 you have to call the {\tt \Ref{DontRewind}} function. */
     inline virtual int __AnalyseData(RecordHeader_t RecordHeader, 
			       BinaryDataBuffer &Buf);
protected:
     /** Input file name */
     char *InFile;
     
     /** Input stream */
     istream *InStream;

};

MAKE_REFCOUNT_POINTER(Daq98HWConfigReader_t,Daq98HWConfigReader_p);

int Daq98HWConfigReader_t::__AnalyseData(RecordHeader_t RecordHeader, 
					 BinaryDataBuffer &Buf)
{
     if(RecordHeader.RecordID==RID_HWCONFIG_ASCII){
	  LogMsg("Ascii config record received",8,8);
	  if(HWConfig!=NULL) delete HWConfig;
	  HWConfig=new HWConfigAscii_t;
     
	  if(HWConfig->ReadFromBuffer(Buf)==0){
	       delete HWConfig;
	       HWConfig=0;
	       return CRITICAL_ERROR;
	  }
	  return 1;
     }
     return 0;
}


#endif // __DAQ98HWCONFIG_H__
