#if ! defined __DAQ98HWCONFIGEXTRACT_H__
#define __DAQ98HWCONFIGEXTRACT_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98hwconfigextract.h,v 1.4 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include "daqprocessbase.h"
#include "daq98rawdataanalysis.h"

/** Class that extracts the detector config information from
    a binary data file and writes it to an ASCII file */
class Daq98HWConfigExtract_t:public Daq98RawDataAnalysis_t 
{
public:
     /** Constructor */
     Daq98HWConfigExtract_t(DaqProcessBase *parent);

     /** Destructor */
     virtual ~Daq98HWConfigExtract_t();
     
     /** Initalize */
     virtual int Init();
     
     /** Set output file name.
	 @param File file name */
     void SetFile(const char *File);
     
protected:
     /** Analyses data.
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 1 if the data was read from the buffer and the 
	 data processing was successful. Returning 1 tells the 
	 base class to rewind the buffer that it can be used by
	 other data handling classes.\\ \\
	 0 if the data was not touched.\\ \\
	 {\tt CRITICAL\_ERROR} on, you guessed it, critical errors. This leads to the abortion
	 of the program. \\ \\
	 {\tt NONCRITICAL\_ERROR} on internal failure. 
	 It is assumed that the data was read (how else
	 could you tell that there was an error) so the buffer will be rewind after
	 that by the base class. 
	 If you did not touch the data and somehow managed to produce an error
	 you have to call the {\tt \Ref{DontRewind}} function. */
     inline virtual int __AnalyseData(RecordHeader_t RecordHeader, 
				      BinaryDataBuffer &Buf);

protected:
     /** Output file name */
     char *OutFile;
     
     /** Output stream */
     ostream *OutStream;
};

MAKE_REFCOUNT_POINTER(Daq98HWConfigExtract_t,Daq98HWConfigExtract_p);

int Daq98HWConfigExtract_t::__AnalyseData(RecordHeader_t RecordHeader, 
					 BinaryDataBuffer &Buf)
{
     if(RecordHeader.RecordID==RID_HWCONFIG_ASCII){
	  LogMsg("Received an Ascii config record",8,8);
	  HWConfigAsciiExtract_t HWConfigExtract;
	  if(HWConfigExtract.ReadFromBuffer(Buf)==0){
	       return CRITICAL_ERROR;
	  }
	  HWConfigExtract.cfg_strstream->freeze(1);
	  (*OutStream) << HWConfigExtract.cfg_strstream->str();
	  HWConfigExtract.cfg_strstream->freeze(0);
	  exit(EXIT_SUCCESS);
     }
     
     return 0;
}

#endif // __DAQ98HWCONFIGEXTRACT_H__
