#if ! defined __DAQ98MOUNEVENTANA_H__
#define __DAQ98MOUNEVENTANA_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98muoneventana.h,v 1.4 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include "event.h"
#include "daq98rawdataanalysis.h"

class Daq98MuonEventBuilder_t;

/** A base class for analysing muon event data */
class Daq98MuonEventAnalysis_t
{
public:
     /** Constructor */
     Daq98MuonEventAnalysis_t(DaqProcessBase *Parent);
     
     /** Destructor */
     virtual ~Daq98MuonEventAnalysis_t();

     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 If you need to write your own {\tt Init} function, you have to
	 call {\tt Daq98MuonEventAnalysis\_t::Init()} from there to
	 initalize the base class as well.
	 @return 0 on success, -1 on failure.  */
     virtual int Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 If you need to write your own {\tt Destroy} function, you have to
	 call {\tt Daq98MuonEventAnalysis\_t::Destroy()} from there to
	 destroy the base class as well.
	 @return 0 on success, -1 on failure.  */
     virtual int Destroy();

     /** Analyses muon event data.
	 This is the only function that is visible from the
	 outside. After doing some houskeeping it calls
	 the virtual {\tt \Ref{__AnalyseData}} function which
	 does the actual data handling. {\tt \Ref{__AnalyseData}}
	 is the function you have to overwrite.
	 @param EventHeader Muon event header
	 @param Event Muon event data
	 @return 0 on success and -1 on failure */
     int AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event);

     /** Called after reading the last event.
	 This function gives your data analysis object
	 the possibility to do something after reading the
	 last record. This is a virtual function and
	 can be overloaded in your derived analyis classes.
	 @return 0 on success and -1 on failure */
     virtual int AnalyseDataFinal(){return 0;}

     /** Issue a log message depending on set log level */
     void LogMsg(const char *Msg, int LogLevelFile=10, int LogLevelConsole=10, 
		 LogType_t Type=INFO);
     
     friend Daq98MuonEventBuilder_t;
     
protected:
     /** Analyses data.
	 Note that this function is pure virtual and must be overloaded
	 in every derived class. It is called from {\tt \Ref{AnalyseData}}
	 after doing some housekeeping.
	 @param EventHeader Muon event header
	 @param Event Muon event data
	 @return 0 on success and -1 on failure */
     virtual int __AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event)=0;

protected:
     /** Pointer to the parent class */
     DaqProcessBase *Parent;

     /** The run number */
     static int &RunNumber;
     
     /** A pointer to the detector configuration */
     static HWConfig_t *HWConfig;
};     

MAKE_REFCOUNT_POINTER(Daq98MuonEventAnalysis_t,Daq98MuonEventAnalysis_p);
MAKE_INC_VECTOR(Daq98MuonEventAnalysis_p,Daq98MuonEventAnalysisVector_t);    

#endif // __DAQ98MOUNEVENTANA_H__
