#if ! defined __DAQ98MOUNEVENTBUILDER_H__
#define __DAQ98MOUNEVENTBUILDER_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98muoneventbuilder.h,v 1.9 1999/02/10 11:59:51 steffen Exp $
*/

#include <config.h>
#include "event.h"
#include "daq98rawdataanalysis.h"
#include "daq98muoneventana.h"
#include "scalermoduleeval.h"
#include "vmegpsmoduleeval.h"

/** A class that builds muon events from raw data.
  In its {\tt \Ref{__AnalyseData}} function it calls other classes
  that can act on the built muon event. */
class Daq98MuonEventBuilder_t:public Daq98RawDataAnalysis_t 
{
public:
     /** Constructor */
     Daq98MuonEventBuilder_t(DaqProcessBase *Parent);

     /** Destructor */
     ~Daq98MuonEventBuilder_t();

     /** Called after reading the last event.
	 @return 0 on success and -1 on failure */
     virtual int AnalyseDataFinal();

     /** Initializes class      
	 @return 0 on success and -1 on failure */
     virtual int Init();
     
     /** Sets the event data analysis vector.
	 @param Vector Pointer to vector holding muon event data analysis objects.*/
     static void SetAnalysisVector(Daq98MuonEventAnalysisVector_t *Vector);


     static int SetSNFile(const char *f) {
	  static ofstream o;
	  o.open(f);
	  if(!f) {
	       (*Base::Error)="Daq98MuonEventBuilder_t::SetSNFile";
	       (*Base::Error)=SYS_ERROR;
	       Base::Error->SetParam(String("Can't open '")
				+f+String("'"));
	       return -1;
	  }
	  SetSNStreamer(&o);
	  return 0;
     }

     static void SetSNStreamer(ostream *o) {
	  ScalerModuleEvalClass_t::outstream=o;
	  VME_GPSModuleEvalClass_t::outstream=o;
     }
     	  
     /** checks if a valid event is stored in the members */
     inline bool IsValid()const{ return is_valid; }
     
     inline MuonEventData_p GetLastMuonEvent()const{ 
	  return (MuonEventData_t*)MuonEventDataBuffer; 
     }
     inline const MuonEventHeader_t &GetLastMuonEventHeader()const{ 
	  return EventHeader; 
     }

protected:
     /** Builds a muon event from raw data.
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 1 if the data was read from the buffer and the 
	 data processing was successful. Returning 1 tells the 
	 base class to rewind the buffer that it can be used by
	 other data handling classes.\\ \\
	 0 if the data was not touched.\\ \\
	 {\tt CRITICAL\_ERROR} on, you guessed it, critical errors. This leads to the abortion
	 of the program. \\ \\
	 {\tt NONCRITICAL\_ERROR} on internal failure. 
	 It is assumed that the data was read (how else
	 could you tell that there was an error) so the buffer will be rewind after
	 that by the base class. 
	 If you did not touch the data and somehow managed to produce an error
	 you have to call the {\tt \Ref{DontRewind}} function. */
     inline virtual int __AnalyseData(RecordHeader_t RecordHeader, 
			       BinaryDataBuffer &Buf);

     
protected:
     /** Object that is able to build muon events */
     EventBuilder_t MEventBuilder;

     /** Vector that holds all muon event analysis objects */
     static Daq98MuonEventAnalysisVector_t *MuonEventAnalysisVector;
     
private:
     /** Mounon event builder needs hw configuration */
     bool EventBuilderNotConfigured;
     
     /** Buffer to hold muon events */
     void *MuonEventDataBuffer;
     
     /** Header for muon events */
     MuonEventHeader_t EventHeader;

     /** Header for muon events */
     SNEventHeader_t SNEventHeader;

     /** flag that indicate, that the infomration in EventHeader and
     MuonEventDataBuffer is valid */

     bool is_valid;
};     

    
#endif // __DAQ98MOUNEVENTBUILDER_H__
