#if ! defined __DAQ98MOUNEVENTHISTO_H__
#define __DAQ98MOUNEVENTHISTO_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98muoneventhisto.h,v 1.5 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include "globaltypes.h"
#include "event.h"
#include "daq98muoneventana.h"
#include "muoneventhistoset.h"

/** A class that analyses muon event data and does a lot of histogramming */
template <class HistogramType>
class Daq98MuonEventHisto_t:public Daq98MuonEventAnalysis_t{
public:
     /** Constructor */
     Daq98MuonEventHisto_t(DaqProcessBase *base);

     ~Daq98MuonEventHisto_t();

     /** Called after reading the last event.
	 This function saves all histograms that we have in
	 memory in an hbook file.
	 @return 0 on success and -1 on failure */
     virtual int AnalyseDataFinal();

//      /** Sets output filename.
// 	 @param File file name */
//      void SetHBookFile(const char *File);
     
protected:
     /** Analyses data.
       @param EventHeader Muon event header
       @param Event Muon event data
       @return 0 on success and -1 on failure*/ 
     virtual int __AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event);

protected:
     /** Histograms */
     MuonEventHistoSet_t<HistogramType> *Histograms;

     /** Histogram initialization flag */
     bool Init;
     
     /** Type of histograms */
     int Type;

//      /** The hbook file where the histograms are stored */
//      char *HBookFile;

     /** GPS Time of Previous event */
     GPSTime_t GPSPrev;
     
     /** GPS time difference between events */
     u_i64 GPSTimeDiff;
};     

template <class HistogramType>
Daq98MuonEventHisto_t<HistogramType>::Daq98MuonEventHisto_t(DaqProcessBase *parent)
     :Daq98MuonEventAnalysis_t(parent){
     Histograms=new MuonEventHistoSet_t<HistogramType>;
//      HBookFile=0;
     Init=false;
}

template <class HistogramType>
Daq98MuonEventHisto_t<HistogramType>::~Daq98MuonEventHisto_t(){
//      if(HBookFile!=0) delete [] HBookFile;
     delete Histograms;
}

template <class HistogramType>
int Daq98MuonEventHisto_t<HistogramType>::AnalyseDataFinal(){
//      if(HBookFile) HBOOKHistogram_t::SaveAllToFile(HBookFile);
     return 0;
}

// template <class HistogramType>
// void Daq98MuonEventHisto_t<HistogramType>::SetHBookFile(const char *File){
//      if(HBookFile!=0) delete [] HBookFile;
//      HBookFile=new char[strlen(File)+1];
//      strcpy(HBookFile,File);
// }

#define FAST_HISTOGRAMMS 1

template <class HistogramType>
int Daq98MuonEventHisto_t<HistogramType>::__AnalyseData(MuonEventHeader_t &EventHeader, 
							MuonEventData_p Event){    
     if(!Init){
	  if(!HWConfig){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int Daq98MuonEventHisto_t::__AnalyseData(..)";
	       Base::Error->SetParam("Can't book histogram. Class not configured yet");
	       return -1;
	  }
	  Histograms->Init(*HWConfig->ChannelConfigRecordVector);
#if FAST_HISTOGRAMMS==1
	  Histograms->PrepareForFastAccess(*HWConfig->ChannelConfigRecordVector);
#endif
	  Init=true;
     }

#if FAST_HISTOGRAMMS==1
     fa_per_OM_histos_type *fa_per_OM_histos;
     fa_misc_histos_type   *fa_misc_histos=Histograms->fa_GetMiscHistos();
#endif
     // Differential Event properties
#if FAST_HISTOGRAMMS==1
     fa_misc_histos->nom->Fill(Event->NoOfChannels);
#else
     Histograms->GetHisto("nom",0)->Fill(Event->NoOfChannels);
#endif
     SubEventData_p se_i=Event->SubEvents;
     for(int i=0;i<Event->NoOfChannels;i++){
	  int ID=(*se_i).ID;
#if FAST_HISTOGRAMMS==1
	  fa_per_OM_histos=Histograms->fa_GetChannelHistos(ID);
#endif	  
	  if((*se_i).Amplitude.isamp!=0){
	       // Trigger per OM, ADC
#if FAST_HISTOGRAMMS==1
	       fa_misc_histos->tom_adc->Fill(ID);
#else
	       Histograms->GetHisto("misc",TOM_ADC_H_OFFSET)->Fill(ID);
#endif
	       // ADC spectrum for channel
#if FAST_HISTOGRAMMS==1
	       fa_per_OM_histos->adc->Fill((*se_i).Amplitude.ampval);
#else
	       Histograms->GetHisto("adc",ID)->Fill((*se_i).Amplitude.ampval);
#endif
	  }
	  for(int j=0;j<(*se_i).NoOfEdges;j++){
	       // TDC spectrum for channel
#if FAST_HISTOGRAMMS==1
	       fa_per_OM_histos->tdc->Fill((*se_i).Times[j].timeval);
#else
	       Histograms->GetHisto("tdc",ID)->Fill((*se_i).Times[j].timeval);
#endif
	       // Trigger per OM, TDC
#if FAST_HISTOGRAMMS==1
	       fa_misc_histos->tom_tdc->Fill(ID);
#else
	       Histograms->GetHisto("misc",TOM_TDC_H_OFFSET)->Fill(ID);
#endif
	  }
	  // Number of Edges for channel
#if FAST_HISTOGRAMMS==1
	  fa_per_OM_histos->ned->Fill((*se_i).NoOfEdges);
#else
	  Histograms->GetHisto("ned",ID)->Fill((*se_i).NoOfEdges);
#endif
	  //  Get 1st and 2nd hit or edge 
	  if ((*se_i).NoOfEdges>1){
	       int e1,e2,t1,t2;
	       t1=(*se_i).Times[0].timeval;
	       e1=(*se_i).Times[0].edge;
	       t2=(*se_i).Times[1].timeval;         // next edge
	       e2=(*se_i).Times[1].edge;
	       if ((0==e1 && 1==e2) | (1==e1 && 0==e2)) {
		    // leading |LE2-LE1|
#if FAST_HISTOGRAMMS==1
		    fa_per_OM_histos->le->Fill(abs(t1-t2));
#else
		    Histograms->GetHisto("le",ID)->Fill(abs(t1-t2));
#endif
	       }
	  }
	  se_i++;
     }
     static u_i32 time_unix_start;
     static first = 1;
     
     if (first) {
	  first=0;
          time_unix_start = EventHeader.UnixTime;
     }
     
     u_i32  time_unix = EventHeader.UnixTime - time_unix_start;
     
     u_i64 diff=time_diff(GPSPrev, Event->GPSTime[0]);

#if FAST_HISTOGRAMMS==1
     fa_misc_histos->ratas->Fill(time_unix);
     fa_misc_histos->noh_adc->Fill(EventHeader.NHitADC);
     fa_misc_histos->noh_tdc->Fill(EventHeader.NHitTDC);
     fa_misc_histos->camac_trigger->Fill(Event->Trigger);
     fa_misc_histos->gps_1->Fill(diff);
     fa_misc_histos->gps_2->Fill(diff/1000);
     fa_misc_histos->gps_3->Fill(diff/1000);
#else
     // Rate vs. Time in Array
     Histograms->GetHisto("ratas",0)->Fill(time_unix);	
     // Number of hits ADC
     Histograms->GetHisto("misc",NOH_ADC_H_OFFSET)->Fill(EventHeader.NHitADC);
     // Number of hits TDC
     Histograms->GetHisto("misc",NOH_TDC_H_OFFSET)->Fill(EventHeader.NHitTDC);
     // CAMAC Trigger
     Histograms->GetHisto("misc",CAMAC_TRIGGER_H_OFFSET)->Fill(Event->Trigger);
     // GPS time differenc between two events 	  
     Histograms->GetHisto("misc",GPS_1_H_OFFSET)->Fill(diff);
     Histograms->GetHisto("misc",GPS_2_H_OFFSET)->Fill(diff/1000);
     Histograms->GetHisto("misc",GPS_3_H_OFFSET)->Fill(diff/1000);
#endif
     GPSPrev=Event->GPSTime[0];
     return 0;
}

#endif // __DAQ98MOUNEVENTHISTO_H__
