#if ! defined __DAQ98MOUNEVENTHITBUILDER_H__
#define __DAQ98MOUNEVENTHITBUILDER_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98muoneventhitbuilder.h,v 1.4 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include <stdio.h>
#include "globaltypes.h"
#include "event.h"
#include "daq98muoneventana.h"

/** A class that sorts the channels by their ID in one event */
class Daq98MuonEventHitBuilder_t:public Daq98MuonEventAnalysis_t{
public:
     /** Constructor */
     Daq98MuonEventHitBuilder_t(DaqProcessBase *base):Daq98MuonEventAnalysis_t(base){}
     
protected:
     /** Analyses data.
	 @param EventHeader Muon event header
	 @param Event Muon event data
	 @return 0 on success and -1 on failure*/ 
     virtual int __AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event){
	  SubEventData_StaticSize_t *SubEvents=Event->SubEvents;
	  int NumberOfChannels=Event->NoOfChannels;
	  for(int i=0;i<NumberOfChannels;i++){

	       int ne=SubEvents[i].NoOfEdges;
	       SubEvents[i].NoOfHits=0 ; // u_i16 nle_ch=0 //**
	       SubEvents[i].FifoOV=0;      //   ** FIFO overflow, if ==1 

	       u_i16 t_fullscale=32767;


	       if (ne > MAX_TIME_HITS) {              // this should never happen !!! ********
		    char msg[1000];
		    if(Parent->IsVerboseLevel(2)){
			 sprintf(msg,"too many edges ne %d, Event %d, chan %d",
				 ne,EventHeader.EventNumber,i);
			 LogMsg(msg,2,2,WARNING);
		    }
		    ne = MAX_TIME_HITS;
	       }

	       // Set the FIFO-OV ,  prelim. *** (K32)
	       if (ne==MAX_TIME_HITS)    SubEvents[i].FifoOV = 1;

#warning Wild Hack to get sorting order  
	       // fill all edges,  check if increasing (reversed)  order    (k32 vs 3377)
	       //   ** better: do sort in moduleeval module-type-wise !
	       bool reverse((ne>1) &&((SubEvents[i].Times[0].timeval)<(SubEvents[i].Times[1].timeval)));

	       // form a hit from next 1 or two edges:     
	       // 1)  LE & TE (hit_qual=3) ,  2) LE only (=1),  3) TE only (=2), 4) no LE &TE but Amp (!=0)
	       u_i16  e_1=0;
	       u_i16  t_1=0;                                
	       u_i16  e_2=0;
	       u_i16  t_2=0;                                
	       
	       bool  new_hit(true);
	       bool  stop_hit(false);
	       int   hit_qual=0;
	       
	       for (int j=0;j<ne;j++){
#warning Wild Hack to get sorting order  
		    u_i16 ti = t_fullscale-SubEvents[i].Times[reverse?ne-j-1:j].timeval;
		    u_i16 ed = SubEvents[i].Times[reverse?ne-j-1:j].edge;
	       
		    if(new_hit){
			 new_hit=false;
			 hit_qual=0;
			 stop_hit=0;
			 if(ed==TDCTime_t::LEADING_EDGE){         // ok - LE
			      e_1=ed;
			      t_1=ti;
			      hit_qual+=1;
			 }else{               // TE only
			      t_2=ti;
			      hit_qual+=2;
			      stop_hit=true;
			 }
		    }else{                     
			 stop_hit=true;
			 if(ed==TDCTime_t::TRAILING_EDGE){         // ok - TE
			      e_2=ed;
			      t_2=ti;
			      hit_qual+=2;
			 } else j--;               // next LE - reset
		    }
	       
		    if (stop_hit || (j==(ne-1)) ) {           //  hit ready for output (hit finished or last hit) 
			 SubEvents[i].Times_le_tot[SubEvents[i].NoOfHits].flags=hit_qual;
			 if(hit_qual==3){                 //  LE & TE
			      SubEvents[i].Times_le_tot[SubEvents[i].NoOfHits].le=t_1;
// ***			      tot[nle_ch-1]=t_1-t_2;
			      SubEvents[i].Times_le_tot[SubEvents[i].NoOfHits].tot=t_2-t_1;
			 } else if(hit_qual==1){             // LE only
			      SubEvents[i].Times_le_tot[SubEvents[i].NoOfHits].le=t_1;
			 } else if(hit_qual==2){             // TE only
			      SubEvents[i].Times_le_tot[SubEvents[i].NoOfHits].tot=t_2;
			 } else if(Parent->IsVerboseLevel(2)){
			      char msg[1000];
			      sprintf(msg,"hit_qual = %d",hit_qual);
			      LogMsg(msg,2,2,WARNING);
			 }
			 SubEvents[i].NoOfHits++;
			 new_hit=true;
			 stop_hit=false;
			 hit_qual=0;
		    }
	       }
     
	  }
	  return 0;
     }
};     

#endif 
