#if ! defined __DAQ98RUNINFO_H__
#define __DAQ98RUNINFO_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98runinfo.h,v 1.9 1999/02/03 03:42:14 steffen Exp $

  $Log: daq98runinfo.h,v $
  Revision 1.9  1999/02/03 03:42:14  steffen
  * introduced DaqProcessBase. Classes that only need Error excess can
  now use base

  Revision 1.8  1999/01/29 08:50:04  steffen
  - changes for compilation under egcs
  - install target works now

  Revision 1.7  1998/11/23 14:41:39  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.6  1998/11/03 06:43:54  daq
  merged pamanda2 and pamanda3 versions

  Revision 1.5  1998/08/11 01:49:27  steffen
  configure changes


  Revision 1.4  1998/08/03 01:41:12  steffen
  class description added

*/

#include <config.h>
#include "event.h"
#include "daqprocessbase.h"
#include "daq98rawdataanalysis.h"

/** This class extracts information about a run from the raw data stream.
    Information as defined in {\tt \Ref{RunInfo_t}} is extracted and printed
    verbose level 3. */
class Daq98RunInfo_t:public Daq98RawDataAnalysis_t 
{
public:
     /** Constructor 
       @param Parent Pointer to parent class which must be derived from class {\tt \Ref{Base}}.*/
     Daq98RunInfo_t(DaqProcessBase *Parent);

     /** Destructor */
     ~Daq98RunInfo_t();
     
     /** Initializes class      
	 @return 0 on success and -1 on failure */
     virtual int Init();
     

protected:
     /** Builds the run info from raw data.
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 1 if the data was read from the buffer and the 
	 data processing was successful. Returning 1 tells the 
	 base class to rewind the buffer that it can be used by
	 other data handling classes.\\ \\
	 0 if the data was not touched.\\ \\
	 {\tt CRITICAL\_ERROR} on, you guessed it, critical errors. This leads to the abortion
	 of the program. \\ \\
	 {\tt NONCRITICAL\_ERROR} on internal failure. 
	 It is assumed that the data was read (how else
	 could you tell that there was an error) so the buffer will be rewind after
	 that by the base class. 
	 If you did not touch the data and somehow managed to produce an error
	 you have to call the {\tt \Ref{DontRewind}} function. */
     inline virtual int __AnalyseData(RecordHeader_t RecordHeader, 
				      BinaryDataBuffer &Buf);

};     

int Daq98RunInfo_t::__AnalyseData(RecordHeader_t RecordHeader, 
				  BinaryDataBuffer &Buf)
{
     if(RecordHeader.RecordID!=RID_RUNINFO)
       return 0;
     LogMsg("Run info received",8,8);
     RunInfo_t RunInfo;
     RunInfo.ReadFromBuffer(Buf);
     
     PROGRAM_VERSION=(int)(atof(RunInfo.ProgramVersion)*1000);
     FORMAT_VERSION=(int)(atof(RunInfo.FormatVersion)*1000);
     RunNumber=RunInfo.RunNumber;
     
     if(Parent->IsVerboseLevel(3)){
	  ostrstream Info;
	  if(RunInfo.Magic)
	       Info << "Magic          : " <<  RunInfo.Magic;
	  Info << "\nProgram Version: " << RunInfo.ProgramVersion
	       << "\nFormat Version : " << RunInfo.FormatVersion
	       << "\nRun Number     : " << RunInfo.RunNumber
	       << "\nDebug Level    : " << RunInfo.DebugLevel
	       << "\nLog Level      : " << RunInfo.LogLevel
	       << "\nRun Stop       : " << RunInfo.RunStop
	       << "\nComment        : " << RunInfo.Comment << endl << ends;
	  Info.freeze(1);
	  LogMsg(Info.str(),3,3);
	  Info.freeze(0);
     }
     return 1;
}

    
#endif // __DAQ98RUNINFO_H__
