/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98runtime.C,v 1.5 1999/02/03 03:42:14 steffen Exp $
*/

#include "daq98runtime.h"
#include "time.h"


Daq98RunTime_t::Daq98RunTime_t(DaqProcessBase *parent):
     Daq98MuonEventAnalysis_t(parent)
{
     rate=0;
}

Daq98RunTime_t::~Daq98RunTime_t()
{
     rate=0;
}

void Daq98RunTime_t::SetRate(Daq98Rate_t *r)
{
     rate=r;
}

int Daq98RunTime_t::__AnalyseData(MuonEventHeader_t &EventHeader, MuonEventData_p Event)
{
     static bool first=true;
     static unsigned long int rate_time_last;
     
     if (first) {
	  first=false;
          UnixStartTime=EventHeader.UnixTime;
	  GPSStartTime0=Event->GPSTime[0];
	  GPSStartTime1=Event->GPSTime[1];
	  rate_time_last=
	  + GPSStartTime0.day * 86400
	  + GPSStartTime0.hour * 3600
	  + GPSStartTime0.min * 60
	  + GPSStartTime0.sec;
	  cerr << "first: " << rate_time_last << endl;
	  
	  if(!rate){
	       (*Base::Error)="int Daq98RunTime_t::__AnalyseData(...)";
	       (*Base::Error)=GENERIC_ERROR;
	       Base::Error->SetParam("Rate object not initialized.");
	       return -1;
	  }
     }
     UnixEndTime=EventHeader.UnixTime;
     GPSEndTime0=Event->GPSTime[0];
     GPSEndTime1=Event->GPSTime[1];
     unsigned long int rate_time_now=
	  + GPSEndTime0.day * 86400
	  + GPSEndTime0.hour * 3600
	  + GPSEndTime0.min * 60
	  + GPSEndTime0.sec;
     time_t time_diff;
     
     if((time_diff=rate_time_now - rate_time_last) >= 20){
	  rate->Calc(time_diff,EventHeader.EventNumber);
	  rate_time_last=rate_time_now;
     }
     
     if(Parent->IsVerboseLevel(12)){
	  struct tm T;
	  T=*gmtime(&UnixEndTime);
	  ostrstream Out;
	  Out << "current time: " << ctime(&UnixEndTime)
	      << "        UNIX: " << T.tm_yday << " "
	      << T.tm_hour << ":"
	      << T.tm_min << ":"
	      << T.tm_sec << endl	     
	      << "        GPS1: " << GPSEndTime0.day << " "
	      << GPSEndTime0.hour << ":"
	      << GPSEndTime0.min << ":"
	      << GPSEndTime0.sec << ":"
	      << GPSEndTime0.nsec << endl
	      << "        GPS2: " << GPSEndTime1.day << " "
	      << GPSEndTime1.hour << ":"
	      << GPSEndTime1.min << ":"
	      << GPSEndTime1.sec << ":"
	      << GPSEndTime1.nsec << endl << ends;
	  Out.freeze(1);
	  LogMsg(Out.str(),12,12);
	  Out.freeze(0);
     }
          
     return 0;
}

int Daq98RunTime_t::AnalyseDataFinal()
{
     struct tm T;
     T=*gmtime(&UnixStartTime);
     ostrstream Out;
     Out << "start time: " << ctime(&UnixStartTime)
	 << "      UNIX: " << T.tm_yday << " "
	 << T.tm_hour << ":"
	 << T.tm_min << ":"
	 << T.tm_sec << endl
	 << "      GPS1: " << GPSStartTime0.day << " "
	 << GPSStartTime0.hour << ":"
	 << GPSStartTime0.min << ":"
	 << GPSStartTime0.sec << ":"
	 << GPSStartTime0.nsec << endl
	 << "      GPS2: " << GPSStartTime1.day << " "
	 << GPSStartTime1.hour << ":"
	 << GPSStartTime1.min << ":"
	 << GPSStartTime1.sec << ":"
	 << GPSStartTime1.nsec << endl;
	  
     T=*gmtime(&UnixEndTime);
     Out << "end time  : " << ctime(&UnixEndTime)
	 << "      Unix: " << T.tm_yday << " "
	 << T.tm_hour << ":"
	 << T.tm_min << ":"
	 << T.tm_sec << endl	     
	 << "      GPS1: " << GPSEndTime0.day << " "
	 << GPSEndTime0.hour << ":"
	 << GPSEndTime0.min << ":"
	 << GPSEndTime0.sec << ":"
	 << GPSEndTime0.nsec << endl
	 << "      GPS2: " << GPSEndTime1.day << " "
	 << GPSEndTime1.hour << ":"
	 << GPSEndTime1.min << ":"
	 << GPSEndTime1.sec << ":"
	 << GPSEndTime1.nsec << endl << ends;
     Out.freeze(1);
     LogMsg(Out.str(),1,1);
     Out.freeze(0);
     return 0;
}
