# -* perl *-
# use strict;

# add processes you want to watch
my @PROCESSES=("daqcontrol","vmecontrol","dispatcher",
	       "comserver","dataprocess","datacollector",
	       "datatransport","moniclient");

####################
#
# Print usage message
#
sub Usage(){
    print <<"EOF";
Usage:
% daq_watch [OPTION]

Options:
  --help     Print this message
  --version  Print version information

Send bug reports to <steffen\@alizarin.physics.wisc.edu>
EOF
    exit(1);
}

####################
#
# Print version info
#
sub Version(){
    no strict;
    print <<"EOF";
daq_watch ($PACKAGE) $VERSION - Watch AMANDA DAQ processes 

Copyright (C) 1998 Steffen Richter <steffen\@alizarin.physics.wisc.edu>
This is free software; the author gives unlimited permission to copy 
and/or distribute it, with or without modifications, as long as this 
notice is preserved.
This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

EOF
    use strict;
    exit(1);
}

####################
#
# Print invalid usage message
#
sub Invalid(){
    print "Invalid usage. For help:\n";
    print "% daq_watch --help\n";
    exit(1);
}


setpriority 0, 0, 10;
if($#ARGV==0){
    do Version() if ($ARGV[0] eq "--version");
    do Usage() if ($ARGV[0] eq "--help");
    do Invalid();
}

require POSIX;
my $Termios= new POSIX::Termios;
$Termios->getattr;
my $ospeed = $Termios->getospeed;
require Term::Cap;
my $Terminal= Tgetent Term::Cap {TERM=>undef, OSPEED =>$ospeed };
$Terminal->Trequire(qw/ce ku kd/);
$Terminal->Tputs('mk',1,STDOUT);
$Terminal->Tputs('cl',1,STDOUT);
my $Pat;
while(1){
    my $HaveOne=0;
    my ($Line,$i);
    open(INFILE,"ps axu |") || die "Can't run 'ps axu':$!\n";
    $Terminal->Tgoto('cm', 0, 0, STDOUT);
    while(<INFILE>){
	$i=0;
	if(/USER/){
	    $Terminal->Tputs('md',1,STDOUT);
	    print STDOUT;
	    $Terminal->Tputs('me',1,STDOUT);
	}
	foreach $Pat(@PROCESSES){
	    if(/$Pat/){
		$i++;
		$Line.=$_;
		$HaveOne=1;
	    }
	}
	if($i<9){
	    for(my $a=$i;$a==9;$a++){
		$Line.="\n";
	    }
	}
    }
    if(! $HaveOne){
	print STDOUT "\nNo DAQ processes found\n\n";
    }else{
	print STDOUT $Line;
    }
    sleep 1;
}	    
