/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daqcontrol.C,v 1.7 1999/02/03 03:42:15 steffen Exp $
*/

// SYSTEM
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
 
#include "base.h"
#include "globaltypes.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "dispatcherlib.h"
#include "daqcontrol.h"

DAQControl_SignalHandler::DAQControl_SignalHandler(DAQControlDaemon *parent)
{
     Parent=parent;
}

int DAQControl_SignalHandler::Handle_SIGCHLD(Process &P){
     DEB_F("int DC_SignalHandler::Handle_SIGCHLD(..)");

     int pid=SignalHandlerClass::Handle_SIGCHLD(P);
     if(pid==DAQControlDaemon::Dispatcher_PID){
	  Logger Entry;
	  Entry() << "Dispatcher was killed, exiting too." << endl;
	  Quit_FLAG=true;
     }
     return pid;
}

int DAQControl_SignalHandler::Handle_SIGUSR1(Process &){
     DEB_F("int DC_SignalHandler::Handle_SIGUSR1(..)");
     return 0;
}

int DAQControl_SignalHandler::Handle_SIGALRM(Process &){
     DEB_F("int DC_SignalHandler::Handle_SIGALRM(..)");
     Parent->Restart_FLAG=true;
     Parent->Selecter.SetTimer(0);
     return 0;
}

DAQControlDaemon::DAQControlDaemon():Daemon(0),SignalHandler(this){
     DEB_F("DAQControlDaemon::DAQControlDaemon()");

     IPC_CMD_Interpreter.AddFunction("start",&IPC_CMD_StartProcess,
				"start name                -- starts the process name");
     IPC_CMD_Interpreter.AddFunction("vme",&IPC_CMD_VmeCmd,
				"vme cmd                   -- send a command to vmecontrol");
     IPC_CMD_Interpreter.AddFunction("runstart",&IPC_CMD_StartRun,
				"runstart                  -- start run");
     IPC_CMD_Interpreter.AddFunction("stoprun",&IPC_CMD_StopRun,
				"stoprun                   -- stop run");
     IPC_CMD_Interpreter.AddFunction("restart",&IPC_CMD_Restart,
				"restart                   -- restart run");
     IPC_CMD_Interpreter.AddFunction("SetAutoRestart",&IPC_CMD_SetAutoRestartRun,
				"SetAutoRestart runtime    -- set auto-restart mode");
     IPC_CMD_Interpreter.AddFunction("SetAutoStop",&IPC_CMD_SetAutoStopRun,
				"SetAutoStop runtime       -- set auto-stop mode");
     IPC_CMD_Interpreter.AddFunction("ResetAutoStop",&IPC_CMD_ResetAutoStopRun,
				"ResetAutoStop             -- reset auto-stop and auto-restart mode");
     IPC_CMD_Interpreter.AddFunction("help",&IPC_CMD_Help,
				"help [COMMAND]            -- prints help for COMMAND or for all commands");

     IPC_CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     IPC_CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);

     // set the signal handler
     ThisProcess->GetSignalHandler().SignalAction(SIGALRM,(sighandler_t)SIG_MY);
     ThisProcess->SetSignalHandler(SignalHandler);

     TimeOut=60*1000;
     DesiredRunTime=60*60*24;
     AutoRestartMode=false;
     AutoStopMode=false;
     IsDispatcherRunning=false;
     AutoStartInitFile="";
     Restart_FLAG=false;
}

DAQControlDaemon::~DAQControlDaemon(){
     DEB_F("DAQControlDaemon::~DAQControlDaemon()");
}

int DAQControlDaemon::Init(){
     DEB_F("DAQControlDaemon::Init()");
     
     /* First of all start the Messagedispatcher and save its PID,
        just in case it exits, it will send an SIGCHLD and we can exit
        this program too, becaus it make no sense to run this program
        without the dispatcher */
     if ((Dispatcher_PID=StartSlaveServer("./dispatcher",NULL))<0){
	  (*Error)=COULD_NOT_START_DISPATCHER;
	  (*Error)="DAQControl::Init";
	  return -1;
     }
     IsDispatcherRunning=true;

     /* then initialize th base class, must do it after we started the
         dispatcher */
     TEST_FAILURE(Daemon::Init());
     
     /* open all sockets where we have to listen on and set the close
      on exec flag, such that the sockets will be closed when a new
      process is excecuted after a fork 
      
      and set the notifier objects for all sockets and add them to the
      selecter*/
     ServerPortMap_type::iterator i=ServerPortMap.begin();
     while(i!=ServerPortMap.end()){
	  TEST_FAILURE((*i)->SSocket.Open((*i)->port,10));
	  TEST_FAILURE((*i)->SSocket.SetCloseExec());
	  (*i)->SSocketNotifier.Set((*i)->SSocket,IONotifier::READ);
	  Selecter.AddNotifier((*i)->SSocketNotifier);
	  i++;
     }
     return 0;
}

/** destruction */
int DAQControlDaemon::Destroy(){
     // no need to destroy the sockets, its done by themself
     DEB_F("DAQControlDaemon::Destroy()");

     Daemon::Destroy();

     /* shut down the dispatcher, and we don't want the signal handler
        for SIGCHLD to get a notice about that, therefor set the PID
        to 0 */
     Dispatcher_PID=0;
     if(IsDispatcherRunning) Dispatcher.Quit();
     return 0;
}

/** the event dispatcher */
int DAQControlDaemon::DispatchEvent(){
     // call the dispatcher of the base class
     TEST_FAILURE(Daemon::DispatchEvent());
     // check out all other sockets 
     
     ServerPortMap_type::iterator spmi=ServerPortMap.begin();
     while(spmi<ServerPortMap.end()){
	  if(Selecter.CheckNotifier((*spmi)->SSocketNotifier))
	       // yes -> start the corresponding server program
	       if(StartSlaveServer((*spmi)->servername,&((*spmi)->SSocket))<0){
		    // failure
		    // just do not care now
	       }
	  spmi++;
     } 
     return 0;
}

/** the IPC event handler */
int DAQControlDaemon::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     return IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

/** returns a String with the configuration file name */
const String DAQControlDaemon::GetConfigFileName()const{
     return  DAQCONTROLDAEMON_CNF_FILENAME;   // from globalmacros.h
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ClassWithTypeInfo_t *DAQControlDaemon::GetConfigSection(){
     return new class DAQControlDaemonConfig:public ConfigSection_t
     {
     public:
	  /** ctor */
	  DAQControlDaemonConfig(DAQControlDaemon::ServerPortMap_type &serverportmap,
				 IPAddress_t &vmecontroladdress)
	       :ConfigSection_t(),
		ServerPortMap(serverportmap),
		VMEControlAddress_cnf(vmecontroladdress){}
	  BeginItems(ConfigSection_t){
	       BeginActions{
		    ItemAction_SET("Server",
			       CurServPortMap=new DAQControlDaemon::ServerPortMapper_type;
			       ServerPortMap.push_back(CurServPortMap);
			 );
	       }EndActions;
	       BeginLinks{
		    ItemLink2("Server",CurServPortMap->servername,CurServPortMap->port);
		    ItemLink2("VMEControlAddress",VMEControlAddress_cnf.Hostname,
			      VMEControlAddress_cnf.Port);
		    ItemLink1("VMEControlPath",VMEControlPath);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(DAQCONTROLDAEMON_SECTION_NAME);
     private:
	  DAQControlDaemon::ServerPortMapper_p CurServPortMap;
	  DAQControlDaemon::ServerPortMap_type &ServerPortMap;
	  IPAddress_t &VMEControlAddress_cnf;
	  String VMEControlPath;
     }(ServerPortMap,VMEControlAddress);

}

ClassWithTypeInfo_t *DAQControlDaemon::GetRunParamConfigSection(){
     return new class RunParamConfig:public ConfigSection_t
     {
     public:
	  /** ctor */
	  RunParamConfig(String &RunParam)
	       :ConfigSection_t(),
		RunParam_cnf(RunParam){}
	  BeginItems(ConfigSection_t){
	       BeginActions{
	       }EndActions;
	       BeginLinks{
		    ItemLink1("RunParam",RunParam_cnf);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(RUN_PARAM_SECTION_NAME);
     private:
	  String &RunParam_cnf;
     }(RunParam);

}

/** returns the type of the process */
const String DAQControlDaemon::GetProcessType()const{
     return PROCESS_TYPE_DAQCONTROL; // whatever
}

int DAQControlDaemon::WaitToDie()
{
     pause(); 
     int LastSig=ThisProcess->GetSignalHandler().LastSignal();
     if (LastSig!=SIGCHLD) return -1;
     return 0;
}

int DAQControlDaemon::StartSlaveServer(const char* Progname,ServerSocket *SS){
     DEB_F("DAQControlDaemon::StartSlaveServer(..)");
     int r;
     // pass a socket to the child process -> SS!=NULL
     filedescriptor_t SocketDesc=0;
     char sockstr[20];
     String Path= String("./") + Progname;
     if(SS){
	  TEST_FAILURE(SocketDesc=SS->Accept());
	  sprintf(sockstr,"%i",SocketDesc);
     }
     TEST_FAILURE(r=ThisProcess->Fork());
     if(r==0){  // child process
	  if (SS) execlp((const char*)Path,(const char*)Path,sockstr,NULL);
	  else execlp((const char*)Path,(const char*)Path,NULL);
	  exit(EXIT_FAILURE);
     } else { // parent
	  if(SS) close(SocketDesc);
	  // wait for process to send the SIGUSR1 signal, to indicate,
	  // that it is running, if not, the signal was SIGCHLD, than
	  // starting failed
	  pause(); 
	  int LastSig=ThisProcess->GetSignalHandler().LastSignal();
	  if (LastSig!=SIGUSR1) return -1;
     }
     return r;
}

int DAQControlDaemon::IPC_CMD_StartProcess(IOSocket &S,String *param,int numparams){
     DEB_F("int DAQControlDaemon::StartProcess(..)");
     if(numparams!=2) return SendUsageInfo(S,param[0]);
     String short_name;
     S.SetCloseExec();
     if((compare(param[1],"DP")==0)||
	(compare(param[1],"dataprocess")==0)){
	  param[1]="dataprocess";
	  short_name="DP";
     } else {
	  TEST_FAILURE(S.WriteText("cmd_error\r\nno such process: '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
	  return 0;
     }
     PortLocker PL(Dispatcher,short_name,USER_DAQ_INTERN);     
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port>=0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nprocess already running: '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
	  return 0;
     }
     if(StartSlaveServer((const char*)param[1],NULL)<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\ncould not start process: '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
     } else{
	  TEST_FAILURE(S.WriteText("cmd_info\r\nprocess '")); 
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("' started\r\n"));
     }
     return 0;
}


int DAQControlDaemon::IPC_CMD_SetAutoStopRun(IOSocket &S,String *param,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_Restart(..)");
     if(numparams!=2) {
	  SendUsageInfo(S,"SetAutoStopRun");
	  return 0;
     }
     DesiredRunTime=StringToTime(param[1]);
     if(SetDuration(DesiredRunTime)<0){
	  S.OutputStream() << "cmd_error\r\nCould not set auto-stop mode: "
			   << Base::Error->GetErrorMessage(true) << "\r\n";
	  return -1;
     }
     AutoStopMode=true;
     S.OutputStream() << "cmd_info\r\n"
		      << "Auto-stop mode set. Runtime is "
		      << param[1] << " (" << DesiredRunTime
		      << " sec).\r\n" << flush;

     return 0;
}

int DAQControlDaemon::IPC_CMD_SetAutoRestartRun(IOSocket &S,String *param,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_SetAutoRestartRun(..)");
     if(numparams!=2){
	  SendUsageInfo(S,"SetAutoRestart");
     }
     String Opt="";
     for(int i=2;i<numparams;i++){
	  Opt+=param[i]+" ";
     }
     AutoRestartMode=true;
     DesiredRunTime=StringToTime(param[1]);
     if(SetDuration(DesiredRunTime)<0){
	  S.OutputStream() << "cmd_error\r\nCould not set auto-restart mode: "
			   << (*Base::Error).GetErrorMessage(true) << "\r\n";
	  return -1;
     }
     S.OutputStream() << "cmd_info\r\n"
		      << "Auto-restart mode set. Runtime is "
		      << param[1] << " (" << DesiredRunTime
		      << " sec).\r\n" << flush;
     AutoRestartMode=true;
     return 0;
}

int DAQControlDaemon::IPC_CMD_ResetAutoStopRun(IOSocket &S,String */*param*/,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_ResetAutoStopRun(..)");
     if(numparams!=1) {
	  SendUsageInfo(S,"ResetAutoStopRun");
	  return 0;
     }
     AutoRestartMode=false;
     AutoStopMode=false;
     S.OutputStream() << "cmd_info\r\n"
		      << "Auto-stop mode reset." << "\r\n" << flush;
     return 0;
}

int DAQControlDaemon::IPC_CMD_Restart(IOSocket &S,String *param,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_Restart(..)");
     S.SetCloseExec();
     TEST_FAILURE(RestartRun(param,numparams));
     return 0;
}

int DAQControlDaemon::IPC_CMD_StopRun(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DAQControlDaemon::IPC_CMD_Restart(..)");
     S.SetCloseExec();
     TEST_FAILURE(StopRun());
     return 0;
}

int DAQControlDaemon::IPC_CMD_StartRun(IOSocket &S,String *param,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_StartRun(..)");
     S.SetCloseExec();
     TEST_FAILURE(AutoStartRun(param,numparams));
     return 0;
}

int DAQControlDaemon::SetDuration(long Duration)
{
     DEB_F("int DAQControlDaemon::SetDuration(long Duration)");
     struct itimerval TimerVal;
     TimerVal.it_interval.tv_sec=0;
     TimerVal.it_interval.tv_usec=0;
     TimerVal.it_value.tv_sec=Duration;
     TimerVal.it_value.tv_usec=0;
     TEST_SYS_FAILURE(setitimer(ITIMER_REAL,&TimerVal,NULL),"setitimer");
     return 0;
}

int DAQControlDaemon::GetDuration(long &Duration)
{
     DEB_F("int DAQControlDaemon::GetDuration(long &Duration)");
     struct itimerval TimerVal;
     TEST_SYS_FAILURE(getitimer(ITIMER_REAL,&TimerVal),"getitimer");
     Duration=TimerVal.it_value.tv_sec;
     return 0;
}


int DAQControlDaemon::IPC_CMD_Help(IOSocket &s,String *param,int numparams){
     DEB_F("int DAQControlDaemon::CmdHelp(..)");
     String Help;
     if(numparams>2) return SendUsageInfo(s,param[0]);
     if(numparams>1)
	  IPC_CMD_Interpreter.GetHelp(param[1],Help);
     else
	  IPC_CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));
     return 0;
}

int DAQControlDaemon::IPC_CMD_VmeCmd(IOSocket &s,String *param,int numparams){
     DEB_F("int DAQControlDaemon::IPC_CMD_VmeCmd(..)");
     if(numparams<2) return SendUsageInfo(s,param[0]);
     ClientSocket CS;
     if(CS.Open(VMEControlAddress.Hostname,VMEControlAddress.Port)>=0){
	  char cmd[MAX_TEXT_SIZE];
	  sprintf(cmd,"%s\r\n",(const char*)join(param+1,numparams-1," "));
	  TEST_FAILURE(CS.WriteText(cmd));
	  while(CS.InputStream().good()){
	       CS.InputStream().getline(cmd,MAX_TEXT_SIZE);
	       s.OutputStream() << cmd << "\n" << flush;
	  }
	  return CS.Close();
     }else{
	  s.OutputStream() << "cmd_error\r\nCould not open connection to VMEControl\r\n" << flush;
     }
     return 0;
}

int DAQControlDaemon::IPC_CMD_UnknownCommand(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DAQControlDaemon::UnknownCommand(..)");
     TEST_FAILURE(S.WriteText("cmd_unknown\r\nDAQControl: unknown command\r\n"));
     return 0;
}

int DAQControlDaemon::IPC_CMD_WrongTypeCommand(IOSocket &/*S*/,String */*param*/,int){
     DEB_F("int DAQControlDaemon::WrongTypeCommand(..)");
     return 0;
}

int DAQControlDaemon::SendUsageInfo(IOSocket &s,String param){
     DEB_F("int DAQControlDaemon::SenUsageInfo(..)");
     String Help;
     IPC_CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     return s.WriteText(Help);
}

int DAQControlDaemon::RestartRun(String *param,int numparams)
{
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nEnd of run reached. Restarting run automatically\r\n");

     if(StopRun()<0){
	  String Msg="cmd_error\r\nCould not reset auto-stop mode: "
	       +String(Base::Error->GetErrorMessage(true))+"\r\n";
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Msg);
	  LogMsg(String("Could not restart run: ")+Base::Error->GetErrorMessage(true),1,1,ERROR);
	  return 0;
     }
     if(SetDuration(DesiredRunTime)<0){
	  String Msg="cmd_error\r\nCould not reset auto-stop mode: " 
	       +String((*Base::Error).GetErrorMessage(true)) + "\r\n";
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Msg);
	  LogMsg(String("Could not restart run: ")+(*Base::Error).GetErrorMessage(true),1,1,ERROR);
	  return 0;
     }
     String Msg="cmd_info\r\nAuto-restart mode set. Runtime is "+TimeToString(DesiredRunTime)
	  + "\r\n";
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Msg);

     if(AutoStartRun(param,numparams)<0){
	  String Msg="cmd_error\r\nCould not reset auto-stop mode: "
	       +String(Base::Error->GetErrorMessage(true))+"\r\n";
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Msg);
	  LogMsg(String("Could not restart run: ")+Base::Error->GetErrorMessage(true),1,1,ERROR);
	  return 0;
     }
     return 0;
}

     
int DAQControlDaemon::StopRun()
{
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nEnd of run reached. Stopping run automatically\r\n");
     // Stop datcollector
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStopping process 'datacollector'\r\n");
     PortLocker PL_0(Dispatcher,PROCESS_TYPE_DATACOLLECTOR,USER_DAQ_INTERN);
     IPAddress_t IPAddr_0=PL_0.GetAddress();
     
     if(IPAddr_0.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Datacollector not running. Can't stop this process. DAQ restart aborted.",1,1,ERROR);
	  return -1;
     }
     ClientSocket CS_0;
     TEST_FAILURE(CS_0.Open(IPAddr_0));
     CS_0.OutputStream() << "AutoStop\r\n" << flush;
     TEST_FAILURE(ServReply(CS_0)); 
     String Reply="";
     char txt[MAX_TEXT_SIZE];
     while(CS_0.InputStream().good()){
 	  CS_0.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_0.Close();
     PL_0.UnLock();

     // Stop Datatransport
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStopping process 'datatransport'\r\n");
     PortLocker PL_1(Dispatcher,PROCESS_TYPE_DATATRANSPORT,USER_DAQ_INTERN);
     IPAddress_t IPAddr_1=PL_1.GetAddress();
     
     if(IPAddr_1.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Datatransport not running. Can't stop this process. DAQ restart aborted.",1,1,ERROR);
     	  return -1;
     }
     ClientSocket CS_1;
     TEST_FAILURE(CS_1.Open(IPAddr_1));
     CS_1.OutputStream() << "stop\r\n" << flush;
     TEST_FAILURE(ServReply(CS_1)); 
     Reply="";
     while(CS_1.InputStream().good()){
 	  CS_1.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_1.Close();
     PL_1.UnLock();

     // Stop Dataprocess
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStopping process 'dataprocess'\r\n");
     PortLocker PL_2(Dispatcher,PROCESS_TYPE_DATAPROCESS,USER_DAQ_INTERN);
     IPAddress_t IPAddr_2=PL_2.GetAddress();
     
     if(IPAddr_2.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Dataprocess not running. Can't stop this process. DAQ restart aborted.",1,1,ERROR);
     	  return -1;
     }
     ClientSocket CS_2;
     TEST_FAILURE(CS_2.Open(IPAddr_2));
     CS_2.OutputStream() << "stop\r\n" << flush;
     TEST_FAILURE(ServReply(CS_2)); 
     Reply="";
     while(CS_2.InputStream().good()){
 	  CS_2.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_2.Close();
     PL_2.UnLock();
     
     // Quit datatransport
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nQuiting process 'datatransport'\r\n");
     PortLocker PL_4(Dispatcher,PROCESS_TYPE_DATATRANSPORT,USER_DAQ_INTERN);
     IPAddress_t IPAddr_4=PL_4.GetAddress();
     if(IPAddr_4.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Datatransport not running. Can't kill this process. DAQ restart aborted.",1,1,ERROR);
     
	  return -1;
     }
     ClientSocket CS_4;
     TEST_FAILURE(CS_4.Open(IPAddr_4));
     CS_4.OutputStream() << "bye\r\n" << flush;
     TEST_FAILURE(ServReply(CS_4)); 
     Reply="";
     while(CS_4.InputStream().good()){
 	  CS_4.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_4.Close();
     PL_4.UnLock();

     // Quit dataprocess 
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nQuiting process 'dataprocess'\r\n");
     PortLocker PL_5(Dispatcher,PROCESS_TYPE_DATAPROCESS,USER_DAQ_INTERN);
     IPAddress_t IPAddr_5=PL_5.GetAddress();
     if(IPAddr_5.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Dataprocess not running. Can't kill this process. DAQ restart aborted.",1,1,ERROR);
     
	  return -1;
     }
     ClientSocket CS_5;
     TEST_FAILURE(CS_5.Open(IPAddr_5));
     CS_5.OutputStream() << "bye\r\n" << flush;
     TEST_FAILURE(ServReply(CS_5)); 
     Reply="";
     while(CS_5.InputStream().good()){
 	  CS_5.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     WaitToDie();
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_5.Close();
     PL_5.UnLock();
     
     // Quiting datacollector
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nQuiting process 'datacollector'\r\n");
     PortLocker PL_6(Dispatcher,PROCESS_TYPE_DATACOLLECTOR,USER_DAQ_INTERN);
     IPAddress_t IPAddr_6=PL_6.GetAddress();
     if(IPAddr_6.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::StopRun()";
	  Base::Error->SetParam("Datatransport not running. Can't kill this process. DAQ restart aborted.",1,1,ERROR);
     
	  return -1;
     }
     ClientSocket CS_6;
     TEST_FAILURE(CS_6.Open(IPAddr_6));
     CS_6.OutputStream() << "bye\r\n" << flush;
     TEST_FAILURE(ServReply(CS_6)); 
     Reply="";
     while(CS_6.InputStream().good()){
 	  CS_6.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_6.Close();
     PL_6.UnLock();
     return 0;
}


int DAQControlDaemon::AutoStartRun(String */*param*/,int /*numparams*/)
{
     DEB_F("int DAQControlDaemon::AutoStartRun()");
     
     // Start dataprocess
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStart process 'dataprocess'\r\n");
     int r=StartSlaveServer("dataprocess",NULL);
     if(r<0){
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			       "cmd_error\r\nCould not start process 'dataprocess'1\r\n");
     }else{
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			       "cmd_info\r\nProcess 'dataprocess' started\r\n");
     }

     // Start datacollector
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStart process 'datacollector'\r\n");
     ClientSocket CS_1;
     CS_1.SetCloseExec();
     TEST_FAILURE(CS_1.Open(VMEControlAddress.Hostname,VMEControlAddress.Port));
     char txt[MAX_TEXT_SIZE];
     String cmd="start datacollector\r\n";
     TEST_FAILURE(CS_1.WriteText(cmd));
     TEST_FAILURE(ServReply(CS_1));
     String Reply="";
     while(CS_1.InputStream().good()){
	  CS_1.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     TEST_FAILURE(CS_1.Close());
  // start datatransport
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStart process 'datatransport'\r\n");
     ClientSocket CS_2;
     CS_2.SetCloseExec();
     TEST_FAILURE(CS_2.Open(VMEControlAddress.Hostname,VMEControlAddress.Port));
     cmd="start datatransport\r\n";
     TEST_FAILURE(CS_2.WriteText(cmd));
     TEST_FAILURE(ServReply(CS_2));
     Reply="";
     while(CS_2.InputStream().good()){
	  CS_2.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     TEST_FAILURE(CS_2.Close());

     // Get init options
     ConfigReader cfr;
     ifstream infile(RUN_PARAM_FILENAME);
     cfr.AddSection(GetRunParamConfigSection());
     TEST_FAILURE(cfr.Read(infile));
     
     // Init Dataprocess
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nInitalizing DAQ\r\n");
     PortLocker PL_3(Dispatcher,PROCESS_TYPE_DATAPROCESS,USER_DAQ_INTERN);
     IPAddress_t IPAddr_3=PL_3.GetAddress();
     if(IPAddr_3.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::AutoStart()";
	  Base::Error->SetParam("Could not get IP address of process 'datatransport'. DAQ restart aborted.");
	  return -1;
     }
     ClientSocket CS_3;
     TEST_FAILURE(CS_3.Open(IPAddr_3));
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nDP initialize "+RunParam+"\r\n");

     CS_3.OutputStream() << "initialize " << RunParam << "\r\n" << flush;
     TEST_FAILURE(ServReply(CS_3));
     Reply="";
     while(CS_3.InputStream().good()){
	  CS_3.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_3.Close();
     PL_3.UnLock();

     // Start dataprocess
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStarting dataprocess\r\n");
     PortLocker PL_4(Dispatcher,PROCESS_TYPE_DATAPROCESS,USER_DAQ_INTERN);
     IPAddress_t IPAddr_4=PL_4.GetAddress();
     if(IPAddr_4.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::AutoStart()";
	  Base::Error->SetParam("Could not get IP address of process 'dataprocess'. DAQ restart aborted.");
	  return -1;
     }
     ClientSocket CS_4;
     TEST_FAILURE(CS_4.Open(IPAddr_4));
     CS_4.OutputStream() << "start\r\n" << flush;
     TEST_FAILURE(ServReply(CS_4));
     Reply="";
     while(CS_4.InputStream().good()){
	  CS_4.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_4.Close();
     PL_4.UnLock();

     // Start datatransport
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStarting datatransport\r\n");
     PortLocker PL_5(Dispatcher,PROCESS_TYPE_DATATRANSPORT,USER_DAQ_INTERN);
     IPAddress_t IPAddr_5=PL_5.GetAddress();
     if(IPAddr_5.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::AutoStart()";
	  Base::Error->SetParam("Could not get IP address of process 'datatransport'. DAQ restart aborted.");
	  return -1;
     }
     ClientSocket CS_5;
     TEST_FAILURE(CS_5.Open(IPAddr_5));
     CS_5.OutputStream() << "start\r\n" << flush;
     TEST_FAILURE(ServReply(CS_5));
     Reply="";
     while(CS_5.InputStream().good()){
	  CS_5.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_5.Close();
     PL_5.UnLock();

     // Start datacollector
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			  "cmd_info\r\nStarting datacollector\r\n");
     PortLocker PL_6(Dispatcher,PROCESS_TYPE_DATACOLLECTOR,USER_DAQ_INTERN);
     IPAddress_t IPAddr_6=PL_6.GetAddress();
     if(IPAddr_6.Port<0){
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DAQControlDaemon::AutoStart()";
	  Base::Error->SetParam("Could not get IP address of process 'dataprocess'2. DAQ restart aborted.");
	  return -1;
     }
     ClientSocket CS_6;
     TEST_FAILURE(CS_6.Open(IPAddr_6));
     CS_6.OutputStream() << "start\r\n" << flush;
     TEST_FAILURE(ServReply(CS_6));
     Reply="";
     while(CS_6.InputStream().good()){
	  CS_6.InputStream().getline(txt,MAX_TEXT_SIZE);
	  Reply+=txt+String("\n");
     }
     Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Reply);
     CS_6.Close();
     PL_6.UnLock();

     return 0;
}

int DAQControlDaemon::ServReply(IOSocket &S){
     DEB_F("int DAQControlDaemon::ServReply()");
     int r;
     if((r=S.IsReadReady(TimeOut))<=0){
	  if(r==0){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DAQControlDaemon::ServReply(IOSocket &S)";
	       char str[200];
	       sprintf(str,"Connection timed out (%d sec).",TimeOut/1000);
	       Base::Error->SetParam(str);
	  }
     	  return -1;
     }
          
     return 0;
} 

 
int DAQControlDaemon::EventLoop(){
     DEB_F("DAQControlDaemon::DoLoop()");
     int ret;
     // do the loop until someone set th Quit_flag, might be a signal
     // handler, a subroutine or some one alse
     while(!Quit_FLAG){
	  // check all sockets added to the Selecter class for a
	  // connection request
	  TEST_TEMP_FAILURE(ret=Selecter.DoSelect());
	  if(ret<0){
	       // select interrupted by a signal
	       continue;
	  }
	  // if there was no error dispatch the event
	  if(ret>0) TEST_FAILURE(DispatchEvent());
	  if(0==ret) TEST_FAILURE(DoTIMEREvent());
	  if(Restart_FLAG){
	       if(AutoRestartMode){
		    TEST_FAILURE(RestartRun(NULL,0));
	       }else if(AutoStopMode){
		    TEST_FAILURE(StopRun());
	       }
	       Selecter.SetTimer(-1);
	       Restart_FLAG=false;
	  }
     }
     return 0;
}


int DAQControlDaemon::Dispatcher_PID;

int main(int /*argc*/,char **/*argv*/){
     DEB_F("main()");
     DaqProcessBase::InitializeGlobals("daqcontrol");

     LogFile LF(DaqProcessBase::ThisProcess->GetName(),LogFile::APPEND);
     Logger::SetLogger(&LF);

     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }
     DAQControlDaemon DaemonServer;
//     DaemonServer->Fork2Background();
     if(DaemonServer.Init()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     } else
     if(DaemonServer.EventLoop()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     }
     DaemonServer.Destroy();
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0;
}
