/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daqcontrol.h,v 1.3 1998/11/23 14:41:48 haase Exp $
*/

#if ! defined __DAQCONTROL_H__
#define __DAQCONTROL_H__

#include <config.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <vector.h>
#include <String.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "dispatcherlib.h"
#include "daemon.h"

class DAQControlDaemon;

class DAQControl_SignalHandler:public SignalHandlerClass
{
public:
     DAQControl_SignalHandler(DAQControlDaemon *parent);
protected:
     virtual int Handle_SIGCHLD(Process &P);
     virtual int Handle_SIGUSR1(Process &P);
     virtual int Handle_SIGALRM(Process &P);
protected:
     DAQControlDaemon *Parent;
};


class DAQControlDaemon:public Daemon
{
public:
     DAQControlDaemon();
     ~DAQControlDaemon();

     int Init();
     int Destroy();
     /** returns the type of the process, must be overwritten in
         derived class */
     virtual const String GetProcessType()const;
     int EventLoop();
     
protected:
     int ServReply(IOSocket &S);
     /** the event dispatcher */
     virtual int DispatchEvent();
     /** the IPC event handler */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** returns a String with the configuration file name */
     virtual const String GetConfigFileName()const;
     /** returns a pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ClassWithTypeInfo_t *GetConfigSection();
     virtual ClassWithTypeInfo_t *GetRunParamConfigSection();
     /** Stopss a run */
     int StopRun();
     int WaitToDie();
     
private:
     int StartSlaveServer(const char* Progname,ServerSocket *SS);
private:
     struct ServerPortMapper_type
     {
	  port_t port;
	  String servername;
	  ServerSocket SSocket;
	  SocketNotifier SSocketNotifier;
     };

     typedef PointerWrapper_Ref<ServerPortMapper_type> ServerPortMapper_p;
     typedef vector<ServerPortMapper_p> ServerPortMap_type;
     // for accessing the typedefs
     friend class DAQControlDaemonConfig;
     friend class RunParamConfig;
     
     ServerPortMap_type ServerPortMap;

     bool IsDispatcherRunning;
     static int Dispatcher_PID;
     
     Interpreter<DAQControlDaemon> IPC_CMD_Interpreter;
     int IPC_CMD_StartProcess(IOSocket &S,String *param,int numparams);
     int IPC_CMD_StartRun(IOSocket &S,String *param,int numparams);
     int IPC_CMD_StopRun(IOSocket &S,String *param,int numparams);
     int IPC_CMD_Restart(IOSocket &S,String *param,int numparams);
     int IPC_CMD_SetAutoRestartRun(IOSocket &S,String *param,int numparams);
     int IPC_CMD_SetAutoStopRun(IOSocket &S,String *param,int numparams);
     int IPC_CMD_ResetAutoStopRun(IOSocket &S,String *param,int numparams);
     int IPC_CMD_Help(IOSocket &s,String *param,int numparams);
     int IPC_CMD_UnknownCommand(IOSocket &S,String *param,int numparams);
     int IPC_CMD_WrongTypeCommand(IOSocket &S,String *param,int numparams);
     int IPC_CMD_VmeCmd(IOSocket &S,String *param,int numparams);

     /** Sets duration of a run.
	 @param Duration Duration of the run in seconds
	 @return 0 on success, -1 on failure */
     int SetDuration(long Duration);

     /** Gets the remaining time of a run.
	 @param Duration Remaining time of the run in seconds upon return 
	 @return 0 on success, -1 on failure */
     int GetDuration(long &Duration);
     int AutoStartRun(String *param,int numparams);
     int RestartRun(String *param,int numparams);
     int TimeOut;
     bool Restart_FLAG;
     int SendUsageInfo(IOSocket &s,String param);
     String RunParam;
     time_t DesiredRunTime;
     bool AutoRestartMode;
     bool AutoStopMode;
     
     DAQControl_SignalHandler SignalHandler;
     friend DAQControl_SignalHandler;
     IPAddress_t VMEControlAddress;

     /** File for auto start initialization */
     String AutoStartInitFile;
};

#endif // __DAQCONTROL_H__








