/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datacollector.C,v 1.20 1999/02/10 21:35:14 daq Exp $
*/

#include <strstream.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <String.h>
#include <localdir.h>
#include "simplevector.h"
#include "stringvector.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "globalfunctions.h"
#include "base.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "crateclasses.h"
#include "datacollector.h"
#include "datafilecode.h"
#include "sched.h"
#include "runfilename.h"

// ????????????
#undef  DEB_EVTHEAD_MODULES
#define DEB_EVTHEAD_MODULES   //   test the debugging with event header in the modules

int DataCollector_SignalHandler::Handle_SIGTERM(Process &/*P*/)
{
     Quit_FLAG=true;
     return 0;
}

DataCollector::DataCollector()
     :Daemon(0),hal(this),Stop_FLAG(false){
     DEB_F("DataCollector::DataCollector()");
     // set the signal handler
     ThisProcess->SetSignalHandler(SignalHandler);
     CMD_Interpreter.AddFunction("Help",       &IPC_CMD_Help,
			 "help [COMMAND]     -- prints help for COMMAND or for all commands");
     CMD_Interpreter.AddFunction("GetStatus",  &IPC_CMD_GetStatus,
				 "GetStatus  -- Get status information");
     CMD_Interpreter.AddFunction("Start",      &IPC_CMD_Start,
				 "Start              -- Start DAQ");
     CMD_Interpreter.AddFunction("Stop",       &IPC_CMD_Stop,
				 "Stop               -- Stop DAQ");
     CMD_Interpreter.AddFunction("AutoStop",       &IPC_CMD_AutoStop);
     
     CMD_Interpreter.AddFunction("Bye",        &IPC_CMD_Quit,
				 "Bye                -- Quits the process");
     CMD_Interpreter.AddFunction("Initialize", &IPC_CMD_Initialize,
				 "Initialize         -- initialize datacollector");
     CMD_Interpreter.AddFunction("Init", &IPC_CMD_Initialize,
				 "Init               -- see Initialize");
     CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);     
     IsConfigured=false;
     WriteBuffer1=new ShmBinaryDataBuffer;
     WriteBuffer2=new ShmBinaryDataBuffer;
     
     // some defaults
     AutoRestartMode=false;
     RunInfo.SetComment("Standard run");
     DataBufferSize=500000L;
     CatchSize=1024*1024*1;
     AsynchReadout=5;
     NoOfBytesWritten=0;
     RunNumber=0;
}

/** dtor */
DataCollector::~DataCollector(){
     DEB_F("DataCollector::~DataCollector()");
     delete WriteBuffer1;
     delete WriteBuffer2;
}

/** initialization */
int DataCollector::Init(){
     DEB_F("DataCollector::Init()");
     NumMuonTrigger=0;
     TEST_FAILURE(Daemon::Init());
     key_t  shmkeys[3];
     int i;
     // create shared mem and semaphore keys
     for(i=0;i<3;i++) {
	  TEST_SYS_FAILURE(shmkeys[i]=ftok("./datacollector",(char)i+1),"ftok");
     }
     key_t semkey;
     TEST_SYS_FAILURE(semkey=ftok("./datacollector",(char)1),"ftok");
     TEST_FAILURE(WriteBuffer1->CreateBuffer(MAX_RAW_DATA_BUFFER_SIZE,shmkeys,semkey,true));
     for(i=3;i<6;i++) {
	  TEST_SYS_FAILURE(shmkeys[i-3]=ftok("./datacollector",(char)i+1),"ftok");
     }
     TEST_SYS_FAILURE(semkey=ftok("./datacollector",(char)2),"ftok");
     TEST_FAILURE(WriteBuffer2->CreateBuffer(MAX_RAW_DATA_BUFFER_SIZE,shmkeys,semkey,true));

     key_t shmkey;
     TEST_SYS_FAILURE(shmkey=ftok("./datacollector",(char)7),"ftok");
     TEST_FAILURE(StopFlag.Init(shmkey,sizeof(bool),true));
     
     CurWriteBuffer=WriteBuffer1;

     return 0;
}

/** destruction */
int DataCollector::Destroy(){
     DEB_F("DataCollector::Destroy()");
     WriteBuffer1->Release();
     WriteBuffer2->Release();;
     TEST_FAILURE(Dispatcher.LogClose(LOG_TYPE_RUNLOG));
     TEST_FAILURE(Dispatcher.LogClose(LOG_TYPE_ERRORLOG));
     return Daemon::Destroy();
}

/** the event dispatcher */
int DataCollector::DispatchEvent(){
     // call the dispatcher of the base class
     return Daemon::DispatchEvent();
}

int DataCollector::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     int r=Daemon::DoIPCEvent(Cmd,s);
     if(WasInternalCommand()) return r;
     
     return CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

int DataCollector::DoTIMEREvent(){
     static Catch_t CatchRecord;
     static StringVector_t ErrorVec;
     static int first=true;
     int ret=0;

     if(first){
	  cerr << "**1";
	  first=false;
	  ErrorVec.Clear();
	  if((ret=hal.StartModules_Cratewise(hal.GetCrateVector(),
					     HAL_t::ASYNCH_MODULES,
					     ErrorVec))<0){
	       ostrstream err_s;
	       err_s << "Error while reseting modules:\n";
	       StringVector_t::iterator ei=ErrorVec.begin();
	       while (ei<ErrorVec.end()){
		    err_s << *ei; 
		    ei++;
	       }
	       err_s.freeze(1);
	       LogMsg(err_s.str(),1,1);
	       err_s.freeze(0);
	  }
     }
     
     // you should only play with the prio if you run into
     // big trouble

     //errno=0;
     //int prio=getpriority(PRIO_PROCESS,getpid());
     
     if(hal.CheckForTrigger()){
	  //setpriority(PRIO_PROCESS,getpid(),prio + 10);
	  hal.InhibitTrigger();
	  if((ret=BuildAndSendEvent())>=0){
	       ErrorVec.Clear();
	       if((ret=hal.ResetModules_Cratewise(hal.GetCrateVector(),
						  HAL_t::EVENT_MODULES,ErrorVec))<0){
		    ostrstream err_s;
		    err_s << "Error while reseting modules:\n";
		    StringVector_t::iterator ei=ErrorVec.begin();
		    while (ei<ErrorVec.end()){
			 err_s << *ei; 
			 ei++;
		    }
		    err_s.freeze(1);
		    LogMsg(err_s.str(),1,1);
		    err_s.freeze(0);
	       }
	  }
	  hal.ReleaseTrigger();
	  NumMuonTrigger++;
	  //setpriority(PRIO_PROCESS,getpid(),prio);
     }
     if((ret=BuildAndSendAsynchEvent())>=0) {
	  ErrorVec.Clear();
	  if((ret=hal.ResetModules_Cratewise(hal.GetCrateVector(),
					     HAL_t::ASYNCH_MODULES,
					     ErrorVec))<0){
	       ostrstream err_s;
	       err_s << "Error while reseting modules:\n";
	       StringVector_t::iterator ei=ErrorVec.begin();
	       while (ei<ErrorVec.end()){
		    err_s << *ei; 
		    ei++;
	       }
	       err_s.freeze(1);
	       LogMsg(err_s.str(),1,1);
	       err_s.freeze(0);
	  }
     }
	  
     if(NoOfBytesWritten>CatchSize){
	  NoOfBytesWritten=0;
	  WriteRecord(CatchRecord,*(BinaryDataBuffer *)CurWriteBuffer);
     }

     sched_yield();
     return ret;
}

/** returns a String with the configuration file name */
const String DataCollector::GetConfigFileName()const{
     return  DATACOLLECTOR_CNF_FILENAME; // from globalmacros.h
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ClassWithTypeInfo_t *DataCollector::GetConfigSection(){
     return new class DataCollectorConfig:public ConfigSection_t
     {
     public:
	  /** ctor */
	  DataCollectorConfig(u_i64 &DataBufferSize,int &VerboseLevelFile,
			      int &VerboseLevelConsole,
			      unsigned int &CatchSize,
			      unsigned int &AsynchReadout)
	       :ConfigSection_t(),
		DataBufferSize_cnf((u_i32&)DataBufferSize),
		VerboseLevelFile_cnf(VerboseLevelFile),
		VerboseLevelConsole_cnf(VerboseLevelConsole),
		CatchSize_cnf(CatchSize),
		AsynchReadout_cnf(AsynchReadout){}
	  BeginItems(ConfigSection_t){
// 	       BeginActions{
// 		    // no actions yet
// 	       }EndActions;
	       BeginLinks{
		    ItemLink1("DataBufferSize",DataBufferSize_cnf);
		    ItemLink1("VerboseLevelFile",VerboseLevelFile_cnf);
		    ItemLink1("VerboseLevelConsole",VerboseLevelConsole_cnf);
		    ItemLink1("CatchSize",CatchSize_cnf);
		    ItemLink1("AsynchReadout",AsynchReadout_cnf);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(DATACOLLECTOR_SECTION_NAME);
     private:
	  u_i32 &DataBufferSize_cnf;
	  int &VerboseLevelFile_cnf;
	  int &VerboseLevelConsole_cnf;
	  unsigned int &CatchSize_cnf;
	  unsigned int &AsynchReadout_cnf;
     }(DataBufferSize,VerboseLevelFile,VerboseLevelConsole,CatchSize,
       AsynchReadout);
}

/** returns the type of the process, must be overwritten in
    derived class */
const String DataCollector::GetProcessType()const{
     return PROCESS_TYPE_DATACOLLECTOR;
}



int DataCollector::SetUpHAL(StringVector_t &errors){
     ConfigReader cr;
    // clear the config vectors
     CrateConfig.Clear();
     ModuleConfig.Clear();
     ChannelConfig.Clear();     
     
     cr.AddSection(new CrateSection_t(CrateConfig));
     cr.AddSection(new ModuleSection_t(ModuleConfig));
     cr.AddSection(new ChannelSection(ChannelConfig));

     String cnf_file=HW_CONFIG_FILE_NAME;
     String cnf_files[4];
     cnf_files[0]="./" + cnf_file;
     char *path=getenv("HOME");
     if(path){
	  cnf_files[1]=path + String("/.") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[1]="";
     } 
     path=getenv("DAQ_ROOT_DIR");
     if(path){
	  cnf_files[2]=path + String("/share/") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[2]="";
     } 
     cnf_files[3]= String(DAQ_ROOT_DIR) + String("/share/") + 
	  String(PACKAGE) + "/" + cnf_file;
     ifstream infile;
     for(int i=0;i<4;i++){
	  infile.open(cnf_files[i]);
	  if(infile) {
//	       Logger Entry;
	       String msg="Using configuration file '" +
		    cnf_files[i] + "'.";
//	       Entry() << msg <<endl;
	       LogMsg(msg,3,3,INFO);
	       i=3;
	  }
     }

     if(!infile) {
	  String str="Can't open detector configuration '" + String(HW_CONFIG_FILE_NAME)+"'.";
	  (*Base::Error)=SYS_ERROR;
	  Base::Error->SetParam(str);	  
	  (*Base::Error)="DataCollector::SetUpHAL(..)";	

	  ostrstream err;
	  (*Error).Print(err);
	  err << "\r\n" << ends;
	  err.freeze(1);	  
	  errors.push_back(String(err.str()));
	  err.freeze(0);
	  return -1;
     }
     ostrstream msg;
     msg << "Detector Configuration:\n";
     
     char line[MAX_TEXT_SIZE];
     while(!infile.eof()){
	  infile.getline(line,MAX_TEXT_SIZE);
	  msg << line << "\n";
     }
     msg << ends;
     
     msg.freeze(1);
     
     LogMsg((const char *)msg.str(),1,6);
     msg.freeze(0);
     infile.seekg(0,ios::beg);
     infile.clear();
     if(cr.Read(infile)<0){
	  ostrstream err;
	  (*Error).Print(err);
	  err << "\r\n" << ends;
	  err.freeze(1);	  
	  errors.push_back(String(err.str()));
	  err.freeze(0);
	  return -1;
     }
     
     int err=0;
     hal.ClearStructure();
     if(err==0) err+=hal.InsertCrates     (CrateConfig,errors);  
     if(err==0) err+=hal.InsertModules    (ModuleConfig,errors);
     if(err==0) err+=hal.InsertChannels   (ChannelConfig,errors);
     if(0==err){
	  IsConfigured=true;
     }

     //PrintOutConfig(hal);
     return err;
}


int DataCollector::WriteRunInfoToBuffer(BinaryDataBuffer &Buf)
{
     WriteRecord(RunInfo,Buf);
     
     return 0;
}

// writes the detector configuration to our data socket in binary format
int DataCollector::WriteDetectorConfigToBufferAscii(BinaryDataBuffer &Buf){
     String cnf_file=HW_CONFIG_FILE_NAME;
     String cnf_files[4];
     cnf_files[0]="./" + cnf_file;
     char *path=getenv("HOME");
     if(path){
	  cnf_files[1]=path + String("/.") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[1]="";
     } 
     path=getenv("DAQ_ROOT_DIR");
     if(path){
	  cnf_files[2]=path + String("/share/") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[2]="";
     } 
     cnf_files[3]= String(DAQ_ROOT_DIR) + String("/share/") + 
	  String(PACKAGE) + "/" + cnf_file;
     ifstream infile;
     for(int i=0;i<4;i++){
	  infile.open(cnf_files[i]);
	  if(infile) {
	       i=3;
	  }
     }
     if(!infile) {
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)="DataCollector::WriteDetectorConfigToBufferAscii(..)";	

	  ostrstream err;
	  (*Error).Print(err);
	  err << "\r\n" << ends;
	  err.freeze(1);	  
	  LogMsg((const char *)err.str(),2,2,ERROR);
	  err.freeze(0);
	  return -1;
     }
     HWConfigAscii_t HWConfig;
     HWConfig.cfg_instream=&infile;
     
     WriteRecord(HWConfig,Buf);
     return 0;
}


int DataCollector::IPC_CMD_Quit(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int DataCollector::CMD_Quit(..)");
     Quit_FLAG=true;
     TEST_FAILURE(s.WriteText("cmd_info\r\nBye Bye\r\n"));
     return 0;
}

int DataCollector::IPC_CMD_Help(IOSocket &s,String *params,int numparams){
     DEB_F("int DataCollector::CMD_Help(..)");
     String Help;
     if(numparams>1)
	  CMD_Interpreter.GetHelp(params[1],Help);
     else
	  CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));

     return 0;
}

int DataCollector::IPC_CMD_GetStatus(IOSocket &/*S*/,String */*param*/,int /*numparams*/){
     DEB_F("int Data::IPC_CMD_GetStatus(..)");
     return 0;
}

int DataCollector::IPC_CMD_Start(IOSocket &s,String */*params*/,int /*numparams*/){
     if(IsConfigured){
	  TEST_FAILURE(s.WriteText("cmd_info\r\nProcess 'datacollector' started\r\n"));
	  RunInfo.RunNumber=RunNumber;
	  TEST_FAILURE(Dispatcher.IncRunNumber());
	  WriteRunInfoToBuffer(*CurWriteBuffer);
	  WriteDetectorConfigToBufferAscii(*CurWriteBuffer);
	  Selecter.SetTimer(0);
	  (*(bool*)StopFlag())=false;
	  if(AutoRestartMode && !NoDaqControlInit){
	       if(SetAutoRestartRun(s,DesiredRunTimeStr)<0){
		    return -1;
	       }
	  }
	  else {
	       if(!compare(DesiredRunTimeStr,"")){
		    s.OutputStream() << "cmd_info\r\n"
				     << "No run time was given. "
				     << "Setting run time to 24h. Hope that's OK."
				     << "\r\n" << flush;
		    DesiredRunTimeStr="24h";
	       }
	       
	       if(!NoDaqControlInit && SetAutoStopRun(s,DesiredRunTimeStr)<0){
		    return -1;
	       }
	  }
	  TEST_FAILURE(Dispatcher.RunStatStart(RunName));
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,
			       "cmd_message\r\n===> New run started: "+RunName+" <===\r\n");
     } else {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nStart called, but DataCollector not initialized yet\r\n"));
     }
     
     return 0;
}

int DataCollector::IPC_CMD_Stop(IOSocket &s,String */*params*/,int /*numparams*/){
     (*(bool*)StopFlag())=true;
     ResetAutoStopRun(s);
     if(CurWriteBuffer==WriteBuffer1){
	  CurWriteBuffer=WriteBuffer2;
	  TEST_FAILURE(CurWriteBuffer->Get());
	  CurWriteBuffer->Reset();
	  TEST_FAILURE(WriteBuffer1->Release());
     }
     else {
	  CurWriteBuffer=WriteBuffer1;
	  TEST_FAILURE(CurWriteBuffer->Get());
	  CurWriteBuffer->Reset();
	  TEST_FAILURE(WriteBuffer2->Release());
     }
     TEST_FAILURE(Dispatcher.RunStatStop());
     TEST_FAILURE(s.WriteText("cmd_info\r\nProcess 'datacollector' stopped\r\n"));

     Selecter.SetTimer(-1);
     return 0;
}

int DataCollector::IPC_CMD_AutoStop(IOSocket &s,String */*params*/,int /*numparams*/){
     (*(bool*)StopFlag())=true;
     if(CurWriteBuffer==WriteBuffer1){
	  CurWriteBuffer=WriteBuffer2;
	  TEST_FAILURE(CurWriteBuffer->Get());
	  CurWriteBuffer->Reset();
	  TEST_FAILURE(WriteBuffer1->Release());
     }
     else {
	  CurWriteBuffer=WriteBuffer1;
	  TEST_FAILURE(CurWriteBuffer->Get());
	  CurWriteBuffer->Reset();
	  TEST_FAILURE(WriteBuffer2->Release());
     }
     TEST_FAILURE(Dispatcher.RunStatStop());
     TEST_FAILURE(s.WriteText("cmd_info\r\nProcess 'datacollector' stopped\r\n"));

     Selecter.SetTimer(-1);
     return 0;
}

int DataCollector::IPC_CMD_Initialize(IOSocket &s,String *params,int numparams){
     // now open the log files
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_RUNLOG));
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_ERRORLOG));
     // then read the detector config file
     StringVector_t errors;
     int err=SetUpHAL(errors);
     TEST_FAILURE(s.CheckOutStream());
     if(err<0){
 	  s.OutputStream() << "cmd_error\r\n"
 			   << -err 
 			   << " error(s) found in detector configuration file (VME)\r\n"
			   << "Please correct errors in " << HW_CONFIG_FILE_NAME 
			   << " and reinitialize.\r\n";
	  Logger Entry;
	  
	  StringVector_t::iterator ei=errors.begin();
 	  while (ei<errors.end()){
 	       s.OutputStream() << "cmd_info\r\n" << *ei; 
	       Entry() << *ei << flush;
	       ei++;
 	  }
     } else {
	  // make the hardware init of all enabled modules: Muon & Asynch modules

	  err=hal.InitCrates(hal.GetCrateVector(),errors);
	  err+=hal.InitModules_Cratewise(hal.GetCrateVector(),HAL_t::EVENT_MODULES,errors);
	  err+=hal.InitModules_Cratewise(hal.GetCrateVector(),HAL_t::ASYNCH_MODULES,errors);
	  err+=hal.InitModules_Cratewise(hal.GetCrateVector(),HAL_t::DMAD_MODULES,errors);
	  if(err<0){
	       s.OutputStream() << "cmd_error\r\n"
				<< -err 
				<< " error(s) during module initialization\r\n";
	       StringVector_t::iterator ei=errors.begin();
	       while (ei<errors.end()){
		    s.OutputStream() << "cmd_info\r\n" << *ei; 
		    ei++;
	       }
	  }
	  else {
	       s.OutputStream() << "cmd_info\r\nDataCollector initialized\r\n";
	  }
     }
     ParseArgs(s,params,numparams);
     // continue even in case of errors. The user has to reinitialize
     return 0;
}

int DataCollector::IPC_CMD_UnknownCommand(IOSocket &s,String *params,int numparams){
     DEB_F("int DataCollector::IPCUnknownCommand(..)");
     s.OutputStream() << "cmd_error\r\nDataCollector: Unknown command: "
		      << join(params,numparams," ") << " \r\n" << flush;
     return 0;
}

int DataCollector::IPC_CMD_WrongTypeCommand(IOSocket &/*s*/,String */*params*/,int){
     DEB_F("int DataCollector::IPCIgnoreCommand(..)");
     return 0;
}

int DataCollector::SendCatchRecord()
{
     static Catch_t CatchRecord;
     WriteRecord(CatchRecord,*(BinaryDataBuffer *)CurWriteBuffer);
     TEST_FAILURE(SwitchBuffersIfNecessary());
     return 0;
}

int DataCollector::BuildAndSendAsynchEvent(){
     static int NoOfReadouts=0;
     
     if(0 == (NumMuonTrigger % AsynchReadout)){
	  NoOfReadouts++;
	  SNEventHeader_t EventHeader;
	  EventHeader.NoOfReadouts=NoOfReadouts;
	  CurWriteBuffer->PushPosition();
	  NoOfBytesWritten+=WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
	  CurWriteBuffer->RecordBegin();     
	  hal.ReadoutModules_Cratewise(hal.GetCrateVector(),
				       HAL_t::ASYNCH_MODULES,*CurWriteBuffer);
	  NoOfBytesWritten+=EventHeader.NoOfBytes=CurWriteBuffer->RecordEnd_GetSize();
	  
	  CurWriteBuffer->PopPosition();
	  WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
	  
	  TEST_FAILURE(SwitchBuffersIfNecessary());     
	  return 0;
     }
}

int DataCollector::BuildAndSendDMADEvent(){

     DMAD_EventHeader_t EventHeader;
     CurWriteBuffer->PushPosition();
     NoOfBytesWritten+=WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
     CurWriteBuffer->RecordBegin();     
     hal.ReadoutModules_Cratewise(hal.GetCrateVector(),
				  HAL_t::DMAD_MODULES,*CurWriteBuffer);
     NoOfBytesWritten+=EventHeader.NoOfBytes=CurWriteBuffer->RecordEnd_GetSize();

     CurWriteBuffer->PopPosition();
     WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
     
     TEST_FAILURE(SwitchBuffersIfNecessary());     
     return 0;
}


#ifdef DEB_EVTHEAD_MODULES   // **RW**
int DataCollector::PrintDummy(){
     cout << " _______________________ here is DataCollector::PrintDummy "<<endl;
     return 0;
}
#endif


int DataCollector::BuildAndSendEvent(){
     static int EventNumber=0;
     MuonEventHeader_t EventHeader;
     EventHeader.EventNumber=EventNumber++;
     
     EventHeader.UnixTime=time(NULL);
     
     EventHeader.MuTime=0;
     EventHeader.NHitTDC=0;
     EventHeader.NHitADC=0;
     EventHeader.TriggerReg=0;
     
     CurWriteBuffer->PushPosition();
     NoOfBytesWritten+=WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
     CurWriteBuffer->RecordBegin();     
     hal.ReadoutModules_Cratewise(hal.GetCrateVector(),HAL_t::EVENT_MODULES,*CurWriteBuffer);
     NoOfBytesWritten+=EventHeader.NoOfBytes=CurWriteBuffer->RecordEnd_GetSize();
     //// huah who wrote this               ^


     CurWriteBuffer->PopPosition();
     WriteRecord(EventHeader,*(BinaryDataBuffer *)CurWriteBuffer);
     TEST_FAILURE(SwitchBuffersIfNecessary());
     return 0;
}

int DataCollector::SwitchBuffersIfNecessary(){
     if(CurWriteBuffer->GetSizeOfData()>MAX_USED_RAW_DATA_BUFFER_SIZE){
	  CurWriteBuffer->Release();
	  // check if other write buffer is ready
	  if(CurWriteBuffer==WriteBuffer1){
	       CurWriteBuffer=WriteBuffer2;
	  }
	  else {
	       CurWriteBuffer=WriteBuffer1;
	  }
	  TEST_FAILURE(CurWriteBuffer->Get())
	       CurWriteBuffer->Reset();
     }
     return 0;
}

void DataCollector::ArgsHelp(IOSocket &S)
{
     S.OutputStream() << "cmd_info\r\n"
		      << "Datacollector initialization arguments:\n"
		      << " --help        -?\n\tDisplay this help screen\n\n"
		      << " --no-runnumber\n"
		      << "\tprevents the runnumber from being incrementd\n\n"
		      << " --run-comment\n\t"
		      << "A comment that will be included in the datafile.\n\n"
		      << " --verbose-console\n\t"
		      << " Console verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << " --verbose-file\n\t"
		      << " Log file verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << "\r\n" << flush;
  
}

int DataCollector::ParseArgs(IOSocket &S, String *param, int numparams)
{
     if(numparams<=1) return 0;
     LogMsg("Parsing initilization arguments",2,4);
     TEST_FAILURE(S.WriteText("cmd_info\r\nParsing initilization arguments\r\n"));
     char **argv;
     argv=new char*[numparams + 1];
     int i;
     for(i=0;i<numparams;i++){
	  argv[i]=new char[param[i].length()+1];
	  strcpy(argv[i],(const char*)param[i]);
     }

     argv[i]=NULL;
     String OptArg;
     int c;
     int ret=0;
     int digit_optind=0;
     optind=0;
     char str[512];
     RunName="";
     NoDaqControlInit=false;
          
     while (1)
     {
	  int this_option_optind = optind ? optind : 1;
	  int option_index = 0;
	  static struct option long_options[] =
	  {
	       // when you add options, please maintain an 
	       // alphabetical order
	       {"help", 0, 0, '?'},
	       {"auto-restart",no_argument,0,'A'},
	       {"no-dc-init",no_argument,0,305},
	       {"runname-intern",required_argument,0,304},
	       {"runnumber-intern",required_argument,0,303},
	       {"run-comment",required_argument, 0, 300},
 	       {"run-time",required_argument,0,'t'},
	       {"verbose-console", 1, 0, 301},
               {"verbose-file", 1, 0, 302},
               {0, 0, 0, 0}
	  };

	  c = getopt_long (numparams, argv, "?t:A",
			   long_options, &option_index);
	  if (c == -1)
	       break;
	  
	  switch (c)
	  {
	  case 0:
	       S.OutputStream() << "cmd_info\r\n"
				<< "option " 
				<< long_options[option_index].name;
	       if (optarg)
		    S.OutputStream() << " with arg " 
				     << optarg;
	       S.OutputStream() << "\r\n" << flush;
	       break;
	  case 300:
	       // Run comment
	       //get rid of the "
	       OptArg=(const char *)optarg;
	       OptArg.gsub("\"","");
	       sprintf(str,"Run comment set to: %s",(const char *)OptArg);
	       S.OutputStream() <<"cmd_info\r\n"
				<< str << "\r\n" << flush;
	       LogMsg((const char *)str,2,4);
	       RunInfo.SetComment((const char*)OptArg);
	       
	       break;
	       
	  case 301:
	       // verbose level console
	       SetVerboseLevel(GetFileVerboseLevel(),atoi(optarg));
	       sprintf(str,"Console verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,4);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	  case 302:
	       // Verbose level file
	       SetVerboseLevel(atoi(optarg),GetConsoleVerboseLevel());
	       sprintf(str,"File verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,4);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	       
	  case 303:
	       RunNumber=atol(optarg);;
	       
	       break;
	  case 304:
	       RunName=(const char*)optarg;
	       break;
	  case 305:
	       NoDaqControlInit=true;
	       break;
	  case '0':
	  case '1':
	  case '2':
	       if (digit_optind != 0 && digit_optind != this_option_optind)
		    printf ("digits occur in two different argv-elements.\n");
	       digit_optind = this_option_optind;
	       printf ("option %c\n", c);
	       break;

	  case '?':
	       ArgsHelp(S);
	       break;
	  case 'A':
	       // autorestart
	       AutoRestartMode=true;
	       break;
	       
	  case 't':
	       DesiredRunTimeStr=(const char*)optarg;
	       break;

	  default:
	       sprintf(str,"Getopt returned character code 0%o",c);
	       S.OutputStream() << "cmd_error\r\n" 
				<< str
				<< "\r\n" << flush;
	       LogMsg((const char *)str,1,1,ERROR);
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="DataProcessDaemon::ParseArgs(..)";
	       Base::Error->SetParam(str);

	       ret=-1;
	       goto CLEAN_EXIT;
	  }
     }
     if (optind < numparams)
     {
	  ostrstream err_s;
	  err_s << "non-option ARGV-elements: ";
	  while (optind < numparams)
	       err_s << argv[optind++] << " ";
	  err_s << ends;
	  err_s.freeze(1);	  
	  LogMsg((const char *)err_s.str(),1,1,ERROR);
	  S.OutputStream() << "cmd_error\r\n" << err_s.str() << "\r\n";
	  err_s.freeze(0);
	  ret=-1;
	  goto CLEAN_EXIT;
     }

 CLEAN_EXIT:
     
     for(i=0;i<=numparams;i++){
	  delete [] argv[i];
	  argv[i]=NULL;
     }
     
     delete [] argv;
     return ret;
}

int DataCollector::ServReply(IOSocket &S){
     DEB_F("int DataCollector::ServReply()");
     int r;
     if((r=S.IsReadReady(100000))<=0){
	  if(r==0){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataCollector::ServReply(IOSocket &S)";
	       char str[200];
	       sprintf(str,"Connection timed out (%i sec).",100);
	       Base::Error->SetParam(str);
	  }
     	  return -1;
     }
          
     return 0;
} 

int DataCollector::SetAutoRestartRun(IOSocket &S, String &Opt)
{
     PortLocker PL(Dispatcher,PROCESS_TYPE_DAQCONTROL,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nProcess 'daqcontrol' not running\nCan't connect to this process\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Process 'daqcontrol' not running\nCan't connect to this process");
     	  return -1;
     }
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     CS.OutputStream() << "SetAutoRestart " << Opt << "\r\n" << flush;
     TEST_FAILURE(ServReply(CS)); 
     char txt[MAX_TEXT_SIZE];
     String Reply="";
     while(CS.InputStream().good()){
 	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=String(txt) + "\n";
     }
     TEST_FAILURE(S.WriteText(Reply));
     return 0;
}

int DataCollector::SetAutoStopRun(IOSocket &S, String &Time)
{
     PortLocker PL(Dispatcher,PROCESS_TYPE_DAQCONTROL,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nProcess 'daqcontrol' not running\nCan't connect to this process\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Process 'daqcontrol' not running\nCan't connect to this process");
     	  return -1;
     }
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     CS.OutputStream() << "SetAutoStop " << Time << "\r\n" << flush;
     TEST_FAILURE(ServReply(CS)); 
     char txt[MAX_TEXT_SIZE];
     String Reply="";
     while(CS.InputStream().good()){
 	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=String(txt) + "\n";
     }
     TEST_FAILURE(S.WriteText(Reply));
     return 0;
}

int DataCollector::ResetAutoStopRun(IOSocket &S)
{
     PortLocker PL(Dispatcher,PROCESS_TYPE_DAQCONTROL,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nProcess 'daqcontrol' not running\nCan't connect to this process\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Process 'daqcontrol' not running\nCan't connect to this process");
     	  return -1;
     }
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     CS.OutputStream() << "ResetAutoStop\r\n" << flush;
     TEST_FAILURE(ServReply(CS)); 
     char txt[MAX_TEXT_SIZE];
     String Reply="";
     while(CS.InputStream().good()){
 	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=String(txt) + "\n";
     }
     TEST_FAILURE(S.WriteText(Reply));
     return 0;
}

int main(int /*argc*/,char */*argv[]*/){
     DEB_F("int main(int argc,char *argv[])");
     DaqProcessBase::InitializeGlobals("datacollector");
      // for program error logging
     LogFile LF(DaqProcessBase::ThisProcess->GetName());					
     Logger::SetLogger(&LF);				
     // Lock the process, 
     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }
     // create the daemon object
     DataCollector RServer;
     if(RServer.Init()<0){
	  Logger Entry;
 	  (*Base::Error).Print();
     }else{
	  /* signal parent process, that the process was started
	     successfully */
	  kill(getppid(),SIGUSR1);
	  if(RServer.EventLoop()<0){
	       (*Base::Error).Print();
	       RServer.LogMsg((*Base::Error).GetErrorMessage(true),1,1,ERROR);
	  }
     }
     if(RServer.Destroy()<0){
	  Logger Entry;
 	  (*Base::Error).Print();
     }
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0;
}

 
