/*
$Header: /home/elrond/baikal/src/CVS/daq98/datacollector.h,v 1.8 1999/02/08 16:08:01 steffen Exp $
*/

#if ! defined __VMEDATAPROCESS_H__
#define __VMEDATAPROCESS_H__
#include <config.h>
#include <strstream.h>
#include <String.h>
#if defined (HAVE_GETOPT_H)
#include <getopt.h>
#else
#include <my_getopt.h>
#endif 
#include "simplevector.h"
#include "stringvector.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "daemon.h"
#include "vmedaqhal.h"
#include "memman.h"

#undef  DEB_EVTHEAD_MODULES
#define DEB_EVTHEAD_MODULES   //   test the debugging with event header in the modules

class DataCollector;

class DataCollector_SignalHandler:public SignalHandlerClass
{
     virtual int Handle_SIGTERM(Process &P);
};

class DataCollector:public Daemon
{
public:
     
     /** Constructor */
     DataCollector();

     /** Destructor */
     ~DataCollector();

     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 @return 0 on success, -1 on failure.  */
     int Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 @return 0 on success, -1 on failure.  */
     int Destroy();
     
     /** Gets the proces type.
	 This function  must be overwritten in every derived class
	 @return Type of the process */
     virtual const String GetProcessType()const;

     /** Sets up the data structure for the Hardware, according to the
         configuration in the detetcor config file
	 @param erros Vector of error strings. This data should be evaluated 
	 whenever the return value is less than 0
	 @return 0 on success or negative number of errors on failure. 
	 A negative number corresponds to the number of fields in the error vector given as a paramter.*/
     int SetUpHAL(StringVector_t &errors);

     /** Writes the detector configuration into the data buffer using an ASCII format.
	 @param Buf Pointer to current databuffer
	 @return 0 on success, -1 on failure. */
     int WriteDetectorConfigToBufferAscii(BinaryDataBuffer &Buf);

     /** Writes run info to the data buffer.
	 @param Buf Pointer to current databuffer
	 @return 0 on success, -1 on failure. */
     int WriteRunInfoToBuffer(BinaryDataBuffer &Buf);

     /** Stops a run */
     int StopRun();

     int ServReply(IOSocket &S);
     
#ifdef DEB_EVTHEAD_MODULES   //  for debugging 
     int PrintDummy();
#endif

//      /** */
//      int StartTransportProcess();

     
protected:

     /** Builds and sends a raw data muon event.
	 The modules are read out and the raw data is written to a the current 
	 data buffer. If the buffer is full the function writes to the second data buffer
	 and the other buffer is send to the dataprocess by the datatransport process.
	 @return 0 on success, -1 on failure. */
     int BuildAndSendEvent(); 

     /** Builds the asynchronous events, saves it in the big buffer, and when the
         buffer is full, send it over the net to the dataprocess */
     int BuildAndSendAsynchEvent(); 

     /** Builds the DMAD events, saves it in the big buffer, and when the
         buffer is full, send it over the net to the dataprocess */
     int BuildAndSendDMADEvent();
     
     /** Includes a catch record in the data.
	 A catch record allows to synchronize on the data of corrupt data files.
	 @return 0 on success, -1 on failure. */
     int SendCatchRecord();
     
     /** Checks if the current data buffer and switches buffer if the buffer is full.
	 @return 0 on success, -1 on failure. */
     int SwitchBuffersIfNecessary();

     /** Parses initialization arguments.
	 @param S Command socket, used to send any reply commands.
	 @param Param Pointer to vector of parameters.
	 @param NumParams Number of parameters in {\tt \Ref{Param}}. 
	 @return 0 on success, -1 on failure. */
     int ParseArgs(IOSocket &S, String *Param, int NumParams);
     
     /** Prints a help message for all initialization arguments.
       @param S Command socket where the message is written to.*/
     void ArgsHelp(IOSocket &S);

     /** Dispatches incomming events.
	 @return 0 on success, -1 on failure. */
     virtual int DispatchEvent();

     /** This function polls for a trigger.
	 The timer value can be set with {\tt \Ref{SetTimer}}
	 @return 0 on success, -1 on failure. */
     virtual int DoTIMEREvent(); 

     /** The IPC event handler
	 Calls the appropriate command handling function.
	 @return 0 on success, -1 on failure. */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);

     /** Gets the configuration file name.
	 @return Name of configuration file for this class */
     virtual const String GetConfigFileName()const;

     /** Creates a class for handling the configuration of this class.
         The class instances is created with new on the heap.
	 This function must be overwritten in derived class 
	 @return Pointer to the configuration class */
     virtual ClassWithTypeInfo_t *GetConfigSection();

     int SetAutoRestartRun(IOSocket &S, String &Time);
     int SetAutoStopRun(IOSocket &S, String &Time);
     int ResetAutoStopRun(IOSocket &S);
private:
     /** declared it a friend in order to accss the typedefs */
     friend class DataCollectorConfig;

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Quit(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Help(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_GetStatus(IOSocket &s,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Start(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Stop(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_AutoStop(IOSocket &s,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Initialize(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param numParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_UnknownCommand(IOSocket &s,String *Params,int NumParams);
 
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_WrongTypeCommand(IOSocket &s,String *Params,int NumParams);
 
     /** Sends usage info for a certain command.
	 @param S Command socket
	 @params Param Command string for which we want to send the usage info
	 @return 0 on success, -1 on failure. */
     int SendUsageInfo(IOSocket &S,String Param);     

protected:
     /** Run info data structure.
	 This information is written to the data file at the begining
	 of a run. */
     RunInfo_t RunInfo;

     /** No of bytes written after last catch record */
     u_i32 NoOfBytesWritten;

     /** Tells the transport process when to stop */
     SharedMem StopFlag;
     
     /** Flag for run name determination. */
     u_i32 RunNumber;

     /** Run name */
     String RunName;
     
     /** Desired run time.
	 The run will be automatically stopped after this time has elapsed.
	 The time value is given in seconds. */
     long DesiredRunTime;
     
     /** Default runt time. */
     long DefaultRunTime;

     /** Auto-restart mode flag */
     bool AutoRestartMode;

     /** Desired run time */
     String DesiredRunTimeStr;
     
     /** Don't initialize Daqcontrol if true */
     bool NoDaqControlInit;
     
     /** Special signal hndler. */
     DataCollector_SignalHandler SignalHandler;
     friend DataCollector_SignalHandler;
private:
     /** Configuration flag.
	 A run can only be started after a successful initialization.*/
     bool IsConfigured;

     /** Interpreter for incomming comands */
     Interpreter<DataCollector> CMD_Interpreter;

     /** Hardware abstarction layer */
     HAL_t hal;

     /** The vector that holds the configuration for the OM channels */
     ChannelConfigRecordVector_t ChannelConfig;

     /** The vector that holds the configuration for the OM - DMAD channels */
     DMAD_ChannelConfigRecordVector_t DMAD_ChannelConfig;

     /** The vector that holds the configuration for the crates */
     CrateConfigRecordVector_t   CrateConfig;  

     /** The vector that holds the configuration for the Modules */
     ChannelModuleConfigRecordVector_t  ModuleConfig;

     /** Pointer to data buffer 1.
	 A new buffer will be allocated upon intialization */
     ShmBinaryDataBuffer *WriteBuffer1;

     /** Pointer to data buffer 2.
	 A new buffer will be allocated upon intialization */
     ShmBinaryDataBuffer *WriteBuffer2;

     /** Pointer to the currently used data buffer.
	 The datacollector writes data to one buffer. The other buffer is
	 send to the dataprocess by the datatransport process at the same time.
     Switching buffers is organized using a semaphore (see {\tt \Ref{ShmBinaryDataBuffer}}.*/
     ShmBinaryDataBuffer *CurWriteBuffer;

     /** Size of data buffer */
     u_i64 DataBufferSize;

     /** Determines after how many bytes a catch record will be added */
     unsigned int CatchSize;
     
     /** Run stop Flag */
     bool Stop_FLAG;

     /** Number of muon trigger that occured */
     unsigned long int NumMuonTrigger;

     /** Number of muon triggers after which asynchronous modules are read out */
     unsigned int AsynchReadout;
};

#endif // __VMEDATAPROCESS_H__



