/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datafilecode.C,v 1.7 1999/02/05 18:35:02 steffen Exp $

  $Log: datafilecode.C,v $
  Revision 1.7  1999/02/05 18:35:02  steffen
  * Major cleanup:
    - threw out code for binary config records
    - cleaned up some module classes
  * Fixed bug which prevent 32 bit values to be read correctly from
    detector cnf
  * Added masking to s7201 module

*/

#include "datafilecode.h"
#include "channelmodule.h"
#include "cnfreader.h"
#include "hwconfigitems2.h"

bool IsValidRecordID(u_i32 RecordID)
{
     switch(RecordID){
	  // list all nown RIDs here:
     case RID_RUNINFO:
     case RID_MUONEVENT:
     case RID_COUNTEREVENT:
     case RID_HWCONFIG_ASCII:
     case RID_CATCH:
	  return true;
     }
     
     return false;
}

bool IsReasonableRecordID(u_i32 RecordID)
{
     return  ((RecordID>=10000 && RecordID<=10050)
	      || (RecordID>=30000 && RecordID<=30050));
}

RunInfo_t::RunInfo_t(){
     Magic=0;
     FormatVersion=0;
     ProgramVersion=0;
     RunNumber=0;
     DebugLevel=0;
     LogLevel=0;
     RunStop=0;
     NumHardwareTriggers=0;
     HardwareTriggers=0;
     HardwareTrigger_NoOfBytes=0;
     NumSoftwareTriggers=0;
     SoftwareTriggers=0;
     SoftwareTrigger_NoOfBytes=0;
     Comment=0;
	  
}
     
RunInfo_t::~RunInfo_t(){
     if(Magic!=0) delete[] Magic;
     if(FormatVersion!=0) delete[] FormatVersion;
     if(ProgramVersion!=0) delete[] ProgramVersion;
     if(Comment!=0) delete[] Comment;
}

u_i32 RunInfo_t::GetRecordID(){
     return RID_RUNINFO;
}       

size_t RunInfo_t::ReadFromBuffer(BinaryDataBuffer &Buf){
     if(Magic!=0) delete[] Magic;
     if(FormatVersion!=0) delete[] FormatVersion;
     if(ProgramVersion!=0) delete[] ProgramVersion;
     if(Comment!=0) delete[] Comment;

     // Mark the current position as a new Record
     Buf.RecordBegin();

     // read all data elements
     char *str_temp;
     Buf.ReadFromBuffer(str_temp);
     if(!strcmp(str_temp,MAGIC))	{
	  Magic=new char[strlen(str_temp)+1];
	  strcpy(Magic,str_temp);
	  Buf.ReadFromBuffer(str_temp);
     }
     FormatVersion=new char[strlen(str_temp)+1];
     strcpy(FormatVersion,str_temp);
     Buf.ReadFromBuffer(str_temp);
     ProgramVersion=new char[strlen(str_temp)+1];
     strcpy(ProgramVersion,str_temp);
     Buf.ReadFromBuffer(RunNumber);
     Buf.ReadFromBuffer(DebugLevel);
     Buf.ReadFromBuffer(LogLevel);
     Buf.ReadFromBuffer(RunStop);
     Buf.ReadFromBuffer(HardwareTrigger_NoOfBytes);
     // not interested yet
     Buf.IncPosition(HardwareTrigger_NoOfBytes);
     
     Buf.ReadFromBuffer(SoftwareTrigger_NoOfBytes);
     // not interested yet
     Buf.IncPosition(SoftwareTrigger_NoOfBytes);

     Buf.ReadFromBuffer(str_temp);
     Comment=new char[strlen(str_temp)+1];
     strcpy(Comment,str_temp);

     // return the size in bytes, that we read
     size_t s= Buf.RecordEnd_GetSize();
     return s;
}
     
size_t RunInfo_t::WriteToBuffer(BinaryDataBuffer &Buf){
     // Mark the current position as a new Record
     Buf.RecordBegin();
     // write all data elements
     if(FORMAT_VERSION > 900){
	  if(Magic)
	       Buf.WriteToBuffer(Magic);
	  else 
	       Buf.WriteToBuffer(MAGIC);
     }
     if(FormatVersion)
	  Buf.WriteToBuffer(FormatVersion);
     else {
	  char tmp[20];
	  sprintf(tmp,"%f",FORMAT_VERSION/1000.0);
	  Buf.WriteToBuffer(tmp);
     }
     
     if(ProgramVersion)
	  Buf.WriteToBuffer(ProgramVersion);
     else {
	  char tmp[20];
	  sprintf(tmp,"%f",PROGRAM_VERSION/1000.0);
	  Buf.WriteToBuffer(tmp);
     }
     
     Buf.WriteToBuffer(RunNumber);
     Buf.WriteToBuffer(DebugLevel);
     Buf.WriteToBuffer(LogLevel);
     Buf.WriteToBuffer(RunStop);
     
     Buf.PushPosition();
     // don't know yet
     HardwareTrigger_NoOfBytes=0;
     Buf.WriteToBuffer(HardwareTrigger_NoOfBytes);
     Buf.RecordBegin();
     int i;
     for(i=0;i<NumHardwareTriggers;i++)
	  Buf.WriteToBuffer(HardwareTriggers[i]);
     HardwareTrigger_NoOfBytes=Buf.RecordEnd_GetSize();
     Buf.PopPosition();
     Buf.WriteToBuffer(HardwareTrigger_NoOfBytes);
     Buf.IncPosition(HardwareTrigger_NoOfBytes);
          
     // don't know yet
     Buf.PushPosition();
     SoftwareTrigger_NoOfBytes=0;
     Buf.WriteToBuffer(SoftwareTrigger_NoOfBytes);
     Buf.RecordBegin();
     for(i=0;i<NumSoftwareTriggers;i++)
	  Buf.WriteToBuffer(SoftwareTriggers[i]);
     SoftwareTrigger_NoOfBytes=Buf.RecordEnd_GetSize();
     Buf.PopPosition();
     Buf.WriteToBuffer(SoftwareTrigger_NoOfBytes);
     Buf.IncPosition(SoftwareTrigger_NoOfBytes);

     if(Comment==0)
	  Buf.WriteToBuffer("");
     else
	  Buf.WriteToBuffer(Comment);
     // return the size in bytes, that we read
     size_t s= Buf.RecordEnd_GetSize();
     return s;
}     
void RunInfo_t::SetComment(const char *comment){
     if(Comment!=0) delete[] Comment;
     Comment=new char[strlen(comment)+1];
     strcpy(Comment,comment);
}

HWConfig_t::HWConfig_t(){
      
     CrateConfigRecordVector=0;
     ModuleConfigRecordVector=0;
     ChannelConfigRecordVector=0;
};

HWConfig_t::~HWConfig_t(){
     if(CrateConfigRecordVector!=0) 
	  delete CrateConfigRecordVector;     
     if(ModuleConfigRecordVector!=0) 
	  delete ModuleConfigRecordVector;
     if(ChannelConfigRecordVector!=0)
	  delete ChannelConfigRecordVector;
};

HWConfigAscii_t::HWConfigAscii_t(){
     cfg_instream=0;
     cfg_strstream=0;
}

HWConfigAscii_t::~HWConfigAscii_t(){
     if (cfg_strstream!=0) 
	  delete cfg_strstream;
}
 
u_i32 HWConfigAscii_t::GetRecordID(){
     return RID_HWCONFIG_ASCII;
}       

size_t HWConfigAscii_t::ReadFromBuffer(BinaryDataBuffer &Buf){  
     if(CrateConfigRecordVector!=0) 
	  delete CrateConfigRecordVector;     
     if(ModuleConfigRecordVector!=0) 
	  delete ModuleConfigRecordVector;
     if(ChannelConfigRecordVector!=0)
	  delete ChannelConfigRecordVector;
     u_i32 NoOfBytes_r=sizeof(RID_HWCONFIG_ASCII);
     CrateConfigRecordVector  =new CrateConfigRecordVector_t;     
     ModuleConfigRecordVector =new ChannelModuleConfigRecordVector_t;
     ChannelConfigRecordVector=new ChannelConfigRecordVector_t;  

     char *tmp_str;
     if (cfg_strstream!=0) delete cfg_strstream;
     cfg_strstream=new ostrstream;
     RecordHeader_t RecordHeader;
     // rewind
     Buf.SetPosition(Buf.GetPosition() - sizeof(RecordHeader.RecordID) - sizeof(RecordHeader.NoOfBytes));
     // get number of bytes
     ReadNextRecordHeader(RecordHeader,Buf);

     // Mark current position as the beginning of a new record
     Buf.RecordBegin();
     // read all data
     while(NoOfBytes_r < RecordHeader.NoOfBytes) {
	  Buf.ReadFromBuffer(tmp_str);
	  NoOfBytes_r+=strlen(tmp_str) + 1;
	  (*cfg_strstream) << tmp_str;
     }
     (*cfg_strstream) << ends;
     u_i32 EndTag;
     Buf.ReadFromBuffer(EndTag);
     if(EndTag!=SUB_RID_ENDOFHWCONFIG){
	  cerr << "*********End tag did not match\n***";
     }
     
     ConfigReader cr;
     CrateConfigRecordVector->Clear();
     ModuleConfigRecordVector->Clear();
     ChannelConfigRecordVector->Clear();
     
     cr.AddSection(new CrateSection_t(*CrateConfigRecordVector));
     cr.AddSection(new ModuleSection_t(*ModuleConfigRecordVector));
     cr.AddSection(new ChannelSection(*ChannelConfigRecordVector));

     cfg_strstream->freeze(1);
     istrstream i_str(cfg_strstream->str());
     cfg_strstream->freeze(0);
	  
     if(cr.Read(i_str)<0){
	  return 0;
     }
     
     // return the size in bytes, that we read
     return Buf.RecordEnd_GetSize();
}     

int HWConfigAscii_t::ReadFromStream(istream &InStream){  
     if(CrateConfigRecordVector!=0) 
	  delete CrateConfigRecordVector;     
     if(ModuleConfigRecordVector!=0) 
	  delete ModuleConfigRecordVector;
     if(ChannelConfigRecordVector!=0)
	  delete ChannelConfigRecordVector;

     CrateConfigRecordVector  =new CrateConfigRecordVector_t;     
     ModuleConfigRecordVector =new ChannelModuleConfigRecordVector_t;
     ChannelConfigRecordVector=new ChannelConfigRecordVector_t;  

     ConfigReader cr;
     CrateConfigRecordVector->Clear();
     ModuleConfigRecordVector->Clear();
     ChannelConfigRecordVector->Clear();
     
     cr.AddSection(new CrateSection_t(*CrateConfigRecordVector));
     cr.AddSection(new ModuleSection_t(*ModuleConfigRecordVector));
     cr.AddSection(new ChannelSection(*ChannelConfigRecordVector));

     return cr.Read(InStream);
}     

size_t HWConfigAscii_t::WriteToBuffer(BinaryDataBuffer &Buf){
     char tmp_str[MAX_TEXT_SIZE + 1];
     
     // Mark current position as the beginning of a new recor
     Buf.RecordBegin();
     //write all Data
     int len;
     while(!cfg_instream->eof()){
	  cfg_instream->getline(tmp_str,MAX_TEXT_SIZE);
	  len=strlen(tmp_str);
	  tmp_str[len]='\n';
	  tmp_str[len+1]='\0';
	  Buf.WriteToBuffer(tmp_str);
     }
     Buf.WriteToBuffer(SUB_RID_ENDOFHWCONFIG);
     return Buf.RecordEnd_GetSize();
}     

size_t HWConfigAsciiExtract_t::ReadFromBuffer(BinaryDataBuffer &Buf){  
     if(CrateConfigRecordVector!=0) 
	  delete CrateConfigRecordVector;     
     if(ModuleConfigRecordVector!=0) 
	  delete ModuleConfigRecordVector;
     if(ChannelConfigRecordVector!=0)
	  delete ChannelConfigRecordVector;

     u_i32 NoOfBytes_r=sizeof(RID_HWCONFIG_ASCII);
     CrateConfigRecordVector  =new CrateConfigRecordVector_t;     
     ModuleConfigRecordVector =new ChannelModuleConfigRecordVector_t;
     ChannelConfigRecordVector=new ChannelConfigRecordVector_t;  

     char *tmp_str;
     if (cfg_strstream!=0) delete cfg_strstream;
     cfg_strstream=new ostrstream;
     RecordHeader_t RecordHeader;
     // rewind
     Buf.SetPosition(Buf.GetPosition() - sizeof(RecordHeader.RecordID) - sizeof(RecordHeader.NoOfBytes));
     // get number of bytes
     ReadNextRecordHeader(RecordHeader,Buf);

     // Mark current position as the beginning of a new record
     Buf.RecordBegin();
     // read all data
     while(NoOfBytes_r < RecordHeader.NoOfBytes) {
	  Buf.ReadFromBuffer(tmp_str);
	  NoOfBytes_r+=strlen(tmp_str) + 1;
	  (*cfg_strstream) << tmp_str;
     }
     (*cfg_strstream) << ends;
     u_i32 EndTag;
     Buf.ReadFromBuffer(EndTag);
     if(EndTag!=SUB_RID_ENDOFHWCONFIG){
	  cerr << "*********End tag did not match\n***";
     }
     
     // return the size in bytes, that we read
     return Buf.RecordEnd_GetSize();
}     

size_t HWConfigAsciiExtract_t::WriteToBuffer(BinaryDataBuffer &/*Buf*/){
     return 0;
}


/*---------------------------------------------------------*/
/*                                                         */
/*        ChannelConfigRecord_t member functions           */
/*                                                         */
/*---------------------------------------------------------*/
ChannelConfigRecord_t::ChannelConfigRecord_t(){
     ID=0;
     StringNumber=0;
     StringPosition=0;
     
     DISC_Id=0;
     DISC_Channel=0;
     DISC_On=0;
     
     TDC_Id=0;
     TDC_Channel=0;
     TDC_On=0;
     
     ADC_Id=0;
     ADC_Channel=0;
     ADC_On=0;
     
     SCALER_Id=0;
     SCALER_Channel=0;
     SCALER_On=0;
     
     SpecialINT=0;
     SpecialString=0;
     
     Group=NONE_val;
}

ChannelConfigRecord_t::~ChannelConfigRecord_t(){
     if(DISC_Id!=0) delete[] DISC_Id;
     if(ADC_Id!=0) delete[] ADC_Id;
     if(TDC_Id!=0) delete[] TDC_Id;
     if(SCALER_Id!=0) delete[] SCALER_Id;
}
