/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datafilecode.h,v 1.12 1999/02/08 15:18:27 wischnew Exp $
*/

#if ! defined __DATAFILECODE_H__
#define __DATAFILECODE_H__

#include <config.h>
#include "globalmacros.h"
#include "binarydatabuffer.h"
#include "shmbinarydatabuffer.h"
#include "stringutil.h"
#include "daqprocessbase.h"
#include "channelmodule.h"
#include <strstream.h>

// The Identifier for the Records in the data file

#define RID_RUNINFO        ((u_i32)10001)
#define RID_DISCSTATUS     ((u_i32)10004)
#define RID_MUONEVENT      ((u_i32)10005)
#define RID_STATUSINFO     ((u_i32)10006)
#define RID_COUNTEREVENT   ((u_i32)10007)
#define RID_HWCONFIG_ASCII ((u_i32)10008)
#define RID_DMADEVENT      ((u_i32)10009)
#define RID_SNEVENT        ((u_i32)10010)

#define SUB_RID_ENDOFHWCONFIG            ((u_i32)20099)
#define SUB_RID_CRATECONFIG              ((u_i32)20001)
#define SUB_RID_CHANNELCONFIG            ((u_i32)20007)

#define RID_HOUSEKEEPING   ((u_i32)30000)
#define RID_CATCH          ((u_i32)30001)
#define END_OF_EVENT_TAG   ((binary_tag_t) 0xFFF7)

#define CATCH_DATA "Auferstanden aus Ruinen und der Zunkunft zugewandt"
#define REASONABLE_RECORD_SIZE 100000

/** Checks for valid record IDs.
    @param RecordID Record ID
    @return true if record ID is a valid ID -1 otherwise. */
bool IsValidRecordID(u_i32 RecordID);

/** Checks if record ID is a reasonable ID.
    You want to have your reader just skip unknown ID but
    still be able to identify corrupt files 
    @param RecordID Record ID
    @return true if record ID is a reasonable ID -1 otherwise. */
bool IsReasonableRecordID(u_i32 RecordID);

/** Record for the Run Information */
struct RunInfo_t
{
     RunInfo_t();
     ~RunInfo_t();
     u_i32 GetRecordID();
     size_t ReadFromBuffer(BinaryDataBuffer &Buf);
     size_t WriteToBuffer(BinaryDataBuffer &Buf);
     void SetComment(const char *comment);
          
     char * Magic;
     char * FormatVersion;
     char * ProgramVersion;
     u_i32 RunNumber;
     u_i32 DebugLevel;
     u_i32 LogLevel;
     u_i32 RunStop;
     u_i32 HardwareTrigger_NoOfBytes;
     char **HardwareTriggers;
     int NumHardwareTriggers;
     u_i32 SoftwareTrigger_NoOfBytes;
     char **SoftwareTriggers;
     int NumSoftwareTriggers;
     char *Comment;
};



MAKE_REFCOUNT_POINTER(RunInfo_t,RunInfo_p);

/** Catch Record */
struct Catch_t
{
     Catch_t(){
	  CatchData=0;
	  NoOfBytes=0;
     }
     
     ~Catch_t(){
	  if(CatchData!=0) delete[] CatchData;
     }
     
     u_i32 GetRecordID(){
	  return RID_CATCH;
     }       

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  if(CatchData!=0) delete[] CatchData;
	  Buf.RecordBegin();
	  
	  char *str_temp;

	  Buf.ReadFromBuffer(str_temp);
	  CatchData=new char[strlen(str_temp)+1];
	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
	  Buf.WriteToBuffer(CATCH_DATA);
	  // retrieve the number of bytes for the last subrecord
	  return Buf.RecordEnd_GetSize();
     }     
     
     bool FindCatch(BinaryDataBuffer &Buf){
	  int Length=sizeof(CATCH_DATA);
	  char *CatchStr=CATCH_DATA;
	  u_i8 data;
	  bool success=false;
	  
	  while(!success && !Buf.end_of_read()) {
	       for(int i=0;i<Length;i++){
		    Buf.ReadFromBuffer(data);
		    if((char) data!=CatchStr[i])
			 break;
		    else if(i==Length-1){
			 success=true;
		    }
		    
	       }
	  }
	  return success;
     }
	  
     char *CatchData;
     u_i32 NoOfBytes;
};

MAKE_REFCOUNT_POINTER(Catch_t,Catch_p);

/** Record for some status information */
struct StatusInfo_t
{
     StatusInfo_t(){
     }
     ~StatusInfo_t(){
     }
	  
     u_i32 GetRecordID(){
	  return RID_STATUSINFO;
     }
	       
     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements

	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements

	  // return the size in bytes, that we wrote
	  return Buf.RecordEnd_GetSize();
     }     

};

MAKE_REFCOUNT_POINTER(StatusInfo_t,StatusInfo_p);
MAKE_INC_VECTOR(StatusInfo_p,StatusInfoVector_t);

struct CrateConfigRecord_t
{
     CrateConfigRecord_t(){
	  NoOfBytes=0;
	  Enabled=0;
	  CrateID=0;
	  CrateType=0;
	  ConnectedTo=0;
     }
     
     virtual ~CrateConfigRecord_t(){
	  if(CrateID!=0) delete[] CrateID;
	  if(CrateType!=0) delete[] CrateType;
	  if(ConnectedTo!=0) delete[] ConnectedTo;
     }

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  if(CrateID!=0) delete[] CrateID;
	  if(CrateType!=0) delete[] CrateType;
	  if(ConnectedTo!=0) delete[] ConnectedTo;

	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements

	  Buf.ReadFromBuffer(NoOfBytes);
	  // mark the position as a new subrecord
	  Buf.RecordBegin();
	  Buf.ReadFromBuffer(Enabled);
	  char *str_temp;

	  Buf.ReadFromBuffer(str_temp);
	  CrateID=new char[strlen(str_temp)+1];
	  strcpy(CrateID,str_temp);

	  Buf.ReadFromBuffer(str_temp);
	  CrateType=new char[strlen(str_temp)+1];
	  strcpy(CrateType,str_temp);

	  Buf.ReadFromBuffer(str_temp);
	  ConnectedTo=new char[strlen(str_temp)+1];
	  strcpy(ConnectedTo,str_temp);
	  
	  u_i32 NoOfBytes_r=Buf.RecordEnd_GetSize();
	  if(NoOfBytes!=NoOfBytes_r){
	       // error read bytes != the specified size
	  }
	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     
     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
	  
	  // save the position of the NoOfBytes number on the stack ,
	  // since we not it after writing the rest
	  Buf.PushPosition();

	  Buf.WriteToBuffer(NoOfBytes);
	  
	  // mark the position as a new subrecord
	  Buf.RecordBegin();
	  
	  Buf.WriteToBuffer(Enabled);
	  Buf.WriteToBuffer(CrateID);
	  Buf.WriteToBuffer(CrateType);
	  Buf.WriteToBuffer(ConnectedTo);
	  // retrieve the number of bytes for the last subrecord
	  NoOfBytes=Buf.RecordEnd_GetSize();
	  // retrive the position where we have to write the size
	  Buf.PopPosition();
	  Buf.WriteToBuffer(NoOfBytes);
	  Buf.IncPosition(NoOfBytes);

	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     
     
     virtual u_i32 GetsubRecordID(){
	  return SUB_RID_CRATECONFIG;
     }

     u_i32 NoOfBytes;
     u_i32 Enabled;
     char *CrateID;
     char *CrateType;
     char *ConnectedTo;
};

MAKE_REFCOUNT_POINTER(CrateConfigRecord_t,CrateConfigRecord_p);
MAKE_INC_VECTOR(CrateConfigRecord_p,CrateConfigRecordVector_t);






/*---------------------------------------------------------*/
/*                                                         */
/*        OM channel configuration data structure          */
/*                                                         */
/*---------------------------------------------------------*/

/** Data structure for OM channel configuration */
struct ChannelConfigRecord_t
{
     /** Constructor */
     ChannelConfigRecord_t();
     
     /** Destructor */
     virtual ~ChannelConfigRecord_t();
          	

     /** OM id (or channel) */
     ID_t ID;

     /** Number of string this OM is connected to*/
     u_i16 StringNumber;

     /** Position on string, counting from the top of the string. */
     u_i16 StringPosition;

     /** Id string of discriminator module used by this OM. */
     char *DISC_Id;

     /** Discriminbator channel used by this OM.
	 This is the channel of the discriminator module given
	 by {\tt \Ref{DISC_Id}}. */
     channel_t DISC_Channel;

     /** On/Off flag of the discriminator channel. */
     u_i32 DISC_On;
     
     /** Id string of TDC module used by this OM. */
     char *TDC_Id;

     /** TDC channel used by this OM.
	 This is the channel of the TDC module given
	 by {\tt \Ref{DISC_Id}}. */
     channel_t TDC_Channel;

     /** On/Off flag of the TDC channel. */
     u_i32 TDC_On;
     
     /** Id string of ADC module used by this OM. */
     char *ADC_Id;

     /** ADC channel used by this OM.
	 This is the channel of the ADC module given
	 by {\tt \Ref{DISC_Id}}. */
     channel_t ADC_Channel;

     /** On/Off flag of the ADC channel. */
     u_i32 ADC_On;

     /** Id string of scaler module used by this OM. */
     char *SCALER_Id;

     /** SCALER channel used by this OM.
	 This is the channel of the scaler module given
	 by {\tt \Ref{DISC_Id}}. */
     channel_t SCALER_Channel;

     /** On/Off flag of the scaler channel. */
     u_i32 SCALER_On;
     u_i32 Group;

     /** ??? */
     u_i32 SpecialINT;

     /** String name for special channels to be used in F2000 */
     char *SpecialString;

};

MAKE_REFCOUNT_POINTER(ChannelConfigRecord_t,ChannelConfigRecord_p);
MAKE_INC_VECTOR(ChannelConfigRecord_p,ChannelConfigRecordVector_t);


struct StringConfigRecord_t{
     u_i16 StringNumber;                    // Number of the String 
                                            // (ChannelConfigRecord_t.StringNumber)
     u_i16 MaxOMPosition;                   // Max OM-Number 
                                            // (ChannelConfigRecord_t.StringPosition)
     ChannelConfigRecordVector_t Channels;  // a vector of pointer to the channels 
     // in this strings
};


MAKE_REFCOUNT_POINTER(StringConfigRecord_t,StringConfigRecord_p);
MAKE_INC_VECTOR(StringConfigRecord_p,StringConfigRecordVector_t);

struct ID_comp{
     inline ID_comp(u_i16 sn):StringNumber(sn){}
     inline bool operator()(StringConfigRecord_p &scr){
	  return scr->StringNumber==StringNumber;
     }
     u_i16 StringNumber;
};

inline void MakeStringConfig(ChannelConfigRecordVector_t &cc,
			     StringConfigRecordVector_t &sc){
     ChannelConfigRecordVector_t::iterator c_first=cc.begin(),c_last=cc.end();
     while(c_first!=c_last){
	  StringConfigRecordVector_t::iterator sci;
	  if((sci=Find_If(sc.begin(),sc.end(),ID_comp((*c_first)->StringNumber)))!=sc.end()){
	       (*sci)->Channels.push_back(*c_first);
	       if((*sci)->MaxOMPosition<(*c_first)->StringPosition)
		    (*sci)->MaxOMPosition=(*c_first)->StringPosition;
	  } else {
	       StringConfigRecord_p scr(new StringConfigRecord_t);
	       scr->StringNumber=(*c_first)->StringNumber;
	       scr->Channels.push_back(*c_first);
	       scr->MaxOMPosition=(*c_first)->StringPosition;
	       sc.push_back(scr);
	  }
	  ++c_first;
     }
}

/** Record  for the Hardware configuration */
class HWConfig_t
{
public:
     HWConfig_t();
     virtual ~HWConfig_t();
     virtual u_i32 GetRecordID()=0;
     virtual size_t ReadFromBuffer(BinaryDataBuffer &Buf)=0;
     virtual size_t WriteToBuffer(BinaryDataBuffer &Buf)=0;

public:
     CrateConfigRecordVector_t        *CrateConfigRecordVector;     
     ChannelModuleConfigRecordVector_t  *ModuleConfigRecordVector;
     ChannelConfigRecordVector_t      *ChannelConfigRecordVector;
};


/** Record  for the Hardware configuration */
class HWConfigAscii_t: public HWConfig_t
{
public:
     HWConfigAscii_t();
     virtual ~HWConfigAscii_t();
     virtual u_i32 GetRecordID();
     virtual size_t ReadFromBuffer(BinaryDataBuffer &Buf);
     int ReadFromStream(istream &InStream);
     virtual size_t WriteToBuffer(BinaryDataBuffer &Buf);

public:     
     istream *cfg_instream;
     ostrstream *cfg_strstream;
};

/** Record  for the Hardware configuration extraction */
class HWConfigAsciiExtract_t: public HWConfigAscii_t
{
public:
     virtual size_t ReadFromBuffer(BinaryDataBuffer &Buf);
     virtual size_t WriteToBuffer(BinaryDataBuffer &Buf);
};


/** Record for the header for an  Event */
struct CounterEventHeader_t
{
     CounterEventHeader_t(){
	  NoOfBytes=0;
	  NoOfReadouts=0;
     }
     
     virtual ~CounterEventHeader_t(){
     }

     u_i32 GetRecordID(){
	  return RID_COUNTEREVENT;
     }       

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements
	  Buf.ReadFromBuffer(NoOfReadouts);
	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
//	  Buf.WriteToBuffer(EventLength);
	  Buf.WriteToBuffer(NoOfReadouts);

	  // return the size in bytes, that we wrote+  size of the
	  // Bytes of the actual event data, which should be already
	  // written
	  return Buf.RecordEnd_GetSize()+NoOfBytes;
     }          
     u_i32 NoOfBytes;
     u_i32 NoOfReadouts;
};


/** Record for the header of an  Super Nova event */
struct SNEventHeader_t
{
     SNEventHeader_t(){
	  NoOfBytes=0;
	  NoOfReadouts=0;
     }
     
     virtual ~SNEventHeader_t(){
     }

     u_i32 GetRecordID(){
	  return RID_SNEVENT;
     }       

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements
	  Buf.ReadFromBuffer(NoOfReadouts);
	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
	  Buf.WriteToBuffer(NoOfReadouts);

	  // return the size in bytes, that we wrote+  size of the
	  // Bytes of the actual event data, which should be already
	  // written
	  return Buf.RecordEnd_GetSize()+NoOfBytes;
     }          
     u_i32 NoOfBytes;
     u_i32 NoOfReadouts;
};

/** Record for the header for an  Event */
struct DMAD_EventHeader_t{
     DMAD_EventHeader_t(){
	  NoOfBytes=0;
     }
     
     virtual ~DMAD_EventHeader_t(){
     }

     u_i32 GetRecordID(){
	  return RID_DMADEVENT;
     }       

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements

	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements

	  // return the size in bytes, that we wrote+  size of the
	  // Bytes of the actual event data, which should be already
	  // written
	  return Buf.RecordEnd_GetSize()+NoOfBytes;
     }          
     u_i32 NoOfBytes;
     u_i32 NoOfReadouts;
};

// /** Record for the DMAD-Status in bin-file */
// struct DMAD_Status_t{
//      DMAD_Status_t():
// 	  NoOfBytes(0),
// 	  NoOfChannels(0),
// 	  DMAD_ChannelConfigRecordVector(0){
//      }     

//      u_i32 GetRecordID(){
// 	  return RID_DMAD_STATUS;
//      }

//      size_t ReadFromBuffer(BinaryDataBuffer &Buf){
// 	  Buf.RecordBegin();
// 	  // read all data elements
// 	  Buf.ReadFromBuffer(NoOfBytes);
// 	  // Mark the current position as a new Record
// 	  Buf.RecordBegin();
// 	  // read all data elements
// 	  Buf.ReadFromBuffer(NoOfChannels);
// 	  for(i=0;i<NoOfChannels;i++){
// 	       DMAD_ChannelConfigRecord_p DMAD_ChannelConfigRecord=new DMAD_ChannelConfigRecord_t;
// 	       DMAD_ChannelConfigRecord->ReadFromBuffer(Buf);
// 	       DMAD_ChannelConfigRecordVector->push_back(DMAD_ChannelConfigRecord);
// 	  }
// 	  u_i32 NoOfBytes_r=Buf.RecordEnd_GetSize();
// 	  if(NoOfBytes!=NoOfBytes_r){
// 	       // error read bytes != the specified size
// 	  }
// 	  // return the size in bytes, that we read
// 	  return Buf.RecordEnd_GetSize();
//      }     

//      size_t WriteToBuffer(BinaryDataBuffer &Buf){
// 	  // Mark the current position as a new Record
// 	  Buf.RecordBegin();
// 	  // write all data elements
// 	  Buf.PushPosition();
// 	  Buf.WriteToBuffer(NoOfBytes);
// 	  // Mark the current position as a new Record
// 	  Buf.RecordBegin();
// 	  // write all data elements
// 	  NoOfChannels=DMAD_ChannelConfigRecordVector->size();
// 	  Buf.WriteToBuffer(NoOfChannels);

// 	  for(i=0;i<NoOfChannels;i++)
// 	       (*DMAD_ChannelConfigRecordVector)[i]->WriteToBuffer(Buf);

// 	  NoOfBytes=Buf.RecordEnd_GetSize();
// 	  Buf.PopPosition();
// 	  Buf.WriteToBuffer(NoOfBytes);
// 	  Buf.IncPosition(NoOfBytes);
// 	  // return the size in bytes, that we wrote
// 	  return Buf.RecordEnd_GetSize();
//      }
//      u_i32 NoOfBytes;
//      u_i32 NoOfChannels;
//      DMAD_ChannelConfigRecordVector_t *DMAD_ChannelConfigRecordVector;
// };

/** Record for the header for an  Event */
struct MuonEventHeader_t
{
     MuonEventHeader_t(){
	  NoOfBytes=0;
	  EventNumber=0;
	  UnixTime=0;
	  MuTime=0;
	  NHitTDC=0;
	  NHitADC=0;
	  TriggerReg=0; 
     }
     
     virtual ~MuonEventHeader_t(){
     }

     u_i32 GetRecordID(){
	  return RID_MUONEVENT;
     }       

     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements
	  Buf.ReadFromBuffer(EventNumber);
	  Buf.ReadFromBuffer(UnixTime);
	  Buf.ReadFromBuffer(MuTime);
	  Buf.ReadFromBuffer(NHitTDC);
	  Buf.ReadFromBuffer(NHitADC);
	  Buf.ReadFromBuffer(TriggerReg); 

	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     

     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
//	  Buf.WriteToBuffer(EventLength);
	  Buf.WriteToBuffer(EventNumber);
	  Buf.WriteToBuffer(UnixTime);
	  Buf.WriteToBuffer(MuTime);
	  Buf.WriteToBuffer(NHitTDC);
	  Buf.WriteToBuffer(NHitADC);
	  Buf.WriteToBuffer(TriggerReg); 

	  // return the size in bytes, that we wrote+ size size of the
	  // Bytes of the actual event data, which should be already
	  // written
	  return Buf.RecordEnd_GetSize()+NoOfBytes;
     }          
     u_i32 NoOfBytes;
     u_i32 EventNumber;
     u_i32 UnixTime;
     u_i32 MuTime;
     u_i32 NHitTDC;
     u_i32 NHitADC;
     u_i32 TriggerReg; 
};

/** the Header of every Record */
struct RecordHeader_t
{
     RecordHeader_t(){
	  RecordID=0;
	  NoOfBytes=0;
     }
     
     virtual ~RecordHeader_t(){
     }
     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // read all data elements
	  Buf.ReadFromBuffer(RecordID);
	  Buf.ReadFromBuffer(NoOfBytes);
	  
	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }     
     
     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
	  Buf.WriteToBuffer(RecordID);
	  Buf.WriteToBuffer(NoOfBytes);

	  // return the size in bytes, that we wrote
	  return Buf.RecordEnd_GetSize();
     }     
     inline size_t Size(){
	  return (sizeof(RecordID) + sizeof(NoOfBytes));
     }
     
     u_i32 RecordID;
     u_i32 NoOfBytes;
};


inline int ReadNextRecordHeader(RecordHeader_t &RecordHeader,BinaryDataBuffer &Buf){
     return RecordHeader.ReadFromBuffer(Buf);
}

inline void SkipRecord(RecordHeader_t &RecordHeader,BinaryDataBuffer &Buf){
     Buf.IncPosition(RecordHeader.NoOfBytes);
}

template <class Record_t>
inline int ReadRecord(Record_t &aRecord,BinaryDataBuffer &Buf){
     return aRecord.ReadFromBuffer(Buf);
}

template <class Record_t>
inline int WriteRecord(Record_t &aRecord,BinaryDataBuffer &Buf){
     // fill a record header structure
     RecordHeader_t RecordHeader;
     RecordHeader.RecordID=aRecord.GetRecordID();
     RecordHeader.NoOfBytes=0; // we don't know
     // save the position where we have to write the header
     Buf.PushPosition();
     RecordHeader.WriteToBuffer(Buf);
     // the write function says us, how many bytes where written
     RecordHeader.NoOfBytes=aRecord.WriteToBuffer(Buf);
     // restore the old position
     Buf.PopPosition();
     // write the header again
     RecordHeader.WriteToBuffer(Buf);
     // jump to the end of the record
     Buf.IncPosition(RecordHeader.NoOfBytes);
     return RecordHeader.NoOfBytes;
}



#endif // __DATAFILECODE_H__
