/*
  $Header: /home/elrond/baikal/src/CVS/daq98/dataprocess.C,v 1.24 1999/02/11 07:07:05 daq Exp $
*/

#include <strstream.h>
#include <iostream.h>
#include <fstream.h>
#include <errno.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#error is defined
#else
#include <my_getopt.h>
#endif 
#include "globalfunctions.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "event.h"
#include "histosections.h"
#include "dataprocess.h"
#include "datafilecode.h"
#include "globalfunctions.h"
#include "stringutil.h"
#include "runfilename.h"
#include "mailer.h"
#define __NO_HBOOK__
#include "muoneventhistoset.h"
#include "transferclass.h"

///////////////////////////
//
// put your DAQ98 data analysis header files here
#include "daq98hwconfig.h"
#include "daq98muoneventbuilder.h"
#include "daq98counterevent.h"
#include "daq98dmadevent.h"
#include "daq98f2000spectrig.h"
#include "daq98runtime.h"
#include "daq98runinfo.h"

///////////////////////////
//
// put your DAQ98 muon event analysis header files here
#include "daq98muoneventhisto.h"
#include "daq98f2000.h"


clock_t my_times();

int DataProcess_SignalHandler::Handle_SIGALRM(Process &/*P*/)
{
     Quit_FLAG=true;
     return 0;
}

DataProcess::DataProcess():Daemon(D_USE_SHM|D_CREATE_SHM),
     Streamer(this), MEventBuilder(this){
     DEB_F("DataProcess::DataProcess()");
     SetVerboseLevel(15,15);
     
     // set the signal handler
     ThisProcess->GetSignalHandler().SignalAction(SIGALRM,(sighandler_t)SIG_MY);
     ThisProcess->SetSignalHandler(SignalHandler);

     
     IPC_CMD_Interpreter.AddFunction("bye",     &IPC_CMD_Quit,
				     "bye\n\t-- quits the dataprocess");
     IPC_CMD_Interpreter.AddFunction("Suspend", &IPC_CMD_SuspendProcess,
				     "suspend\n\t-- suspends the execution of the dataprocess");
     IPC_CMD_Interpreter.AddFunction("Continue",&IPC_CMD_ContinueProcess,
				     "continue\n\t-- contiues the execution of the dataprocess");
     IPC_CMD_Interpreter.AddFunction("GetStatus",  &IPC_CMD_GetStatus,
				     "GetStatus\n\t-- returns the status of the dataprocess");
     IPC_CMD_Interpreter.AddFunction("Initialize",     &IPC_CMD_Initialize,
 				     "initialize [all]\n\t-- initializes the Dataprocess.\
\n\tUse option all to initialize all processes");
     IPC_CMD_Interpreter.AddFunction("Init",     &IPC_CMD_Initialize);
     IPC_CMD_Interpreter.AddFunction("GetDataFileName",&IPC_CMD_GetDataFileName,
				     "GetDataFileName\n\t-- Returns the base file anme of the current run");
     IPC_CMD_Interpreter.AddFunction("switch",    &IPC_CMD_Switch,
				     "switch Histo on|off\n\t-- switches histograming on or off");

     IPC_CMD_Interpreter.AddFunction("GetEvent",    &IPC_CMD_MuonEvent,
				     "GetEvent -- returns the last event in F2000-Format");

     IPC_CMD_Interpreter.AddFunction("Help",    &IPC_CMD_Help,
				     "help [COMMAND]\n\t-- prints help for COMMAND or for all commands");

     IPC_CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     IPC_CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);
     REPLY_Interpreter.AddFunction("cmd_error",&REPLY_Error);
     REPLY_Interpreter.DefaultFunction(&REPLY_Default);
     REPLY_Interpreter.WrongTypeFunction(&REPLY_WrongType);

     CatchLevel=2;
     RIDWarn=false;
     AnalysisVector         =new Daq98AnalysisVector_t(1);
     MuonEventAnalysisVector=new Daq98MuonEventAnalysisVector_t(1);
     
     Suspend=false;
     NoData=false;
     
     ReceivedBytes=0;
     ReceivedEvents=0;
     TimeOut=60000;
     RawData.CreateBuffer(MAX_RAW_DATA_BUFFER_SIZE);
     DataFileName="./daq_test";
     DataDir="./";
     CriticalDiskSize=90;
     RateDeviationTime=60;
//     F2000File="f2000.dat";
//     F2000Writer=new Daq98F2000_t(this);
     MaxRate=135;
     MinRate=125;
     Histo_enabled=false;
//     F2000_enabled=false;
}

DataProcess::~DataProcess(){
     DEB_F("DataProcess::~DataProcess()");
     delete AnalysisVector;
     delete MuonEventAnalysisVector;
}
   
int DataProcess::Init(){
     DEB_F("int DataProcess::Init()");
     // Base class initialization (open IPC port, create SHM, attach
     // to shm, read config file)
     TEST_FAILURE(Daemon::Init());

     DirectoryEntry_t DEntry;
     DataDir.gsub("\"","");
     StringSplitter R(DataDir,':');
     for(int i=0;i<R.NumSplits();i++){
	  StringSplitter U(R()[i],' ');
	  if(U.NumSplits()==2){
	       DEntry.Directory=U()[0];
	       DEntry.Prio=atoi((const char*)U()[1]);
	       DataDirectories.push_back(DEntry);
	  }
	  else {
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataProcess::Init()";
	       Base::Error->SetParam(String(String("Not a valid directory entry: ")
					    +R()[i]));
	       return -1;
	  }
     }
     Streamer.DiskUsage.SetDirectories(&DataDirectories);
     Streamer.SetCriticalSize(CriticalDiskSize);

     AnalysisVector->push_back(new Daq98HWConfigReader_t(this));
     Anal_MuonEventBuilder=new Daq98MuonEventBuilder_t(this);
     AnalysisVector->push_back(Anal_MuonEventBuilder);
     Daq98MuonEventBuilder_t::SetAnalysisVector(MuonEventAnalysisVector);
     EnableHisto(true);
     
//      F2000Writer=new Daq98F2000_t(this);
//      F2000Writer->SetFile(F2000File,ios::trunc);
//     MuonEventAnalysisVector->push_back(F2000Writer);

     VMEDataIPAddress=Dispatcher.AllocIPAddress(LocalHostname);
     return 0;
}

int DataProcess::InitAnalysisVector()
{
     for(Daq98AnalysisVector_t::iterator AnalysisModule=AnalysisVector->begin();
	 AnalysisModule<AnalysisVector->end();
	 AnalysisModule++) {
	  if((*AnalysisModule)->Init()<0)
	       return -1;
     }
     return 0;
}

void DataProcess::EnableHisto(bool h){
     if(h){
	  if(Histo_enabled) return;
	  HistoPointer_not_owning=new Daq98MuonEventHisto_t<SharedMemHistogram_t>(this);
	  MuonEventAnalysisVector->push_back(HistoPointer_not_owning);
	  Histo_enabled=true;
     } else {
	  if(!Histo_enabled) return;
	  int i=0;
	  while((i<MuonEventAnalysisVector->size())&&
		((*MuonEventAnalysisVector)[i]!=HistoPointer_not_owning)) i++;
	  if(i<MuonEventAnalysisVector->size()){
	       MuonEventAnalysisVector->Erase(i);
	  }
	  Histo_enabled=false;
     }
}

void DataProcess::WriteF2000(ostream *stream){
     // ugly i know, but safest method !
     const Daq98MuonEventBuilder_t *MEB=
	  (const Daq98MuonEventBuilder_t *)(const Daq98RawDataAnalysis_t*)Anal_MuonEventBuilder;
     if(!MEB->IsValid()){
	  *stream << "No Valid Event availiable" << endl;
	  return;
     }
     Daq98F2000_t F2000(this,Daq98F2000_t::OUTMODE_SINGLE_EVENT);
     F2000.SetOutputStream(stream);
     F2000.AnalyseData((MuonEventHeader_t&)MEB->GetLastMuonEventHeader(),MEB->GetLastMuonEvent());
}

int DataProcess::WriteF2000(IOSocket &s){
     // ugly i know, but safest method !
     const Daq98MuonEventBuilder_t *MEB=
	  (const Daq98MuonEventBuilder_t *)(const Daq98RawDataAnalysis_t*)Anal_MuonEventBuilder;
     if(!MEB->IsValid()){
	  TEST_FAILURE(s.WriteText("cmd_error\r\nNo Valid Event availiable\r\n"));
	  return 0;
     }
     TEST_FAILURE(s.WriteText("cmd_muonevent\r\n"))
     Daq98F2000_t F2000(this,Daq98F2000_t::OUTMODE_SINGLE_EVENT);
     f2000transport_t f2000trans(&F2000,
				 (MuonEventHeader_t&)MEB->GetLastMuonEventHeader(),
				 MEB->GetLastMuonEvent());
     TransferClass<f2000transport_t> tc(s);
     TEST_FAILURE(tc.Send(f2000trans));
     return 0;
}



int DataProcess::Destroy(){
     DEB_F("int DataProcess::Destroy()");
     Dispatcher.FreeIPAddress(VMEDataIPAddress);
     Streamer.Close();
     for(Daq98AnalysisVector_t::iterator AnalysisModule=AnalysisVector->begin();
	 AnalysisModule<AnalysisVector->end();
	 AnalysisModule++) {
	  if((*AnalysisModule)->Destroy()<0)
	       return -1;
     }
     TEST_FAILURE(Dispatcher.LogAllowTransfer(LOG_TYPE_RUNLOG));
     TEST_FAILURE(Dispatcher.LogAllowTransfer(LOG_TYPE_ERRORLOG));
     return Daemon::Destroy();
}

int DataProcess::OpenDataConnection(IOSocket &S){
     Socket::SocketState_t r;
     r=(Socket::SocketState_t) DataSocket.GetState();
     if(r==Socket::SOS_CONNECTED){
	  TEST_FAILURE(S.WriteText("cmd_info\r\nDataconnection between Dataprocess and Datatransport already established\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,
			      "Data connection between Dataprocess and Datatransport already established");
	  return 0;
     }
          
     ServerSocket SS;
     SS.Open(VMEDataIPAddress.Port,1);

     // get IPC port number of Datatransport 
     PortLocker PL(Dispatcher,PROCESS_TYPE_DATATRANSPORT,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nDatatransport process not running\nCan't open data connection\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,
			      "Datatransport process not running\nCan't open data connection");
	  return -1;
     }
     // send the initialize command and hope that VDP will try to
     // connecto to SS
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     
     CS.OutputStream() << "OpenDataConnection "
		       << LocalHostname << " "
		       << VMEDataIPAddress.Port << "\r\n" << flush;
     // wait some time for the connection
     if(SS.IsReadReady(1500)<=0){
	  // timeout
	  TEST_FAILURE(
	       S.WriteText(
		    "cmd_error\r\nDatatransport process does not answer within \
timeout periode\nCan't open data connection\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,
			      "Datatransport process does not answer within timeout period\
\nCan't open data connection");
	  return -1;
     }
     // accept the connection
     filedescriptor_t sd=SS.Accept();
     if(sd<0){
	  TEST_FAILURE(
	       S.WriteText("cmd_error\r\nCan't open data connection to datatransport process\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,
			      "Can't open data connection to datatransport process");
	  return -1;
     }
     // open a socket for the connection and include it into the
     // notifier List for select
     DataSocket.Open(sd);

     // set buffer for socket to a very huge size
     size_t buflen=1000*1024;
     TEST_FAILURE(DataSocket.SetSockOpt(SOL_SOCKET,SO_RCVBUF,&buflen,sizeof(buflen)));

     DataSocketNotifier.Set(DataSocket,IONotifier::READ);
     Selecter.AddNotifier(DataSocketNotifier);
     Dispatcher.LogEntry(LOG_TYPE_RUNLOG,
			 "Data connection between Dataprocess and Datatransport established");
     TEST_FAILURE(S.WriteText("cmd_info\r\nData Connection between Dataprocess and Datatransport established\r\n"));
     return 0;
}

int DataProcess::InitDataTransport(String &Reply, String &Opt){
     // get IPC port number of Datatransport (DT)
     PortLocker PL(Dispatcher,PROCESS_TYPE_DATATRANSPORT,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     
     if(IPAddr.Port<0){
	  Reply="cmd_error\r\nDatatransport not running\nCan't initialize this process\r\n";
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Datatransport not running\nCan't initialize this process");
     
	  return -1;
     }
     // send the initialize command an hope that DT will try to
     // connecto to SS
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     CS.OutputStream() << "init " << (const char*)Opt << "\r\n" << flush;
     // now recieve messages from the vmedataprocess and send them to the client
     TEST_FAILURE(ServReply(CS)); 
	  
     char txt[MAX_TEXT_SIZE];
     Reply="cmd_info\r\n";
     while(CS.InputStream().good()){
 	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=txt+String("\n");
     }
     return HasReplyError();
}

int DataProcess::InitDataCollector(String &Reply, String &Opt){
     // get IPC port number of Datacollector (DCO)
     PortLocker PL(Dispatcher,PROCESS_TYPE_DATACOLLECTOR,USER_DAQ_INTERN);
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port<0){
	  Reply="cmd_error\r\nDatacollector not running\nCan't initialize this process\r\n";
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Datacollector not running\nCan't initialize this process");
     
	  return -1;
     }
     // send the initialize command an hope that DCO will try to
     // connecto to SS
     ClientSocket CS;
     TEST_FAILURE(CS.Open(IPAddr));
     CS.OutputStream() << "init " << (const char *)Opt << "\r\n" << flush;
     // now recieve error messages from the vmedataprocess and send them to the client
     TEST_FAILURE(ServReply(CS)); 
	  
     char txt[MAX_TEXT_SIZE];
     Reply="cmd_info\r\n";
     while(CS.InputStream().good()){
 	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
 	  Reply+=txt+String("\n");
     }
     return HasReplyError();
}

void DataProcess::ArgsHelp(IOSocket &S)
{
     S.OutputStream() << "cmd_info\r\n"
		      << "Dataprocess initialization arguments:\n"
		      << " --all         -a\n\tInitialize all DAQ processes\n\n"
		      << " --filecount   -c <num>\n\tNumber of data files to keep\n"
		      << "\tDefault is keeping all files\n\n"
		      << " --help        -?\n\tDisplay this help screen\n\n"
		      << " --name        -n <filename>\n\tSets base name of date files\n"
		      << "\tThe name can also include a path\n\n"
		      << " --no-rundate\n"
		      << "\texcludes the day of the year from the filename\n\n"
		      << " --no-runnumber\n"
		      << "\texcludes the runnumber from the filename\n\n"
		      << " --size        -s <filesize>\n\tSize of one data file. Example:\n"
		      << "\t\t-s 5000  = 5000 Bytes\n"
		      << "\t\t-s 5000k = 5000 kBytes\n"
		      << "\t\t-s 5000M = 5000 MBytes\n"
		      << "\tdefault size is 50 MBytes\n\n"
		      << " --verbose-console\n\t"
		      << " Console verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << " --verbose-file\n\t"
		      << " Log file verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << "\r\n" << flush;
  
}

int DataProcess::ParseArgs(IOSocket &S, String *param, int numparams)
{
     if(numparams<=1) return 0;
     Dispatcher.LogEntry(LOG_TYPE_RUNLOG,"Parsing initilization arguments");
     TEST_FAILURE(S.WriteText("cmd_info\r\nParsing initilization arguments\r\n"));

     char **argv;
     argv=new char*[numparams + 1];
     bool InitAll=false;
     
     String RunOptString="";
     int i;
     for(i=0;i<numparams;i++){
	  argv[i]=new char[param[i].length()+1];
	  strcpy(argv[i],(const char*)param[i]);
	  if(i>0)RunOptString+=param[i] + " ";
     }
     if(WriteParams(RunOptString)<0){
	  S.OutputStream() << "cmd_error\r\nCan't write run parameter to file: " 
			   << Base::Error->GetErrorMessage(true) << "\r\n";
	  LogMsg(String("Can't write run parameter to file: ")+ Base::Error->GetErrorMessage(true),2,2,(LogType_t)ERROR);
	  return 0;
     }

     argv[i]=NULL;
     int c;
     int ret=0;
     int digit_optind=0;
     optind=0;
     char str[512];
     String DataCollectorOpt="";
     String DataTransportOpt="";
     bool RunNumber=true;
     bool RunDate=true;
     bool RunYear=true;
     
     while (1)
     {
	  int this_option_optind = optind ? optind : 1;
	  int option_index = 0;
	  static struct option long_options[] =
	  {
	       // when you add options, please maintain an 
	       // alphabetical order
               {"all", 0, 0, 'a'},
	       {"auto-restart",no_argument,0,'A'},
//                {"f2000",required_argument, 0, 'f'},
               {"filecount",required_argument, 0, 'c'},
	       {"help",no_argument, 0, '?'},
	       {"histo",no_argument,0,'h'},
               {"name",required_argument, 0, 'n'},
	       {"no-data",no_argument,0,'N'},
	       {"no-dc-init",no_argument,0,305},
	       {"no-runday",no_argument,0,303},
	       {"no-runnumber",no_argument,0,304},
	       {"no-runyear",no_argument,0,306},
               {"size",required_argument, 0, 's'},
               {"super-nova",optional_argument, 0, 'S'},
               {"run-comment",required_argument, 0, 300},
	       {"run-time",required_argument,0,'t'},
               {"verbose-console",required_argument, 0, 301},
               {"verbose-file",required_argument, 0, 302},
               {0, 0, 0, 0}
	  };

	  c = getopt_long (numparams, argv, "an:s:S::c:?t:AN",
			   long_options, &option_index);
	  if (c == -1)
	       break;
	  
	  switch (c)
	  {
	  case 0:
	       S.OutputStream() << "cmd_info\r\n"
				<< "option " 
				<< long_options[option_index].name;
	       if (optarg)
		    S.OutputStream() << " with arg " 
				     << optarg;
	       S.OutputStream() << "\r\n" << flush;
	       break;
	       
	  case 300:
	       DataCollectorOpt+=String("--run-comment ");
	       DataCollectorOpt+=String((const char *)optarg);
	       
	       break;
	  case 301:
	       DataCollectorOpt+=String(" --verbose-console ");
	       DataCollectorOpt+=String((const char *)optarg);
	       DataTransportOpt+=String(" --verbose-console ");
	       DataTransportOpt+=String((const char *)optarg);
	       SetVerboseLevel(atoi(optarg),GetConsoleVerboseLevel());
	       sprintf(str,"Console verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,2);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	  case 302:
	       DataCollectorOpt+=String(" --verbose-file ");
	       DataCollectorOpt+=String((const char *)optarg);
	       DataTransportOpt+=String(" --verbose-file ");
	       DataTransportOpt+=String((const char *)optarg);
	       SetVerboseLevel(atoi(optarg),GetConsoleVerboseLevel());
	       sprintf(str,"File verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,2);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	  case 303:
	       RunDate=false;
	       
	       break;
	  case 304:
	       RunNumber=false;
	       
	       break;
	  case 305:
	       DataCollectorOpt+=" --no-dc-init ";
	       break;

	  case 306:
	       RunYear=false;
	       break;
	       
	  case '0':
	  case '1':
	  case '2':
	       if (digit_optind != 0 && digit_optind != this_option_optind)
		    printf ("digits occur in two different argv-elements.\n");
	       digit_optind = this_option_optind;
	       printf ("option %c\n", c);
	       break;
	  case 'A':
	       // Auto start
	       DataCollectorOpt+=" --auto-restart";
	       break;
	       
	  case 'a':
	       InitAll=true;
	       break;
	       
	  case 'h':
	       // histogramming
	       EnableHisto(true);
	       break;

	  case 'N':
	       NoData=true;
	       break;
	       
	  case 'n':
	       DataFileName=(char*)optarg;
	       break;

	  case 'c':
	  {
	       int num=atoi(optarg);
	       if(num<2){
		    sprintf(str,"Datastreamer setup: Keeping just %c file doesn't make sense.\n\
Value set to 2 instead.",num);
		    Dispatcher.LogEntry(LOG_TYPE_RUNLOG,str);
		    S.OutputStream() << "cmd_info\r\n"
				     << str << "\r\n" << flush;
		    num=2;
	       }
	       Streamer.SetLimit(DataStreamer::KEEPFIRST,num);
	       sprintf(str,"Datastreamer setup: Keeping %d files, including \
first one",atoi(optarg));
	       Dispatcher.LogEntry(LOG_TYPE_RUNLOG,str);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       break;
	  }
// 	  case 'f':
// 	       F2000File=(const char*)optarg;
// 	       EnableF2000(true);
// //	       F2000Writer->SetFile((const char*)optarg,ios::trunc);
// 	       break;
	       
	  case '?':
	       ArgsHelp(S);
	       break;

	  case 's':
	  {
	       u_i64 size;
	       String OptArg=optarg;
	       Regex Units("[kKmMgG]");
	       String Number;
	       if(OptArg.contains(Units,1))
		    Number=OptArg.before(Units);
	       else 
		    Number=OptArg;
	       char u;
	       int index;
	       if((index=OptArg.index(Units))>=0) {
		    u=OptArg[index];
		    switch (u=toupper(u)){
		    case 'K':
			 // kBytes
			 size=1024;
			 break;
		    case 'M':
			 // MBytes
			 size=1024 * 1024;
			 break;
		    case 'G':
			 // GBytes
			 size=1024 * 1024 * 1024;
			 break;
		    default:
			 size=1;
		    }
		    size*=atoi((const char *)Number);
		    sprintf(str,"Datastreamer setup: File size set to %d %cBytes (%d Bytes)",
			    atoi((const char *)Number),u,(int)size);
	       }
	       else {
		    size=atoi((const char *)Number);
		    sprintf(str,"Datastreamer setup: File size set to %d Bytes",
			    (int)size);
	       }
	       Streamer.SetSize(size);
	       Dispatcher.LogEntry(LOG_TYPE_RUNLOG,str);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       break;
	  }
	  case 'S':
	  {
	       if(optarg){
		    if(((const Daq98MuonEventBuilder_t *)(const Daq98RawDataAnalysis_t*)Anal_MuonEventBuilder)->SetSNFile((const char*)optarg)<0)
			 goto CLEAN_EXIT;
	       }else{
		    ((const Daq98MuonEventBuilder_t *)(const Daq98RawDataAnalysis_t*)Anal_MuonEventBuilder)->SetSNStreamer(&cout);
	       }
	       break;
	  }
	  case 't':
	  {
	       DataCollectorOpt+=String(" --run-time ")+(const char*)optarg+" ";
	       break;
	  }
	       
	  default:
	       sprintf(str,"Getopt returned character code 0%o",c);
	       S.OutputStream() << "cmd_error\r\n" 
				<< str
				<< "\r\n" << flush;
	       Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,str);
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="DataProcess::ParseArgs(..)";
	       Base::Error->SetParam(str);

	       ret=-1;
	       goto CLEAN_EXIT;
	  }
     }
     if (optind < numparams)
     {
	  ostrstream err_s;
	  err_s << "non-option ARGV-elements: ";
	  while (optind < numparams)
	       err_s << argv[optind++] << " ";
	  err_s << ends;
	  err_s.freeze(1);	  
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,String(err_s.str()));
	  S.OutputStream() << "cmd_error\r\n" << err_s.str() << "\r\n";
	  
	  err_s.freeze(0);
	  ret=-1;
	  goto CLEAN_EXIT;
     }
     {
	  int flags=0;
	  if(RunNumber) flags=RunFileName_t::RUNNUMBER;
	  if(RunDate) flags=flags|RunFileName_t::RUNDATE;
	  if(RunYear) flags=flags|RunFileName_t::RUNYEAR;
	  RunFileName_t RunName(this);
	  String RunNumberStr;
	  if(RunName.GetName(RunNameStr)<0){
	       ret--;
	       ostrstream err_s;
	       err_s << (*Base::Error).GetErrorMessage() << ends;
	       err_s.freeze(1);	  
	       Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,String(err_s.str()));
	       S.OutputStream() << "cmd_error\r\n" << err_s.str() << "\r\n";
	       err_s.freeze(0);
	       goto CLEAN_EXIT;
	  }
	  else if(RunName.GetName(RunNumberStr,RunFileName_t::RUNNUMBER)<0){
	       ret--;
	       ostrstream err_s;
	       err_s << (*Base::Error).GetErrorMessage() << ends;
	       err_s.freeze(1);	  
	       Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,String(err_s.str()));
	       S.OutputStream() << "cmd_error\r\n" << err_s.str() << "\r\n";
	       err_s.freeze(0);
	       goto CLEAN_EXIT;
	  }
	  else {
	       DataCollectorOpt+=String(" --runnumber-intern=");
	       DataCollectorOpt+=RunNumberStr;
	       DataFileName+="_";
	       DataFileName+=RunNameStr;
	       DataCollectorOpt+=String(" --runname-intern=");
	       DataCollectorOpt+=DataFileName;
	  }
	  sprintf(str,"Data filename set to '%s'.",(const char*)DataFileName);
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,str);
	  S.OutputStream() << "cmd_info\r\n"
			   << str << "\r\n" << flush;
	  Streamer.FileNameFactory.SetBaseFileName(DataFileName);
     }
	       
     if(InitAll) {
	  String Reply;
	  if(InitDataTransport(Reply, DataTransportOpt)<0) {
	       ret=-1;
	       S.OutputStream() << Reply << flush;
	       goto CLEAN_EXIT;
	       
	  }
	  S.OutputStream() << Reply << flush;
	  
	  if(InitDataCollector(Reply, DataCollectorOpt)<0) {
	       ret=-1;
	       	  S.OutputStream() << Reply << flush;
		  goto CLEAN_EXIT;
	  }
	  S.OutputStream() << Reply << flush;
     }
 CLEAN_EXIT:
     
     for(i=0;i<=numparams;i++){
	  delete [] argv[i];
	  argv[i]=NULL;
     }
     
     delete [] argv;

     return ret;
}

int DataProcess::WriteParams(String &Param){
     ofstream outfile(RUN_PARAM_FILENAME);
     if(!outfile){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)="opening '" + String(RUN_PARAM_FILENAME) + "': ";
	  return -1;
     }
     if(!Param.contains("--no-dc-init"))Param+=" --no-dc-init";

     outfile << "# Run parameter for auto-restart" << endl
	     << "# Do not delete or change manually!" << endl
	     << "section " << RUN_PARAM_SECTION_NAME << endl
	     << "RunParam " << Param << endl;
     return 0;
}

int DataProcess::ReadDetectorConfigFromBufferAscii(BinaryDataBuffer &Buf){
     return HWConfigAscii.ReadFromBuffer(Buf);
}

int DataProcess::DispatchEvent(){
     DaemonBase::DispatchEvent();     // process IPC commands -> calls DoIPCEvent 
     if(Selecter.CheckNotifier(DataSocketNotifier)){
	  TEST_FAILURE(ServVMEDataSocket());
     }     

     return 0;
}
     
int DataProcess::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     DEB_F("int DataProcess::DoIpCEvent(..)");
     return IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

const String DataProcess::GetConfigFileName()const{
     return DATAPROCESS_CNF_FILENAME; // from globalmacros.h
}

const String DataProcess::GetProcessType()const{
     return PROCESS_TYPE_DATAPROCESS; // from globalmacros.h
}

int DataProcess::ServVMEDataSocket(){
     DEB_F("int DataProcess::ServVMEData()");
     static u_i32 CurFileSize=0;
     static char Msg[MAX_TEXT_SIZE];
     static time_t PrevTime=0;
     static bool first=true;
     
     if(first){
	  // time for performance
	  Time_Start=my_times();
	  Time_LastPerfCalc=Time_Start; //avoid first messages
	  first=false;
     }
     
     if(DataSocket.ReadBinI(RawData)<0){
	  LogMsg("Couldn't read from socket stream",1,1,(LogType_t)ERROR);
	  return -1;
     }
     
     ReceivedBytes+=RawData.GetSizeOfData();
     Daq98AnalysisVector_t::iterator AnalysisModule;
     int r;
     RecordHeader_t RecordHeader;
     CurFileSize+=RawData.GetSizeOfData();
//     cerr << CurFileSize << endl;
     if(!NoData) TEST_FAILURE(Streamer.Write((char *)RawData.GetDataBuffer(),RawData.GetSizeOfData()));
     
     while(!RawData.end_of_read()){
	  ReadNextRecordHeader(RecordHeader,RawData);	  
	  
  	  for(AnalysisModule=AnalysisVector->begin();AnalysisModule<AnalysisVector->end();
  	      AnalysisModule++) {
  	       if((r=(*AnalysisModule)->AnalyseData(RecordHeader,RawData))==-1)
  		    return -1;
  	  }

	  if(RecordHeader.RecordID==RID_RUNINFO){
	       LogMsg("Run info received",10,10);
	       Streamer.SetRunInfoRecord(RecordHeader,RawData);
	  }
	  else if(RecordHeader.RecordID==RID_HWCONFIG_ASCII){
	       // read the detector configuration
	       Streamer.SetConfigRecord(RecordHeader,RawData);
	  }else{
	       if(RecordHeader.RecordID==RID_MUONEVENT){
		    ReceivedEvents++;
	       }
	       // go to the next record     
	       SkipRecord(RecordHeader,RawData);
	  }
     }
     
     
     // performace timing calculations all 20 seconds
     clock_t Time_Current=my_times();
     int div=(Time_Current-Time_LastPerfCalc);
     int clk=CLK_TCK;
     
     if((div/clk)>20){
	  static unsigned long long lastbytes,lastevents;
	  float event_perf=
	       (ReceivedEvents-lastevents)/
	       ((float)(Time_Current-Time_LastPerfCalc)/CLK_TCK);
	  float bytes_perf=
	       (ReceivedBytes-lastbytes)/
	       (1024*(float)(Time_Current-Time_LastPerfCalc)/CLK_TCK);
	  
	  Time_LastPerfCalc=Time_Current;
	  lastbytes=ReceivedBytes;
	  lastevents=ReceivedEvents;
	  sprintf(Msg,"cmd_message\r\n%.2f kB , Rates: %.2f kB/sec / %.2f Events/sec\r\n",
		  ReceivedBytes/1024.0,
		  bytes_perf,
		  event_perf);
	  Dispatcher.Broadcast(PROCESS_TYPE_COMMUNICATIONSERVER,Msg);
	  if(bytes_perf< MinRate){
	       if((Time_Current-Time_Start)/CLK_TCK > 40){
		    sprintf(Msg,"Data rate (%.2f kB/sec) is less than the limit of %d kB/sec",
			    bytes_perf,MinRate);
		    LogMsg(Msg,2,2,WARNING);
	       }
	  }
	  else if(bytes_perf>MaxRate){
	       if((Time_Current-Time_Start)/CLK_TCK > 40){
		    sprintf(Msg,"Data rate (%.2f kB/sec) exceeds the limit of %d kB/sec",
			    bytes_perf,MaxRate);
		    LogMsg(Msg,2,2,WARNING);
	       }
	  }
     }
     return 0;
}

int DataProcess::SetTimeOut(int timeout)
{
     int tmp=TimeOut;
     TimeOut=timeout;
     return tmp;
}

int DataProcess::GetTimeOut()
{
     return TimeOut;
}

int DataProcess::ServReply(IOSocket &S){
     DEB_F("int DataProcess::ServReply()");
     const char *txt;
     SetReplyStatus(SUCCESS);
     int r;
     if((r=S.IsReadReady(TimeOut))<=0){
	  if(r==0){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataProcess::ServReply(IOSocket &S)";
	       char str[200];
	       sprintf(str,"Connection timed out (%i sec).",TimeOut/1000);
	       Base::Error->SetParam(str);
	  }
     	  return -1;
     }
          
     TEST_FAILURE(0==(txt=S.ReadText())?-1:0);
     if(REPLY_Interpreter.Do_wo_type(this,txt,S)<0) 
	  return -1;
     return 0;
} 

int DataProcess::REPLY_Error(IOSocket &/*s*/,String * /*params*/,int /*numparams*/){
     DEB_F("int DataProcess::REPLY_Error(..)");
     SetReplyStatus(ERROR);

     return 0;
}

int DataProcess::REPLY_Default(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int DataProcess::REPLY_CmdDefault(..)");
     SetReplyStatus(SUCCESS);

     return 0;
}


int DataProcess::REPLY_WrongType(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
      DEB_F("int DataProcess::REPLY_WrongType(..)");

      return 0;
}

int DataProcess::IPC_CMD_Quit(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_Quit(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\nBye Bye\r\n"));
     Quit_FLAG=true;
     return 0;
}

int DataProcess::IPC_CMD_SuspendProcess(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_SuspendProcess(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\nprocess suspended\r\n"));
     Selecter.RemoveNotifier(DataSocketNotifier);
     Suspend=true;
     return 0;
}

int DataProcess::IPC_CMD_ContinueProcess(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_ContinueProcess(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\nprocess continued\r\n"));
     Selecter.AddNotifier(DataSocketNotifier);
     Suspend=false;
     return 0;
}

int DataProcess::IPC_CMD_GetDataFileName(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_GetDataFileName(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\n"));
     String RunName=Streamer.FileNameFactory.GetDirectory() + 
	  Streamer.FileNameFactory.GetBaseFileName() + 
	  String("\r\n");
     TEST_FAILURE(S.WriteText(RunName));
     return 0;
}

int DataProcess::IPC_CMD_GetStatus(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_GetStatus(..)");
     String Status;
     Dispatcher.GetStatus(Status);
     TEST_FAILURE(S.WriteText("cmd_info\r\n"));
     TEST_FAILURE(S.WriteText(Status));
     TEST_FAILURE(S.WriteText("\r\n"));
     return 0;
}

int DataProcess::IPC_CMD_Help(IOSocket &s,String *params,int numparams){
     DEB_F("int DataProcess::IPC_CMD_Help(..)");
     String Help;
     if(numparams>1)
	  IPC_CMD_Interpreter.GetHelp(params[1],Help);
     else
	  IPC_CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));
     return 0;
}

int DataProcess::IPC_CMD_UnknownCommand(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int DataProcess::IPC_CMD_UnknownCommand(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\nDatatprocess: Unknown Command\r\n"));
     return 0;
}

int DataProcess::IPC_CMD_WrongTypeCommand(IOSocket &S,String */*param*/,int){
     DEB_F("int DataProcess::IPC_CMD_IgnoreCommand(..)");
     TEST_FAILURE(S.WriteText("cmd_info\r\nDataprocess: Wrong Type\r\n"));
     return 0;
}

int DataProcess::CheckDataConnection(){
     TEST_FAILURE(DataSocket.IsReadReady(5000));
     return 0;
}

int DataProcess::IPC_CMD_Switch(IOSocket &S,String *param,int numparams){
//     param[1] histo/f2000
//     param[2] on/off
     if(numparams==3){
	  if(compare(param[1],"histo")==0){
	       if(compare(param[2],"on")==0){
		    EnableHisto(true);
		    TEST_FAILURE(S.WriteText("cmd_info\r\nHistograms on\r\n"));
		    return 0;
	       } else if(compare(param[2],"off")==0){
		    EnableHisto(false);
		    TEST_FAILURE(S.WriteText("cmd_info\r\nHistograms off\r\n"));
		    return 0;
	       }
	  }
// 	  else if(compare(param[1],"f2000")==0){
// 	       if(compare(param[2],"on")==0){
// 		    EnableF2000(true);
// 		    TEST_FAILURE(S.WriteText("cmd_info\r\nF2000 on\r\n"));
// 		    return 0;
// 	       } else if(compare(param[2],"off")==0){
// 		    EnableF2000(false);
// 		    TEST_FAILURE(S.WriteText("cmd_info\r\nF2000 off\r\n"));
// 		    return 0;
// 	       }
// 	  }
     }
     TEST_FAILURE(S.WriteText("cmd_info\r\nDon't understand: nothing changed\r\n"));
     return 0;
}

int DataProcess::IPC_CMD_MuonEvent(IOSocket &S,String *param,int numparams){
     return WriteF2000(S);
}

int DataProcess::IPC_CMD_Initialize(IOSocket &S,String *param,int numparams){
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_RUNLOG,String(FILENAME_RUNLOG)+
				     DO_NOT_TRANSFER));
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_ERRORLOG,String(FILENAME_ERRORLOG)+
				     DO_NOT_TRANSFER));
     
     // first open a connection to the VME datatransport process
     int err=0;
     TEST_FAILURE(S.WriteText("cmd_info\r\nInitializing Dataprocess\r\n"));
     Dispatcher.LogEntry(LOG_TYPE_RUNLOG,"Initializing Dataprocess");
     if(OpenDataConnection(S)<0){
	  err-=1;
     }
     
     if(ParseArgs(S,param,numparams)<0){
	  err-=1;
     }
     String Dir;
     if(Streamer.DiskUsage.GetNextFreeDir(200000,Dir)){
	  err-=1;
	  S.OutputStream() << "cmd_error\r\n" << " *** Error **** No Free Directory found for Datafiles" << "\r\n" << flush;
     }
     String file_name=FILENAME_RUNLOG;
     file_name.gsub(".log","");
     String RunLog=Dir + file_name +"_"+RunNameStr+LOG_EXTENSION+DO_NOT_TRANSFER;
     if((Dispatcher.LogMove(LOG_TYPE_RUNLOG,RunLog))<0){
	  err-=1;
	  S.OutputStream() << "cmd_error\r\n" << Base::Error->GetErrorMessage() << "\r\n" << flush;
     }
     else{
	  LogMsg(String("Run log kept in '"+RunLog+"'"),3,0);
     }
     file_name=FILENAME_ERRORLOG;
     file_name.gsub(".log","");
     String ErrorLog=Dir+file_name+"_"+RunNameStr+LOG_EXTENSION+DO_NOT_TRANSFER;
     if((Dispatcher.LogMove(LOG_TYPE_ERRORLOG,ErrorLog))<0){
	  err-=1;
	  S.OutputStream() << "cmd_error\r\n" << Base::Error->GetErrorMessage() << "\r\n" << flush;
     }
     else{
	  LogMsg(String("Error log kept in '"+ErrorLog+"'"),3,0);
     }
//      if((F2000Writer->Init())<0){
// 	  err-=1;
// 	  S.OutputStream() << "cmd_error\r\n" << Base::Error->GetErrorMessage() << "\r\n" << flush;
//      }
     if(DataDirectories.size()==1){
	  S.OutputStream() << "cmd_info\r\n" << "Data directory is:\n";
     }
     else {
	  S.OutputStream() << "cmd_info\r\n" << "Data directories are:\n";
     }
     DirectoryEntryVector_t::iterator i;
     for(i=DataDirectories.begin();i<DataDirectories.end();i++){
	  S.OutputStream() << (*i).Directory << " (" << (*i).Prio << ")\n";
     }
     S.OutputStream() << "\r\n" << flush;
     TEST_FAILURE(InitAnalysisVector());
     
     if(err==0) {
	  TEST_FAILURE(S.WriteText("cmd_info\r\nDataprocess successfully initialized\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,"Dataprocess successfully initialized");
     }
     else {
	  TEST_FAILURE(S.WriteText("cmd_error\r\nDataprocess not initialized\r\n"));
	  Dispatcher.LogEntry(LOG_TYPE_ERRORLOG,"Dataprocess not initialized");
     }
     // Enable Histos
     EnableHisto(true);
     return 0;
}

   
int DataProcess::SendUsageInfo(IOSocket &s,String param){
     DEB_F("int RunServer::SendUsageInfo(..)");
     String Help;
     IPC_CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     TEST_FAILURE(s.WriteText("cmd_infor\r\n"));
     return s.WriteText(Help);
}

ClassWithTypeInfo_t *DataProcess::GetConfigSection(){
     return new class DataProcessConfig:public ConfigSection_t
     {
     public:
	  /** Constructor */
	  DataProcessConfig(int &CriticalDiskSize,String &DataDir,
			    /*String &F2000File,*/ int &MaxRate, int &MinRate,
			    int &RateDeviationTime)
	       :ConfigSection_t(),
		CriticalDiskSize_cnf(CriticalDiskSize),
		DataDir_cnf(DataDir),
//		F2000File_cnf(F2000File),
		MaxRate_cnf(MaxRate),
		MinRate_cnf(MinRate),
		RateDeviationTime_cnf(RateDeviationTime){};
	  BeginItems(ConfigSection_t){
	       BeginActions{
	       }EndActions;
	       BeginLinks{
		    ItemLink1("CriticalDiskSize",CriticalDiskSize_cnf);
		    ItemLink1("DataDir",DataDir_cnf);
//		    ItemLink1("F2000File",F2000File_cnf);
		    ItemLink1("MaxDataRate",MaxRate_cnf);
		    ItemLink1("MinDataRate",MinRate_cnf);
		    ItemLink1("RateDeviationTime",RateDeviationTime_cnf);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(DATAPROCESS_SECTION_NAME);
     private:
	  int &CriticalDiskSize_cnf;
	  String &DataDir_cnf;
//	  String &F2000File_cnf;
	  int &MaxRate_cnf;
	  int &MinRate_cnf;
	  int RateDeviationTime_cnf;
     }(CriticalDiskSize,DataDir,MaxRate, MinRate,
       RateDeviationTime);
}
	
int main(int /*argc*/,char **/*argv*/){
     DEB_F("int main(int argc,char *argv[])");
     DaqProcessBase::InitializeGlobals("dataprocess");

     // for program error logging
     LogFile LF(DaqProcessBase::ThisProcess->GetName());					
     Logger::SetLogger(&LF);				
     
     // Lock the process, 
     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }
     // create the daemon object
     DataProcess DPServer;
     // initialize the daemon, allocate recources
     if(DPServer.Init()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     }else{
	  // signal parent process, that the process was started
	  // successfully 
	  kill(getppid(),SIGUSR1);
	  // enter the event loop
	  if(DPServer.EventLoop()<0){
	       Logger Entry;
	       (*Base::Error).Print(Entry());
	       DPServer.LogMsg((*Base::Error).GetErrorMessage(true),1,1,ERROR);
	  }
     }
     // destroy the demon, free all recources
     if(DPServer.Destroy()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     }
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0; 
}
