/*
  $Header: /home/elrond/baikal/src/CVS/daq98/dataprocess.h,v 1.7 1999/02/05 18:35:03 steffen Exp $
*/

#if ! defined __DATAPROCESS_H__
#define __DATAPROCESS_H__

#include <config.h>
#include <strstream.h>
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "memhistogram.h"
#include "event.h"
#include "cnfreader.h"
#include "datastreamer.h"
#include "daemon.h"
#include "vmedaqhal.h"

#include "daq98rawdataanalysis.h"
#include "daq98muoneventana.h"
#include "daq98muoneventhisto.h"
#include "daq98f2000.h"

#define DATA_FILE_SIZE 51200000L // 50 MB per File!

#define MAX_HIST_DPROC  2008


class DataProcess;

class DataProcess_SignalHandler:public SignalHandlerClass
{
public:
     virtual int Handle_SIGALRM(Process &P);
};

class DataProcess:public Daemon
{
     enum {INITIAL_RECV_BLOCK_SIZE= 100*1024}; // 100kb
     
public:
     DataProcess();
     virtual ~DataProcess();
   
     int Init();
     int Destroy();
     
     /** Does some sanity checks on a record.
	 @param RecordHeader Record header to check on.
	 @param instream Input data stream.
	 @param CurPos Current Psition in data buffer.
	 @return 0 on success, -1 on failure. */
     int DoSanityCheck(RecordHeader_t &RecordHeader, istream *instreaam,
		       size_t & CurPos);
     
     /** Initialyses analysis vector 
       @return 0 on success, -1 on failure. */
     int InitAnalysisVector();

     /** Synchronizes on corrupt files.
       @param instream Input stream to synchronize on. 
       @return Number of bytes skipped if catch was successful, -1 otherwise.*/
     u_i32 Catch(istream *instream);

     /** Reads next record from the raw data stream.
	 @param RecordHeader Record header structure. It holds the record ID 
	 and the number of bytes in the record on successful return. These
	 values are undefined on failure.
	 @return Number of bytes read on success or -1 on failure. */
     long ReadNextRecordFromStream(istream *instream, RecordHeader_t &RecordHeader);

     int OpenDataConnection(IOSocket &S);
     int CheckDataConnection();

     int InitDataCollector(String &Reply, String &Opt);
     int InitDataTransport(String &Reply, String &Opt);

     int ServVMEDataSocket();
     /** Set timeout for command reply.
	 @param timeout Timeout value in Milliseconds.
	 @return old timeout value */
     int SetTimeOut(int timeout);
     /** Gets timeout value for command replies.
	 @return timeout value */
     int GetTimeOut();
     
     virtual int DispatchEvent();
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** Sets up the data structure for the Hardware, according to the
         configuration from the Config Tree */
     int ReadDetectorConfigFromBufferAscii(BinaryDataBuffer &Buf);
     virtual const String GetProcessType()const;

protected:
     enum ReplyStatus_t 
     {
	  ERROR,
	  SUCCESS,
	  UNKNOWN
     };
     ReplyStatus_t ReplyStatus;

     /** returns the size fo the shared memory, if the flag
         D_CREATE_SHM was set */
     virtual unsigned int GetSHMSize(){return 8*1024*1024;}
     virtual unsigned int GetSHMSegments(){return 8;}
     virtual const String GetConfigFileName()const;
     /** returns xa pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ClassWithTypeInfo_t *GetConfigSection();
     int BuildEvents();

     /** Parses arguments during initialization */
     int ParseArgs(IOSocket &S, String *param, int numparams);
     
     /** Prints a help message for all arguments */
     void ArgsHelp(IOSocket &S);

     /** Checks Reply Status of a Command.*/
     int ServReply(IOSocket &S);

     /** Sets reply status.
	 @param r Reply status */
     inline void SetReplyStatus(ReplyStatus_t r){
	  ReplyStatus=r;
     }

     /** Gets current reply status.
	 @return Current reply status. */
     ReplyStatus_t GetReplyStatus(){
	  return ReplyStatus;
     }
     
          /** Checks if there was an reply error.
	 @return {\tt 0} if {\tt ReplyStatus} is set to {\tt SUCCESS} or 
	 or {\tt -1} otherwise. */
     int HasReplyError(){
	  if(ReplyStatus!=SUCCESS) return -1;
	  return -0;
     }     
          
private:
     void EnableHisto(bool h);
     void WriteF2000(ostream *stream);
     int WriteF2000(IOSocket &S);

     /** The Buffer for storing the received raw data */
     BinaryDataBuffer RawData;
     /** the number of received Bytes */
     unsigned long long ReceivedBytes;
     /** the number of received Events */
     unsigned long long ReceivedEvents;
     /** the Time when the process was started */
     clock_t Time_Start;
     /** the time when the last performance calculation was done */
     clock_t Time_LastPerfCalc;

     DataStreamer Streamer;
     bool Suspend;

//     Daq98MuonEventHisto_t *MuonEventHisto;
     IPAddress_t VMEDataIPAddress;

     SlaveServerSocket DataSocket;
     SocketNotifier DataSocketNotifier;

     Interpreter<DataProcess> IPC_CMD_Interpreter;
     Interpreter<DataProcess> REPLY_Interpreter;
     int IPC_CMD_Quit(IOSocket &S,String *param,int numparams);
     int IPC_CMD_SuspendProcess(IOSocket &S,String *param,int numparams);
     int IPC_CMD_ContinueProcess(IOSocket &S,String *param,int numparams);
     int IPC_CMD_GetStatus(IOSocket &S,String *param,int numparams);
     int IPC_CMD_VMECommand(IOSocket &S,String *param,int numparams);
     int IPC_CMD_GetDataFileName(IOSocket &S,String *param,int numparams);
     int IPC_CMD_Help(IOSocket &s,String *params,int numparams);
     int IPC_CMD_UnknownCommand(IOSocket &S,String *param,int numparams);
     int IPC_CMD_WrongTypeCommand(IOSocket &S,String *param,int numparams);     
     int IPC_CMD_Initialize(IOSocket &S,String *param,int numparams);
     int IPC_CMD_Switch(IOSocket &S,String *param,int numparams);
     int IPC_CMD_MuonEvent(IOSocket &S,String *param,int numparams);
     int REPLY_Error(IOSocket &s,String *params,int numparams);
     int REPLY_Default(IOSocket &s,String *params,int numparams); 
     int REPLY_WrongType(IOSocket &s,String *params,int numparams);

     int SendUsageInfo(IOSocket &s,String param);
     WriteParams(String &Param);

protected:
     int TimeOut;
     /** The vector that holds the configuration for the PMT channels */
     ChannelConfigRecordVector_t ChannelConfig;
     /** The vector that holds the configuration for the crates */
     CrateConfigRecordVector_t   CrateConfig;  
     /** The vector that holds the configuration for the ADCs */
     ChannelModuleConfigRecordVector_t  ModuleConfig;
     /** Name of the datafile */
     String DataFileName;
     /** Run name */
     String  RunNameStr;
     /** Data directories */
     String DataDir;
     /** Start time of a run */
     time_t RunStartTime;
     /** Vector of possible data directories */
     DirectoryEntryVector_t DataDirectories;
     /** Critical disk size.
	 Programm issues a warning if more than {\tt \Ref{CriticalDiskSize}}
	 is used in all file systems that are used to write data */
     int CriticalDiskSize;
     /** No data is written to file if true */
     bool NoData;

     /** Maximum data rate.
	 If this rate is exceeded a warning will be given */
     int MaxRate;
     
     /** Minimum data rate
	 If the data rate is less tha this number a warning
	 will be given */
     int MinRate;

     /** Data rate deviation time frame.
	 If the data rate deviates from its limits more than once 
	 within this given time frame a warning will be issued. */
     int RateDeviationTime;
     
     /** The number of bytes read */
     unsigned long long BytesRead;

     /** The number of events read */
     unsigned long long EventsRead;

     /** ref-count pointer to the event building analysis object */
     Daq98RawDataAnalysis_p Anal_MuonEventBuilder;
     
     /** Vector that contains all raw data analysis objects */
     Daq98AnalysisVector_t *AnalysisVector;
     
     /** Vector that contains all raw data analysis objects */
     Daq98MuonEventAnalysisVector_t *MuonEventAnalysisVector;
     /** Catch level.
	 Controls the handling of corrupt files */
     int CatchLevel;
     
     /** Unknown record ID warning flag.
	 If true a warning will be issued if an unknown record ID is
	 encountered */
     bool RIDWarn;

     DataProcess_SignalHandler SignalHandler;
     friend DataProcess_SignalHandler;
private:
     HWConfigAscii_t HWConfigAscii;
     EventBuilder_t MEventBuilder;
     Daq98MuonEventHisto_t<SharedMemHistogram_t> *HistoPointer_not_owning;
     bool Histo_enabled;
};
#endif //  __DATAPROCESS_H__

