/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datastreamer.C,v 1.6 1999/02/04 09:18:34 steffen Exp $
*/

#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "datastreamer.h"
#include <errno.h>

DataStreamer::DataStreamer(DaqProcessBase *parent,Mode_t mode, long maxsize)
{
     Parent=parent;
     LFlags=NOLIMIT;
     Limit=-1;
     Mode=mode;
     MaxSize=maxsize;
     CurSize=MaxSize+1;
     OldFilename="";
     FileNameFactory.SetBaseFileName("testrun");
     CriticalSize=95;
}

void DataStreamer::SetSize(long maxsize)
{
     MaxSize=maxsize;
}

void DataStreamer::SetCriticalSize(int Size)
{
     CriticalSize=Size;
}

int DataStreamer::SetLimit(Limit_t flags, int lim)
{
     // only one option so far!
     if(!(flags==KEEPFIRST || flags ==NOLIMIT)){
	  (*Base::Error)=UNKNOWN_FLAG;
	  char tmp[20];
	  sprintf(tmp,"%i",flags);
	  Base::Error->SetParam(tmp);
	  (*Base::Error)="DataStreamer::SetLimit(..)";
	  return -1;
     }
     LFlags=flags;
     Limit=lim;
     Lock=true;
     return 0;
}

int DataStreamer::Close()
{
     File.close();
     if(rename((const char *)String(OldFilename+String(".do_not_transfer")),(const char*)OldFilename)<0){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)="rename";
	  return -1;
     }
     CurSize=MaxSize+1;
     OldFilename="";
     	  
     return 0;
}
	  
int DataStreamer::Write(char *data, long size)
{
     static String Filename;
     if((CurSize + size)>MaxSize){
	  if(Mode==STRICT){
	       long size1=MaxSize-CurSize;
	       File.write(data,size1);
	       CurSize+=size1;
	       File.close();
 	       if(OldFilename!=""){
 		    if(rename((const char *)String(OldFilename+String(".do_not_transfer")),(const char*)OldFilename)<0){
 			 (*Base::Error)=SYS_ERROR;
 			 (*Base::Error)="rename";
 			 return -1;
 		    }
 	       }
	       String Directory;
	       // cerr << "W: " << MaxSize/1024 << endl;
	       if(DiskUsage.GetNextFreeDir(MaxSize/1024,Directory)<0){
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="int DataStreamer::Write(..)";
		    char Msg[MAX_TEXT_SIZE];
		    sprintf(Msg,"Not enough disk space available (%Ld)",
			    MaxSize) ;
		    Base::Error->SetParam(Msg);
		    return -1;
	       }
	       if(DiskUsage.IsCritical(CriticalSize)){
		    char msg[512];
		    sprintf(msg,"Disk space below critical size (%d%%)",
			    CriticalSize);
		    Parent->LogMsg(msg,2,2,WARNING);
	       }
	       FileNameFactory.SetDirectory(Directory);
	       String Filename=FileNameFactory.GetNextFileName();
	       OldFilename=Filename;
	       File.open(Filename + String(".do_not_transfer"));
	       if(!File){
		    (*Base::Error)=SYS_ERROR;
		    (*Base::Error)="DataStreamer::Write(..): open '"
			 + Filename +String(".do_not_transfer")+"':";
		    
		    return -1;
	       }
	       CheckForLimits(FileNameFactory.GetFileCount());
	       File.write(RunInfoBuffer.GetDataBuffer(),RunInfoBuffer.GetSizeOfData());
	       File.write(DetectorConfigBuffer.GetDataBuffer(),DetectorConfigBuffer.GetSizeOfData());
	       CurSize=RunInfoBuffer.GetSizeOfData();
	       CurSize+=DetectorConfigBuffer.GetSizeOfData();
	       File.write(data,size-size1);
	       CurSize+=size-size1;
	  }
	  else if(Mode==RELAXED){
	       File.close();
	       if(OldFilename!=""){
 		    if(rename((const char *)String(OldFilename+String(".do_not_transfer")),(const char*)OldFilename)<0){
 			 (*Base::Error)=SYS_ERROR;
 			 (*Base::Error)="rename";
 			 return -1;
 		    }
	       }
	       String Directory;
	       // cerr << "W: " << MaxSize/1024 << endl;
	       if(DiskUsage.GetNextFreeDir(MaxSize/1024,Directory)<0){
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="int DataStreamer::Write(..)";
		    char Msg[MAX_TEXT_SIZE];
		    sprintf(Msg,"Not enough disk space available (%Ld)",
			    MaxSize);
		    Base::Error->SetParam(Msg);
		    return -1;
	       }
	       if(DiskUsage.IsCritical(CriticalSize)){
		    char msg[512];
		    sprintf(msg,"Disk space below critical size (%d%%)",
			    CriticalSize);
		    Parent->LogMsg(msg,2,2,WARNING);
	       }
	       FileNameFactory.SetDirectory(Directory);
	       String Filename=FileNameFactory.GetNextFileName();
	       OldFilename=Filename;
	       File.open(Filename+String(".do_not_transfer"));
	       if(!File){
		    (*Base::Error)=SYS_ERROR;
		    (*Base::Error)="DataStreamer::Write(..): open '"
			 + Filename +String(".do_not_transfer")+"':";
		    return -1;
	       }
	       CheckForLimits(FileNameFactory.GetFileCount());
	       File.write(RunInfoBuffer.GetDataBuffer(),RunInfoBuffer.GetSizeOfData());
	       File.write(DetectorConfigBuffer.GetDataBuffer(),DetectorConfigBuffer.GetSizeOfData());
	       CurSize=RunInfoBuffer.GetSizeOfData();
	       CurSize+=DetectorConfigBuffer.GetSizeOfData();
	       File.write(data,size);
	       CurSize+=size;
	  }
	  else {
	       (*Base::Error)=UNKNOWN_FLAG;
	       char tmp[20];
	       sprintf(tmp,"%i",Mode);
	       Base::Error->SetParam(tmp);
	       (*Base::Error)="DataStreamer::Write(..)";
	       return -1;
	  }
     }
     else {
	  File.write(data,size);
	  CurSize+=size;
     }
     
     return 0;
}

int DataStreamer::SetConfigRecord(RecordHeader_t &RecordHeader, BinaryDataBuffer &RawData)
{
     if(!(RecordHeader.RecordID==RID_HWCONFIG_ASCII)) {
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DataStreamer::SetRunInfoRecord(..)";
	  char err[512];
	  sprintf(err,"Unknown record ID %d",RecordHeader.RecordID);
	  Base::Error->SetParam(err);
	  return -1;
     }
     // don't know why, but the record ends up bigger as the original one (2 or 3 bytes)
     // maybe it has to do with empty lines, for now, we add some extra space to the buffer
     DetectorConfigBuffer.CreateBuffer(RecordHeader.Size()+RecordHeader.NoOfBytes + 1000);
     HWConfig_t *Config=0;
     int ret=-1;
     
     switch(RecordHeader.RecordID) {
/*
     case RID_HWCONFIG_BIN:
	  Config=new HWConfigBin_t;
	  if(Config->ReadFromBuffer(RawData)>0 
	     && Config->WriteToBuffer(DetectorConfigBuffer)>0)
	  ret=0;
	  break;
*/  
     case RID_HWCONFIG_ASCII:
	  Config=new HWConfigAscii_t;
	  Config->ReadFromBuffer(RawData);
	  ((HWConfigAscii_t *)Config)->cfg_strstream->freeze(1);
	  ((HWConfigAscii_t *)Config)->cfg_instream=new istrstream(((HWConfigAscii_t *)Config)->cfg_strstream->str());
	  ((HWConfigAscii_t *)Config)->cfg_strstream->freeze(0);
	  
	  if(WriteRecord(*Config,DetectorConfigBuffer)>0){
	       ret=0;
	  }
	  
	  delete ((HWConfigAscii_t *)Config)->cfg_instream;
	  ((HWConfigAscii_t *)Config)->cfg_instream=0;
	  
	  break;
     }
     delete Config;
     
     return ret;
}

int DataStreamer::SetRunInfoRecord(RecordHeader_t &RecordHeader, BinaryDataBuffer &RawData)
{
     if(RecordHeader.RecordID!=RID_RUNINFO) {
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int DataStreamer::SetRunInfoRecord(..)";
	  char err[512];
	  sprintf(err,"Unknown record ID %d.",RecordHeader.RecordID);
	  Base::Error->SetParam(err);
	  return -1;
     }
     
     RunInfoBuffer.CreateBuffer(RecordHeader.Size()+RecordHeader.NoOfBytes);
     if(RecordHeader.WriteToBuffer(RunInfoBuffer)!=RecordHeader.Size()){
	  return -1;
     }
     RunInfo_t RunInfo;
     if(RunInfo.ReadFromBuffer(RawData)>0 
	&& RunInfo.WriteToBuffer(RunInfoBuffer)>0)
	  return 0;
     return -1;
}

int DataStreamer::CheckForLimits(s_i32 filecount)
{
     if(LFlags!=NOLIMIT){
	  if(LFlags==KEEPFIRST){
	       if(filecount>Limit && Limit > 1) {
		    FileNameFactory_t FFactory(FileNameFactory);
		    FFactory.SetFileCount(filecount-Limit);
		    String file=FFactory.GetNextFileName();
		    if(remove((const char *)file)<1){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)="DataStreamer::CheckForLimits(..)";
			 return -1;
		    }
	       }
	  }
     }
     return 0;
}

int DataStreamer::SetLock()
{
     String LockFileName=FileNameFactory.GetFileName() + String(".lock");
     int r;
     if((r=open((const char *)LockFileName,O_RDWR|O_CREAT|O_EXCL,S_IRUSR|S_IWUSR))<0){
	  if (errno==EEXIST){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataStreamer::SetLock()";
	       String tmp=String("Lockfile '") + LockFileName + String("' already exists.");
	       Base::Error->SetParam((const char *)tmp);
	       return -1;
	  } else {
	       (*Base::Error)=SYS_ERROR;
	       String tmp=String("open '") + LockFileName +String("'");
	       (*Base::Error)=(const char*)tmp;
	       return -1;
	  }
     }
     ofstream O(r);
     O << 0;
     O.close();
     return 0;
}

int DataStreamer::RemoveLock()
{
     String LockFileName=FileNameFactory.GetFileName() + String(".lock");
     int r;
     if((r=open((const char *)LockFileName,O_RDWR|O_CREAT|O_EXCL,S_IRUSR|S_IWUSR))<0){
	  if (errno!=EEXIST){
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int DataStreamer::SetLock()";
	       String tmp=String("Lockfile '") + LockFileName + String("' already removed.");
	       Base::Error->SetParam((const char *)tmp);
	       Base::Error->WarningOnly();
	       return 1;
	  } else {
	       remove((const char *)LockFileName);
	  }
     }
     return 0;
}
