/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datatransport.C,v 1.8 1999/02/05 18:35:03 steffen Exp $

  $Log: datatransport.C,v $
  Revision 1.8  1999/02/05 18:35:03  steffen
  * Major cleanup:
    - threw out code for binary config records
    - cleaned up some module classes
  * Fixed bug which prevent 32 bit values to be read correctly from
    detector cnf
  * Added masking to s7201 module

  Revision 1.7  1999/02/03 03:42:16  steffen
  * introduced DaqProcessBase. Classes that only need Error excess can
  now use base

  Revision 1.6  1999/01/29 08:50:12  steffen
  - changes for compilation under egcs
  - install target works now

  Revision 1.5  1998/11/23 14:42:07  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.4  1998/11/03 06:43:54  daq
  merged pamanda2 and pamanda3 versions

*/

#include <strstream.h>
#include <time.h>

#include <String.h>
#include "simplevector.h"
#include "stringvector.h"
#include "globalmacros.h"
#include "globalfunctions.h"
#include "globaltypes.h"
#include "base.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "crateclasses.h"
#include "datatransport.h"
#include "datafilecode.h"
#include "daqdebug.h"

/** ctor */
DataTransport::DataTransport()
     :Daemon(0){
     DEB_F("DataTransport::DataTransport()");
     CMD_Interpreter.AddFunction("Help",       &IPC_CMD_Help,
				 "help [COMMAND]     -- prints help for COMMAND or for all commands");
     CMD_Interpreter.AddFunction("Start",      &IPC_CMD_Start,
				 "Start              -- Start data transport");
     CMD_Interpreter.AddFunction("Stop",      &IPC_CMD_Stop,
				 "Stop               -- Stop data transport");
     CMD_Interpreter.AddFunction("Bye",        &IPC_CMD_Quit,
				 "Bye                -- Quits the process");
     CMD_Interpreter.AddFunction("OpenDataConnection", &IPC_CMD_OpenDataConnection,
				 "OpenDataConnection host port --  open a data connection to Host:port"/*,false*/);
     CMD_Interpreter.AddFunction("CheckDataConnection", &IPC_CMD_CheckDataConnection,
				 "CheckDataConnection host port --  checks a data connection to Host:port"/*,false*/);
     CMD_Interpreter.AddFunction("Initialize", &IPC_CMD_Initialize,
				 "Initialize         -- initialize datatransport");
     CMD_Interpreter.AddFunction("Init", &IPC_CMD_Initialize,
				 "Init               -- see Initialize");
     CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);     
     IsConfigured=false;
     TransportBuffer1=new ShmBinaryDataBuffer;
     TransportBuffer2=new ShmBinaryDataBuffer;
}

/** dtor */
DataTransport::~DataTransport(){
     DEB_F("DataTransport::~DataTransport()");
     delete TransportBuffer1;
     delete TransportBuffer2;
}

/** initialization */
int DataTransport::Init(){
     DEB_F("DataTransport::Init()");
     TEST_FAILURE(Daemon::Init());
     key_t  shmkeys[3];
     int i;
     for(i=0;i<3;i++) {
	  TEST_SYS_FAILURE(shmkeys[i]=ftok("./datacollector",(char)i+1),"ftok");
     }
     key_t semkey;
     TEST_SYS_FAILURE(semkey=ftok("./datacollector",(char)1),"ftok");
     TEST_FAILURE(TransportBuffer1->CreateBuffer(MAX_RAW_DATA_BUFFER_SIZE,shmkeys,semkey,false));
     for(i=3;i<6;i++) {
	  TEST_SYS_FAILURE(shmkeys[i-3]=ftok("./datacollector",(char)i+1),"ftok");
     }
     TEST_SYS_FAILURE(semkey=ftok("./datacollector",(char)2),"ftok");
     TEST_FAILURE(TransportBuffer2->CreateBuffer(MAX_RAW_DATA_BUFFER_SIZE,shmkeys,semkey,false));
     TEST_FAILURE(TransportBuffer2->Release());
     
     CurTransportBuffer=TransportBuffer1;

     key_t shmkey;
     TEST_SYS_FAILURE(shmkey=ftok("./datacollector",(char)7),"ftok");
     TEST_FAILURE(StopFlag.Init(shmkey,sizeof(bool),false));

     return 0;
}

/** destruction */
int DataTransport::Destroy(){
     DEB_F("DataTransport::Destroy()");
     TransportBuffer1->Release();
     TransportBuffer2->Release();
     TEST_FAILURE(Dispatcher.LogClose(LOG_TYPE_RUNLOG));
     TEST_FAILURE(Dispatcher.LogClose(LOG_TYPE_ERRORLOG));
     return Daemon::Destroy();
}

/** the event dispatcher */
int DataTransport::DispatchEvent(){
     DEB_F("DataTransport::DispatchEvent()");
     // call the dispatcher of the base class
     return Daemon::DispatchEvent();
}

int DataTransport::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     DEB_F("DataTransport::DoIPCEvent()");
     int r=Daemon::DoIPCEvent(Cmd,s);
     if(WasInternalCommand()) return r;
     
     return CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

/** returns a String with the configuration file name */
const String DataTransport::GetConfigFileName()const{
     DEB_F("DataTransport::GetConfigFileName()");
     return DATATRANSPORT_CNF_FILENAME;
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ConfigSection *DataTransport::GetConfigSection(){
     DEB_F("DataTransport::GetConfigSection()");
     return NULL; 
}

/** returns the type of the process, must be overwritten in
    derived class */
const String DataTransport::GetProcessType()const{
     DEB_F("");
     return PROCESS_TYPE_DATATRANSPORT; // whatever
}



int DataTransport::IPC_CMD_Quit(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int DataTransport::CMD_Quit(..)");
     Quit_FLAG=true;
     TEST_FAILURE(s.WriteText("cmd_info\r\nBye Bye\r\n"));
     return 0;
}

int DataTransport::IPC_CMD_Help(IOSocket &s,String *params,int numparams){
     DEB_F("int DataTransport::CMD_Help(..)");
     String Help;
     if(numparams>1)
	  CMD_Interpreter.GetHelp(params[1],Help);
     else
	  CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));

     return 0;
}

int DataTransport::IPC_CMD_Start(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("DataTransport::IPC_CMD_Start()");
     if(IsConfigured){
	  TEST_FAILURE(s.WriteText("cmd_info\r\nProcess 'datatransport' started\r\n"));
	  Selecter.SetTimer(0);
     } else {
	  TEST_FAILURE(s.WriteText("cmd_error\r\nStart called, but DataTransport not initialized\r\n"));
     }
     return 0;
}

int DataTransport::IPC_CMD_Stop(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("DataTransport::IPC_CMD_Stop()");
     TEST_FAILURE(s.WriteText("cmd_info\r\nProcess 'datatransport' stopped\r\n"));
     Selecter.SetTimer(-1);
     return 0;
}

int DataTransport::DoTIMEREvent(){
     DEB_F("DataTransport::DoTIMEREvent()");
     int r;
     if((*(bool*)StopFlag())==true) return 0;
     
     TEST_FAILURE(CurTransportBuffer->Get());
     r=DataSocket.WriteBinI(*CurTransportBuffer);
     if(r<0){
	  String errstr=Base::Error->GetErrorMessage();
	  
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="DataTransport::DoTIMEREvent()";
	  char err[5000];
	  sprintf(err,"Error while writing data to buffer. Return value was %d.\n%s",
		  r,(const char*)errstr);
	  Base::Error->SetParam(err);
	  return -1;
     } else {
	  // Complete buffer was written
	  TEST_FAILURE(CurTransportBuffer->Release());
	  if(CurTransportBuffer==TransportBuffer1) {
	       CurTransportBuffer=TransportBuffer2;
	  } else {
	       CurTransportBuffer=TransportBuffer1;
	  }
     }
     //    }
     return 0;
}


int DataTransport::IPC_CMD_OpenDataConnection(IOSocket &s,String *params,int numparams){
     DEB_F("DataTransport::IPC_CMD_OpenDataConnection()");
     if(numparams!=3) return SendUsageInfo(s,params[0]);

     // open the data connection

     IPAddress_t IPAddr;
     IPAddr.Hostname=params[1];
     IPAddr.Port=atoi(params[2]);
     TEST_FAILURE(DataSocket.Open(IPAddr));
     // try to allocate a huge buffer for the socket
     // if it fails try half of the size until success
     size_t buflen=1000*1024;

     while((DataSocket.SetSockOpt(SOL_SOCKET,SO_RCVBUF,&buflen,sizeof(buflen))<0)&&
	   (buflen>1000*10)){
	  buflen/=2;
     }
     if (buflen<1000*10){
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,"failed to set the Socket Buffer");
     } else{
	  char txt[MAX_TEXT_SIZE];
	  sprintf(txt,"Socket Buffer set to %i bytes",buflen);
	  Dispatcher.LogEntry(LOG_TYPE_RUNLOG,txt);
     }

     DataSocketNotifier.Set(DataSocket,IONotifier::READ);
     Selecter.AddNotifier(DataSocketNotifier);

     return 0;
}
     
int DataTransport::IPC_CMD_CheckDataConnection(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("DataTransport::IPC_CMD_CheckDataConnection(...)");
     // yes, were are ready
     TEST_FAILURE(DataSocket.IsWriteReady(10000));
     return 0;
}

int DataTransport::IPC_CMD_Initialize(IOSocket &s,String *params,int numparams){
     DEB_F("DataTransport::IPC_CMD_Initialize()");
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_RUNLOG));
     TEST_FAILURE(Dispatcher.LogOpen(LOG_TYPE_ERRORLOG));
     TEST_FAILURE(ParseArgs(s,params,numparams));
     TEST_FAILURE(s.WriteText("cmd_info\r\nDataTransport initialized\r\n"));
     IsConfigured=true;

     return 0;
}

int DataTransport::IPC_CMD_UnknownCommand(IOSocket &s,String *params,int numparams){
     DEB_F("int DataTransport::IPCUnknownCommand(..)");
     s.OutputStream() << "cmd_error\r\nDataTransport: Unknown command: "
		      << join(params,numparams," ") << " \r\n" << flush;
     return 0;
}

int DataTransport::IPC_CMD_WrongTypeCommand(IOSocket &/*s*/,String */*params*/,int){
     DEB_F("int DataTransport::IPCIgnoreCommand(..)");
     return 0;
}

int DataTransport::SendUsageInfo(IOSocket &s,String param){
     DEB_F("int DataTransport::SendUsageInfo(..)");
     String Help;
     CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     TEST_FAILURE(s.WriteText("cmd_infor\r\n"));
     return s.WriteText(Help);
}


void DataTransport::ArgsHelp(IOSocket &S)
{
     S.OutputStream() << "cmd_info\r\n"
		      << "Datactransport initialization arguments:\n"
		      << " --help        -?\n\tDisplay this help screen\n\n"
		      << " --verbose-console\n\t"
		      << " Console verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << " --verbose-file\n\t"
		      << " Log file verbose level (0-10)\n\t"
		      << " default is 5\n\n"
		      << "\r\n" << flush;
  
}

int DataTransport::ParseArgs(IOSocket &S, String *param, int numparams)
{
     if(numparams<=1) return 0;
     LogMsg("Parsing initilization arguments",2,2);
     TEST_FAILURE(S.WriteText("cmd_info\r\nParsing initilization arguments\r\n"));

     char **argv;//[MAX_TEXT];
     argv=new char*[numparams + 1];
     int i;
     for(i=0;i<numparams;i++){
	  argv[i]=new char[param[i].length()+1];
	  strcpy(argv[i],(const char*)param[i]);
     }

     argv[i]=NULL;
     String OptArg;
     int c;
     int ret=0;
     int digit_optind=0;
     optind=0;
     char str[512];
     
     while (1)
     {
	  int this_option_optind = optind ? optind : 1;
	  int option_index = 0;
	  static struct option long_options[] =
	  {
	       // when you add options, please maintain an 
	       // alphabetical order
	       {"help", 0, 0, '?'},
               {"verbose-console", 1, 0, 301},
               {"verbose-file", 1, 0, 302},
               {0, 0, 0, 0}
	  };

	  c = getopt_long (numparams, argv, "?",
			   long_options, &option_index);
	  if (c == -1)
	       break;
	  
	  switch (c)
	  {
	  case 0:
	       S.OutputStream() << "cmd_info\r\n"
				<< "option " 
				<< long_options[option_index].name;
	       if (optarg)
		    S.OutputStream() << " with arg " 
				     << optarg;
	       S.OutputStream() << "\r\n" << flush;
	       break;

	  case 301:
	       SetVerboseLevel(GetFileVerboseLevel(),atoi(optarg));
	       sprintf(str,"Console verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,2);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	  case 302:
	       SetVerboseLevel(atoi(optarg),GetConsoleVerboseLevel());
	       sprintf(str,"File verbose level set to %d.",atoi(optarg));
	       LogMsg((const char *)str,2,2);
	       S.OutputStream() << "cmd_info\r\n"
				<< str << "\r\n" << flush;
	       
	       break;
	       
	  case '0':
	  case '1':
	  case '2':
	       if (digit_optind != 0 && digit_optind != this_option_optind)
		    printf ("digits occur in two different argv-elements.\n");
	       digit_optind = this_option_optind;
	       printf ("option %c\n", c);
	       break;

	  case '?':
	       ArgsHelp(S);
	       break;

	  default:
	       sprintf(str,"Getopt returned character code 0%o",c);
	       S.OutputStream() << "cmd_error\r\n" 
				<< str
				<< "\r\n" << flush;
	       LogMsg((const char *)str,2,2,ERROR);
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="DataTransport::ParseArgs(..)";
	       Base::Error->SetParam(str);

	       ret=-1;
	       goto CLEAN_EXIT;
	  }
     }
     if (optind < numparams)
     {
	  char err[5000];
	  strcpy(err,"non-option ARGV-elements: ");
	  while (optind < numparams){
	       strcat(err,argv[optind++]);
	       strcat(err," ");
	  }
	  LogMsg(err,2,2,ERROR);
	  S.OutputStream() << "cmd_error\r\n" << err << "\r\n";
	  ret=-1;
	  goto CLEAN_EXIT;
     }
     
 CLEAN_EXIT:
     for(i=0;i<=numparams;i++){
	  delete [] argv[i];
	  argv[i]=NULL;
     }
     
     delete [] argv;
     return ret;
}

int main(int /*argc*/,char */*argv[]*/){
     gv_DebugLevel=DL_FUNCTIONS;
     DEB_F("int main(int argc,char *argv[])");
     // ofstream deb("debug.log");
     // DebugStreamer.SetOutput(&deb);
     
     DaqProcessBase::InitializeGlobals("datatransport");
      // for program error logging
     LogFile LF(DaqProcessBase::ThisProcess->GetName());					
     Logger::SetLogger(&LF);				
     
     // Lock the process, 
     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }
     // create the daemon object
     DataTransport RServer;
     if(RServer.Init()<0){
	  Logger Entry;
	  (*Base::Error).Print();
     }else{
	  /* signal parent process, that the process was started
	     successfully */
	  kill(getppid(),SIGUSR1);
	  if(RServer.EventLoop()<0){
	       (*Base::Error).Print();
	       RServer.LogMsg((*Base::Error).GetErrorMessage(true),1,1,ERROR);
	  }
     }
     if(RServer.Destroy()<0){
	  Logger Entry;
	  (*Base::Error).Print();
     }
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0;
}

