/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datatransport.h,v 1.5 1998/11/23 14:42:08 haase Exp $

  $Log: datatransport.h,v $
  Revision 1.5  1998/11/23 14:42:08  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.4  1998/08/01 03:06:15  daq
  new version number (1.200)


*/

#if ! defined __VMEDATAPROCESS_H__
#define __VMEDATAPROCESS_H__
#include <config.h>
#include <strstream.h>
#include <String.h>
#include "simplevector.h"
#include "stringvector.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "daemon.h"
#include "vmedaqhal.h"
#if defined (HAVE_GETOPT_H)
#include <getopt.h>
#else
#include <my_getopt.h>
#endif 

/** This class transfers the data collected by the datacollector process to the dataprocess */
class DataTransport:public Daemon
{

public:
     /** Constructor */
     DataTransport();

     /** Destructor */
     ~DataTransport();

     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 @return 0 on success, -1 on failure.  */
     Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 @return 0 on success, -1 on failure.  */
     Destroy();

     /** Gets the proces type.
	 This function  must be overwritten in every derived class
	 @return Type of the process */
     virtual const String GetProcessType()const;

protected: 
     /** Dispatches incomming events.
	 @return 0 on success, -1 on failure. */
     virtual int DispatchEvent();

     /** The IPC event handler
	 Calls the appropriate command handling function.
	 @return 0 on success, -1 on failure. */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);

     /** This function is exectued on timer event.
	 The timer value can be set with {\tt Selecter.SetTimer(..);}
	 @return 0 on success, -1 on failure. */
     virtual int DoTIMEREvent(); 

     /** Gets the configuration file name.
	 @return Name of configuration file for this class */
     virtual const String GetConfigFileName()const;

     /** Creates a class for handling the configuration of this class.
	 @return Pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ConfigSection *GetConfigSection();

     /** Parses initialization arguments.
	 @param S Command socket, used to send any reply commands.
	 @param Param Pointer to vector of parameters.
	 @param NumParams Number of parameters in {\tt \Ref{Param}}. 
	 @return 0 on success, -1 on failure. */
     int ParseArgs(IOSocket &S, String *param, int numparams);

     /** Prints a help message for all initialization arguments.
       @param S Command socket where the message is written to.*/
     void ArgsHelp(IOSocket &S);
     
protected:
     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Quit(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Help(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Start(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Stop(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_Initialize(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_OpenDataConnection(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_CheckDataConnection(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_UnknownCommand(IOSocket &S,String *Params,int NumParams);

     /** Command handling function.
	 @params S Command socket
	 @param Params Pointer to array of command paramters, 
	 {\tt param[0]} is the command itself.
	 @param NumParams Number of params in {\tt \Ref{Params}}.
	 @return 0 on success, -1 on failure. */
     int IPC_CMD_WrongTypeCommand(IOSocket &S,String *Params,int NumParams);

     /** Sends usage info for a certain command.
	 @param S Command socket
	 @params Param Command string for which we want to send the usage info
	 @return 0 on success, -1 on failure. */
     int SendUsageInfo(IOSocket &S,String param);     

protected:
     /** Configuration flag.
	 A run can only be started after a successful initialization.*/
     bool IsConfigured;

     /** Socket where the raw data is written to */
     ClientSocket DataSocket;

     /** Notifier for data socket */
     SocketNotifier DataSocketNotifier;

     /** Interpreter for incomming comands */
     Interpreter<DataTransport> CMD_Interpreter;

     /** Pointer to data buffer 1.
	 A new buffer will be allocated upon intialization */
     ShmBinaryDataBuffer *TransportBuffer1;

     /** Pointer to data buffer 2.
	 A new buffer will be allocated upon intialization */
     ShmBinaryDataBuffer *TransportBuffer2;
     /** Pointer to the currently used data buffer.
	 The datacollector writes data to one buffer. The other buffer is
	 send to the dataprocess by the datatransport process at the same time.
     Switching buffers is organized using a semaphore (see {\tt \Ref{ShmBinaryDataBuffer}}.*/
     ShmBinaryDataBuffer *CurTransportBuffer;

     /** Process stops writing buffers out if true.
       This flag is set by the datacollector process. It's
       never altered by the datatransport. */
     SharedMem StopFlag;
};

#endif // __VMEDATAPROCESS_H__



