# -* bash *-
# Copyright (C) 1998 Heiko Haase  <haase@mus.urz.uni-magdeburg.de>
  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


###########################################################################
# Prefix for all installation directories, usually /usr or /usr/local
prefix=/usr/local

###########################################################################
# Prefix for all installation directories of all exeutables, usually the 
# same as $prefix
exec_prefix=${prefix}

###########################################################################
# The working directory of the local Programs
LOCALPATH=${exec_prefix}/bin

###########################################################################
# Path to the decoapi.sh file
DECOAPIPATH=${prefix}/share/daq98

###########################################################################
# Path to the DAQ configuration file(s)
CNFPATH=${prefix}/share/daq98

###########################################################################
# no configuration beyond this point
###########################################################################

if ! [ -f $DECOAPIPATH/decoapi.sh ]; then 
    echo $red"*** can not find $DECOAPIPATH/decoapi.sh, please configure me (edit deco) *** "$clear
    exit 2
fi

savedir=`pwd`
cd $DECOAPIPATH
. decoapi.sh
cd $savedir


DefineLog $2

CNFFILE=$CNFPATH/"daq.cnf"
VMEHOST=$(ReadConfig $CNFFILE VMEControlAddress)
VMEPATH=$(ReadConfig $CNFFILE VMEControlPath)
DAQHOST=$(ReadConfig $CNFFILE DispatcherAddress)

if [ "$1" != "conf" ]; then 

    if [ "`ExecBashCmd_internal hostname $DAQHOST`" != "`hostname`" ]; then
	Print $red" +++ deco/dcc must be run on the DAQ-HOST !!!! check daq.cnf +++ "$clear
	exit 2
    fi
    
    if [ "$1" != "state" ]; then
	if ! [ -f $LOCALPATH/daqcontrol ]; then 
	    Print $red"+++ can not find $LOCALPATH/daqcontrol, please configure me (edit deco) +++ "$clear
	    exit 2 
	fi

	if ! [ -f $LOCALPATH/dispatcher ]; then 
	    Print $red"+++ can not find $LOCALPATH/dispatcher, please configure me (edit deco) +++ "$clear
	    exit 2
	fi

	if ! ExecBashCmd_internal hostname $VMEHOST >/dev/null ; then
	    Print $red" +++ not remote shell access to $VMEHOST, please fix this +++ "$clear
	    exit 2
	fi

	if [ "`ExecBashCmd_internal \"if [ -f $VMEPATH/vmecontrol ]; then echo yes ; else echo no; fi\" $VMEHOST`" != "yes" ]; then
	    Print $red"+++ can not find $VMEHOST:$VMEPATH/vmecontrol, please configure me (edit deco) +++ "$clear
	    exit 2
	fi
    fi
fi
    
##checks state of a process and it's lock - file
# $1 process name
# [$2] host
function check_process_state(){
    local ret 
    if CheckProcess $1 $2; then ret="1" 
    else 
	if CheckLOCK $1 $2 ; then
	    Print $red"removing $1.lock"$clear
	    RmLOCK $1 $2
	fi
    fi
    [ "$ret" = "1" ]
}

function check_deamon_state() {
    local ret
    Print $green""Checking state of DAQ Daemons$clear
    if check_process_state daqcontrol;          then ret="1"    ; fi
    if check_process_state vmecontrol $VMEHOST; then ret=$ret"1"; fi
    if check_process_state dispatcher ;         then ret=$ret"1"; fi
    [ "$ret" = "111" ]
}

function check_daq_program_state() {
    Print $green""Checking state of DAQ Programms$clear
    check_process_state datatransport $VMEHOST
    check_process_state datacollector $VMEHOST
    check_process_state dataprocess
}

case $1 in 
state)
    check_daq_program_state
;;

start)    
    Print $green"Update of VME-Side daq.cnf"$clear
    CopyFile $CNFFILE $VMEPATH daq.cnf $VMEHOST
    
    Print $green"Starting DAQ Daemons"$clear

    StartDaemon $LOCALPATH ./daqcontrol
    sleep 2

    StartDaemon $VMEPATH ./vmecontrol $VMEHOST
    sleep 2
;;

stop)    
    Print $green"Shuting down DAQ Daemons"$clear
    KillProcess vmecontrol -TERM $VMEHOST 
    KillProcess daqcontrol -TERM
    KillProcess dispatcher -TERM
    sleep 2 
        
;;

conf)
    Print $green"DECO - CONFIGURATION"$clear
    Print $red"LOCALPATH:  "$clear$blue  $LOCALPATH $clear
    Print $red"DECOAPIPATH:"$clear$blue  $DECOAPIPATH $clear
    Print $red"ConfigFile: "$clear$blue  $LOCALPATH/"daq.cnf" $clear
    Print $red"VMEHOST:    "$clear$blue  $VMEHOST $clear
    Print $red"VMEPATH:    "$clear$blue  $VMEPATH $clear
    Print $red"DAQHOST:    "$clear$blue  $DAQHOST $clear
    exit 0
;;

kill)
    Print $red"Killing"$clear$green" all DAQ-Processes"$clear
    Print $blue"You must be the owner of the DAQ-Processes"$clear
    KillProcess comserver     -KILL  
    KillProcess datacollector -TERM $VMEHOST
    KillProcess datatransport -TERM $VMEHOST
    KillProcess dataprocess   -TERM   

    KillProcess vmecontrol -TERM $VMEHOST 
    KillProcess daqcontrol -TERM
    KillProcess dispatcher -TERM

    check_daq_program_state
;;

*)
    Print $pink"start/stop of DAQ-Control Daemons, state check of DAQ-System"$clear    
    Print $pink"usage: $0 (start|stop|state|conf)"$clear
    exit 0
    
;;
esac

if check_deamon_state 
then 
    ret=1
else
    ret=0
fi

Print $red"Ready"$clear

[ "$ret" = "1" ] 
