#! /bin/bash
# Copyright (C) 1998 Heiko Haase  <haase@mus.urz.uni-magdeburg.de>
  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

. color.sh

BG=""

# $1 -> cnf-file
# $2 -> cnf-item
function ReadConfig () {
    sed 's/\#.*//' < $1 | grep $2 | awk '{ print $2; }'
}

# $1 -> name of the log file or nothing
function DefineLog () {
    export DAQ_LOG=$1
}

function IsFlog () {
    [ "$DAQ_LOG" != "" ]
}

function Print () {
    if [ "$DAQ_LOG" != "" ] ; then echo $@ >> $DAQ_LOG 2>&1 
    else echo $@
    fi
}

# $1 command
# [$2] host
function ExecBashCmd () {
    if [ "$DAQ_LOG" != "" ]; then
	if [ "$2" = "" ]; then 
	    echo $1 | bash >> $DAQ_LOG 2>&1 
	else  
	    echo $1 | rsh $2 "exec bash" >> $DAQ_LOG 2>&1 
	fi
    else
	if [ "$2" = "" ]; then 
	    echo $1 | bash 
	else  
	    echo $1 | rsh $2 "exec bash" 
	fi
    fi
}

# $1 command
# [$2] host
function ExecBashCmd_internal () {
    if [ "$2" = "" ]; then 
	echo $1 | bash 
    else  
	echo $1 | rsh $2 "exec bash" 
    fi
}

# $1 process name
# [$2] host
function GetPID(){
    cat > /tmp/decoapi.cmd <<EOF
if [ "\$OS" = "LynxOS" ] ; 
then
    ps ax;
else
    ps axc;
fi
EOF
    ExecBashCmd_internal "`cat /tmp/decoapi.cmd`" $2 > /tmp/ps.out
    local res
    res=`grep -w $1 /tmp/ps.out| awk "{print \\\$1;}"`
    rm /tmp/ps.out
    rm /tmp/decoapi.cmd
    echo $res
}

# $1 path
# $2 process name
# [$3] host
## Runs a program in Background on a specified host only, and only if there 
## is no program already running with that name
function StartProcess () {
    if [ "`GetPID $2 $3`" = "" ] ; then
	Print $yellow"** Atempting to start $2 in $1 on HOST $3 **"$clear
	ExecBashCmd "cd $1; $2" $3 
    else
	Print $red"** $2 already running on HOST $3 **"$clear
    fi
}

# $1 path
# $2 process name
# [$3] host
## Runs a program in Background on a specified host only, and only if there 
## is no program already running with that name
function StartDaemon () {
    if [ "`GetPID $2 $3`" = "" ] ; then
	Print $yellow"** Atempting to start $2 in $1 on HOST $3 **"$clear
	ExecBashCmd "cd $1; $2" $3 &
    else
	Print $red"** $2 already running on HOST $3 **"$clear
    fi
}

# $1 process name
# $2 SIGNAL
# [$3] host
function KillProcess () {
    PID=`GetPID $1 $3`
    if [ "$PID" != "" ] ; then
	Print $yellow"** Atempting to kill $1 with $2 on HOST $3 **"$clear
	ExecBashCmd "kill $2 $PID" $3
    else
	Print $red"** $1 not running on HOST $3 **"$clear
    fi
}

# $1 process name
# [$2] host
function CheckProcess () {
    res=`GetPID $1 $2`
    if [ "$res" != "" ] ; then
	Print $blue"** $1 "$red"running"$blue" on HOST $2 **"$clear
    else
	Print $blue"** $1 "$red"not running"$blue" on HOST $2 **"$clear
    fi
    [ "$res" != "" ]
}

# $1 process whose lock should be checked
# [$2] host
function CheckLOCK () { 
    res=`ExecBashCmd_internal "ls /tmp/$1.lock 2>/dev/null" $2`
    if [ "$res" != "" ]; 
    then
	Print $blue"** lock file of $1 "$red"exists"$blue" on HOST $2 **"$clear
    fi
    [ "$res" != "" ]
}

# $1 process whose lock should be removed
# [$2] host
function RmLOCK () { 
    ExecBashCmd_internal "rm /tmp/$1.lock" $2
}

# $1 Local File 
# $2 remote path
# $2 remote name
# $3 remote host name
function CopyFile () {
    if [ "$4" = "" ] ; then Print $red"no remote Host specified"$clear ; fi  
    rcp $1 $4:$2/$3
}

