# -* bash *-

# Copyright (C) 1998 Heiko Haase  <haase@mus.urz.uni-magdeburg.de>
  
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without 
# modifications, as long as this notice is preserved.
 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

function save_prompt { 
    if [ "$PS1_SAVE" = "" ]; then
	export PS1_SAVE=$PS1
    fi
}

function restore_prompt { 
    if [ "$PS1_SAVE" != "" ]; then
	export PS1=$PS1_SAVE
    fi
}

function daq_kill() {
    restore_prompt
    kill $CLIENT_PID 2>/dev/null
    kill -9 $CLIENT_PID 2>/dev/null
    rm $PIPE_SEND 2>/dev/null
    rm $PIPE_RECV 2>/dev/null
    export -n DAQ_OPEN    
    export -n DAQ_MACRO
    export -n PIPE_SEND
    export -n PIPE_RECV
    export -n CLIENT_PID
    unalias exit
}

function internal_daq_cmd() {
    if [ "$DAQ_OPEN" != "TRUE" ]
    then
	echo "Not connected"
	return 1
    fi
    echo $@ >> $PIPE_SEND
    if [ "`cat $PIPE_RECV`" != "ok" ]
    then
	echo "Fatal error: aborting"
	daq_kill
	return 1
    fi
    return 0
}

function internal_daq_cmd_p() {
   echo $@
   internal_daq_cmd $@
}

function internal_record_macro() {
    if [ "$DAQ_MACRO" != "" ]
    then
	echo $@ >> $DAQ_MACRO
    fi
}

# $1 hostname
# [ $2 ] HBOOK-name
function daq_open_internal() {
    if [ "$DAQ_OPEN" = "TRUE" ]
    then
	echo "Already connected"
	return
    fi
    echo "echo \$PPID" | bash > /tmp/pid
    PID=`cat /tmp/pid`
    rm /tmp/pid
    export PIPE_SEND=/tmp/to_sclient.pipe_$1_$PID
    export PIPE_RECV=/tmp/from_sclient.pipe_$1_$PID
    
    if [ -e $PIPE_SEND ] 
    then
    rm $PIPE_SEND
    fi

    if [ -e $PIPE_RECV ] 
    then
    rm $PIPE_RECV
    fi
    
    if [ "$2" != "" ] 
    then 
	HBOOK_FILE=$2
    else
	HBOOK_FILE="histograms.hbook.saves"
    fi
    
    mknod $PIPE_SEND p
    mknod $PIPE_RECV p
    sclient $PIPE_SEND $PIPE_RECV $HBOOK_FILE &
#    echo sclient $PIPE_SEND $PIPE_RECV $HBOOK_FILE 
    export CLIENT_PID=$!
    export DAQ_OPEN=TRUE
    internal_daq_cmd open $1
    if [ "$DAQ_MACRO" != "" ] 
    then
	if [ -e $DAQ_MACRO ]
	then
	    rm $DAQ_MACRO
	fi
    fi
    internal_record_macro ". scapi.sh"
    internal_record_macro "daq_open" $1 $HBOOK_FILE
    export PS1="unknown@daq> "
}

# $1 hostname
# [ $2 ] HBOOK-name
function daq_open() {
    local host
    local hbook
    if [ "$1" = "" ] ; then
	echo -n "Host: "
	read host
	echo -n "HBOOK-file:"
	read hbook
    else
	host=$1
	hbook=$2
    fi
    daq_open_internal $host $hbook
    alias exit="daq_close;exit" 
}
    

function daq_close() {
    restore_prompt
    internal_daq_cmd bye
    rm $PIPE_SEND 2>/dev/null
    rm $PIPE_RECV 2>/dev/null
    internal_record_macro "daq_close"
    export -n DAQ_OPEN    
    export -n DAQ_MACRO
    export -n PIPE_SEND
    export -n PIPE_RECV
    export -n CLIENT_PID
    unalias exit
}

function daq_login() {
    if [ "$1" = "" ]; then 
	echo -n "login: "
	read login
    else
	login=$1
    fi
    if [ "$2" = "" ]; then 
	echo -n "passwd: "
	stty -echo
	read passwd
	stty echo
    else
	passwd=$2
    fi
    
    if internal_daq_cmd CS login $login $passwd ; then
	export PS1="$login@daq> "
    fi
    internal_record_macro "daq_login" $1 $2
}

function daq_login_i() {
    daq_login $1
}

function daq_cmd() {
    internal_daq_cmd $@
    internal_record_macro "daq_cmd" $@
}

function daq_record_macro() {
    export DAQ_MACRO=$1
}

function daq_runads(){
    internal_record_macro "daq_runads" $1
    sed s/\"/\\\\\"/g  $1 | \
    awk "{ print \"internal_daq_cmd_p \"\"\\\"\" \$0 \"\\\"\"}" > /tmp/ads.tmp
    shift
    . /tmp/ads.tmp $@
    rm /tmp/ads.tmp
}

cat <<EOF

****************************************************************
**    HELLO this is SCAPI , an interface to the AMANDA DAQ    **
**      You just loaded some new function into your bash      **
****************************************************************  
use:
 daq_open [host [hbook-file]] -- to connect to the daq 
 daq_login [name [passwd]]    -- for non-interactive login 
 daq_login_i [name]           -- for interactive login 
 daq_cmd                      -- to send a command to the daq 
 daq_close                    -- to close the connection 
 daq_kill                     -- to realy kill the connection 
 daq_record_macro filename    -- to record your session for later 
			         use as a shell-script 
 daq_runads args              -- to run an ads-script ala 
                                 initrun.ads (there can be args in 
				 the ads -sricpt like $1 ..)

 ** if the program hangs, use ^C and daq_kill and try again **

EOF

save_prompt

#if [ "$1" != "" ]
#then 
#daq_open $1
#daq_login_i
#alias exit="daq_close;exit" 
#fi
