/*
  $Header: /home/elrond/baikal/src/CVS/daq98/diskusage.C,v 1.10 1998/11/23 14:42:12 haase Exp $

  $Log: diskusage.C,v $
  Revision 1.10  1998/11/23 14:42:12  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.9  1998/11/11 14:13:48  daq
  *** empty log message ***

  Revision 1.8  1998/11/06 21:58:39  haase
  first commit of heiko only verry minor chages + new text base client

  Revision 1.5  1998/07/31 07:43:15  daq
  * diskusage.C: Fixed header file inclusion for 'statfs' to compile under OSF

*/

#include <limits.h>
#if defined __osf__
#include <sys/mount.h>
#elsif defined __lynx__
#include <statfs.h>
#else
#include <sys/vfs.h>
#endif
#include "diskusage.h"
#include "error.h"
#include "base.h"

DiskUsage_t::DiskUsage_t() {
     Directories=NULL;
     CurrentDirectory.Directory="";
     CurrentDirectory.Prio=INT_MAX;
}

DiskUsage_t::DiskUsage_t(DirectoryEntryVector_t *directories)
{
     Directories=directories;
     AddSlashes();
}


DiskUsage_t::~DiskUsage_t()
{
}

void DiskUsage_t::SetDirectories(DirectoryEntryVector_t *directories)
{
     if(Directories) delete Directories;
     Directories=directories;
     AddSlashes();
}

     
long long DiskUsage_t::GetFreeSpace(const String &Directory,long long &Size)
{
     return __GetFreeSpace(Directory,Size);
}

bool DiskUsage_t::IsFree(long Size)
{
     if(CurrentDirectory.Prio<INT_MAX){
	  long long FreeSize;
	  __GetFreeSpace(CurrentDirectory.Directory,FreeSize);
	  return (Size<=FreeSize);
     }
     return false;
}

bool DiskUsage_t::IsFree(const String &Directory, long Size)
{
     long long FreeSize=0;
     // cerr << "IF 1: " << Size << endl;
     if(__GetFreeSpace(Directory,FreeSize)<0){
	  // cerr << "********* statfs error\n" << Base::Error->GetErrorMessage();
	  return false;
     }
     // cerr << "IF 2: " << Size << " " << FreeSize << endl;
     return (Size<=FreeSize);
}

int DiskUsage_t::GetNextFreeDir(long Size,String &Directory)
{
#define __A__
#ifdef __A__
     // cerr << "GNFD: " << Size << endl;
     
     DirectoryEntryVector_t *AvailDirectories=new DirectoryEntryVector_t;
     DirectoryEntryVector_t::iterator i;
     
     for(i=Directories->begin();i<Directories->end();i++){
	  if(IsFree((*i).Directory,Size)){
	       AvailDirectories->push_back((*i));
	  }
     } 
     if(AvailDirectories->size()==0){
	  delete AvailDirectories;
	  return -1;
     }
          
     DirectoryEntryVector_t::iterator HighestPrio=AvailDirectories->begin();
     for(i=AvailDirectories->begin();i<AvailDirectories->end();i++){
	  if((*i).Prio<(*HighestPrio).Prio){
	       HighestPrio=i;
	  }
     }
     CurrentDirectory=(*HighestPrio);
     Directory=CurrentDirectory.Directory;
#else
     Directory="/tmp/";
#endif     
     return 0;
}

bool DiskUsage_t::IsCritical(int CriticalValue)
{
     long long FreeSpace=0, Space=0;
     long long SpaceTmp;
     DirectoryEntryVector_t::iterator i;
     for(i=Directories->begin();i<Directories->end();i++){
	  if(__GetBlocks((*i).Directory,SpaceTmp)==0){
	       Space+=SpaceTmp;
	  }
	  if(__GetFreeBlocks((*i).Directory,SpaceTmp)==0){
	       FreeSpace+=SpaceTmp;
	  }
     }
     if((Space-FreeSpace)/(double)Space*100.0 >= CriticalValue)
	  return true;
     return false;
}

     
bool DiskUsage_t::IsCritical(const String &Directory, int CriticalValue)
{
     long long FreeSpace=0, Space=0;
     long long SpaceTmp;
     if(__GetBlocks(Directory,SpaceTmp)==0){
	  Space+=SpaceTmp;
     }
     if(__GetFreeBlocks(Directory,SpaceTmp)==0){
	  FreeSpace+=SpaceTmp;
     }
     if((Space-FreeSpace)/(double)Space*100.0 >= CriticalValue)
	  return true;
     return false;
}

int DiskUsage_t::__GetFreeSpace(const String &Directory, long long &Space)
{
     struct statfs buf;
     Space=0;
     // cerr << "__GFS 1: " << Space << endl;
     if(statfs((const char*)Directory,&buf)<0){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)=(const char *)String(String("statfs :")+String((const char*)Directory));
	  // cerr << "__GFS 2: " << Space << endl;
	  return -1;
     }
     // cerr << "__GFS 3: " << buf.f_bsize << " " <<  buf.f_bavail << " " << buf.f_bsize * buf.f_bavail << endl;
     Space=buf.f_bavail / 1024;
     Space *= buf.f_bsize;
     // cerr << "__GFS 4: " << Space << endl;
     return 0;
}

int DiskUsage_t::__GetSpace(const String &Directory, long long &Size)
{
     struct statfs buf;
     Size=0;
     if(statfs((const char*)Directory,&buf)<0){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)=(const char *)String(String("statfs :")+String((const char*)Directory));
	  return -1;
     }
     Size=buf.f_blocks/ 1024;
     Size *=buf.f_bsize;
     return 0;
}
     
int DiskUsage_t::__GetFreeBlocks(const String &Directory, long long &Space)
{
     struct statfs buf;
     Space=0;
     if(statfs((const char*)Directory,&buf)<0){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)=(const char *)String(String("statfs :")+String((const char*)Directory));
	  return -1;
     }
     Space=buf.f_bavail;
     return 0;
}

int DiskUsage_t::__GetBlocks(const String &Directory, long long &Size)
{
     struct statfs buf;
     Size=0;
     if(statfs((const char*)Directory,&buf)<0){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)=(const char *)String(String("statfs :")+String((const char*)Directory));
	  return -1;
     }
     Size=buf.f_blocks;
     return 0;
}

void DiskUsage_t::AddSlashes()
{
     DirectoryEntryVector_t::iterator i;
     for(i=Directories->begin();i<Directories->end();i++){
	  AddSlashes((*i).Directory);
     }
}

     
void DiskUsage_t::AddSlashes(String & Directory)
{
     if(Directory[Directory.length()-1]!='/')
	  Directory+=String("/");
}
