#if ! defined __DISKUSAGE_H__
#define __DISKUSAGE_H__

#include <config.h>
#include <String.h>
#include "pointerutil.h"
#include "simplevector.h"
#include "globaltypes.h"

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/diskusage.h,v 1.4 1998/11/23 14:42:13 haase Exp $
*/

/** A directory entry.
    {\tt \Ref{Directory}} is the directory path and
    {\tt \Ref{Prio}} is the priority with which this path
    should be used. */
struct DirectoryEntry_t
{
     DirectoryEntry_t(){
	  Directory=".";
	  Prio=0;
     }
     
     String Directory;
     int Prio;
};

MAKE_INC_VECTOR(DirectoryEntry_t,DirectoryEntryVector_t);

class DiskUsage_t
{
public:
     /** Constructor */
     DiskUsage_t();
     
     /** Constructor.
	 @param Directories Vector of directories to use */
     DiskUsage_t(DirectoryEntryVector_t *Directories);
     
     /** Destructor */
     ~DiskUsage_t();
     
     /** Sets list of directories.
	 @param Directories Vector of directories to use */
     void SetDirectories(DirectoryEntryVector_t *Directories);

     /** Get available space for a directory.
	 This is the protected counterpart of {\tt \Ref{GetFreeSpace}}
	 @param Directory Directory path
	 @param Size Contains available space in KBytes on return
	 @return 0 on success, -1 on failure */
     long long GetFreeSpace(const String &Directory, long long &Space);
     
     /** Checks if current directory has enough space remaining.
	 @Directory Directory to check on
	 @Size Number of KBytes that should be available
	 @return true if number of available KBytes is $\ge$ 
	 {\tt \Ref{Size}}, false otherwise */
     bool IsFree(long Size);

     /** Checks if directory has enough space remaining.
	 @Directory Directory to check on
	 @Size Number of KBytes that should be available
	 @return true if number of available bytes is $\ge$ 
	 {\tt \Ref{Size}}, false otherwise */
     bool IsFree(const String &Directory, long Size);
     
     /** Get next directory with free disk space.
	 Searches the classes list of directories for the
	 next directory which has $\eg$ {\tt \Ref{Size}} space
	 available. If more than one directory is available the
	 one with the highest priority number is returned. If more
	 than one available directory has the same prioraty the first
	 one in the directory vector is returned.
	 @param Size Amount of disk space in KBytes that should be available
	 @return The path of the directory which has the requested space available or {\tt ""} if none of the directories has enough space left.*/
     int GetNextFreeDir(long Size,String &Directory);
     
     /** Checks if available disk space has reached a critical value.
	 The critical value is given in percent.
	 @param CriticalValue Amount of disk space in percent that should be 
	 available in all directories in the directory vector together
	 @return -1 if $>$ {\tt \Ref{CriticalValue}} is already used, 0
	 otherwise */
     bool IsCritical(int CriticalValue);
     
     /** Checks if available disk space has reached a critical value.
	 The critical value is given in percent.
	 @param Directory Directory to check
	 @param CriticalValue Amount of disk space in percent that should be 
	 available in {\tt \Ref{Directory}}
	 @return -1 if $>$ {\tt \Ref{CriticalValue}} is already used, 0
	 otherwise */
     bool IsCritical(const String &Directory, int CriticalValue);

protected:
     /** Get available space for a directory.
	 This is the protected counterpart of {\tt \Ref{GetFreeSpace}}
	 @param Directory Directory path
	 @param Size Contains available space in KBytes on return
	 @return 0 on success, -1 on failure */
     int __GetFreeSpace(const String &Directory, long long &Space);

     /** Get total size of file system.
	 @param Directory Directory path
	 @param Size Contains total size in KBytes on return
	 @return 0 on success, -1 on failure */
     int __GetSpace(const String &Directory, long long &Space);

     /** Get available blocks for a directory.
	 This is the protected counterpart of {\tt \Ref{GetFreeSpace}}
	 @param Directory Directory path
	 @param Blocks Contains available blocks on return
	 @return 0 on success, -1 on failure */
     int __GetFreeBlocks(const String &Directory, long long &Blocks);

     /** Get total number of blocks for file system.
	 @param Directory Directory path
	 @param Blocks Contains total number of blocks on return
	 @return 0 on success, -1 on failure */
     int __GetBlocks(const String &Directory, long long &Blocks);

     /** Adds trailing slashes to directories if missing */
     void AddSlashes();
     
     /** Adds trailing slashes to directories if missing 
	 @param Directory Directory to check */
     void AddSlashes(String &Directory);

protected:
     /** Pointer to directory vector */
     DirectoryEntryVector_t *Directories;

     /** Directory currently used */
     DirectoryEntry_t CurrentDirectory;
};


#endif __DISKUSAGE_H__
