/*
$Header: /home/elrond/baikal/src/CVS/daq98/dispatcher.C,v 1.7 1999/02/04 03:38:31 steffen Exp $
*/

#include <strstream.h>
#include <String.h>
#include <list.h>

#include "globaltypes.h"
#include "globalmacros.h"
#include "errorlog.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "memman.h"
#include "interpreter.h"
#include "pointerutil.h"
#include "privilege.h"
#include "cnfreader.h" 
#include "dispatcher.h"
#include "stringutil.h"
#include "runfilename.h"
#include "runstat.h"

MainDispatcher::MainDispatcher():DaemonBase(){
     DEB_F("MainDispatcher::MainDispatcher()");

     IPC_CMD_Interpreter.AddFunction("RegisterProcess",     &IPC_CMD_RegisterProcess);
     IPC_CMD_Interpreter.AddFunction("UnregisterProcess",   &IPC_CMD_UnregisterProcess);
     IPC_CMD_Interpreter.AddFunction("SetUser",             &IPC_CMD_SetUser);
     IPC_CMD_Interpreter.AddFunction("GetUser",             &IPC_CMD_GetUser);
     IPC_CMD_Interpreter.AddFunction("AddPriv",             &IPC_CMD_AddPriv);
     IPC_CMD_Interpreter.AddFunction("SetPriv",             &IPC_CMD_SetPriv);
     IPC_CMD_Interpreter.AddFunction("ClearPriv",           &IPC_CMD_ClearPriv);
     IPC_CMD_Interpreter.AddFunction("IsSetPriv",           &IPC_CMD_IsSetPriv);
     IPC_CMD_Interpreter.AddFunction("GetProcessPortLock",  &IPC_CMD_GetProcessPortLock);
     IPC_CMD_Interpreter.AddFunction("FreeProcessPortLock", &IPC_CMD_FreeProcessPortLock);
     IPC_CMD_Interpreter.AddFunction("Broadcast",           &IPC_CMD_Broadcast);
     IPC_CMD_Interpreter.AddFunction("bye",                 &IPC_CMD_QuitProcess);
     IPC_CMD_Interpreter.AddFunction("AllocIPAddress",      &IPC_CMD_AllocIPAddress);
     IPC_CMD_Interpreter.AddFunction("FreeIPAddress",       &IPC_CMD_FreeIPAddress);
     IPC_CMD_Interpreter.AddFunction("CreateSharedMem",     &IPC_CMD_CreateSharedMemory);
     IPC_CMD_Interpreter.AddFunction("DestroySharedMem",    &IPC_CMD_DestroySharedMemory);
     IPC_CMD_Interpreter.AddFunction("GetShmManKeys",       &IPC_CMD_GetShmManKeys);
     IPC_CMD_Interpreter.AddFunction("IncRunNumber",        &IPC_CMD_IncRunNumber);
     IPC_CMD_Interpreter.AddFunction("RunStatStart",        &IPC_CMD_RunStatStart);
     IPC_CMD_Interpreter.AddFunction("RunStatStop",         &IPC_CMD_RunStatStop);
     IPC_CMD_Interpreter.AddFunction("Check",               &IPC_CMD_Check);
     IPC_CMD_Interpreter.AddFunction("LogEntry",            &IPC_CMD_LogEntry);
     IPC_CMD_Interpreter.AddFunction("LogOpen",             &IPC_CMD_LogOpen);
     IPC_CMD_Interpreter.AddFunction("LogClose",            &IPC_CMD_LogClose);
     IPC_CMD_Interpreter.AddFunction("LogMove",             &IPC_CMD_LogMove);
     IPC_CMD_Interpreter.AddFunction("LogAllowTransfer",    &IPC_CMD_LogAllowTransfer);
     IPC_CMD_Interpreter.AddFunction("MailMsg",             &IPC_CMD_MailMsg);
     IPC_CMD_Interpreter.AddFunction("GetStatus",           &IPC_CMD_GetStatus);
     IPC_CMD_Interpreter.AddFunction("GetPID",           &IPC_CMD_GetPID);
     IPC_CMD_Interpreter.AddFunction("close",               &IPC_CMD_Close);
     IPC_CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     IPC_CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);

     SharedMemCreated=false;
}

MainDispatcher::~MainDispatcher(){
     DEB_F("MainDispatcher::~MainDispatcher()");
     if(SharedMemCreated) DestroySharedMemory();
}

int MainDispatcher::Init(){
     DEB_F("int MainDispatcher::Init(..)");
     // first Read the config file
     TEST_FAILURE(ReadConfigFile());
     IPCAddress=DispatcherAddress;
     MailListEntry_t Entry;
     MailRecipients.gsub("\"","");
     StringSplitter R(MailRecipients,':');
     for(int i=0;i<R.NumSplits();i++){
	  StringSplitter U(R()[i],',');
	  if(U.NumSplits()==2){
	       Entry.Name=U()[0];
	       Entry.Address=U()[1];
	       Mailer.AddRecipient(Entry);
	  }
	  else if(U.NumSplits()==3){
	       Entry.Name=U()[0];
	       Entry.NickName=U()[1];
	       Entry.Address=U()[2];
	       Mailer.AddRecipient(Entry);
	  }
	  else {
	       (*Base::Error)=GENERIC_ERROR;
	       (*Base::Error)="int Daemon::Init()";
	       Base::Error->SetParam(String(String("Not a valid address entry: ")
					    +R()[i]));
	       return -1;
	  }
     }
     
     // open the IPC port
     TEST_FAILURE(DaemonBase::Init());
     return 0; 
}

int MainDispatcher::Destroy(){
     DEB_F("int MainDispatcher::Destroy(..)");
     return DaemonBase::Destroy();
}

int MainDispatcher::DispatchEvent(){
     DaemonBase::DispatchEvent();
     vector<OpenSockets_p>::iterator si=OpenSockets.begin();
     while(si<OpenSockets.end()){
	  if(Selecter.CheckNotifier((*si)->SSocketNotifier)){
	       char *txt;
	       txt=(*si)->SSocket->ReadText();
	       if(0==txt){
		    // we could not read, so we consider the
		    // connection as broken, the process was killed so
		    // we should remove the socket
		    port_t LocalPort=(*si)->SSocket->GetLocalPort();
		    Selecter.RemoveNotifier((*si)->SSocketNotifier);
		    (*si)->SSocket->Close();
		    OpenSockets.erase(si);
		    // and also remove the Process ID if not already
		    // done by unregisterprocess command, we can
		    // identify the process because during register it
		    // saved the local port of the socket and we know
		    // the local port of the socket too

		    ProcessListIterator_t process=ProcessList.begin();
		    while((process<ProcessList.end())&&
			  ((*process)->LocalPort!=LocalPort)) process++;
		    if(process<ProcessList.end()){
			 ProcessList.erase(process);
		    }
	       } else {
		    // call the Event handler for the IPC message
		    TEST_FAILURE(DoIPCEvent(txt,*((*si)->SSocket)));
	       }
	  }
	  si++;
     }       
     return 0;
}

int MainDispatcher::ServIPCSocket(){
     // if someone opens a connection to the dispatcher, we want this
     // connection to stay as long as the process exist, thus we can
     // find out when the process is killed because then the
     // connection is broken 

     // we must have a vector with all accepted sockets 
     DEB_F("int ServIPCSocket()");
     filedescriptor_t sd;
     // IPC socket is a Server socket so we must accept the connection
     // and create and open a slave server socket in order to get the
     // message/command or whatever was sent

     OpenSockets_p os=new OpenSockets_t;
     os->SSocket=new SlaveServerSocket;
     port_t localport;
     TEST_FAILURE(sd=IPCSocket.Accept(&localport));
     TEST_FAILURE(os->SSocket->Open(sd,localport));
     os->SSocketNotifier.Set(*(os->SSocket),IONotifier::READ);
     Selecter.AddNotifier(os->SSocketNotifier);
     OpenSockets.push_back(os);
     return 0;
}


int MainDispatcher::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     return IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

const String MainDispatcher::GetProcessType()const{
     return PROCESS_TYPE_MESSAGEDISPATCH; // from globalmacros.h
}

const String MainDispatcher::GetConfigFileName()const{
     return DISPATCHER_CNF_FILENAME; // from globalmacros.h
}

ClassWithTypeInfo_t *MainDispatcher::GetConfigSection(){
     return new class MainDispatcherConfig:public ConfigSection_t{
     public:
	  /** ctor */
	  MainDispatcherConfig(port_t &ipcportfirst,port_t &ipcportlast,
			       port_t &freeportfirst,port_t &freeportlast)     
	       :ConfigSection_t(),
		IPCPortFirst(ipcportfirst),
		IPCPortLast(ipcportlast),
		FREEPortFirst(freeportfirst),
		FREEPortLast(freeportlast){};
	  BeginItems(ConfigSection_t){
	       BeginLinks{
		    ItemLink1("IPCPortFirst",IPCPortFirst);
		    ItemLink1("IPCPortLast",IPCPortLast);
		    ItemLink1("FREEPortFirst",FREEPortFirst);
		    ItemLink1("FREEPortLast",FREEPortLast);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(DISPATCHER_SECTION_NAME);
     private:
	  port_t &IPCPortFirst;
	  port_t &IPCPortLast;
	  port_t &FREEPortFirst;
	  port_t &FREEPortLast;
     }(IPC_Port_first,IPC_Port_last,
       FREE_USE_Port_first,FREE_USE_Port_last);
}


int MainDispatcher::SendMessage(const IPAddress_t &IPCAddress,const String &msg){
     DEB_F("int MainDispatcher::SendMessage(..)");
     ClientSocket S;
     TEST_FAILURE(S.Open(IPCAddress));
     TEST_FAILURE(S.WriteText(msg));
     return S.Close();
}

int MainDispatcher::CreateSharedMemory(unsigned int BlockSize,unsigned int NumBlocks){
     DEB_F("int MainDispatcher::CreateSharedMemory(..)");
     if(!SharedMemCreated){
	  key_t shmkeys[NumBlocks+1];
	  CreateKeys(shmkeys,&shmMan_semkey,NumBlocks+1);
	  shmMan_shmkey=shmkeys[NumBlocks];
	  if(shmMan.Init(shmkeys,shmMan_semkey,BlockSize,NumBlocks)<0){
	       Logger Entry;
	       Entry() << "Could not create Shared Memory" << endl;
	       (*Error).Print(Entry());
	       return -1;
	  }else{
	       Logger Entry;
	       Entry() << "Shared Memory Created" << endl;
	       // notify the processes of the creation of shared memory
	       ProcessListIterator_t process=ProcessList.begin();
	       while(process<ProcessList.end()){
		    if((*process)->NotifySharedMem){
			 SendMessage((*process)->IPCAddress,"AttachSharedMemory\r\n");
		    }
		    process++;
	       }
	       SharedMemCreated=true;
	  }
     }
     return 0;
}

int MainDispatcher::DestroySharedMemory(){
     DEB_F("int MainDispatcher::DestroySharedMemory()");
     if(SharedMemCreated){
	  shmMan.Destroy();
	  Logger Entry;
	  Entry() << "Shared Memory Destroyed" << endl;
	  // notify the processes of the destruction of shared memory
	  ProcessListIterator_t process=ProcessList.begin();
	  while(process<ProcessList.end()){
	       if((*process)->NotifySharedMem){
		    SendMessage((*process)->IPCAddress,"DetachSharedMemory\r\n");
	       }
	       process++;
	  }
     }
     SharedMemCreated=false;
     return 0;
}

int MainDispatcher::CreateKeys(key_t *shmkey,key_t *semkey,int numkeys){
     DEB_F("int MainDispatcher::CreateKeys(..)");
     for(int i=0;i<numkeys;i++)
	  shmkey[i]=ftok("/.",(char)i);
     *semkey=shmkey[numkeys-1];
     return 0;
}

int MainDispatcher::IPC_CMD_RegisterProcess(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_RegisterProcess(..)");
     PID_t pid;
     IPAddress_t IPAddr;
     static port_t port=IPC_Port_first;
     int MaxPorts=IPC_Port_last-IPC_Port_first;
     port++;
     if(port>IPC_Port_last) port=IPC_Port_first;
     if(5==numparams){
	  pid.pid=atol(params[1]);
	  pid.Hostname=params[2];
	  pid.ProcessID=params[3];
	  bool NotifySharedMem=atoi(params[4])==1?true:false;
	  // look for a free port
	  ProcessListIterator_t process=ProcessList.begin();
	  while((process<ProcessList.end())&&(MaxPorts>0)){
	       if(port==(*process)->IPCAddress.Port){
		    port++;
		    if(port>IPC_Port_last) port=IPC_Port_first;
		    process=ProcessList.begin();
	       } else process++;
	       MaxPorts--;
	  }
	  if(0==MaxPorts){
	       IPAddr.Port=-1;
	       IPAddr.Hostname="-1";
	  }else{
	       // create a new Procces_t structure on the heap
	       // because we use PointerWrapper_Ref, as a pointer
	       // wrapper we don't need to care about deletion
	       IPAddr.Port=port;
	       IPAddr.Hostname=pid.Hostname;
	       Process_t_p newProcess=new Process_t;
	       newProcess->pid=pid;
	       newProcess->IPCAddress=IPAddr;
	       newProcess->NotifySharedMem=NotifySharedMem;
	       newProcess->UserName="";
	       newProcess->LocalPort=s.GetLocalPort();
	       // append it to the list
	       ProcessList.push_back(Process_t_p(newProcess));
	  }
     } else {
	  IPAddr.Port=-1;
	  IPAddr.Hostname="-1";
     }
     s.OutputStream() << IPAddr/*.Hostname << " "
		      << IPAddr.Port */<< "\r\n"<< flush;
     return 0;
}

int MainDispatcher::IPC_CMD_UnregisterProcess(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_UnregisterProcess(..)");
     if(4==numparams){
	  PID_t pid;
	  pid.pid=atol(params[1]);
	  pid.Hostname=params[2];
	  pid.ProcessID=params[3];
	  ProcessListIterator_t process=ProcessList.begin();
	  while((process<ProcessList.end())&&
		(((*process)->pid.pid!=pid.pid)||
		 (fcompare((*process)->pid.Hostname,pid.Hostname)!=0))) process++;

	  if(process<ProcessList.end()){
	       s.OutputStream() << "0\r\n" << flush;
	       ProcessList.erase(process);
	  }else{
	       s.OutputStream() << "-1\r\n" << flush;
	  }
     }
     return 0;
}

int MainDispatcher::IPC_CMD_GetPID(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_GetPID(..)");
     if(2==numparams){
	  String ProcessID=params[1];
	  ProcessListIterator_t process=ProcessList.begin();
	  while(process<ProcessList.end()){
	       if(!fcompare((*process)->pid.ProcessID,ProcessID)) {
		    s.OutputStream() << (*process)->pid.pid << "\r\n" << flush;
		    return 0;
	       }
	       process++;
	  }
     }
     s.OutputStream() << "0\r\n" << flush;
     
     return 0;
}

int MainDispatcher::IPC_CMD_SetUser(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_UnregisterProcess(..)");
     if(4==numparams){
 	  IPAddress_t IPAddr;
	  IPAddr.Hostname=params[1];
	  IPAddr.Port=atol(params[2]);
	  String &UserName=params[3];
	  ProcessListIterator_t process=ProcessList.begin();
	  while((process<ProcessList.end())&&
		((*process)->IPCAddress.Port!=IPAddr.Port)) process++;
	  if(process<ProcessList.end()){
	       (*process)->UserName=UserName;
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  }
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_GetUser(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_UnregisterProcess(..)");
     if(3==numparams){
 	  IPAddress_t IPAddr;
	  IPAddr.Hostname=params[1];
	  IPAddr.Port=atol(params[2]);
	  ProcessListIterator_t process=ProcessList.begin();
	  while((process<ProcessList.end())&&
		((*process)->IPCAddress.Port!=IPAddr.Port)) process++;
	  if(process<ProcessList.end()){
	       s.OutputStream() << (*process)->UserName << "\r\n" << flush;
	       return 0;
	  }
     }
     s.OutputStream() << "\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_AddPriv(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_AddPriv(..)");
     if(4==numparams){
          String &User=params[1];
	  String &Priv=params[2];

          bool init = atoi(params[3])==1?true:false;
	  UserPrivs.Add (User, Priv, init);
	  s.OutputStream() << "0\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_SetPriv(IOSocket &s,String *params,int numparams){
     DEB_F ("int MainDispatcher::IPC_CMS_SetPriv(..)");
     if (3 == numparams){
	  String &User = params[1];
	  String &Priv = params[2];	  

	  UserPrivs.Set (User, Priv);
	  s.OutputStream() << "0\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_ClearPriv(IOSocket &s,String *params,int numparams){
     DEB_F ("int MainDispatcher::IPC_CMS_ClearPriv(..)");
     if (3 == numparams){
	     
	  String &User = params[1];
	  String &Priv = params[2];

	  UserPrivs.Clear (User, Priv);
	  s.OutputStream() << "0\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_IsSetPriv(IOSocket &s,String *params,int numparams){
     DEB_F ("int MainDispatcher::IPC_CMD_IsSetPriv(..)");
     if (3 == numparams){
	  String &User = params[1];
	  String &Priv = params[2];
	  
	  bool result = UserPrivs.IsSet (User, Priv);
	  String Result;
	  if (result)
	       Result = "1";
	  else
	       Result = "0";
	  s.OutputStream() << Result << "\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}
     
int MainDispatcher::IPC_CMD_GetProcessPortLock(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_GetProcessPortLock(..)");
     IPAddress_t IPAddr;
     if(3==numparams){
	  String &ProcessID=params[1];
	  String &User=params[2];
	  if(compare(ProcessID,"CS")!=0){
		  
	       ProcessListIterator_t process=ProcessList.begin();
	       while((process<ProcessList.end())&&
		     (compare((*process)->pid.ProcessID,ProcessID)!=0)) process++;
	       
	       if(process<ProcessList.end()){
		    if((*process)->UserName.length()>0) {
			 IPAddr.Port=0;
			 IPAddr.Hostname="0";
		    }else{
			 (*process)->UserName=User;
			 IPAddr=(*process)->IPCAddress;
		    }
	       }else{
		    IPAddr.Port=-1;
		    IPAddr.Hostname="-1";
	       }  
	  } else {    
	       IPAddr.Port=-1;
	       IPAddr.Hostname="-1";
	  }
     }else{
	  IPAddr.Port=-1;
	  IPAddr.Hostname="-1";
     } 
     s.OutputStream() << IPAddr << "\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_FreeProcessPortLock(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_GetProcessPortLock(..)");
     IPAddress_t IPAddr;
     if(3==numparams){
	  IPAddr.Hostname=params[1];
	  IPAddr.Port=atoi(params[2]);
	  ProcessListIterator_t process=ProcessList.begin();
	  while((process<ProcessList.end())
		&&((*process)->IPCAddress.Port!=IPAddr.Port)) process++;
	  if(process<ProcessList.end()){
	       (*process)->UserName="";
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  }
     } 
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_Broadcast(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_Broadcast(..)");
     if(2==numparams){
	  String ProcessID=params[1];
	  String Msg="";
	  // Message might be more than one string ended with /r/n so
	  // we must read until we get the string "BroadcastEnd/r/n"
	  // wich is append to the actual string by the routine
	  // Dispatcher::Broadcast(..) in dispatcherlib.C
	  char *txt=0;
	  while(true){
	       if((txt=s.ReadText())==0){
		    Msg+="\r\n"; 
		    break;
	       }
	       if(strcmp(txt,"_*_BroadcastEnd_*_\r\n")==0) break;
	       Msg+=txt;
	  }
	  /* If we got more than one command, assume that we have a 
	     structure like 'cmd_header\r\nmessage\r\n', which is natural.
	     Break down string into single commands according to that
	     structure */
	  int Freq=Msg.freq("\r\n");
	  String *SplitAll=new String[Freq+1];
	  split(Msg,SplitAll,Freq,String("\r\n"));
	  String *Split=new String[Freq/2 + 1];
	  int Num=0;
	  for(int i=0;i<Freq;i++){
	       Split[Num]=SplitAll[i]+"\r\n"+SplitAll[i+1]+"\r\n";
	       Num++;
	       i++;
	  }
	  delete [] SplitAll;
	  ProcessListIterator_t process=ProcessList.begin();
	  while(process<ProcessList.end()){
	       if((*process)->pid.ProcessID==ProcessID){
		    for(int i=0;i<Num;i++){
			 SendMessage((*process)->IPCAddress,Split[i]);
		    }
	       }
	       process++;
	  }
	  delete [] Split;
     }
     return 0;
}

int MainDispatcher::IPC_CMD_SetAutoRestart(IOSocket &/*s*/,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_Broadcast(..)");
     if(2==numparams){
	  String Msg="SetAutoRestart" + params[1] + "\r\n";
	  ProcessListIterator_t process=ProcessList.begin();
	  while(process<ProcessList.end()){
	       if((*process)->pid.ProcessID==PROCESS_TYPE_DAQCONTROL){
		    SendMessage((*process)->IPCAddress,Msg);
	       }
	       process++;
	  }
     }
     return 0;
}

int MainDispatcher::IPC_CMD_AllocIPAddress(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_AllocPort(..)");
     IPAddress_t IPAddr;
     if(2==numparams){
	  IPAddr.Hostname=params[1];
	  static port_t port=FREE_USE_Port_first;
	  int MaxPorts=FREE_USE_Port_last-FREE_USE_Port_first;
	  port++;
	  if(port>FREE_USE_Port_last) port=FREE_USE_Port_first;
	  PortListIterator_t port_item=PortList.begin();
	  while((port_item<PortList.end())&&(MaxPorts>0)){
	       if(port==(*port_item)){
		    port++;
		    if(port>FREE_USE_Port_last) port=FREE_USE_Port_first;
		    port_item=PortList.begin();
	       } else port_item++;
	       MaxPorts--;
	  }
	  if(0==MaxPorts){
	       IPAddr.Hostname="-1";
	       IPAddr.Port=-1;
	  } else {
	       PortList.push_back(port);
	       IPAddr.Port=port;
	  }
     }else{
	  IPAddr.Hostname="-1";
	  IPAddr.Port=-1;
     }
     s.OutputStream() << IPAddr << "\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_FreeIPAddress(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_FreePort(..)");
     if(3==numparams){
 	  IPAddress_t IPAddr;
	  IPAddr.Hostname=params[1];
	  IPAddr.Port=atol(params[2]);
	  PortListIterator_t port_item=PortList.begin();
	  while((PortList.end()<port_item)&&
		((*port_item)!=IPAddr.Port)) port_item++;
	  if(port_item<PortList.end()){
	       PortList.erase(port_item);
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  } 
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_CreateSharedMemory(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_CreateSharedMemory(..)");
     if(3==numparams){
	  unsigned int BlockSize=atol(params[1]);
	  unsigned int NumBlocks=atol(params[2]);
	  if (CreateSharedMemory(BlockSize,NumBlocks)>=0){
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  }
     }
     s.OutputStream() << "-1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_DestroySharedMemory(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_DestroySharedMemory(..)");
     if (DestroySharedMemory()>=0){
	  s.OutputStream() << "0\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "0\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_GetShmManKeys(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_GetShmManKeys(..)");
     if(SharedMemCreated)
	  s.OutputStream() << shmMan_shmkey << " " 
			   << shmMan_semkey << "\r\n" << flush;
     else
	  s.OutputStream() << "-1 -1\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_IncRunNumber(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_IncRunNumber(..)");
     RunFileName_t RunFileName(this);
     if(RunFileName.Increment()<0){
	  s.OutputStream() << "-1\r\n" << flush;
	  return 0;
     }
     s.OutputStream() << "0\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_Check(IOSocket &s,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_Check(..)");
	char buffer[80];
	ProcessListIterator_t process = ProcessList.begin ();
	while (process != ProcessList.end () &&
			(*process)->pid.ProcessID != "HV")
		++process;
	if (ProcessList.end () == process)
	{
		s.OutputStream () << "-1\r\n" << flush;
		return 0;
	}
	else
	{
		ClientSocket CS;
		CS.Open ((*process)->IPCAddress);
		CS.WriteText ("status\r\n");
		CS.ReadText (buffer, 80);
		CS.Close ();
	}
//     s.OutputStream() << "1\r\n" << flush;
     s.OutputStream() << buffer << "\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_LogEntry(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_LogEntry(..)");
     String msg;
     s.ReadText(msg);
     if(2==numparams){
	  String &LogfileID=params[1];
	  LogFileListIterator_t LogFile=LogFileList.begin();
	  while((LogFile<LogFileList.end())&&
		(!((*LogFile)->LogfileID==LogfileID))){
	       LogFile++;
	  }
	  if(LogFileList.end()==LogFile){
	       s.OutputStream() << "-1\r\n" << flush;
	       return 0;
	  } else {
	       (*((*LogFile)->logfilestream)) << msg << endl << flush;
	       if(LogfileID!=LOG_TYPE_ERRORLOG)
		    s.OutputStream() << "0\r\n" << flush;
	  }
	  if(LogfileID==LOG_TYPE_ERRORLOG){
	       // errors are logged in both
	       // the error and the run log
	       LogfileID=LOG_TYPE_RUNLOG;
	       LogFileListIterator_t LogFile=LogFileList.begin();
	       while((LogFile<LogFileList.end())&&
		     (!((*LogFile)->LogfileID==LogfileID))){
		    LogFile++;
	       }
	       if(LogFileList.end()==LogFile){
		    s.OutputStream() << "-1\r\n" << flush;
	       } else {
		    (*((*LogFile)->logfilestream)) << msg << endl << flush;
		    s.OutputStream() << "0\r\n" << flush;
	       }
	  }
	  
     } else s.OutputStream() << "-1\r\n" << flush;
     return 0;     
}

int MainDispatcher::IPC_CMD_LogOpen(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_LogOpen(..)");     
     
     bool error=false;
     if(3==numparams){
	  const String &filename=params[1];
	  const String &LogfileID=params[2];
	  LogFileListIterator_t LogFile=LogFileList.begin();
	  while((LogFile<LogFileList.end())&&
		(!((*LogFile)->LogfileID==LogfileID)))
	       LogFile++;
	  if(LogFileList.end()==LogFile){
	       if(compare(filename,"none")==0){
		    s.OutputStream() << "-1\r\n" << flush;
		    return 0;
	       }
	       LogFile_t_p LF=new LogFile_t;
	       LF->LogfileID=LogfileID;
	       LF->FileName=filename;
	       ifstream infile(filename);
	       char tmp[512];
	       if(infile) {
		    char Line[512];
		    Line[0]='\0';
		    while(1){
			 infile.getline(Line,512);
			 if(infile.eof() || Line[0]!='\0') break;
		    }
		    String Str(Line);
		    if(Line[0]=='\0') Str=" ";
		    int r=Str.gsub("Log started: ","");
		    if(r==0) {
			 (*Error)=GENERIC_ERROR;
			 (*Error)="MainDispatcher::IPC_CMD_LogOpen(..)";
			 sprintf(tmp,"No start date in old log file '%s'.\n\
Name of old file changed to '%s.unknown'.",
				 (const char *)filename,
				 (const char *)filename);
			 Error->SetParam(tmp);
			 Error->WarningOnly();
			 error=true;
			 s.OutputStream() << "cmd_info\r\n"
					  << tmp << "\r\n" << flush;
			 
			 sprintf(tmp,"%s.unknown",(const char *)filename);
		    }
		    else {
			 Str.gsub(RXwhite,"_");
			 Str.gsub("__","_");
			 Str.gsub(":","_");
			 sprintf(tmp,"%s.%s",(const char *)filename,
				 (const char *)Str);
		    }
		    
		    if(rename(filename,tmp)<0){
			 (*Error)=SYS_ERROR;
			 (*Error)="MainDispatcher::IPC_CMD_LogOpen(..)";
			 s.OutputStream() << "-1\r\n" << flush;
			 Error->Print();
			 return -1;
		    }
	       }
	       LF->logfilestream=new ofstream;
	       LF->logfilestream->open(filename,ios::out|ios::trunc);
	       if(!(*LF->logfilestream)){
		    (*Error)=SYS_ERROR;
		    (*Error)="MainDispatcher::IPC_CMD_LogOpen(..)";
		    s.OutputStream() << "-1\r\n" << flush;
		    Error->Print();
		    return -1;
	       }else{
		    LF->NumOpens=1;
		    LogFileList.push_back(LF);
		    time_t t, dummy;
		    t=time(&dummy);
		    *(LF->logfilestream) << "Log started: " << ctime(&t) << "\n\n";
	       }
	       if(error){
		    Logger Entry;
		    Error->Print();
		    Entry() << Error->GetErrorMessage();
	       }
	       
	       s.OutputStream() << "0\r\n" << flush;
	  } else {
	       (*LogFile)->NumOpens++;
	       s.OutputStream() << "0\r\n" << flush;
	  }
     } else s.OutputStream() << "-1\r\n" << flush;
     return 0;     
}

int MainDispatcher::IPC_CMD_LogMove(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_LogMove(..)");     
     if(3==numparams){
	  const String &newfilename=params[1];
	  const String &LogfileID=params[2];
	  LogFileListIterator_t LogFile=LogFileList.begin();
	  while((LogFile<LogFileList.end())&&
		(!((*LogFile)->LogfileID==LogfileID)))
	       LogFile++;
	  if(LogFileList.end()==LogFile){
	       s.OutputStream() << "-1\r\n" << "Unknow log file ID '"
				<< (const char*)LogfileID << "'" << flush;
	       return 0;
	  }
	  (*LogFile)->logfilestream->close();
	  String cmd="mv " + (*LogFile)->FileName + " " + newfilename;
	  if(system(cmd)<0){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)=String("rename(")+(*LogFile)->FileName
		    +String(", ")+newfilename+String(")");
	       s.OutputStream() << "-1\r\n" << Base::Error->GetErrorMessage()
				<< "\r\n" << flush;   
	       return 0;
	  }
	  (*LogFile)->logfilestream->open(newfilename,ios::out|ios::app);
	  if(!(*LogFile)->logfilestream){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)=String("open(")+newfilename
		    +String(")");
	       s.OutputStream() << "-1\r\n" << Base::Error->GetErrorMessage()
				<< "\r\n" << flush;   
	       
	       return 0;
	  }
	  (*LogFile)->FileName=newfilename;
     } else s.OutputStream() << "-1\r\n" <<flush;
     s.OutputStream() << "0\r\n" <<flush;
     return 0;     
}

int MainDispatcher::IPC_CMD_LogAllowTransfer(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_LogAllowTransfer(..)");     
     if(2==numparams){
	  const String &LogfileID=params[1];
	  LogFileListIterator_t LogFile=LogFileList.begin();
	  while((LogFile<LogFileList.end())&&
		(!((*LogFile)->LogfileID==LogfileID)))
	       LogFile++;
	  if(LogFileList.end()==LogFile){
	       s.OutputStream() << "-1\r\n" << "Unknown log file ID '"
				<< (const char*)LogfileID << "'" << flush;
	       return 0;
	  }
	  (*LogFile)->logfilestream->close();
	  String NewFileName=(*LogFile)->FileName;
	  NewFileName.gsub(DO_NOT_TRANSFER,"");
	  String cmd="mv " + (*LogFile)->FileName + " " + NewFileName;
	  if(system(cmd)<0){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)=String("rename(")+(*LogFile)->FileName
		    +String(", ")+NewFileName+String(")");
	       s.OutputStream() << "-1\r\n" << Base::Error->GetErrorMessage()
				<< "\r\n" << flush;   
	       return 0;
	  }
	  if((--((*LogFile)->NumOpens))==0){
	       delete (*LogFile)->logfilestream;
	       LogFileList.erase(LogFile);
	  }
     } else s.OutputStream() << "-1\r\n" <<flush;
     s.OutputStream() << "0\r\n" <<flush;
     return 0;     
}

int MainDispatcher::IPC_CMD_LogClose(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_LogClose(..)");     
     if(2==numparams){
	  const String &LogfileID=params[1];
	  LogFileListIterator_t LogFile=LogFileList.begin();
	  while((LogFile<LogFileList.end())&&(!((*LogFile)->LogfileID==LogfileID)))
	       LogFile++;
	  if(LogFileList.end()==LogFile){
	       s.OutputStream() << "-1\r\n" << flush;
	       return 0;
	  }else{
	       if((--((*LogFile)->NumOpens))==0){
		    delete (*LogFile)->logfilestream;
		    LogFileList.erase(LogFile);
	       }
	       
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;	  
	  }
     } else  s.OutputStream() << "-1\r\n" << flush;
     return 0;     
}


int MainDispatcher::IPC_CMD_MailMsg(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_MailMsg(..)");
     String Msg,Subject="";
     s.ReadText(Msg);
     if(numparams>=2){
	  s.OutputStream() << "0\r\n" << flush;
	  for(int i=1;i<numparams;i++)
	       Subject+=params[i]+String(" ");
	  if(Mail)
	       Mailer.SendMail(Msg,Subject);
     } else s.OutputStream() << "-1\r\n" << flush;
     return 0;     
}

int MainDispatcher::IPC_CMD_UnknownCommand(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_UnknownCommand(..)");
     int a=1;
     a=a;
     return 0;
}

int MainDispatcher::IPC_CMD_WrongTypeCommand(IOSocket &/*s*/,String */*cmd*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_WrongTypeCommand(..)");
     int a=1;
     a=a;
     return 0;
}

int MainDispatcher::IPC_CMD_QuitProcess(IOSocket &/*s*/,String */*params*/,int /*numparams*/){
     DEB_F("int MainDispatcher::IPC_CMD_QuitProcess(..)");
     Quit_FLAG=true;
     return 0;
}

int MainDispatcher::IPC_CMD_GetStatus(IOSocket &s,String */*params*/,int /*numparams*/){
     ostream &out=s.OutputStream();
     out << "registered Processes: \n";
     ProcessListIterator_t pli=ProcessList.begin();
     while(pli<ProcessList.end()){
	  out << "ID: " << (*pli)->pid.ProcessID << " "
	      << "Pid:  " << (*pli)->pid.pid << " "
	      << "Host: " << (*pli)->pid.Hostname << endl
	      << "IP: " << (*pli)->IPCAddress.Hostname << ":" << (*pli)->IPCAddress.Port << " "
	      << "User: " << (*pli)->UserName << endl;
	  if((*pli)->NotifySharedMem)
	       out << "Notify SHM\n"; 
	  else
	       out << "Does not use SHM\n";
	  pli++;
     }
     out << "used free ports: \n";
     PortListIterator_t poli=PortList.begin();
     while(poli<PortList.end()){
	  out << (*poli) << " ";
	  poli++;
     }
     out << "\n";
     
     out << "Log Files: \n";
     LogFileListIterator_t lfli=LogFileList.begin();
     while(lfli<LogFileList.end()){
	  out << "ID: " << (*lfli)->LogfileID  << " ";
	  out << "Attached Processes: " << (*lfli)->NumOpens  << "\n";
	  lfli++;
     }
     if(SharedMemCreated){
	  out << "Shared Memory created\n";
     } else{
	  out << "Shared Memory not created\n";
     }
     out << "\r\n" << flush;
     return 0;
}

int MainDispatcher::IPC_CMD_RunStatStart(IOSocket &s,String *params,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_RunStatStart(..)");
     if(numparams==2){
	  time_t Start=time(NULL);
	  String Time=asctime(gmtime(&Start));
	  Time.gsub("\n","");
	  if(RunStat_t::RunStartEntry(params[1],Time)<0){
	       s.OutputStream() << "-1\r\n" << flush;
	       LogMsg(Base::Error->GetErrorMessage(true),2,2,WARNING);
	  }
	  else{
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  }
     }
     s.OutputStream() << "-1\r\n" << flush;
     LogMsg("RunStatStart: Wrong number of arguments",2,2,WARNING);
     return 0;
}

int MainDispatcher::IPC_CMD_RunStatStop(IOSocket &s,String */*params*/,int numparams){
     DEB_F("int MainDispatcher::IPC_CMD_RunStatStop(..)");
     if(numparams==1){
	  time_t Stop=time(NULL);
	  String Time=asctime(gmtime(&Stop));
	  Time.gsub("\n","");
 	  if(RunStat_t::RunStopEntry(Time)<0){
	       s.OutputStream() << "-1\r\n" << flush;
	       LogMsg(Base::Error->GetErrorMessage(true),2,2,WARNING);
	  }
	  else {
	       s.OutputStream() << "0\r\n" << flush;
	       return 0;
	  }
     }
     s.OutputStream() << "-1\r\n" << flush;
     LogMsg("RunStatStop: Wrong number of arguments",2,2,WARNING);
     return 0;
}

int MainDispatcher::IPC_CMD_Close(IOSocket &s,String */*params*/,int /*numparams*/){
     // remove the socket from the list
     vector<OpenSockets_p>::iterator si=OpenSockets.begin();
     while(si<OpenSockets.end()){
	  if((*si)->SSocket->GetLocalPort()==s.GetLocalPort()){
	       Selecter.RemoveNotifier((*si)->SSocketNotifier);
	       (*si)->SSocket->Close();
	       OpenSockets.erase(si);
	  }
	  si++;
     }       
     return 0;
}

int main(){
     DEB_F("main()");
     DaqProcessBase::InitializeGlobals("dispatcher");
     LogFile LF(DaqProcessBase::ThisProcess->GetName(),LogFile::APPEND);					
     Logger::SetLogger(&LF);				

     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }  
     MainDispatcher MyMainDispatcher;
     if(MyMainDispatcher.Init()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     }else{
	  /* signal parent process, that the process was started
	     successfully */
	  kill(getppid(),SIGUSR1);
	  if (MyMainDispatcher.EventLoop()<0){
	       Logger Entry;
	       (*Base::Error).Print(Entry());
	  }
     }
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0;
}
