/*
$Header: /home/elrond/baikal/src/CVS/daq98/dispatcher.h,v 1.3 1998/11/23 14:42:17 haase Exp $
*/

#if ! defined __DISPATCHER_H__
#define __DISPATCHER_H__ 
#include <config.h>
#include <strstream.h>
#include <String.h>
#include <vector.h>

#include "globaltypes.h"
#include "globalmacros.h"
#include "errorlog.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "memman.h"
#include "interpreter.h"
#include "daemonbase.h"
#include "pointerutil.h"
#include "privilege.h"
#include "mailer.h"

class MainDispatcher:public DaemonBase
{
     MAKE_REFCOUNT_POINTER(SlaveServerSocket,SlaveServerSocket_p);
public:
     /** Ctor */
     MainDispatcher();
     /** Dtor */
     ~MainDispatcher();
     /** initialization */
     int Init();     
     /** destruction */
     int Destroy();
     /** the event dispatcher, mus be overwritten to handle other than
         IPC events */
     virtual int DispatchEvent();
     /** servs a connection request on the IPCSocket ,
	 is called whenever something happend on the IPC socket */
     int ServIPCSocket();
     /** processes a message recieved on the IPC socket */
     int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** returns the Assigned ID -(string) of the process */
     virtual const String GetProcessType()const;
protected:
     /** returns the name of the configuration file name */
     virtual const String GetConfigFileName()const;
     /** returns a pointer to the configuration class, created with
         new on the heap */
     virtual ClassWithTypeInfo_t *GetConfigSection();	       

private:
     int CreateSharedMemory(unsigned int BlockSize,unsigned int NumBlocks);
     int DestroySharedMemory();
     int CreateKeys(key_t *shmkey,key_t *semkey,int numkeys);
     int SendMessage(const IPAddress_t &IPCAddress,const String &msg);
     
     struct OpenSockets_t
     {
	  /** the socket we have opend to this process */
	  SlaveServerSocket_p SSocket;
	  /** the notifier for the socket */
	  SocketNotifier SSocketNotifier;
     };
     /** a list of all Sockets, taht we have opend to all processes */
     MAKE_REFCOUNT_POINTER(OpenSockets_t,OpenSockets_p);
     vector<OpenSockets_p> OpenSockets;
     
     /** struct to store the information of a registered process */
     struct Process_t
     {
	  enum shm_caps_t{
	       SHM_WANTS_TO_BE_NOTIFIED,
	       SHM_CAN_CREATE_DESTROY
	  };
	  
	  /** the process id, (pid and hostname and ID string of the process) */
	  PID_t pid;
	  /** the IPC address of that process */
	  IPAddress_t IPCAddress;
	  /** will the process be notified on SHM creation/deletion */
	  bool NotifySharedMem;
	  /** the name of the user that started the process */
	  String UserName;
	  /** the local port of the socket that is connected to the
              process */
	  port_t LocalPort;
     }; 
     /** a pointer to a Process_t structure */
     typedef PointerWrapper_Ref<Process_t> Process_t_p;
     /** list of pointers to Process_t structures, stores the
         information of all registered processes */
     typedef vector<Process_t_p> ProcessList_t;
     /** the  iterator  for the ProcessList */
     typedef ProcessList_t::iterator ProcessListIterator_t;     

     /** struct to store information about open log files */
     struct LogFile_t
     {
	  /** the file stream */
	  ofstream *logfilestream;
	  /** the Type of the log file */
	  String LogfileID;
	  /** the number of processes that opend thsi log file */
	  int NumOpens;
	  /** The Filename */
	  String FileName;
     };
     /** a pointer to a LogFile_t structure */
     typedef PointerWrapper_Ref<LogFile_t> LogFile_t_p;
     /** a list of pointers to LogFile_t structures, used to storee
         all ionformation about Log Files */
     typedef vector<LogFile_t_p> LogFileList_t;
     /** the interator for LogFileList */
     typedef LogFileList_t::iterator LogFileListIterator_t;

     /** list of IP Addresses, is used to store temporary allocated
         IP adresses for free usage */
     typedef vector<port_t> PortList_t;

     /** the iterator for the IPAdressList */
     typedef PortList_t::iterator PortListIterator_t;     

     /** contains all registered processes */
     ProcessList_t ProcessList;
     /** contains all used ports, to ensure that no port is used twice */
     PortList_t PortList;
     /** contains all open logfiles */
     LogFileList_t LogFileList;

     /** the privileges of the users */
     Privilege UserPrivs;
     /* PN							*/
     /* Add Priv with UserPrivs.Add ("USER", "PRIV", true/false);	*/
     /* Check with UserPrivs.IsSet ("USER", "PRIV");			*/
     /* Set with UserPrivs.Set ("USER", "PRIV");			*/
     /* Clear with UserPrivs.Clear ("USER", "PRIV");			*/

     /** the keys for the shared Memory and for the Semaphore */
     key_t shmMan_shmkey,shmMan_semkey;
     /** the shared Memory Manager */
     SharedMemoryManager shmMan;
     /** flag that indicates whether the SHM was created or not */
     bool SharedMemCreated;

     /** port range for the use for IPC ports, is read from config file */
     port_t IPC_Port_first,IPC_Port_last;

     /** port range for the allocation fo "free use" ports */ 
     port_t FREE_USE_Port_first,FREE_USE_Port_last;

     /** the command interpreter */
     Interpreter<MainDispatcher> IPC_CMD_Interpreter;
     
     /** Mail notifier.
	 Informs users via email in case of errors */
     Mailer_t Mailer;
     
     /** the functions that are called via the command interpreter */
     int IPC_CMD_RegisterProcess(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_UnregisterProcess(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_SetUser(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_GetUser(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_AddPriv(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_SetPriv(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_ClearPriv(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_IsSetPriv(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_Broadcast(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_GetProcessPortLock(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_FreeProcessPortLock(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_QuitProcess(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_AllocIPAddress(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_FreeIPAddress(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_CreateSharedMemory(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_DestroySharedMemory(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_GetShmManKeys(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_Check(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_LogEntry(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_LogOpen(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_LogMove(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_LogAllowTransfer(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_LogClose(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_MailMsg(IOSocket &s,String *params,int numparams);
     int IPC_CMD_SetAutoRestart(IOSocket &s,String *params,int numparams);
     int IPC_CMD_RunStatStart(IOSocket &s,String *params,int numparams);
     int IPC_CMD_RunStatStop(IOSocket &s,String *params,int numparams);
     int IPC_CMD_UnknownCommand(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_GetStatus(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_GetPID(IOSocket &s,String *params,int numparams);
     /** */
     int IPC_CMD_Close(IOSocket &s,String *params,int numparams);

     /** */
     int IPC_CMD_IncRunNumber(IOSocket &s,String *params,int numparams);
};

#endif //__DISPATCHER_H__













