/*
  $Header: /home/elrond/baikal/src/CVS/daq98/dispatcherlib.C,v 1.2 1998/11/23 14:42:20 haase Exp $
*/

#include <strstream.h>
#include <String.h>
#include <list.h>

#include "globaltypes.h"
#include "globalmacros.h"
#include "errorlog.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "memman.h"
#include "interpreter.h"
#include "dispatcherlib.h"
#include "transferclass.h"

DispatcherLib_t::DispatcherLib_t(){
     DEB_F("DispatcherLib_t()");
}

DispatcherLib_t::~DispatcherLib_t(){
     DEB_F("~DispatcherLib_t()");
}

int DispatcherLib_t::SendMessage(const String &msg){
     DEB_F("int DispatcherLib_t::SendMessage(..)");
     CS.OutputStream() << msg << flush;
     return 0;
}

// message must end with "\r\n" !!!!!!!!!!!!!!!!!!!!!!!!!!!!
int DispatcherLib_t::Broadcast(const String &ProcessID,const String &msg){
     DEB_F("int DispatcherLib_t::Broadcast(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "Broadcast "
		       << ProcessID << "\r\n"
		       << msg << "_*_BroadcastEnd_*_\r\n" << flush;
     return 0;
}

IPAddress_t DispatcherLib_t::Register(PID_t &pid,bool NotifySharedMem){
     DEB_F("DispatcherLib_t::Register(..)");
     IPAddress_t IPCAddress;

     if(CS.Open(DispatcherAddress)<0)return IPCAddress;;
     if(CS.CheckOutStream()<0)return IPCAddress;
     String nshm;
     if(NotifySharedMem) nshm="1"; else nshm="0";
     CS.OutputStream() << "RegisterProcess "
		       << pid.pid << " "
		       << pid.Hostname << " "
		       << pid.ProcessID << " "
		       << nshm << "\r\n" << flush;
     if(CS.CheckInStream()<0) return IPCAddress;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> IPCAddress;
     // read the reast of the line
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);

     if(IPCAddress.Port<=0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="Register";
     }
     return IPCAddress;
}

int DispatcherLib_t::Unregister(PID_t &pid){
     DEB_F("int DispatcherLib_t::Unregister(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "UnregisterProcess "
		       << pid.pid << " "
		       << pid.Hostname << " "
		       << pid.ProcessID << "\r\n" << flush;
     
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret ;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     CS.Close();
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="Unregister";
     }
     return ret;
}

pid_t DispatcherLib_t::GetPID(String &ProcessID){
     DEB_F("int DispatcherLib_t::Unregister(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "GetPID "
		       << ProcessID 
		       << "\r\n" << flush;
     
     TEST_FAILURE(CS.CheckInStream());
     pid_t ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret ;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     CS.Close();
     if(ret==0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="GetPID";
     }
     return ret;
}

int DispatcherLib_t::SetUser(const IPAddress_t &IPC_IPAddr,const String &User){
     DEB_F("int DispatcherLib_t::SetUser(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "SetUser "
		       << IPC_IPAddr.Hostname << " "
		       << IPC_IPAddr.Port << " " 
		       << User << "\r\n" << flush;
     
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="SetUser";
     }
     return ret;
}

String DispatcherLib_t::GetUser(const IPAddress_t &IPC_IPAddr){
     DEB_F("int DispatcherLib_t::GetUser(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "GetUser "		// altered Set to Get *PN*
		       << IPC_IPAddr.Hostname << " "
		       << IPC_IPAddr.Port << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     char txt[MAX_TEXT_SIZE];
     String User;
     CS.InputStream() >> User;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     return User;
}

void DispatcherLib_t::Quit(){
     ClientSocket cs;
     cs.Open(DispatcherAddress);
     cs.CheckOutStream();
     cs.OutputStream() << "bye\r\n" << flush;
}


int DispatcherLib_t::AddPriv (const String &User, const String &Priv, bool init){
     DEB_F ("int DispatcherLib_t::AddPriv(..)");
     TEST_FAILURE(CS.CheckOutStream());
     String in;
     if(init) in="1"; else in="0";
     CS.OutputStream() << "AddPriv "
		       << User << " "
		       << Priv << " "
		       << in << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="AddPriv";
     }
     return ret;
}

int DispatcherLib_t::SetPriv (const String &User, const String &Priv){
     DEB_F ("int DispatcherLib_t::SetPriv(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "SetPriv "
		       << User << " "
		       << Priv << "\r\n" << flush;
     
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="SetPriv";
     }
     return ret;
}


int DispatcherLib_t::ClearPriv (const String &User, const String &Priv){
     DEB_F ("int DispatcherLib_t::ClearPriv(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "ClearPriv "
		       << User << " "
		       << Priv << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="ClearPriv";
     }
     return ret;
}

bool DispatcherLib_t::IsSetPriv (const String &User, const String &Priv){
     DEB_F ("int DispatcherLib_t::IsSetPriv(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "IsSetPriv "
		       << User << " "
		       << Priv << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     bool ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     return ret;
}

IPAddress_t DispatcherLib_t::GetPortLock(const String &ProcessID,const String &User){
     DEB_F("DispatcherLib_t::GetPortLock(..)");
     IPAddress_t IPAddress;
     do{	
	  if(CS.CheckOutStream()<0) return IPAddress;
	  CS.OutputStream() << "GetProcessPortLock "
			    << ProcessID << " "
			    << User << "\r\n" << flush;
	  if(CS.CheckInStream()<0) return IPAddress;
	  char txt[MAX_TEXT_SIZE];
	  CS.InputStream() >> IPAddress;
	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     }while(0==IPAddress.Port);
     if(IPAddress.Port<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="GetPortLock";
     }
     return IPAddress;
}

int DispatcherLib_t::FreePortLock(const IPAddress_t &IPAddress){
     DEB_F("DispatcherLib_t::FreePortLock(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "FreeProcessPortLock "
		       << IPAddress.Hostname << " "
		       << IPAddress.Port << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="FreePortLock";
     }
     return ret;
}

IPAddress_t DispatcherLib_t::AllocIPAddress(const String &Hostname){
     IPAddress_t IPAddress;

     if(CS.CheckOutStream()<0) return IPAddress;
     CS.OutputStream() << "AllocIPAddress " 
		       << Hostname << "\r\n" << flush;

     if(CS.CheckInStream()<0)return IPAddress;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> IPAddress;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(IPAddress.Port<=0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="AllocPort";
     }
     return IPAddress;
}

int DispatcherLib_t::FreeIPAddress(const IPAddress_t &IPAddress){
     DEB_F("int DispatcherLib_t::FreePort(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "FreeIPAddress "
		       << IPAddress.Hostname << " "
		       << IPAddress.Port << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="FreePort";
     }
     return ret;
}

int DispatcherLib_t::CreateSharedMemory(unsigned int BlockSize,unsigned int NumBlocks){
     DEB_F("int DispatcherLib_t::CreateSharedMemory(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "CreateSharedMem "
		       << BlockSize << " "
		       << NumBlocks << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="CreateSharedMemory";
     }
     return ret;
}

int DispatcherLib_t::DestroySharedMemory(){
     DEB_F("int DispatcherLib_t::DestroySharedMemory(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "DestroySharedMem\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="DestroySharedMemory";
     }
     return ret;
}


int DispatcherLib_t::GetShmManKeys(key_t &shmkey,key_t &semkey){
     DEB_F("int DispatcherLib_t::GetShmManKeys(..)");
     shmkey=semkey=0;
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "GetShmManKeys\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> shmkey >> semkey;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     return 0;
}

int DispatcherLib_t::Check(){
     DEB_F("int DispatcherLib_t::Check()");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "Check\r\n";
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="check";
     }
     return ret;
}

int DispatcherLib_t::LogEntry(const String &LogfileID,const char* msg){
     DEB_F("int DispatcherLib_t::LogEntry(..)");
     TEST_FAILURE(CS.CheckOutStream());
     time_t curr_time=time(NULL);
     int ret;
     if(strlen(msg)>MAX_STRING_SIZE){
	  char *str=new char[strlen(msg)+1];
	  int c=(strlen(msg)+1)/MAX_STRING_SIZE;
	  c++;
	  for(int i=0;i<c;i++){
	       strcpy(str,msg+i*MAX_STRING_SIZE);
	       str[MAX_STRING_SIZE]='\0';
	       if(i==0) {
		    CS.OutputStream() << "LogEntry "
				      << LogfileID << "\r\n"
				      << "----------------------------------------------------\n"
				      << ctime(&curr_time)
				      << ThisProcess->GetName() << ": "
				      << str <<"\r\n" << flush;
		    TEST_FAILURE(CS.CheckInStream());
		    char txt[MAX_TEXT_SIZE];
		    CS.InputStream() >> ret;
		    CS.InputStream().getline(txt,MAX_TEXT_SIZE);
		    if(ret<0){
			 (*Error)=DISPATCHER_ERROR;
			 (*Error)="LogEntry";
		    }
	       }
	       else {
		    TEST_FAILURE(CS.CheckOutStream());
		    CS.OutputStream() << "LogEntry "
				      << LogfileID << "\r\n"
				      << str <<"\r\n" << flush;
		    TEST_FAILURE(CS.CheckInStream());
		    char txt[MAX_TEXT_SIZE];
		    CS.InputStream() >> ret;
		    CS.InputStream().getline(txt,MAX_TEXT_SIZE);
		    if(ret<0){
			 (*Error)=DISPATCHER_ERROR;
			 (*Error)="LogEntry";
		    }
	       }
	  }
     }
     else {
	  CS.OutputStream() << "LogEntry "
			    << LogfileID << "\r\n"
			    << "----------------------------------------------------\n"
			    << ctime(&curr_time)
			    << ThisProcess->GetName() << ": "
			    << msg <<"\r\n" << flush;
	  TEST_FAILURE(CS.CheckInStream());
	  char txt[MAX_TEXT_SIZE];
	  CS.InputStream() >> ret;
	  CS.InputStream().getline(txt,MAX_TEXT_SIZE);
	  if(ret<0){
	       (*Error)=DISPATCHER_ERROR;
	       (*Error)="LogEntry";
	  }
     }
     
     return ret;
}

int DispatcherLib_t::LogOpen(const String &LogfileID,const String &filename="none"){
     DEB_F("int DispatcherLib_t::LogOpen(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "LogOpen "
		       << filename << " "
		       << LogfileID << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="LogOpen";
     }
     return ret;
}

int DispatcherLib_t::LogClose(const String &LogfileID){
     DEB_F("int DispatcherLib_t::LogOpen(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "LogClose "
		       << LogfileID << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="LogClose";
     }
     return ret;
}

int DispatcherLib_t::LogMove(const String &LogFileID, const String &NewName)
{
     DEB_F("int DispatcherLib_t::LogMove(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "LogMove "
		       << NewName << " "
		       << LogFileID << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)=String("LogMove: ") + txt;
     }
     return ret;
}

int DispatcherLib_t::LogAllowTransfer(const String &LogFileID)
{
     DEB_F("int DispatcherLib_t::LogAllowTransfer(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "LogAllowTransfer "
		       << LogFileID << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)=String("LogAllowTransfer: ") + txt;
     }
     return ret;
}

int DispatcherLib_t::MailMsg(const String &Msg,const String &Subject){
     DEB_F("int DispatcherLib_t::MailMsg(..)");
     TEST_FAILURE(CS.CheckOutStream());
     time_t curr_time=time(NULL);
     int ret;
     String Time=ctime(&curr_time);
     Time.gsub("\n","");
     String tmp=Subject+String(" ")+Time;
     CS.OutputStream() << "MailMsg " << tmp
		       <<"\r\n" << flush;
     CS.OutputStream() << Msg << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="MailMsg";
     }
     return ret;
}

int DispatcherLib_t::GetStatus(String &Status)
{
     DEB_F("int DispatcherLib_t::GetStatus(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "GetStatus"
		       <<"\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     char txt[MAX_TEXT_SIZE];
     CS.ReadText(txt,MAX_TEXT_SIZE);
     Status=txt;
     Status.gsub("\r\n","");
     return 0;
}

int DispatcherLib_t::IncRunNumber(){
     DEB_F("int DispatcherLib_t::IncRunNumber(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "IncRunNumber\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="IncRunNumber";
     }
     return ret;
}

int DispatcherLib_t::RunStatStart(String &RunName){
     DEB_F("int DispatcherLib_t::RunStatStart(..)");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "RunStatStart " << RunName << "\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="RunStatStart";
     }
     return ret;
}

int DispatcherLib_t::RunStatStop(){
     DEB_F("int DispatcherLib_t::RunStatStop()");
     TEST_FAILURE(CS.CheckOutStream());
     CS.OutputStream() << "RunStatStop\r\n" << flush;
     TEST_FAILURE(CS.CheckInStream());
     int ret;
     char txt[MAX_TEXT_SIZE];
     CS.InputStream() >> ret;
     CS.InputStream().getline(txt,MAX_TEXT_SIZE);
     if(ret<0){
	  (*Error)=DISPATCHER_ERROR;
	  (*Error)="RunStatStop";
     }
     return ret;
}
     

     
