#if ! defined __DMADMODULE_H__
#define  __DMADMODULE_H__

#include <config.h>
#include "channelmodule.h"

struct DMAD_ChannelConfigRecord_t{
     DMAD_ChannelConfigRecord_t()
	  :NoOfBytes(0),
	   ID(0),
	   DMAD_On(0),
	   DMAD_CurrentState_On(0),
	   MULT20_Address(0),
	   MULT20_Channel(0){}
     size_t ReadFromBuffer(BinaryDataBuffer &Buf){
	  Buf.RecordBegin();
	  // read all data elements
	  Buf.ReadFromBuffer(NoOfBytes);

	  Buf.RecordBegin();
	  Buf.ReadFromBuffer(ID);
	  Buf.ReadFromBuffer(DMAD_On);
	  Buf.ReadFromBuffer(DMAD_CurrentState_On);
	  Buf.ReadFromBuffer(MULT20_Address);
	  Buf.ReadFromBuffer(MULT20_Channel);
	  u_i32 NoOfBytes_r=Buf.RecordEnd_GetSize();
	  if(NoOfBytes!=NoOfBytes_r){
	       // error read bytes != the specified size
	  }

	  // return the size in bytes, that we read
	  return Buf.RecordEnd_GetSize();
     }
     size_t WriteToBuffer(BinaryDataBuffer &Buf){
	  // Mark the current position as a new Record
	  Buf.RecordBegin();
	  // write all data elements
	  Buf.PushPosition();
	  Buf.WriteToBuffer(NoOfBytes);

	  Buf.RecordBegin();
	  Buf.WriteToBuffer(ID);
	  Buf.WriteToBuffer(DMAD_On);
	  Buf.WriteToBuffer(DMAD_CurrentState_On);
	  Buf.WriteToBuffer(MULT20_Address);
	  Buf.WriteToBuffer(MULT20_Channel);
	  	
	  NoOfBytes=Buf.RecordEnd_GetSize();
	  Buf.PopPosition();
	  Buf.WriteToBuffer(NoOfBytes);
	  Buf.IncPosition(NoOfBytes);
	  // return the size in bytes, that we wrote
	  return Buf.RecordEnd_GetSize();
     }     

     u_i32 NoOfBytes;
     ID_t ID;
     u_i32 DMAD_On;
     u_i32 DMAD_CurrentState_On;
     u_i32 MULT20_Address;
     u_i32 MULT20_Channel;
};


MAKE_REFCOUNT_POINTER(DMAD_ChannelConfigRecord_t,DMAD_ChannelConfigRecord_p);
MAKE_INC_VECTOR(DMAD_ChannelConfigRecord_p,DMAD_ChannelConfigRecordVector_t);

class DMAD_ModuleConfigRecord_t:public ChannelModuleConfigRecord_t{
     ClassTypeInfo(ChannelModuleConfigRecord_t);

     u_i32 NoOfChannels;
     DMAD_ChannelConfigRecordVector_t DMAD_ChannelConfigRecordVector;
};

MAKE_REFCOUNT_POINTER(DMAD_ModuleConfigRecord_t,DMAD_ModuleConfigRecord_p);

class DMAD_Module_t:public ChannelModuleBaseClass_t{
     ClassTypeInfo(ChannelModuleBaseClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 0,
	  BYTES_PER_INFO_UNIT= 0        //  header word, 2 data words per hit (& log-info)
     };
public:
     
     DMAD_Module_t(DaqProcessBase *Parent):
	  ChannelModuleBaseClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,Parent){}
    /***/
     ~DMAD_Module_t(){
     }
     
     /** returns true, if the module has data or false if the module
         does not have any data */
     virtual bool HasModuleData(){
	  return true;
     }

     /** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);
     	
     /** this function initializes the TDC module */
     virtual int InitModule(StringVector_t &errors);
     /** Resets the module 
       @param errors Vector of errors that may occur during initialization.
       @return 0 on success. Negative number of errors occured during 
       initialization on failure.*/
     virtual int ResetModule(StringVector_t &errors){
	  return 0;
     }
};

MAKE_REFCOUNT_POINTER(DMAD_Module_t, DMAD_Module_p);

#endif // __DMADMODULE_H__
