/*
$Header: /home/elrond/baikal/src/CVS/daq98/error.C,v 1.4 1999/02/03 03:42:17 steffen Exp $
*/

/*------------------------------------------------------------------------------
   Files: error.C 
   Author: Heiko Haase 
           Institution: DESY - IFH Zeuthen
   
   Classes:
   implements
   ErrorClass_t -> Class for handling of errors
------------------------------------------------------------------------------*/
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <strstream.h>

#include "globalmacros.h"
#include "errornum.h"
#include "process.h"
#include "error.h"

/** Global instance of the error-object */
//ErrorClass_t Error;

ErrorClass_t::ErrorMap_t ErrorClass_t::ErrorMap[MAX_ERRORS] = {
     // fill in the database with the error-messages and the belonging numbers
     // the number are defined in errornum.h
     { SYS_ERROR,                   "System lib error"},
     { PROC_NOT_CHILD_OF,           "Process #%s is not child of process #%s"},
     { TOO_MANY_CHILDREN,           "Too many children for process #%s"},
     { IPC_ACCESS_SHMWOSEM,         "Tried to access shared memory without using semaphore"},
     { IPC_NAME_TOO_LONG,           "Identifier name too long: %s"},
     { IPC_NO_SUCH_RESOURCE,        "No IPC resource with the name %s"},   
     { IPC_NO_SPACE_FOR_RESOURCE,   "No space left for IPC recources"},
     { SOCKET_TIMEOUT,              "Socket connection timeout"},
     { NO_IPC_MANAGER,              "Can't connect/create IPC manager"},
     { NO_MESSAGEQ,                 "Can't connect to MessageQ"},
     { NO_CONNECTION_ESTABISHED,    "Can't establish connection"},
     { PROCESS_ALLREADY_RUNNING,    "Process already running"},
     { CLIENT_DIED,                 "Client did not respond, assuming it died"},
     { SOCKETSTREAM_NOT_READABLE,   "Can't read from socket stream"},
     { SOCKETSTREAM_NOT_WRITEBLE,   "Can't write to socket stream"},
     { COULD_NOT_START_DISPATCHER,  "The Dispatcher could not be started"},
     { SOCKET_NOT_CONNECTED,        "Socket is not connected or connection broken"},
     { NO_SPACE_IN_SHM,             "No space left in shared memory"},
     { COULD_NOT_FIND_ID_IN_SHM,    "Can't not find the block with the id %s"},
     { COULD_NOT_READ_FROM_INSTREAM,"Can't read from input stream"},
     { BUFFER_TOO_SMALL,            "Buffer too small for operation"},
     { HISTOGRAM_EXISTS,            "Histogram already exists"},
     { DISPATCHER_ERROR,            "Dispatcher failed doing command"},
     { FILE_INTERPRETER_ERROR,      "Interpreter failed to do command in line %s"},
     { ERROR_IN_CONFIG_FILE,        "Error in configuration file"},
     { NO_CONFIG_FILE,              "No configuration file found"},
     { INCOMPLETE_CONFIG_FILE,      "Incomplete configuration file"},
     { ERROR_RECEIVING_HISTOGRAM,   "Can't recieve histogram successfully"},
     { NO_CONFIGURATION_FOUND,      "Can't find the configuration path %s "},
     { UNKNOWN_FLAG,                "Unknown flag: %s"},
     { UNKNOWN_OPTION,              "Unknown option: %s"},
     { GENERIC_ERROR,               "%s"}
     
};

ErrorClass_t::ErrorClass_t(ostream *err){
     ErrorStream=err; 
#if defined DEBUG
     Errorline=0;
#endif
}

ErrorClass_t::~ErrorClass_t(){}

void ErrorClass_t::SetStream(ostream *err){ErrorStream=err;}

void ErrorClass_t::Location(const String &loc){location=loc;}

#if defined DEBUG
void ErrorClass_t::FileLocation(const String &file, const int line){
     Errorfile=file;
     Errorline=line;
}
#endif

void ErrorClass_t::Print(){
     char s[1000];
     *ErrorStream << "----------------------------------------------------------------------" 
		  << endl;
     if(Warning)
	  *ErrorStream << "WARNING OF PROCESS " ;
     else
	  *ErrorStream << "ERROR OF PROCESS ";
     *ErrorStream << DaqProcessBase::ThisProcess->GetName() 
		  << " with pid " 
		  <<  getpid() 
		  << endl;
#if defined DEBUG
     if(Errorline>0)
	  *ErrorStream << " in C-file " 
		       << Errorfile 
		       << " at line " 
		       << Errorline 
		       << endl;
#endif
     if (location.length()>0){
	if(Warning)
	     *ErrorStream << "Warning: ";
	else 
	     *ErrorStream << "***Error***: ";
	*ErrorStream  << location  
			<< endl;
     }
     
     if (number==SYS_ERROR){
	  if(param[0].length()>0){
	       *ErrorStream << (const char*)param[0] << endl;
	  }
     	  *ErrorStream << strerror(errno) << endl << flush ;
     } else {
	  int i;
	  for (i=0;i<MAX_ERRORS;i++){
	       if (ErrorMap[i].number==number){
		    if(param[3].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1],
				 (const char*)param[2],
				 (const char*)param[3]);
			 *ErrorStream << s << endl << flush;
		    } else if(param[2].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1],
				 (const char*)param[2]);
			 *ErrorStream << s << endl << flush;
		    } else if(param[1].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1]);
			 *ErrorStream << s << endl << flush;
		    } else if(param[0].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0]);
			 *ErrorStream << s << endl << flush;
		    } else {
			 *ErrorStream << ErrorMap[i].message << endl << flush;
		    }
	       } 
	  }
// 	  if(MAX_ERRORS==i){
// 	       *ErrorStream << "Unknwon Error !" << endl << flush;
// 	  }
     }
     *ErrorStream << "----------------------------------------------------------------------" << endl;
}     

const String ErrorClass_t::GetErrorMessage(bool NoHead){
     ostrstream outstream;
     Print(outstream,NoHead);
     outstream << ends;
     outstream.freeze(1);
     String em(outstream.str());
     outstream.freeze(0);
     return em;
}     

void ErrorClass_t::Print(ostream &out, bool NoHead){
#if defined DEBUG
     if(Errorline>0)
	  out << " in C-file " << Errorfile << " at line " << Errorline << endl;
#endif
     if(location.length()>0)
	  if(!NoHead){
	       if(!Warning) 
		    out << "***Error***: ";
	       else
		    out << "Warning: ";
	  }
     out << location  << endl;
     if(number==SYS_ERROR){
	  if(param[0].length()>0){
	       out << (const char*)param[0] << endl;
	  }
	  out << strerror(errno) << endl << flush ;
	  return;
     }else{
	  int i;
	  for (i=0;i<MAX_ERRORS;i++){
	       if(ErrorMap[i].number==number){
		    char s[1000];
		    if(param[3].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1],
				 (const char*)param[2],
				 (const char*)param[3]);
			 out << s << endl << flush;
		    } else if(param[2].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1],
				 (const char*)param[2]);
			 out << s << endl << flush;
		    } else if(param[1].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0],
				 (const char*)param[1]);
			 out << s << endl << flush;
		    }else if(param[0].length()>0){
			 sprintf(s,ErrorMap[i].message,
				 (const char*)param[0]);
			 out << s << endl << flush;
		    } else {
			 out<< ErrorMap[i].message << endl << flush;
		    }
	       }
	  }
// 	  if(MAX_ERRORS==i){
// 	       out<< "Unknwon Error !" << endl << flush;
// 	  }
     }
}

int ErrorClass_t::operator=(int nr){
     Warning=false;
     return number=nr;
}

const String &ErrorClass_t::operator=(const String &loc){
     Location(loc);
     return loc;
}

void ErrorClass_t::SetParam(const String &p1,
			       const String &p2="",
			       const String &p3="",
			       const String &p4=""){
     param[0]=p1;
     param[1]=p2;
     param[2]=p3;
     param[3]=p4;
}

void ErrorClass_t::WarningOnly()
{
     Warning=true;
}












