/*
$Header: /home/elrond/baikal/src/CVS/daq98/event.C,v 1.7 1999/02/05 18:35:05 steffen Exp $
*/

#include <string.h>

#include "globalmacros.h"
#include "globaltypes.h"
#include "memman.h"
#include "netconv.h"
#include "datafilecode.h"
#include "event.h"
#include "moduleevalbase.h"

#define __EVALUATOR__
#include "register.h"

EventBuilder_t::EventBuilder_t(DaqProcessBase *parent)/***/
{
     MaxChannelID=0;
     TempEventBuffer=0;
     Parent=parent;
}

int EventBuilder_t::SetUpDetectorConfig(HWConfig_t &HWConfig){

     ModuleEvalVector_t::iterator i;
     for(i=ModuleEvalVector.begin();i<ModuleEvalVector.end();i++)
	  delete (*i);
     ModuleEvalVector.Clear();     

     ChannelModuleConfigRecordVector_t::iterator mcr_i;

     for(mcr_i=HWConfig.ModuleConfigRecordVector->begin();
	 mcr_i<HWConfig.ModuleConfigRecordVector->end();
	 mcr_i++){
	  
	  ModuleEvalBaseClass_t *ModuleEvaluator=GetModuleEvaluator((*mcr_i)->ModuleType);
	  if(ModuleEvaluator==0){
	       // no evaluator for module, better to continie without it
	       continue;
//	       return -1;
	  }    
	  // configure each Module evaluator
	  ModuleEvaluator->Configure((*mcr_i),HWConfig);
	  ModuleEvalVector.push_back(ModuleEvaluator);
     }
     ChannelConfigRecordVector_t::iterator chcr_i;
     ChannelGroup_p CurGrp;
     
     for(chcr_i=HWConfig.ChannelConfigRecordVector->begin();
	 chcr_i<HWConfig.ChannelConfigRecordVector->end();
	 chcr_i++){
	  // find max. channel ID
	  MaxChannelID=MaxChannelID>(*chcr_i)->ID?MaxChannelID:(*chcr_i)->ID;
	  // setup groups
	  if((*chcr_i)->Group!=NONE_val){
	       if((BEGIN_END_t)(*chcr_i)->Group==BEGIN_val){
		    CurGrp=new ChannelGroup_t;
		    CurGrp->start=(*chcr_i)->ID;
	       }
	       if((BEGIN_END_t)(*chcr_i)->Group==END_val){
		    CurGrp->end=(*chcr_i)->ID;
		    ChannelGroupVector.push_back(CurGrp);
	       }
	  }
     }
     if(ChannelGroupVector.size()==0) {
	  CurGrp=new ChannelGroup_t;
	  CurGrp->start=0;
	  CurGrp->end=MaxChannelID;
	  ChannelGroupVector.push_back(CurGrp);
     }
     ChannelGroupVector_t::iterator t;
     if(Parent->IsVerboseLevel(8)) {
	  ostrstream str_1;
	  str_1 << "Channelgroups:\n";
	  for(t=ChannelGroupVector.begin();t< ChannelGroupVector.end();t++){
	       str_1 << (*t)->start << " " << (*t)->end << endl;
	  }
	  str_1 << ends;
	  str_1.freeze(1);
	  Parent->LogMsg(str_1.str(),8,8);
	  str_1.freeze(0);
     }
     
     TempEventBuffer=new TempDecode_EventData_t(MaxChannelID+1);               // = 1 (!!), ..., NOM
     return 0;
}

EventBuilder_t::~EventBuilder_t(){
     if(0!=TempEventBuffer) delete TempEventBuffer;
     ModuleEvalVector_t::iterator i;
     for(i=ModuleEvalVector.begin();i<ModuleEvalVector.end();i++)
	  delete (*i);
}

int EventBuilder_t::BuildEvent_fast(BinaryDataBuffer &RawDataBuffer, 
				    MuonEventHeader_t &EventHeader,
				    MuonEventData_p EventDataBuffer){
     static ChannelGroupVector_t::iterator gi;
     
     SubEventData_StaticSize_t *CurSubEvent;
     BuildGeneralEvent(RawDataBuffer);
     EventHeader.NHitADC=0;
     EventHeader.NHitTDC=0;
     time_t time=EventHeader.UnixTime;
     struct tm *t=gmtime(&time);
     TempEventBuffer->UnixTime.sec=t->tm_sec;
     TempEventBuffer->UnixTime.min=t->tm_min;
     TempEventBuffer->UnixTime.hour=t->tm_hour;
     TempEventBuffer->UnixTime.day=t->tm_yday;
     TempEventBuffer->UnixTime.year=t->tm_year;
     EventDataBuffer->NoOfChannels=0;
     SubEventData_StaticSize_t *TargetSubEvent=EventDataBuffer->SubEvents;
     while(TempEventBuffer->ReadChannelData(*TargetSubEvent)){
	  if(TargetSubEvent->Amplitude.isamp!=0)
	       EventHeader.NHitADC++;
	  if(TargetSubEvent->NoOfEdges!=0)
	       EventHeader.NHitTDC++;
	  EventDataBuffer->NoOfChannels++;
	  TargetSubEvent++;
     }
     
//      for(gi=ChannelGroupVector.begin();gi<ChannelGroupVector.end();gi++){
// 	  for(int i=(*gi)->start;i<=(*gi)->end;i++){
// 	       if((CurSubEvent=TempEventBuffer->SubEvents+i)->ID!=0){
// 		    memcpy(TargetSubEvent,CurSubEvent,sizeof(SubEventData_StaticSize_t));
// 		    if(CurSubEvent->Amplitude.isamp!=0)
// 			 EventHeader.NHitADC++;
// 		    if(CurSubEvent->NoOfEdges!=0)
// 			 EventHeader.NHitTDC++;
// 		    EventDataBuffer->NoOfChannels++;
// 		    TargetSubEvent++;
// 	       }    
// 	  }
//      }
     
     memcpy(&EventDataBuffer->UnixTime,&TempEventBuffer->UnixTime,sizeof(UnixMTime_t));
     memcpy(EventDataBuffer->GPSTime,TempEventBuffer->GPSTime,MAX_GPS_TIMES*sizeof(GPSTime_t));
     memcpy(&EventDataBuffer->Trigger,&TempEventBuffer->Trigger,sizeof(Trigger_t));
     return 0;
}

int EventBuilder_t::BuildGeneralEvent(BinaryDataBuffer &RawDataBuffer){
     binary_tag_t Tag;
     ModuleEvalBaseClass_t *ModuleEvaluator;
     TempEventBuffer->Clear();

     while(!RawDataBuffer.end_of_read()){
	  // get the binary tag
	  RawDataBuffer.ReadFromBuffer(Tag);
 	  if((ModuleEvaluator=MapTag2Module(Tag))){
	       // we found a Module for the tag
	       // now built the part of the event that this module is good for !
	       ModuleEvaluator->BuildEventPart(RawDataBuffer,TempEventBuffer);
 	  } else break; // it was the unknown or END_OF_EVENT tag
     }
     return 0;
}

TempDecode_EventData_t *EventBuilder_t::GetDecodedGeneralEvent(){
     return TempEventBuffer;
}

/** returns a pointer to a TDC evaluator class of Type Type */
ModuleEvalBaseClass_t *EventBuilder_t::GetModuleEvaluator(const char *Type){
     return ModuleCreator(ModuleEvalBaseClass_t,Type,Parent); // macro from register.h
}

ModuleEvalBaseClass_t *EventBuilder_t::MapTag2Module(binary_tag_t tag){
     ModuleEvalVector_t::iterator i=ModuleEvalVector.begin();
     while((i<ModuleEvalVector.end())&&(tag!=(*i)->GetTag())) i++;
     if(i<ModuleEvalVector.end()){
	  return *i;
     }
     return 0;
}

void PrintEvent(ostream &o,MuonEventData_p e){
     o << "Event Number: " << e->EventNumber  
       << "  Number of Channels: " << e->NoOfChannels << endl
       << "Unix Time: " << e->UnixTime <<  endl
       << "Trigger: " << e->Trigger << endl;
     for(int k=0;k<MAX_GPS_TIMES;k++){
	  o << "GPSTime[" << k << "]: " << e->GPSTime[k] << endl;
     }
     SubEventData_p se_i=e->SubEvents;
     for(int i=0;i<e->NoOfChannels;i++){
	  o << "OM: "  << (*se_i).ID;
// ***amp	  o << "  Amp: " << (*se_i).Amplitude;
// ***	  o << "  Amp: " << ( ((int)(*se_i).Amplitude.isamp) ? (*se_i).Amplitude.ampval :  " ? ");
	  if ((*se_i).Amplitude.isamp)    o <<  "  Amp: " <<  (*se_i).Amplitude.ampval;
	  else                            o <<  "  Amp:   ? ";
	  o << "  Tim: ";
	  for(int j=0;j<(*se_i).NoOfEdges;j++){
	       o << ((*se_i).Times[j].timeval) << " "
		 << ( ((*se_i).Times[j].edge)==0?"LE":"TE" ) << " ";
	  }
	  o<<endl;
	  se_i++;
     }
     o << endl;
}

