/*
  $Header: /home/elrond/baikal/src/CVS/daq98/event.h,v 1.10 1999/02/09 14:26:30 daq Exp $
*/

#if !defined __EVENT_H__
#define __EVENT_H__

#include <config.h>
#include <iostream.h>
#include <assert.h>
#include "vectorinclude.h"
#include <algo.h>
#include "globalmacros.h"
#include "globaltypes.h"
#include "memman.h"
#include "netconv.h"
#include "simplevector.h"
#include "binarydatabuffer.h"
#include "datafilecode.h"
#include "pointerutil.h"
#include "moduleevalbase.h"

#define MAX_TIME_HITS 16
// #define MAX_OM_CHANNEL_ID 10000
 
#define MAX_GPS_TIMES 3

class ModuleEvalBaseClass_t;
class Base; /***/

// #include "hwconfigitems.h"
// #include "vmedaqhal.h"

struct Times_le_tot_t{
     s_i32 le;
     s_i32 tot;
     u_i32 flags;
};
     

/** this struct represents one subevent, with its channel number, the
    amplitude, the number of hits, and an array of the leading edge-
    and trailing edge times as a linear structure in memory. */
struct SubEventData_StaticSize_t{
     ID_t      ID;
     amp_t     Amplitude;
     count_t   NoOfEdges;
     count_t   NoOfHits;
     u_i32     FifoOV;
#if 0
     struct Times_t_boundschecker{
	  TDCTime_t &operator[](int index){
	       assert((index>=0)&&(index<MAX_TIME_HITS));
	       return Times_array[index];
	  }
     private:     
	  TDCTime_t Times_array[MAX_TIME_HITS];
     } Times;

     struct Times_le_tot_t_boundschecker{
	  Times_le_tot_t &operator[](int index){
	       assert((index>=0)&&(index<MAX_TIME_HITS));
	       return Times_le_tot_array[index];
	  }
     private:     
	  Times_le_tot_t Times_le_tot_array[MAX_TIME_HITS];
     } Times_le_tot;
#else 
     TDCTime_t Times[MAX_TIME_HITS];
     Times_le_tot_t Times_le_tot[MAX_TIME_HITS];
#endif
     
};

typedef SubEventData_StaticSize_t SubEventData_t;
typedef SubEventData_t* SubEventData_p;

inline SubEventData_StaticSize_t Get_clear_SubEventData(){
     SubEventData_StaticSize_t se;
     se.ID=0;
     se.Amplitude.ampval=0;
     se.Amplitude.isamp=0;
     se.NoOfEdges=0;
     for(int index=0;index<MAX_TIME_HITS;index++) {
	  se.Times[index].timeval=0;
	  se.Times[index].edge=0;
     }
     return se;
}

/** Linear structure of a muon event in memory */
struct MuonEventData_t
{
     count_t NoOfChannels;
     u_i32   EventNumber;
     UnixMTime_t UnixTime;
     Trigger_t Trigger;
     GPSTime_t GPSTime[MAX_GPS_TIMES];
     SubEventData_StaticSize_t SubEvents[1];
     size_t Sizeof(){
	  return sizeof(MuonEventData_t)+
	       ((0==NoOfChannels)?0:(NoOfChannels-1))*sizeof(SubEventData_StaticSize_t);
     }       
};

struct S7201CounterEvent_t{
     S7201CounterEvent_t(int maxChannels){
	 MaxChannels=maxChannels+1;
	 CounterVals=new u_i32[MaxChannels];
     }
     S7201CounterEvent_t(const S7201CounterEvent_t &copy)
	  :NoOfReadouts(copy.NoOfReadouts),
	   MaxChannels(copy.MaxChannels){
	  CounterVals=new u_i32[MaxChannels];
	  memcpy(copy.CounterVals,CounterVals,MaxChannels*sizeof(u_i32));
     }
     ~S7201CounterEvent_t(){
	  delete CounterVals;
     }
     u_i32 NoOfReadouts;
     u_i32 *CounterVals; 
     u_i32 MaxChannels;
};

/** Linear structure of a muon event in memory */
struct TempDecode_EventData_t
{
     TempDecode_EventData_t(int maxChannels)
	  : ClearSubevent(Get_clear_SubEventData()),
	    MaxChannels(maxChannels+1),
	    SubEventIndex(new int[MaxChannels]),
	    S7201CounterEvent(new S7201CounterEvent_t(MaxChannels))
	  /*, DMAD_ModuleConfigRecord(new DMAD_ModuleConfigRecord_t)*/
	  {
	       // initialize the array with -1 which means there is no corresponding 
	       // element in SubEvents - vector.
	       for(int index=0;index<MaxChannels;index++) SubEventIndex[index]=-1;
	  }
     ~TempDecode_EventData_t(){
	  if(SubEventIndex) delete[] SubEventIndex;
	  delete S7201CounterEvent;
     }
     // the clear takes far too long
     void Clear(){
	  NoOfChannels=0;
	  memset(&UnixTime,0,sizeof(UnixMTime_t));
	  memset(GPSTime,0,MAX_GPS_TIMES*sizeof(GPSTime_t));
	  Trigger=0;
//	  memset(SubEvents,0,sizeof(SubEventData_StaticSize_t)*MaxChannels);
     }
     inline SubEventData_StaticSize_t *GetChannelForWriting(int ID){
	  assert(ID<MaxChannels);
	  if(SubEventIndex[ID]<0){ // Channel ID not used yet
	       SubEventIndex[ID]=SubEvents.size();
	       SubEvents.push_back(ClearSubevent);
	  }
	  return &SubEvents[SubEventIndex[ID]];
     }
     
     inline bool ReadChannelData(SubEventData_StaticSize_t &sed){
	  if(!SubEvents.empty()){
	       sed=SubEvents.back();
	       SubEvents.pop_back();
	       SubEventIndex[sed.ID]=-1;
	       return true;
	  }
	  return false;
     }	       
private:
     SubEventData_StaticSize_t ClearSubevent;
     int MaxChannels;
     /** Array which contains the index of a SubEvent structure in the
	 SubEvents at the position [ChannelID] of the corresponding
	 channel so e.g. SubEventIndex[10028] is 2 which means
	 SubEvents[2] contains the data for the OM-Channel 10028 */
     int *SubEventIndex;
public:
     count_t NoOfChannels;
     UnixMTime_t UnixTime;
     Trigger_t Trigger;
     GPSTime_t GPSTime[MAX_GPS_TIMES];
     S7201CounterEvent_t *S7201CounterEvent;
     // this is the same for Config and raw data
//     DMAD_ModuleConfigRecord_p DMAD_ModuleConfigRecord;
private:
     /** pointer to a vector of all subevents */
     vector<SubEventData_StaticSize_t> SubEvents;
};
     
/** pointer to a subevent */
MAKE_SMART_POINTER(MuonEventData_t,MuonEventData_p);


/** Class to build events from the raw data.  Reads the raw data from
    a {\tt \Ref{BinaryDataBuffer}} and builds events.  
    An example is given in class {\tt \Ref{OfflineReader}}.  */
class EventBuilder_t
{
public:
     /** Constructor */
     EventBuilder_t(DaqProcessBase *Parent);/***/
     
     /** Destructor */
     ~EventBuilder_t();

     /** Sets up the data structures with the config information.
	 The config information is retreived from a binary data structure.
	 Please NOTE, that this structure isn't used enymore */
     int SetUpDetectorConfig(HWConfig_t &HWConfig);

     /** builds the event */
     BuildEvent_fast(BinaryDataBuffer &RawDataBuffer, 
		     MuonEventHeader_t &EventHeader,
		     MuonEventData_p EventDataBuffer);

     BuildGeneralEvent(BinaryDataBuffer &RawDataBuffer);
     TempDecode_EventData_t *GetDecodedGeneralEvent();
protected:
     
     /** Number of subevents we achieved in any of the ReadAndBuild
         functions */
     int SubEventMaxIndex;

     MAKE_REFCOUNT_POINTER(ChannelGroup_t,ChannelGroup_p);
     MAKE_INC_VECTOR(ChannelGroup_p,ChannelGroupVector_t);
     
     /** Vector that holds channel groups */
     ChannelGroupVector_t ChannelGroupVector;
          
private:

     MAKE_INC_VECTOR(ModuleEvalBaseClass_t*,ModuleEvalVector_t);
     ModuleEvalVector_t ModuleEvalVector;

     /** returns a pointer to a TDC evaluator class of Type Type or
         zero if the type is unknown*/
     ModuleEvalBaseClass_t * GetModuleEvaluator(const char *Type);

     /** converts a Module-Tag to a Tag2ModuleMap structure which
         contains a pointer to the class which can evalute the modul
         data and a vector with all om channels connected to the
         module, returns zero if the tag is no known  */
     ModuleEvalBaseClass_t *MapTag2Module(binary_tag_t tag);
     TempDecode_EventData_t *TempEventBuffer;
// my version     
//     MuonEventData_StaticSize_t *TempMuonEventBuffer;

     int MaxChannelID;
     DaqProcessBase *Parent;/***/
};

void PrintEvent(ostream &o,MuonEventData_p e);


#endif //  __EVENT_H__ 
