/*
$Header: /home/elrond/baikal/src/CVS/daq98/graph.C,v 1.3 1998/11/23 14:42:56 haase Exp $
*/

#include <qpopmenu.h>
#include <qkeycode.h>
#include <qapp.h>
#include <qpainter.h>
#include <iostream.h>
#include <qwidget.h>
#include <qmenubar.h>
#include <algo.h>
#include "graph.h"

Graph::Graph()
     :xmin(1e30),xmax(-1e30),
      ymin(1e30),ymax(-1e30),
      xLogScale(false),yLogScale(false),
      xFactor(0),yFactor(0),
      Xint(0),Yint(0),
      Wint(0),Hint(0){}

Graph::~Graph(){}     

void Graph::Draw(QPainter* P){
     DrawAxis(P);
}       

void Graph::DrawAxis(QPainter* P){
     double xstart,xstep,ystart,ystep,xLabel,yLabel;
     int i,xsteps,ysteps;	  
     int x,y,w,h;
     char num[20];
     CalcLabel(xmin,xmax,xstart,xstep,xsteps);
     CalcLabel(ymin,ymax,ystart,ystep,ysteps);
     CalcDrawParams();
	  
     x=TranslateX(xmin);
     w=TranslateX(xmax)-x;

     y=TranslateY(ymin);
     h=TranslateY(ymax)-y;

     P->drawRect(x,y,w,h);
     h=Hint/100;
     w=Wint/100;
     xLabel=xstart;
     for(i=0;i<xsteps;i++){
	  x=TranslateX(xLabel);
	  y=TranslateY(ymin);
	  P->drawLine(x,y,x,y-w);
	  xLabel+=xstep;
     }
     yLabel=ystart;
     for(i=0;i<ysteps;i++){
	  y=TranslateY(yLabel);
	  x=TranslateX(xmin);
	  P->drawLine(x,y,x-h,y);
	  yLabel+=ystep;
     }
     QRect R=P->window();
     P->setWindow(R.x(),R.height()-R.y(),R.width(),-R.height());
     xLabel=xstart;
     for(i=0;i<xsteps;i++){
	  sprintf(num,"%G",xLabel);
	  x=TranslateX(xLabel);
	  y=TranslateY(ymin);
	  P->drawText(x,R.height()-(y+4*w),
		      0,0,AlignTop|AlignHCenter|DontClip,num);
	  xLabel+=xstep;
     }
     yLabel=ystart;
     for(i=0;i<ysteps;i++){
	  sprintf(num,"%G",yLabel);
	  y=TranslateY(yLabel);
	  x=TranslateX(xmin);
	  P->drawText(x-2*h,
		      R.height()-y,
		      0,0,AlignVCenter|AlignRight|DontClip,num);
	  yLabel+=ystep;
     }
     P->setWindow(R);
}

void Graph::XLogScale(bool X){
     if(xmin<0) xmin=1; 
     xLogScale=X;
}

void Graph::YLogScale(bool Y){
     if(ymin<0) ymin=1; 
     yLogScale=Y;
}

void Graph::SetDrawParams(int x,int y,int w,int h){
     Wint=w;
     Hint=h;
     Xint=x;	
     Yint=y;
}

void Graph::CalcDrawParams(){
     if(xLogScale)
	  xFactor=((double)Wint)/(log10(xmax)-log10(xmin));
     else
	  xFactor=((double)Wint)/(xmax-xmin);
	  
     if(yLogScale)
	  yFactor=((double)Hint)/(log10(ymax)-log10(ymin));
     else
	  yFactor=((double)Hint)/(ymax-ymin);
}

void Graph::ZoomX(double Xmin,double Xmax){
     xmin=Xmin;
     xmax=Xmax;
}	  

void Graph::ZoomY(double Ymin,double Ymax){
     ymin=Ymin;
     ymax=Ymax;
}

void Graph::GetMinMax(){
     xmin=1e30;
     xmax=-1e30;
     ymin=1e30;
     ymax=-1e30;
}

void Graph::CalcLabel(double &min,double &max,double &Start,double &Step,int &numsteps){
     double RangeBase=pow10((int)log10(max-min));
     double End;
     if(min==0) Start=0;
     else if(min>0){
	  Start=RangeBase*((int)(min/RangeBase));
     }else{
	  Start=RangeBase*((int)(min/RangeBase-0.999));
     }
     if(max==0) End=0;
     else if(max>0){
	  End=RangeBase*((int)(max/RangeBase+0.999));
     }else{
	  End=RangeBase*((int)(max/RangeBase));
     }
     min=Start;
     max=End;
     RangeBase=pow10((int)log10(max-min));
     int Range=((int)((max-min)/RangeBase))-1;
     switch(Range){
     case 0: Step=RangeBase*0.2;numsteps=6;break;
     case 1: Step=RangeBase*0.5;numsteps=5;break;
     case 2: Step=RangeBase;numsteps=4;break;
     case 3: Step=RangeBase;numsteps=5;break;
     case 4: Step=RangeBase;numsteps=6;break;
     case 5: Step=RangeBase;numsteps=7;break;
     case 6: Step=RangeBase;numsteps=8;break;
     case 7: Step=RangeBase*2;numsteps=5;break;
     case 8: Step=RangeBase*3;numsteps=4;break;
     case 9: Step=RangeBase*2;numsteps=6;break;
     default: Step=RangeBase*0.2;numsteps=5;break;
     }
}


HistogramGraph::HistogramGraph(MemHistogram_t *Hist):Graph(){
     Histogram=new MemHistogram_t(*Hist);
     NumBin=(*Histogram)().NumBin;
     BinWidth=((double)(*Histogram)().Xmax-(double)(*Histogram)().Xmin)/NumBin;
     HistogramGraph::GetMinMax();
     ymin=0;
}

HistogramGraph::~HistogramGraph(){
     delete Histogram;
}

void HistogramGraph::Update(MemHistogram_t *Hist){
     delete Histogram;
     Histogram=new MemHistogram_t(*Hist);
     NumBin=(*Histogram)().NumBin;
     BinWidth=((double)(*Histogram)().Xmax-(double)(*Histogram)().Xmin)/NumBin;
     HistogramGraph::GetMinMax();
     ymin=0;
}       

void HistogramGraph::Draw(QPainter* P){
     Graph::Draw(P);
     double X=(*Histogram)().Xmin;
     int x1,x2,y1,y2;
     x1=TranslateX(X);
     y1=TranslateY(ymin);
     Entries_t *Ent=(*Histogram)().Ent;
     for(int i=0;i<NumBin;i++){
	  x2=TranslateX(X+BinWidth);
	  y2=TranslateY((double)Ent[i]);
	  P->drawLine(x1,y1,x1,y2);
	  P->drawLine(x1,y2,x2,y2);
	  x1=x2;
	  y1=y2;
	  X+=BinWidth;
     }
     P->drawLine(x2,y2,x2,TranslateY(ymin));
}       

void HistogramGraph::GetMinMax(){
     Graph::GetMinMax();
     xmin=(double)(*Histogram)().Xmin;
     xmax=(double)(*Histogram)().Xmax;
     Entries_t *Ent=(*Histogram)().Ent;
     for(int i=0;i<NumBin;i++){
	  ymax=max(ymax,(double)Ent[i]);
	  ymin=min(ymin,(double)Ent[i]);
     }
     if(yLogScale) 
	  if(0==ymin)ymin=1;
}


GraphView::GraphView(Graph_p agraph,QWidget *parent,const char *name=0)
     :QWidget(parent,name){
     aGraph=agraph; 
     resize(250,200);
}

GraphView::~GraphView(){
}

void GraphView::paintEvent(QPaintEvent *){
     QPainter P;
     P.begin(this);
     QRect R=P.window();
     int xf=R.width()/10,yf=R.height()/10;
     aGraph->SetDrawParams(R.x()+xf,R.y()+yf,R.width()-2*xf,R.height()-2*yf);
     P.setWindow(R.x(),R.height()-R.y(),R.width(),-R.height());
     aGraph->Draw(&P);
     P.end();
}


