#if ! defined __HBOOKHISTOGRAM_H__
#define __HBOOKHISTOGRAM_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/hbookhistogram.h,v 1.4 1998/11/23 14:43:02 haase Exp $

  $Log: hbookhistogram.h,v $
  Revision 1.4  1998/11/23 14:43:02  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.3  1998/11/03 06:43:55  daq
  merged pamanda2 and pamanda3 versions

*/

#if ! defined __PAWINIT_ONLY__
#include <config.h>
#include <strstream.h>
#include "globaltypes.h"
#include "globalmacros.h"
#include "ipcclasses.h"
#include "memman.h"
#include "netconv.h"
#include "event.h"
#include "histogram.h"

#endif // __PAWINIT_ONLY__

#if ! defined __NO_HBOOK__
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdarg.h>
#include "f2c.h"
#define f2cFortran
#include "cfortran.h"

/*****************************************************************************/
/* Hbook functions                                                           */
/*****************************************************************************/
#include "hbook.h"

/*****************************************************************************/
/* Prototyping mandatory in C++ !					     */
/* but only the ones you use						     */
/* PROTOCCALSFSUBn (NAME, name, type_1, ..., type_n)                         */ 
/*****************************************************************************/
PROTOCCALLSFSUB1(HDELET,hdelet,INT);
PROTOCCALLSFSUB1(HLIMIT,hlimit,INT);
PROTOCCALLSFSUB6(HBOOK1,hbook1,INT,STRING,INT,FLOAT,FLOAT,FLOAT);
PROTOCCALLSFSUB4(HFILL,hfill,INT,FLOAT,FLOAT,FLOAT);
PROTOCCALLSFSUB3(HRPUT,hrput,INT,STRING,STRING);

#endif // __NO_HBOOK__

#if ! defined __PAWINIT_ONLY__

#if ! defined __NO_HBOOK__
extern "C" void init_hbook();

/** A class that can book HBOOK histograms */
class HBOOKHistogram_t: public Histogram_t
{
public:
     /** Constructor */
     HBOOKHistogram_t(){};
     
     /** Constructor.
	 This function actually initializes the HBOOK histogram
	 with all necessary parameters.	 
	 @param HistogramDescription Histogram description (see \Ref{HistogramDescription_t})
     */
     HBOOKHistogram_t(HistogramDescription_t &HistogramDescription){
	  HBOOKHistogram_t::Create(HistogramDescription);
     }
     /** Destructor */
     virtual ~HBOOKHistogram_t(){
	  HDELET(hID);
     }

     /** Create histogram
	 Note, that this function {\bf must} be called if the default constructor
	 was used. This function actually initializes the HBOOK histogram
	 with all necessary parameters.
	 @param HistogramDescription Histogram description (see \Ref{HistogramDescription_t})
     */
     inline virtual int Create(HistogramDescription_t &HistogramDescription){
	  Histogram_t::Create(HistogramDescription);
	  hID=HistogramDescription.ID;
	  HBOOK1(hID,
		 HistogramDescription.Title,
		 HistogramDescription.NumBin,
		 HistogramDescription.Xmin,
		 HistogramDescription.Xmax,
		 0.); /* Histogram buchen */
	  return 0;
     }
     
     /** Fill the histogram.
       The channel with ID {\tt X} gets increased by 1.
       Calls HBOOK function HFILL.
       @param X Channel to increase by 1.*/
     inline virtual void Fill(X_t X){
	  HFILL(hID,X,0.,1);
     }
     /** Fill the histogram with a weighted X.
       The channel with ID {\tt X} gets increased by {\tt Weight}.
       Calls HBOOK function HFILL.
       @param X Channel to increase by {\tt Weight.
       @param Weight Amount by which channel {\tt X} gets increased*/
     inline virtual void Fill(X_t X, Entries_t Weight){
	  HFILL(hID,X,0.,Weight);
     }
     /** Write histogram to file.
	 Note that this is a static function. It writes all \Ref{HBOOKHistogram_t}
	 currently in memory to file.
	 Calls HBOOK function HRPUT */
     static void SaveAllToFile(char *Filename){
	  HRPUT(0,Filename,"N"); 
     }
     /** Init HBOOK.
	 Note that this is a static function. 
	 Calling this function is absolutely necessary before booking
	 any histograms. It should therefore be called before
	 any \Ref{HBOOKHistogram_t} is allocated.
     */
     static int InitHBook(){
	  init_hbook();
	  return 0;
     }
private:
     ID_t hID;
};

#else
class HBOOKHistogram_t: public Histogram_t
{
public:
     HBOOKHistogram_t(){};
     virtual ~HBOOKHistogram_t(){};
     inline virtual int Create(HistogramDescription_t &){
	  return 0;
     }
     inline virtual void Fill(X_t){};
     inline virtual void Fill(X_t, Entries_t){};
};

#endif // __NO_HBOOK__

MAKE_REFCOUNT_POINTER(HBOOKHistogram_t,HBOOKHistogram_p);
MAKE_INC_VECTOR(HBOOKHistogram_p,HBBOKHistogramVector_t);

#endif // __PAWINIT_ONLY__

#endif // __HBOOKHISTOGRAM_H__
