#if ! defined __HISTOGRAMLIST_H__
#define __HISTOGRAMLIST_H__

/* 
   $Header: /home/elrond/baikal/src/CVS/daq98/histogramlist.h,v 1.2 1999/02/03 03:42:18 steffen Exp $
*/

#include <config.h>
#include <String.h>
#include "vectorinclude.h"
#include "daqprocessbase.h"
#include "globalmacros.h"
#include "stringutil.h"

class HistogramList_t;

class HistogramListItem_t
{
public:
     enum type {
	  NON,
	  CATEGORY,
	  HISTOGRAM
     };
     
     HistogramListItem_t():
	  id(0), item_type(NON) {};

     HistogramListItem_t(const String &n,
			 unsigned int i=0,
			 type t=NON)
	  :title(n),id(i),item_type(t){};
     
     inline bool IsCategory() {
	  return (item_type==CATEGORY);
     }
     
     inline bool IsHistogram() {
	  return (item_type==HISTOGRAM);
     }
     
     /** Make a serial stream of the histogram list  */
     inline int Serialize(ostream &outstream) {
	  outstream << "\"" << title
		    << "\" "
		    << id
		    << " "
		    << item_type
		    << "\n";
     }
     
     /** Deserializes the histogram list */
     inline int Deserialize(istream &instream){
	  char txt[MAX_TEXT_SIZE];
	  instream.getline(txt,MAX_TEXT_SIZE);
	  return Deserialize(txt);
     }
     

     /** Deserializes the histogram list */
     inline int Deserialize(const char* txt){
	  if(strlen(txt)>0){
	       StringSplitter sp(txt,' ');
	       if(sp.NumSplits()==3){
		    sp()[0].gsub("\"","");
		    title=sp()[0];
		    id=atoi(sp()[1]);
		    item_type=(HistogramListItem_t::type)atoi(sp()[2]);
	       }
	  }
     }
     

public:
     String title;
     unsigned int id;
     type item_type;
};


class HistogramList_t
{
public:
     HistogramList_t(HistogramList_t *c=0, bool d=false)
	  :child(c), delete_child(d){};
     
     ~HistogramList_t() {
	  vector<HistogramListItem_t *>::iterator it=histograms.begin();
	  while(it!=histograms.end()){
	       delete (*it);
	       it++;
	  }
     }
     
	  
     /** Make a serial stream of the histogram list  */
     int Serialize(ostream &outstream);

     /** Deserializes the histogram list */
     int Deserialize(istream &instream);

     inline void AddItem(const HistogramListItem_t &item) {
	  histograms.push_back(new HistogramListItem_t(item));
     }
     
     inline void AddChild(HistogramList_t *c,bool d=false){
	  if(child && delete_child) delete child;
	  child=c;
	  delete_child=d;
     }
     
     
protected:
     vector<HistogramListItem_t *> histograms;
     HistogramList_t *child;

private:
     bool delete_child;
};

MAKE_REFCOUNT_POINTER(HistogramList_t,HistogramList_p);

#endif // __HISTOGRAMLIST_H__
