/*
$Header: /home/elrond/baikal/src/CVS/daq98/histosections.C,v 1.2 1998/11/23 14:43:07 haase Exp $
*/

#include "stringutil.h"
#include "histosections.h"

#define SECTION_NAME_HIST_ADC           "HIST_ADC"
#define SECTION_NAME_HIST_LE            "HIST_LEADINGEDGE"
#define SECTION_NAME_HIST_TOT           "HIST_TOT"
#define SECTION_NAME_HIST_MULTIPLICITY  "HIST_MULTIPLICITY"
#define SECTION_NAME_HIST_TIME          "HIST_TIME" 

#define ITEM_IDOFFSET                  "IDOFFSET"
#define ITEM_NUMBINS                   "NUMBINS" 
#define ITEM_XMIN                      "XMIN"  
#define ITEM_XMAX                      "XMAX"
#define ITEM_TITLE                     "TITLE"
#define ITEM_HISTOGRAM                 "HISTOGRAM"
#define ITEM_FOR                       "FOR"  
#define ITEM_CHANNEL                   "CHANNEL"  
#define ITEM_STRING                    "STRING"  
#define ITEM_ENABLED                   "ENABLED"

#define TEXT_TRUE                      "TRUE"                      
#define TEXT_FALSE                     "FALSE"                      


String HistogramSection_t::ItemNames[]={ITEM_IDOFFSET,	// 0
					ITEM_NUMBINS,	// 1
					ITEM_XMIN,	// 2
					ITEM_XMAX,	// 3
					ITEM_TITLE, 	// 4
					ITEM_HISTOGRAM,	// 5
					ITEM_FOR,       // 6
					ITEM_ENABLED    // 7
};


HistogramSection_t::HistogramSection_t(){
     SetProperties(sizeof(ItemNames)/sizeof(ItemNames[0]),ItemNames);
     CurrentHistogramConfig=0;
}

int HistogramSection_t::SetPropertyByNo(int propnum,const String &val){
     switch(propnum){

     case 0:{
	  if(!val.matches(RXint)) return -1;
	  SetIDOffset(atoi(val));
     } break;

     case 1:{
	  if(!val.matches(RXint)) return -1;
	  SetNumBin(atoi(val));
     } break;

     case 2:{
     	  if(!val.matches(RXdouble)) return -1;
	  SetXmin(atof(val));
     } break;

     case 3:{
     	  if(!val.matches(RXdouble)) return -1;
	  SetXmax(atof(val));
     } break;

     case 4:{
	  SetTitle(val);
     } break;

     case 5:{
	  StringSplitter sp(val,' ');
	  if(sp.NumSplits()!=2) return -1;
	  CurrentHistogramConfig=new MulChannelHistConfig_t;
	  MulChannelHistConfig.push_back(CurrentHistogramConfig);
	  if(!sp()[0].matches(RXint)) return -1;
	  CurrentHistogramConfig->SetHistID(atol(sp()[0]));
	  CurrentHistogramConfig->SetName(sp()[1]);
     } break;
     
     case 6:{
	  if(CurrentHistogramConfig==0) return -1;
	  StringSplitter sp(val,' ');
	  if (sp.NumSplits()<2) return -1;
	  if(fcompare(ITEM_CHANNEL,sp()[0])){
	       for(int i=1;i<sp.NumSplits();i++){
		    if(!val.matches(RXint)) return -1;
		    CurrentHistogramConfig->AddChannel(atoi(sp()[i]));
	       }
	  } else if(fcompare(ITEM_STRING,sp()[0])){
	       for(int i=1;i<sp.NumSplits();i++){
		    if(!val.matches(RXint)) return -1;
		    CurrentHistogramConfig->AddString(atoi(sp()[i]));
	       }
	  } else return -1;
     } break;
	    
     case 7:{
	  if(fcompare(TEXT_TRUE,val)) SetEnabled(true);
	  else SetEnabled(false);
     } break;

     default: return -1;
     }
     return 0;
}

const String ADCHistogramSection_t::GetSectionName()const{
     return SECTION_NAME_HIST_ADC;
}

const String LeadingEdgeHistogramSection_t::GetSectionName()const{     
     return SECTION_NAME_HIST_LE;
}

const String TOTHistogramSection_t::GetSectionName()const{
     return SECTION_NAME_HIST_TOT;
}

const String MultiplicityHistogramSection_t::GetSectionName()const{
     return SECTION_NAME_HIST_MULTIPLICITY;
}

const String HitTimesHistogramSection_t::GetSectionName()const{
     return SECTION_NAME_HIST_TIME;
}
