/*
$Header: /home/elrond/baikal/src/CVS/daq98/interpreter.h,v 1.4 1998/11/23 14:43:16 haase Exp $
*/

#if ! defined __INTERPRETER_H__
#define __INTERPRETER_H__

#include <config.h>
#include <String.h>
#include "simplevector.h"
#include "stringvector.h"
#include "pointerutil.h"
// #define red stl_red
// #define black stl_black
// #include <map>
// #undef red
// #undef black 
#include "socketclass.h"
#include "stringutil.h"


/** template class for the interpretation of commands send over the
    net or typed in from the keyboard 
    the commands have the following format:

    TYPE COMMAND PARAMTER_1 PARAMTER_2 ... PARAMETER_N

    TYPE is a unique ID of the procerss that should excecute the command e.g.
    RS for runserver, DP for dataprocess or DC for daqcontrol

    COMMAND is the command to excecute, which is mapped to a C++ function

    PARAMETER are handed over to the C++ function 

    During interpretation first the TYPE is checked, if it does not
    match the required type the <tt>Ignore</tt> function is called
    
    then the command is searched for in the database of known commands and
    if it is found the associated function is called, otherwise the
    default function is called */
template <class T>
class Interpreter:public Base
{
public:

     /** Type of a function with no paramter.
	 That function is called via the interpreter when a command is found 
	 @return return value of the called function that executes the parameter */
     typedef int (T::*CommandFunction0)();

     /** Type of a function with paramters.
	 That function is called via the interpreter when a command is found 
	 @param params Parameter string that is passed to the function 
         @param  numparam Number of command parameters in \Ref{params} 
	 @return return value of the called function that executes the paramete`r */
     typedef int (T::*CommandFunction1)(String *params,int numparams);

     /** Type of a function with a command socket and paramters.
	 That function is called via the interpreter when a command is found 
         @param s Command IO socket
	 @param params Parameter string that is passed to the function 
         @param  numparam Number of command parameters in \Ref{params} 
	 @return return value of the called function that executes the parameter */
     typedef int (T::*CommandFunction2)(IOSocket &s,String *params,int numparams);
     /** Ctor */
     Interpreter(){}
     /** Dtor */
     ~Interpreter(){}
     /** Adds the function Cmdfn, that is associated to the command
         string Cmd and the help string Help */
     void AddFunction(const String &Cmd,const CommandFunction0 Cmdfn,const String &Help=String("")){
	  DEB_F("void Interpreter::AddFunction(..)");
	  Command_p newCommand=new Command_t;
	  newCommand->CommandText=Cmd;
	  newCommand->CmdFn=Cmdfn;
	  newCommand->Help=Help;
	  newCommand->Type=0;
	  CommandVector.push_back(newCommand);
     }
     /** Adds the function Cmdfn, that is associated to the command
         string Cmd and the help string Help */
     void AddFunction(const String &Cmd,CommandFunction1 Cmdfn,const String &Help=String("")){
	  DEB_F("void Interpreter::AddFunction(..)");
	  Command_p newCommand=new Command_t;
	  newCommand->CommandText=Cmd;
	  newCommand->CmdFn=Cmdfn;
	  newCommand->Help=Help;
	  newCommand->Type=1;
	  CommandVector.push_back(newCommand);
     }
     /** Adds the function Cmdfn, that is associated to the command
         string Cmd and the help string Help */
     void AddFunction(const String &Cmd,CommandFunction2 Cmdfn,const String &Help=String("")){
	  DEB_F("void Interpreter::AddFunction(..)");
	  Command_p newCommand=new Command_t;
	  newCommand->CommandText=Cmd;
	  newCommand->CmdFn=Cmdfn;
	  newCommand->Help=Help;
	  newCommand->Type=2;
	  CommandVector.push_back(newCommand);
     }
     /** Sets the function that is called when the type is not correct */
     void WrongTypeFunction(CommandFunction0 Cmdfn){
 	  DEB_F("void Interpreter::WrongtypeFunction(..)");
 	  wrongtypeFunction.CmdFn=Cmdfn;
 	  wrongtypeFunction.Help="";
 	  wrongtypeFunction.Type=0;	  
     }
     /** Sets the function that is called when the type is not correct */
     void WrongTypeFunction(CommandFunction1 Cmdfn){
 	  DEB_F("void Interpreter::WrongtypeFunction(..)");
 	  wrongtypeFunction.CmdFn=Cmdfn;
 	  wrongtypeFunction.Help="";
 	  wrongtypeFunction.Type=1;	  
     }
     /** Sets the function that is called when the type is not correct */
     void WrongTypeFunction(CommandFunction2 Cmdfn){
 	  DEB_F("void Interpreter::WrongtypeFunction(..)");
 	  wrongtypeFunction.CmdFn=Cmdfn;
 	  wrongtypeFunction.Help="";
 	  wrongtypeFunction.Type=2;	  
     }
     /** sets the function that is called when the command is not known */
     void DefaultFunction(CommandFunction0 Cmdfn){
	  DEB_F("void Interpreter::DefaultFunction(..)");
	  defaultFunction.CmdFn=Cmdfn;
	  defaultFunction.Help="";
	  defaultFunction.Type=0;
     }
     /** sets the function that is called when the command is not known */
     void DefaultFunction(CommandFunction1 Cmdfn){
	  DEB_F("void Interpreter::DefaultFunction(..)");
	  defaultFunction.CmdFn=Cmdfn;
	  defaultFunction.Help="";
	  defaultFunction.Type=1;
     }
     /** sets the function that is called when the command is not known */
     void DefaultFunction(CommandFunction2 Cmdfn){
	  DEB_F("void Interpreter::DefaultFunction(..)");
	  defaultFunction.CmdFn=Cmdfn;
	  defaultFunction.Help="";
	  defaultFunction.Type=2;
     }
     /** interprets the command string CmdText, which has a maximum of
	 numparams-2 parameter (numparams tokens) and came from the Socket
	 S and was recieved from the class instance object and calls the
	 associated function 
	 @return 0 if string was not a valid command string (i.e. comment, or 
	 empty line), otherwise return value of the command function */
     int Do(T* object,String CmdText,IOSocket &S=(*(IOSocket*)0)){
	  DEB_F("int Interpreter::Do(..)");
	  IStringSplitter sp(CmdText,' ');
	  if(!sp.IsValid()) return 0;
	  /*
	    sp()[0] -> Type
	    sp()[1] -> Command
	    sp()[2] -> Parameter 1
	    .
	    sp()[n+1] -> Parameter n */
	  // check the type
	  if(sp()[0]==object->GetProcessType()){
	       // type ok, do interpretation
	       return DoInterpretation(object,CmdText,sp()+1,sp.NumSplits()-1,S);
	  }else{
	       // type not ok
	       String C[2];
	       // hand the default function the  command string
	       if(sp.NumSplits()>1) C[0]=join(sp()+1,sp.NumSplits()-1," ");
	       else C[0]="";
	       // and the type
	       C[1]=sp()[0];
	       return CallCmdFunction(object,&wrongtypeFunction,S,C,sp.NumSplits()-1);
	  }
	  return 0;
     }
     /** interpretes the Command string, the same way as Do(), but it
         does not check the type, so the command must be without
         type, calls Ignore if the command is not found in the
         database */
     int Do_wo_type(T* object,String CmdText,IOSocket &S=(*(IOSocket*)0)){
	  DEB_F("int Interpreter::Do_or_Ignore(..)");
	  IStringSplitter sp(CmdText, ' ');
	  if(!sp.IsValid()) return 0;
	  /*
	    sp()[0] -> Command
	    sp()[1] -> Parameter 1
	    .
	    sp()[n] -> Parameter n */
	  // check the type
	  
	  return DoInterpretation(object,CmdText,sp(),sp.NumSplits(),S);
     }
     /** fills in the Help parameter with the Help string associated with Cmd*/
     int GetHelp(String &Cmd,String &Help){
	  DEB_F("int Interpreter::GetHelp(..)");
	  Command_p Command=FindCommand(Cmd); 
	  if(0==(void*)Command){
	       Help="unknown command\r\n";
	       return 0;
	  }
	  Help=Command->Help;
	  if(Help=="") Help="no help available";
	  Help+="\r\n";
	  return 0;
     }
     /** fills in the Help parameter with the Help string all commands*/
     int GetHelp(String &Help){
	  DEB_F("int Interpreter::GetHelp(..)");
	  Help="Help:";
	  CommandVector_t::iterator ifn=CommandVector.begin();
	  while(ifn!=CommandVector.end()){
	       if((*ifn)->Help!=""){
		    Help+="\n";
		    Help+=(*ifn)->Help;
	       }
	       ifn++;
	  }
	  Help+="\r\n";
	  return 0;
     }
private:     
     /** to hold a pointer to a function, that is associated with a
         command */
     struct Command_t
     {
	  Command_t():CmdFn(0){};
	  
	  String CommandText;
	  /** pointer to the function */
	  CommandFunction0 CmdFn;
	  /** Help string */
	  String Help;
	  /** Type of the Command function (0,1 or 2) */
	  short int Type;
     };
     MAKE_REFCOUNT_POINTER(Command_t,Command_p);
	       
protected:
     /** calls the command function */
     int CallCmdFunction(T* object,Command_t *Cmd,IOSocket &S,String *CmdParams,int np){
	  switch(Cmd->Type){
	  case 0: return ((object)->*((CommandFunction0)Cmd->CmdFn))();
	  case 1: return ((object)->*((CommandFunction1)Cmd->CmdFn))(CmdParams,np);
	  case 2: return ((object)->*((CommandFunction2)Cmd->CmdFn))(S,CmdParams,np);
	  }    
	  return 0;
     }
private:
     Command_p FindCommand(String &CmdText){
	  CommandVector_t::iterator Cmd_it;
	  for(Cmd_it=CommandVector.begin();Cmd_it<CommandVector.end();Cmd_it++){
	       if(fcompare((*Cmd_it)->CommandText,CmdText)==0) return (*Cmd_it);
	  }
	  return Command_p(0);
     }
     int DoInterpretation(T *object,const String &OrigCmdText,String *Cmds,
			  int numparams,IOSocket &S=(*(IOSocket*)0)){
	  Command_p Cmd=FindCommand(Cmds[0]);
	  if(0!=(void*)Cmd){
	       // found, call the function
	       return CallCmdFunction(object,Cmd,S,Cmds,numparams);
	  }
	  // not found -> default function
	  String C[2];
	  // hand  the entire command string to the default function 
	  C[0]=OrigCmdText;
	  // and the command as an extra parameter
	  C[1]=Cmds[0];
	  return CallCmdFunction(object,&defaultFunction,S,C,1);
     }
protected:
     class IStringSplitter:public StringSplitter
     {
     public:
	  IStringSplitter(){
	  }
	  IStringSplitter(String txt,char separator){
	       if(!CleanUpString(txt)) {
		    valid=false;
		    return;
	       }
	       valid=true;
	       Split(txt,separator);
	  }
	  bool IsValid(){return valid;}
     public:
	  bool CleanUpString(String &txt){
	       if(txt.contains("#")) txt=txt.before("#"); // remove comments
	       
	       // remove leading and trailing Whitespaces
	       // format it nice, 
	       txt.gsub(MyRxWhite," ");
	       if(txt.length()<2) return false; // no command in line, return 
	       if(' '==txt[0]) txt=txt.after(0);
	       if(txt.length()<2) return false; // no command in line, return 
	       txt.reverse();
	       if(' '==txt[0]) txt=txt.after(0);
	       txt.reverse();
	       return true;
	  }
     protected:
	  bool valid;
     };
private:
//      /** database of the commands and the associated functions and the
//          Help strings */
     MAKE_INC_VECTOR(Command_p,CommandVector_t);
     CommandVector_t CommandVector;
     /** pointer to the function, that is called when an unknown
         command was encountered */
     Command_t defaultFunction;
     /** pointer to the function, that is called when the type does
         not match */
     Command_t wrongtypeFunction;
};

#endif //__INTERPRETER_H__
















