/*
$Header: /home/elrond/baikal/src/CVS/daq98/ionotifier.C,v 1.3 1998/11/23 14:43:17 haase Exp $
*/

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>     
#include "globaltypes.h"
#include "ionotifier.h"

SelectClass::SelectClass(){
     FD_ZERO(&read_fd_set);
     FD_ZERO(&write_fd_set);
     FD_ZERO(&except_fd_set);
     requested_timer_time=-1;     
}

int SelectClass::DoSelect(){
     static int actual_timer_time=0;
     int ret=0;
     timeval timeout;
     timeval active_timeout;
     // do we have a timeout ? if not requested_timer_time is < 0 and
     // we can let select sit and wait until we receive something on a
     // socket
     if(requested_timer_time<0){
	  //no timeout
	  return Select(0);
     } else {
	  timeout.tv_sec=0;
	  // if the timeout is !=0 
	  if(requested_timer_time!=0)
	       //set the timeout to 100 msec
	       timeout.tv_usec=100000;
	  else
	       // otherwise to 0 msec 
	       timeout.tv_usec=0;
     }
     // now loop over select until requested_timer_time is smaller
     // than actual_timer_time wich is increased every 100 msecs
     // because select returns every 100 msecs
     do{
	  active_timeout.tv_sec=timeout.tv_sec;
	  active_timeout.tv_usec=timeout.tv_usec;
	  ret=Select(&active_timeout);
	  if(ret!=0) break;
	  actual_timer_time++;
     }while(actual_timer_time<requested_timer_time);
     if(0==ret) actual_timer_time=0;
     return ret;
}

int SelectClass::Select(timeval *tv){
     int ret;
     active_read_fd_set=read_fd_set;
     active_write_fd_set=write_fd_set;
     active_except_fd_set=except_fd_set;

     TEST_SYS_FAILURE(ret=select(FD_SETSIZE,
				 &active_read_fd_set,
				 &active_write_fd_set,
				 &active_except_fd_set,tv),"select");     
     return ret;
}

void SelectClass::AddNotifier(IONotifier &ion){
     if(ion.GetDescriptor()<0) return;
     if(0!=(ion.GetFlag()&IONotifier::READ))
	  FD_SET(ion.GetDescriptor(),&read_fd_set);
     if(0!=(ion.GetFlag()&IONotifier::WRITE))
	  FD_SET(ion.GetDescriptor(),&write_fd_set);
     if(0!=(ion.GetFlag()&IONotifier::EXCEPTION))
	  FD_SET(ion.GetDescriptor(),&except_fd_set);
}

void SelectClass::RemoveNotifier(IONotifier &ion){
     if(ion.GetDescriptor()<0) return;
     if(0!=(ion.GetFlag()&IONotifier::READ))
	  FD_CLR(ion.GetDescriptor(),&read_fd_set);
     if(0!=(ion.GetFlag()&IONotifier::WRITE))
	  FD_CLR(ion.GetDescriptor(),&write_fd_set);
     if(0!=(ion.GetFlag()&IONotifier::EXCEPTION))
	  FD_CLR(ion.GetDescriptor(),&except_fd_set);
}

bool SelectClass::CheckNotifier(IONotifier &ion){
     if(ion.GetDescriptor()<0) return false;
     if(0!=(ion.GetFlag()&IONotifier::READ))
	  if(FD_ISSET(ion.GetDescriptor(),&active_read_fd_set)) return true;
     if(0!=(ion.GetFlag()&IONotifier::WRITE))
	  if(FD_ISSET(ion.GetDescriptor(),&active_write_fd_set)) return true;
     if(0!=(ion.GetFlag()&IONotifier::EXCEPTION))
	  if(FD_ISSET(ion.GetDescriptor(),&active_except_fd_set)) return true;
     return false;
}

void SelectClass::SetTimer(int timer_val){
     requested_timer_time=timer_val;
}

