/*
$Header: /home/elrond/baikal/src/CVS/daq98/ionotifier.h,v 1.4 1999/02/03 03:42:18 steffen Exp $
*/

#if !defined __IONOTIFIER_H__
#define __IONOTIFIER_H__

#include <config.h>
#include <sys/types.h>
#include "globaltypes.h"
#include "socketclass.h"
#include "base.h"

/** Base class for selectable objects */
class IONotifier: public Base
{     
public:
     /** flags, that inidcate when we want to be notified, see {\tt
         socket, open, close } and {\tt select } system calls */
     enum  {
	  /** notify when object is readable */
	  READ=1,
	  /** notify when object is writable */
	  WRITE=2,
	  /** notify when there is an exception on the object */
	  EXCEPTION=4
     };
     typedef int select_type;
     /** returns the filedescriptor of the object */
     virtual filedescriptor_t GetDescriptor()=0;
     /** returns the flags of the object */
     inline select_type GetFlag(){return SelectFlag;}
protected:
     /** ctor.
	 we don't want to call it, since this is a ABC */
     IONotifier():SelectFlag(READ){}
     /** copy ctor.
	 we don't want to call it, since this is a ABC */
     IONotifier(select_type sf):SelectFlag(sf){}
     /** set the flags of the Object */
     void Set(select_type sf){SelectFlag=sf;}
protected:
     /** hold the flags of the object */
     select_type SelectFlag;
};

/** notifier class for objects of type Socket */
class SocketNotifier:public IONotifier
{
public:
     /** ctor */
     SocketNotifier():IONotifier(),nSocket(0){}       
     /** ctor , construct an notifier for a socket .
	 @param sock Socket we wnat the notifier for
	 @param sf Flags, that indicate when we want to be notified */
     SocketNotifier(Socket &sock,select_type sf):IONotifier(sf),nSocket(&sock){}
     /** returns the discriptor of the socket.
       @return Descriptor of the socket or -1 if the socket is not connected */
     virtual filedescriptor_t GetDescriptor(){
	  if(0==nSocket)return -1;
	  if((Socket::SOS_CLOSED==(*nSocket).GetState())||
	     (Socket::SOS_DISCONNECTED==Socket::SOS_CLOSED)) return -1;
	  return (*nSocket).Descriptor();
     }
     /** initializes the notifier for a Socket
	 @param sock Socket we wnat the notifier for
	 @param sf Flags, that indicate when we want to be notified */
     void Set(Socket &sock,select_type sf){
	  IONotifier::Set(sf);
	  nSocket=&sock;
     }
private:
     /** pointer to the socket */
     Socket *nSocket;
};

/** notifier class for objects of type Files (filedescriptors) */
class FileNotifier:public IONotifier
{
public:
     /** ctor */
     FileNotifier():IONotifier(),descriptor(-1){}       
     /** ctor, construct an notifier for a file descriptor 
	 @param fd the descriptor of a file we want the notifier for 
	 @param sf Flags, that indicate when we want to be notified */
     FileNotifier(filedescriptor_t fd,select_type sf):IONotifier(sf),descriptor(fd){} 
     /** returns the discriptor of the socket.
       @return Descriptor of the socket or -1 if the socket is not connected */
     virtual filedescriptor_t GetDescriptor(){return descriptor;}
     /** initializes a notifier for a file descriptor 
	 @param fd the descriptor of a file we want the notifier for 
	 @param sf Flags, that indicate when we want to be notified */
     void Set(filedescriptor_t fd,select_type sf){
	  IONotifier::Set(sf);
	  descriptor=fd;
     }
private:
     /** the file descriptor */
     filedescriptor_t descriptor;
};


/** class to manage a set of notifiers and a time using select */
class SelectClass: public Base
{
public:
     /** ctor */
     SelectClass();
     /** does the select on all Notifier that where added using the
	 {\tt \Ref{AddNotifier}} function.
	 
	 @return -1 on error, 0 on
	 Timer timeout or the number of Objects the where active */
     int DoSelect();
     /** Adds a notifier to the Set of selectable notifiers 
	 @param ion the notifier to be added */
     void AddNotifier(IONotifier &ion);
     /** Removes a notifier from the list
	 @param ion the notifier to be removed */
     void RemoveNotifier(IONotifier &ion);
     /** check wether a notifier is active or not. 
	 @param ion the notifier to be checked
	 @return true, if the Object of the given Notifier was active:
	 flag READ set -> Object is readable 
	 flag WRITE set -> object is writable etc. otherwise false */
     bool CheckNotifier(IONotifier &ion);
     /** sets the value for the timer.
	 DoSelect will return after that time with an return value of 0, 
	 @param timer_val the time in 10msecs steps */
     void SetTimer(int timer_val);
private:
     /** do the actual select system call */
     int Select(timeval *tv);
     /** set of the filedescriptors we wait for readable */
     fd_set active_read_fd_set;
     /** set of the filedescriptors we wait for writeable */
     fd_set active_write_fd_set;
     /** set of the filedescriptors we wait for exception */
     fd_set active_except_fd_set;
     /** temporary set of the filedescriptors we wait for readable */
     fd_set read_fd_set;
     /** temporary set of the filedescriptors we wait for writeable */
     fd_set write_fd_set;
     /** temporary set of the filedescriptors we wait for exception */
     fd_set except_fd_set;
     /** value of the timer */
     int requested_timer_time;
};

#endif // __IONOTIFIER_H__
