/*
$Header: /home/elrond/baikal/src/CVS/daq98/ipcclasses.h,v 1.6 1999/02/03 03:42:18 steffen Exp $
*/


#if ! defined __IPCCLASSES_H__
#define __IPCCLASSES_H__

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/sem.h>
#include <sys/shm.h> 

#include "base.h"
#include "error.h"
#include "daqdebug.h"

#ifdef  __osf__ 
union semun {
     int val; /* used for SETVAL only */
     struct semid_ds *buf; /* for IPC_STAT and IPC_SET */
     ushort *array;  /* used for GETALL and SETALL */
};
#endif

/** Class for IPC semaphores */
class Semaphore: public Base
{
public:
     /** Constructor */
     inline Semaphore(){
	  DEB_F("Semaphore::Semaphore()");
	  SemaphoreState=SEMS_NOEXIST;
     }
     /** Destructor */
     virtual inline ~Semaphore(){
	  DEB_F("Semaphore::~Semaphore()");
	  if(SEMS_NOEXIST!=SemaphoreState) Destroy();
     }
     /** Initialization of the Object and creation/attachment to the
       Semaphore.
       @param akey key for the resource see man shmget, semget, msgget for
       details
       @param create If true the resource is created, returns an
	      error if the resource allready exists.
	      If false, recources will not be created, it will only be
	      attached to the already existing recource, returns an
	      error if the recource does not exist */
     int Init(key_t akey,bool create){
	  DEB_F("Semaphore::Init()");
	  if(SEMS_NOEXIST!=SemaphoreState) return 0;
	  if(create){
	       TEST_SYS_FAILURE(SemID=semget(akey,1,0660|IPC_CREAT|IPC_EXCL),"semget");
	       SemaphoreState=SEMS_CREATED;
	  }else{
	       TEST_SYS_FAILURE(SemID=semget(akey,1,0660),"semget");
	       SemaphoreState=SEMS_GOT;
	  }
	  /* SR */
#ifdef __LYNX__
#define buf 0
#else
	       semun buf;
	       buf.val=0;
#endif
	  TEST_SYS_FAILURE(semctl(SemID, 0, SETVAL, buf),"semctl");
#ifdef buf
#undef buf
#endif
	  return 0;
     }
     /** Destroys the Semaphore */
     int Destroy(){
	  DEB_F("Semaphore::Destroy()");
	  if(SEMS_CREATED==SemaphoreState){
#ifdef __LYNX__
#define buf 0
#else
	       semun buf;
#endif
	       TEST_SYS_FAILURE(semctl(SemID,0,IPC_RMID,buf),"semctl");
#ifdef buf
#undef buf
#endif
	  }
	  SemaphoreState=SEMS_NOEXIST;
	  return 0;
     }
     /** Gets the Semaphore, or waits until the semaphore is
       available*/
     inline int Get(){
	  DEB_F("Semaphore::Get()");
 	  struct sembuf semb;
 	  semb.sem_num=0;
 	  semb.sem_op=-1;
 	  semb.sem_flg=SEM_UNDO;
//	  cerr << "get1 " << SemID << endl;
 	  TEST_SYS_FAILURE(semop(SemID,&semb,1),"semop");
//	  cerr << "get2 " << SemID << endl;

//   	  int val;
	  
// #ifdef __LYNX__
// #define buf 0
// #else
//  	       semun buf;
// #endif
//  	       val=semctl(SemID,0,GETVAL,buf);
// #ifdef buf
// #undef buf
// #endif
	  
	  return 0;
     }
     /**  Realeases the semaphore */
     inline int Release(){
	  DEB_F("Semaphore::Release()");
	  struct sembuf semb;
 	  semb.sem_num=0;
 	  semb.sem_op=1;
 	  semb.sem_flg=SEM_UNDO;
//	  cerr << "rel1 " << SemID << endl;
	  
 	  TEST_SYS_FAILURE(semop(SemID,&semb,1),"semop");
//	  cerr << "rel2 " << SemID << endl;
//   	  int val;
	  
// #ifdef __LYNX__
// #define buf 0
// #else
//   	       semun buf;
// #endif
//  	       val=semctl(SemID,0,GETVAL,buf);
// #ifdef buf
// #undef buf
// #endif
	  
	  return 0;
     }
     enum SemaphoreState_t {SEMS_CREATED, SEMS_GOT, SEMS_NOEXIST};
private:
     SemaphoreState_t SemaphoreState;
     int SemID;
};


/** Class to lock and unlock a recource via a semaphore.
    If this class
    is created as a local variable in a function the constructor get the
    semaphore, when the object gets destroyed at the end of the function
    the destructor releases the semaphore usage:
    \begin{verbatim}
    void some_function(){
        SemLock SL(a_pointer_to_Semaphore); // Gets the semaphore
        do_something();
    } // Destructor of SL is called and semaphore gets released
    \end{verbatim}
*/
class SemLock
{
public:
     inline SemLock(Semaphore *Sem){
	  DEB_F("SemLock::SemLock(..)");
	  sem=Sem;
	  sem->Get();
     }
     inline ~SemLock(){
	  DEB_F("SemLock::~SemLock()");
	  sem->Release();
     }
private:
     Semaphore *sem;
};

/** Class for shared memory */
class SharedMem: public Base
{
public:
     /** Constructor */
     inline SharedMem(){
	  DEB_F("SharedMem::SharedMem()");
	  SHMState=SHMS_NOEXIST;
     }
     inline ~SharedMem(){
	  DEB_F("SharedMem::~SharedMem()");
	  if(SHMS_NOEXIST!=SHMState) Destroy();
     }
     /** Initialization , see semaphore, param=length in byte*/
     inline int Init(key_t akey,long param,bool create){
	  DEB_F("SharedMem::Init()");
	  if(SHMS_NOEXIST!=SHMState) return 0;
	  if(create){
 	       TEST_SYS_FAILURE(ShmID=shmget(akey,param,0660|IPC_CREAT|IPC_EXCL),"shmget");
 	       SHMState=SHMS_CREATED;
 	  }else{
 	       TEST_SYS_FAILURE(ShmID=shmget(akey,param,0660),"shmget");
 	       SHMState=SHMS_GOT;
 	  }
 	  return Attach();
     }
     /** Destruction */
     inline int Destroy(){
	  DEB_F("SharedMem::Destroy()");
	  if((SHMS_ATTACHED_C==SHMState)||
	     (SHMS_ATTACHED_G==SHMState)) 
	       Detach();
	  if(SHMS_CREATED==SHMState){
	       shmid_ds *buf=0;
	       TEST_SYS_FAILURE(shmctl(ShmID,IPC_RMID,buf),"shmctl");
	  }
	  return 0;
     }
     /** returns a pointer to the shared memory*/
     inline void *operator()(){
	  DEB_F("SharedMem::operator()()");\
	  return ptr;
     }
     enum SharedMemState_t {SHMS_NOEXIST, SHMS_CREATED, SHMS_GOT, SHMS_ATTACHED_C, SHMS_ATTACHED_G};
private:
     /** Attach to the shared memory (see sys-function shmat()) */
     inline int Attach(){
 	  DEB_F("SharedMem::Attach()");
 	  if(SHMS_CREATED==SHMState){
 	       TEST_SYS_FAILURE((void *)(ptr=(char*)shmat(ShmID,0,0)),"shmat");
 	       SHMState=SHMS_ATTACHED_C;
 	  } else if(SHMS_GOT==SHMState){
 	       TEST_SYS_FAILURE((void *)(ptr=(char*)shmat(ShmID,0,0)),"shmat");
 	       SHMState=SHMS_ATTACHED_G;
 	  }
 	  return 0;
     }
     /** Detach from the shared memory (see shmdt())*/
     inline int Detach(){
	  DEB_F("SharedMem::Detach()");
	  if(SHMS_ATTACHED_C==SHMState){
	       TEST_SYS_FAILURE(shmdt(ptr),"shmdt");
	       SHMState=SHMS_CREATED;
	  } else if(SHMS_ATTACHED_G==SHMState){
	       TEST_SYS_FAILURE(shmdt(ptr),"shmdt");
	       SHMState=SHMS_GOT;
	  }
	  return 0;
     }
     /**  pointer to the shared Memory */
     char *ptr;
     /** ID */
     int ShmID;
     /** State */
     SharedMemState_t SHMState;
};


#endif // __ICPCLASSES_H__
