/*
  $Header: /home/elrond/baikal/src/CVS/daq98/k32module.C,v 1.4 1999/02/03 03:42:19 steffen Exp $
*/

#include <assert.h>
#include "k32module.h"

#define DEBUG_LOCAL(A)  cout   A


K32_TDCModule::K32_TDCModule(DaqProcessBase *parent):
     TDCModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,parent)
{
     Parent->LogMsg("K32 TDC Module created",9,9);
     events_this_k32=0;       
     fifoov_this_k32=0;
}

#ifdef VME_ACCESS
K32_TDCModule::~K32_TDCModule()
{
     DEBUG_MESSAGE( << "Module destroyed" << endl);
     // release memory 
     vme_rel(p_base,nw_map);
}     
#else
K32_TDCModule::~K32_TDCModule()
{
}
#endif // VME_ACCESS

#ifdef VME_ACCESS
bool K32_TDCModule::HasModuleData(){
     return GetModuleConfig()->Enabled&&EmptyBuf(*p_rang);
} 
   
int K32_TDCModule::ReadToBuffer(BinaryDataBuffer &DataBuffer){
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read" << endl);
	  
     // for the number of 32Bit words read from the TDC
     u_i16 NumWords=0;
     /* buffer readout */
     
     int nw_buff=0;
     int eofflag=0;
     int tdc_data;	  
     int evtnumber;
     int chan;
     int edge;
     int fifoov=0;
     int last_chan_fifoov=-1;
     int nb_fifoov=0;
     int tdcword;
     int NumWords_Max = 32*16+1;                // strict: 1 event only
//     int NumWords_max = 4*(32*16+1);              // relaxed: 4 events
     
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     DataBuffer.WriteToBuffer(NumWords);


     events_this_k32++;
     
	  
//     while (!eofflag){ 
//       check later, if fifo-break really needed, may be limit to 32x16  ********* 
//     while (!eofflag && !fifoov){                // break at FIFO_ov 
           // break at FIFO_ov & too  many data
//     while (!eofflag && !fifoov && NumWords<=NumWords_Max){     
#warning ### check this condition
     while (!eofflag && !fifoov && NumWords<NumWords_Max){           // 11/02/98
     // will try 14/02  (fifov debugging)
//     while (!eofflag  && NumWords<NumWords_Max){           
	  
	  tdc_data = *p_fifo;     // next dword 

	  nw_buff++;
	        
	  DEBUG_MESSAGE(
	       <<"         nw" << nw_buff		\
	       << ", data= "   << tdc_data		\
	       << ", flags:"   << NoData(tdc_data)	\
	       << endl);

	       
	  if (EventNumberFlag(tdc_data)){    
	       // event number
	       evtnumber = EventNumber(tdc_data);
	       DEBUG_MESSAGE( << "evtnumber = " << evtnumber <<  endl);
	  }else{           
	       // is tdc word 
	       fifoov  = FifoOverflow(tdc_data);
	       if (fifoov) nb_fifoov++;

	       chan=Channel(tdc_data);

	       if (fifoov) last_chan_fifoov=chan;


	       if(!IsChannelInUse(chan)){
		    // error unused channel has data
	       }
	       // do we have to record the channel data
	       if(IsChannelOn(chan)){
	       
		    edge=Edge(tdc_data);
		    tdcword=TimeVal(tdc_data);
		    
		    DEBUG_MESSAGE(
			 << "chan = "   << chan	\
			 << ", time = " << tdcword	\
			 << ", egde = " << edge	\
			 << ", fifov =" << fifoov	\
			 << endl );
	       

		    // save the raw data from the tdc
		    DataBuffer.WriteToBuffer(tdc_data);
		    NumWords++;
	       }
	  }
	  
	  // test for end of buffer
	  if (EOB(tdc_data))  eofflag++;  /* [24]=0  end of buffer */
     }
     // Buffer is read out
     if (nb_fifoov)   {
	  fifoov_this_k32++;
/*
	  if (1==fifoov_this_k32%100)
        	  cout << "########## fifo ov in " << GetModuleConfig()->ModuleID
		       <<"  k32_evt, k32_fifo_tot, nbchans, last_chan= "<<events_this_k32<<" "
		       << fifoov_this_k32 <<" "<<nb_fifoov<<" "<<last_chan_fifoov << endl;
*/
     }
     
     //  *RW* ******* (should never happen)
     assert (NumWords<=NumWords_Max);

     // reset the module's FIFO & register

     ResetCLAL();
     
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     u_i16 NumBytes=NumWords*GetBytesPerInfoUnit();
     // write the size
     DataBuffer.WriteToBuffer(NumBytes);
     // restore the old position
     DataBuffer.IncPosition(NumBytes);
     return 0;
}

int K32_TDCModule::InitModule(StringVector_t &/*errors*/){
     
     u_i32 d;
     u_i32 VMEAddress=GetModuleConfig()->Address_Slot;
     u_i16 MaskL_data, MaskH_data;

     
     // init the KLOE 32Channel TDC board  

//     p_base = (void*)vme_map(0x00000100,nw_map,0x09);
     //     cout << "setting " << GetModuleConfig()->ModuleID << \
     //" to address : " << (void*)VMEAddress << endl;
     nw_map=256;
     p_base = (void*)vme_map(VMEAddress,nw_map,0x09);
     
     p_fifo  = (u_i32 *) (p_base + 0x00);
     p_main  = (u_i32 *) (p_base + 0x08);
     p_creg  = (u_i32 *) (p_base + 0x48);
     p_rang  = (u_i32 *) (p_base + 0x0c);
     p_maskl = (u_i32 *) (p_base + 0x18);
     p_maskh = (u_i32 *) (p_base + 0x58);
     p_clal  = (u_i32 *) (p_base + 0x3c);
     p_beck  = (u_i32 *) (p_base + 0x88);
   
     d = *p_main;

//     printf("main reg: ad= 0x%08x d=0x%08x\n",(u_i32)p_main,d); 
//     printf("Initial Settings :\n");
//     printf("Main   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_main,*p_main);
//     printf("Creg   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_creg,*p_creg);
//     printf("Rang   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_rang,*p_rang);
//     printf("MaskL  register adr, data = 0x%08x  0x%08x\n",(u_i32)p_maskl,*p_maskl);
//     printf("MaskH  register adr, data = 0x%08x  0x%08x\n",(u_i32)p_maskh,*p_maskh);


     // define the module settings

     int n = GetNoOfChannels();
     MaskL_data=0; 
     MaskH_data=0; 
     //     cout << "number of channels= " << n <<endl;
     nchan_on=0;
     for (int i=0;i<n;i++){
	  if (i<16) 
	  {if (IsChannelOn(i)) {
	    nchan_on++;
	    MaskL_data = MaskL_data | (0x01<<i);
	  }}
	  else 
	  {if(IsChannelOn(i)) {
	    nchan_on++;
	    MaskH_data = MaskH_data | (0x01<<i);
	  }}
     }

     // forcing ..........
     MaskH_data=0xFFFF;
     cout << "MaskH = "<< (void*) MaskH_data<<endl;
     

     //     printf ("MaskL MaskH =0x%x   0x%x \n",MaskL_data,MaskH_data);
     
   
     // setting the module
     //
     // Common stop/ DLL mode/ seq.reading/ 16 hits max/ 16 bits evt.counter/
     //  64usec window/ LE&TE / Chan0-15 enable 
   
     int evwidth,test,smode,askev,tdcmode,ckdis,ckmode,nhits;
     int  compdepth,timemax;
	

     //	 testmode enable, intclock disable 

         
     evwidth = 0xf; // evtnumber register: 0=1bit, 15=16bit
     test    = 1;   // Test bit to allow setting internal registers
     smode   = 3;   // LE=1, TE=2, LE&TE=3
     askev   = 0;   // must be 0
     tdcmode = 1;   // c-stop = 1 
     ckdis   = 1;   // disable = 1 
     ckmode  = 1;   // dll clock 
     nhits   = 0xf; // max nb of hits: 0=1, 0xF=16 
	
     d = 0;
     d = d|
	  (0xf000&(evwidth<<12))|
	  (0x0400&(test<<10))|
	  (0x0300&(smode<<8))|
	  (0x0080&(askev<<7))|
	  (0x0040&(tdcmode<<6))|
	  (0x0020&(ckdis<<5))|
	  (0x0010&(ckmode<<4))|
	  (0x000f&(nhits<<0));
     
     *p_main = d;
 
     //	printf(" mainreg = 0x%08x 0x%08x \n",d,*p_main);
	
     // Rang
     compdepth = 0x2;
     timemax = 0xe;         // max= 0xE = 64usec ?? 

     d = 0;     
     d = d|
	  (0x30&(compdepth<<4))|
	  (0x0f&(timemax<<0));
     *p_rang = d;
	
	
     // Clock
     d = 0;     
     *p_creg = d;

     // BECK & CLAL command
     d = *p_beck;
     d = *p_clal;

     // MASK registers
     //     *p_maskl = 0x0003;             // chans 0-15
     //     *p_maskh = 0x0;                // chans 16-31
     *p_maskl = MaskL_data;                // chans 0-15
     *p_maskh = MaskH_data;                // chans 16-31

	
     // main register
     test    = 0;    // set work mode 
     ckdis   = 0;    // disable = 1 
	
     d = 0;
     d = d|
	  (0xf000&(evwidth<<12))|
	  (0x0400&(test<<10))|
	  (0x0300&(smode<<8))|
	  (0x0080&(askev<<7))|
	  (0x0040&(tdcmode<<6))|
	  (0x0020&(ckdis<<5))|
	  (0x0010&(ckmode<<4))|
	  (0x000f&(nhits<<0));

     *p_main = d;
     //	printf(" mainreg = 0x%08x 0x%08x \n",d,*p_main);	

     //     printf("Run Settings :\n");
     //     printf("Main   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_main,*p_main);
     //     printf("Creg   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_creg,*p_creg);
     //     printf("Rang   register adr, data = 0x%08x  0x%08x\n",(u_i32)p_rang,*p_rang);
     //     printf("MaskL  register adr, data = 0x%08x  0x%08x\n",(u_i32)p_maskl,*p_maskl);
     //     printf("MaskH  register adr, data = 0x%08x  0x%08x\n",(u_i32)p_maskh,*p_maskh);


     //     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " initialized" << endl);
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << ":  Adr= " 
                    <<(void*)VMEAddress << "  NChan/On= "<< n << "/ " << nchan_on 
                    << "  MaskL/H= "<< (void*)MaskL_data << "/ " << (void*)MaskH_data << "  initialized."<<endl);
     DEBUG_LOCAL  ( << "Module " << GetModuleConfig()->ModuleID << ":  Adr= " 
                    <<(void*)VMEAddress << "  NChan/On= "<< n << "/ " << nchan_on 
                    << "  MaskL/H= "<< (void*)MaskL_data << "/ " <<(void*)MaskH_data << "  initialized."<<endl);
     return 0;
}

#else //VME_ACCESS
bool K32_TDCModule::HasModuleData(){
     return false;
} 
   
int K32_TDCModule::ReadToBuffer(BinaryDataBuffer &/*DataBuffer*/){
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " read" << endl);
     return 0;
}

int K32_TDCModule::InitModule(StringVector_t &/*errors*/){
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " initialized. (SIMUL)" << endl);
     DEBUG_LOCAL  ( << "Module " << GetModuleConfig()->ModuleID << " initialized. (SIMUL)" << endl);
     return 0;
}

#endif // VME_ACCESS

const void* K32_TDCModule::ReadFromBuffer(BinaryDataBuffer &DataBuffer,int numItems){
     static u_i32 Data[NUMBER_OF_CHANNELS];
     DataBuffer.ReadArrayFromBuffer(Data,numItems);
     return (void *)Data;
}

int K32_TDCModule::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}



