/*
$Header: /home/elrond/baikal/src/CVS/daq98/k32tdcmodule.h,v 1.3 1998/11/23 14:43:30 haase Exp $
*/

#if ! defined __TDCMODULE_H__
#define  __TDCMODULE_H__

#include <config.h>
#include "moduleclasses.h"



class K32_TDCModule:public TDCModuleClass_t
{
     ClassTypeInfo(TDCModuleClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 32,
	  BYTES_PER_INFO_UNIT= 4
     };
public:

     /** ctor */
     K32_TDCModule();
     /** dtor */
     ~K32_TDCModule();
     /*------------------------------------------------------------
       The virtual functions we must overwrite
       -----------------------------------------------------------*/
     /** returns true, if the module has data or false if the module
         does not have any data */

     /** returns true if the  module has data to be read */
     virtual bool HasModuleData();

     /** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);

     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer,int numItmes);     

     /** this function initializese the TDC module */
     virtual int InitModule(StringVector_t &errors);     

     /** this function extracts the data words of one channel
         from the local buffer where the information for the whole
         module is stored, if ChannelData->ModuleChannel is -1 it
         starts with the first available channel, if
         ChannelData->ModuleChannel is not equal -1 it returns the
         next channel, the fuction returns true if the last call was
         succesful, and false if there are no more channels for this
         module*/
//     virtual bool GetNextChannel(TDCChannelData_t *ChannelData);

public:
     /*------------------------------------------------------------
       Inline Function to convert the tdc data word into the flags
       and bits and bytes we need 
       -----------------------------------------------------------*/
     static inline int NoData(const u_i32 tdc_data){
	  // [25]=0  no data flag 
	  return (tdc_data >> 25) & 0x1;
     }
     
     static inline int Channel(const u_i32 tdc_data){
	  return (tdc_data >> 18) & 0x1F;
     }
     
     static inline int Edge(const u_i32 tdc_data){
          return (tdc_data >> 16) & 0x1;
     }

     static inline int FifoOverflow(const u_i32 tdc_data){
	  return (tdc_data >> 17) & 0x1;
     }

     static inline int TimeVal(const u_i32 tdc_data){
	  return tdc_data & 0xFFFF;
     }

     static inline int EventNumberFlag(const u_i32 tdc_data){
	  return (tdc_data >> 23 ) & 0x1;	  
     }

     static inline int EventNumber(const u_i32 tdc_data){
	  return tdc_data & 0xFFFF;
     }
     
     static inline int EOB(const u_i32 tdc_data){
	  return !((tdc_data >> 24) & 0x1); // eob -> bit=0 !!!!!!!!
     }

     static inline int EmptyBuf(const u_i32 tdc_data){
	  return (tdc_data >> 28)& 0x1;
     }

     static inline int FullBuf(const u_i32 tdc_data){
	  return (tdc_data >> 29)& 0x1;
     }     

     /**  Module reset: FIFO &Registers, apply after each readout */

     inline u_i32 ResetCLAL(void ) {
	  // asm ("eieio");
	  u_i32 dum = *p_clal;
	  return  dum;
     }     
// ---------------------------------------------------------------------------------
//     for data evaluator
// ---------------------------------------------------------------------------------
     virtual inline int Channel(const void *tdc_data){
	  return Channel(*(u_i32*)tdc_data);
     }

     virtual inline int Edge(const void* tdc_data){
	  return Edge(*(u_i32*)tdc_data);
     }     

     virtual inline int TimeVal(const void* tdc_data){
	  return TimeVal(*(u_i32*)tdc_data);
     }
// ---------------------------------------------------------------------------------
private:

     void *p_base;
     u_i32 *p_fifo,
	  *p_main,
	  *p_creg,
	  *p_rang,
	  *p_maskl;
     u_i32 *p_maskh,
	  *p_clal,
	  *p_beck;

     int   nw_map;
};

#endif // __TDCMODULE_H__

