#! /usr/bin/perl
$VERSION="0.6";
# -* perl *-
# Copyright (C) 1988 Eleftherios Gkioulekas <lf@amath.washington.edu>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
sub usage
{
 print <<"EOF";
Usage: mkm4 <m4filename> <macroname> <file1> <file2> <file3> ...

 <m4filename> = Name of the m4 file to create (e.g. foo.m4)
 <macroname>  = Name of the macro to create   (e.g. AC_MAKE_FOOBAR)
 <file1>, ... = Files that we want <macroname> to generate

This program will generate m4 files that define Autoconf macros that generate
files in the current working directory. These files can be used combined
with AC_SUBST_FILE.
EOF
 exit(1);
}

sub version
{
 print <<"EOF";
mkm4 $VERSION - File to m4 compiler
Copyright (C) 1998 Eleftherios Gkioulekas <lf\@amath.washington.edu>
This is free software, and you are welcome to redistribute it and modify
it under certain conditions. There is ABSOLUTELY NO WARRANTY for this
software. For details refer to version 2 of the GNU General Public License.
EOF
 exit(1);
}

sub invalid
{
 print "Invalid usage. For help:\n";
 print "% mkm4 --help\n";
 exit(1);
}

# Make sure that a filename has been passed.
do invalid() if ($#ARGV == -1);

# Get the first argument
$first_argument = $ARGV[0];

# Check for --version or --help
do version() if ($first_argument eq "--version");
do usage()   if ($first_argument eq "--help");

# Otherwise, grab in the arguments
do invalid() if ($#ARGV < 2);
$m4_filename = $first_argument;
shift(@ARGV);
$macro_name  = $ARGV[0];
shift(@ARGV);
@files = @ARGV;

# Call shar on the files
open(SHAR,"shar -VQx @files|") || die "Cannot run shar: $!";
@shar_contents = <SHAR>;
close(SHAR);

# Get rid of the last line (which is exit 0)
$shar_contents[$#shar_contents]="";

# Get rid of the first line (which is #! bla bla)
$shar_contents[0]="";

# Now open file to write the output
open(OUT,">$m4_filename") || die "Cannot open $m4_filename: $!";

# Write preamble
print OUT <<"EOF";
dnl This macro has been generated by mkm4, written by Eleftherios Gkioulekas
dnl Copyright (C) 1998 Eleftherios Gkioulekas <lf\@amath.washington.edu>
dnl This file is free software; Eleftherios Gkioulekas gives unlimited
dnl permission to copy and/or distribute it, with or without modifications,
dnl as long as this notice is preserved. 
dnl
dnl This program is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY, to the extent permitted by law; without
dnl even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
dnl PARTICULAR PURPOSE.

AC_DEFUN($macro_name,[
changequote()dnl
EOF

# Write the shar script
foreach $line (@shar_contents)
{
 print OUT $line;
}

# Close the macro
print OUT <<"EOF";
changequote([,])dnl
])
EOF

# And this is it
close(OUT);

