/*
$Header: /home/elrond/baikal/src/CVS/daq98/memhistogram.C,v 1.2 1998/11/23 14:43:35 haase Exp $
*/

#include "memhistogram.h"


MemHistogram_t::MemHistogram_t(const bool Mem):RawBytesManager(Mem){
     DEB_F("MemHistogram_t::MemHistogram_t(bool UseSharedMem)");
}

MemHistogram_t::MemHistogram_t(const bool Mem,
			       HistogramDescription_t &HistogramDescription)
     :RawBytesManager(Mem){
     DEB_F("MemHistogram_t::MemHistogram_t(bool USM,HistogramDescription_t &)");
     MemHistogram_t::Create(HistogramDescription);
}

MemHistogram_t::MemHistogram_t(void *HistData_pointer):RawBytesManager(HistData_pointer){
     DEB_F("MemHistogram_t::MemHistogram_t(void *HistData_pointer,bool fromnet)");
     SetSize(GetPointer()->Sizeof());
}

MemHistogram_t::MemHistogram_t(ID_t ID):RawBytesManager(ID){
     DEB_F("MemHistogram_t::MemHistogram_t(ID_t ID,bool fromnet");
     // might be invalid because the ID was not found
     if(!IsValid()) return;
     SetSize(GetPointer()->Sizeof());
     GetPointer()->ID=ID;
}

MemHistogram_t::MemHistogram_t(MemHistogram_t &Histogram):RawBytesManager(Histogram){
     DEB_F("MemHistogram_t::MemHistogram_t(MemHistogram_t &Histogram)");
}

MemHistogram_t::~MemHistogram_t(){
     DEB_F("MemHistogram_t::~MemHistogram_t()");
}

int MemHistogram_t::Create(HistogramDescription_t &HistogramDescription){
     DEB_F("int MemHistogram_t::Create(..)");
     Histogram_t::Create(HistogramDescription);
     size_t size=sizeof(HistogramData_t)+
	  (HistogramDescription.NumBin-1)*sizeof(Entries_t);
     if(RawBytesManager::Create(size,HistogramDescription.ID)!=0) return -1;
     GetPointer()->ID=HistogramDescription.ID;

     strcpy(GetPointer()->Title,HistogramDescription.Title);
     GetPointer()->NumBin=HistogramDescription.NumBin;
     GetPointer()->Xmin=HistogramDescription.Xmin;
     GetPointer()->Xmax=HistogramDescription.Xmax;
     for(unsigned int i=0;i<HistogramDescription.NumBin;i++) 
	  GetPointer()->Ent[i]=0;
     GetPointer()->NumEntries=0;
     GetPointer()->NumUnderflows=0;
     GetPointer()->NumOverflows=0;
     binwidth=(X_t)(HistogramDescription.Xmax-HistogramDescription.Xmin)/(X_t)GetPointer()->NumBin;
     return 0;
}

int MemHistogram_t::Serialize(ostream &outstream){
     DEB_F("int MemHistogram_t::Serialize2ASCII(..)");
     NumBin_t NumBin=GetPointer()->NumBin;
     Entries_t *Ent=GetPointer()->Ent;
     if(outstream){
	  outstream << GetPointer()->Title << "\n" << flush
		    << GetPointer()->ID << "\n"
		    << GetPointer()->NumBin << "\n"
		    << GetPointer()->Xmin << "\n"
		    << GetPointer()->Xmax << "\n"
		    << GetPointer()->NumEntries << "\n"
		    << GetPointer()->NumUnderflows << "\n"
		    << GetPointer()->NumOverflows << "\n" << flush;
     } else {
	  (*Error)=BUFFER_TOO_SMALL;
	  (*Error)="Histogra_t::Serialize2ASCII(..)";
	  return -1;
     }
     for(unsigned int i=0;i<NumBin;i++){
	  if(outstream){
	       outstream << Ent[i] << " ";
	  }else{
	       (*Error)=BUFFER_TOO_SMALL;
	       (*Error)="MemHistogram_t::Serialize2ASCII(..)";
	       return -1;
	  }
     }
     outstream << " finish\r\n" << flush;
     return 0;
}

int MemHistogram_t::Deserialize(istream &instream){
     DEB_F("int MemHistogram_t::DeserializeFromASCII(..)");
     HistogramDescription_t histdesc;
     if(instream){
	  instream.getline(histdesc.Title,HISTOGRAM_TITLE_SIZE);
	  instream >> histdesc.ID
		   >> histdesc.NumBin
		   >> histdesc.Xmin 
		   >> histdesc.Xmax;
     } else {
	  (*Error)=COULD_NOT_READ_FROM_INSTREAM;
	  (*Error)="MemHistogram_t::DeserializeFromASCII(..)";
	  return -1;
     }
     Create(histdesc);
     if(instream){
	  instream >> GetPointer()->NumEntries
		   >> GetPointer()->NumUnderflows
		   >> GetPointer()->NumOverflows;
     } else {
	  (*Error)=COULD_NOT_READ_FROM_INSTREAM;
	  (*Error)="MemHistogram_t::DeserializeFromASCII(..)";
	  return -1;
     }
     NumBin_t NumBin=GetPointer()->NumBin;
     Entries_t *Ent=GetPointer()->Ent;	  
     for(unsigned int i=0;i<NumBin;i++){
	  if(instream){
	       instream >> Ent[i];
	  }else{
	       (*Error)=COULD_NOT_READ_FROM_INSTREAM;
	       (*Error)="MemHistogram_t::DeserializeFromASCII(..)";
	       return -1;
	  }
     }
     // read the trailing "finish\r\n" and other stuff 
     char finish[20];
     while ((finish[0]=instream.get())!='f');
     instream.getline(finish+1,18);
     // getline removes the "\n" !!!
     if(strcmp(finish,"finish\r")!=0){
	  (*Error)=ERROR_RECEIVING_HISTOGRAM;
	  (*Error)="MemHistogram_t::DeserializeFromASCII(..)";
	  return -1;
     }     
     return 0;
}



