#if ! defined __MEMHISTOGRAM_H__
#define __MEMHISTOGRAM_H__

/* 
   $Header: /home/elrond/baikal/src/CVS/daq98/memhistogram.h,v 1.2 1998/11/23 14:43:36 haase Exp $
*/

#include <config.h>
#include "globaltypes.h"
#include "globalmacros.h"
#include "memman.h"
#include "histogram.h"
#include "pointerutil.h"
#include "simplevector.h"

struct  HistogramData_t{
     ID_t ID;
     char Title[HISTOGRAM_TITLE_SIZE];
     NumBin_t NumBin;
     X_t Xmin;
     X_t Xmax;
     
     Entries_long_t NumEntries;
     Entries_long_t NumUnderflows;
     Entries_long_t NumOverflows;
     Entries_t Ent[1];
     inline size_t Sizeof(){
	  return sizeof(HistogramData_t)+(NumBin-1)*sizeof(Entries_t);
     }
};

/** Class to represent a histogram in [shared] memory */ 
class MemHistogram_t:public RawBytesManager, public Histogram_t{
public:
     /** Constructor, no memory allocation
       @param Mem Use shared memory if {\tt USE_SHM}, don't use if {\tt NO_SHM}.
       Default is {\tt NO_SHM}
     */
     MemHistogram_t(const bool Mem=NO_SHM);

     /** Constructor, craeted a new histogram, allocates the nessecary memory 
       @param Mem Use shared memory if {\tt USE_SHM}, don't use if {\tt NO_SHM}.
       Default is {\tt NO_SHM} 
     */
     MemHistogram_t(const bool Mem,HistogramDescription_t &HistogramDescription);

     /** Constructor, attaches to existing histogram data at ptr */
     MemHistogram_t(void *HistData_pointer);

     /** Constructor, attaches to existing  histogram data with ID */
     MemHistogram_t(ID_t ID);

     /** Copy Constructor, copies into procmem ! */
     MemHistogram_t(MemHistogram_t &Histogram);

     /** Destructor */
     virtual ~MemHistogram_t();

     /** create the data-fields of the histogram and initializes
         them,HistogramDescription describes the histogram */
     virtual int Create(HistogramDescription_t &HistogramDescription);

     /** make a serial stream of the data in the histogram  */
     int Serialize(ostream &outstream);

     /** deserializes the histogram data from the serial stream */
     int Deserialize(istream &instream);

     // Inlines 
     /** Fills the histogram */
     inline virtual void Fill(X_t X){
	  DEB_F("void Histogram::Fill(..)(int)");
	  GetPointer()->NumEntries++;
	  if(X<GetPointer()->Xmin) {
	       GetPointer()->NumUnderflows++;
	       return;
	  }
	  if(X>=GetPointer()->Xmax){
	       GetPointer()->NumOverflows++;
	       return;
	  }
	  NumBin_t index=(NumBin_t)((X-GetPointer()->Xmin)/binwidth);
	  // take care for overflow
	  if(GetPointer()->Ent[index]<=MAX_ENTRIES) 
	       GetPointer()->Ent[index]++;
     }
     /** Fills the histogram with a weighted X */
     inline virtual void Fill(X_t X, Entries_t weight){
	  DEB_F("void Histogram::Fill(..)(int)");
	  GetPointer()->NumEntries++;
	  if(X<GetPointer()->Xmin) {
	       GetPointer()->NumUnderflows+=weight;
	       return;
	  }
	  if(X>=GetPointer()->Xmax){
	       GetPointer()->NumOverflows+=weight;
	       return;
	  }
	  NumBin_t index=(NumBin_t)((X-GetPointer()->Xmin)/binwidth);
	  // take care for overflow
	  if(GetPointer()->Ent[index]<=MAX_ENTRIES) 
	       GetPointer()->Ent[index]+=weight;
     }
     /** returns a reference to the Histogram data object */
     inline HistogramData_t &operator()(){
	  DEB_F("HistogramData_t &Histogram::operator()()");
	  return *GetPointer();
     }
private:
     inline HistogramData_t *GetPointer(){
	  DEB_F("HistogramData_t *Histogram::GetPointer()");
	  return (HistogramData_t *)RawBytesManager::GetPointer();
     }
private:
     X_t binwidth;
};

MAKE_REFCOUNT_POINTER(MemHistogram_t,MemHistogram_p);
MAKE_INC_VECTOR(MemHistogram_p,MemHistogramVector_t);

class LocalMemHistogram_t:public MemHistogram_t{
public:
     LocalMemHistogram_t():MemHistogram_t(NO_SHM){}     
};

class SharedMemHistogram_t:public MemHistogram_t{
public:
     SharedMemHistogram_t():MemHistogram_t(USE_SHM){}     
};


#endif // __MEMHISTOGRAM_H__
