/*
$Header: /home/elrond/baikal/src/CVS/daq98/memman.C,v 1.5 1998/11/23 14:43:39 haase Exp $
*/

#include "ipcclasses.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "memman.h"


void *SharedMemAllocator_t::alloc(size_t bytes,ID_t ID)const{
     DEB_F("void *SharedMemAllocator_t::alloc(..)");
     return ShmMan->shmalloc(bytes,ID);
}

void SharedMemAllocator_t::free(void *ptr)const{
     DEB_F("void SharedMemAllocator_t::free(..)");
     ShmMan->shmfree(ptr);
}
void SharedMemAllocator_t::SetSharedMemoryManager(SharedMemoryManager *sm){
     ShmMan=sm;
}

void *ProcessMemAllocator_t::alloc(size_t bytes,ID_t)const{
     DEB_F("void *ProcessMemAllocator_t::alloc(..)");
     return ::malloc(bytes);
}

void ProcessMemAllocator_t::free(void *ptr)const{
     DEB_F("void ProcessMemAllocator_t::free(..)");
     ::free(ptr);
}

SharedMemoryManager *SharedMemAllocator_t::ShmMan;
ProcessMemAllocator_t RawBytesManager::ProcessMemAllocator;
SharedMemAllocator_t RawBytesManager::SharedMemAllocator;


SharedMemoryManager::SharedMemoryManager(){
     DEB_F("SharedMemoryManager::SharedMemoryManager()");
     ShmSegments=0;
}

SharedMemoryManager::~SharedMemoryManager(){
     DEB_F("SharedMemoryManager::~SharedMemoryManager()");
     if(ShmSegments!=0) Destroy();
}

int SharedMemoryManager::Init(key_t shmkey,key_t semkey){
     DEB_F("int SharedMemoryManager::Init(..)");
     // initialize the Management Info Segment
     TEST_FAILURE(SegmentInfo.Init(shmkey,1,false));   
     // initialize the semaphore
     TEST_FAILURE(sem.Init(semkey,false));
     // release semaphore *SR*
     TEST_FAILURE(sem.Release());
     // get segmentsize
     SegmentSize=SegmentInfo[0].SegmentSize;                   
     // get number of segments
     NoSegments=SegmentInfo[0].NoSegments;                   
     // create the array of pointers to the shared memory
     ShmSegments=new SharedMemSegment[NoSegments];            
     // initialize the segments
     for(unsigned short i=0;i<NoSegments;i++){
	  TEST_FAILURE(ShmSegments[i].Init(SegmentInfo[0].ShmKey[i],0,false));
     }
     SharedMemAllocator_t::SetSharedMemoryManager(this);
     return 0;
}

int SharedMemoryManager::Init(key_t *shmkey,key_t semkey,size_t segmentsize,unsigned short nosegments) {
     DEB_F("int SharedMemoryManager::Init(..) (creation)");
     // create and initialize the Management Info Segment
     TEST_FAILURE(SegmentInfo.Init(shmkey[nosegments],1,true));
     // initialize the semaphore
     TEST_FAILURE(sem.Init(semkey,true));
     // release semaphore *SR*
     TEST_FAILURE(sem.Release());
     SemLock SL(&sem);
     // set segement size
     SegmentSize=SegmentInfo[0].SegmentSize=segmentsize;
     // set number of segments
     NoSegments=SegmentInfo[0].NoSegments=nosegments;
     // create the array of pointers to the shared memory	  
     ShmSegments=new SharedMemSegment[NoSegments];
     // create and initialize the segments
     for(unsigned short i=0;i<NoSegments;i++){
	  SegmentInfo[0].ShmKey[i]=shmkey[i];
	  ShmSegments[i].Init(SegmentInfo[0].ShmKey[i],segmentsize,true);
	  AllocInfo_t *ai=(AllocInfo_t *)ShmSegments[i]();
	  ai->Next=0;
	  ai->Prev=0;
	  ai->Size=0;
	  ai->ID=0;
	  ai->AllocCount=0;
     }
     SharedMemAllocator_t::SetSharedMemoryManager(this);
     return 0;
}

int SharedMemoryManager::Destroy(){
     DEB_F("int SharedMemoryManager::Destroy()");
     delete[] ShmSegments;
     ShmSegments=0;
     return 0;
}

AllocInfo_t *SharedMemoryManager::priv_shmalloc(size_t size,ID_t ID,unsigned short SegmentNo){
     DEB_F("int SharedMemoryManager::priv_shmalloc(..)");
     if (SegmentNo>=NoSegments) {
	  (*Error)=NO_SPACE_IN_SHM;
	  (*Error)="SharedMemoryManager::shmalloc()";
	  ASSERTION_FAILED; 
	  return 0;
     }
     SharedMemSegment *ShmSegment=ShmSegments+SegmentNo;
     size_t ai=0;
     size_t soai=sizeof(AllocInfo_t);
     AllocInfo_t *new_aip,*aip=(AllocInfo_t *)(*ShmSegment)(ai);
	  
     // Find free segment with a size >= size or the end of the list
     while((aip->Next!=0)&&((aip->Next-ai)<(size+2*soai+aip->Size))){
	  ai=aip->Next;
	  aip=(AllocInfo_t *)(*ShmSegment)(ai);
     }
     if(aip->Next==0){
	  // end of list
	  // enough space left in segment SegmentNo ?
	  if(ai+aip->Size+2*soai+size>SegmentSize){
	       // no ! try next segment !
	       return priv_shmalloc(size,ID,SegmentNo+1);
	  }
	  // yes
	  aip->Next=ai+soai+aip->Size;
	  new_aip=(AllocInfo_t *)(*ShmSegment)(aip->Next);
	  new_aip->Prev=ai;
	  new_aip->Next=0;
	  new_aip->Size=size;
	  new_aip->ID=ID;
	  new_aip->AllocCount=0;
     } else {
	  // found space within the list
	  size_t old_ai_next=aip->Next;
	  aip->Next=ai+soai+aip->Size;
	  new_aip=(AllocInfo_t *)(*ShmSegment)(aip->Next);
	  new_aip->Prev=ai;
	  new_aip->Next=old_ai_next;
	  new_aip->Size=size;
	  new_aip->ID=ID;
	  new_aip->AllocCount=0;
	  ((AllocInfo_t *)(*ShmSegment)(new_aip->Next))->Prev=aip->Next;
     }
     return new_aip;
}

void SharedMemoryManager::priv_shmfree(AllocInfo_t *aip){
     DEB_F("int SharedMemoryManager::priv_shmfree(..)");
     // get the segment Number
     unsigned short SegmentNo=0;
     while(((ShmSegments[SegmentNo]()>(void*)aip)||
	    (ShmSegments[SegmentNo](SegmentSize)<(void*)aip))&&
	   (SegmentNo<NoSegments)) SegmentNo++;
     if(SegmentNo<NoSegments){
	  SharedMemSegment *ShmSegment=ShmSegments+SegmentNo;
	  ((AllocInfo_t *)(*ShmSegment)(aip->Prev))->Next=aip->Next;
	  if(aip->Next!=0)
	       ((AllocInfo_t *)(*ShmSegment)(aip->Next))->Prev=aip->Prev;
     }
     return;
}

AllocInfo_t *SharedMemoryManager::priv_shmfind(ID_t ID,unsigned short SegmentNo){
     DEB_F("int SharedMemoryManager::priv_find(..)");
     if (SegmentNo>=NoSegments) return 0;
     SharedMemSegment *ShmSegment=ShmSegments+SegmentNo;
     AllocInfo_t *aip=(AllocInfo_t *)(*ShmSegment)();
     // Find segment with ID or the end of the list
     while((aip->Next!=0)&&(aip->ID!=ID)) 
	  aip=(AllocInfo_t *)(*ShmSegment)(aip->Next);
     if((aip->Next==0)&&(aip->ID!=ID)){
	  // end of list
	  // try next segment !
	  return priv_shmfind(ID,SegmentNo+1);
     } 
     // found
     return aip;
}


