/*
$Header: /home/elrond/baikal/src/CVS/daq98/memman.h,v 1.5 1999/02/03 03:42:19 steffen Exp $
*/

#if ! defined __MEMMAN_H__
#define __MEMMAN_H__
#include <config.h>
#include "ipcclasses.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "pointerutil.h"

#define USE_SHM true
#define NO_SHM  false


/** template used to represent a Shared Memory Block as an Array of
 Data of type T */
template <class T>
class SharedMemArray: public Base
{
public:
     /** constructor */
     SharedMemArray(){
	  DEB_F("SharedMemArray::SharedMemArray()");
     }
     /** destructor */
     ~SharedMemArray(){
	  DEB_F("SharedMemArray::~SharedMemArray()");
     }
     /** initialization 
       shmkey -> key for the shared Memory 
       ( see shmget  system function for 
         more information on keys )
	 size -> number of elements in the array
	 create -> Flag see semaphore*/
     int Init(key_t shmkey,int size,bool create){
	  DEB_F("SharedMemArray::Init(...)");
	  TEST_FAILURE(shm.Init(shmkey,sizeof(T)*size,create));
	  return 0;
     }
     /** Destroys shm */
     int Destroy(){
	  DEB_F("SharedMemArray::Destroy()");
	  TEST_FAILURE(shm.Destroy());
	  return 0;
     }
     /** copies one Array-element (index) form shared memory into
       conventional Memory */
     inline void Get(T &data,int index){
	  DEB_F("T SharedMemArray::Get(..)");
	  memcpy(&data,((T*)shm())+index,sizeof(T));
     }
     /** copies an Array-element from conventional memory into shared
       memory to position index */
     inline void Set(T &data,int index){
	  DEB_F("void SharedMemArray::Set(..)");
	  memcpy(((T*)shm())+index,&data,sizeof(T));
     }
     /** returns a reference to the Array-element at position i in the
       array within the shared memory */
     inline T& operator[](int i){
	  DEB_F("T& SharedMemArray::operator[]");
	  return ((T*)shm())[i];
     }
private:
     /** Object to handle the shared Memory */
     SharedMem shm;
};

/** Class to handle a Segment  of Shared Memory without
  type-specification, uses semaphore to avoid accses conflicts */
class SharedMemSegment: public Base
{
public:
     /** Constructor */
     SharedMemSegment(){
	  DEB_F("SharedMemBlock::SharedMemBlock()");
     }
     /**  Destrutor */
     ~SharedMemSegment(){
	  DEB_F("SharedMemBlock::~SharedMemBlock()");
     }
     /** initialization shmkey -> key for the shared Memory ( see
       shmget system function for more information on keys ) 
       size -> number of bytes in the segment to create */
     int Init(key_t shmkey,int size,bool create){
	  DEB_F("SharedMemBlock::Init()");
	  TEST_FAILURE(shm.Init(shmkey,size,create));
	  return 0;
     }
     /** returns the pointer to the first Byte in the Block */
     inline void* operator()(){
	  DEB_F("void* SharedMemBlock::operator()()");
	  return shm();
     }
     /** returns the pointer to the index-th Byte in the block */
     inline void* operator()(size_t index){
	  DEB_F(" void* SharedMemBlock::operator()(..)");
	  return shm()+index;
     }
     /** copys  size bytes from shared memory starting at index into
       conventional memory at data */
     inline void GetBlock(void *data,size_t index,size_t size){
	  DEB_F("void SharedMemBlock::GetBlock(..)");
	  memcpy(data,shm()+index,size);
     }
     /** copys a size bytes from conventional memory at data into
       shared memory starting at index*/
     inline void SetBlock(void *data,size_t index,size_t size){
	  DEB_F("void SharedMemBlock::SetBlock(..)");
	  memcpy(shm()+index,data,size);
     }
private:
     /** Object to handle the shared Memory */
     SharedMem shm;
};


/** Memory Manager for the shared Memory, the max size for one segment
  of Shared Memory is 16Mbytes, if more is needed more than one
  segment must be used. this class is to manage this and to provide
  functions to Allocate, free and find (from other applications)
  allocated Memory. it uses a list to handle the information, this is
  shown in the figure below, the other segments look similar, all
  addresses are relativ to the beginning of the block since the start
  addrese can be different in different processe\\*/
/*  begin{verbatim}
                Index    Shared Memory Segment 0                                            
                    ______________________________                                        
    Begin->       0|Prev=0    <--+                |                                       
             +-----|Next=20      |                |                                       
             |     |Size=0       |                |                                       
             |     |ID=0         |                |                                       
             |     |             |                |                                       
             |     |-------------|----------------|--+                                    
             +-> 20|Prev=0 ------+ <----+AllocCnt.|  |                                    
         +---------|Next=245            |         |  |                                    
         |         |Size=40 -+          |         |  |                                    
         |         |ID=5631  |          |         |   > Block 0                           
       	 |    20+so|---------|----------|---------|  |                                    
         |         |      |  |   Data   |         |  |                                    
         |         |      |--+ 40 bytes |         |  |                                    
         |         |      |      ..     |         |  |                                    
         | 20+so+40|--------------------|---------|--+                                    
         |         |            free    |         |                                       
         |         |             ..     |         |                                       
         |         |            ....    |         |                                       
         |         |                    |         |                                       
         |         |                    |         |                                       
         |         |                    |         |                                       
         |         |                    |         |                                       
         |         |--------------------|---------|--+                                    
         +---->	245|Prev=20 ------------+AllocCnt.|  |                                    
                   |Next=0                        |  |                                    
                   |size=10                       |  |                                    
                   |ID=349875                     |   > Block 1                           
                   |------------------------------|  |                                    
                   |            Data              |  |                                    
                   |          10 bytes            |  |                                    
                   |------------------------------|--+                                    
                   |            free              |                                       
                   |                              |                                       
                   |______________________________|                                       
						       
\end{verbatim}						       
*/				 		       
						       


/** Managment structure, is attached in front of each allocated segment */
struct AllocInfo_t{
     /** index of the next block */
     size_t Prev;
     /** index of the previos block */
     size_t Next;
     /** size of the block */
     size_t Size;
     /** ID of the block */
     ID_t ID;
     /** number of allocations of this block */
     unsigned short AllocCount;
};

class SharedMemoryManager: public Base
{
public:
     SharedMemoryManager();
     ~SharedMemoryManager();
     int Init(key_t shmkey,key_t semkey);
     int Init(key_t *shmkey,key_t semkey,size_t segmentsize,unsigned short nosegments);
     int Destroy();
     void *shmalloc(size_t size,ID_t ID){
	  DEB_F("void *SharedMemoryManager::shmalloc(..)");
	  // first look, if ID allready exists
	  AllocInfo_t *ai;
	  if(!(ai=priv_shmfind(ID))){
	       // no
	       if(size>0) ai=priv_shmalloc(size,ID); else return 0;
	  }
	  if(ai==0) return 0;
	  ai->AllocCount++;
	  return ai+1;
     }  
     void shmfree(void *ptr){
	  DEB_F("void SharedMemoryManager::shmfree(..)");
	  AllocInfo_t *ai=((AllocInfo_t*)(ptr))-1;
	  if((--(ai->AllocCount))==0){
	       priv_shmfree(ai);
	  }
     }
     void *shmfind(ID_t ID){
	  DEB_F("void *SharedMemoryManager::shmfind(..)");
	  AllocInfo_t *ai=priv_shmfind(ID);
	  if(ai==0) return 0;
	  return ai+1;
     }
     inline int LockMem(){
	  DEB_F("int SharedMemoryManager::LockMem()");
	  return sem.Get();
     }
     inline int UnlockMem(){
	  DEB_F("int SharedMemoryManager::UnlockMem()");
	  return sem.Release();
     }
private:
     AllocInfo_t *priv_shmalloc(size_t size,ID_t ID,unsigned short SegmentNo=0);
     void priv_shmfree(AllocInfo_t *ptr);
     AllocInfo_t *priv_shmfind(ID_t ID,unsigned short SegmentNo=0);
    /** Managment structure, contains information about the size, the
       number of segments, and the ipc-keys of all segments, is used to
       provide this information to processes that want to attach to
       the shared memory */
      struct SegmentInfo_t{
	  unsigned short NoSegments;
	  size_t SegmentSize;
	  key_t ShmKey[MAX_SHM_SEGMENTS];
     };
     /** Shared Memory Object to hold the Management information */
     SharedMemArray<SegmentInfo_t> SegmentInfo;
     /** Array of shared memory to hold the Segments, where the menory
       for the blocks is allocated from */
     SharedMemSegment *ShmSegments;
     /** Number of Segments */
     unsigned short NoSegments;
     /** size of Segments */
     size_t SegmentSize;
     Semaphore sem;
};


/** abstract base class for an allocator class, that is used for
    memory allocation and deallocation */
class Allocator_t: public Base
{
public:
     /** allocate size bytes of memory and assign ID to it if
         possible,ID Assignment only possible for Shared Memory, not
         for process memory */
     virtual void *alloc(size_t bytes,ID_t ID)const=0;
     /** deallocate the memory on ptr */
     virtual void free(void *ptr)const=0;
};

/** class, responsible for the alloction of shared memory */
class SharedMemAllocator_t:public Allocator_t
{
public:
     /** allocation and ID assignment*/
     virtual void *alloc(size_t bytes,ID_t ID)const;
     /** deallocation */
     virtual void free(void *ptr)const;
     /** set a pointer to the shared memory manager */
     static void SetSharedMemoryManager(SharedMemoryManager *sm);
private:
     static SharedMemoryManager *ShmMan;
};


/** class, responsible for the alloction of process memory */
class ProcessMemAllocator_t:public Allocator_t
{
public:
     /** allocation, ID not needed */
     void *alloc(size_t bytes,ID_t)const;
     /** deallocation */
     void free(void *ptr)const;
};     

/** base class used to handle an array of raw bytes, that are actually
    a structure with variable length, e.g. an event or a histogram */
class RawBytesManager: public Base
{
public:
     /** default constructor, no memory is allocated */
     inline RawBytesManager(){
	  DEB_F("RawBytesManager::RawBytesManager()");
	  Allocator=&ProcessMemAllocator;
	  WasCreated=false;
	  Pointer2Data=0;
	  SizeOfData=0;
     } 
     /** default constructor, no memory is allocated */
     inline RawBytesManager(const bool Mem){
	  DEB_F("RawBytesManager::RawBytesManager(const int Mem)");
	  if(Mem==USE_SHM)
	       Allocator=&SharedMemAllocator;
	  else 
	       Allocator=&ProcessMemAllocator;
	  WasCreated=false;
	  Pointer2Data=0;
	  SizeOfData=0;
     } 
     /** constructor, no memory is allocated, the object is attached
         to existing data in memory at ptr, no copy !!! */
     inline RawBytesManager(const void *ptr){
	  DEB_F("RawBytesManager::RawBytesManager(const void *ptr)");
	  Allocator=&ProcessMemAllocator;
	  WasCreated=false;
	  Attach(ptr);
	  SizeOfData=0;
     }       
     /** constructor, no memory is allocated, the object is attached
         to existing data in memory with ID ID, no copy !!! (only
         shared memory)*/
     inline RawBytesManager(const ID_t ID){
	  DEB_F("RawBytesManager::RawBytesManager(const ID_t ID)");
	  Allocator=&SharedMemAllocator;
	  WasCreated=false;
	  Attach(ID);
	  SizeOfData=0;
     }
     /** constructor, memory is allocated, but not filled with any
         data */
     inline RawBytesManager(const bool Mem,const size_t size){
	  DEB_F("RawBytesManager::RawBytesManager(const bool UseSharedMem,const size_t size)");
	  if(Mem==USE_SHM)
	       Allocator=&SharedMemAllocator;
	  else 
	       Allocator=&ProcessMemAllocator;
	  WasCreated=false;
	  Create(size,0);
	  SizeOfData=size;
     }
     /** copy constructor, memory is allocated, an the content of the
         RBM object is copied into the new memory area */
     inline RawBytesManager(const RawBytesManager &RBM){
	  DEB_F("RawBytesManager::RawBytesManager(const RawBytesManager &RBM)");
	  Allocator=&ProcessMemAllocator;
	  WasCreated=false;
	  SizeOfData=RBM.SizeOfData;
	  if(Create(SizeOfData,0)==0)
	       memcpy(Pointer2Data,RBM.Pointer2Data,SizeOfData);
     }       
     /** destructor, deallocates the allocated memory */
     inline ~RawBytesManager(){
	  DEB_F("RawBytesManager::~RawBytesManager()");
	  Delete();
     }       
     /** checks wether the instnace has valid data */
     inline const bool IsValid()const{
	  DEB_F("bool RawBytesManager::IsValid()");
	  return Pointer2Data!=0;
     }    
     /** returns the size of the data in bytes */
     inline const size_t Size()const {
	  DEB_F("size_t RawBytesManager::Size()");
	  return SizeOfData;
     }
     /** creates a serial stream of the data in buffer buf, buf must
         be big enough */
     inline void CopySerialize(void *buf)const{
	  DEB_F("void RawBytesManager::Serialize(void *buf)");
	  if(Pointer2Data){
	       memcpy(buf,Pointer2Data,SizeOfData);
	  }
     }
     /** deserializes a serial stream from buf, allocates new memory
         and fills it with the data from the stream */
     inline void CopyDeserialize(const void *buf,const size_t size){
	  DEB_F("void RawBytesManager::Deserialize(const void *buf,const size_t size)");
	  if (Create(size,0)==0){
	       SizeOfData=size;
	       memcpy(Pointer2Data,buf,SizeOfData);
	  }
     }       
protected:
     /** creates a memry array */
     inline const int Create(size_t size,ID_t ID){	
	  DEB_F("int RawBytesManager::Create(size_t size,ID_t ID)");
	  if(WasCreated) Delete();
	  if((Pointer2Data=Allocator->alloc(size,ID))==0) return -1;
	  SizeOfData=size;
	  WasCreated=true;
	  return 0;
     }
     /** attaches the instance to an existing memory array at ptr */
     inline int Attach(const void *ptr){
	  DEB_F("int RawBytesManager::Attach(const void *ptr)");
	  if(WasCreated) Delete();
	  Pointer2Data=(void*)ptr;
	  WasCreated=false;
	  return 0;
     }
     /** attaches the instance to an existing memory with ID id (only
         sharde memory) */
     inline int Attach(ID_t ID){
	  DEB_F("int RawBytesManager::Attach(ID_t ID)");
	  if(WasCreated) Delete();
	  if((Pointer2Data=Allocator->alloc(0,ID))==0) return -1;
	  WasCreated=false;
	  return 0;
     }
     /** deallocates the allocated memory */
     inline void Delete(){
	  DEB_F("void RawBytesManager::Delete()");
	  if(WasCreated){
	       Allocator->free(Pointer2Data);
	  }
     }
     /** set the real size of the data, must be called from derived
         objects after construction when the size parameter is omitted !!!! */
     inline void SetSize(size_t size){
	  DEB_F("void RawBytesManager::SetSize(size_t size)");
	  SizeOfData=size;
     }
     inline void *GetPointer(){
	  DEB_F("void *RawBytesManager::GetPointer()");
	  return Pointer2Data;
     }
private:
     /** flag, indicates that this instance allocated memory */
     bool WasCreated;
     /** reference to the allocator object */
     const Allocator_t *Allocator;
     /** pointer to the data */
     void *Pointer2Data;
     /** soze of the data */
     size_t SizeOfData;
     /** default allocator object for shared memory, is static*/
     static SharedMemAllocator_t SharedMemAllocator;
     /** default allocator object for process memory, is static */
     static ProcessMemAllocator_t ProcessMemAllocator;
};


#endif //__MEMMAN_H__
