/*
$Header: /home/elrond/baikal/src/CVS/daq98/moduleevalbase.h,v 1.6 1999/02/05 18:35:07 steffen Exp $
*/

#if !defined __MODULEEVALBASE_H__
#define __MODULEEVALBASE_H__

#include <config.h>
#include "daqprocessbase.h"
#include "pointerutil.h"
#include "globalfunctions.h"
#include "event.h"
#include "binarydatabuffer.h"

class TempDecode_EventData_t;

/** Evaluator base class for modules.
    All classes that implement a module must be derived from 
    either from this class or from {\tt \Ref{GenericModuleEvalBaseClass}}
    To implement a module you have to write a class 
    following this example:\\
    \begin{verbatim}
    class NameOfModuleEvalClass_t: public NameOfModuleEvalBaseClass_t
    {
        enum 
	{ 
	    NUMBER_OF_CHANNELS =HERE_THE_NUMBER_OF_CHANNELS_OF_THE_MODULE,
	    BYTES_PER_INFO_UNIT=HERE_THE_NUMBER_OF_BYTES_PER_INFO_UNIT
	};
	ClassTypeInfo(NameOfModuleEvalBaseClass_t);
     public:
        inline NameOfModuleEvalClass_t()
	    :ModuleEvalBaseClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT){}
	// set additional information out of the config file, like channel mapping etc.
        virtual void Configure(ModuleConfigRecord_p &ModuleConfig,HWConfig_t &HWConfig);

        // read the data and build the event part that this module is good for 
        virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				   TempDecode_EventData_t *TempEventBuffer);

     protected:
        virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){
	// allocate some staic memory, so we can return a pointer to
	// the memory with no worry
	static u_i32 Data;
	// read the data from the buffer
	DataBuffer.ReadFromBuffer(Data);
	// return a pointer to the data
	return (void*)&Data;
     };
	
    }
    \end{verbatim}
*/

class ModuleEvalBaseClass_t:public ClassWithTypeInfo_t
{
     ClassTypeInfo(ClassWithTypeInfo_t);
public:
     inline ModuleEvalBaseClass_t(int numChannels,int bytesPerInfoUnit, DaqProcessBase *parent){
	  Parent=parent;
	  NumChannels=numChannels;
	  BytesPerInfoUnit=bytesPerInfoUnit;

	  ChannelVector=new ChannelVector_t(NumChannels);
	  for(int i=0;i<NumChannels;i++) (*ChannelVector)[i]=-1;
     }

     virtual ~ModuleEvalBaseClass_t(){
	  delete ChannelVector;
     }
     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &HWConfig)=0;

     inline int GetBytesPerInfoUnit(){return BytesPerInfoUnit;}
     inline int GetNoOfChannels(){ return NumChannels; }
     inline binary_tag_t GetTag(){return Tag;}
     inline void SetTag(binary_tag_t tag){Tag=tag;}
     inline s_i32 GetChannelMapping(u_i32 ModuleChannel){
	  return (*ChannelVector)[ModuleChannel];
     }
     inline void SetChannelMapping(u_i32 ModuleChannel,s_i32 OMChannel){
	  (*ChannelVector)[ModuleChannel]=OMChannel;
     }

     // read the data and build the event part that this module is good for 
     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer)=0;
     /** Prints a message to a common log file.
	 @param Msg Message to print
	 @param LogLevelFile Log file verbose level
	 @param LogLevelConsole Console verbose level
	 @param Type Message Type, can be either {\tt ERROR}, {\tt WARNING},
	 or {\tt INFO} */
     void LogMsg(const char* Msg, int LogLevelFile=10, int LogLevelConsole=10,LogType_t Type=INFO){
	  Parent->LogMsg(Msg,LogLevelFile,LogLevelConsole,Type);
     }

protected:
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer)=0;     

protected:
     int NumChannels;
     int BytesPerInfoUnit;
     MAKE_VECTOR(s_i32,ChannelVector_t);
     binary_tag_t Tag;
     ChannelVector_t *ChannelVector;
     DaqProcessBase *Parent;
};


#endif // __MODULEEVALBASE_H__

