/*
$Header: /home/elrond/baikal/src/CVS/daq98/moduletest.h,v 1.3 1998/11/23 14:43:46 haase Exp $
*/

#if ! defined __MODULETEST_H__
#define __MODULETEST_H__
#include <config.h>
// STL
#include <strstream.h>
//
#include <String.h>
#include "vmeglobalmacros.h"
#include "simplevector.h"
#include "stringvector.h"

#include "globalmacros.h"
#include "globaltypes.h"

#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "errorlog.h"
#include "dispatcherlib.h"
#include "daemon.h"
#include "vmedaqhal.h"
#include "memman.h"

class ModuleTest_SignalHandler:public SignalHandlerClass
{
     virtual int Handle_SIGCHLD(Process &P);
     virtual int Handle_SIGUSR1(Process &P);
};

class ModuleTestConfig:public ConfigSection
{
public:
     /** ctor */
     ModuleTestConfig();
     /**return the name of the section in the config file, where the
	config items are placed */
     virtual const String GetSectionName()const;
     /** sets property propnum */
     virtual int SetPropertyByNo(int propnum,const String &val);
     /** returns property propnum */
private:
     port_t DataPort;
     String Props[2];
};


class ModuleTest:public Daemon
{
public:
     
     /** ctor */
     ModuleTest();
     /** dtor */
     ~ModuleTest();
     /** initialization */
     Init();
     /** destruction */
     Destroy();
     
     /** returns the type of the process, must be overwritten in
         derived class */
     virtual const String GetProcessType()const;
     /** Sets up the data structure for the Hardware, according to the
         configuration from the Config Tree */
     int SetUpHAL(StringVector_t &errors);
     /** write the detector configuration into the data buffer */
     int WriteDetectorConfigToBuffer(BinaryDataBuffer &Buf);
     
protected:

     /** Builds the event, saves it in a big buffer, and when the
         buffer is full, send it over the net to the dataprocess */
     int BuildAndSendEvent(); 
protected: 
     /** the event dispatcher */
     virtual int DispatchEvent();
     /** The function that polls for a trigger, set the time value
         with Selecter.SetTimer(0);*/
     virtual int DoTIMEREvent(); 
     /** the IPC event handler */
     virtual int DoIPCEvent(const String &Cmd,SlaveServerSocket &s);
     /** returns a String with the configuration file name */
     virtual const String GetConfigFileName()const;
     /** returns a pointer to the configuration class, created with
         new on the heap, must be overwritten in derived class */
     virtual ConfigSection *GetConfigSection();
     
private:
     bool IsConfigured;
     ClientSocket DataSocket;
     SocketNotifier DataSocketNotifier;

     Interpreter<ModuleTest> CMD_Interpreter;
     
     int IPC_CMD_Quit(IOSocket &s,String *params,int numparams);
     int IPC_CMD_Help(IOSocket &s,String *params,int numparams);
     int IPC_CMD_Start(IOSocket &s,String *params,int numparams);
     int IPC_CMD_Stop(IOSocket &s,String *params,int numparams);
     int IPC_CMD_Initialize(IOSocket &s,String *params,int numparams);
     int IPC_CMD_UnknownCommand(IOSocket &s,String *params,int numparams);
     int IPC_CMD_WrongTypeCommand(IOSocket &s,String *params,int numparams);
     

     int SendUsageInfo(IOSocket &s,String param);     
private:
     HAL_type hal;
     /** The vector that holds the configuration for the PMT channels */
     ChannelConfigVector_t ChannelConfig;
     /** The vector that holds the configuration for the crates */
     CrateConfigVector_t   CrateConfig;  
     /** The vector that holds the configuration for the ADCs */
     ModuleConfigVector_t  ModuleConfig;
     /** class that holds the buffer for the events */
     BinaryDataBuffer *RawDataBuffer;
     
public:
     static pid_t DataTransporter_PID;
};

#endif // __MODULETEST_H__



