/*
  $Header: /home/elrond/baikal/src/CVS/daq98/moniclient.C,v 1.13 1999/02/03 07:07:44 steffen Exp $
*/

#include <config.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qapp.h>
#include <qtimer.h>
#include <qmsgbox.h> 
#include <qpushbt.h> 
#include <qfiledlg.h> 
#include <qpixmap.h> 
#include <iostream.h>
#include <string.h>

#include <qwidget.h>
#include <qlined.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qbttngrp.h> 

#include <localdir.h>
#include "cnfreader.h"
#include "buttonbox.h"
#include "conswidget.h"
#include "pointerutil.h"
#include "clientlib.h"
#include "moniclient.h"
#include "interpreter.h"

#define USE_PAWDISPLAY          // display via PAW
#define USE_ROOTDISPLAY         // display via ROOT

typedef void (*PTR_VOID)();
typedef void (*PTR_CHAR)(const char *);


int MoniClient_SignalHandler::Handle_SIGPIPE(Process &/*P*/)
{
     return 0;
}

int MoniClient_SignalHandler::Handle_SIGCHLD(Process &/*P*/)
{
     return 0;
}

MonitorClient::MonitorClient(QWidget *parent,const char *name)
     :QWidget(parent,name), TableView (0){
     ThisProcess->SetSignalHandler(SignalHandler);
     setIcon(QPixmap(DAQ_ICON));
     setCaption(DAQ_CAPTION);
     Display=PAW;
     ADS_Dir=String(DAQ_ROOT_DIR) +String("/share/") + PACKAGE;
     StartRunAllFile="startrunall.ads";
     StopRunAllFile="stoprunall.ads";
     InitRunFile="initrun.ads";
     // ---------- Build Menu -----------------------------------------------------
     // ---------- File --------------
     FileMenu=new QPopupMenu();
     CHECK_PTR(FileMenu);
     OpenID=FileMenu->insertItem("Open Connection",this,SLOT(SL_OpenConnection()),CTRL+ALT+Key_O);
     CloseID=FileMenu->insertItem("Close Connection",this,SLOT(SL_CloseConnectionUnsafe()),CTRL+ALT+Key_C);
     FileMenu->setItemEnabled(CloseID, FALSE);
     FileMenu->insertSeparator();
     FileMenu->insertItem("Exit",this,SLOT(quit()),CTRL+ALT+Key_Q);
     
     // ---------- Tools -------------
     ToolMenu=new QPopupMenu();
     CHECK_PTR(ToolMenu);
     ToolMenu->insertItem("&Runstart",this,SLOT(SL_RunStartWidget()),CTRL+ALT+Key_R);
     MsgLoggingID=ToolMenu->insertItem("&Message Logging",this,SLOT(SL_MsgLogging()),CTRL+ALT+Key_M);
     MsgLogging=FALSE;
     ToolMenu->setCheckable(TRUE);
     
     ToolMenu->insertItem("&PAW",this,SLOT(SL_startPAW()),CTRL+ALT+Key_P);
//     ToolMenu->insertItem("&ROOT",this,SLOT(SL_startROOT()),CTRL+ALT+Key_R);
     
     // ---------- Histograms --------
     HistogramMenu=new QPopupMenu();
     CHECK_PTR(HistogramMenu);
     GetHistoListID=HistogramMenu->insertItem("&Get Histogram List",this,SLOT(SL_GetHistoList()),CTRL+ALT+Key_G);
//     MsgLogging=FALSE;
//     ToolMenu->setCheckable(TRUE);

     // ---------- Help  -------------
     QPopupMenu *help=new QPopupMenu;
     CHECK_PTR(help);
     help->insertItem("Topics",this,SLOT(SL_HelpTopics()),Key_F1);
     help->insertItem("Index",this,SLOT(SL_HelpIndex()),SHIFT+Key_F1);
     help->insertSeparator();
     help->insertItem("About",this,SLOT(SL_HelpAbout()),ALT+Key_A);

     // ---------- Menu --------------
     menu=new QMenuBar(this);
     CHECK_PTR(menu);
     menu->insertItem("&File",FileMenu);
     menu->insertItem("&Histograms",HistogramMenu);
     menu->insertItem("&Tools",ToolMenu);
     menu->insertItem("&Histograms",HistogramMenu);
     menu->insertSeparator();
     menu->insertItem("&Help",help);
//     menu->setSeparator( QMenuBar::InWindowsStyle );
     menu->setFrameStyle(QFrame::WinPanel|QFrame::Raised);
     menu->setStyle(MotifStyle);
     
     setMinimumSize(100,80);
     PAWConsole=0;
     ROOTConsole=0;
     
//------------------- Widgets ----------------------------------------------
     Input=new QLineEdit(this);
     QToolTip::add(Input,"Enter next command here");
     Output=new QTextView(this);
     QToolTip::add(Output,"Daemon output messages");
     Messages=new QTextView(this);
     QToolTip::add(Messages,"Daemon info messages");

//------------------- Other Widgets -------------------------------------------
     RunStartW =new RunStartWidget(this,0);


//------------------- Geometry --------------------------------------------
     SetGeometry();
     connect(Input,SIGNAL(returnPressed()),
	     this,SLOT(ServCommandFromUser()));
     ServerSocketNotifier=0;
     
     LoggedIn=0;
     MozillaColorMap=ON_val;
     HelpRoot="file:/";
     char *home=getenv("HOME");
     if(home)
	  HelpRoot+=String(home);
     else
	  HelpRoot+=String(SYSTEM_HELP_ROOT);
     HelpRoot+=String(USER_HELP_ROOT);
}

MonitorClient::~MonitorClient()
{
}

int MonitorClient::Init()
{
     TEST_FAILURE(ReadConfigFile());
     Mozilla.UseColorMap(MozillaColorMap);
     return 0;
}

int MonitorClient::Destroy()
{
     return 0;
}

int MonitorClient::InfoMsg(const char *txt)
{
     QMessageBox::information(this, DAQ_CAPTION,
			      txt);
     
     return 0;
}

int MonitorClient::PrintInfoText(const char *txt){
     Output->append(txt);
     return 0;
}
     
int MonitorClient::PrintMessageText(const char *txt){
     static Regex rates("rates: .*");
     static Regex slash(" / .*");
     
     Messages->append(txt);
     if(MsgLogging){
	  time_t t, dummy;
	  t=time(&dummy);
	  String date(ctime(&t));
	  date.gsub("\n"," ");
	  msgfile <<  date << txt << flush;
	  String eventstr1=(String(txt)).at(rates);
	  String byterate=(String(eventstr1)).at(RXdouble);
	  String eventstr2=(String(eventstr1)).at(slash);
	  String eventrate=(String(eventstr2)).at(RXdouble);
     
	  datafile << t << "\t"
		   << (const char *)byterate << "\t" 
		   << (const char *)eventrate 
		   << endl;
     }
     
     return 0;
}

int MonitorClient::PrintErrorText(const char *txt){
     QMessageBox::critical(this, DAQ_CAPTION,
			   txt);
     
     return 0;
}

int MonitorClient::PrintWarningText(const char *txt){
     Output->append("Warning: ");
     Output->append(txt);
     return 0;
}

int MonitorClient::PrintRUSUREText (const char *txt)
{
	// txt = "Message\nCommand\r\n"

	char msg[80];
	char cmd[80];

	char *cmd_p;

	strcpy (msg, txt);
	cmd_p = index (msg, '\n');
	*cmd_p = '\0';
	cmd_p++;

	strcpy (cmd, cmd_p);
	cmd_p = index (cmd, '\r');
	*cmd_p = '\0';

	int ret = QMessageBox::warning (this, DAQ_CAPTION, msg,
			 "OK", "Cancel");

	if (0 == ret)
		ServCommand (cmd, true);

	return 0;
}

int MonitorClient::DoPAWPipe (const char *txt)		/* *PN* 211097 */
{
	if (!PAWConsole)
	{
		SL_startPAW ();
	}

	if (!PAWConsole->stdinput ())
		return 0;

	ostream &out = PAWConsole->stdinput ();
	out.setf (ios::showpoint);		// format

	// remove the \r 
	char *r_pos = index (txt, '\r');
	*r_pos = '\0';
	out << txt << flush;		// endl ??

	return 0;
}

int MonitorClient::DoROOTPipe (const char *txt)		/* *PN* 211097 */
{
	if (!ROOTConsole)
	{
		SL_startROOT ();
	}

	if (!PAWConsole->stdinput ())
		return 0;

	ostream &out = ROOTConsole->stdinput ();
	out.setf (ios::showpoint);		// format

	// remove the \r 
	char *r_pos = index (txt, '\r');
	*r_pos = '\0';
	out << txt << flush;		// endl ??

	return 0;
}

int MonitorClient::DoTableView (const char *txt)		/* *PN* 041297 */
{
	static TableViewParams TVP;
	bool eot = false;
	eot = (strstr (txt, "EOT") != NULL);
	if (!eot)
	{
		String words[20];
		int ntoken = split (txt, words, 20, RXwhite);
		if (ntoken > 1)
			TEST_FAILURE (TVP.SetProperty (words[0],
				join (words+1, ntoken - 1, " ")));
	}
	else
	{
	     if (!TableView)
		  SL_startTableView (TVP);
	     TVP.Clear ();			// really necessary
	     // because TVP is static
	}
	return 0;
}

int MonitorClient::DoHistogram(MemHistogram_t &Histogram){
     if(Display==PAW)
	  DoHistogramPAW(Histogram);
     else if(Display==ROOT)
	  DoHistogramROOT(Histogram);
/*     else {
	  map<int,GraphView_p,less<int> >::iterator hv_it=HistWindows.find(Histogram().ID);
	  if(hv_it!=HistWindows.end()){
	       ((HistogramGraph*)(Graph*)((*hv_it).second->GraphObject()))->Update(&Histogram);
	       (*hv_it).second->show();
	       (*hv_it).second->repaint();
	  } else {
	       Graph_p g=new HistogramGraph(&Histogram);
	       GraphView_p hv=new GraphView(g);
	       char cap[100];
	       sprintf(cap,"Hist. %s, %i",Histogram().Title,Histogram().ID);
	       hv->setCaption(cap);
	       HistWindows[Histogram().ID]=hv;
	       hv->show();
	  }
     }
     */     
     return 0;
}

int MonitorClient::DoHistogramPAW(MemHistogram_t &Histogram){
     if(!PAWConsole){
	  SL_startPAW();
     }
     if(!PAWConsole->stdinput()) return 0;
     ostream &out=PAWConsole->stdinput();
     out.setf(ios::showpoint);
//     out << "vector/create tempvec(" << Histogram().NumBin << ") r ";           //  paw input line is cutted
//     for(unsigned int i=0;i<Histogram().NumBin;i++)
//	  out << Histogram().Ent[i] << " ";

//   vec/put
//
//     unsigned int  nl = 50;              // problems with nl~250 observed
//     unsigned int  max_s_vec = 10000;
//     char s_vec[max_s_vec];
//     s_vec[0]='\0';
//     out << "vector/create tempvec(" << Histogram().NumBin << ") r " << endl;
//     for(unsigned int i=0, j=0; i<Histogram().NumBin; i++){
//	  j++;
//	  sprintf (s_vec+strlen(s_vec),"%d  ",Histogram().Ent[i]);
//	  if (strlen(s_vec) > max_s_vec)   {
//	       cerr << "**** to long PAW-vector string ***** " << endl;
//	       out  << "vector/del tempvec" 
//		    << endl << flush;
//	       break;
//	  }
//	  if (j==nl){
//	       out << "vec/input tempvec(" 
//		   << i-nl+2 << ":" 
//		   << i+1 << ") " 
//		   << s_vec << endl;          // vec=1,...nl
//	       j=0;
//	       s_vec[0]='\0';
//	  }
//    }
//     out << endl << flush;


// vec/read
//
     char templ[]="tmp_pawvec_XXXXXX";
     char *tmp= mktemp((char*)templ);

     ofstream file;
     file.open(tmp,ios::out|ios::bin);
     if (!file){
	  cerr << "**** error opening PAW temp file ******" << endl;
      return (-1);
     }
     
     char s_out[255];
     for(unsigned int i=0; i<Histogram().NumBin; i++){
//	  cout <<" i, ent =" << i << " " << Histogram().Ent[i] << endl;
	  sprintf (s_out,"%d  \n",Histogram().Ent[i]);
	  file.write (s_out,strlen(s_out));
     }
     file.close();
     
     out << "vector/create tempvec(" << Histogram().NumBin << ") r " << endl << flush;

     out << "vec/read  tempvec " 
	 << templ << endl;
     out << "sh rm -f " 
	 << templ <<endl;

     
     out << "/histogram/create/1dhisto " 
	 << Histogram().ID << " " 
	 << "'" << Histogram().Title << "' "
	 << Histogram().NumBin << " " 
	 << Histogram().Xmin << " "
	 << Histogram().Xmax << " " 
	 << endl << flush;

     out << "put/contents " 
	 << Histogram().ID 
	 << " tempvec "  
	 << endl << flush;

     out << "call hfill(" 
	 << Histogram().ID << "," 
	 << Histogram().Xmin-1 << ",0.,"
	 << Histogram().NumUnderflows
	 << ".)"
	 << endl << flush;
	  
     out << "call hfill(" 
	 << Histogram().ID << "," 
	 << Histogram().Xmax+1 << ",0.,"
	 << Histogram().NumOverflows
	 << ".)"
	 << endl << flush;

     out << "vector/del tempvec" << endl << flush;
     out << "histogram/plot " << Histogram().ID << endl << flush;
        
     return 0;
}

int MonitorClient::DoHistogramROOT(MemHistogram_t &Histogram){
     if(!ROOTConsole){
	  SL_startROOT();
     }
     if(!ROOTConsole->stdinput()) return 0;
     ostream &out=ROOTConsole->stdinput();
     out.setf(ios::showpoint);
     out << "{\n"
	 << "gROOT->Reset();"
	 << "daq98histo" << Histogram().ID << "=new TCanvas(\"h" 
	 <<  Histogram().ID << "\",\""
	 << Histogram().Title
	 << "\",200,10,600,400);\n"
	 << "daq98histo" << Histogram().ID << "->SetGrid();\n"
	 << "h" << Histogram().ID << "= new TH1F(\"h" << Histogram().ID << "\",\""
	  << Histogram().Title
	  << "\"," << Histogram().NumBin << ","
	  << Histogram().Xmin << ","
	  << Histogram().Xmax << ");\n" 
	 << "h" << Histogram().ID << "->SetMarkerStyle(21);\n"
	 << "h" << Histogram().ID << "->SetMarkerSize(0.7);\n"
	 << "h" << Histogram().ID << "->SetFillColor(16);\n";
     
     for(unsigned int i=0; i<Histogram().NumBin; i++){
	  out << "h" << Histogram().ID << "->Fill(" << i << "," << Histogram().Ent[i] << ");\n";
     }
     out << "h" << Histogram().ID << "->Draw();\n"
	  << "daq98histo" << Histogram().ID << "->Update();\n"
	  << "}\n" << flush;
     
     return 0;
}

void MonitorClient::quit()
{
  int ret=0;
  if(LoggedIn) {
    ret = QMessageBox::information(this, DAQ_CAPTION,
				   "You are still logged in.\n\
Do you want to close the\nconnection and quit?",
				   "Yes", "No");
    if(ret==0) {
	 SL_CloseConnection();
    }else {
	 return;
    }
  }
  qApp->quit();
}

void MonitorClient::SL_Help()
{
     InfoMsg("Yeah, dream on!\n\nDo you really think\n\
you get help for a beta version?");
}

void MonitorClient::SL_RunScript(){
     if(!LoggedIn) {
	  SL_OpenConnection();
	  if(!LoggedIn) {
	       PrintErrorText("You must login before running a script.");
	       return;
	  }
     }
     
     String s((const char*) QFileDialog::getOpenFileName(0,DAQ98_SCRIPT_EXTENSION) );
     if ( s=="" )
	  return;
     RunScript(s);
}

void MonitorClient::RunScript(String &s)
{
     String file= ADS_Dir + "/" +s;
     ifstream infile((const char *)file);
     if(!infile) {
	  String err=(String)"Can't open '" + file +
	       (String)"':\n"+(String)strerror(errno);
	  PrintErrorText(err);
	  
	  return;
     }
     char linebuffer[MAX_TEXT_SIZE];
     Interpreter<MonitorClient>::IStringSplitter I;
     String str;
     
     while(!infile.eof()) {
	  infile.getline(linebuffer,MAX_TEXT_SIZE);
	  str=linebuffer;
	  
	  if(I.CleanUpString(str)) {
	       ServCommand(linebuffer,true);
	       ServReply();
	       if(HasReplyError()){
		    PrintErrorText("Error while running script\n");
		    break;
	       }
	  }
     }
}

void MonitorClient::SL_MsgLogging(){
     bool error=false;
     MsgLogging = !MsgLogging;
     if(MsgLogging) {
	  String s((const char*) QFileDialog::getOpenFileName(0,LOG_EXTENSION_GLOB) );
	  if ( s=="" ) {
	       error=true;
	  }
	  else {
	       msgfile.open(s);
	       if(!msgfile){
		    String err=String("Opening log file '") + 
			 s + String("': ") + String(strerror(errno));
		    PrintErrorText(err);
		    error=true;
	       }
	       s.gsub(Regex("\\..*"),".data");
	       datafile.open(s);
	       if(!datafile){
		    String err=String("Opening data file '") + 
			 s + String("': ") + String(strerror(errno));
		    PrintErrorText(err);
		    error=true;
	       }
	       
	  }
	       
	  
     }
     else {
	  msgfile.close();
	  datafile.close();
     }
     if(error) {
	  MsgLogging=!MsgLogging;
     }
     ToolMenu->setItemChecked( MsgLoggingID, MsgLogging);
}

void MonitorClient::SL_HelpAbout(){
     QMessageBox::about( this, "About AMANDA DAQ98",
                         "AMANDA DAQ98\n\n"
                         "Heiko Haase, Peter Niessen, \n"
			 "Steffen Richter, and Ralf Wischnewski\n\n"
                         "http://www.ifh.de/~strichte/daq98\n" );}

void MonitorClient::SL_HelpTopics()
{
     if(Help("topics.html")<0){
	  PrintErrorText(Base::Error->GetErrorMessage());
     }
}

void MonitorClient::SL_HelpIndex()
{
     if(Help("hindex.html")<0){
	  PrintErrorText(Base::Error->GetErrorMessage());
     }
}

void MonitorClient::resizeEvent(QResizeEvent *){
     SetGeometry();
}

void MonitorClient::SL_OpenConnection(){
     LoginDialog LD;
     char txt[100];
     int timeout;
     
     QString HostName=getenv("HOST");
     QString LoginName=getenv("USER");
     QString Password="";
     
     if(HostName) LD.HostName->setText(HostName);
     if(LoginName) LD.LoginName->setText(LoginName);
     if(LD.exec()){
	  HostName=LD.HostName->text();
	  if(HostName.length()<1){
	       PrintErrorText("No hostname specified");
	       return;
	  }
	  LoginName=LD.LoginName->text();
	  if(LoginName.length()<1){
	       PrintErrorText("No login name specified");
	       return;
	  }
	  Password=LD.Password->text();
	  if(Password.length()<1){
	       PrintErrorText("No password specified");
	       return;
	  }
	  sprintf(txt,"open %s",(const char*)HostName);
	  timeout=SetTimeOut(1);
	  ServCommand(txt,true);
     }
     ServReply();
     if(!HasReplyError()) {
	  DoLogin(LoginName,Password);
	  CheckForConnection();
	  ServReply();
	  if(!HasReplyError()) {
	       LoggedIn=true;
	       FileMenu->setItemEnabled( CloseID, TRUE);
	       FileMenu->setItemEnabled( OpenID, FALSE);
	       menu->repaint();
	  }
     }
     SetTimeOut(timeout);
}

void MonitorClient::SL_GetHistoList(){
     ServCommand("DP gethistolist",true);
     ServReply();
}

void MonitorClient::SL_CloseConnection(){
     ServCommand("close",true);
     ServReply();
     if(!HasReplyError()) {
	  LoggedIn=false;
	  FileMenu->setItemEnabled( OpenID, TRUE);
	  FileMenu->setItemEnabled( CloseID, FALSE);
	  menu->repaint();
     }
}

void MonitorClient::SL_CloseConnectionUnsafe(){
    int ret = QMessageBox::information(this, DAQ_CAPTION,
				       "Close connection to server?",
				       "Yes", "No");
    if(ret==0){
	 SL_CloseConnection();
    }
}

void MonitorClient::SL_InitRun()
{
     RunScript(InitRunFile);
}

void MonitorClient::SL_StartRunAll()
{
     RunScript(StartRunAllFile);
}

void MonitorClient::SL_StopRunAll()
{
     RunScript(StopRunAllFile);
}

void MonitorClient::SL_StartDataProcess(){
     ServCommand("DC start dataprocess",true);
     ServReply();
}

void MonitorClient::SL_StartDataCollector(){
     ServCommand("DC vme start datacollector",true);
     ServReply();
}

void MonitorClient::SL_StopDataCollector(){
     ServCommand("DCO stop",true);
}

void MonitorClient::SL_QuitDataCollector(){
     ServCommand("DCO bye",true);
     ServReply();
}


void MonitorClient::SL_StartRun()
{
     ServCommand("DCO start",true);
}

void MonitorClient::SL_StopRun()
{
     ServCommand("DCO stop",true);
}

void MonitorClient::SL_InitDataProcess(){
     String cmd=String("DP initialize ") + RunParams;
     ServCommand(cmd,true);
}

void MonitorClient::SL_InitDataCollector(){
     ServCommand("DCO initialize",true);
}

void MonitorClient::SL_QuitDataProcess(){
     ServCommand("DP bye",true);
     ServReply();
}

void MonitorClient::SL_StartDataTransport()
{
     ServCommand("DC vme start datatransport",true);
} 

void MonitorClient::SL_RunDataTransport()
{
     ServCommand("DT start",true);
} 

void MonitorClient::SL_StopDataTransport()
{
     ServCommand("DT stop",true);
} 

void MonitorClient::SL_QuitDataTransport()
{
     ServCommand("DT bye",true);
}

void MonitorClient::SL_InitDataTransport()
{
     ServCommand("DT init",true);
}

void MonitorClient::SL_RunStartWidget()
{
     if(!LoggedIn) {
	  SL_OpenConnection();
	  if(!LoggedIn) {
	       PrintErrorText("You must login before running a script.");
	       return;
	  }
     }
     
     RunStartW->show();
}

void MonitorClient::SetGeometry(){
     // ------------- Button Bar, not always visible ----------------------
     int ButtonBarHeight, Div;
	  ButtonBarHeight=0;
	  Div=3;
     
     int Y1_Buttons=menu->height()+2;
     int Y2_Buttons=menu->height();
     int DX_Buttons=34;
     // ------------- Message View ----------------------------------------
     int Y1_Message=menu->height()+2;
     int Y2_Input=height()-2;
     int Y1_Input=Y2_Input-Input->fontMetrics().lineSpacing()-10;
     int Y2_Message=(Y1_Input-Y1_Message)/Div-2;
     int Y1_Output=Y2_Message+4;
     int Y2_Output=Y1_Input-4;
     int X1=2;
     int X2=width()-4;

     Messages->setGeometry(X1,Y1_Message,X2-X1,Y2_Message-Y1_Message);
     Output->setGeometry(X1,Y1_Output,X2-X1,Y2_Output-Y1_Output);
     Input->setGeometry(X1,Y1_Input,X2-X1,Y2_Input-Y1_Input);     
}

void MonitorClient::SL_startPAW(){
     if(!PAWConsole){
	  PAWConsole=new ConsoleWidget();
	  PAWConsole->setCaption ("PAW on Amanda");
	  PAWConsole->Process().RunProcess("paw");
	  PAWConsole->Process().SetQuitSequence("quit\n");
	  PAWConsole->stdinput() << "1\n" << flush;
	  PAWConsole->show();
	  show();
	  connect(PAWConsole,SIGNAL(widgetClosed()),
		  this,SLOT(SL_stopPAW())); // stopPAW -> SL_stopPAW ??
     }
}

void MonitorClient::SL_startROOT(){
     if(!ROOTConsole){
	  ROOTConsole=new ConsoleWidget();
	  ROOTConsole->setCaption ("ROOT on Amanda");
	  ROOTConsole->Process().RunProcess("root");
	  ROOTConsole->Process().SetQuitSequence("quit\n");
	  ROOTConsole->show();
	  show();
	  connect(ROOTConsole,SIGNAL(widgetClosed()),
		  this,SLOT(SL_stopROOT())); 
     }
}

void MonitorClient::SL_stopPAW(){
     if(PAWConsole){
	  disconnect(PAWConsole,SIGNAL(widgetClosed()),
		     this,SLOT(SL_stopPAW())); // stopPAW -> SL_stopPAW ??
	  delete PAWConsole;
	  PAWConsole=0;
     }
}

void MonitorClient::SL_stopROOT(){
     if(ROOTConsole){
	  disconnect(ROOTConsole,SIGNAL(widgetClosed()),
		     this,SLOT(SL_stopROOT())); // stopPAW -> SL_stopPAW ??
	  delete ROOTConsole;
	  ROOTConsole=0;
     }
}

void MonitorClient::SL_startTableView (TableViewParams &tvp)
{
	cout << "startTableView" << endl;
	if (!TableView)
	{
		TableView = new TableViewWidget (tvp);
		connect (TableView, SIGNAL (SI_TableViewCmdReady ()),
			this, SLOT (SL_applyFromTableView ()));
		connect (TableView, SIGNAL (SI_TableViewWasQuit ()),
				this, SLOT (SL_stopTableView ()));
	}
}

void MonitorClient::SL_stopTableView ()
{
	cout << "stopTableView" << endl;
	disconnect (TableView, SIGNAL (SI_TableViewCmdReady ()),
			this, SLOT (SL_applyFromTableView ()));
	if (TableView)
	{
		delete TableView;
		TableView = 0;
	}
}

void MonitorClient::SL_applyFromTableView ()
{
	ServCommand (TableView->getCommand (), true);
}

void MonitorClient::ServReplyFromServer(){
     ServReply();
}

void MonitorClient::ServCommand(String cmd,bool print){
     if(print){
	  PrintInfoText(String("CS> ")+cmd+String("\n"));
     }
     Client::ServCommand(cmd);
     CheckForConnection();
}

void MonitorClient::ServCommandFromUser(){
     ServCommand(Input->text(),true);
     Input->setText("");
}

void MonitorClient::CheckForConnection(){
     if((IsConnected)&&(!ServerSocketNotifier)){
	  ServerSocketNotifier=new QSocketNotifier(GetSocketDescriptor(),
						   QSocketNotifier::Read,this);
	  connect(ServerSocketNotifier,SIGNAL(activated(int)),
		  this,SLOT(ServReplyFromServer()));
     }
     if((ServerSocketNotifier)&&(!IsConnected)){
	  delete ServerSocketNotifier;
	  ServerSocketNotifier=0;
     }     
}     

int MonitorClient::Help(const String &topic)
{
     String Topic=HelpRoot+topic;
     return Mozilla.DisplayPage(Topic);
}

void MonitorClient::SetInitParams(const String &param)
{
     RunParams=param;
}


/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ClassWithTypeInfo_t *MonitorClient::GetConfigSection(){
//     return new DAQControlDaemonConfig(ServerPortMap,VMEControlPort,VMEControlHost);
     return new class MoniclientConfig:public ConfigSection_t
     {
     public:
	  /** ctor */
	  MoniclientConfig(String &defaultRunParams, 
			   String &ads_Dir,
			   String &startRunAllFile,
			   String &stopRunAllFile,String &initRunFile,
			   String &helpRoot, ON_OFF_t &mozillaColorMap)
	       :ConfigSection_t(),
		DefaultRunParams_cnf(defaultRunParams),
		ADS_Dir_cnf(ads_Dir),
		StartRunAllFile_cnf(startRunAllFile),
		StopRunAllFile_cnf(stopRunAllFile),
		InitRunFile_cnf(initRunFile),
		HelpRoot_cnf(helpRoot),
		MozillaColorMap_cnf(mozillaColorMap){}
	  BeginItems(ConfigSection_t){
	       BeginActions{
	       }EndActions;
	       BeginLinks{
		    ItemLink1("DefaultRunParams",DefaultRunParams_cnf);
		    ItemLink1("ADS_DIR",ADS_Dir_cnf);
		    ItemLink1("StartRunAllFile",StartRunAllFile_cnf);
		    ItemLink1("StopRunAllFile",StopRunAllFile_cnf);
		    ItemLink1("InitRunFile",InitRunFile_cnf);
		    ItemLink1("HelpRoot",HelpRoot_cnf);
		    ItemLink1("MozillaColorMap",MozillaColorMap_cnf);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(MONICLIENT_SECTION_NAME);
     private:
	  String &DefaultRunParams_cnf;
	  String &ADS_Dir_cnf;
	  String &StartRunAllFile_cnf;
	  String &StopRunAllFile_cnf;
	  String &InitRunFile_cnf;
	  String &HelpRoot_cnf;
	  ON_OFF_t &MozillaColorMap_cnf;
     }(DefaultRunParams,ADS_Dir, StartRunAllFile,StopRunAllFile,InitRunFile,HelpRoot,MozillaColorMap);
}

/** returns a String with the configuration file name */
const String MonitorClient::GetConfigFileName()const{
     return  MONICLIENT_CNF_FILENAME;   // from globalmacros.h
}

int MonitorClient::ReadConfigFile(){
     // create an instance of the config file reader class
     ConfigReader cfr;
     // add our config section, wich must be and was created on the
     // heap using new
     ClassWithTypeInfo_t *cs=GetConfigSection();
     if(cs!=0){
	  cfr.AddSection(cs);
     }
     String cnf_file=GetConfigFileName();
     String cnf_files[4];
     cnf_files[0]="./" + cnf_file;
     char *path=getenv("HOME");
     if(path){
	  cnf_files[1]=path + String("/.") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[1]="";
     } 
     path=getenv("DAQ_ROOT_DIR");
     if(path){
	  cnf_files[2]=path + String("/share/") + String(PACKAGE) + 
	       "/" + cnf_file;
     }else{
	  cnf_files[2]="";
     } 
     cnf_files[3]= String(DAQ_ROOT_DIR) + String("/share/") + 
	  String(PACKAGE) + "/" + cnf_file;
     ifstream file;
     for(int i=0;i<4;i++){
	  file.open(cnf_files[i]);
	  if(file) {
	       Logger Entry;
	       Entry() << "Using configuration file '" << 
		    cnf_files[i] << "'." << endl;
	       i=3;
	  }
     }
     TEST_FAILURE(cfr.Read(file));
     return 0;
}

int main(int argc,char **argv){
     DaqProcessBase::InitializeGlobals(argv[0]);
     LogFile LF(DaqProcessBase::ThisProcess->GetName(),LogFile::APPEND);					
     Logger::SetLogger(&LF);				
     QApplication a(argc,argv);
     a.setStyle(WindowsStyle);
     MonitorClient MonClient;
     if(MonClient.Init()<0){
	  Logger Entry;
	  Entry() << "Couldn't Initialize Moniclient\n";
	  Base::Error->Print(Entry());
	  Base::Error->Print();
	  return 1;
     }
     
     a.setMainWidget(&MonClient);
     MonClient.show();
     a.exec();
     if(MonClient.Destroy()<0){
	  Logger Entry;
	  Entry() << "Couldn't Initialize Moniclient\n";
	  Base::Error->Print(Entry());
	  Base::Error->Print();
	  return 1;
     }
     return 0;
}

