#ifndef __MONICLIENT_H__
#define __MONICLIENT_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/moniclient.h,v 1.10 1999/02/03 03:42:19 steffen Exp $
*/

#include <config.h>
#include "mapinclude.h"
#include "clientlib.h"
#if defined USE_PAWDISPLAY
#include "graph.h"
#endif
#include <qwidget.h>
#include <qdialog.h>
#include <qlined.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qbttngrp.h> 
#include <qmsgbox.h> 
#include <qtooltip.h> 
#include "buttonbox.h"
#include "conswidget.h"
#include "tvwidget.h"
#include "pointerutil.h"
#include "cnfreader.h"
#include "mozillahelp.h"
#include "runstartwidget.h"
#include "logindialog.h"

#define MONICLIENT_SECTION_NAME "MoniClient"
#define MONICLIENT_CNF_FILENAME "moniclient.cnf"
#define DAQ_ICON "pengi.xpm"
#define DAQ_CAPTION "AMANDA DAQ98"
#define SYSTEM_HELP_ROOT "/usr/doc/amandadaq/help/html/"
#define USER_HELP_ROOT "/daqzeuthen/doc/help/html/"

class MoniClient_SignalHandler:public SignalHandlerClass
{
     virtual int Handle_SIGCHLD(Process &P);
     virtual int Handle_SIGPIPE(Process &P);
};

// class StartUpDialog: public QDialog
// {
//      Q_OBJECT;
// public:
//      StartUpDialog(QWidget *parent=0,const char *name=0)
// 	  :QDialog(parent,name,TRUE){
// 	       setCaption(DAQ_CAPTION);
// 	       QLabel *movielabel = new QLabel( this, "label0" );
// 	       QMovie movie = QMovie("anim.gif");
// 	       //movie.connectStatus(this, SLOT(movieStatus(int)));
// 	       //movie.connectUpdate(this, SLOT(movieUpdate(const QRect&)));
// 	       movielabel->setFrameStyle( QFrame::Box | QFrame::Plain );
// 	       movielabel->setMovie( movie );
// 	       movielabel->setMargin( 0 );
// //	       movielabel->setFixedSize( 128+movielabel->frameWidth()*2,
// //					 64+movielabel->frameWidth()*2 );
// 	       QToolTip::add( movielabel, "movie" );
//      }
// };

enum Display_t
{
     PAW,
     ROOT,
     NATIVE
};

class MonitorClient:public QWidget,public Client
{
     Q_OBJECT;
public:
     MonitorClient(QWidget *parent=0,const char *name=0);
     ~MonitorClient();
     int Init();
     int Destroy();
     virtual int PrintInfoText(const char *txt);
     virtual int PrintErrorText(const char *txt);
     virtual int PrintWarningText(const char *txt);
     virtual int PrintRUSUREText (const char *txt);	/* *PN* */
     virtual int PrintMessageText(const char *txt);
     virtual int DoPAWPipe (const char *txt);	/* *PN* 211097 */
     virtual int DoROOTPipe (const char *txt); 
     virtual int DoTableView (const char *txt);	/* *PM* 041297 */
     virtual int DoHistogram(MemHistogram_t &Histogram);
     virtual int DoHistogramPAW(MemHistogram_t &Histogram);
     virtual int DoHistogramROOT(MemHistogram_t &Histogram);
     int InfoMsg(const char *);
     void RunScript(String &s);
     int Help(const String &topic);
     void SetInitParams(const String &param);
     String GetDefaultRunParams() {return DefaultRunParams;};
     	       
public slots:
     void quit();
     void SL_Help();
     void SL_RunScript();
     void SL_HelpAbout();
     void SL_HelpIndex();
     void SL_HelpTopics();
     void SL_startROOT();
     void SL_stopROOT();
     void SL_startPAW();
     void SL_stopPAW();
     void SL_startTableView (TableViewParams &tvp);	/* *PN* 041297 */
     void SL_stopTableView ();				/* *PN* 041297 */
     void SL_applyFromTableView ();			/* *PN* 121297 */
     void SL_OpenConnection();
     void SL_CloseConnection();
     void SL_CloseConnectionUnsafe();
     void SL_StartRunAll();
     void SL_StopRunAll();
     void SL_InitRun();
     void SL_StartDataProcess();
     void SL_StartDataCollector();
     void SL_StopDataCollector();
     void SL_InitDataProcess();
     void SL_InitDataCollector();
     void SL_QuitDataProcess(); 
     void SL_StartDataTransport();
     void SL_RunDataTransport();
     void SL_StopDataTransport();
     void SL_QuitDataTransport();
     void SL_QuitDataCollector();
     void SL_InitDataTransport();
     void SL_StartRun();
     void SL_StopRun();
     void SL_RunStartWidget();
     void SL_MsgLogging();
     void SL_GetHistoList();
     void ServReplyFromServer();
     void ServCommandFromUser();

protected:
     int ReadConfigFile();
     const String GetConfigFileName()const;
     ClassWithTypeInfo_t *GetConfigSection();

     /** Special signal hndler. */
     MoniClient_SignalHandler SignalHandler;
     friend MoniClient_SignalHandler;

protected:
     void resizeEvent(QResizeEvent *);
     QPopupMenu  *FileMenu;
     QPopupMenu *ToolMenu;
     QPopupMenu *HistogramMenu;
     
     String ADS_Dir;
     String StartRunAllFile;
     String StopRunAllFile;
     String InitRunFile;
     RunStartWidget *RunStartW;
     int Test;
     
signals:
     void explain(const char *);
private:
     void SetGeometry();
public:						/* *PN* 121297 */
     void ServCommand(String cmd,bool print);
private:
     void CheckForConnection();
#if defined USE_PAWDISPLAY
     typedef PointerWrapper_Ref<GraphView> GraphView_p;
     map<int,GraphView_p,less<int> > HistWindows;
#endif
     
     QSocketNotifier *ServerSocketNotifier;
     QLineEdit *Input;
     QTextView *Messages;
     QTextView *Output;     
     QMenuBar *menu;
     ConsoleWidget *PAWConsole;
     ConsoleWidget *ROOTConsole;
     TableViewWidget *TableView;		/* *PN* 041297 */
     bool MsgLogging;
     
     int GetHistoListID;
     int OpenID;
     int CloseID;
     int MsgLoggingID;
     

     String HelpRoot;
     MozillaHelp Mozilla;
     String RunParams;
     String DefaultRunParams;
     ofstream msgfile;
     ofstream datafile;
     ON_OFF_t MozillaColorMap;
     Display_t Display;
     
private:
     bool LoggedIn;
     
};

#endif //__MONICLIENT_H__
