/*
 * Netscape remote control
 *
 * Modified for AMANDA DAQ98 by Steffen Richter steffen@magnolia.spole.gov
 * 
 * here is the original disclaimer:
 *
 * remote.c --- remote control of Netscape Navigator for Unix.
 * version 1.1.3, for Netscape Navigator 1.1 and newer.
 *
 * Copyright  1996 Netscape Communications Corporation, all rights reserved.
 * Created: Jamie Zawinski <jwz@netscape.com>, 24-Dec-94.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#include "mozilla_remote.h"
#include "error.h"
#include "base.h"
#include <string.h>
#include <time.h>

/* vroot.h is a header file which lets a client get along with `virtual root'
   window managers like swm, tvtwm, olvwm, etc.  If you don't have this header
   file, you can find it at "http://home.netscape.com/newsref/std/vroot.h".
   If you don't care about supporting virtual root window managers, you can
   comment this line out.
 */
#include "my_vroot.h"

MozillaRemote::MozillaRemote()
{
     XA_MOZILLA_VERSION  = 0;
     XA_MOZILLA_LOCK     = 0;
     XA_MOZILLA_COMMAND  = 0;
     XA_MOZILLA_RESPONSE = 0;
     lock_data = 0;
}


int MozillaRemote::InitAtoms(Display *dpy)
{
     if (! XA_MOZILLA_VERSION)
	  XA_MOZILLA_VERSION = XInternAtom (dpy, MOZILLA_VERSION_PROP, False);
     if (! XA_MOZILLA_LOCK)
	  XA_MOZILLA_LOCK = XInternAtom (dpy, MOZILLA_LOCK_PROP, False);
     if (! XA_MOZILLA_COMMAND)
	  XA_MOZILLA_COMMAND = XInternAtom (dpy, MOZILLA_COMMAND_PROP, False);
     if (! XA_MOZILLA_RESPONSE)
	  XA_MOZILLA_RESPONSE = XInternAtom (dpy, MOZILLA_RESPONSE_PROP, False);
     return 0;
}

Window MozillaRemote::FindWindow(Display *dpy)
{
     int i;
     Window root = RootWindowOfScreen (DefaultScreenOfDisplay (dpy));
     Window root2, parent, *kids;
     unsigned int nkids;
     Window result = 0;
     Window tenative = 0;
     unsigned char *tenative_version = 0;

     if (! XQueryTree (dpy, root, &root2, &parent, &kids, &nkids))
     {
	  char errstr[512];
	  sprintf (errstr, "%s: XQueryTree failed on display %s\n", MOZILLA_NAME,
		   DisplayString (dpy));
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FindWindow(Display *dpy)";
	  Base::Error->SetParam(errstr);
	   
	  return 0;
     }

     /* root != root2 is possible with virtual root WMs. */

     if (! (kids && nkids))
     {
	  char errstr[512];
	  sprintf (errstr, "%s: root window has no children on display %s\n",
		   MOZILLA_NAME, DisplayString (dpy));
	  
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FindWindow(Display *dpy)";
	  Base::Error->SetParam(errstr);
	  
 	  return 0;
     }

     for (i = nkids-1; i >= 0; i--)
     {
 	  Atom type;
 	  int format;
 	  unsigned long nitems, bytesafter;
 	  unsigned char *version = 0;
 	  Window w = XmuClientWindow (dpy, kids[i]);
 	  int status = XGetWindowProperty (dpy, w, XA_MOZILLA_VERSION,
 					   0, (65536 / sizeof (long)),
 					   False, XA_STRING,
 					   &type, &format, &nitems, &bytesafter,
 					   &version);
 	  if (! version)
 	       continue;
	  if (strcmp ((char *) version, EXPECTED_MOZILLA_VERSION) && !tenative)
	  {
 	       tenative = w;
 	       tenative_version = version;
 	       continue;
	  }
	  
 	  XFree (version);
 	  if (status == Success && type != None)
 	  {
 	       result = w;
 	       break;
 	  }
     }
     

     if (result && tenative)
     {
	  return result;
     }
     else if (tenative)
     {
	  return tenative;
     }
     else if (result)
     {
	  return result;
     }
     else
     {
	  char errstr[512];
	  sprintf (errstr, "%s: not running on display %s\n", 
		   MOZILLA_NAME,
		   DisplayString (dpy));
	  
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FindWindow(Display *dpy)";
	  Base::Error->SetParam(errstr);

	  return 0;
     }
}

int MozillaRemote::CheckWindow (Display *dpy, Window window)
{
     Atom type;
     int format;
     unsigned long nitems, bytesafter;
     unsigned char *version = 0;
     int status = XGetWindowProperty (dpy, window, XA_MOZILLA_VERSION,
				      0, (65536 / sizeof (long)),
				      False, XA_STRING,
				      &type, &format, &nitems, &bytesafter,
				      &version);
     if (status != Success || !version)
     {
	  char errstr[512];
	  sprintf(errstr, "%s: window 0x%x is not a Netscape window.\n",
		  MOZILLA_NAME, (unsigned int) window);
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::CheckWindow(..)";
	  Base::Error->SetParam(errstr);

	  return -1;
     }
     XFree (version);
     return 0;
}



int MozillaRemote::ObtainLock(Display *dpy, Window window)
{
     Bool locked = False;
     Bool waited = False;

     if (! lock_data)
     {
	  lock_data = (char *) malloc (255);
	  sprintf (lock_data, "pid%i@", getpid ());
	  if (gethostname (lock_data + strlen (lock_data), 100))
	  {
	       perror("gethostname");
	       return -1;
	  }
     }

     do
     {
	  int result;
	  Atom actual_type;
	  int actual_format;
	  unsigned long nitems, bytes_after;
	  unsigned char *data = 0;

	  XGrabServer (dpy);   /* ################################# DANGER! */

	  result = XGetWindowProperty (dpy, window, XA_MOZILLA_LOCK,
				       0, (65536 / sizeof (long)),
				       False, /* don't delete */
				       XA_STRING,
				       &actual_type, &actual_format,
				       &nitems, &bytes_after,
				       &data);
	  if (result != Success || actual_type == None)
	  {
	       /* It's not now locked - lock it. */
#ifdef DEBUG_PROPS
	       fprintf (stderr, "%s: (writing " MOZILLA_LOCK_PROP
			" \"%s\" to 0x%x)\n",
			progname, lock_data, (unsigned int) window);
#endif
	       XChangeProperty (dpy, window, XA_MOZILLA_LOCK, XA_STRING, 8,
				PropModeReplace, (unsigned char *) lock_data,
				strlen (lock_data));
	       locked = True;
	  }

	  XUngrabServer (dpy); /* ################################# danger over */
	  XSync (dpy, False);
	  time_t t_start, t_end, dummy;

	  if (! locked)
	  {
	       /* We tried to grab the lock this time, and failed because someone
		  else is holding it already.  So, wait for a PropertyDelete event
		  to come in, and try again. */
	       
	       t_start=time(&dummy);
	       
	       waited = True;

	       while (1)
	       {
		    XEvent event;
		    XNextEvent (dpy, &event);
		    if (event.xany.type == DestroyNotify &&
			event.xdestroywindow.window == window)
		    {
			 char errstr[512];
			 sprintf (errstr, "%s: window 0x%x unexpectedly destroyed.\n",
				  MOZILLA_NAME, (unsigned int) window);
			 (*Base::Error)=GENERIC_ERROR;
			 (*Base::Error)="MozillaRemote::ObtainLock(..)";
			 Base::Error->SetParam(errstr);
			 
			 return -1;
		    }
		    else if (event.xany.type == PropertyNotify &&
			     event.xproperty.state == PropertyDelete &&
			     event.xproperty.window == window &&
			     event.xproperty.atom == XA_MOZILLA_LOCK)
		    {
			 /* Ok!  Someone deleted their lock, so now we can try
			    again. */
#ifdef DEBUG_PROPS
			 fprintf (stderr, "%s: (0x%x unlocked, trying again...)\n",
				  progname, (unsigned int) window);
#endif
			 break;
		    }
	       }
	  }
	  if (data)
	       XFree (data);
	  t_end=time(&dummy);
	  if( (t_end - t_start)>=5)
	       break;
     }
     while (! locked);

     return 0;
}


void MozillaRemote::FreeLock(Display *dpy, Window window)
{
     int result;
     Atom actual_type;
     int actual_format;
     unsigned long nitems, bytes_after;
     unsigned char *data = 0;

#ifdef DEBUG_PROPS
     fprintf (stderr, "%s: (deleting " MOZILLA_LOCK_PROP
	      " \"%s\" from 0x%x)\n",
	      progname, lock_data, (unsigned int) window);
#endif

     result = XGetWindowProperty (dpy, window, XA_MOZILLA_LOCK,
				  0, (65536 / sizeof (long)),
				  True, /* atomic delete after */
				  XA_STRING,
				  &actual_type, &actual_format,
				  &nitems, &bytes_after,
				  &data);
     if (result != Success)
     {
	  char errstr[512];
	  sprintf(errstr, "%s: unable to read and delete " MOZILLA_LOCK_PROP
		  " property\n",
		  MOZILLA_NAME);
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FreeLock (..)";
	  Base::Error->SetParam(errstr);
	  
	  return;
     }
     else if (!data || !*data)
     {
	  char errstr[512];
	  sprintf (errstr, "%s: invalid data on " MOZILLA_LOCK_PROP
		   " of window 0x%x.\n",
		   MOZILLA_NAME, (unsigned int) window);
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FreeLock(..)";
	  Base::Error->SetParam(errstr);
	  
	  return;
     }
     else if (strcmp ((char *) data, lock_data))
     {
	  char errstr[512];
	  sprintf (errstr, "%s: " MOZILLA_LOCK_PROP
		   " was stolen!  Expected \"%s\", saw \"%s\"!\n",
		   MOZILLA_NAME, lock_data, data);
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="MozillaRemote::FreeLock(..)";
	  Base::Error->SetParam(errstr);
	  
	  return;
     }

     if (data)
	  XFree (data);
}


int MozillaRemote::Command (Display *dpy, Window window, const char *command,
			    Bool raise_p)
{
     int result;
     Bool done = False;
     char *new_command = 0;
     
     /* The -noraise option is implemented by passing a "noraise" argument
	to each command to which it should apply.
     */
     if (! raise_p)
     {
	  char *close;
	  new_command = (char *) malloc (strlen (command) + 20);
	  strcpy (new_command, command);
	  close = strrchr (new_command, ')');
	  if (close)
	       strcpy (close, ", noraise)");
	  else
	       strcat (new_command, "(noraise)");
	  command = new_command;
     }

#ifdef DEBUG_PROPS
     fprintf (stderr, "%s: (writing " MOZILLA_COMMAND_PROP " \"%s\" to 0x%x)\n",
	      progname, command, (unsigned int) window);
#endif

     XChangeProperty (dpy, window, XA_MOZILLA_COMMAND, XA_STRING, 8,
		      PropModeReplace, (unsigned char *) command,
		      strlen (command));

     while (!done)
     {
	  XEvent event;
	  XNextEvent (dpy, &event);
	  if (event.xany.type == DestroyNotify &&
	      event.xdestroywindow.window == window)
	  {
	       /* Print to warn user...*/
	       fprintf (stderr, "%s: window 0x%x was destroyed.\n",
			MOZILLA_NAME, (unsigned int) window);
	       result = 6;
	       goto DONE;
	  }
	  else if (event.xany.type == PropertyNotify &&
		   event.xproperty.state == PropertyNewValue &&
		   event.xproperty.window == window &&
		   event.xproperty.atom == XA_MOZILLA_RESPONSE)
	  {
	       Atom actual_type;
	       int actual_format;
	       unsigned long nitems, bytes_after;
	       unsigned char *data = 0;

	       result = XGetWindowProperty (dpy, window, XA_MOZILLA_RESPONSE,
					    0, (65536 / sizeof (long)),
					    True, /* atomic delete after */
					    XA_STRING,
					    &actual_type, &actual_format,
					    &nitems, &bytes_after,
					    &data);
#ifdef DEBUG_PROPS
	       if (result == Success && data && *data)
	       {
		    fprintf (stderr, "%s: (server sent " MOZILLA_RESPONSE_PROP
			     " \"%s\" to 0x%x.)\n",
			     progname, data, (unsigned int) window);
	       }
#endif

	       if (result != Success)
	       {
		    fprintf (stderr, "%s: failed reading " MOZILLA_RESPONSE_PROP
			     " from window 0x%0x.\n",
			     MOZILLA_NAME, (unsigned int) window);
		    result = 6;
		    done = True;
	       }
	       else if (!data || strlen((char *) data) < 5)
	       {
		    fprintf (stderr, "%s: invalid data on " MOZILLA_RESPONSE_PROP
			     " property of window 0x%0x.\n",
			     MOZILLA_NAME, (unsigned int) window);
		    result = 6;
		    done = True;
	       }
	       else if (*data == '1')	/* positive preliminary reply */
	       {
		    fprintf (stderr, "%s: %s\n", MOZILLA_NAME, data + 4);
		    /* keep going */
		    done = False;
	       }
#if 1
	       else if (!strncmp ((char *)data, "200", 3)) /* positive completion */
	       {
		    result = 0;
		    done = True;
	       }
#endif
	       else if (*data == '2')		/* positive completion */
	       {
		    fprintf (stderr, "%s: %s\n", MOZILLA_NAME, data + 4);
		    result = 0;
		    done = True;
	       }
	       else if (*data == '3')	/* positive intermediate reply */
	       {
		    fprintf (stderr, "%s: internal error: "
			     "server wants more information?  (%s)\n",
			     MOZILLA_NAME, data);
		    result = 3;
		    done = True;
	       }
	       else if (*data == '4' ||	/* transient negative completion */
			*data == '5')	/* permanent negative completion */
	       {
		    fprintf (stderr, "%s: %s\n", MOZILLA_NAME, data + 4);
		    result = (*data - '0');
		    done = True;
	       }
	       else
	       {
		    fprintf (stderr,
			     "%s: unrecognised " MOZILLA_RESPONSE_PROP
			     " from window 0x%x: %s\n",
			     MOZILLA_NAME, (unsigned int) window, data);
		    result = 6;
		    done = True;
	       }

	       if (data)
		    XFree (data);
	  }
#ifdef DEBUG_PROPS
	  else if (event.xany.type == PropertyNotify &&
		   event.xproperty.window == window &&
		   event.xproperty.state == PropertyDelete &&
		   event.xproperty.atom == XA_MOZILLA_COMMAND)
	  {
	       fprintf (stderr, "%s: (server 0x%x has accepted "
			MOZILLA_COMMAND_PROP ".)\n",
			progname, (unsigned int) window);
	  }
#endif /* DEBUG_PROPS */
     }

 DONE:

     if (new_command)
	  free (new_command);

     return result;
}

int MozillaRemote::Commands(Display *dpy, Window window, char **commands)
{
     Bool raise_p = True;
     int status = 0;
     TEST_FAILURE(InitAtoms (dpy));

     if (window == 0)
	  window=FindWindow(dpy);
     else
	  CheckWindow(dpy, window);

     XSelectInput(dpy, window, (PropertyChangeMask|StructureNotifyMask));

     if(ObtainLock(dpy, window)<0) {
	  // error handling
		  return -1;
     }
     

     while (*commands)
     {
	  if (!strcmp (*commands, "-raise"))
	       raise_p = True;
	  else if (!strcmp (*commands, "-noraise"))
	       raise_p = False;
	  else
	       status = Command(dpy,window,*commands, raise_p);

	  if (status != 0)
	       break;
	  commands++;
     }

     /* When status = 6, it means the window has been destroyed */
     /* It is invalid to free the lock when window is destroyed. */

     if ( status != 6 )
	  FreeLock(dpy, window);

     return status;
}

int MozillaRemote::ExecCommand(int argc, char **argv)
{
     Display *dpy;
     char *dpy_string = 0;
     char **remote_commands = 0;
     int remote_command_count = 0;
     int remote_command_size = 0;
     unsigned long remote_window = 0;
     Bool sync_p = False;
     int i;

     for (i = 0; i < argc; i++)
     {
	  if (!strcmp (argv [i], "-d") ||
	      !strcmp (argv [i], "-dpy") ||
	      !strcmp (argv [i], "-disp") ||
	      !strcmp (argv [i], "-display"))
	  {
	       i++;
	       dpy_string = argv [i];
	  }
	  else if (!strcmp (argv [i], "-sync") ||
		   !strcmp (argv [i], "-synchronize"))
	  {
	       sync_p = True;
	  }
	  else if (!strcmp (argv [i], "-remote"))
	  {
	       if (remote_command_count == remote_command_size)
	       {
		    remote_command_size += 20;
		    remote_commands =(char **)
			 (remote_commands
			  ? realloc (remote_commands,
				     remote_command_size * sizeof (char *))
			  : calloc (remote_command_size, sizeof (char *)));
	       }
	       i++;
	       if (!argv[i] || *argv[i] == '-' || *argv[i] == 0)
	       {
		    char errstr[512];
		    sprintf (errstr, "%s: invalid `-remote' option \"%s\"\n",
			     MOZILLA_NAME, argv[i] ? argv[i] : "");
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="MozillaRemote::ExecCommand(..)";
		    Base::Error->SetParam(errstr);
		    			 
		    return -1;
	       }
	       remote_commands [remote_command_count++] = argv[i];
	  }
	  else if (!strcmp (argv [i], "-raise") ||
		   !strcmp (argv [i], "-noraise"))
	  {
	       char *r = argv [i];
	       if (remote_command_count == remote_command_size)
	       {
		    remote_command_size += 20;
		    remote_commands =(char **)
			 (remote_commands
			  ? realloc (remote_commands,
				     remote_command_size * sizeof (char *))
			  : calloc (remote_command_size, sizeof (char *)));
	       }
	       remote_commands [remote_command_count++] = r;
	  }
	  else if (!strcmp (argv [i], "-id"))
	  {
	       char c;
	       if (remote_command_count > 0)
	       {
		    char errstr[512];
		    sprintf (errstr,
			     "%s: the `-id' option must preceed all `-remote' options.\n",
			     MOZILLA_NAME);
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="MozillaRemote::ExecCommand(..)";
		    Base::Error->SetParam(errstr);
		    return -1;
	       }
	       else if (remote_window != 0)
	       {
		    char errstr[512];
		    sprintf (errstr, "%s: only one `-id' option may be used.\n",
			     MOZILLA_NAME);
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="MozillaRemote::ExecCommand(..)";
		    Base::Error->SetParam(errstr);
		    return -1;
	       }
	       i++;
	       if (argv[i] &&
		   1 == sscanf (argv[i], " %ld %c", &remote_window, &c))
		    ;
	       else if (argv[i] &&
			1 == sscanf (argv[i], " 0x%lx %c", &remote_window, &c))
		    ;
	       else
	       {
		    char errstr[512];
		    sprintf (errstr, "%s: invalid `-id' option \"%s\"\n",
			     MOZILLA_NAME, argv[i] ? argv[i] : "");
		    (*Base::Error)=GENERIC_ERROR;
		    (*Base::Error)="MozillaRemote::ExecCommand(..)";
		    Base::Error->SetParam(errstr);
		    return -1;
	       }
	  }
     }

     dpy = XOpenDisplay (dpy_string);
     if (! dpy)
	  return -1;

     if (sync_p)
	  XSynchronize (dpy, True);

     return Commands (dpy, (Window) remote_window,
		      remote_commands);
}



bool MozillaRemote::IsRunning()
{
     Display *dpy;
     char *dpy_string = 0;
     dpy = XOpenDisplay (dpy_string);
     if (! dpy)
	  return false;
     TEST_FAILURE(InitAtoms(dpy));
     int i;
     Window root = RootWindowOfScreen (DefaultScreenOfDisplay (dpy));
     Window root2, parent, *kids;
     unsigned int nkids;
     Window result = 0;
     Window tenative = 0;
     unsigned char *tenative_version = 0;

     if (! XQueryTree (dpy, root, &root2, &parent, &kids, &nkids))
     {
	  return false;
     }

     /* root != root2 is possible with virtual root WMs. */

     if (! (kids && nkids))
     {
	  return false;
     }

     for (i = nkids-1; i >= 0; i--)
     {
	  Atom type;
	  int format;
	  unsigned long nitems, bytesafter;
	  unsigned char *version = 0;
	  Window w = XmuClientWindow (dpy, kids[i]);
	  int status = XGetWindowProperty (dpy, w, XA_MOZILLA_VERSION,
					   0, (65536 / sizeof (long)),
					   False, XA_STRING,
					   &type, &format, &nitems, &bytesafter,
					   &version);
	  if (! version)
	       continue;
	  if (strcmp ((char *) version, EXPECTED_MOZILLA_VERSION) &&
	      !tenative)
	  {
	       tenative = w;
	       tenative_version = version;
	       continue;
	  }
	  XFree (version);
	  if (status == Success && type != None)
	  {
	       result = w;
	       break;
	  }
     }

     if (result && tenative)
     {
	  XFree (tenative_version);
	  return true;
     }
     else if (tenative)
     {
	  XFree (tenative_version);
	  return true;
     }
     else if (result)
     {
	  return true;
     }
     else
     {
	  return false;
     }
     return false;
}

