#if ! defined __MUONEVENTHISTOSET_H__
#define __MUONEVENTHISTOSET_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/muoneventhistoset.h,v 1.7 1999/02/08 08:08:54 steffen Exp $

  $Log: muoneventhistoset.h,v $
  Revision 1.7  1999/02/08 08:08:54  steffen
  - * code for scalerbase and s7201 added and/or debugged

  Revision 1.6  1999/02/04 08:47:31  steffen
  * corrected number of bins for 'Trigger per OM ( ADC )' histogram

  Revision 1.5  1998/12/13 18:06:38  wischnew
  vmedaqhal.h:    trigger status for asynch modules
  datacollector:  future change of readout for asynch (7201/3808) modules


  ?? CVS: Modified Files:
  ?? CVS: 	datacollector.C dmadmodule.C muoneventhistoset.h vmedaqhal.h
  ?? CVS: ----------------------------------------------------------------------

  Revision 1.4  1998/11/23 14:43:55  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.3  1998/10/22 16:02:51  daq
  added 2 GPS event time difference histograms (second an minute range)

  Revision 1.2  1998/10/06 10:45:14  daq
  changed RAT_TIMMAX to 1000

*/

#include "histoset.h"
#include <config.h>
//#include "hbookhistoset.h"

// /** Shared memory histogram */
// #define SHM_HISTO   0
// /** Normal memeory histogram */
// #define MEM_HISTO   1
// /** HBOOK histogram */
// #define HBOOK_HISTO 2

// Histogramming
#define     MISC_HISTOGRAM_OFFSET      100
#define     TOM_ADC_H_OFFSET 0
#define     TOM_TDC_H_OFFSET 100
#define     NOH_ADC_H_OFFSET 200
#define     NOH_TDC_H_OFFSET 300
#define     CAMAC_TRIGGER_H_OFFSET 400
#define     VME_TRIGGER_H_OFFSET 400
#define     GPS_1_H_OFFSET     600
#define     GPS_2_H_OFFSET     700
#define     GPS_3_H_OFFSET     800
#define     GPS_1_TIMMAX              10000000               // 10msec time scale in nsecs
#define     GPS_1_TIMBIN                   100 //2000               // width in nsecs
#define     GPS_2_TIMMAX              1000000             // 1 sec, time scale in usecs
#define     GPS_2_TIMBIN                   100 //2000               // width in usecs
#define     GPS_3_TIMMAX              10000000            // 10 sec, time scale in usecs
#define     GPS_3_TIMBIN                   100 //2000               // width in usecs

#define     NOM_HISTOGRAM_OFFSET    1000

//#define NUM_RATAS_HISTOGRAMS  NSTRINGS+1
#define NUM_RATAS_HISTOGRAMS             1
#define     RATAS_HISTOGRAM_OFFSET    2000
#define     RAT_TIMMAX                1000                  // time scale in secs
#define     RAT_TIMBIN                   1                  // width in secs

#define     ADC_HISTOGRAM_OFFSET   100000

#define     TDC_HISTOGRAM_OFFSET   200000
#define                LE_OFFSET    50000
#define     TOT_HISTOGRAM_OFFSET   300000
#define     NED_HISTOGRAM_OFFSET   350000

#warning You should change this if you change the Histogramms
struct fa_per_OM_histos_type{
     Histogram_t *adc;
     Histogram_t *tdc;
     Histogram_t *le;
     Histogram_t *tot;
     Histogram_t *ned;
};

struct fa_per_String_histos_type{
};

struct fa_misc_histos_type{
     Histogram_t *nom;
     Histogram_t *tom_adc;
     Histogram_t *tom_tdc;
     Histogram_t *noh_adc;
     Histogram_t *noh_tdc;
     Histogram_t *camac_trigger;
     Histogram_t *gps_1;
     Histogram_t *gps_2;
     Histogram_t *gps_3;
     Histogram_t *ratas;
};


template <class HistogramType>
class MuonEventHistoSet_t:public HistoSet_t<HistogramType>{
private:
     /** Struct that holds the {\it less than} function for the histogram ID map */
     struct ltid{
	  inline bool operator()(ID_t i1,ID_t i2) const{
	       return i1<i2;
	  }
     };
public:

     MuonEventHistoSet_t()
	  :Initialized(false),
	   preparedForFastAccess(false),
	   fa_per_OM_histos(0),
	   fa_per_String_histos(0),
	   fa_misc_histos(0){}

     ~MuonEventHistoSet_t(){
	  if(fa_per_OM_histos) delete fa_per_OM_histos;
	  if(fa_per_String_histos) delete fa_per_String_histos;
	  if(fa_misc_histos) delete fa_misc_histos;
     }
     
     void Init_per_Channel_Histograms(X_t Xmin,X_t Xmax,NumBin_t NumBin,
				      ChannelConfigRecordVector_t::iterator start,
				      ChannelConfigRecordVector_t::iterator end,
				      const char *Title,const char *Category);

     void Init_per_String_Histogram(X_t Xmin,X_t Xmax,NumBin_t NumBin,
				    StringConfigRecord_p &scr,
				    const char *Title,const char *Category);

     void Init_per_String_Histograms(X_t Xmin,X_t Xmax,NumBin_t NumBin,
				     StringConfigRecordVector_t::iterator start,
				     StringConfigRecordVector_t::iterator end,
				     const char *Title,const char *Category);

     void Init_misc_Histogram(X_t Xmin,X_t Xmax,NumBin_t NumBin,
			      const char *Title,const char *Category,ID_t ID);
     
     /** Intialize set.
	 Initializes set and allocates all necessary histograms.
	 @param Vector with channel configuration
	 @return 0 on sucess, -1 on failure.
     */
     int Init(ChannelConfigRecordVector_t &Channels);
     
     
     void PrepareForFastAccess(HistoSet_t<HistogramType> *HistoSet,
			       ChannelConfigRecordVector_t Channels);
     

     inline fa_per_OM_histos_type *fa_GetChannelHistos(ID_t id){
	  if(!preparedForFastAccess) return 0;
	  
	  return fa_per_OM_histos+id;
     }

     inline fa_per_String_histos_type *fa_GetStringHistos(ID_t id){
	  if(!preparedForFastAccess) return 0;
	  return fa_per_String_histos+id;
     }

     inline fa_misc_histos_type *fa_GetMiscHistos(){
	  if(!preparedForFastAccess) return 0;
	  return fa_misc_histos;
     }

     void PrepareForFastAccess(ChannelConfigRecordVector_t &Channels);
	       
 
protected:
     /** Initializes the adc-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_ADC(ChannelConfigRecordVector_t &Channels,
			histo_id_t id_offset);

     /** Initializes the tdc-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_TDC(ChannelConfigRecordVector_t &Channels,
			histo_id_t id_offset);

     /** Initializes the tot-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_TOT(ChannelConfigRecordVector_t &Channels,
			histo_id_t id_offset);

     /** Initializes the nb-of-edges-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_NED(ChannelConfigRecordVector_t &Channels,
			histo_id_t id_offset);

     /** Initializes the nom\_hitted-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_NOM(histo_id_t id_offset);

     /** Initializes the rate array/string\_hitted-histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_RATAS(int nid, histo_id_t id_offset);

     /** Initializes the misc. histograms 
	 @return 0 on success or -1 on failure. */
     int InitHistos_Misc(histo_id_t id_offset);

protected:
//      /** Pointer used histogram set */
//      HistoSet_t *HistoSet;

     /** Number of OM channels */
     ID_t NumberOfChannels;

     /** Maximum OM Channel ID , is filed from Init*/
     ID_t max_Channel_ID;
     
     /** Number of bins, sum of all histograms */
     unsigned long int NumBin;

     /** String to max number of channels map */
     map<ID_t, ID_t,ltid> *String2MaxIDMap;

     bool Initialized;
     bool preparedForFastAccess;
     fa_per_OM_histos_type 	*fa_per_OM_histos;
     fa_per_String_histos_type  *fa_per_String_histos;
     fa_misc_histos_type   	*fa_misc_histos;
};


template <class HistogramType>
void MuonEventHistoSet_t<HistogramType>::Init_per_Channel_Histograms(X_t Xmin,X_t Xmax,NumBin_t NumBin,
						      ChannelConfigRecordVector_t::iterator start,
						      ChannelConfigRecordVector_t::iterator end,
						      const char *Title,const char *Category){
     histo_id_t id_offset=GetCategory(Category);
     HistogramDescription_t histdesc;
     histdesc.Xmin=Xmin;
     histdesc.Xmax=Xmax;
     histdesc.NumBin=NumBin;
     while(start!=end){
	  histdesc.ID=id_offset+(*start)->ID;
	  sprintf(histdesc.Title,"%s %i",Title,(*start)->ID);
	  NumBin+=histdesc.NumBin;
	  GetHisto_by_real_ID(histdesc.ID)->Create(histdesc);
	  start++;
     }
}

template <class HistogramType>
void  MuonEventHistoSet_t<HistogramType>::Init_per_String_Histogram(X_t Xmin,X_t Xmax,NumBin_t NumBin,
						     StringConfigRecord_p &scr,
						     const char *Title,const char *Category){
     histo_id_t id_offset=GetCategory(Category);
     HistogramDescription_t histdesc;
     histdesc.Xmin=Xmin;
     histdesc.Xmax=Xmax;
     histdesc.NumBin=NumBin;
     histdesc.ID=id_offset+scr->StringNumber;
     sprintf(histdesc.Title,"%s %i",Title,(int)scr->StringNumber);
     NumBin+=histdesc.NumBin;
     GetHisto_by_real_ID(histdesc.ID)->Create(histdesc);
}

template <class HistogramType>
void  MuonEventHistoSet_t<HistogramType>::Init_per_String_Histograms(X_t Xmin,X_t Xmax,NumBin_t NumBin,
						      StringConfigRecordVector_t::iterator start,
						      StringConfigRecordVector_t::iterator end,
						      const char *Title,const char *Category){
     histo_id_t id_offset=GetCategory(Category);
     HistogramDescription_t histdesc;
     histdesc.Xmin=Xmin;
     histdesc.Xmax=Xmax;
     histdesc.NumBin=NumBin;
     while(start!=end){
	  histdesc.ID=id_offset+(*i)->StringNummber;
	  sprintf(histdesc.Title,"%s %i",Title,(int)(*i)->StringNummber);
	  NumBin+=histdesc.NumBin;
	  GetHisto_by_real_ID(histdesc.ID)->Create(histdesc);
	  start++;
     }
}

template <class HistogramType>
void  MuonEventHistoSet_t<HistogramType>::Init_misc_Histogram(X_t Xmin,X_t Xmax,NumBin_t NumBin,
					       const char *Title,const char *Category,ID_t ID){
     histo_id_t id_offset=GetCategory(Category);
     HistogramDescription_t histdesc;
     histdesc.Xmin=Xmin;
     histdesc.Xmax=Xmax;
     histdesc.NumBin=NumBin;
     histdesc.ID=id_offset+ID;
     sprintf(histdesc.Title,"%s",Title);
     NumBin+=histdesc.NumBin;
     GetHisto_by_real_ID(histdesc.ID)->Create(histdesc);
}
     
template <class HistogramType>
int  MuonEventHistoSet_t<HistogramType>::Init(ChannelConfigRecordVector_t &Channels){
     assert(!Initialized);
     NumBin=0;
     NumberOfChannels=Channels.size();
     ChannelConfigRecordVector_t::iterator i;
     max_Channel_ID=0;
     for(i=Channels.begin();i<Channels.end();i++){
	  max_Channel_ID=max(max_Channel_ID,(*i)->ID);
     }
     max_Channel_ID++;   // array must be of size maxID+1
     StringConfigRecordVector_t Strings;
     MakeStringConfig(Channels,Strings);
	  
//------------------------ ADC --------------------------------
     SetCategory("adc",(histo_id_t)ADC_HISTOGRAM_OFFSET);
     Init_per_Channel_Histograms(	0,	4100,	82,
					Channels.begin(),Channels.end(),
					"ADC","adc");
//------------------------ TDC --------------------------------
     SetCategory("tdc",(histo_id_t)TDC_HISTOGRAM_OFFSET);
     Init_per_Channel_Histograms(	0,	33000,	3300,
					Channels.begin(),Channels.end(),
					"All Edges","tdc");

//------------------------ LE1-LE2 --------------------------------
     SetCategory("le",(histo_id_t)(TDC_HISTOGRAM_OFFSET+LE_OFFSET));
     Init_per_Channel_Histograms(	0,	1000,	1000,
					Channels.begin(),Channels.end(),
					"LE1-LE2","le");
	  
//------------------------ TOT --------------------------------
     SetCategory("tot",(histo_id_t)TOT_HISTOGRAM_OFFSET);
     Init_per_Channel_Histograms(	0,	1000,	1000,
					Channels.begin(),Channels.end(),
					"TOT","tot");

//------------------------ NED --------------------------------
     SetCategory("ned",(histo_id_t)NED_HISTOGRAM_OFFSET);
     Init_per_Channel_Histograms(	0,	17,	17,
					Channels.begin(),Channels.end(),
					"Number of Edges","ned");
	  
//------------------------ NOM --------------------------------
     SetCategory("nom",(histo_id_t)NOM_HISTOGRAM_OFFSET);
     Init_misc_Histogram(0,NumberOfChannels+1,NumberOfChannels+1,
			 "NOM Array","nom",0);

     StringConfigRecordVector_t::iterator sci=Strings.begin();
     while(sci!=Strings.end()){
	  if((*sci)->StringNumber!=0)
	       Init_per_String_Histogram(0,(*sci)->MaxOMPosition+1,
					 (*sci)->MaxOMPosition+1,*sci,
					 "NOM String","nom");
	  sci++;
     }
	  
//------------------------ RATAS --------------------------------
     SetCategory("ratas",RATAS_HISTOGRAM_OFFSET);
     Init_misc_Histogram(0,	RAT_TIMMAX+1,	RAT_TIMMAX/RAT_TIMBIN,
			 "Rate vs time Array (secs)","ratas",0);
//      Init_per_String_Histogramms(0,	NOMS_STRING_MAX+1,	NOMS_STRING_MAX+1,
// 				"Rate vs time String (secs)", "ratas");
	  
//------------------------ MISC --------------------------------
     SetCategory("misc",MISC_HISTOGRAM_OFFSET);
     Init_misc_Histogram(1,NumberOfChannels+1,NumberOfChannels,
			 "Trigger per OM (ADC)","misc",
			 TOM_ADC_H_OFFSET);
	  
     Init_misc_Histogram(1,NumberOfChannels+1,NumberOfChannels,
			 "Trigger per OM (TDC)","misc",
			 TOM_TDC_H_OFFSET);
				     
     Init_misc_Histogram(1,NumberOfChannels+1,NumberOfChannels,
			 "Number of Hits in Array (ADC)","misc",
			 NOH_ADC_H_OFFSET);

     Init_misc_Histogram(1,NumberOfChannels+1,NumberOfChannels,
			 "Number of Hits in Array (TDC)","misc",
			 NOH_TDC_H_OFFSET);

     Init_misc_Histogram(0,	33,	33,
			 "Trigger Register - CAMAC","misc",
			 CAMAC_TRIGGER_H_OFFSET);

     Init_misc_Histogram(0,	GPS_1_TIMMAX,	GPS_1_TIMBIN,
			 "Time difference between events in nsec (GPS Time)","misc",
			 GPS_1_H_OFFSET);
	  
     Init_misc_Histogram(0,	GPS_2_TIMMAX,	GPS_2_TIMBIN,
			 "Time difference between events in usec (GPS Time)","misc",
			 GPS_2_H_OFFSET);

     Init_misc_Histogram(0,	GPS_3_TIMMAX,	GPS_3_TIMBIN,
			 "Time difference between events in usec (GPS Time)","misc",
			 GPS_3_H_OFFSET);
     Initialized=true;
     return 0;
}

#warning You should change this if you change the Histogramms
template <class HistogramType>
void MuonEventHistoSet_t<HistogramType>::PrepareForFastAccess(ChannelConfigRecordVector_t &Channels){
     // we need huge tables for fast acces, so this function may fail
     // we therefore limit the id of an histograms to be less than 100000
     if(preparedForFastAccess) return;
     fa_per_OM_histos=new fa_per_OM_histos_type[max_Channel_ID];
     for(int i=0;i<max_Channel_ID;i++){
	  fa_per_OM_histos[i].adc=0;
	  fa_per_OM_histos[i].tdc=0;
	  fa_per_OM_histos[i].le =0;
	  fa_per_OM_histos[i].tot=0;
	  fa_per_OM_histos[i].ned=0;
     }
     
     assert(fa_per_OM_histos);
     // now fill the table
     ChannelConfigRecordVector_t::iterator i;
     for(i=Channels.begin();i<Channels.end();i++){
	  fa_per_OM_histos[(*i)->ID].adc=GetHisto("adc",(*i)->ID);
	  fa_per_OM_histos[(*i)->ID].tdc=GetHisto("tdc",(*i)->ID);
	  fa_per_OM_histos[(*i)->ID].le =GetHisto( "le",(*i)->ID);
	  fa_per_OM_histos[(*i)->ID].tot=GetHisto("tot",(*i)->ID);
	  fa_per_OM_histos[(*i)->ID].ned=GetHisto("ned",(*i)->ID);
     }
     fa_misc_histos=new fa_misc_histos_type;
     fa_misc_histos->nom=GetHisto("nom",0);
     fa_misc_histos->tom_adc=GetHisto("misc",TOM_ADC_H_OFFSET);
     fa_misc_histos->tom_tdc=GetHisto("misc",TOM_TDC_H_OFFSET);
     fa_misc_histos->noh_adc=GetHisto("misc",NOH_ADC_H_OFFSET);
     fa_misc_histos->noh_tdc=GetHisto("misc",NOH_TDC_H_OFFSET);
     fa_misc_histos->camac_trigger=GetHisto("misc",CAMAC_TRIGGER_H_OFFSET);
     fa_misc_histos->gps_1=GetHisto("misc",GPS_1_H_OFFSET);
     fa_misc_histos->gps_2=GetHisto("misc",GPS_2_H_OFFSET);
     fa_misc_histos->gps_3=GetHisto("misc",GPS_3_H_OFFSET);
     fa_misc_histos->ratas=GetHisto("ratas",0);  
     preparedForFastAccess=true;
}

#endif //__MUONEVENTHISTOSET_H__

