/*
$Header: /home/elrond/baikal/src/CVS/daq98/mytree.h,v 1.4 1999/02/03 03:42:19 steffen Exp $
*/

#if !defined __TREE_H__
#define __TREE_H__

#include <config.h>
#include "mapinclude.h"
#include "vectorinclude.h"
#include <String.h>
#include "stringutil.h"

template <class ItemType>
class TreeBase:public Base
{
private:
     /** wrapper class for a subtree of a tree */
     class SubTree
     {
     public:
 	  /** ctor, create the subtree */
 	  inline SubTree(){subtree=new TreeBase<ItemType>;}
 	  /** copy ctor */
 	  inline SubTree(const SubTree &t){subtree=new TreeBase<ItemType>(*(t.subtree));}
 	  /** asignemnt operator */
 	  inline SubTree &operator=(SubTree &t){
 	       delete subtree;
 	       subtree=new TreeBase<ItemType>(*(t.subtree));
 	       return t;
 	  }    
 	  /* dtor */
 	  inline ~SubTree(){delete subtree;}
 	  /** returns a pointer to the actual subtree */
 	  inline TreeBase<ItemType> *GetTree(){return subtree;}    
     private:
 	  /** pointer to the actual subtree */
 	  TreeBase<ItemType> *subtree;
     };
public: 
     /** container type, for the leaves */
     typedef map<String,ItemType,StringCompare> Leave_type;
     /** container iterator type, for the leaves */ 
     typedef Leave_type::iterator LeaveIterator_type;
     /** container type, for the branches (subtrees) */
     typedef map<String,SubTree,StringCompare> Branch_type;
     /** container iterator type, for the branches (subtrees) */
     typedef Branch_type::iterator BranchIterator_type;
public:
     /** ctor */
     inline TreeBase(){}
     /** copy ctor */
     inline TreeBase(const TreeBase<ItemType> &t):
 	  PathProp(t.PathProp),
 	  branch(t.branch),
 	  leave(t.leave){}
     /** assignment operator */
     inline const TreeBase &operator=(const TreeBase<ItemType> &t){
	  PathProp=t.PathProp;
 	  branch=t.branch;
 	  leave=t.leave;
 	  return t;
     }
     /** insert an item with the path path in the tree, the path
	 consists of the actual path and the name of the item separated
	 by the separator */
     inline void insertItem(const String &path,const ItemType &Data,const char separator='/'){
 	  StringSplitter sp(path,separator);
 	  insert(sp(),sp.NumSplits(),Data);
     }
     /** inserts a subtree at path */
     inline void insertTree(const String &path,const TreeBase<ItemType> &tree,const char separator='/'){
 	  StringSplitter sp(path,separator);
 	  insert(sp(),sp.NumSplits(),tree);
     }
     /** returns a pointer to the item with the path path */
     inline const ItemType &find(const String &path,char separator='/'){
 	  StringSplitter sp(path,separator);
 	  return find(sp(),sp.NumSplits());
     }
     /** returns a pointer to the subtree with the path path */
     inline TreeBase<ItemType> *GetSubTree(const String path,char separator='/'){
 	  StringSplitter sp(path,separator);
 	  return findSubTree(sp(),sp.NumSplits());
     }
     /** returns a reference to the container of the branches at this
	 hierarchy level of the tree */
     inline Branch_type &GetBranch(){return branch;}
     /** returns a reference to the container of the leaves (items) at
	 this hierarchy level of the tree */
     inline Leave_type &GetLeave(){return leave;}
     /** returns a vector of all items in the current hierarchie level */
     inline vector<ItemType> GetItemVector(){
	  vector<ItemType> ItemVector;
	  LeaveIterator_type li=leave.begin();
	  while(li!=leave.end()){
	       ItemVector.push_back((*li).second);
	       li++;
	  }
	  return ItemVector;
     }       
     /** pushs all item in the current hierarchie level in the  vector */
     inline void PushBackItems(vector<ItemType> &ItemVector){
	  LeaveIterator_type li=leave.begin();
	  while(li!=leave.end()){
	       ItemVector.push_back((*li).second);
	       li++;
	  }
     }       
     inline ItemType GetPathItem() const{return PathProp;}
     inline void SetPathItem(ItemType Data){
 	  PathProp=Data;
     }
protected:
     /** insert an item in the tree , only for internal use */
     void insert(const String *path,int pathlength,const ItemType &IBP){
 	  if(1==pathlength) leave[path[0]]=IBP;
	  else branch[path[0]].GetTree()->insert(path+1,pathlength-1,IBP);
     }
     /** insert a subtree  in the tree , only for internal use */
     void insert(const String *path,int pathlength,const TreeBase<ItemType> &tree){
 	  if(1==pathlength) (*(branch[path[0]].GetTree()))=tree;
	  else branch[path[0]].GetTree()->insert(path+1,pathlength-1,tree);
     }
     /** search an  item in the tree , only for internal use */
     const ItemType &find(const String *path,const int pathlength){
 	  TreeBase<ItemType> *SubTree=findSubTree(path,pathlength);
 	  if(SubTree){
 	       LeaveIterator_type val=SubTree->leave.find(path[pathlength-1]);
 	       if(SubTree->leave.end()!=val) return (*val).second;
	  }
 	  return *(ItemType*)0; // return a null reference
	                        // very dangerous, but don't have a
				// better idea
     }
     /** search a subtree */
     TreeBase<ItemType> *findSubTree(const String *path,const int pathlength){
 	  if(0==pathlength) return this;
 	  BranchIterator_type branch_it=branch.find(path[0]);
 	  if(branch_it==branch.end()) return 0;
 	  return (*branch_it).second.GetTree()->findSubTree(path+1,pathlength-1);
     }
private:
     /** item that describes the properties of tree path itself */
     ItemType PathProp;
     /** container for the branches */
     Branch_type branch;
     /** container for the leaves (items) */
     Leave_type leave;
};

// template<class ItemType>
// void Tree2Vector(TreeBase<ItemType> *T,vector<ItemType> &itemvector){
//      if(T->GetBranch().size()>0){
// 	  TreeBase<ItemType>::BranchIterator_type bi=T->GetBranch().begin();
// 	  while(bi!=T->GetBranch().end()){
// 	       TreeBase<itemType> *subtree=(*bi).second.GetTree();
// 	       itemvector.append(Tree2Vector(subtree,itemvector));
// 	       bi++;
// 	  }
//      }
//      if(T->GetLeave().size>0){
// 	  TreeBase<ItemType>::LeaveIterator_type li=T->GetLeave().begin();
// 	  while(li!=T->GetLeave().end()){
// 	       ItemType *item=(*li).second;
// 	       itemvector.push_back(*item);
// 	  }
// 	  li++;
// 	  return 0;
//      }
// }
#endif // __TREE_H__
