/*
$Header: /home/elrond/baikal/src/CVS/daq98/objectfactory.h,v 1.4 1998/11/23 14:44:04 haase Exp $
*/

#if !defined __OBJECTFACTORY_H__
#define __OBJECTFACTORY_H__
#include <config.h>
//#include "mapinclude.h"
#include <String.h>
#include "stringutil.h"
#include "pointerutil.h"
#include "simplevector.h"

template<class ClassID_t,class BaseObject>
class ObjectFactory_t
{
public:
     typedef PointerWrapper_Ref<BaseObject> ObjectType;
     struct CreatorBaseBase
     {
	  virtual ObjectType operator()()=0;
     };
     typedef PointerWrapper_Ref<CreatorBaseBase> Creator_p;
     ObjectFactory_t(){
	  IDObjectMap=new IDObjectMap_t;
     }
     virtual ~ObjectFactory_t(){
	  // we must delete the object elements
	  IDObjectMap_t::iterator i;
	  for(i=IDObjectMap->begin();i<IDObjectMap->end();i++)
	       delete (*i);
	  delete IDObjectMap;
     }
     void AddClass(const ClassID_t &ID,const Creator_p ocp){
	  if((void*)find(ID)()==0){
	       MapElement_p m=new MapElement_t;
	       m->_ID=ID;
	       m->_Creator=ocp;
	       IDObjectMap->push_back(m);
	  }
     }       
     ObjectType CreateObject(const ClassID_t &ID)const{
	  return find(ID);
     }       
protected:
     virtual ObjectType find(const ClassID_t &ID)const{
	  IDObjectMap_t::iterator i;
	  for(i=IDObjectMap->begin();i<IDObjectMap->end();i++)
	       if(((*i)->_ID)==ID) break;
	  if(i!=IDObjectMap->end()){
	       return (*((*i)->_Creator))();
	  }
	  return ObjectType(0);
     }       
     struct MapElement_t
     {
	  ClassID_t _ID;
	  Creator_p _Creator;
     };
     typedef MapElement_t *MapElement_p;
     MAKE_INC_VECTOR(MapElement_p,IDObjectMap_t);
     IDObjectMap_t *IDObjectMap;
};


template<class BaseObject>
class ObjectFactory:public ObjectFactory_t<String,BaseObject>
{
     struct CreatorBase:public ObjectFactory_t<String,BaseObject>::CreatorBaseBase
     {
	  virtual ObjectType operator()()=0;
     };
private:
     virtual ObjectType find(const String &ID)const{
	  IDObjectMap_t::iterator i;
	  for(i=IDObjectMap->begin();i<IDObjectMap->end();i++)
	       if(fcompare((*i)->_ID,ID)==0) break;
	  if(i!=IDObjectMap->end()){
	       return (*((*i)->_Creator))();
	  }
	  return ObjectType(0);
     }       
};

template<class Factory,class Object>
struct Creator:public Factory::CreatorBase
{
     virtual Factory::ObjectType operator()(){
 	  return new Object;
     }
};

#define RegisterClass(Factory_,Class_t_,TypeID_) \
     Factory_.AddClass(TypeID_,new Creator<typeof(Factory_),Class_t_>);


#if 0
#include "vectorinclude.h"
#include "pointerutil.h"
#include "objectfactory.h"

//  Example

// Base class must implement a general interface for all classes that
// have to be generated with an object factory
class Base{
public:
     Base(){
	  cout << "ctor Base" << endl;
     }
     virtual ~Base(){
	  cout << "dtor Base" << endl;
     }
     virtual void Func1()=0;
     virtual int Func2(double param)=0;
     virtual void Func3()=0;
};

class Derived1:public Base{
public:
     Derived1():Base(){
	  cout << "ctor Derived1" << endl;
     } 
     virtual ~Derived1(){
	  cout << "dtor Dervied1" << endl;
     }
     virtual void Func1(){
	  cout << "Derived1::Func1" << endl;
     }
     virtual int Func2(double param){
	  cout << "Derived1::Func2(" << param << ")"  << endl;
	  return 0;
     }
     virtual void Func3(){
	  cout << "Derived1::Func3" << endl;
     }
};

class Derived2:public Base{
public:
     Derived2():Base(){
	  cout << "ctor Derived2" << endl;
     }
     virtual ~Derived2(){
	  cout << "dtor Dervied2" << endl;
     }
     virtual void Func1(){
	  cout << "Derived2::Func1" << endl;
     }
     virtual int Func2(double param){
	  cout << "Derived2::Func2(" << param << ")"  << endl;
	  return 0;
     }
     virtual void Func3(){
	  cout << "Derived2::Func3" << endl;
     }
};

class Derived3:public Base{
public:
     Derived3():Base(){
	  cout << "ctor Derived3" << endl;
     }
     virtual ~Derived3(){
	  cout << "dtor Dervied3" << endl;
     }
     virtual void Func1(){
	  cout << "Derived3::Func1" << endl;
     }
     virtual int Func2(double param){
	  cout << "Derived3::Func2(" << param << ")"  << endl;
	  return 0;
     }
     virtual void Func3(){
	  cout << "Derived3::Func3" << endl;
     }
};


typedef ObjectFactory<Base> MyFactory_type;
typedef PointerWrapper_Ref<Base> Base_p;

int main()
{
     MyFactory_type MyFactory;
     
     MyFactory.AddClass("DERIVED_1",new Creator<MyFactory_type,Derived1>); //<MyFactory_type,Derived1>);
     MyFactory.AddClass("DERIVED_2",new Creator<MyFactory_type,Derived2>);
     MyFactory.AddClass("DERIVED_3",new Creator<MyFactory_type,Derived3>);

     Base_p Pointer2Derived1=MyFactory.CreateObject("DERIVED_1");
     Base_p Pointer2Derived2=MyFactory.CreateObject("DERIVED_2");
     Base_p Pointer2Derived3=MyFactory.CreateObject("DERIVED_3");
     
     Pointer2Derived1->Func1();
     int ret=Pointer2Derived1->Func2(3.1415);
     Pointer2Derived1->Func3();
     
     Pointer2Derived2->Func1();
     ret=Pointer2Derived2->Func2(3.1415);
     Pointer2Derived2->Func3();

     Pointer2Derived3->Func1();
     ret=Pointer2Derived3->Func2(3.1415);
     Pointer2Derived3->Func3();

     vector<Base_p> Derived_vector;

     Derived_vector.push_back(Pointer2Derived1);
     Derived_vector.push_back(Pointer2Derived2);
     Derived_vector.push_back(Pointer2Derived3);

     for(unsigned int i=0;i<Derived_vector.size();i++){
	  Derived_vector[i]->Func1();
     }
}

/***

    The Big Quiestion 
            What is the Output ??????????????

*/
#endif //0

#endif //__OBJECTFACTORY_H__
     
