/* 
   $Header: /home/elrond/baikal/src/CVS/daq98/pointerutil.h,v 1.4 1999/02/03 03:42:20 steffen Exp $
*/

#if ! defined __POINTERUTIL_H__
#define __POINTERUTIL_H__

#include <config.h>
#include "daqprocessbase.h"

typedef int TypeInfo_t;
typedef TypeInfo_t* TypeInfo_p;

#define ClassTypeInfoHelper(Class) TypeInfo_t Class##::TypeInfo=0

#define ClassTypeInfoBase					\
private:							\
static TypeInfo_t TypeInfo;					\
public:								\
inline static TypeInfo_p GetTypeInfo(){return &TypeInfo;}	\
inline virtual bool IsOfType(TypeInfo_p TypeInf)		\
{return TypeInf==&TypeInfo;}				

#define ClassTypeInfo(BaseClass)						\
private:									\
static TypeInfo_t TypeInfo;							\
public:										\
inline static TypeInfo_p GetTypeInfo(){return &TypeInfo;}			\
inline virtual bool IsOfType(TypeInfo_p TypeInf)				\
{return BaseClass##::IsOfType(TypeInf)?true:TypeInf==&TypeInfo;}     	

#define DOWNCAST(TYPE,Pointer) ((TYPE *)Pointer)

#define TYPESAFE_DOWNCAST(TYPE,Pointer) \
(Pointer?((Pointer->IsOfType(TYPE##::GetTypeInfo()))?(TYPE *)Pointer:((TYPE*)0)):((TYPE*)0))


/* classes for "smart" pointers, that take care of the objects they
   are pointing to, 

   PointerWrapper_Ref does it by counting the number of pointer that
   point to the object, if all pointers  go out of scope the object is
   deleted from the heap

   PointerWrapper_Clone does it by always creating a copy
   of the object for every pointer and to destroy the object when the
   pointer gets out of scope 

   both types are useful depending on the purpose */

/* Alway create the object the pointer wrappers point to on the heap
   with the new operator ! */

template <class T>
class PointerWrapper{
public:
     /** ctor */
     inline PointerWrapper():ptr(0){}
     /** copy ctor, do not copy the pointer, but the actual object the
         pointer points to */
     inline PointerWrapper(const PointerWrapper<T> &){}    
     /** ctor */
     inline PointerWrapper(T *p):ptr(p){} 
     /** dtor */
     ~PointerWrapper(){}
     /** assignment operator, do not assign the pointer, but the
         actual object the pointer points to */
     inline const PointerWrapper &operator=(const PointerWrapper &PW){return PW;}
     /** returns a reference to the object */
     inline T&operator*() const{return *ptr;}
     /** returns a pointer to the object */
     inline T*operator()() const{return ptr;}
     /** returns a pointer to the object */
     inline T*operator->() const{return ptr;}
     /** returns a pointer to the object */
     inline operator T*() const{return ptr;}
     /** index operator for arrays */
     inline T& operator[](int index){return *(ptr+index);}
     /** the ! operator, return true if the pointer is != 0 */
     inline bool operator!()const{return !ptr;}
     /** the != operator, return true if the pointer is != p */
     inline bool operator!=(int p)const{return ptr!=(void*)p;}
     /** the == operator, return true if the pointer is == p */
     inline bool operator==(int p)const{return ptr==(void*)p;}
     /** the != operator, return true if the pointer is != p */
     inline bool operator!=(void *p)const{return ptr!=p;}
     /** the == operator, return true if the pointer is == p */
     inline bool operator==(void *p)const{return ptr==p;}
     /** the != operator, return true if the pointer is != p */
     inline bool operator!=(PointerWrapper &p)const{return ptr!=p.ptr;}
     /** the == operator, return true if the pointer is == p */
     inline bool operator==(PointerWrapper &p)const{return ptr==p.ptr;}
protected:
     /** pointer to the object */
     T *ptr;
};

/** reference counting Wrapper */
template <class T>
class PointerWrapper_Ref:public PointerWrapper<T>{
public:
     /** ctor */
     inline PointerWrapper_Ref():PointerWrapper<T>(),refcount(0){}
     /** ctor, with pointer argument , if del=false, the pointer gets
         not delete, if true it will be destroyed when there are no more
         references*/
     inline PointerWrapper_Ref(T *p,bool del=true):PointerWrapper<T>(p){
	  /** reference counter */
	  if(del) refcount=new int(1); else refcount=0;
     }    
     /** copy ctor, copy the pointer, and increase the ref counter */
     inline PointerWrapper_Ref(const PointerWrapper_Ref &PW):PointerWrapper<T>(){
	  ptr=PW.ptr;
	  refcount=PW.refcount;
	  if(refcount) (*refcount)++;
     }    
     /** dtor */
     inline ~PointerWrapper_Ref(){Destroy();}
     inline void Destroy(){
	  if(refcount){
	       (*refcount)--;
	       if(!(*refcount)){
		    delete ptr;
		    delete refcount;
	       }
	  }
     }
     /** assignment operator, assign the pointer, but the
         actual object the pointer points to */
     inline const PointerWrapper_Ref<T>  &operator=(const PointerWrapper_Ref<T> & PW){
	  Destroy();
	  ptr=PW.ptr;
	  refcount=PW.refcount;
	  if(refcount)(*refcount)++;
	  return PW;
     }
     inline bool operator==(const PointerWrapper_Ref<T> &PW)const {return ptr==PW.ptr;} // *SR* 21.11.97
     inline bool operator==(const PointerWrapper_Ref<T> &PW){return ptr==PW.ptr;} 
     inline bool operator==(const void * p) const {return ptr==p;} // *SR* 21.11.97
     inline bool operator==(const void * p){return ptr==p;} 
     inline bool operator<(const PointerWrapper_Ref<T> &PW) {return ptr<PW.ptr;} 
     inline bool operator<(const PointerWrapper_Ref<T> &PW) const {return ptr<PW.ptr;} // *SR* 21.11.97
private:
     /** pointer to the reference counter */
     int *refcount;
};

#define DOWNCAST_RCP(TYPE,Pointer) (*((PointerWrapper_Ref<TYPE>*)(&Pointer)))
#define UPCAST_RCP(TYPE,Pointer) (PointerWrapper_Ref<TYPE>(*((PointerWrapper_Ref<TYPE>*)(&Pointer))))

#define TYPESAFE_DOWNCAST_RCP(TYPE,Pointer)	\
(Pointer?					\
 ((Pointer->IsOfType(TYPE##::GetTypeInfo()))?	\
  DOWNCAST_RCP(TYPE,Pointer)			\
  :PointerWrapper_Ref<TYPE>(0))			\
 :PointerWrapper_Ref<TYPE>(0))

#define MAKE_REFCOUNT_POINTER(__type__,__PointerType__)\
typedef PointerWrapper_Ref<__type__> __PointerType__

/** wrapper that clones the objects it is pointing to */
template <class T>
class PointerWrapper_Clone:public PointerWrapper<T>
{
public:
     /** ctor */
     inline PointerWrapper_Clone():PointerWrapper<T>(){}
     /** ctor, with pointer argument , if del=false, the pointer gets
         not deleted, if true it is destroyed */
     inline PointerWrapper_Clone(T *p,bool del=true):PointerWrapper<T>(){
	  if(p){
	       ptr=p->clone();
	       if(del) delete p;
	  }
     }    
     /** copy ctor, do not copy the pointer, but the actual object the
         pointer points to */
     inline PointerWrapper_Clone(const PointerWrapper_Clone &PW){
	  if(PW.ptr) ptr=PW.ptr->clone();
	  else ptr=0;
     }    
     /** dtor */
     inline ~PointerWrapper_Clone(){
	  if(ptr) delete ptr;
     }
     /** assignment operator, do not assign the pointer, but the
         actual object the pointer points to */
     inline const PointerWrapper_Clone<T> &operator=(const PointerWrapper_Clone<T> &PW){
	  if(PW.ptr) {
	       if(ptr) delete ptr;
	       ptr=PW.ptr->clone();
	  }
	  return PW;
     }
};     
     
/** class to represent a pointer to a structure, that has no specific
  size, but it must have a member function, that returns the size */
template<class T>
class SmartPointer:public PointerWrapper<T>
{
public:
     inline SmartPointer():PointerWrapper<T>(){}
     inline SmartPointer(T *p):PointerWrapper<T>(){ptr=p;}
     inline SmartPointer(SmartPointer &sp){ptr=sp.ptr;}
     inline SmartPointer &operator++(int){
	  ptr=(T*)(((void*)ptr)+ptr->Sizeof());
	  return *this;
     }
     inline SmartPointer &operator+=(int index){
	  for(int i=0;i<index;i++) (*this)++;
	  return *this;
     }       
     inline SmartPointer operator+(int sw){
	  SmartPointer SP=*this;
	  for(int i=0;i<sw;i++) SP++;
	  return SP;
     }
     inline SmartPointer &operator=(SmartPointer &SP){
	  ptr=SP.ptr;
	  return SP;
     }
     inline T* operator=(T* p){return ptr=p;}       
     inline T& operator[](int index){return *((*this)+index).ptr;}
};

#define MAKE_SMART_POINTER(__type__,__PointerType__)\
typedef SmartPointer<__type__> __PointerType__

class ClassWithTypeInfo_t{
     ClassTypeInfoBase;
};

MAKE_REFCOUNT_POINTER(ClassWithTypeInfo_t,ClassWithTypeInfo_p);

#endif // __POINTERUTIL_H__
