/*
$Header: /home/elrond/baikal/src/CVS/daq98/privilege.h,v 1.3 1998/11/23 14:44:10 haase Exp $
*/

/* privilege.h								*/
/* Header file for a privilege class					*/
/* See for usage in privilege.C						*/

#ifndef __PRIVILEGE_H_
#define __PRIVILEGE_H_

#include <config.h>
#include "mapinclude.h"
#include <algo.h>
#include <String.h>


class Privilege
{
	public:
		/** contructor */
		Privilege ()
		{};

		/** destructor */
		~Privilege ()
		{};

		/** for comparing strings, required by the map algorithms */
		struct streq
		{
			bool operator () (const String &s1, const String &s2)
			const
			{
				return fcompare (s1, s2) < 0;
			}
		};

		/** List of Properties for a user */
		typedef map<String, bool, streq> Prop_t;
		/** iterator to a property */
		typedef Prop_t::iterator Prop_p;

		/** List of the users and their properties */
		typedef map<String, Prop_t, streq> PropList_t;
		/** iterator to a property */
		typedef PropList_t::iterator PropList_p;

	private:
		/** to store the properties of all users */
		PropList_t Properties;
		
	public:
		/** set the Prop of the User to init (for initializing new
		users and properties */
		int Add (String User, String Prop, bool init)
		{
			DEB_F ("Privilege::Add");
			Properties[User][Prop] = init;

			return 0;
		}

		/** checks if Prop is set for User */
		bool IsSet (String User, String Prop)
		{
			DEB_F ("Privilege::IsSet");
			PropList_p u = Properties.find (User);
			if (u != Properties.end ())
			{
				Prop_p p = (*u).second.find (Prop);
				if (p != (*u).second.end ())
				{
					return Properties[User][Prop];
				}
			}

			return false;
		};

		/** set Prop to true for User */
		int Set (String User, String Prop)
		{
			DEB_F ("Privilege::Set");
			PropList_p u = Properties.find (User);

			if (u == Properties.end ())
			{
				return -1;
			}

			Prop_p p = (*u).second.find (Prop);
			if (p == (*u).second.end ())
			{
				return -1;
			}

			Properties[User][Prop] = true;

			return 0;
		};

		/** set Prop to false for User */
		int Clear (String User, String Prop)
		{
			DEB_F ("Privilege::Clear");
			PropList_p u = Properties.find (User);

			if (u == Properties.end ())
			{
				return -1;
			}

			Prop_p p = (*u).second.find (Prop);
			if (p == (*u).second.end ())
			{
				return -1;
			}

			Properties[User][Prop] = false;

			return 0;
		};
};

#endif

