/*
$Header: /home/elrond/baikal/src/CVS/daq98/process.C,v 1.4 1999/02/03 03:42:20 steffen Exp $
*/

/*------------------------------------------------------------------------------
   Files: process.C 
   Author: Heiko Haase 
           Institution: DESY - IFH Zeuthen
   
   implementation of the classes 
      Process            -> Representation of a Process
                         keeps track of all children 
      SignalHandlerClass -> for signal handling

------------------------------------------------------------------------------*/
#include <config.h>
#include <errno.h>
#include <localdir.h>
#include "process.h"
#include "base.h"


/** Global function to handle all signals */
void SignalHandlerFnc(int sig){
     DEB_F("void SignalHandlerFnc(int sig)");
     sigset_t sigset,oldsigset;
     sigfillset(&sigset);
     // prevent the delivery of further signals
     sigprocmask(SIG_BLOCK,&sigset,&oldsigset);
     // call signal handler
     DaqProcessBase::ThisProcess->GetSignalHandler().Handle(
	  *DaqProcessBase::ThisProcess,sig);
     // allow signals
     sigprocmask(SIG_SETMASK,&oldsigset,NULL);
}

SignalHandlerClass::SignalHandlerClass(){
     lastsignal=-1;
}

SignalHandlerClass::~SignalHandlerClass(){
}

sighandler_t SignalHandlerClass::SignalAction(int sig,sighandler_t handler){
     DEB_F("int SignalHandlerClass::SignalAction(int sig,void (*handler)(int))");
     return signal(sig,handler);
}

void SignalHandlerClass::Handle(Process &P,int sig){
     DEB_F("void SignalHandlerClass::Handle(Process &P,int sig)");
     cerr << "signal " << sig << " caught " << endl;
     switch(sig){
     case SIGHUP:    Handle_SIGHUP(P); break;
     case SIGINT:    Handle_SIGINT(P); break;
     case SIGQUIT:   Handle_SIGQUIT(P); break;
     case SIGILL:    Handle_SIGILL(P); break;
     case SIGTRAP:   Handle_SIGTRAP(P); break;
     case SIGABRT:   Handle_SIGABRT(P); break;
     case SIGBUS:    Handle_SIGBUS(P); break;
     case SIGFPE:    Handle_SIGFPE(P); break;
     case SIGUSR1:   Handle_SIGUSR1(P); break;
     case SIGSEGV:   Handle_SIGSEGV(P); break;
     case SIGUSR2:   Handle_SIGUSR2(P); break;
     case SIGPIPE:   Handle_SIGPIPE(P); break;
     case SIGALRM:   Handle_SIGALRM(P); break;
     case SIGTERM:   Handle_SIGTERM(P); break;
#if defined SIGSTKFLT
     case SIGSTKFLT: Handle_SIGSTKFLT(P); break;
#endif
     case SIGCHLD:   Handle_SIGCHLD(P); break;
     case SIGCONT:   Handle_SIGCONT(P); break;
     case SIGTSTP:   Handle_SIGTSTP(P); break;
     case SIGTTIN:   Handle_SIGTTIN(P); break;
     case SIGTTOU:   Handle_SIGTTOU(P); break;
     case SIGURG:    Handle_SIGURG(P); break;
     case SIGXCPU:   Handle_SIGXCPU(P); break;
     case SIGXFSZ:   Handle_SIGXFSZ(P); break;
     case SIGVTALRM: Handle_SIGVTALRM(P); break;
     case SIGPROF:   Handle_SIGPROF(P); break;
     case SIGWINCH:  Handle_SIGWINCH(P); break;
     case SIGIO:     Handle_SIGIO(P); break;
#if defined SIGPWR
     case SIGPWR:    Handle_SIGPWR(P); break;
#endif
#if defined SIGUNUSED
     case SIGUNUSED: Handle_SIGUNUSED(P); break;
#endif
     default: break;
     }
     lastsignal=sig;
     SignalAction(sig,SignalHandlerFnc);
     return;
}

int SignalHandlerClass::Handle_SIGHUP(Process &){ 
     DEB_M(<< "Signal SIGHUP caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGINT(Process &){
     DEB_M(<< "Signal SIGINT caught, exiting"); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGQUIT(Process &){ 
     DEB_M(<< "Signal SIGQUIT caught"); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGTRAP(Process &){
     DEB_M(<< "Signal SIGTRAP caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGABRT(Process &){ 
     DEB_M(<< "Signal SIGABRT caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGBUS(Process &){
     DEB_M(<< "Signal SIGBUS caught");
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGFPE(Process &){
     DEB_M(<< "Signal SIGFPE caught"); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGILL(Process &){ 
     DEB_M(<< "Signal SIGILL caught"); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGUSR1(Process &){
     DEB_M(<< "Signal SIGUSR1 caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGSEGV(Process &){
     DEB_M(<< "Signal SIGSEGV caught, exiting"); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGUSR2(Process &){
     DEB_M(<< "Signal SIGUSR2 caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGPIPE(Process &){
     DEB_M(<< "Signal SIGPIPE caught, broken Pipe/Socket connection, exiting..."); 
     Quit_FLAG=true;
     return 0;
}

int SignalHandlerClass::Handle_SIGALRM(Process &){
     DEB_M(<< "Signal SIGALRM caught");
     return 0;
}

int SignalHandlerClass::Handle_SIGTERM(Process &){
     DEB_M(<< "Signal SIGTERM caught"); 
     Quit_FLAG=true; // exit on next possible point
     return 0;
}

int SignalHandlerClass::Handle_SIGSTKFLT(Process &){
     DEB_M(<< "Signal SIGSTKFLT caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGCHLD(Process &P){
     DEB_M(<< "Signal SIGCHLD caught"); 
     pid_t pid;
#ifdef __LYNX__
     union wait statusp;
#else // __LYNX__
     int statusp;
#endif // __LYNX__ 

     pid=waitpid(-1,&statusp,WNOHANG|WUNTRACED);
     if(WIFEXITED(statusp)){
	  cerr << "child [" << pid << "] of process " 
	       << P.GetName() << " removed." << endl;
	  P.RmChild(pid);
	  return pid;
     }
     return 0;
}

int SignalHandlerClass::Handle_SIGCONT(Process &){
     DEB_M(<< "Signal SIGCONT caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGTSTP(Process &){
     DEB_M(<< "Signal SIGTSTP caught");
     return 0;
}

int SignalHandlerClass::Handle_SIGTTIN(Process &){
     DEB_M(<< "Signal SIGTTIN caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGTTOU(Process &){
     DEB_M(<< "Signal SIGTTOU caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGURG(Process &){
     DEB_M(<< "Signal SIGURG caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGXCPU(Process &){
     DEB_M(<< "Signal SIGXCPU caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGXFSZ(Process &){
     DEB_M(<< "Signal SIGXFSZ caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGVTALRM(Process &){
     DEB_M(<< "Signal SIGVTALRM caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGPROF(Process &){
     DEB_M(<< "Signal SIGPROF caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGWINCH(Process &){
     DEB_M(<< "Signal SIGWINCH caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGIO(Process &){
     DEB_M(<< "Signal SIGIO caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGPWR(Process &){
     DEB_M(<< "Signal SIGPWR caught"); 
     return 0;
}

int SignalHandlerClass::Handle_SIGUNUSED(Process &){
     DEB_M(<< "Signal SIGUNUSED caught"); 
     return 0;
}

int SignalHandlerClass::LastSignal(){
     return lastsignal;
}


Process::Process(){
     DEB_F("Process::Process()");
     pid=getpid();
     ppid=getppid();
//     SetSignalHandler(*DefaultSignalHandler);
     //  Set up some common signal Handlers
     // Hanlde signal SIGCHLD, (a child died)
     GetSignalHandler().SignalAction(SIGCHLD,(sighandler_t)SIG_MY);
     // Terminate the process on SIGTERM
     GetSignalHandler().SignalAction(SIGTERM,(sighandler_t)SIG_MY);
     // Ignore SIGINT (Ctrl-C)
     GetSignalHandler().SignalAction(SIGINT,(sighandler_t)SIG_IGN);
     // Exit on exception like SIGBUS, SIGFPE, SIGILL, SIGQUIT and SIGSEGV
     GetSignalHandler().SignalAction(SIGILL,(sighandler_t)SIG_DFL);
     GetSignalHandler().SignalAction(SIGBUS,(sighandler_t)SIG_DFL);
     GetSignalHandler().SignalAction(SIGFPE,(sighandler_t)SIG_MY);
     GetSignalHandler().SignalAction(SIGSEGV,(sighandler_t)SIG_DFL);
     GetSignalHandler().SignalAction(SIGQUIT,(sighandler_t)SIG_MY);
     // Used by some process to be informed about the 
     // successful starting of a child process
     GetSignalHandler().SignalAction(SIGUSR1,(sighandler_t)SIG_MY);
     // leave all others in their default state
}

Process::~Process(){
     DEB_F("Process::~Process()");
}

int Process::Fork(){
     DEB_F("int Process::Fork()");
     int ret;
     TEST_SYS_FAILURE(ret=fork(),"fork");
     if(ret==0){
	  // Child Process
	  Children.erase(Children.begin(),Children.end());
	  pid=getpid();
	  ppid=getppid();
//	  SetSignalHandler(*DefaultSignalHandler);
     } else {
	  // Parent Process
	  InsChild(ret);
     }
     return ret;
}

int Process::InsChild(int pid){
     DEB_F("int Process::InsChild(int pid)");
     Children.push_back(pid);
     return 0;
}

int Process:: RmChild(int cpid){
     DEB_F("int Process::RmChild(int cpid)");
     list<int>::iterator i=find(Children.begin(),Children.end(),cpid);
     if(i!=Children.end()){
	  Children.erase(i);
     } else {  	
	  char s1[10],s2[10];
	  sprintf(s1,"%i",cpid);
	  sprintf(s2,"%i",pid);
	  (*Error)=PROC_NOT_CHILD_OF;
	  (*Error).SetParam(s1,s2);
	  (*Error)="Process::RmChild()";
	  return -1;
     }
     return 0;
}

int Process::NumberChildren(){
     DEB_F("int Process::NumberChildren()");
     return Children.size();
}

SignalHandlerClass & Process::GetSignalHandler(){
     DEB_F("SignalHandlerClass & Process::GetSignalHandler()");
     return *aSignalHandler;
}

void Process::SetSignalHandler(SignalHandlerClass &SH){
     DEB_F("void Process::SetSignalHandler(SignalHandlerClass &SH)");
     aSignalHandler=&SH;
}

int Process::LockProcess(){
     DEB_F("int Process::LockProcess(char *fn)");
     int r;
     String LockFileName=DAQ_LOCK_DIR + String("/") +String(Name)+String(".lock");
     if((r=open((const char *)LockFileName,O_RDWR|O_CREAT|O_EXCL,S_IRUSR|S_IWUSR))<0){
	  if (errno==EEXIST){
	       (*Error)=PROCESS_ALLREADY_RUNNING;
	       (*Error)=Name;
	       return -1;
	  } else {
	       (*Error)=SYS_ERROR;
	       (*Error)="open (lockfile)";
	       return -1;
	  }
     }
     ofstream O(r);
     O << pid;
     O.close();
     return 0;
}

int Process::UnlockProcess(){
     DEB_F("int Process::UnockProcess(char *fn)");
     int p;
     String LockFileName=DAQ_LOCK_DIR + String("/") +String(Name)+String(".lock");
     ifstream I((const char *)LockFileName);
     I >> p;
     I.close();
     if(p==pid){
	  remove((const char *)LockFileName);
     }
     return 0;
}
     















