/*
  $Header: /home/elrond/baikal/src/CVS/daq98/process.h,v 1.6 1999/02/03 03:42:21 steffen Exp $
*/

#if ! defined __PROCESS_H__
#define __PROCESS_H__

/* ------------------------------------------------------------------------------
   Files: process.h 
   Author: Heiko Haase 
           Institution: DESY - IFH Zeuthen
   
   Classes:
   Process            -> Representation of a Process
                         keeps track of all children 
   SignalHandlerClass -> for signal handling

------------------------------------------------------------------------------*/

// system
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
// STL-includes
#include <list.h>
#include <algo.h>
#include <String.h>
// my own includes
#include <config.h>
#include "base.h"
#include "daqdebug.h"
#include "error.h"
#include "errorlog.h"

// type of a signal handler function
typedef void (*sighandler_t)(int);

class Process;

class SignalHandlerClass:public DaqProcessBase
{
public:
     /** Constructor */
     SignalHandlerClass();
     /** Destructor */
     virtual ~SignalHandlerClass();

     /** Set a signal handler for the specified signal Should be the
       global Function void SignalHandlerFnc(int sig) or SIG\_IGN or
       SIF\_DFL
       If void SignalHandlerFnc(int sig) is specified the
       Memberfunction Handle\_SIGXXX(Process *P) according to the
       delivered signal is called */
     sighandler_t SignalAction(int sig,sighandler_t handler);
     /** Handles all signals and calls the appropriate signal handling
       routine */
     void Handle(Process &P,int sig);
     /** Signal handlers for the signals */
     virtual int Handle_SIGHUP(Process &P);
     virtual int Handle_SIGINT(Process &P);
     virtual int Handle_SIGQUIT(Process &P);
     virtual int Handle_SIGILL(Process &P);
     virtual int Handle_SIGTRAP(Process &P);
     virtual int Handle_SIGABRT(Process &P);
     virtual int Handle_SIGBUS(Process &P);
     virtual int Handle_SIGFPE(Process &P);
     virtual int Handle_SIGUSR1(Process &P);
     virtual int Handle_SIGSEGV(Process &P);
     virtual int Handle_SIGUSR2(Process &P);
     virtual int Handle_SIGPIPE(Process &P);
     virtual int Handle_SIGALRM(Process &P);
     virtual int Handle_SIGTERM(Process &P);
     virtual int Handle_SIGSTKFLT(Process &P);
     virtual int Handle_SIGCHLD(Process &P);
     virtual int Handle_SIGCONT(Process &P);
     virtual int Handle_SIGTSTP(Process &P);
     virtual int Handle_SIGTTIN(Process &P);
     virtual int Handle_SIGTTOU(Process &P);
     virtual int Handle_SIGURG(Process &P);
     virtual int Handle_SIGXCPU(Process &P);
     virtual int Handle_SIGXFSZ(Process &P);
     virtual int Handle_SIGVTALRM(Process &P);
     virtual int Handle_SIGPROF(Process &P);
     virtual int Handle_SIGWINCH(Process &P);
     virtual int Handle_SIGIO(Process &P);
     virtual int Handle_SIGPWR(Process &P);
     virtual int Handle_SIGUNUSED(Process &P);
     /** returns the ID of the caught signal */
     int LastSignal();
private:
     /** last signal that was caught */
     int lastsignal;
};


class Process: public Base
{
public:
     /** Constructor */
     Process();
     /**  Destructor */
     ~Process();
     /** forks a new process with system function fork and registers
       the child in a table */
     int Fork();
     /** Returns a Refrence to the currently used Signal hanlder class */
     SignalHandlerClass &GetSignalHandler();
     /** Sets the Signal hanlder Class ro be used */
     void SetSignalHandler(SignalHandlerClass &SH);
     /** "Locks" the current process by writing a file such that no
       other process of the same kind can be started or returns -1 if
       another processs is already running, should be called
       immediatly after process creation */
     int LockProcess();
     /** "Unlocks" the process call right befor process termination */
     int UnlockProcess();
     /** sets the name of the process */
     void SetName(char *N){
	  Name=N;
     }
     /** returns the name of the process */
     char *GetName(){
	  return Name;
     }
     
     /** Get process ID.
	 @return Process ID */ 
     int GetPid(){
	  return pid;
     }
     
     /** Get parent's process ID.
	 @return Process ID of parent process */
     int GetPPid(){
	  return ppid;
     }

     /**  removes a dead child (invoked from SIGCHILD -handler) */
     int RmChild(int cpid);
private:
     /**  Insert a newly createt child into the children table */
     int InsChild(int cpid);
     /** returns the number of children */
     int NumberChildren();
private:
     /**  PID of the process */
     int pid;
     /**  identifiing name of the process */
     char *Name;
     /**  PID of the parent process */
     int ppid;
     /** Pointer to the Signal handler class */
     SignalHandlerClass *aSignalHandler;
     /** STL -list which contains the pids of the child-processes */
     list<int> Children;
};

void SignalHandlerFnc(int sig);

#define SIG_MY (SignalHandlerFnc)

#endif





