/*
$Header: /home/elrond/baikal/src/CVS/daq98/qtreewidget.h,v 1.3 1998/11/23 14:44:16 haase Exp $
*/

#if ! defined __QTREEWIDGET_H__
#define __QTREEWIDGET_H__

#include <config.h>
#include "itemtree.h"
#include <qlistbox.h>
#include <qpainter.h>
#include <qfontmet.h>

class TreeBoxItem: public QListBoxItem
{
public:
     TreeBoxItem(const char *s,ItemTree *subtree,int level=0,const TreeBoxItem *parent=0,bool last=true)
	  :QListBoxItem(){
	  setText(s); 
	  SubTree=subtree;
	  Level=level;
	  SubTreeOpened=false;
	  LastMark=new bool[Level+1];
	  if(parent!=0){
	       for(int i=0;i<Level;i++) LastMark[i]=parent->LastMark[i];
	  }
	  LastMark[Level]=last;
     }
     ~TreeBoxItem(){
	  delete LastMark;
     }       
     int GetLevel() const{
	  return Level;
     }
     ItemTree *GetTree() const{
	  return SubTree;
     }
     bool IsOpened() const{
	  return SubTreeOpened;
     }
     void SetOpened(bool o)const{
	  bool *temp=(bool*)&SubTreeOpened;
	  *temp=o;
     }       
     virtual void paint(QPainter *,QColorGroup *g,bool isCurrent,bool hasFocus);
protected:
     virtual void paint(QPainter *){};
     virtual int height(const QListBox *) const;
     virtual int width(const QListBox *) const;
private:
     bool SubTreeOpened;
     ItemTree *SubTree;
     int Level;
     bool *LastMark;
};

class ListBoxTreeItem:public QListBoxText
{
public:
     ListBoxTreeItem(const ItemBase *TDI,const char *txt=0)
	  :QListBoxText(txt),Data(TDI){
     }
     ~ListBoxTreeItem(){
     }
     const ItemBase *GetData() const{
	  return Data;
     }
private:
     const ItemBase *Data;
};

class ListBoxWithData: public QListBox
{
     Q_OBJECT;
public:
     ListBoxWithData(QWidget* parent=0,const char* name=0,WFlags f=0)
	  :QListBox(parent,name,f){}
     ~ListBoxWithData(){}
     const QListBoxItem *GetItem(int index){
	  return item(index);
     }     
};

class TreeBox:public ListBoxWithData
{
     Q_OBJECT;
public:
     TreeBox(QWidget* parent=0,const char* name=0,WFlags f=0):ListBoxWithData(parent,name,f){
	  connect(this,SIGNAL(selected(int)),
		  this,SLOT(SubTreeAction(int)));
     }
     ~TreeBox(){}
     const TreeBoxItem *GetItem(int index){
	  return (TreeBoxItem *)item(index);
     }     
protected:
     void paintCell(QPainter *,int row,int col);
private slots:
     void SubTreeAction(int index);
};


class QTreeWidget:public QWidget
{
     Q_OBJECT;
public:
     QTreeWidget(ItemTree *datatree,char *name,QWidget *parent=0,const char *name=0);
     ~QTreeWidget();
     void SetGeometry();
     const ItemBase *GetItem(int index){
	  return ((const ListBoxTreeItem*)ItemView->GetItem(index))->GetData();
     }       
public slots:
     void TreeItemSelected(int index);
     void ItemViewSelected(int index);
     void ItemViewHighlighted(int index);
protected:
     void resizeEvent(QResizeEvent *);
signals:
     void selected(int index);
     void highlighted(int index);
private:
     ItemTree *TheTree;
     TreeBox *TreeView;
     ListBoxWithData *ItemView;
};


#endif //__QTREEWIDGET_H__
