/*
  $Header: /home/elrond/baikal/src/CVS/daq98/s7201module.C,v 1.8 1999/02/10 17:31:46 daq Exp $
*/


#include <iostream.h> 
#include "s7201module.h"

#define DEBUG_LOCAL(A)  cout   A


S7201_ScalerModule_t::S7201_ScalerModule_t(DaqProcessBase *parent):
     SCALERModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,parent)
{
}

int S7201_ScalerModule_t::ResetModule(StringVector_t &/*errors*/) {
     
     return 0;
}


/*-------------------------------------------------------*/
/*                                                       */
/*                VME ACCESS Routines                    */
/*                                                       */
/*                                                       */
/*                                                       */
/*-------------------------------------------------------*/
#ifdef VME_ACCESS

S7201_ScalerModule_t::~S7201_ScalerModule_t(){
     vme_rel(BaseAdr,nw_map_7201);
}      

int S7201_ScalerModule_t::InitModule(StringVector_t &/*errors*/){

     cerr << "**** 7201 init\n";
     ModuleAccess=0;
// set to 2kB   **** ????
     nw_map_7201=1024;
     u_i32 addr = GetModuleConfig()->Address_Slot;
     BaseAdr = (u_int *)vme_map(addr,nw_map_7201,0x09);  // Base address
     StatusAndControl = BaseAdr; // Status and control register, offset 0
     IRQAndVersion = (u_int *) ((void *) BaseAdr + 0x04); // IRQ and Version register, offeset 4 
     CopyDisable = (u_int *) ((void *) BaseAdr + 0x0C); // Copy Disable Register, offset C
     FIFO = (u_int *) ((void *) BaseAdr + 0x100); // FIFO 
     ClearFIFO = (u_int *) ((void *) BaseAdr + 0x20);   // Clear FIFO, Counter, Logic
     EnableNext = (u_int *) ((void *) BaseAdr + 0x28);   // Enable NEXT Logic
     DisableNext = (u_int *) ((void *) BaseAdr + 0x2C);   // Disable NEXT Logic
     KeyReset = (u_int *) ((void *) BaseAdr + 0x60);   // Reset register (Key reset)

   
     *KeyReset = 0;                      // key reset
     LED_Off();

     *StatusAndControl = 0xff00ff00;          // disable & clear all
     int inp_mode  = 0;                       // input mode =0,1,2,3
     *StatusAndControl = (0x03 & inp_mode)<< 2;  
     *StatusAndControl = 0x1 << 16;           // enable EXT_NEXT input

     // ??? What is this ? THis actually sets bit 16 which is version number (read only)
     *IRQAndVersion = 0x10 << 11;              // disable VME interrupts
     NumActiveChannels=0;
     for(int i=0;i<NumChannels;i++){
	  if(!((0x1<<i) & 
	       ((SCALERModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->Mask)){
	       NumActiveChannels++;
	  }
     }
     *CopyDisable = 
	  ((SCALERModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->Mask;  // Mask channels, bit set to 0 is on

     *ClearFIFO = 0;                          // clear FIFO

     Enable_Next();                           // enable NEXT-logic
     
     LED_On();
     char txt[1024];
     sprintf(txt,"%s init: addr 0x%x status/control 0x%x IRO/VSN 0x%x\n",
	     GetModuleConfig()->ModuleID,
	     addr,*StatusAndControl,*IRQAndVersion);
     LogMsg(txt,4,4,INFO);
     return 0;
}

bool S7201_ScalerModule_t::HasModuleData(){
     ModuleAccess++;
     return (0x01&(*StatusAndControl>>8)) ? 0 : 1;           // FIFO empty flag
}

int S7201_ScalerModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     u_i16 NumWords=0;
     DataBuffer.WriteToBuffer(NumWords);
     DataBuffer.WriteToBuffer(ModuleAccess);
     NumWords++;
     
     u_i32 data;
     u_i32 count;
     int   chan;
     
     u_i32 StatusWord=(*StatusAndControl>>8) & 0x1F;             // All FIFO bits before readout
     
//     while (!FIFO_Empty()) {
//	  data = *FIFO;
//	  DataBuffer.WriteToBuffer(data);
//	  NumWords++;
//     }

// readout full buffer, in multiples of NumActiveChannels ("events"), apply a timeout
     int i,j;
     do {      
	  for (i=0; i<NumActiveChannels; i++){
	       data = *FIFO;
	       DataBuffer.WriteToBuffer(data);
	       NumWords++;
	       if ((i<(NumActiveChannels-1)) && FIFO_Empty()) {
		    for (j=0; j<WAIT_LOOPS_S7201_READ; j++){
			 if (!FIFO_Empty())  break;
		    }
		    if (j==(WAIT_LOOPS_S7201_READ-1))  {
//			 cout << " **** 7201_FIFO timeout, nwords = "<< nwords <<endl;
			 break;
		    }
	       }
	  }
     } while (!FIFO_Empty());
     
     int rem=(NumWords - 1) % NumActiveChannels;
     if(rem){
	  for(int k=0;k< (NumActiveChannels - rem);k++){
	       DataBuffer.WriteToBuffer((u_i32)0);
	       NumWords++;
	  }
     }
     
     DataBuffer.WriteToBuffer(StatusWord);
     NumWords++;
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     // write the size
     cerr << "* " << NumWords << " " << NumWords*GetBytesPerInfoUnit() << endl;
     DataBuffer.WriteToBuffer((u_i16)(NumWords*GetBytesPerInfoUnit()));
     // restore the old position
     DataBuffer.IncPosition(NumWords*GetBytesPerInfoUnit());

     return 0;
}


 
 
/*-------------------------------------------------------*/
/*                                                       */
/*                NO VME ACCESS Routines                 */
/*                                                       */
/*                Software simulation only               */
/*                                                       */
/*-------------------------------------------------------*/
#else // VME_ACCESS
 
S7201_ScalerModule_t::~S7201_ScalerModule_t(){
     DEBUG_MESSAGE( << "Module destroyed"<< GetModuleConfig()->ModuleID<< endl);
}      

int S7201_ScalerModule_t::InitModule(StringVector_t &/*errors*/){
     
     ModuleAccess=0;
     u_i32 addr = GetModuleConfig()->Address_Slot;
     NumActiveChannels=0;
     for(int i=0;i<NumChannels;i++){
	  if(!((0x1<<i) & ((SCALERModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)GetModuleConfig())->Mask)){
	       NumActiveChannels++;
	  }
     }
     char txt[1024];
     sprintf(txt,"%s init: addr 0x%x\n",
	     GetModuleConfig()->ModuleID,
	     addr);
     LogMsg(txt,4,4,INFO);
     return 0;
}

bool S7201_ScalerModule_t::HasModuleData(){
     ModuleAccess++;
     return 1;
}
     
int S7201_ScalerModule_t::ReadToBuffer(BinaryDataBuffer &DataBuffer)
{
     DEBUG_MESSAGE( << "Module " << GetModuleConfig()->ModuleID << " ReadToBuffer." << endl);
     
     // first there must be the tag for the module in the bytestream
     DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
     // save the position where we have to store the number of hits 
     DataBuffer.PushPosition();
     // Write a zero size first
     u_i16 NumWords=0;
     DataBuffer.WriteToBuffer(NumWords);
     DataBuffer.WriteToBuffer((u_i32)ModuleAccess);
     NumWords++;
	  
     int nw_buff=0;
     u_i32 data;
     u_i32 count;
     int   chan;
     for (int j=0; j<NumActiveChannels; j++) {
	  
	  u_i32 sim_cou = 10*j;
	  data = 0x1FFFFFFF & ((j<<24) | sim_cou) ;    // simulate the 24 Bit mode
	  DataBuffer.WriteToBuffer(data);
	  NumWords++;
     }
     
     u_i32 StatusWord=0;
     NumWords++;
     DataBuffer.WriteToBuffer(StatusWord);
     // set the position to the place where we have to write the size 
     DataBuffer.PopPosition();
     // write the size
     DataBuffer.WriteToBuffer((u_i16)(NumWords*GetBytesPerInfoUnit()));
     // restore the old position
     DataBuffer.IncPosition(NumWords*GetBytesPerInfoUnit());
     
     return 0;
}


#endif // VME_ACCESS
