#if ! defined __S7201CNTMODULE_H__
#define  __S7201CNTMODULE_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/s7201module.h,v 1.7 1999/02/10 17:31:46 daq Exp $

  $Log: s7201module.h,v $
  Revision 1.7  1999/02/10 17:31:46  daq
   Version 1.310

  Revision 1.6  1999/02/09 22:14:17  daq
  *** empty log message ***

  Revision 1.5  1999/02/09 17:58:08  wischnew
  *** empty log message ***

  Revision 1.4  1999/02/09 16:28:43  wischnew
  overloaded data(), channel();

  Revision 1.3  1999/02/09 15:52:12  steffen
  soory, last second changes

  Revision 1.2  1999/02/08 08:08:54  steffen
  - * code for scalerbase and s7201 added and/or debugged

  Revision 1.1  1999/02/05 18:35:08  steffen
  * Major cleanup:
    - threw out code for binary config records
    - cleaned up some module classes
  * Fixed bug which prevent 32 bit values to be read correctly from
    detector cnf
  * Added masking to s7201 module

*/

#include <config.h>
#include "scalermodule.h"


/*---------------------------------------------------------*/
/*                                                         */
/*    Struck 7201 scaler configuration data structure      */
/*                                                         */
/*---------------------------------------------------------*/
/** Configuration data structure for Struck 7201 32 Channel VME Multiscaler */
typedef SCALERModuleConfigRecord_t S7201ScalerConfigRecord_t;
MAKE_REFCOUNT_POINTER(S7201ScalerConfigRecord_t,S7201ScalerConfigRecord_p);

#define  MAX_FIFO_S7201 32*1024          // 32Chan * 4kB FIFO = 32k Words
#define  WAIT_LOOPS_S7201_READ   1000       // waitloops in fifo-readout

/*---------------------------------------------------------*/
/*                                                         */
/*    Struck 7201 scaler module class                      */
/*                                                         */
/*---------------------------------------------------------*/
/* Class for STR7201 32 Channel VME Multiscaler */
class S7201_ScalerModule_t:public SCALERModuleClass_t
{
     ClassTypeInfo(SCALERModuleClass_t);
     enum 
     { 
	  NUMBER_OF_CHANNELS = 32,
	  BYTES_PER_INFO_UNIT = 4
     };
public:
     
     /** Constructor.
	 @param parent Pointer to parent class. Used for sending messages in Parent context. */
     S7201_ScalerModule_t(DaqProcessBase *Parent);

     /** Destructor */
     ~S7201_ScalerModule_t();
     
     /** Check for data
      @return {\it true} if module has data, {\it false} otherwise */
     virtual bool HasModuleData();

     /** Read the module data and write to the data buffer
	 @param DataBuffer Reference to the data buffer
	 @return 0 on success, -1 on failure */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer);
     	
     /** Initialize the module
      @param  errors Reference to a vector of error strings. 
      If an error occures, the function can add a descriptive messages.
      @return 0 on success, -1 on failure */
     virtual int InitModule(StringVector_t &errors);

     /** Reset the module
	 @param  errors Reference to a vector of error strings. 
	 If an error occures, the function can add a descriptive messages.
	 @return 0 on success, -1 on failure */
     virtual int ResetModule(StringVector_t &errors);


protected:
     /**  Check if FIFO is empty
	  @return 1 if empty, 0 if module has data in FIFO.*/
     inline int FIFO_Empty (){
	  return  (0x1&(*StatusAndControl>>8)) ?  1 : 0;
     }

     /**  Check if FIFO is almost empty
	  @return 1 if FIFO is almost empty, 0 otherwise  */
     inline int FIFO_Almost_Empty (){
	  return  (0x1&(*StatusAndControl>>9)) ?  1 : 0;       
     }
     
     /**  Check if FIFO is half full
	  @return 1 if FIFO is half full, 0 otherwise  */
     inline int FIFO_Half_Full (){
	  return  (0x1&(*StatusAndControl>>10)) ?  1 : 0;           
     }
     
     /**  Check if FIFO is almost full
	  @return 1 if FIFO is almost full, 0 otherwise  */
     inline int FIFO_Almost_Full (){
	  return  (0x1&(*StatusAndControl>>11)) ?  1 : 0;           
     }
     
     /**  Check if FIFO is full
	  @return 1 if FIFO is full, 0 otherwise  */
     inline int FIFO_Full (){
	  return  (0x1&(*StatusAndControl>>12)) ?  1 : 0;
     }
     
     /**  Enable Next Logic: Start the counting process 
       */
     inline void Enable_Next (){
	  *EnableNext = 0;
     }

     /**  Turn user LED on */ 
     inline void LED_On(){
	  *StatusAndControl = 0x1;
     }
     
     /**  Turn user LED off */ 
     inline void LED_Off(){
	  *StatusAndControl = 0x100;
     }
     
     /**  Get channel number in 24bit counter mode
      @param data 32 bit data word from scaler
      @return Channel number */
     virtual inline int Channel(u_i32 data){
       return ( 0x1F & (data>>24) );
     }
 
     /**  Get counter value in 24bit counter mode
      @param data 32 bit data word from scaler
      @return Counter value */
     virtual inline u_i32 Count(u_i32 data) {
	  return ( 0x00FFFFFF & data);
     }

protected:

     /** Base address of module */
     u_i32 *BaseAdr;
     
     /** Status and control register. */
     u_i32 *StatusAndControl;
     
     /** VME IRQ and version register.
	 This read/write register controls the VME interrupts 
	 of the module. It also holds the desgin number and version number
	 of the selected LCA file of the installed boot EPROM. */
     u_i32 *IRQAndVersion;
     
     /** Copy disable register address.
	 This register control which channeles are copied to the FIFO.
	 By writing a one to the corresponding bit of the register, a channel 
	 can be excluded from being copied to the Fifo. */
     u_i32 *CopyDisable;

     /** Clear FIFO, logic and counters */
     u_i32 *ClearFIFO;

	  
     /** Enable next-logic */
     u_i32 *EnableNext;

     /** Disable next-logic */
     u_i32 *DisableNext;

     /** Reset register (key reset) */
     u_i32 *KeyReset;

     /** Fifo read access address*/
     u_i32 *FIFO;

     u_i32 nw_map_7201;

     /** Number of channels that are being copied to the FIFO */
     int NumActiveChannels;

     /** Number of times {\tt \Ref{HasModuleData}} is accessed */
     u_i32 ModuleAccess;
};

#endif // __S7201CNTMODULE_H__
