#if ! defined __SCALERMODULEEVAL_H__
#define __SCALERMODULEEVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/scalermoduleeval.h,v 1.9 1999/02/10 11:59:51 steffen Exp $
*/

#include <config.h>
#include "moduleevalbase.h"
#include "scalermodule.h"

/** Class to evaluate the data received from a S7201 Counter module  */
class ScalerModuleEvalClass_t: public ModuleEvalBaseClass_t
{
     enum 
     { 
	  NUMBER_OF_CHANNELS =32,
	  BYTES_PER_INFO_UNIT=4
     };
     ClassTypeInfo(ModuleEvalBaseClass_t);
     inline ScalerModuleEvalClass_t(DaqProcessBase *Parent)
	  :ModuleEvalBaseClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT, Parent){}
     /** set additional information out of the config file, like channel mapping etc.*/ 
     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &  HWConfig){
	  SetTag(ModuleConfig->BinaryTag);
	  NumActiveChannels=0;
	  for(int i=0;i<NumChannels;i++){
	       if(!((0x1<<i) 
		    & ((SCALERModuleConfigRecord_t *)(ChannelModuleConfigRecord_t*)ModuleConfig)->Mask)){
		    NumActiveChannels++;
	       }
	  }
	  ChannelConfigRecordVector_t::iterator chcr_i;
	  for(chcr_i=HWConfig.ChannelConfigRecordVector->begin();
	      chcr_i<HWConfig.ChannelConfigRecordVector->end();
	      chcr_i++){
	       if((*chcr_i)->SCALER_Id && (strcasecmp(ModuleConfig->ModuleID,(*chcr_i)->SCALER_Id)==0)){
		    // we found a channel, that is connected to the current module
		    // insert it in our vector
		    SetChannelMapping((*chcr_i)->SCALER_Channel,(*chcr_i)->ID);
		    if(outstream) *outstream << "CONFIG " << Tag 
					     << "\t" 
					     << (*chcr_i)->SCALER_Channel 
					     << "\t"
					     << (*chcr_i)->ID 
					     << endl;
	       }
	  }
     }
     
     /** read the data and build the event part that this module is good for */
     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
	  
	  u_i32 *ModuleData;
	  int NumReads;
	  u_i16 NumBytes;
	  s_i32 CurOMChannel;

	  //S7201CounterEvent_t &CntEvnt(*(TempEventBuffer->S7201CounterEvent)); // ref, dh no cop-constr. (wie alias)
	  RawDataBuffer.ReadFromBuffer(NumBytes);
	  NumReads=NumBytes/BytesPerInfoUnit;
	  
	  ModuleData=(u_i32*)ReadFromBuffer(RawDataBuffer);
	  NumReads--;
	  if(outstream){
	       *outstream << endl  << "ACCESS " << Tag << " " 
			  << *ModuleData << endl;
	       for(int i=1;i<NumReads;i++){
		    if(!((i - 1) % NumActiveChannels))
			 *outstream << "DATA " << Tag;
		    ModuleData=(u_i32*)ReadFromBuffer(RawDataBuffer);
		    *outstream << " " << GetCounterVal(*ModuleData);
		    if(!(i % NumActiveChannels))
			 *outstream << endl;
	       }
	       ModuleData=(u_i32*)ReadFromBuffer(RawDataBuffer);
	       *outstream << "STATUS " << Tag << " " << *ModuleData << endl;
	  }
	  else{
	       for(int i=0;i<NumReads-1;i++){
		    ModuleData=(u_i32*)ReadFromBuffer(RawDataBuffer);
		    // get the  channel
		    /*
		      if((CurOMChannel=GetChannelMapping(GetChannel(*ModuleData)))<0){
		      // error handling, couldn't map channel
		      char txt[1000];
		      sprintf(txt,"Can't map channel %i of scaler module with tag %i",
		      *ModuleData,GetTag());
		      LogMsg(txt,2,2,WARNING);
		      continue;
		      }
		      CntEvnt.CounterVals[CurOMChannel]=*((u_i32*)ModuleData);
		    */
	       }
	       ModuleData=(u_i32*)ReadFromBuffer(RawDataBuffer);
	       return 0;
	  }
     }
     
protected:
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){	  
	  static u_i32 Data;
	  DataBuffer.ReadFromBuffer(Data);
	  return (void*)&Data;
     }

     /**  Get channel number in 24bit counter mode
      @param data 32 bit data word from scaler
      @return Channel number */
     inline int GetChannel(u_i32 data){
       return ( 0x1F & (data>>24) );
     }
 
     /**  Get counter value in 24bit counter mode
      @param data 32 bit data word from scaler
      @return Counter value */
     inline u_i32 GetCounterVal(u_i32 data) {
	  return ( 0x00FFFFFF & data);
     }

public:
     static ostream *outstream;
     
protected:
     u_i32 NumActiveChannels;
};

#endif // __SCALERMODULEEVAL_H__
