
#include <config.h>
#include "mapinclude.h"
#include "clientlib.h"
#include "hbookhistogram.h"

class ScriptingClient:public Client{
public:
     ScriptingClient(const char *pfn_r,const char*pfn_s,char *hfn)
	  :pipefilename_recv(pfn_r),
	   pipefilename_send(pfn_s),
	   hbookfilename(hfn),
	   MustReopenInput(true),
	   ComservReady(true),
	   hbookinit(false),
	   ErrorFlag(0){}
     virtual int DoReady();
     virtual int DoHistogram(MemHistogram_t &Histogram);
     virtual void QuitRequest();
     int EventLoop();
private:
     const char *pipefilename_recv;
     const char *pipefilename_send;
     char *hbookfilename;
     bool MustReopenInput;
     bool ComservReady;
     bool hbookinit;
     int ErrorFlag;
};

int ScriptingClient::DoReady(){
     ComservReady=true;
     ofstream answer(pipefilename_send);
     answer << ((ErrorFlag<0)?"failure":"ok") << endl << flush;
//     cerr << "Got Ready from Comserver" << endl;
}

void ScriptingClient::QuitRequest(){
     if(hbookinit){
	  HBOOKHistogram_t::SaveAllToFile(hbookfilename);
     }
}    

int ScriptingClient::EventLoop(){
     DEB_F("int Client::DoClient()");
     FileNotifier *InputNotifier;
     int ret;
     int fd;
     // open the input pipe
     while(!Quit_FLAG){
	  if(ComservReady&&MustReopenInput){
	       TEST_FAILURE(fd=open(pipefilename_recv,O_RDWR));
	       InputNotifier=new FileNotifier(fd,IONotifier::READ);
	       Selecter.AddNotifier(*InputNotifier);
	  }
	  TEST_TEMP_FAILURE(ret=Selecter.DoSelect());
	  if(Selecter.CheckNotifier(CmdSocketNotifier)){
	       if((ErrorFlag=ServReply())<0){
		    // failure
		    // just don't care about now
	       } 
	  }
	  if(InputNotifier!=0){
	       if(Selecter.CheckNotifier(*InputNotifier)){
		    char txt[MAX_TEXT_SIZE];
		    ifstream inp(fd);
		    inp.getline(txt,MAX_TEXT_SIZE);
		    
		    Selecter.RemoveNotifier(*InputNotifier);
		    delete InputNotifier;
		    InputNotifier=0;
		    MustReopenInput=true;
		    ComservReady=false;
		    close(fd);
		    
		    if(ServCommand(txt)<0){
			 // failure so Comserver is still ready
			 ComservReady=false;
		    }
	       }
	  }
     }
     ofstream answer(pipefilename_send);
     answer << "ok" << endl << flush;
     return 0;
}

int ScriptingClient::DoHistogram(MemHistogram_t &Histogram){
     if(!hbookinit){
	  HBOOKHistogram_t::InitHBook(); // only ones
	  hbookinit=true;
     }
     HistogramDescription_t hd;
     hd.ID=Histogram().ID;
     strcpy(hd.Title,Histogram().Title);
     hd.NumBin=Histogram().NumBin;
     hd.Xmin=Histogram().Xmin;
     hd.Xmax=Histogram().Xmax;
     
     double dx=(hd.Xmax-hd.Xmin)/(hd.NumBin);
     HBOOKHistogram_t *hh=new HBOOKHistogram_t;
     hh->Create(hd);
     for(unsigned int i=0;i<Histogram().NumBin;i++){
	  if(Histogram().Ent[i]>0)
	       hh->Fill(hd.Xmin+(i+0.5)*dx,Histogram().Ent[i]);
     }
     hh->Fill(hd.Xmin-1,Histogram().NumUnderflows);
     hh->Fill(hd.Xmax+1,Histogram().NumOverflows);
     return 0;
}


int main(int argc,char *argv[]){
     DaqProcessBase::InitializeGlobals(argv[0]);
     ScriptingClient SClient(argv[1],argv[2],argv[3]);     
     SClient.EventLoop();
     return 0;
}


