/*
  $Header: /home/elrond/baikal/src/CVS/daq98/shmbinarydatabuffer.h,v 1.3 1998/11/23 14:44:30 haase Exp $
*/

#if ! defined __SHMBINARYDATABUFFER_H__
#define __SHMBINARYDATABUFFER_H__

#include <config.h>
#include "binarydatabuffer.h"

/** Class that has all features of class {\tt \Ref{BinaryDataBuffer}} and can share
    its buffer with other processes using semaphores. */
class ShmBinaryDataBuffer: public BinaryDataBuffer
{
public:
     /** Constructor */
     ShmBinaryDataBuffer();
     
     /** Constructor
       @param Size Size of buffer to create.
       @param shmkeys Keys for for the shared memories (3).
       @param semkey Key for the buffer semaphore.
       @param Create Is true if we create the rescources, false if we only attach. */
     ShmBinaryDataBuffer(size_t size,key_t shmkeys[], key_t semkey, bool create);
     
     /** Destructor */
     virtual ~ShmBinaryDataBuffer();
     
     /** Creates buffer.
       @param Size Size of buffer to create.
       @param shmkeys Keys for for the shared memories (3).
       @param semkey Key for the buffer semaphore.
       @param Create Is true if we create the rescources, false if we only attach. 
       @return 0 on success -1 on failure.
     */
     inline int CreateBuffer(size_t Size,key_t shmkeys[], key_t semkey, bool create){
	  TEST_FAILURE(DataShm.Init(shmkeys[0],Size,create));
	  DataBuffer_begin=DataShm();
	  DataBuffer_write_end=DataBuffer_begin;
	  DataBuffer_pos=DataBuffer_begin;
	  DataBuffer_finish=DataBuffer_begin+Size;
	  TEST_FAILURE(DataBuffer_pos_Index.Init(shmkeys[1],sizeof(Position_t),create));
	  *(Position_t*)DataBuffer_pos_Index()=0;
	  TEST_FAILURE(DataBuffer_write_end_Index.Init(shmkeys[2],sizeof(Position_t),create));
	  *(Position_t*)DataBuffer_write_end_Index()=0;
	  Sem.Init(semkey,create);
	  
	  return 0;
     }      
     /** Gets buffer.
	 The function blocks till the buffer is released by
	 other processes that are using it.
	 @return 0 on success, -1 on failure.*/
     inline int Get(){
	  int r=Sem.Get();
	  Shm2Pos();
	  return r;
     }
     /** Releases buffer 
	 @return 0 on success, -1 on failure.*/
     inline int Release(){
	  Pos2Shm();
	  return Sem.Release();
     }
     	       
	       
protected:
     /** Copies current position pointers to their shared memory equivalents */
     inline void Pos2Shm(){
	  *(Position_t*)DataBuffer_pos_Index()=GetPosition();
	  *(Position_t*)DataBuffer_write_end_Index()=DataBuffer_write_end-DataBuffer_begin;
     }
     
     /** Copies current position pointers in shared memory to their local equivalents */
     inline void Shm2Pos(){
 	  DataBuffer_pos=DataBuffer_begin + *(Position_t*)DataBuffer_pos_Index();
 	  DataBuffer_write_end=DataBuffer_begin + *(Position_t*)DataBuffer_write_end_Index();
     }
     
private:
     /** Declared private to prevent user from using the base class function */
     inline void CreateBuffer(size_t /*Size*/){};
     
protected:
     /** Shared memory for data buffer */
     SharedMem DataShm;
     /** Position index, has to be in shared memory as well since it changes*/
     SharedMem DataBuffer_pos_Index;
     /** Write end index, has to be in shared memory as well since it changes*/
     SharedMem DataBuffer_write_end_Index;
     /** Buffer semaphore */
     Semaphore Sem;
};

#endif // __SHMBINARYDATABUFFER_H__
