/*
$Header: /home/elrond/baikal/src/CVS/daq98/simplevector.h,v 1.4 1998/11/23 14:44:32 haase Exp $
*/

#if !defined __SIMPLEVECTOR_H__
#define __SIMPLEVECTOR_H__

#include <config.h>
#include <sys/types.h>
#include <String.h>	/* *PN* 201197 */

template <class Element_t>
class SimpleVector
{
public:
     typedef Element_t *iterator;
     typedef Element_t  element_type;
     typedef Element_t  &reference_type;

     inline SimpleVector(size_t size_){
	  Size=size_;
	  elements=new Element_t[Size];
	  ActualSize=0;
     }
     inline ~SimpleVector(){
	  delete[] elements;
     }
     inline Element_t &operator[](size_t index){
	  return elements[index];
     }
     inline bool InRange(size_t index){
	  return Size>index;
     }       
     inline bool push_back(Element_t e){
	  if(!InRange(ActualSize)) return false;
	  elements[ActualSize]=e;
	  ActualSize++;
	  return true;
     }
     inline bool pop_back(Element_t &e){
	  ActualSize--;					// swapped these
	  if(!InRange(ActualSize)) return false;	// two lines
	  e=elements[ActualSize];
	  return true;
     }
     inline iterator begin(){
	  return elements;
     }
     inline iterator end(){
	  return elements+ActualSize;
     }
     inline size_t size(){
	  return ActualSize;
     }       
     inline void IncreaseSize(size_t incsize){
	  Element_t *newElements=new Element_t[Size+incsize];
	  iterator from_it=begin();
	  iterator to_it=newElements;
	  while(from_it<end()){
	       (*to_it++)=(*from_it++);
	  }
	  delete[] elements;
	  elements=newElements;
	  Size+=incsize;
     }
     inline void Clear(){
	  delete[] elements;
	  elements=new Element_t[Size];
	  ActualSize=0;
     }       
     inline void Erase(size_t index){
	  if(index>=ActualSize) return;
	  ActualSize--;
	  while(index<ActualSize){
	       elements[index]=elements[index+1];
	       index++;
	  }
     }
private:
     // dummy copy ctor and = operator, they are private, so the
     // compiler will not generate defaults and we can not use them
     /** copy ctor */
     SimpleVector(const SimpleVector& ){}
     /** assignment operator */
     const SimpleVector& operator=(const SimpleVector& s){return s;}
     Element_t *elements;
     size_t Size;
     size_t ActualSize;
};

#define MAKE_VECTOR(element_type,vector_type)\
typedef SimpleVector<element_type> vector_type

template <class Element_t>
class SimpleVector_SelfInc
{
     enum{
	  DEFAULT_SIZE=20,
	  DEFAULT_INC_SIZE=20
     };
public:
     typedef Element_t *iterator;
     typedef Element_t  element_type;
     typedef Element_t  &reference_type;
     
     inline SimpleVector_SelfInc(size_t size_){
	  Size=size_;
	  elements=new Element_t[Size];
	  ActualSize=0;
     }
     inline SimpleVector_SelfInc(){
	  Size=DEFAULT_SIZE;
	  elements=new Element_t[Size];
	  ActualSize=0;
     }
     inline ~SimpleVector_SelfInc(){
	  delete[] elements;
     }
     inline Element_t &operator[](size_t index){
	  return elements[index];
     }
     inline bool InRange(size_t index){
	  return Size>index;
     }       
     inline bool push_back(Element_t e){
	  if(!InRange(ActualSize)) IncreaseSize(DEFAULT_INC_SIZE);
	  elements[ActualSize]=e;
	  ActualSize++;
	  return true;
     }
     inline bool pop_back(Element_t &e){
	  ActualSize--;					// swapped these
	  if(!InRange(ActualSize)) return false;	// two lines
	  e=elements[ActualSize];
	  return true;
     }
     inline iterator begin(){
	  return elements;
     }
     inline iterator end(){
	  return elements+ActualSize;
     }
     inline size_t size(){
	  return ActualSize;
     }       
     inline void IncreaseSize(size_t incsize){
	  Element_t *newElements=new Element_t[Size+incsize];
	  iterator from_it=begin();
	  iterator to_it=newElements;
	  while(from_it<end()){
	       (*to_it++)=(*from_it++);
	  }
	  delete[] elements;
	  elements=newElements;
	  Size+=incsize;
     }  
     inline void Clear(){
	  delete[] elements;
	  elements=new Element_t[Size];
	  ActualSize=0;
     }       
     inline void Erase(size_t index){
	  if(index>=ActualSize) return;
	  ActualSize--;
	  while(index<ActualSize){
	       elements[index]=elements[index+1];
	       index++;
	  }
     }
protected:
     inline Element_t &GetElement(size_t index){
	  return elements[index];
     }     
private:
     // dummy copy ctor and = operator, they are private, so the
     // compiler will not generate defaults and we can not use them
     /** copy ctor */
     SimpleVector_SelfInc(const SimpleVector_SelfInc& ){}
     /** assignment operator */
     const SimpleVector_SelfInc& operator=(const SimpleVector_SelfInc& s){return s;}
private:
     Element_t *elements;
     size_t Size;
     size_t ActualSize;
};

#define MAKE_INC_VECTOR(element_type,vector_type)\
typedef SimpleVector_SelfInc<element_type> vector_type

/* some vectors of simple types */
MAKE_VECTOR (int, int_v);	/* *PN* 231097 */
MAKE_VECTOR (float, float_v);
MAKE_VECTOR (String, String_v);
MAKE_INC_VECTOR (int, int_inc_v);
MAKE_INC_VECTOR (float, float_inc_v);
MAKE_INC_VECTOR (String, String_inc_v); /* *PN* 201197 */


template <class Iterator,class Function>
inline void ForEach(Iterator begin, Iterator end, Function fn){
     while(begin!=end){
	  fn(begin++);
     }
}

template <class Iterator,class Function>
inline int ForEachCond(Iterator begin, Iterator end, Function fn){
     int ret;
     while(begin!=end){
	  if((ret=fn(begin++))!=0) return ret;
     }
     return 0;
}

template <class Iterator,class Function>
inline Iterator Find_If(Iterator first, Iterator last, Function fn){
     while ((first!=last)&&(!fn(*first))) ++first;
     return first;     
}

template <class Container,class Function>
inline void ForEachIn(Container &cont, Function fn){
     ForEach(cont.begin(),cont.end(),fn);
}

template <class Container,class Function>
inline int ForEachCondIn(Container &cont, Function fn){
     return ForEachCond(cont.begin(),cont.end(),fn);
}



#endif // __SIMPLEVECTOR_H__	       
