/*
$Header: /home/elrond/baikal/src/CVS/daq98/socketclass.C,v 1.8 1999/02/10 17:31:46 daq Exp $
*/

#include <config.h>
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <fcntl.h>

#include "globaltypes.h"
#include "globalmacros.h"
#include "error.h"
#include "datatypes.h"
#include "daqdebug.h"
#include "process.h"
#include "socketclass.h"

#if ! defined HAVE_BZERO
void bzero (char *s,int n){
     memset (s, '\0', n);
}
#endif

int SetCloseExec(filedescriptor_t desc){
     DEB_F("int SetCloseExec(..)");
     int oldflags;
     if(((oldflags=fcntl(desc,F_GETFD,0))<0)&&(errno!=EINTR)){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)="fcntl";
	  return -1;
     }
     oldflags|=FD_CLOEXEC;
     if((fcntl(desc,F_SETFD,oldflags)<0)&&(errno!=EINTR)){
	  (*Base::Error)=SYS_ERROR;
	  (*Base::Error)="fcntl";
	  return -1;
     }
     return 0;
}     

/*
Class Socket 
public member functions
*/


int Socket::SetSockOpt(int level,int optname, void *optval, int optlen){
     DEB_F("int Socket::SetSockOpt(...)");
     // only do this function for the below socket states
     if((SOS_CREATED==SocketState)||(SOS_BOUND==SocketState)||
	(SOS_LISTENING==SocketState)||(SOS_CONNECTED==SocketState))
	  // set the options
#ifdef __LYNX__
	  TEST_SYS_FAILURE(setsockopt(SD,level,optname,(char*)optval,optlen),"setsockopt");
#else
	  TEST_SYS_FAILURE(setsockopt(SD,level,optname,optval,optlen),"setsockopt");
#endif
     return 0;
}

int Socket::IsReadReady(int timeout){
     DEB_F("Socket::IsReadReady()");
     if((SOS_CONNECTED!=SocketState)&&(SOS_LISTENING!=SocketState)) return 0;
     fd_set read_fd_set;
     // set the flag for the specified descriptor 
     FD_ZERO(&read_fd_set);
     FD_SET(SD,&read_fd_set);
     // calculate the timeval structure content values from the timeout value 
     struct timeval to,*top=&to;
     if(timeout<0) top=NULL; 
     else {
	  to.tv_sec=timeout/1000;
	  to.tv_usec=1000*(timeout%1000);
     }
     int ret;
     /* do a select until error timeout or success, if errro was EINTR
        (interrupted by a system call) retry */
     do{
	  TEST_SYS_TEMP_FAILURE(
	       ret=select(FD_SETSIZE,&read_fd_set,NULL,NULL,top),"select");
	  if(ret<0){    // iterrupted by a signal
	       // did signal handler set quit flag ?
	       if(Quit_FLAG) return 0;
	  }
     } while(ret<0);
     // check if flag was set (timeout or readready)
     if(FD_ISSET(SD,&read_fd_set)) return 1;
     return 0;
}

int Socket::IsWriteReady(int timeout){
     DEB_F("Socket::IsWriteReady()");
     // see readready
     if((SOS_CONNECTED!=SocketState)&&(SOS_LISTENING!=SocketState)) return 0;
     fd_set write_fd_set;
     FD_ZERO(&write_fd_set);
     FD_SET(SD,&write_fd_set);
     struct timeval to,*top=&to;
     if(timeout<0) top=NULL; else {
	  to.tv_sec=timeout/1000;
	  to.tv_usec=1000*(timeout%1000);
     }
     int ret;
     do{
	  TEST_SYS_TEMP_FAILURE(
	       ret=select(FD_SETSIZE,NULL,&write_fd_set,NULL,top),"select");
	  if(ret<0){    // iterrupted by a signal
	       // did signal handler set quit flag ?
	       if(Quit_FLAG) return 0;
	  }
     } while(ret<0);
     if(FD_ISSET(SD,&write_fd_set)) return 1;
     return 0;
}

/*
Class Socket 
protected member functions
*/

int Socket::SetStdOptions(){  
     DEB_F("Socket::SetStdOptions()");
     // set the options SO_REUSEADDR and SO_KEEPALIVE
     int so=1;
     TEST_FAILURE(SetSockOpt(SOL_SOCKET,SO_REUSEADDR,&so,sizeof(so)));
     so=1;
     TEST_FAILURE(SetSockOpt(SOL_SOCKET,SO_KEEPALIVE,&so,sizeof(so)));
     return  0;
}  

int Socket::FillAddress(const char *host,int port,sockaddr_in &sa){
     DEB_F("int Socket::FillAddress(char *host,int port,sockaddr_in &sa)");
     int r=0;
     // zero the structure
     bzero((char*)&sa,sizeof(sa));  
     if(host==NULL){ 
	  // host=NULL -> use localhost
	  sa.sin_addr.s_addr=htonl(INADDR_ANY);
     }else{
	  // else determine the hostaddress from the hostname
	  struct hostent *hp;
	  if ((hp=gethostbyname(host))!=NULL){
	       sa.sin_addr=*(in_addr*) hp->h_addr;
	       r=0;
	  }else{
	       r=-1;
	  }
     }
     // fill in the remaining fields of the structure
     sa.sin_family=AF_INET;
     sa.sin_port=htons(port);
     if(r<0){
	  (*Error)=SYS_ERROR;
	  (*Error)="Socket::FillAddress (gethostbyname)";
     }
     return r;
}


int ServerSocket::Open(port_t port,int qs){
     DEB_F("void ServerSocket::Open(...)");
     sockaddr_in sa;
     TEST_FAILURE(FillAddress(NULL,port,sa));
     TEST_FAILURE(Create());
     TEST_FAILURE(Bind(sa));
     TEST_FAILURE(Listen(qs));
     TEST_FAILURE(SetStdOptions());
     LocalPort=port;
     return 0;
}

filedescriptor_t ServerSocket::Accept(port_t *localport=0){
     DEB_F("ServerSocket::Accept()");
     if(SOS_LISTENING!=SocketState) return 0;
     struct sockaddr_in sa;
     unsigned int i=sizeof(sa);
     int ret;
     TEST_SYS_FAILURE(ret=accept(SD,(sockaddr*)&sa,&i),"accept");
     if(localport!=0) *localport=ntohs(sa.sin_port);
     return ret;
}

port_t IOSocket::GetRemotePort(){
     sockaddr_in sa;
     unsigned int size_sa=sizeof(sockaddr_in);
     bzero((char*)&sa,sizeof(sa));
     TEST_SYS_FAILURE(getpeername(SD,(sockaddr*)&sa,&size_sa),"getpeername");
     return ntohs(sa.sin_port);
}

char *IOSocket::GetRemoteHostName(){
     sockaddr_in sa;
     unsigned int size_sa=sizeof(sockaddr_in);
     bzero((char*)&sa,sizeof(sa));
     if(getpeername(SD,(sockaddr*)&sa,&size_sa)<0){
	  (*Error)=SYS_ERROR;
	  (*Error)="getpeername";
	  return 0;
     }
     return inet_ntoa(sa.sin_addr);
}

int IOSocket::ReadText(char *t,int maxlen){
     DEB_F("IOSocket::ReadText(..)");
     // only read from connected sockets
     if(SOS_CONNECTED!=SocketState){ 
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="ReadText()";
	  return -1;
     }
     t[0]=0;
     int len=0;
     maxlen-=2;
     // created an input stream from the socket if it does not already exist
//     if(!isockstream) isockstream=new ifstream(SD);
     bool done=false;
     do{
	  // check wether it is possible to read from the stream
	  // errors might be eof, or broken connection etc.
	  TEST_FAILURE(CheckInStream());
	  /* read a line up to an "\n", the "\n" is discarded so we
             must append it after reading, but only if len >0 */
	  InputStream().getline(t+len,maxlen-len);
// #warning Hallo Heiko. Wenn die Textzeile laenger als maxlen ist, dann \
// wrid ja immer wieder getline mit maximaler Laenge von 0 ausgefuehrt \
// und da anschliessen len groesser 0 ist wird  t[len++]='\n' auch ausgefuehrt, d.h.\
// es wird immer ein Character zum String hinzugefuegt und der wird damit \
// laenger als maxlen. Ausserdem kommen wir nie aus dem while raus, da ja \
// auch kein "\r" kommt. Seh ich das falsch? Wenn nicht solltest Du das \
// ma aendern.

	  len=strlen(t);
	  if(len>0) t[len++]='\n';
	  t[len]=0;
	  DEB_M(<< "Text received :" << Quote(t)); 
	  /* read until we recieve a "\r\n" wich indicate the end of
             the transmission */
	  if(len>1 && t[len-2]=='\r' && t[len-1]=='\n')
	     done=true;
     }while(!done);
     return 0; 
}

int IOSocket::ReadText(String &txt){
     DEB_F("IOSocket::ReadText(..)");
     
     // only read from connected sockets
     if(SOS_CONNECTED!=SocketState){ 
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="ReadText()";
	  return -1;
     }
     static char tmp_txt[MAX_TEXT_SIZE];     
     txt="";
     tmp_txt[0]=0;
     int len;
     bool done=false;
     do{
	  // check wether it is possible to read from the stream
	  // errors might be eof, or broken connection etc.
	  TEST_FAILURE(CheckInStream());
	  /* read a line up to an "\n", the "\n" is discarded so we
             must append it after reading, but only if len >0 */
	  InputStream().getline(tmp_txt,MAX_TEXT_SIZE-1);
	  if(!InputStream().fail()){
	       len=strlen(tmp_txt);
	       if(len>0) tmp_txt[len++]='\n';
	       tmp_txt[len]=0;
	  } else {
	       // the last line was longer than MAX_TEXT_SIZE chars
	       InputStream().clear();
	  }
	  txt+=tmp_txt;
	  DEB_M(<< "Text received :" << Quote(*(const char*)txt)); 
	  /* read until we recieve a "\r\n" wich indicate the end of
             the transmission */
	  len=txt.length();
	  if(len>1 && txt[len-2]=='\r' && txt[len-1]=='\n'){
	       done=true;
	  }
     }while(!done);
     return 0; 
}

int IOSocket::WriteText(const char *t){
     DEB_F("IOSocket::WriteText(..)");
     // socket must be connected
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="WriteText()";
	  return -1;
     }
     TEST_FAILURE(IsWriteReady(0));
     // created an output stream from the socket if it does not already exist
//     if(!osockstream) osockstream=new ofstream(SD);
     // check wether it is possible to read from the stream
     TEST_FAILURE(CheckOutStream());
     // write to the output stream and flush it 
     OutputStream() << t << flush;
     return 0;
}

int IOSocket::ReadBin(DataBin *D){
     DEB_F("IOSocket::ReadBin(DataBin *D)");
     // only read from connected sockets
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="ReadBin()";
	  return -1;
     };
     long len=0,size;
     // first read the size in bytes of the data
     TEST_FAILURE(read(&len,sizeof(len)));
     // convert from net to host format
     size=ntohl(len);
     // Resiye the buffer such that the date will fit in the buffer
     D->Resize(size);
     // and read the data
     TEST_FAILURE(read(D->Buffer(),D->Size()));
     return 0;
}

int IOSocket::ReadBinI(BinaryDataBuffer &D){
     DEB_F("IOSocket::ReadBinI(BinaryDataBuffer *D)");
     // only read from connected sockets
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="ReadBinI()";
	  return -1;
     };
     D.Reset();
     size_t len=0,size;
     // first read the size in bytes of the data
     TEST_FAILURE(read(&len,sizeof(len)));
          
     // convert from net to host format
     size=ntohl(len);
     // check size of buffer
     if(size>=D.GetSizeOfBuffer()) {
	  cerr << "******" << size << "\n";
	  (*Error)=BUFFER_TOO_SMALL;
	  (*Error)="ReadIBin()";
 	  return -1;
     }
     TEST_FAILURE(read(D.GetDataBuffer(),size));
     D.SetSizeOfData(size);
     return 0;
}

int IOSocket::read(void *buf,size_t len){
     DEB_F("IOSocket::read(void *buf,int len)");
     int ret;
     size_t l=0;
     while(true){ //read until all data (len bytes) is read
	  TEST_SYS_TEMP_FAILURE(ret=(::read(SD,buf+l,len-l)),"read");
	  if(ret>0) if(len==(l+=ret)) break; // all bytes read ?
	  if(0==ret){
	       // failure
	       SocketState=SOS_DISCONNECTED;
	       (*Error)=SOCKETSTREAM_NOT_READABLE;
	       (*Error)="read()";
	       
	       return -1;
	  }
	  if(ret<0){    // iterrupted by a signal
	       // did signal handler set quit flag ?
	       if(Quit_FLAG) {
		    return -1;
	       }
	  }
     } 
     
     return len;
}

int IOSocket::WriteBin(DataBin *D){
     DEB_F("IOSocket::WriteBin(DataBin *D");
     // only write to a connected socket
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="WriteBin()";
	  return -1;
     }
     unsigned long len=htonl((unsigned long)D->Size());
     // write the siye of the data
     TEST_FAILURE(write(&len,sizeof(len)));
     // write the data
     TEST_FAILURE(write(D->Buffer(),D->Size()));
     return 0;
}

int IOSocket::WriteBinI(BinaryDataBuffer &D){
     DEB_F("IOSocket::WriteBinI(BinaryDataBuffer *D");
     // only write to a connected socket
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="WriteBinI()";
	  return -1;
     }
     size_t size, len=(size_t)D.GetSizeOfData();
//     cerr << "W " << len  << endl;
     if(len==0)return 0;
     size=htonl(len);
     // write the size of the data
     TEST_FAILURE(write(&size,sizeof(size)));
     // write the data
     TEST_FAILURE(write(D.GetDataBuffer(),len));
     
     return 0;
}

int IOSocket::WriteBinI(BinaryDataBuffer &D, size_t len){
     DEB_F("IOSocket::WriteBinI(BinaryDataBuffer *D");
     int ret=0;
     // only write to a connected socket
     if(SOS_CONNECTED!=SocketState){
	  (*Error)=SOCKET_NOT_CONNECTED;
	  (*Error)="WriteBinI()";
	  return -1;
     }
     if(D.GetPosition()==0) {
	  size_t size=htonl(len);
	  // write the size of the data
	  TEST_FAILURE(write(&size,sizeof(size)));
     }
     // write the data
     size_t leftover=D.GetPosition() - D.GetSizeOfData();
     if(leftover < len) {
	  len=leftover;
	  ret=1;
     }
     
     TEST_FAILURE(write(D.GetDataBuffer()+ D.GetPosition(),len));
     D.IncPosition(len);
     return ret;
}

// int IOSocket::write(const void *buf,size_t len){
//      DEB_F("int IOSocket::write(..)");
//      TEST_FAILURE(CheckOutStream());
//      OutputStream().write(buf,len);
//      if(OutputStream().fail()){
// 	  SocketState=SOS_DISCONNECTED;
// 	  (*Error)=SOCKETSTREAM_NOT_WRITEBLE;
// 	  (*Error)="write()";
// 	  return -1;
//      }
//      return len;	  
// }

int IOSocket::write(const void *buf,size_t len){
     DEB_F("int IOSocket::write(..)");
     int ret=0;
     size_t l=0;
     while(true){ // write all data (len bytes)
	  TEST_SYS_TEMP_FAILURE(ret=::write(SD,buf+l,len-l),"write");
	  if(ret>0) if(len==(l+=ret)) break; // all data written ?
	  if(0==ret){
	       cerr << "socket wrote 0; len=" << len << endl;
	       // failure
	       SocketState=SOS_DISCONNECTED;
	       (*Error)=SOCKETSTREAM_NOT_WRITEBLE;
	       (*Error)="write()";
	       return -1;
	  }
	  if(ret<0){    // iterrupted by a signal
	       // did signal handler set quit flag ?
	       if(Quit_FLAG) {
		    return -1;
	       }
	  }
     }
     return len;
}


// int ClientSocket::Open(port_type port){
//      DEB_F("void ClientSocket::Open(..)");
//      sockaddr_in sa;
//      TEST_FAILURE(FillAddress(NULL,port,sa));
//      TEST_FAILURE(Create());
//      TEST_FAILURE(Connect(sa));
//      return SetStdOptions();
// }

int ClientSocket::Open(const char *host,port_t port){
     DEB_F("void ClientSocket::Open(..)");
     sockaddr_in sa;
     TEST_FAILURE(FillAddress(host,port,sa));
     TEST_FAILURE(Create());
     TEST_FAILURE(Connect(sa));
     LocalPort=port;
     return SetStdOptions();
}














