#if ! defined __STATUSDISPL_H__
#define __STATUSDISPL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/statusdispl.h,v 1.6 1998/11/23 14:44:37 haase Exp $
*/

/*
  $Log: statusdispl.h,v $
  Revision 1.6  1998/11/23 14:44:37  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.5  1998/11/03 06:43:56  daq
  merged pamanda2 and pamanda3 versions

  Revision 1.4  1998/08/11 01:55:44  daq
  added aclocal.m4

  Revision 1.3  1998/07/31 07:51:48  daq
  StatusLine): added output to cerr as default
  (ResetTime): Added ResetTime function

  Revision 1.2  1998/07/27 23:59:12  steffen
  OSF specific changes

  Revision 1.1  1998/06/29 05:16:30  daq
  Added Files:
  	cursesstatusdispl.C cursesstatusdispl.h statusdispl.C
  	statusdispl.h

*/

#include <config.h>
#include "globaltypes.h"
#include <time.h>

/** A class that can display several status messages */
class StatusDisplay_t
{
public:
     /** Constructor.
       @param Update Time in seconds between updates.
     Default value is 0, which means always. */
     inline StatusDisplay_t(int Update);
     
     /** Destructor */
     inline virtual ~StatusDisplay_t();

     /** Initialize class */
     inline virtual int Init();
     
     /** Resets start time for ETA calculation */
     inline void ResetTime();
     
     /** Display a status line. */
     inline virtual int StatusLine(char * Text);
     
     /** Display progress message.
	 This function calls the virtual equivalent if it
	 is time for an update.
	 @param Percent Percentage
	 @param NumBytes Number of Bytes. If this is zero, no byte 
	 information will be displayed
	 @return 0 on success, -1 on failure */
     inline int Progress(float Percent, size_t NumBytes);

protected:     
     /** Display progress message.
	 @param Percent Percentage
	 @param NumBytes Number of Bytes. If this is zero, no byte 
	 information will be displayed
	 @return 0 on success, -1 on failure */
     inline virtual int __Progress(float Percent, size_t NumBytes);

     /** Convert a time to a string.
	 The time given in seconds is converted to a string
	 of the form {\tt ETA: hh::mm:ss}, where {\tt hh:} is 
	 omitted if the time given is less than an hour.
	 The function holds the converted time in a static string
	 and returns a refenrence to it.
	 @param Time Time in seconds
	 @return Time converted to a string.*/
     char *Time2ETAStr(time_t Time);

     /** Covert number of bytes to a string.
	 The function choses a convenient unit. Units from
	 bytes to Gbytes are supported.
	 The function holds the converted byte number in a static
	 string and returns a reference to it.
	 @param NumBytes Number of bytes
	 @param Number of bytes converted to a string*/
     char *Bytes2Str(unsigned int NumBytes);

protected:
     /** Start time for ETA calculation */
     time_t StartTime;

     /** Current time for ETA calculation */
     time_t CurTime;

     /** Time between updates */
     int Update;

     /** Time of last update */
     time_t LastUpdate;
};


StatusDisplay_t::StatusDisplay_t(int update)
{
     StartTime=time(NULL);
     LastUpdate=StartTime-update-1;
     Update=update;
}

StatusDisplay_t::~StatusDisplay_t()
{
}

int StatusDisplay_t::Init()
{
     return 0;
}

void StatusDisplay_t::ResetTime()
{
     StartTime=time(NULL);
     LastUpdate=StartTime-Update-1;
}

int StatusDisplay_t::StatusLine(char * Text)
{
     cerr << Text;
     return 0;
}
	
int StatusDisplay_t::Progress(float Percent, 
			      size_t NumBytes)
{
     if(((CurTime=time(NULL))-LastUpdate)>Update){
	  LastUpdate=CurTime;
	  return __Progress(Percent,NumBytes);
     }
     return 0;
}

int StatusDisplay_t::__Progress(float /* Percent*/, 
				size_t /* NumBytes */)
{
     return 0;
}


#endif // __STATUSDISPL_H__
