#if ! defined __TDCMODULEEVAL_H__
#define __TDCMODULEEVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/tdcmoduleeval.h,v 1.9 1999/02/10 17:31:46 daq Exp $
*/

#include <config.h>
#include "moduleevalbase.h"

class TDCModuleEvalClass_t: public ModuleEvalBaseClass_t
{
     ClassTypeInfo(ModuleEvalBaseClass_t);
public:
     inline TDCModuleEvalClass_t(int numChannels,int bytesPerInfoUnit,DaqProcessBase *Parent)
	  :ModuleEvalBaseClass_t(numChannels,bytesPerInfoUnit,Parent){}
     
     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &HWConfig){
	  SetTag(ModuleConfig->BinaryTag);
	  ChannelConfigRecordVector_t::iterator chcr_i;
	  for(chcr_i=HWConfig.ChannelConfigRecordVector->begin();
	      chcr_i<HWConfig.ChannelConfigRecordVector->end();
	      chcr_i++){
	       if((*chcr_i)->TDC_Id && (strcasecmp(ModuleConfig->ModuleID,(*chcr_i)->TDC_Id)==0)){
		    // we found a channel, that is connected to the current module
		    // insert it in our vector
		    SetChannelMapping((*chcr_i)->TDC_Channel,(*chcr_i)->ID);
	       }
	       
	  }
     }

     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
	  const void *ModuleData;
	  int NumReads;
	  u_i16 NumBytes;
	  int CurTDCChannel;
	  s_i32 CurOMChannel;
	  SubEventData_StaticSize_t *CurSubEvent;
	  
	  RawDataBuffer.ReadFromBuffer(NumBytes);
	  NumReads=NumBytes/BytesPerInfoUnit;

	  for(int i=0;i<NumReads;i++) {
	       ModuleData=ReadFromBuffer(RawDataBuffer);
	       // get the hit channel
	       CurTDCChannel=Channel(ModuleData);
	       if((CurOMChannel=GetChannelMapping(CurTDCChannel))<0){
		    // error handling, couldn't map channel
		    char txt[1000];
		    sprintf(txt,"Can't map TDC with tag %i channel %i to OM channel",GetTag(),CurTDCChannel);
		    LogMsg(txt,2,2,WARNING);
		    
		    continue;
	       }
	       CurSubEvent=TempEventBuffer->GetChannelForWriting(CurOMChannel);
	       CurSubEvent->ID=CurOMChannel;
	       if(CurSubEvent->NoOfEdges > MAX_TIME_HITS){
		    char txt[1024];
		    sprintf(txt,"Too many edges (%i) for OM %i for module %i",
			    CurSubEvent->NoOfEdges,CurOMChannel, Tag);
		    LogMsg(txt,2,2,WARNING);
	       }else{
		    CurSubEvent->Times[CurSubEvent->NoOfEdges].edge=Edge(ModuleData);
		    CurSubEvent->Times[CurSubEvent->NoOfEdges++].timeval=TimeVal(ModuleData);
	       }
	  }
	  return 0;
     }
protected:
     virtual inline int Channel(const void *tdc_data)=0;
     virtual inline int Edge(const void *tdc_data)=0;
     virtual inline int TimeVal(const void *tdc_data)=0;
};

#endif // __TDCMODULEEVAL_H__
