#include <qapp.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlcdnum.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qscrbar.h>

#include <String.h>

#include "item.h"
#include "simplevector.h"
#include "tvwidget.h"

/************************************************************************/
/*									*/
/* TableViewParams							*/
/*									*/
/************************************************************************/

TableViewParams::TableViewParams () : Lines (0), Columns (0), Title (""),
				Subtitle ("")
{
	static String Props[] = {"COLS", "LINS", "TIT", "STIT", "DATA"};
				//0       1       2      3       4

	SetProperties (sizeof (Props)/sizeof (String), Props);
}

TableViewParams::~TableViewParams ()
{
}

TableViewParams::TableViewParams (/*const*/ TableViewParams &SourceParams)
{
	Lines = SourceParams.Lines;
	Columns = SourceParams.Columns;
	Title = SourceParams.Title;
	Subtitle = SourceParams.Subtitle;


	while (Data.begin () != Data.end ())
	{
		FieldEntry_t trash;
		Data.pop_back (trash);
	}

	for (Data_t::iterator i = SourceParams.Data.begin ();
		SourceParams.Data.end () != i; ++i)
	{
		Data.push_back (*i);
	}
}

TableViewParams TableViewParams::operator = (/*const*/ TableViewParams
								&SourceParams)
{
	TableViewParams CopyBuf;

	CopyBuf.Lines = SourceParams.Lines;
	CopyBuf.Columns = SourceParams.Columns;
	CopyBuf.Title = SourceParams.Title;
	CopyBuf.Subtitle = SourceParams.Subtitle;

	for (Data_t::iterator i = SourceParams.Data.begin ();
			SourceParams.Data.end () != i; ++i)
	{
		CopyBuf.Data.push_back (*i);
	}

	return CopyBuf;
}

void TableViewParams::Clear ()
{
	FieldEntry_t trash;
	while (Data.begin () != Data.end ())
		Data.pop_back (trash);
}

int TableViewParams::SetPropertyByNo (int propnum, const String &val)
{
	String token[20];
	int ntoken;

	ntoken = split (val, token, 20, RXwhite);

	FieldEntry_t FE;
	switch (propnum)
	{
		case 0:		// COLS
			if (1 != ntoken) return -1;	// wrong # args
			if (!token[0].matches (RXint)) return -1;
			Columns = atoi ((const char *)token[0]);
			break;
		case 1:		// LINS
			if (1 != ntoken) return -1;	// wrong # args
			if (!token[0].matches (RXint)) return -1;
			Lines = atoi ((const char *)token[0]);
			break;
		case 2:		// TIT
			Title = join (token, ntoken, " ");
			break;
		case 3:		// STIT
			Subtitle = join (token, ntoken, " ");
			break;
		case 4:		// DATA = FieldTag, Data, Command
			if (3 > ntoken) return -1; 	// wrong # args
			if (!token[1].matches (RXint)) return -1;
			FE.FieldTag = token[0];
			FE.Data = atoi ((const char *)token[1]);
			FE.Command = join (token+2, ntoken - 2, " ");
			Data.push_back (FE);
			break;
		default:
			return -1;

	}
	return 0;
}

const String TableViewParams::GetPropertyByNo (int /*propnum*/) const
{
	return "";
}

const String TableViewParams::GetClassID ()
{
	return "TableViewParams";
}


/************************************************************************/
/*									*/
/* LCDRange								*/
/*									*/
/************************************************************************/

LCDRange::LCDRange (const char *FieldTag, int Value,
			QWidget *parent, const char *name)
		: QWidget (parent, name)
{
	label = new QLabel (FieldTag, this);
	label->setFrameStyle (QFrame::Panel | QFrame::Sunken);
	label->setFont (QFont ("Times", 16, QFont::Bold));
	label->setAlignment (AlignCenter);
	label->move (0,0);

	lcd = new QLCDNumber (4, this, "lcd");
	lcd->move (0, 26);		// 26 = 5 + 16 + 5
					// 5 pixel top margin,
					// 16 pixel label,
					// 5 pixel intermediate
	
	sBar = new QScrollBar (0, 2500,		// range
				1, 50,		// line, page steps
				Value,		// initial value
				QScrollBar::Horizontal,
						// orientation
				this, "scrollbar");

	InputLine = new QLineEdit (this, "inputline");

	lcd->display (sBar->value ());	// show current value
	setInputLine ();		// show current value

	// connect the scroll bar actions
	connect (sBar, SIGNAL (valueChanged (int)),
			lcd, SLOT (display (int)));
	connect (sBar, SIGNAL (valueChanged (int)),
			SLOT (setInputLine ()));
	connect (sBar, SIGNAL (valueChanged (int)),
			SIGNAL (valueChanged (int)));

	// connect the input line actions
	connect (InputLine, SIGNAL (returnPressed ()),
			SLOT (setValueFromInput ()));	
}

int LCDRange::currentValue () const
{
	return sBar->value ();
}

void LCDRange::setValue (int value)
{
	sBar->setValue (value);
}

void LCDRange::setValueFromInput ()
{
	int conversion;
	conversion = atoi (InputLine->text ());
	sBar->setValue (conversion);
	lcd->display (conversion);
}

void LCDRange::setInputLine ()
{
	char conversion[80];
	sprintf (conversion, "%d", sBar->value ());
	InputLine->setText (conversion);
}

void LCDRange::resizeEvent (QResizeEvent *)
{
	label->setGeometry (0, 5, width (), 16);
	lcd->setGeometry (0, 26,
		width (), height () - 5 - 16 - 5 - 5 - 16 - 5 - 20 - 5);
	sBar->setGeometry (0, lcd->y() + lcd->height () + 5, width (), 16);
	InputLine->setGeometry (0, sBar->y () + sBar->height () + 5,
		width (), 20);
}

/************************************************************************/
/*									*/
/* TableViewWidget							*/
/*									*/
/************************************************************************/

TableViewWidget::TableViewWidget (TableViewParams &tvp,
					QWidget *parent, const char *name)
		: QWidget (parent, name), TVP (tvp)
{
	setMinimumSize (TVP.Columns*120+20, TVP.Lines*104+300);
	setGeometry (100, 100, TVP.Columns*120+20, TVP.Lines*104+300);
	setPalette (QPalette (QColor (lightGray)));
				//    R   G    B

	setCaption ("TableView");

	// generate labels
	Title = new QLabel (TVP.Title, this);
	Subtitle = new QLabel (TVP.Subtitle, this);

	Title->setFrameStyle (QFrame::Panel | QFrame::Sunken);
	Subtitle->setFrameStyle (QFrame::Panel | QFrame::Sunken);

	Title->setAlignment (AlignCenter);
	Subtitle->setAlignment (AlignCenter);

	// generate buttons
	Apply = new QPushButton ("Apply", this, "Apply");
	Reset = new QPushButton ("Reset", this, "Reset");
	Clear = new QPushButton ("Clear", this, "Clear");
	Quit = new QPushButton ("Quit TV", this, "Quit");

	//  set up geometries
	Title->setGeometry (5, 5, width () - 10, 80);
	Subtitle->setGeometry (5, Title->y () + Title->height () + 5,
				width () - 10, 60);
	Apply->setGeometry (5, Subtitle->y () + Subtitle->height () + 5,
				width () / 2 - 10, 68);
	Reset->setGeometry (width () / 2 + 5,
				Subtitle->y () + Subtitle->height () + 5,
				width () / 2 - 10, 68);
	Clear->setGeometry (5, Apply->y () + Apply->height () + 5,
				width () / 2 - 10, 68);
	Quit->setGeometry (width () / 2 + 5,
				Apply->y () + Apply->height () + 5,
				width () / 2 - 10, 68);

	// set some nice colours
	Apply->setPalette (QPalette (QColor (green)));
	Reset->setPalette (QPalette (QColor (yellow)));
	Clear->setPalette (QPalette (QColor (white)));
	Quit->setPalette (QPalette (QColor (red)));

	// do the connections
	connect (Apply, SIGNAL (clicked ()), SLOT (SL_TableViewApply ()));
	connect (Reset, SIGNAL (clicked ()), SLOT (SL_TableViewReset ()));
	connect (Clear, SIGNAL (clicked ()), SLOT (SL_TableViewClear ()));
	connect (Quit, SIGNAL (clicked ()), SLOT (SL_TableViewQuit ()));

	cout << TVP.Lines << endl;
	cout << TVP.Columns << endl;
	cout << TVP.Title << endl;
	cout << TVP.Subtitle << endl;

	for (TableViewParams::Data_t::iterator i = TVP.Data.begin ();
		TVP.Data.end () != i; ++i)
	{
		cout << i->FieldTag << " " << i->Data;
		cout << " " << i->Command << endl;
	}

	for (int i = 0; i < TVP.Lines; ++i)
		for (int j = 0; j < TVP.Columns; ++j)
		{
			int index = i * TVP.Columns + j;
			value[index]
			= new LCDRange (TVP.Data[index].FieldTag,
					TVP.Data[index].Data,
					this);
		}

	show ();
}

TableViewWidget::~TableViewWidget ()
{
}

void TableViewWidget::SetParams (TableViewParams &tvp)
{
	TVP = tvp;		// sets the TableViewWidget's params

	show ();

	return;
}

char *TableViewWidget::getCommand ()
{
	return formatted;
}

void TableViewWidget::SL_TableViewApply ()
{
	for (int i = 0; i < TVP.Lines; ++i)
		for (int j = 0; j < TVP.Columns; ++j)
		{
			int index = i * TVP.Columns + j;
			sprintf (formatted,
				TVP.Data[index].Command,
				value[index]->currentValue ());
			emit SI_TableViewCmdReady ();
		}
}

void TableViewWidget::SL_TableViewReset ()
{
	for (int i = 0; i < TVP.Lines; ++i)
		for (int j = 0; j < TVP.Columns; ++j)
		{
			int index = i * TVP.Columns + j;
			value[index]->setValue (TVP.Data[index].Data);
		}
}

void TableViewWidget::SL_TableViewClear ()
{
	for (int i = 0; i < TVP.Lines; ++i)
		for (int j = 0; j < TVP.Columns; ++j)
		{
			int index = i * TVP.Columns + j;
			value[index]->setValue (0);
		}
}

void TableViewWidget::SL_TableViewQuit ()
{
	close ();
	emit SI_TableViewWasQuit ();
}

void TableViewWidget::resizeEvent (QResizeEvent *)
{
	Title->setGeometry (5, 5, width () - 10, 80);
	Subtitle->setGeometry (5, Title->y () + Title->height () + 5,
				width () - 10, 60);
	Apply->setGeometry (5, Subtitle->y () + Subtitle->height () + 5,
				width () / 2 - 10, 68);
	Reset->setGeometry (width () / 2 + 5,
				Subtitle->y () + Subtitle->height () + 5,
				width () / 2 - 10, 68);
	Clear->setGeometry (5, Apply->y () + Apply->height () + 5,
				width () / 2 - 10, 68);
	Quit->setGeometry (width () / 2 + 5,
				Apply->y () + Apply->height () + 5,
				width () / 2 - 10, 68);
	int valueWidth = (width () - 20)/TVP.Columns;
	int valueHeight = (height () - 300)/TVP.Lines;
	for (int i = 0; i < TVP.Lines; ++i)
		for (int j = 0; j < TVP.Columns; ++j)
		{
			int index = i * TVP.Columns + j;
			value[index]->setGeometry(
					10+(j%TVP.Columns)*valueWidth,
					300+(i%TVP.Lines)*valueHeight,
					valueWidth - 5,
					valueHeight - 5);
		}
}
