#ifndef __TVWIDGET_H_
#define __TVWIDGET_H_

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/tvwidget.h,v 1.3 1998/11/23 14:44:48 haase Exp $
*/

#include <config.h>
#include <qapp.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlcdnum.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qwidget.h>

#include <String.h>

#include "item.h"
#include "simplevector.h"


#define MAXLCDRANGE 100			// How many fields in TableViewWidget

/** A class for managing TableViewWidgets.
	This class manages the features of TableViewWidget such as title,
	subtitle and the data entry fields. */
class TableViewParams : public ItemWithProperties
{
	public:
		TableViewParams ();
		~TableViewParams ();

		TableViewParams (/*const*/ TableViewParams &SourceParams);
		TableViewParams operator = (/*const*/ TableViewParams
								&SourceParams);

		/** Tidy up the DataList.
			This is necessary if TableViewParams is instantized
			as static. */
		void Clear ();

		/** Do the right thing after the right keyword. 
			@param int propnum: number values + 1 (for the keyword)
			@param const String &val: keyword and values
			@return 0 on success, -1 on failure. */
		virtual int SetPropertyByNo (int propnum, const String &val);

		/** Not needed. */
		virtual const String GetPropertyByNo (int propnum) const;

		/** Return a unique ID.
			@return the claasname  */
		virtual const String GetClassID ();

		/** The number of lines of the entry field array. */
		int Lines;		
			
		/** the number of colums of the entry field array */
		int Columns;
			
		/** Title for the window */
		String Title;		
		/** Subtitle for the window */
		String Subtitle;

		/** Title, data and command template for each entry field */
		typedef struct
		{
			String FieldTag;
			int Data;
			String Command;
		} FieldEntry_t;

		/** Generate a vector type from FieldEntry_t */
		MAKE_INC_VECTOR (FieldEntry_t, Data_t);

		/** Vector to store data and titles for the entry field array.
			*/
		Data_t Data;
};

class QLabel;			// For some odd reason, QLCDNumber and
class QLCDNumber;		// QScrollBar have to bedefined, but
class QScrollBar;		// QLabel and QLineEdit not.
class QLineEdit;

/** A class as building blocl containing some display entities.
	It features a title, an LCD display, a scrollbar and an entry line. */

class LCDRange : public QWidget
{
	Q_OBJECT
	public:
		/** ctor */
		LCDRange (const char *FieldTag = 0, int Value = 0,
				QWidget *parent = 0, const char *name = 0);

		/** Get the current value of the LCDRange.
			@return the current value of the sBar object. */
		int currentValue () const;

	public slots:
		/** set the value of the sBar. */
		void setValue (int);
		/** move the value from the input line to both lcd and sBar. */
		void setValueFromInput ();
		/** set the value of the input line from sBar. */
		void setInputLine ();
	signals:
		/** generated on a changed sBar or input line. */
		void valueChanged (int);
	protected:
		void resizeEvent (QResizeEvent *);
	private:
		/** A label on top. */
		QLabel *label;
		/** show value in lcd-like fashion. */
		QLCDNumber *lcd;
		/** a scroll bar to change the value. */
		QScrollBar *sBar;
		/** an input line for decided calue changing. */
		QLineEdit *InputLine;
};

/** A class for displaying an array of entry fields .
	This offers an easy possibility to enter some values via a gui
	in an ,,spreadsheet'' like way. It is invoked from your application
	by sending a ,,cmd_tv\r\n'' to the moniclient, followed by commands 
	to fill the records of TableViewParams. */
class TableViewWidget : public QWidget
{
	Q_OBJECT
	public:
		TableViewWidget (TableViewParams &tvp,
				QWidget *parent = 0, const char *name = 0);

		~TableViewWidget ();

		/** Set the parameters for the window */
		void SetParams (TableViewParams &tvp);

		/** Read out the command line.
			@return the private member formatted. */
		char *getCommand ();	
	public slots:
		/** is called on pressing the apply button. */
		void SL_TableViewApply ();
		/** is called on pressing the reset button. */
		void SL_TableViewReset ();
		/** is called on pressing the clear button. */
		void SL_TableViewClear ();
		/** is called on pressing the quit button. */ 
		void SL_TableViewQuit ();	
	signals:
		/** announces that access to readCommand is sensible */
		void SI_TableViewCmdReady ();
		/** announces that TableView was quit */
		void SI_TableViewWasQuit ();
	protected:
		/** resize the thing. */
		void resizeEvent (QResizeEvent *);
	private:
		/** the paramters of the widget */
		TableViewParams &TVP;
		/** A label for the title. */
		QLabel *Title;
		/** A label for the Subtitle. */
		QLabel *Subtitle;
		/** A button for applying the current values. */
		QPushButton *Apply;
		/** A button for resetting the values to the initial ones. */
		QPushButton *Reset;
		/** A button for setting all values to zero. */
		QPushButton *Clear;
		/** A button to quit. */
		QPushButton *Quit;
		/** a range of LCDRange objects. */
		LCDRange *value[MAXLCDRANGE];

		/** a place where to store the command to be sent to
			moniclient. */
		char formatted[80];
};

#endif
