/*
$Header: /home/elrond/baikal/src/CVS/daq98/vmecontrol.C,v 1.4 1999/02/03 03:42:22 steffen Exp $
*/

// SYSTEM
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
// STL
#include "vectorinclude.h"
// 
#include "base.h"
#include "process.h"
#include "datatypes.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "dispatcherlib.h"
#include "vmecontrol.h"


int VMEControl_SignalHandler::Handle_SIGCHLD(Process &P){
     DEB_F("int DC_SignalHandler::Handle_SIGCHLD(..)");
     int pid=SignalHandlerClass::Handle_SIGCHLD(P);
     return pid;
}

int VMEControl_SignalHandler::Handle_SIGUSR1(Process &){
     DEB_F("int DC_SignalHandler::Handle_SIGUSR1(..)");
     return 0;
}

/* ctor */
VMEControlDaemon::VMEControlDaemon():DaemonBase(){
     DEB_F("VMEControlDaemon::VMEControlDaemon()");

     // fill the command interpreter structur 
     IPC_CMD_Interpreter.AddFunction("start",&IPC_CMD_StartProcess,
				"start name     -- starts the process name");
     IPC_CMD_Interpreter.AddFunction("stop",&IPC_CMD_StopProcess,
				"stop name     -- starts the process name");
     IPC_CMD_Interpreter.AddFunction("help",&IPC_CMD_Help,
				"help [COMMAND] -- prints help for COMMAND or for all commands");

     IPC_CMD_Interpreter.DefaultFunction(&IPC_CMD_UnknownCommand);
     IPC_CMD_Interpreter.WrongTypeFunction(&IPC_CMD_WrongTypeCommand);

     // set the signal handler
     ThisProcess->SetSignalHandler(SignalHandler);
     // some default values
     VMEControlPort=DEFAULT_VMECONTROL_PORT;
}

/** dtor */
VMEControlDaemon::~VMEControlDaemon(){
     DEB_F("VMEControlDaemon::~VMEControlDaemon()");
}

/** initialization */
int VMEControlDaemon::Init(){
     DEB_F("VMEControlDaemon::Init()");

     TEST_FAILURE(ReadConfigFile());
     IPCAddress.Port=VMEControlPort;
     IPCAddress.Hostname=LocalHostname;
     TEST_FAILURE(DaemonBase::Init());
     pid.pid=getpid();
     pid.Hostname=LocalHostname;
     pid.ProcessID=GetProcessType();
     IPCAddress=Dispatcher.Register(pid,0);
     TEST_FAILURE(IPCAddress.Port);
     /* open all sockets where we have to listen on and set the close
      on exec flag, such that the sockets will be closed when a new
      process is excecuted after a fork 
      
      and set the notifier objects for all sockets and add them to the
      selecter*/
     ServerPortVector_t::iterator i=ServerPortVector.begin();
     while(i<ServerPortVector.end()){
	  TEST_FAILURE((*i)->SSocket.Open((*i)->port,10));
	  TEST_FAILURE((*i)->SSocket.SetCloseExec());
	  (*i)->SSocketNotifier.Set((*i)->SSocket,IONotifier::READ);
	  Selecter.AddNotifier((*i)->SSocketNotifier);
	  i++;
     }
     return 0;
}

/** destruction */
int VMEControlDaemon::Destroy(){
     // no need to destroy the sockets, its done by themself
     DEB_F("VMEControlDaemon::Destroy()");
     DaemonBase::Destroy();
     Dispatcher.Unregister(pid);
     return 0;
}

/** the event dispatcher */
int VMEControlDaemon::DispatchEvent(){
     // call the dispatcher of the base class
     TEST_FAILURE(DaemonBase::DispatchEvent());
     // check out all other sockets 
     
     ServerPortVector_t::iterator spmi=ServerPortVector.begin();
     while(spmi<ServerPortVector.end()){
	  if(Selecter.CheckNotifier((*spmi)->SSocketNotifier))
	       // yes -> start the corresponding server program
	       if(StartSlaveServer((*spmi)->servername,&((*spmi)->SSocket))<0){
		    // failure
		    // just do not care now
	       }
	  spmi++;
     } 
     return 0;
}

/** the IPC event handler */
int VMEControlDaemon::DoIPCEvent(const String &Cmd,SlaveServerSocket &s){
     return IPC_CMD_Interpreter.Do_wo_type(this,Cmd,s);
}

/** returns a String with the configuration file name */
const String VMEControlDaemon::GetConfigFileName()const{
     return  VMECONTROLDAEMON_CNF_FILENAME;   // from globalmacros.h
}

/** returns a pointer to the configuration class, created with
    new on the heap, must be overwritten in derived class */
ClassWithTypeInfo_t *VMEControlDaemon::GetConfigSection(){
     return new class VMEControlDaemonConfig:public ConfigSection_t{
     public:
	  /** ctor */
	  VMEControlDaemonConfig(VMEControlDaemon::ServerPortVector_t &serverportvector,
				 port_t &vmecontrolport)
	       :ConfigSection_t(),
		ServerPortVector(serverportvector),
		VMEControlPort(vmecontrolport){}
	  BeginItems(ConfigSection_t){
	       BeginActions{
		    ItemAction_SET("Server",
			       CurServPorMap=new VMEControlDaemon::ServerPortMapper_t;
			       ServerPortVector.push_back(CurServPorMap);
			 );
	       }EndActions;
	       BeginLinks{
		    ItemLink2("Server",CurServPorMap->servername,CurServPorMap->port);
		    ItemLink1("VMEControlPort",VMEControlPort);
	       }EndLinks;
	  }EndItems(ConfigSection_t);
	  SectionName(VMECONTROLDAEMON_SECTION_NAME);
     private:
	  VMEControlDaemon::ServerPortMapper_p CurServPorMap;
	  VMEControlDaemon::ServerPortVector_t &ServerPortVector;
	  port_t &VMEControlPort;
     }(ServerPortVector,VMEControlPort);
}

/** returns the type of the process */
const String VMEControlDaemon::GetProcessType()const{
     return PROCESS_TYPE_VMECONTROL; // whatever
}

int VMEControlDaemon::StartSlaveServer(const char* Progname,ServerSocket *SS){
     DEB_F("VMEControlDaemon::StartSlaveServer(..)");
     int r;
     // pass a socket to the child process -> SS!=NULL
     filedescriptor_t SocketDesc=0;
     char sockstr[20];
     String Path = String("./") + Progname;
     if(SS){
	  TEST_FAILURE(SocketDesc=SS->Accept());
	  sprintf(sockstr,"%i",SocketDesc);
     }
     TEST_FAILURE(r=ThisProcess->Fork());
     if(r==0){  // child process
	  if (SS) execlp((const char*)Path,(const char*)Path,sockstr,NULL);
	  else execlp((const char*)Path,(const char *)Path,NULL);
	  exit(EXIT_FAILURE);
	  //Quit_FLAG=true;
     } else { // parent
	  if(SS) close(SocketDesc);
	  // wait for process to send the SIGUSR1 signal, to indicate,
	  // that it is running, if not, the signal was SIGCHLD, than
	  // starting failed
	  pause(); 
	  if (ThisProcess->GetSignalHandler().LastSignal()!=SIGUSR1) return -1;
     }
     return r;
}

int VMEControlDaemon::StopProcess(String &ProcessID)
{
     pid_t ID=Dispatcher.GetPID(ProcessID);
     cerr << "VC pid=" << ID << endl;
     
     if(ID==0){
	  cerr << "ID not found\n";
	  return -1;
     }
     
     char cmd[256];
     sprintf(cmd,"kill -TERM %d",ID);
     TEST_SYS_FAILURE(system(cmd),"system");
     return 0;
}


int VMEControlDaemon::IPC_CMD_StartProcess(IOSocket &S,String *param,int numparams){
     DEB_F("int VMEControlDaemon::StartProcess(..)");
     if(numparams!=2) return SendUsageInfo(S,param[0]);
     S.SetCloseExec();
     String short_name;
     if((compare(param[1],"DCO")==0)|| 
	(compare(param[1],"datacollector")==0)){
	  param[1]="datacollector";
	  short_name="DCO";
     } else if((compare(param[1],"DT")==0)||
	(compare(param[1],"datatransport")==0)){
	  param[1]="datatransport";
	  short_name="DT";
     } else {
	  TEST_FAILURE(S.WriteText("cmd_error\r\nno such process: '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
	  return 0;
     }
     PortLocker PL(Dispatcher,short_name,USER_DAQ_INTERN);     
     IPAddress_t IPAddr=PL.GetAddress();
     if(IPAddr.Port>=0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\nprocess already running: '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
	  return 0;
     }

     if(StartSlaveServer((const char*)param[1],NULL)<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\ncould not start process '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
     } else{
	  TEST_FAILURE(S.WriteText("cmd_info\r\nprocess '")); 
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("' started\r\n"));
     }
     
     return 0;
}

int VMEControlDaemon::IPC_CMD_StopProcess(IOSocket &S,String *param,int numparams){
     DEB_F("int VMEControlDaemon::IPC_CMD_StopProcess(..)");
     cerr << "***** IPC_CMD_StopProcess\n";
     
     if(numparams!=2) return SendUsageInfo(S,param[0]);
     S.SetCloseExec();
     if(StopProcess(param[1])<0){
	  TEST_FAILURE(S.WriteText("cmd_error\r\ncould not stop process '"));
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("'\r\n"));
     } else{
	  TEST_FAILURE(S.WriteText("cmd_info\r\nprocess '")); 
	  TEST_FAILURE(S.WriteText(param[1]));
	  TEST_FAILURE(S.WriteText("' stopped\r\n"));
     }
     
     return 0;
}


int VMEControlDaemon::IPC_CMD_Help(IOSocket &s,String *param,int numparams){
     DEB_F("int VMEControlDaemon::CmdHelp(..)");
     String Help;
     if(numparams>2) return SendUsageInfo(s,param[0]);
     if(numparams>1)
	  IPC_CMD_Interpreter.GetHelp(param[1],Help);
     else
	  IPC_CMD_Interpreter.GetHelp(Help);
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     TEST_FAILURE(s.WriteText(Help));
     return 0;
}

int VMEControlDaemon::IPC_CMD_UnknownCommand(IOSocket &S,String */*param*/,int /*numparams*/){
     DEB_F("int VMEControlDaemon::UnknownCommand(..)");
     TEST_FAILURE(S.WriteText("cmd_unknown\r\nVMEControl: unknown command\r\n"));
     return 0;
}

int VMEControlDaemon::IPC_CMD_WrongTypeCommand(IOSocket &/*S*/,String */*param*/,int){
     DEB_F("int VMEControlDaemon::WrongTypeCommand(..)");
     return 0;
}

int VMEControlDaemon::SendUsageInfo(IOSocket &s,String param){
     DEB_F("int VMEControlDaemon::SenUsageInfo(..)");
     String Help;
     IPC_CMD_Interpreter.GetHelp(param,Help);
     Help="Usage: "+Help;
     TEST_FAILURE(s.WriteText("cmd_info\r\n"));
     return s.WriteText(Help);
}


int main(int /*argc*/,char **/*argv*/){
     DEB_F("main()");
     DaqProcessBase::InitializeGlobals("vmecontrol");

     LogFile LF(DaqProcessBase::ThisProcess->GetName());					
     Logger::SetLogger(&LF);				

     if(DaqProcessBase::ThisProcess->LockProcess()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
	  return 1;
     }
     VMEControlDaemon DaemonServer;
     if(DaemonServer.Init()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     } else
     if(DaemonServer.EventLoop()<0){
	  Logger Entry;
	  (*Base::Error).Print(Entry());
     }
     DaemonServer.Destroy();
     DaqProcessBase::ThisProcess->UnlockProcess();
     return 0;
}













