/*
  $Header: /home/elrond/baikal/src/CVS/daq98/vmedaqhal.C,v 1.11 1999/02/10 21:35:14 daq Exp $
*/

#include <config.h>
#include <strstream.h>
#include "vmedaqhal.h"

#define __HWACCESS__
#include "register.h"

#define CRATE_VECTOR_SIZE           10
#define MODULE_VECTOR_SIZE          300

HAL_t::HAL_t(DaqProcessBase *parent){
     Parent=parent;
     CrateVector=  new CrateVector_t (CRATE_VECTOR_SIZE);
     ModuleVector= new ChannelModuleVector_t(MODULE_VECTOR_SIZE);
     TriggerModule=0;         
#if ! defined DUMMY_MODULES_ONLY
     CORBO=new CORBOModule;
#endif
}

HAL_t::~HAL_t(){
     delete CrateVector;
     delete ModuleVector;
#if ! defined DUMMY_MODULES_ONLY
     delete CORBO;
#endif
}

int HAL_t::InsertCrates(CrateConfigRecordVector_t &CrateConfig,
			   StringVector_t &errors){
     DEB_F("HAL_t::InsertCrates()");
     CrateConfigRecordVector_t::iterator cc;
     int err=0;
     for(cc=CrateConfig.begin();cc<CrateConfig.end();cc++){
	  // first get the type of the crate
	  String CrateType=(*cc)->CrateType;
	  // now create it using the CrateFactory and the CrateType
	  // string to create the right class
	  CrateClass_p Crate=CreateCrateClass(CrateType);
	  
	  // was it created or do we have an error ?
	  if(0==(void*)Crate){
	       ostrstream err_s;
	       err_s << "Crate"
		     << " Type: " << CrateType
		     << " ID: " << (*cc)->CrateID
		     << " -- "
		     << "Structure could no be created\r\n" << ends;
	       err_s.freeze(1);	  
	       errors.push_back(String(err_s.str()));
	       err_s.freeze(0);
	       
	       err--;
	       continue;
	  }
	  
	  //  now set the config parameters of the crate
	  Crate->SetCrateConfig(*cc);

	  // insert it into the crate vector
	  CrateVector->push_back(Crate);
     }
     return err;
}

int HAL_t::InsertModules(ChannelModuleConfigRecordVector_t &ModuleConfig,
			 StringVector_t &errors){
     DEB_F("HAL_t::InsertModules()");
     ChannelModuleConfigRecordVector_t::iterator mc;
     int err=0;
     for(mc=ModuleConfig.begin();mc<ModuleConfig.end();mc++){
	  // get the Module Type
	  String ModuleType=(*mc)->ModuleType;
	  // now create a class for the Module
	  ChannelModuleBaseClass_p Module=CreateModuleClass(ModuleType,Parent);
	  // check if someone uses the Tag of the new module
	  binary_tag_t Tag=(*mc)->BinaryTag;
	  for(ChannelModuleVector_t::iterator i=ModuleVector->begin();
	      i<ModuleVector->end();i++){
	       if(Tag==(*i)->GetModuleConfig()->BinaryTag){
		    ostrstream err_s;
		    err_s << "Module ID: " 
			  << (*mc)->ModuleID
			  << " Type: " << ModuleType
			  << " -- "
			  << "Binary tag already used by another module\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    break;
	       }
	  }
		    
	  // was it created or do we have an error ?
	  if(!Module){
	       ostrstream err_s;
	       err_s << "Module ID: " 
		     << (*mc)->ModuleID
		     << " Type: " << ModuleType
		     << " -- "
		     << "Structure for Module could no be created\r\n" << ends;
	       err_s.freeze(1);	  
	       errors.push_back(String(err_s.str()));
	       err_s.freeze(0);
	       
	       err--;
	       continue;
	  }
	  
	  // look for the Crate the ADC is place in
	  String CrateID=(*mc)->CrateID;
	  CrateClass_p Crate=FindCrate(CrateID);
	  // does the Crate exist ?
	  if(!Crate){
	       ostrstream err_s;
	       err_s << "Module ID: " 
		     << (*mc)->ModuleID
		     << " Type: " << ModuleType
		     << " -- "
		     << "Crate "
		     << "ID: " << CrateID 
		     << " does not exist\r\n" << ends;
	       err_s.freeze(1);	  
	       errors.push_back(String(err_s.str()));
	       err_s.freeze(0);

	       err--;
	       continue;
	  }

	  // check if someone else already uses our slot
	  if(Crate->IsAddress_SlotReserved((*mc)->Address_Slot)){
	       //yes
	       ostrstream err_s;
	       err_s << "Module ID: " 
		     << (*mc)->ModuleID
		     << " Type: " << ModuleType
		     << " -- "
		     << "Slot/address " << (*mc)->Address_Slot
		     << " already used by another Module\r\n" << ends;
	       err_s.freeze(1);	  
	       errors.push_back(String(err_s.str()));
	       err_s.freeze(0);
	       err--;
	       continue;
	  }
	  Crate->ReserveAddress_Slot((*mc)->Address_Slot);
	  // set the configuration
	  Module->SetModuleConfig(*mc);
	  // insert the module into the crate
	  if((*mc)->isAsynch!=0){
	       if((*mc)->isAsynch!=111) Crate->InsertModule_Asynch(Module);
	       else Crate->InsertModule_DMAD(Module);
	  } else {
	       Crate->InsertModule(Module);
	  }
	  // insert it into the Module vector
	  ModuleVector->push_back(Module);
     }
     return err;     
}


int HAL_t::InsertChannels(ChannelConfigRecordVector_t &ChannelConfig,
			  StringVector_t &errors){
     DEB_F("HAL_t::InsertChannels()");
     ChannelConfigRecordVector_t::iterator cc;
     int err=0;
     bool OpenGroup=false;
     
     ChannelVector_t ChannelIDVector;
     ChannelGroup_t CurGrp;
     
     ChannelVector_t::iterator PMTsUsed;
     for(cc=ChannelConfig.begin();cc!=ChannelConfig.end();cc++){
	  channel_t ChannelID;
	  
	  // Pointer to the module class where we find our ADC value
	  ADCModuleClass_p ADC_Module;
	  // Channel of the ADC module where we find the ADC value
	  u_i8 ADC_Channel;
	  // Pointer to the channel where we find the TDC values
	  TDCModuleClass_p TDC_Module;
	  // Channel of the TDC modukle where we find our TDC values
	  u_i8 TDC_Channel;
	  // Pointer to the channel where we find the Scaler values
	  SCALERModuleClass_p SCALER_Module;
	  // Channel of the TDC modukle where we find our Scaler values
	  u_i8 SCALER_Channel;
	  
	  // look for the OM channel ID and store it 
	  ChannelID=(*cc)->ID;
	  // Does PMT Channel already exist?
	  for(PMTsUsed=ChannelIDVector.begin();PMTsUsed<ChannelIDVector.end();PMTsUsed++){
	       if(ChannelID==(*PMTsUsed)){
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " PMT Channel already exists\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    err--;
		    continue;
	       }
	  }
	  ChannelIDVector.push_back(ChannelID);

	  // Check for group statemenets	       
	  if((BEGIN_END_t)(*cc)->Group==BEGIN_val){
	       if(OpenGroup){
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " Nested 'GROUP BEGIN' statement\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    err--;
		    continue;
	       }
	       OpenGroup=true;
	       CurGrp.start=ChannelID;
	  }
	  if((BEGIN_END_t)(*cc)->Group==END_val){
	       if(!OpenGroup){
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " No 'GROUP BEGIN' statement for 'GROUP END'\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    err--;
		    continue;
	       }
	       if(CurGrp.start>ChannelID){
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "'GROUP_END' channel ID is higher than 'GROUP_START' channel ID\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    err--;
		    continue;
	       }
//	       CurGrp.end=(*cc)->ChannelID;
	       OpenGroup=false;
	  }
	  
	  // look for the ADCModule the Channel is connected to
	  
	  if((*cc)->ADC_Id){
	       String ADCModuleID=(*cc)->ADC_Id;
	       ADC_Module=TYPESAFE_DOWNCAST_RCP(ADCModuleClass_t,FindModule(ADCModuleID));
	       // does the Module exist ?
	       if(!ADC_Module){
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " ADC Module ID: " << ADCModuleID
			  << " does not exist for PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    err--;
		    continue;
	       }
	       // set the ADC channel No of this channel
	       ADC_Channel=(*cc)->ADC_Channel;
	       // check if the ADC channel is a valid number
	       if(ADC_Channel >= ADC_Channel>=ADC_Module->GetNoOfChannels()){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "ADC Channel " << (*cc)->ADC_Channel
			  << " of Module " << ADCModuleID
			  << " is no a vaild Channel Number\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }
	       
	       // check if someone else already uses our ADC Channel
	       if(ADC_Module->IsChannelInUse(ADC_Channel)){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "ADC Channel " << (*cc)->ADC_Channel
			  << " of Module " << ADCModuleID
			  << " already used by another PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }
	       ADC_Module->SetChannelInUse(ADC_Channel);
	       if((*cc)->ADC_On) ADC_Module->SetChannelOn(ADC_Channel);
	  }
	  
	  
	  if((*cc)->TDC_Id) {
	       String TDCModuleID=(*cc)->TDC_Id;
	       // look for the TDCModule the Channel is connected to
	       TDC_Module=TYPESAFE_DOWNCAST_RCP(TDCModuleClass_t,FindModule(TDCModuleID));
	       // does the Module exist ?
	       if(!TDC_Module){
		    err--;
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " TDC Module ID: " << TDCModuleID
			  << " does not exist for PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    continue;
	       }

	       // set the TDC channel No of this channel
	       TDC_Channel=(*cc)->TDC_Channel;
	       // check if the ADC channel is a valid number
	       if(TDC_Channel >= TDC_Channel>=TDC_Module->GetNoOfChannels()){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "TDC Channel " << (*cc)->TDC_Channel
			  << " of Module " << TDCModuleID
			  << " is no a valid Channel Number\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }

	       // check if someone else already uses our TDC Channel
	       if(TDC_Module->IsChannelInUse(TDC_Channel)){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "TDC Channel " << (*cc)->TDC_Channel
			  << " of Module " << TDCModuleID
			  << " already used by another PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }
	       TDC_Module->SetChannelInUse(TDC_Channel);
	       if((*cc)->TDC_On) TDC_Module->SetChannelOn(TDC_Channel);
	  }
	  if((*cc)->SCALER_Id) {
 	       String SCALERModuleID=(*cc)->SCALER_Id;
	       // look for the ScalerModule the Channel is connected to
	       SCALER_Module=TYPESAFE_DOWNCAST_RCP(SCALERModuleClass_t,FindModule(SCALERModuleID));
	       // does the Module exist ?
	       if(!SCALER_Module){
		    err--;
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << " Scaler Module ID: " << SCALERModuleID
			  << " does not exist for PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    continue;
	       }

	       // set the Scaler channel No of this channel
	       SCALER_Channel=(*cc)->SCALER_Channel;
	       // check if the ADC channel is a valid number
	       if(SCALER_Channel >= SCALER_Channel>=SCALER_Module->GetNoOfChannels()){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "Scaler Channel " << (*cc)->SCALER_Channel
			  << " of Module " << SCALERModuleID
			  << " is no a valid Channel Number\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }

	       // check if someone else already uses our Scaler Channel
	       if(SCALER_Module->IsChannelInUse(SCALER_Channel)){
		    //yes
		    ostrstream err_s;
		    err_s << "PMT Channel ID: " 
			  << (*cc)->ID
			  << " -- "
			  << "Scaler Channel " << (*cc)->SCALER_Channel
			  << " of Module " << SCALERModuleID
			  << " already used by another PMT Channel\r\n" << ends;
		    err_s.freeze(1);	  
		    errors.push_back(String(err_s.str()));
		    err_s.freeze(0);
		    
		    err--;
		    continue;
	       }
	       SCALER_Module->SetChannelInUse(SCALER_Channel);
	       if((*cc)->SCALER_On) SCALER_Module->SetChannelOn(SCALER_Channel);
	  }
     }
     return err;     
}

int HAL_t::ClearStructure(){
     DEB_F("HAL_t::ClearStructure()");
     delete CrateVector;
     delete ModuleVector;

     CrateVector=  new CrateVector_t(CRATE_VECTOR_SIZE);
     ModuleVector= new ChannelModuleVector_t(MODULE_VECTOR_SIZE);

     return 0;
}

CrateClass_p HAL_t::CreateCrateClass(const char *Type){
     DEB_F("HAL_t::CreateCrateClass()");
     return CrateCreator(CrateBaseClass_t,Type);  // macro from register.h
}

ChannelModuleBaseClass_p HAL_t::CreateModuleClass(const char *Type,DaqProcessBase *Parent){ /***/
     DEB_F("HAL_t::CreateModuleClass()");
     return ModuleCreator(ChannelModuleBaseClass_t,Type,Parent); // macro from register.h
}


/** searches the module list for the Crate class with the ID
    Crate_ID */
const CrateClass_p HAL_t::FindCrate(const String &CrateID){
     DEB_F("HAL_t::FindCrate()");
     CrateVector_t::iterator cp;
     for(cp=CrateVector->begin();cp<CrateVector->end();cp++){
	  if(fcompare(CrateID,(*cp)->GetCrateConfig()->CrateID)==0) return (*cp);
     }  
     return CrateClass_p(0);
}

/** searches the module list for the Module class with the ID
    Module_ID */
const ChannelModuleBaseClass_p HAL_t::FindModule(const String &ModuleID){
     DEB_F("HAL_t::FindModule()");
     ChannelModuleVector_t::iterator mp;
     for(mp=ModuleVector->begin();mp<ModuleVector->end();mp++){
	  if(fcompare(ModuleID,(*mp)->GetModuleConfig()->ModuleID)==0) return (*mp);
     }  
     return ChannelModuleBaseClass_p(0);
}


int HAL_t::InitCrates(CrateVector_t &Crates, StringVector_t & /* errors */){
     DEB_F("HAL_t::InitModules()");
     int err=0;
     CrateVector_t::iterator ci;
     
     for(ci=Crates.begin();ci<Crates.end();ci++){
	  (*ci)->Init();
     }
     return err;
}

/** reads out the modules, separate for every crate, one crate after an other */
int HAL_t::ReadoutModules_Cratewise(CrateVector_t &Crates,WhichModuleType_t which,BinaryDataBuffer &DataBuffer){
     DEB_F("HAL_t::ReadoutModules_Cratewise()");
     CrateVector_t::iterator ci;
     if(which==EVENT_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       (*ci)->ReadoutModules(DataBuffer);
	  }
     } else if(which==ASYNCH_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       (*ci)->ReadoutModules_Asynch(DataBuffer);
	  }
     }  else if(which==DMAD_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       (*ci)->ReadoutModules_DMAD(DataBuffer);
	  }
     }
     DataBuffer.WriteToBuffer(END_OF_EVENT_TAG);
     return 0;
}
     
/** initializes the modules, separate for every crate, one crate after an other */
int HAL_t::ResetModules_Cratewise(CrateVector_t &Crates,WhichModuleType_t which, StringVector_t &errors){
     DEB_F("HAL_t::InitModules_Cratewise()");
     int err=0;
     CrateVector_t::iterator ci;
     if(which==EVENT_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       err+=(*ci)->ResetModules(errors);
	  }

	  err+=CORBO->ResetModule(errors);              // handle error to be done ! 
     } else if(which==ASYNCH_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       err+=(*ci)->ResetModules_Asynch(errors);
	  }
     }
     return 0;
}

/** initializes the modules, separate for every crate, one crate after an other */
int HAL_t::StartModules_Cratewise(CrateVector_t &Crates,WhichModuleType_t which, StringVector_t &errors){
     DEB_F("HAL_t::InitModules_Cratewise()");
     int err=0;
     CrateVector_t::iterator ci;
     if(which==ASYNCH_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       err+=(*ci)->StartModules_Asynch(errors);
	  }
     }
     return 0;
}

int HAL_t::InitModules_Cratewise(CrateVector_t &Crates,WhichModuleType_t which, StringVector_t &errors){
     DEB_F("HAL_t::InitModules_Cratewise()");
     int err=0;
     CrateVector_t::iterator ci;
     if(which==EVENT_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       err+=(*ci)->InitModules(errors);
	  }

	  err+=CORBO->InitModule(errors);              // handle error to be done ! 
//       CORBO: Register-Mode 
//     CORBO->InitBUSYModeChan1();             
//       CORBO: ReadOut-Controller Mode 
	  CORBO->InitROCModeChan1();
//     CORBO->PlayChan1();                // just lighting ther LEDs ....

	  CORBO->RemoveBUSYChan1();         // enable data taking

     // set the pointer for the Triggermodule to the Module with the
     // ID K32TDC0001
     //     we are using the CORBO as trigger now      *RW* 
     //     TriggerModule=FindModule("K32TDC001");     ******* use TriggerModule=CORBO !!!!
     } else if(which==ASYNCH_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       err+=(*ci)->InitModules_Asynch(errors);
	  }
     }  else if(which==DMAD_MODULES){
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       (*ci)->InitModules_DMAD(errors);
	  }
     }
     return 0;
}











