/*
$Header: /home/elrond/baikal/src/CVS/daq98/vmedaqhal.h,v 1.8 1999/02/10 17:31:47 daq Exp $
*/

#if !defined __VMEDAQHAL_H__
#define __VMEDAQHAL_H__

#include <config.h>
#include "simplevector.h"
#include "stringvector.h"

#include "pointerutil.h"
#include "settergetter.h"
#include "hwconfigitems2.h"

#include "channelmodule.h"
#include "crateclasses.h"

#if ! defined DUMMY_MODULES_ONLY
#include "corbomodule.h"
#endif

MAKE_INC_VECTOR(channel_t,ChannelVector_t);

/** This class represents a hardware abstraction layer.
    The "Superclass" for the Harware abstraction, contains a list of
    all Modules, that are in the CAMAC and VME crates as well as a
    List of all active optical Modules (described by the ChannelInfo
    structure) 

    - does the initialization and readout of all Modules by iterating
    over the Module list and calling The InitModule/ReadModule
    functions of each Module, 
    
    - does the event building by iterating over all ChannelInfo
    structure (the OM list) and asking all Modules (actually the
    Buffer) whether there is Data or not and transfer it into the
    Event Buffer */
class HAL_t
{
public:
     enum WhichModuleType_t{
	  EVENT_MODULES,
	  ASYNCH_MODULES,
	  DMAD_MODULES
     };
     
     /** Ctor. */
     HAL_t(DaqProcessBase *parent); /***/
     /** Dtor. */
     ~HAL_t();
     
     /** Checks for a trigger.
	 This function is called in the event loop. If there is a trigger all modules 
	 are checked and data is read
	 from the modules if necessary.
 	 @return 1 if there is a trigger, 0 otherwise. */
     inline int CheckForTrigger(){
//	  if(TriggerModule==(void*)0) return true;    // we have dummy tigger
//	  return TriggerModule->HasModuleData();      // trigger is e.g. K32TDC001  
	  return CORBO->InputPresentChan1();          // CORBO is in Trigger mode
     }
     
     /** Inhibits the trigger.
	 This function must be called before we read out the modules after 
	 getting a trigger. It inhibits a trigger while reading out teh module 
	 data. \Ref{ReleasTrigger} must be called after serving the trigger*/
     inline void InhibitTrigger(){
#if ! defined DUMMY_MODULES_ONLY
//          only needed for register mode of CORBO
//	  CORBO->SetBUSYChan1();           // inhibit trigger
#endif
     }
     
     /** Releases the trigger.
	 While reading out the module data the tirgger must be inhibited. This 
	 is acomplished by a call to \Ref{InhibitTrigger}. To release the
	 trigger after the readout is done call this function. */
     inline void ReleaseTrigger(){
#if ! defined DUMMY_MODULES_ONLY
	  CORBO->RemoveBUSYChan1();
#endif
     }

     /** Checks for an asynchrounus module trigger.
	 This function is called in the event loop. 
	 If there is a trigger all asynch modules 
	 are checked and data is read from the modules if necessary.
 	 @return 1 if there is a trigger, 0 otherwise. */
     inline int CheckForAsynchTrigger_Cratewise(CrateVector_t &Crates){
	  static CrateVector_t::iterator ci;
	  for(ci=Crates.begin();ci<Crates.end();ci++){
	       if ((*ci)->CheckTrigger_Asynch()) return 1;
	  }
	  return 0;
     }
     
     /** Inhibits the trigger.
	 This function must be called before we read out the modules after 
	 getting a trigger. It inhibits a trigger while reading out teh module 
	 data. \Ref{ReleasTrigger} must be called after serving the trigger*/



     /** Read out the module data.
	 The data is read from the modules in the order they appear in the corresponding module vectors.
	 This order also determines the order of the data in the raw data file. Following order is used
	 to read out the different modules:
	 <ol>
	 <li> TDC modules (as they appear in <tt>TDCVector</tt></li>)
	 <li> ADC modules (as they appear in <tt>ADCVector</tt></li>)
	 <li> tbc
	 </ol>
	 @param DataBuffer The data buffer that the module data is written to. The data is stored in the
	 buffer in an platform independent format that can be transfered over the net 
	 (see \URL[Binary data format]{__BINARY_DATA_FORMAT_URL__})
	 @return 0 if the modules could be read successfully, -1 otherwise. */     
//     int ReadoutModules(ModuleVector_t &ReadOutModules,BinaryDataBuffer &DataBuffer);
     /** initialize all enabled modules */ 
     int InitCrates(CrateVector_t &Crates, StringVector_t &errors);

     /** reads out the modules, separate for every crate, one crate after an other */
     int ReadoutModules_Cratewise(CrateVector_t &Crates,WhichModuleType_t which,BinaryDataBuffer &DataBuffer);
     /** initializes the modules, separate for every crate, one crate after an other */
     int InitModules_Cratewise(CrateVector_t &InitVector,WhichModuleType_t which, StringVector_t &errors);
      
     /** initializes the modules, separate for every crate, one crate after an other */
     int StartModules_Cratewise(CrateVector_t &InitVector,WhichModuleType_t which, StringVector_t &errors);
     /** RESETS the modules, separate for every crate, one crate after an other */
     int ResetModules_Cratewise(CrateVector_t &InitVector,WhichModuleType_t which, StringVector_t &errors);

     int InsertCrates  (CrateConfigRecordVector_t  &CrateConfig, StringVector_t &errors);  
     int InsertModules (ChannelModuleConfigRecordVector_t &ModuleConfig,StringVector_t &errors);
     int InsertChannels(ChannelConfigRecordVector_t &ChannelConfig,
			StringVector_t &errors);
     
     CrateClass_p CreateCrateClass(const char *Type);

     ChannelModuleBaseClass_p CreateModuleClass(const char *Type, DaqProcessBase *Parent); /***/

     /** removes all items from the data structures */
     int ClearStructure();

     /** searches the module list for the Crate class with the ID
	 Crate_ID 

	 @param Crate_ID Crate identification string as it appears in
	 the config file */
     const CrateClass_p FindCrate(const String  &Crate_ID);

     /** searches the module list for the Module class with the ID
         Module_ID 
	 
	 @param Module_ID module identification string as it appears in
	 the config file */
     const ChannelModuleBaseClass_p FindModule(const String  &Module_ID);

public:
     ChannelModuleVector_t& GetModuleVector(){
	  return *ModuleVector;
     }       
     CrateVector_t& GetCrateVector(){
	  return *CrateVector;
     }       
     
protected:
     DaqProcessBase *Parent; /***/
     CrateVector_t  *CrateVector;
     ChannelModuleVector_t *ModuleVector;
     ChannelModuleBaseClass_p TriggerModule;
#if ! defined DUMMY_MODULES_ONLY
     CORBOModule     *CORBO;
#endif
};

     
#endif //__VMEDAQHAL_H__
