/*
  $Header: /home/elrond/baikal/src/CVS/daq98/vmegpsmodule.C,v 1.11 1999/02/10 21:35:15 daq Exp $
*/

#include "vmegpsmodule.h"
#include "globalfunctions.h"

#define DEBUG_LOCAL(A)  cout   A


VME_GPSModuleClass_t::VME_GPSModuleClass_t(DaqProcessBase *parent)/***/
     :GPSModuleClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT, parent){

	  ievt_vmegps=0;            // local counter
}

VME_GPSModuleClass_t::~VME_GPSModuleClass_t(){}


#ifdef VME_ACCESS
     
/** returns true, if the module has data or false if the module
    does not have any data */
bool VME_GPSModuleClass_t::HasModuleData(){
     ModuleAccess++;
     return (FIFO_Empty() ? false : true) ;
}

/** Initializes the module */
int VME_GPSModuleClass_t::InitModule(StringVector_t &/*errors*/)  {


     u_i32 data, data_in;
     int  select_gps_clock, 
	  enable_pulser, 
	  cal_length;     
     int errors=0;
     ModuleAccess=0;
     u_i32 VMEAddress=GetModuleConfig()->Address_Slot;
     

     nw_map =256;
     p_vgps = (u_i32 *) vme_map(VMEAddress,nw_map,0x09);
      
     p_vgps_fifo =   (u_i32 *) ((void *) p_vgps + 0x10);
     p_vgps_calout = (u_i32 *) ((void *) p_vgps + 0x04);
     p_vgps_eclout = (u_i32 *) ((void *) p_vgps + 0x08);
     p_vgps_ledout = (u_i32 *) ((void *) p_vgps + 0x0C);
     
     // basic init at runstart

     *p_vgps = 0x3; 
     for (int iii=0xffff;iii ;iii--);  // wait

     // clear 
     *p_vgps = 0x0;
      
     // user init
     select_gps_clock = 0x1;
     enable_pulser    = 0x0;  
     cal_length = 0x0;         //  0x0=1.6usec, ... 0xf=16*1.6=25.6usec      
     
     data = 0xff & ( (select_gps_clock<<2)  | (enable_pulser<<3) | (cal_length<<4) );
     
     // write the ContRegister
     *p_vgps = data; 
     last_vgps_contreg = data;

     data_in = 0;
     data_in = *p_vgps; 
     cout << "ContReg  written = " << (void*) (0xff & data) << endl;
     cout << "ContReg  read    = " << (void*) (0xff & data_in) << endl;
     
     if ( (0xf & data) != (0xf & data_in)) {
	  errors++;
	  cout << "*** Error for "<<GetModuleConfig()->ModuleID<<"  ContReg  (write/read)= "<<
	       (void*)data<<" "<<(void*)data_in<<endl;
//	   exit (-1);
     }
     
     
     // now wait for catching the correct GPS time ~ 6 secs

     cout << "wait_loop for "<<GetModuleConfig()->ModuleID<<" ";
     for (int isecs=6; isecs; isecs--){ 
	  if (Is_GPS_On())  {
	       cout << " GPS is OK"<<endl;
	       break;
	  }
	  delay_msec(1000); 
	  cout<<".";
     }
     cout <<endl;
  

     char txt[1024];
     if (errors){
	  sprintf(txt,"%s init: addr 0x%x pulser %i cal_length %i. Not initialized",
		  GetModuleConfig()->ModuleID,
		  VMEAddress, enable_pulser, cal_length);
	  LogMsg(txt,2,2,WARNING);
     }else{
	  sprintf(txt,"%s init: addr 0x%x pulser %i cal_length %i",
		  GetModuleConfig()->ModuleID,
		  VMEAddress, enable_pulser, cal_length);
	  LogMsg(txt,4,4,INFO);
     }
     return 0;
}

int VME_GPSModuleClass_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

/**  check if FIFO empty 
     true = empty */  
bool  VME_GPSModuleClass_t::FIFO_Empty (){

     return (0x1 & *p_vgps_calout) ? false : true ;      // fifo non-empty flag
}
   
/**  read the FIFO eventwise  */
int VME_GPSModuleClass_t::Read_FIFO_Event (u_i32 *w, int &iw)     {

  // read FIFO for one full event 
  // assume readout frequency will cope with >1 events in buffer
  
  // read FIFOWORDS_PER_VMEGPS words from FIFO
  // check after every word if FIFO empty 
  // assume upon call FIFO non empty
  
  int i,j;
  iw=0;     
  int n_reads = FIFOWORDS_PER_VMEGPS;
  int waitloops = WAIT_LOOPS_VMEGPS_READ;

  for (i=0; i<n_reads; i++){
    w[i] = *p_vgps_fifo;
    if ((i<n_reads-1) && FIFO_Empty()) {
      for (j=0; j<waitloops; j++){
	if (!FIFO_Empty())  break;
      }
      if (j==waitloops-1)  {
	   char txt[1024];
	   sprintf(txt,"%s: FIFO timeout at i=%i",i);
	   LogMsg(txt,2,2,WARNING);
	break;
      }
    }
  }

  iw=i;
  return 0;       
}  

/** get the  status register
     4 bits valid */
u_i32 VME_GPSModuleClass_t::GetStatusRegister()  {
     return (0xff & *p_vgps);
}

/** get the FIFO status register
     2 bits valid */
u_i32 VME_GPSModuleClass_t::GetFIFOStatusRegister()  {
     return (0x3 & *p_vgps_calout);
}
 
/** output single calibration pulse
    length defined with cont-register  */
int VME_GPSModuleClass_t::Cal_Out()  {
     *p_vgps_calout = 0;
     return 0;
}
 
/** output to ECL register */
int VME_GPSModuleClass_t::ECL_Out(int ecl) {
     *p_vgps_eclout = 0xff & ecl;
     return 0;
}

/**  write to LED register */
int VME_GPSModuleClass_t::LED_Out(int led) {
     *p_vgps_ledout = 0xff & led;
     return 0;
}

/** check GPS status  true = external GPS on */
bool VME_GPSModuleClass_t::Is_GPS_On ()  {
     return (0x04 & *p_vgps) ? true : false ;      // ext-gps status flag
}

/**  Reset GPS on external GPS on, use last stored status written   */
int VME_GPSModuleClass_t::Reset_GPS_On ()  {
     *p_vgps = (0xfc & (0x04 | last_vgps_contreg)); // No mic & FIFO reset !!    
     return 0;
}

/**  Reset GPS on external GPS on, use last stored status written   */
int VME_GPSModuleClass_t::Reset_GPS_On_TimeOut ()  {
     *p_vgps = (0xfc & (0x04 | last_vgps_contreg)); // No mic & FIFO reset !! 
     usleep((unsigned long)100);
     return 0;
}

void VME_GPSModuleClass_t::Enable_PO()
{
     *p_vgps= (0xfc &(0x08 | last_vgps_contreg));
}

#else   // VME_ACCESS  ..............................................................


/** returns true, if the module has data or false if the module
    does not have any data */
bool VME_GPSModuleClass_t::HasModuleData(){
     ModuleAccess++;
     return true ;
}

/** Initialize the module */
int VME_GPSModuleClass_t::InitModule(StringVector_t &/*errors*/)  {

     ModuleAccess=0;
     int enable_pulser=0, 
         cal_length   =0;     
     u_i32 VMEAddress=GetModuleConfig()->Address_Slot;
     
     char txt[1024];
     sprintf(txt,"%s init: addr 0x%x pulser %i cal_length %i",
	     GetModuleConfig()->ModuleID,
	     VMEAddress, enable_pulser, cal_length);
     LogMsg(txt,4,4,INFO);

     return 0;
}

int VME_GPSModuleClass_t::ResetModule(StringVector_t &/*errors*/)
{
     return 0;
}

/**  check if FIFO empty 
     true = empty */  
bool  VME_GPSModuleClass_t::FIFO_Empty (){

     return false ;   
}
   
/**  read the FIFO eventwise  - simulation option  */
int VME_GPSModuleClass_t::Read_FIFO_Event (u_i32 *w, int &iw)     {

     iw=0;     


     // For simulation, this must be updated for 1999 GPS binary format !!!

     // one fifo-event
     int nsec = ievt_vmegps;
     int sec = ievt_vmegps%60;                   	  // simulate input
     int min = (ievt_vmegps%3600)/60;
     int hour = ievt_vmegps/3600;
     int day = 365;
     int tqc = 0;
     int sb  = 0xf;
     
     int w1 = (0xf&(sec%10))     | ((0xf&(sec/10))<<4) | 
	  ((0xf&(min%10))<<8) | ((0xf&(min/10))<<12) | 
	  ((0xf&(hour%10))<<16) | ((0xf&(hour/10))<<20); 
     int w2 = (0xf&(day%10))     | ((0xf&((day%100)/10))<<4) | 
	  ((0xf&(day/100))<<8)  |
          ((0xf&tqc)<<12)       | ((0xf&sb)<<16);
     
     w[iw++] = ievt_vmegps;	     // 1
     w[iw++] = w1;                   // 2
     w[iw++] = w2;                   // 3
     w[iw++] = nsec;                 // 4
     w[iw++] = (0xcf<<24) | (0xffffff&ievt_vmegps+1) ;      // 5

     return 0;
}
 
/** get the  status register
     6 bits valid */
u_i32 VME_GPSModuleClass_t::GetStatusRegister()  {
     return 0xc;          // Simulate: pulser & gps-clock-ok
}
  
/** get the FIFO status register
     2 bits valid */
u_i32 VME_GPSModuleClass_t::GetFIFOStatusRegister()  {
  return 0x1;            // Simulate: fifo-non empty
}

/** output single calibration pulse
    length defined with cont-register  */
int VME_GPSModuleClass_t::Cal_Out()  {
     return 0;
}
 
/** output to ECL register */
int VME_GPSModuleClass_t::ECL_Out(int /*ecl*/) {
     return 0;
}

/**  write to LED register */
int VME_GPSModuleClass_t::LED_Out(int /*led*/) {
     return 0;
}

/** check GPS status  true = external GPS on */
bool VME_GPSModuleClass_t::Is_GPS_On ()  {
     return true;      // ext-gps status flag
}

/**  Reset GPS on external GPS on, use last stored status written   */
int VME_GPSModuleClass_t::Reset_GPS_On ()  {
     return 0;
}

void VME_GPSModuleClass_t::Enable_PO()
{
}

/**  Reset GPS on external GPS on, use last stored status written   */
int VME_GPSModuleClass_t::Reset_GPS_On_TimeOut () {
     return 0;
}

#endif  // SIMUL - VME_ACCESS        .................................................................................



/** this function reads the entire module and saves the RAW data
         at the current position into the buffers, which is given as
         the paramter */
int VME_GPSModuleClass_t::ReadToBuffer(BinaryDataBuffer &DataBuffer){

  // reads ONE (!) GPS-event

  // for the number of x-Byte Words to be written for the module
  u_i16 NumBytes=0;
  
  u_i32 GPS_StatusRegister;
  u_i32 GPS_FIFOStatusRegister;
  u_i32 GPS_StatusSummary;
  u_i32 FIFO[MAX_FIFO_VMEGPS];
  int WordsRead_FIFO=0;
  
  // write the binary tag
  DataBuffer.WriteToBuffer(GetModuleConfig()->BinaryTag);
  // save the position where we have to store the number of hits 
  DataBuffer.PushPosition();
  // Write a zero size first
  DataBuffer.WriteToBuffer(NumBytes); 
  DataBuffer.WriteToBuffer(ModuleAccess);
  ievt_vmegps++;       // local counter for debugging
  
  // Read the status word before read-access 
  GPS_StatusRegister = GetStatusRegister();
  GPS_FIFOStatusRegister = GetFIFOStatusRegister();
  
  
  // Reading the just a single event (may be incomplete)
  
  Read_FIFO_Event(FIFO,WordsRead_FIFO);
  
  // write the FIFO data words
  for (int i=0 ;i<WordsRead_FIFO ;i++ ){                
    DataBuffer.WriteToBuffer(FIFO[i]);
  }
  
  // add zero's if not complete FIFO read fro 1 event
  if ((FIFOWORDS_PER_VMEGPS-WordsRead_FIFO)>0) {
    for (int i=0 ;i<(FIFOWORDS_PER_VMEGPS-WordsRead_FIFO) ;i++ ){                
      DataBuffer.WriteToBuffer((u_i32)0);
    }
  }

  // write the status word: status (GPS & PO) + fifo (NE & FULL)    
  GPS_StatusSummary=(0xC & GPS_StatusRegister) | (0x3 & GPS_FIFOStatusRegister);
  DataBuffer.WriteToBuffer(GPS_StatusRegister);

     
  // set the position to the place where we have to write the size 
  DataBuffer.PopPosition();
  NumBytes=GetBytesPerInfoUnit(); // it's always 1 event
  // write the size
  DataBuffer.WriteToBuffer(NumBytes);
  // restore the old position
  DataBuffer.IncPosition(NumBytes);
  if(!Is_GPS_On()){
       Reset_GPS_On_TimeOut();
       LogMsg("VME GPS reser ON",2,2,WARNING);
  }
  
  return 0;
}       

