#if ! defined __VMEGPSMODULEEVAL_H__
#define __VMEGPSMODULEEVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/vmegpsmoduleeval.h,v 1.12 1999/02/10 21:35:15 daq Exp $
*/

#include <config.h>
#include "vmegpsmodule.h"
#include "gpsmoduleeval.h"
#include "camacgpsmoduleeval.h"

/** Class to evaluate the data received from a VME GPS module,
    withthe time part of the  data format similar to the  
    CAMAC GPS module.
*/
class VME_GPSModuleEvalClass_t: public GPSModuleEvalClass_t
{
     enum 
     { 
	  NUMBER_OF_CHANNELS =1,
	  BYTES_PER_INFO_UNIT=24
     };
     ClassTypeInfo(GPSModuleEvalClass_t);
public:
     /** Constructor */
     inline VME_GPSModuleEvalClass_t(DaqProcessBase *Parent)
       :GPSModuleEvalClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,Parent){}
     
     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &/*HWConfig*/){
	  isAsynch=ModuleConfig->isAsynch;
	  SetTag(ModuleConfig->BinaryTag);
	  GPSModuleConfigRecord_p GPSConfig=TYPESAFE_DOWNCAST_RCP(GPSModuleConfigRecord_t,ModuleConfig);
	  if(!GPSConfig) return;
	  SetGPSEventIndex(GPSConfig->EventIndex);
	  TQCCount=0;
	  LastTQCCount=1;
     }
     

     /** Reads the data and builds the event part that this module is good for */
     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
       u_i16 NumBytes;
       const void *gps_data;
       
       RawDataBuffer.ReadFromBuffer(NumBytes);

       u_i32 Access;
       RawDataBuffer.ReadFromBuffer(Access);
       int nevts = NumBytes/BytesPerInfoUnit;
       if (nevts!=1) {
	    char txt[1024];
	    sprintf(txt,"Wrong number of events (%d) at GPS module %d (%d)",
		    nevts,Tag,GetGPSEventIndex());
	    LogMsg(txt,2,2,WARNING);
       }
       
       // read one 'event' (>1 should never happen !)
       gps_data=ReadFromBuffer(RawDataBuffer);

       GPSTime_t &t =GetTime(gps_data,Access);
       //       GetTime(gps_data);
	  
       if(! isAsynch) memcpy(TempEventBuffer->GPSTime+GetGPSEventIndex(),&t,sizeof(GPSTime_t));
       
       return 0;
     }       

protected:
     /** Reads the data from a raw data buffer into static memory */
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){
       // allocate some static memory, so we can return a pointer to
       // the memory with worry
       static u_i32 Data[WORDS_PER_VMEGPS];
       // read the data from the buffer
       
	    // ??	    for (int i=0; i< WORDS_PER_VMEGPS+1; i++){
      for (int i=0; i< WORDS_PER_VMEGPS; i++){
	DataBuffer.ReadFromBuffer(Data[i]);
      }
      // return a pointer to the data
      return (void*)&Data;
     }
     
     /** Gets the GPS time.
	 Converts the modules raw data into a GPS time. */
     virtual GPSTime_t &GetTime(const void *gps_data){
     };
     
     
     /** Gets the GPS time.
	 Converts the modules raw data into a GPS time. */
     GPSTime_t &GetTime(const void *gps_data,u_i32 Access){
	  
	  static GPSTime_t t;	  

	  u_i32 *FIFO = &(((u_i32*)gps_data)[0]);
	  u_i64 t_vgps_full, t_vgps_diff;
	  u_i64 t_vgps_last=0;
	  char a[255];

	  // the GPS information
	  
	  u_i32 secs_year = (0xffffff & FIFO[1]) | ((0xff & FIFO[2])<<24);
	  u_i32 tqc       = 0xf & (FIFO[2]>>16);                // quality
	  u_i32 sb        = 0xf & (FIFO[2]>>20);                // status
	  u_i32 secs_day  = secs_year%86400;
	  u_i32 secs_hour = secs_day%3600; 
	  
	  // GSP-year is wrong       !!!!!!!
	  // write 0 							   
	  t.quality       = 10; // write a 10 for missing year
	  t.year          = 0; //0xff & (FIFO[2]>>8);
	  t.day           = secs_year/86400;
	  t.hour          = secs_day/3600;
	  t.min           = secs_hour/60;
	  t.sec           = secs_hour%60;
	  u_i32 nsec      = (0x00ffffff & FIFO[0]) *100; 
	  t.nsec          = nsec;

	  u_i32 eventnb = 0x00ffff & FIFO[3];
	  u_i32 ecl_inp = 0xff & (0xff0000& FIFO[3] >>16);
       

	  t_vgps_full = (u_i64) nsec + (u_i64) (1e9* secs_year);
	  t_vgps_diff = t_vgps_full - t_vgps_last;
	  t_vgps_last = t_vgps_full;

	  if(isAsynch && outstream){
	       *outstream << "\nACCESS " << Tag << " " << Access
			  << endl;
	       *outstream << "STATUS " << Tag << " " << sb << endl;
	       *outstream << "TQC " << Tag << " " << tqc << endl;
	       *outstream << "DATE " << Tag << " " << t.year
			  << " " << t.day
			  << " " << t.hour
			  << ":" << t.min
			  << ":" << t.sec
			  << ":" << t.nsec << endl;
	       *outstream << "DIFF " << Tag << " " << (unsigned long)t_vgps_diff << endl;
	  }
	  
	  if (03> sb || tqc) {    
	       // bad quality
	       t.quality=1;
	       char txt[1024];
	       TQCCount++;
	       if(TQCCount==1){
		    sprintf(txt,"Bad GPS quality (%d,%d) at GPS module %d (%d) for %d event(s)",
			    sb, tqc, Tag, GetGPSEventIndex(),LastTQCCount);
		    LogMsg(txt,2,2,WARNING);
	       }
	       if(TQCCount>GPS_TQC_FAILURE_COUNT){
		    LastTQCCount=TQCCount;
		    TQCCount=0;
	       }
	  }
	  return t;
     }


public:
     static ostream *outstream;
     
protected:
     unsigned int TQCCount;
     unsigned int LastTQCCount;
     bool isAsynch;
};

typedef VME_GPSModuleEvalClass_t DummyVME_GPSModuleEvalClass_t;

#endif // __VMEGPSMODULEEVAL_H__
